/*
 *  (c) Copyright Hewlett-Packard Company 2000 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EnhAlt.java
 *
 * Created on 17 August 2000, 09:55
 */

package Freenet.client.rdf.util;

import Freenet.client.rdf.*;

/** A base class for supporting enhanced Alt's.  An enhanced resource is a
 * resource with extended functionality, provided either by the application or
 * by the implementation.  This class is a base class with Alt functionality
 * which can be extended by the application.
 *
 * <p>Enhanced resources are implemented using encapsulation.  An enhanced
 *   resource contains an embedded resource which provides the basic
 *   functionality of the resource.  The application code provides any extended
 *   behaviour.  This method was chosen, rather than subclassing, so as not
 *   to constrain the applications need to create its own class hierarchy.</p>
 *
 * <p>This class is intended to be subclassed by the application to provide the
 *   specific enhanced behaviour.  It provides Alt funcationality and some
 *   basic housekeeping functions to support the encapsulation.</p>
 *
 * @author bwm
 */
public class EnhAlt extends EnhContainer implements Alt {
    protected Alt alt;
    
    public EnhAlt(String uri, Model m) throws RDFException {
        super(m.createAlt(uri));
        alt = (Alt) resource;
    }
    
    public EnhAlt(Model m) throws RDFException {
        super(m.createAlt());
        alt = (Alt) resource;
    }

    protected EnhAlt(Resource r) throws RDFException {
        super(r.getModel().getAlt(r));
        alt = (Alt) resource;
    }
    
    public RDFNode getDefault() throws RDFException {
        return alt.getDefault();
    }
    public Resource  getDefaultResource() throws RDFException {
        return alt.getDefaultResource();
    }
    public Literal   getDefaultLiteral() throws RDFException {
        return alt.getDefaultLiteral();
    }
    public boolean   getDefaultBoolean() throws RDFException {
        return alt.getDefaultBoolean();
    }
    public byte      getDefaultByte() throws RDFException {
        return alt.getDefaultByte();
    }
    public short     getDefaultShort() throws RDFException {
        return alt.getDefaultShort();
    }
    public int       getDefaultInt() throws RDFException {
        return alt.getDefaultInt();
    }
    public long      getDefaultLong() throws RDFException {
        return alt.getDefaultLong();
    }
    public char      getDefaultChar() throws RDFException {
        return alt.getDefaultChar();
    }
    public float     getDefaultFloat() throws RDFException {
        return alt.getDefaultFloat();
    }
    public double    getDefaultDouble() throws RDFException {
        return alt.getDefaultDouble();
    }
    public String    getDefaultString() throws RDFException {
        return alt.getDefaultString();
    }
    public String    getDefaultLanguage() throws RDFException {
        return alt.getDefaultLanguage();
    }
    public Resource  getDefaultResource(ResourceF f) throws RDFException {
        return alt.getDefaultResource(f);
    }
    public Object    getDefaultObject(ObjectF f)   throws RDFException {
        return alt.getDefaultObject(f);
    }
    public Alt getDefaultAlt() throws RDFException {
        return alt.getDefaultAlt();
    }
    public Bag getDefaultBag() throws RDFException {
        return alt.getDefaultBag();
    }
    public Seq getDefaultSeq() throws RDFException {
        return alt.getDefaultSeq();
    }
    
    public Alt setDefault(RDFNode o) throws RDFException {
        return alt.setDefault(o);
    }    
    public Alt setDefault(boolean o) throws RDFException {
        return alt.setDefault(o);
    }
    public Alt setDefault(long o) throws RDFException {
        return alt.setDefault(o);
    }    
    public Alt setDefault(char o) throws RDFException {
        return alt.setDefault(o);
    }        
    public Alt setDefault(float o) throws RDFException {
        return alt.setDefault(o);
    } 
    public Alt setDefault(double o) throws RDFException {
        return alt.setDefault(o);
    }     
    public Alt setDefault(String o) throws RDFException {
        return alt.setDefault(o);
    }     
    public Alt setDefault(String o, String l) throws RDFException {
        return alt.setDefault(o,l);
    }    
    public Alt setDefault(Object o) throws RDFException {
        return alt.setDefault(o);
    }
}
