/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ContentManagerTest.java,v 1.3 2001/03/27 00:15:14 dac Exp $
 */
package net.jxta.test;

import net.jxta.share.ContentManager;
import net.jxta.share.ContentManagerImpl;
import net.jxta.share.Content;
import net.jxta.share.FileContent;
import net.jxta.share.ContentAdvertisement;
import net.jxta.share.ContentId;
import net.jxta.share.ContentIdImpl;
import net.jxta.document.Document;
import net.jxta.document.MimeMediaType;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

public class ContentManagerTest {
    private String[] args;

    public static void main(String[] args) throws Throwable {
	ContentManagerTest cmt = new ContentManagerTest(args);
	cmt.run();
    }

    ContentManagerTest(String[] args) {
	this.args = args;
    }

    public void run() throws Throwable {
	File dir = new File(args[0]);
	if (!dir.isDirectory()) {
	    p("Directory does not exist: " + dir);
	    return;
	}
	String[] l = dir.list();
	for (int i = 0; i < l.length; i++) {
	    new File(dir, l[i]).delete();
	}
	ContentManager cm = new ContentManagerImpl(dir);
	for (int i = 1; i < args.length; i++) {
	    File f = new File(args[i]);
	    FileContent fc = cm.share(f);
	    p("Sharing content: " + fc);
	}
	p("Restoring content");
	cm = new ContentManagerImpl(dir);
	Content[] cs = cm.getContent();
	for (int i = 0; i < cs.length; i++) {
	    p("Shared content: " + cs[i]);
	    ContentAdvertisement ca = cs[i].getContentAdvertisement();
	    Document d = (Document)
		ca.getDocument(new MimeMediaType("text/xml"));
	    d.sendToStream(System.out);
	}
	ContentAdvertisement ca = cs[0].getContentAdvertisement();
	ContentId id = ca.getContentId();
	String s = id.toString();
	ContentId id2 = new ContentIdImpl(s);
	if (!id.equals(id2)) {
	    p("ContentId equality test failed:");
	    p("    id1 = " + id);
	    p("    id2 = " + id2);
	}
    }

    private static void p(String s) {
	System.out.println(s);
    }
}
