/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ls.java,v 1.1 2001/05/30 20:52:26 chaywood Exp $
 *
 */


package net.jxta.impl.shell.bin.ls;

import net.jxta.impl.shell.*;
import net.jxta.impl.shell.bin.share.share;

import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.lang.reflect.InvocationTargetException;

import net.jxta.pipe.*;
import net.jxta.document.*;
import net.jxta.exception.*;
import net.jxta.peergroup.*;
import net.jxta.protocol.PipeAdvertisement;

import net.jxta.share.*;


/**
 * share Shell command - CMS extension to Shell
 *
 * share command is to share any kind of contents with
 * others in a peer group. Its an extension to shell commands
 * using Content Management Service.
 */

public class ls extends ShellApp {

    ShellEnv env;

    ListRequestor request = null;
    PipeAdvertisement destPipe = null;
    String subStr = null;

    private static String destPipeName = null;
    private static ContentAdvertisement[] lsResults = null;

    public ls() {
    }

    public void stopApp () {
    }

    public int startApp (String[] args) {
        env = getEnv();

        // get the std group
        ShellObject obj = env.get("stdgroup");
        PeerGroup group = (PeerGroup)obj.getObject();

        if (null == args || 0 == args.length) {
            localContent(group);
            return ShellApp.appNoError;
        }

        if (args.length > 0 && "-r".equals(args[0])) {
            // show results of last remote ls
            if (1 == args.length) {
                remoteContent();
                return ShellApp.appNoError;
            }

            // start a new remote ls
            if (args.length > 2) {
                subStr = args[2];
            }

            destPipeName = args[1];
            ShellObject tempObj = env.get(destPipeName);
            if (tempObj == null) {
                println ("ls: cannot access " + destPipeName);
                return ShellApp.appMiscError;
            }

            Object pipeObj = tempObj.getObject();
            if (PipeAdvertisement.class.isInstance(pipeObj)) {
                destPipe = (PipeAdvertisement)pipeObj;

                try {
                    request = new ListRequestor(group, destPipe, subStr);
                } catch (InvocationTargetException e) {
                    // XXX REMIND Should display error dialog here
                    e.printStackTrace();
                }
            }
        }

        return ShellApp.appNoError;
    }

    public void help() {
        println( "NAME" );
        println( "     ls - ls of content (file)" );
        println( " " );
        println( "SYNOPSIS" );
        println( "     ls -r <cms pipe adv> <substring>");
        println( "<substring> is optional" );
    }

    static int localNumber = 0;
    private void localContent(PeerGroup group) {
        int i;
        for (i = 0; i < localNumber; i++) {
            env.remove("content"+i);
        }

        CMS cms = share.getCMS(group);
        ContentManager contentManager = cms.getContentManager();

        Content[] content = contentManager.getContent();
        if (content != null) {
            for (i=0; i<content.length; i++) {
                ContentAdvertisement cadv = content[i].getContentAdvertisement();
                env.add("content"+i,new ShellObject("stored local content advertisement", 
                    cadv));
                println("content"+i+": "+cadv.getName());
            }
            localNumber = i;
        }
    }

    static int remoteNumber = 0;
    private void remoteContent() {
        int i;
        for (i = 0; i < remoteNumber; i++) {
            env.remove(destPipeName+"#content"+i);
        }

        if (lsResults != null) {
            for (i=0; i<lsResults.length; i++) {
                env.add(destPipeName+"#content"+i,new ShellObject("stored remote content advertisement", 
                    lsResults[i]));
                println(destPipeName+"#content"+i+": "+lsResults[i].getName());
            }
            remoteNumber = i;
        }
    }

    class ListRequestor extends ListContentRequest {
        public ListRequestor(PeerGroup group, PipeAdvertisement inRequest, 
                String inSubStr) throws InvocationTargetException {
            super(group, inRequest, inSubStr);
        }

        public void notifyDone() {
            System.out.println("Requestor done");
            lsResults = this.getResults();
        }
    }
}
