/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: share.java,v 1.3 2001/05/30 20:51:59 chaywood Exp $
 *
 */


package net.jxta.impl.shell.bin.share;

import net.jxta.impl.shell.*;


import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;

import net.jxta.pipe.*;
import net.jxta.document.*;
import net.jxta.exception.*;
import net.jxta.peergroup.*;

import net.jxta.share.*;


/**
 * share Shell command - CMS extension to Shell
 *
 * share command is to share any kind of contents with
 * others in a peer group. Its an extension to shell commands
 * using Content Management Service.
 */

public class share extends ShellApp {

    ShellEnv env;
    ContentManager contentManager = null;

    public share() {
    }

    public void stopApp () {
    }

    public int startApp (String[] args) {
        env = getEnv();

        // get the std group
        ShellObject obj = env.get("stdgroup");
        PeerGroup group = (PeerGroup)obj.getObject();

        // get the cms for this group
        CMS cms = getCMS(group);
        contentManager = cms.getContentManager();

        if (null == args || args.length != 1) {
            println("share <path>");
            return ShellApp.appNoError;
        }

        try {
            contentManager.share(new File(args[0]));
            println(args[0]+" shared");
        } catch (IOException e) {
            println("ERROR: could not share "+args[0]);
            e.printStackTrace();
        }

        return ShellApp.appNoError;
    }

    public void help() {
        println( "NAME" );
        println( "     share - share or unshare a content (file)" );
        println( " " );
        println( "SYNOPSIS" );
        println( "     share <path>");
        println( " ");
        println( "DESCRIPTION");
        println( " ");
        println( "share any content or document in the current peer group.");
        println( "The document is made visible to all the members of the peer group.");
        println( "Contents/Documents are searched" );
        println( "either in the local peer cache or remotely via the 'ls' command.");
        println( " " );
        println( "OPTIONS" );
        println( " " );
        println( "EXAMPLE" );
        println( " " );
        println( "    JXTA>share document.txt " );
        println( " ");
        println( "This example shares the document 'document.txt' into the current peer group." );
        println( " " );
        println( "SEE ALSO" );
        println( "    unshare ls rcp" );
    }


    public static final String IP2P_DIRNAME = "IP2P";
    private static Hashtable cmsTable = new Hashtable();

    public static CMS getCMS(PeerGroup group) {
        // check if the cms is already started
        CMS cms = (CMS)cmsTable.get(getShortName(group));

        if (null == cms) {
            // create a new cms for this peergroup
            cms = new CMS();

            try {
                cms.init(group, group.getAdvertisement());
            } catch (PeerGroupException e) {
	      e.printStackTrace ();
            }

            cms.startApp(new File(getGroupDir(group), CMS.DEFAULT_DIR));
        }

        if (null != cms) {
            cmsTable.put(getShortName(group), cms);
        }

        return cms;
    }

    private static File getGroupDir(PeerGroup pg) {
        File dir = null;

        // make sure the root directory exists
        File rootDir = new File(IP2P_DIRNAME);
        if (!rootDir.exists()) {
            rootDir.mkdir();
        }

        dir = new File(rootDir, getShortName(pg));
        if (!dir.exists()) {
            dir.mkdir();
        }

        return dir;
    }

    private static String getShortName(PeerGroup pg) {
        URL url = null;
        try {
            url = new URL(pg.getID().toString());
        } catch (MalformedURLException e) {
            e.printStackTrace();
            return null;
        }

        return Integer.toHexString((url.getHost()).hashCode());
    }
}
