/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: unshare.java,v 1.2 2001/05/30 20:51:59 chaywood Exp $
 *
 */


package net.jxta.impl.shell.bin.unshare;

import net.jxta.impl.shell.*;
import net.jxta.impl.shell.bin.share.share;

import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;

import net.jxta.pipe.*;
import net.jxta.document.*;
import net.jxta.exception.*;
import net.jxta.peergroup.*;

import net.jxta.share.*;


/**
 * share Shell command - CMS extension to Shell
 *
 * share command is to share any kind of contents with
 * others in a peer group. Its an extension to shell commands
 * using Content Management Service.
 */

public class unshare extends ShellApp {

    ShellEnv env;
    ContentManager contentManager = null;

    public unshare() {
    }

    public void stopApp () {
    }

    public int startApp (String[] args) {
        env = getEnv();

        // get the std group
        ShellObject obj = env.get("stdgroup");
        PeerGroup group = (PeerGroup)obj.getObject();

        // get the cms for this group
        CMS cms = share.getCMS(group);
        contentManager = cms.getContentManager();

        if (null == args || args.length != 1) {
            println("unshare <content advertisement>");
            return ShellApp.appNoError;
        }

        ShellObject tempShellObj = env.get(args[0]);
        if (tempShellObj == null) {
            println ("unshare: cannot access " + args[0]);
            return ShellApp.appMiscError;
        }

        Object tempObj = tempShellObj.getObject();
        if (ContentAdvertisement.class.isInstance(tempObj)) {
            try {
                contentManager.unshare((ContentAdvertisement)tempObj);
                println(args[0]+" unshared");
            } catch (IOException e) {
                println("ERROR: could not unshare "+args[0]);
                e.printStackTrace();
            }
        }

        return ShellApp.appNoError;
    }

    public void help() {
        println( "NAME" );
        println( "     unshare - unshare a content (file)" );
        println( " " );
        println( "SYNOPSIS" );
        println( "     unshare <content advertisement>");
        println( " ");
        println( "DESCRIPTION");
        println( " ");
        println( "unshares a document referenced by content advertisment" );
        println( "in the current peer group.");
        println( "The document is nolonger accessible to any member of the peer group.");
        println( " " );
        println( "OPTIONS" );
        println( " " );
        println( "EXAMPLE" );
        println( " " );
        println( "    JXTA>unshare content0" );
        println( " ");
        println( "This example unshares the document referenced by 'content0' in the current peer group." );
        println( " " );
        println( "SEE ALSO" );
        println( "    share ls rcp" );
    }
}
