/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ContentInfo.java,v 1.1 2001/04/15 05:10:18 dac Exp $
 *
 */

package net.jxta.cms;

import net.jxta.document.Document;
import net.jxta.document.StructuredDocument;
import net.jxta.document.Element;
import net.jxta.document.MimeMediaType;
import net.jxta.protocol.PipeAdvertisement;
import java.io.Serializable;

/**
 * A ContentInfo object is used to represent metadata about content
 * that is being shared within a peer group. The searching of content
 * metadata is handled by the SearchService. The wire representation
 * of a content metadata is XML. Here is an example:
 * 
 * <jxta:contentInfo>
 *   <name>SkySaw.mp3</name>
 *   <id>1a8baf7ab82c8fee8fe2a2d9e7ecb7a83</id>
 *   <type>audio/mpeg</type>
 *   <length>234126767</length>
 *   <description>Sky Saw from Brian Eno's "Another Green World"</description>
 * </jxta:contentInfo>
 */
public abstract class ContentInfo implements Serializable {
    /**
     * Appends this advertisement document to the specified structured
     * document element.
     */
    public abstract void appendDocument(StructuredDocument doc, Element el);

    /**
     * Returns the Document for this ContentInfo object.
     */
    public abstract Document getDocument(MimeMediaType mmt);

    /**
     * Returns the name of the content.
     */
    public abstract String getName();

    /**
     * Returns the id of the content.
     */
    public abstract String getID();

    /**
     * Returns the PipeAdvertisement for handling content requests.
     */
    public abstract PipeAdvertisement getPipeAdvertisement();

    /**
     * Returns the length of the content, or -1 if not specified.
     */
    public abstract long getLength();

    /**
     * Returns the mime type of the content, or null if not specified.
     */
    public abstract String getType();

    /**
     * Returns the description of the content, or null if not specified.
     */
    public abstract String getDescription();
}
