/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ContentService.java,v 1.3 2001/04/13 03:21:20 dac Exp $
 *
 */

package net.jxta.cms;

import net.jxta.service.Service;
import net.jxta.peergroup.PeerGroup;
import net.jxta.document.Advertisement;

import java.util.Enumeration;

/**
 * The ContentService is used for sharing arbitrary content within a peer
 * group. Each Content is reference by a unique content identifier string.
 */
public abstract class ContentService implements Service {
    private PeerGroup group;	// content peer group
    private Advertisement adv;	// content service (?) advertisement

    /**
     * Shares the specified ContentHandler and returns its associated
     * Content object.
     */
    public abstract Content share(ContentHandler handler);

    /**
     * Unshares the specified Content object.
     */
    public abstract void unshare(Content c);

    /**
     * Returns an Enumeration of currently shared Content objects.
     */
    public abstract Enumeration getContent();
	
    /**
     * Returns the number of currently shared content items.
     */
    public abstract int size();

    /**
     * Adds the specified ContentListener which will be notified when
     * a ContentEvent has occurred.
     */
    public abstract void addListener(ContentListener listener);

    /**
     * Removes the specified ContentListener from the listener list.
     */
    public abstract void removeListener(ContentListener listener);

    /**
     * Creates a new ContentResponse object.
     */
    public abstract ContentResponse newContentResponse();

    /**
     * Called by the JXTA core to initialize this service.
     */
    public void init(PeerGroup group, Advertisement adv) {
	this.group = group;
	this.adv = adv;
    }

    /**
     * Called by the JXTA core to start this service (not sure about this).
     */
    public int startApp(String[] arg) {
	return 0;
    }

    /**
     * Called by the JXTA core to stop this service (not sure about this).
     */
    public void stopApp() {
    }

    /**
     * Returns the PeerGroup for this ContentService.
     */
    public PeerGroup getPeerGroup() {
	return group;
    }

    /**
     * Returns the content service "interface" object.
     */
    public Service getInterface() {
	return this;
    }

    /**
     * Returns the content service advertisement.
     */
    public Advertisement getAdvertisement() {
	return adv;
    }
}
