/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: EventDispatcherTest.java,v 1.1 2001/04/14 02:56:52 dac Exp $
 *
 */

package net.jxta.cms.test;

import net.jxta.cms.util.EventDispatcher;

import java.util.EventObject;
import java.util.EventListener;
import java.util.Hashtable;

/**
 * Test the EventDispatcher utility class.
 */
public class EventDispatcherTest extends Test implements EventListener {
    private EventDispatcher dispatcher;
    private Hashtable events = new Hashtable();

    public static void main(String[] args) throws Throwable {
	new EventDispatcherTest().run(args);
    }

    public String getName() {
	return "EventDispatcher";
    }

    // event delivery timeout
    private static final long TIMEOUT = 5000;

    public void run(String[] args) throws Throwable {
	int count = 100;
	if (args != null && args.length > 0) {
	    count = Integer.parseInt(args[0]);
	}
	p("+ Start event dispatcher");
	dispatcher = new Dispatcher();
	dispatcher.addListener(this);
	dispatcher.start();
	p("+ Post " + count + " events");
	for (int i = 0; i < count; i++) {
	    EventObject event = new EventObject(new Integer(i));
	    synchronized (events) {
		events.put(event, event);
	    }
	    dispatcher.post(event);
	}
	p("+ Verify events");
	// wait up to 5 seconds for all events to get delivered
	long t0 = System.currentTimeMillis();
	synchronized (events) {
	    while (events.size() > 0) {
		try {
		    events.wait(1000);
		} catch (InterruptedException e) {
		}
		long t1 = System.currentTimeMillis();
		if (t1 - t0 > TIMEOUT) {
		    p("! Timeout exceeded waiting for events");
		    break;
		}
	    }
	}
	if (events.size() > 0) {
	    p("! Verify events failed: " + events.size() + " remaining events");
	}
    }

    public void handleEvent(EventObject event) {
	synchronized (events) {
	    if (!events.containsKey(event)) {
		p("! Possibly duplicate event: " + event.getSource());
	    } else {
		events.remove(event);
		events.notify();
	    }
	}
    }

    class Dispatcher extends EventDispatcher {
	protected void dispatch(EventObject event, EventListener listener) {
	    handleEvent(event);
	}
    }
}
