"""
a curses replacement for sys.stdin and sys.stdout

This is actually very buggy and feature-missing.  I only wrote it to
move the source out from the main genny.py file into a separate file.

Bugs:
 o 'print foo, bar' will not work, IIRC.
 o readline() returns None if there is not a newline, which is different
from normal sys.stdin.readline()
 o There are more.
"""

from sys import platform, __stdin__, __stdout__

from string import join

from curses import  beep, flash, initscr, noecho, cbreak, A_REVERSE, echo, \
	nocbreak, endwin, error
curses_error = error ; del error
if platform == 'freebsd3':
	from curses import key_backspace
	KEY_BACKSPACE = key_backspace ; del key_backspace
else:
	from curses import KEY_BACKSPACE

import sys

class my_stdout:
	def __init__(self):
		self.last_y = stdscr.getmaxyx()[0] - 1
		self.last_x = stdscr.getmaxyx()[1] - 1
	def write(self, message):
		if message != '\n':
			curpos = stdscr.getyx()
			stdscr.addstr(self.last_y - 2, self.last_x, '\n' + \
				message)
			stdscr.move(curpos)
			stdscr.refresh()

class my_stdin:
	fileno = __stdin__.fileno
	def __init__(self):
		self.newstr = []
	def readline(self):
		newch = stdscr.getch()
		command = None
		# Newlines finish our work.  I don't care about \r now.
		if newch == 10:
			command = join(self.newstr, '')
			stdscr.addstr(sys.stdout.last_y, 0, sys.stdout.prompt)
			stdscr.clrtoeol()
			stdscr.refresh()
			self.newstr = []
		# KEY_BACKSPACE is a curses thing that I don't understand.
		# 127 is ASCII DEL, which is sent by <backspace> under Linux.
		# The curses header file says KEY_BACKSPACE is unreliable.
		# see initialise() for other trouble involving KEY_BACKSPACE
		elif newch == KEY_BACKSPACE or newch == 127:
			if self.newstr:
				self.newstr.pop()
				stdscr.move(sys.stdout.last_y, \
					stdscr.getyx()[1] - 1)
				stdscr.clrtoeol()
				stdscr.refresh()
		# We're full, thanks.
		# FIXME:  The user should obviously be allowed to enter bigger
		# lines.
		elif len(self.newstr) == sys.stdout.last_x - \
			len(sys.stdout.prompt):
			beep()
		# I'm trying to avoid dealing with interesting characters
		elif newch > 255:
			flash()
			pass
		# It seems to be a normal character.
		else:
			stdscr.echochar(newch)
			if platform == 'freebsd3':  stdscr.refresh()
			self.newstr.append(chr(newch))
		return command

def display_prompt():
	pass
	return

def initialise_ui(STATUS = '', PROMPT = ''):
	global stdscr
	stdscr = initscr()
	noecho()
	cbreak()
	stdscr.keypad(1)
	#
	sys.stdout = my_stdout()
	# Make status line and pad.
	pad_chars = sys.stdout.last_x - (1 + len(STATUS))
	sys.stdout.status_line = '  ' + STATUS + ' ' * pad_chars
	stdscr.addstr(sys.stdout.last_y - 1, 0, sys.stdout.status_line, \
		A_REVERSE)
	sys.stdout.prompt = PROMPT
	stdscr.addstr(sys.stdout.last_y, 0, sys.stdout.prompt)
	stdscr.setscrreg(0, sys.stdout.last_y - 2)
	stdscr.scrollok(1)
	stdscr.refresh()
	#
	sys.stdin = my_stdin()
	#
	return

def shutdown_ui():
	try:
		print "Trying to clear up curses."
		# Clear up curses.
		stdscr.refresh()
		stdscr.keypad(0)
		echo()
		nocbreak()
		endwin()
		sys.stdin = __stdin__
		sys.stdout = __stdout__
		print "curses cleared up."
	except curses_error, message:
		print "Failed to clean up curses!"
		print "message is: %s" % message
	return

def handle_command_clear():
	stdscr.clear()
	stdscr.addstr(sys.stdout.last_y - 1, 0, sys.stdout.status_line, \
		A_REVERSE)
	stdscr.addstr(sys.stdout.last_y, 0, sys.stdout.prompt)
	stdscr.refresh()
	return

