
# This file is to contain whatever Genny Services considered to be "basic"
# Analogous to builtin services for inetd.

from struct import pack, unpack
from time import time, ctime
from socket import gethostname
from gennylib import broadcast, raw_send, get_random_bytes, rawip2ip, GENNY, \
	respond, for_me, hostorip2rawip

# Servant to servant connection glue
GENNY_CONTROL = 0
CONTROL_FOO = 0			# Are you there?
CONTROL_BAR = 1			# Yes, I'm here.
CONTROL_TOOSMALL = 2		# That packet is too small
GENNY_CONTROL_TOOSMALL = 2
CONTROL_TOOLARGE = 3		# That packet is too large
CONTROL_NOSERVICE = 4		# I don't know that service
GENNY_CONTROL_NOSERVICE = 4
CONTROL_SQUELCH = 5		# Quit your flooding!
CONTROL_SILENCE = 6		# Don't give me any more of these
CONTROL_MESSAGE = 7		# Message to the operator
CONTROL_SERVSASK = 8		# What services do you support?
CONTROL_SERVSSTATE = 9		# These are the services I support
CONTROL_ERROR = 255		# This error happen
CONTROL_NUMERICS = CONTROL_FOO, CONTROL_BAR, CONTROL_TOOSMALL, \
	CONTROL_TOOLARGE, CONTROL_NOSERVICE, CONTROL_SQUELCH, CONTROL_SILENCE, \
	CONTROL_MESSAGE, CONTROL_SERVSASK, CONTROL_SERVSSTATE

# Network information
GENNY_INFO = 1
# FIXME:  More functionality is needed here.
INFO_PING = 0			# Ask for a pong.
INFO_PINGNC = 1			# NC means No-Cache
INFO_PONG = 128			# Give host information
INFO_PONGNC = 129		# NC means Not-Cached
INFO_ERROR = 255
INFO_NUMERICS = INFO_PING, INFO_PINGNC, INFO_PONG, INFO_PONGNC, INFO_ERROR

# Echo a string
GENNY_ECHO = 7
# FIXME:  The concept of the initial packet always being a broadcast is now
# deprecated.  I am just too lazy to rename these numerics now, so I will
# just put a fixme in.
ECHO_BROAD = 0			# Broadcast
# FIXME:  Is there really a point to a cached echo? ...
ECHO_BROADNC = 1		# NC means No-Cache
ECHO_REPLY = 128		# Reply
ECHO_REPLYNC = 129		# NC means Not-Cached
ECHO_ERROR = 255
ECHO_NUMERICS = ECHO_BROAD, ECHO_BROADNC, ECHO_REPLY, ECHO_REPLYNC, ECHO_ERROR

# People readable time of day
GENNY_DAYTIME = 13
DAYTIME_ASK = 0			# Query
DAYTIME_ASKNC = 1		# NC means No-Cache
DAYTIME_STATE = 128		# Reply
DAYTIME_STATENC = 129		# NC means Not-Cached
DAYTIME_ERROR = 255
DAYTIME_NUMERICS = DAYTIME_ASK, DAYTIME_ASKNC, DAYTIME_STATE, DAYTIME_STATENC, \
	DAYTIME_ERROR

# Getting a hostname
# FIXME:  This should probably be merged into GENNY_INFO
GENNY_HOSTNAME = 53		# Hostname reporting.
HOSTNAME_ASK = 0		# Ask someone's hostname
HOSTNAME_ASKNC = 1		# NC means...oh, you know already
HOSTNAME_STATE = 128		# State your hostname.
HOSTNAME_STATENC = 129		# NC means not...oh, right...
HOSTNAME_ERROR = 255
HOSTNAME_NUMERICS = HOSTNAME_ASK, HOSTNAME_ASKNC, HOSTNAME_STATE, \
	HOSTNAME_STATENC, HOSTNAME_ERROR

def handle_command_echo(config, connections, identifiers, arguments):
	if not arguments:
		print "usage: echo stuff to have echoed"
	else:
		payload = pack('B', ECHO_BROAD) + arguments
		broadcast(config, connections, identifiers, GENNY_ECHO, payload)
	return

def handle_command_update(config, connections, identifiers):
	payload = pack('B', INFO_PING)
	broadcast(config, connections, identifiers, GENNY_INFO, payload)
	return

def handle_command_announce(config, connections, identifiers):
	rawipaddy = config['myip']
	rawport = pack('H', config['myport'])
	payload = pack('B', INFO_PONG) + rawipaddy + rawport
	broadcast(config, connections, identifiers, GENNY_INFO, payload)
	return

def handle_command_daytime(config, connections, identifiers):
	payload = pack('B', DAYTIME_ASK)
	broadcast(config, connections, identifiers, GENNY_DAYTIME, payload)
	return

def handle_command_who(config, connections, identifiers):
	payload = pack('B', HOSTNAME_ASK)
	broadcast(config, connections, identifiers, GENNY_HOSTNAME, payload)
	return

def handle_command_myip(config, arguments):
	if not arguments:
		print "myip is: %s" % rawip2ip(config['myip'])
	elif ' ' in arguments:
                print "usage:  myip [ip.ad.dr.es]"
        else:
		try:
			config['myip'] = hostorip2rawip(arguments)
			if config['loglevel'] > 0:
                        	print "myip set to: %s" % \
					rawip2ip(config['myip'])
                except:
                         print "I can't convert that."
	return

def do_foo_bars(config, connections):
	four_minutes_ago = int(time()) - 60*4
	for conn in connections + []:
		if conn.flag != GENNY:
			continue
		if conn.bar_time < four_minutes_ago:
			s_packet = 'GENNY DISCONNECT\nLast Control Bar was ' \
				'over four minutes ago\n\n'
			conn.send(s_packet)
			if config['loglevel'] > 1:
				print "Disconnected: %s:%s %s" % \
					(conn.addr[0], conn.addr[1], \
					`s_packet`)
			connections.remove(conn)
			if config['loglevel'] > 0:
				print "Removed connection: %s:%s" % \
					conn.addr
		else:
			conn.last_foo = get_random_bytes(10)
			payload = pack('B', CONTROL_FOO)
			control(config, conn, conn.last_foo, payload)
			if config['loglevel'] > 4:
				print "Control Fooed %s: " % conn + \
					`conn.last_foo`
	return

def handle_genny_control(config, identifiers, conn, r_packet):
	if len(r_packet) < 13:
		if config['loglevel'] > 3:
			print "Control packet too small %s: " % conn + \
				`r_packet`
		payload = pack('B', CONTROL_ERROR) + "Control had no numeric."
		control(config, conn, None, payload)
		return
	numeric = unpack('B', r_packet[12])[0]
	if numeric == CONTROL_FOO:
		if config['loglevel'] > 4:
			print "Recved Control Foo %s: " % conn + `r_packet`
		conn.last_bar = r_packet[0:10]
		payload = pack('B', CONTROL_BAR)
		control(config, conn, conn.last_bar, payload)
		if config['loglevel'] > 4:
			print "Sended Control Bar %s: " % conn + `r_packet`
	elif numeric == CONTROL_BAR:
		if config['loglevel'] > 4:
			print "Recved Control Bar %s: " % conn + `r_packet`
		if r_packet[0:10] == conn.last_foo:
			conn.bar_time = int(time())
			if config['loglevel'] > 3:
				print "Successful Foo-Bar %s:" % conn
	elif numeric == CONTROL_ERROR:
		if config['loglevel'] > 0:
			print "Control Error from %s: " % conn + `r_packet`
	elif numeric in CONTROL_NUMERICS:
		if config['loglevel'] > 4:
			print "Ignoring legit Control %s: " % conn + `r_packet`
	else:
		if config['loglevel'] > 4:
			print "Bad Control Numeric %s: " % conn + `r_packet`
		payload = pack('B', CONTROL_ERROR) + \
			"Bad Control numeric: %s" % numeric
                control(config, conn, None, payload)
	return

def handle_genny_info(config, identifiers, r_packet):
        if len(r_packet) < 13:
                if config['loglevel'] > 1:
                        print "Info packet too small %s: " % conn + \
                                `r_packet`
                payload = pack('B', INFO_ERROR) + "Packet was too small."
		respond(config, identifiers, r_packet[:12], GENNY_INFO, payload)
                return
        numeric = unpack('B', r_packet[12])[0]
	if numeric == INFO_PING:
		if config['loglevel'] > 1:
			print "Received info ping."
		rawipaddy = config['myip']
		rawport = pack('H', config['myport'])
		payload = pack('B', INFO_PONGNC) + rawipaddy + rawport
		respond(config, identifiers, r_packet[:12], GENNY_INFO, payload)
	elif numeric == INFO_PINGNC:
		if config['loglevel'] > 1:
			print "Received info ping NC."
		rawipaddy = config['myip']
		rawport = pack('H', config['myport'])
		payload = pack('H', INFO_PONGNC) + rawipaddy + rawport
		respond(config, identifiers, r_packet[:12], GENNY_INFO, payload)
	elif numeric == INFO_PONG:
		ipaddy = rawip2ip(r_packet[13:17])
		port = unpack('H', r_packet[17:19])[0]
		host = (ipaddy, port)
		if host not in config['hosts']:
			config['hosts'].append(host)
		if config['loglevel'] > 1 and for_me(identifiers, r_packet):
			print "Received info pong: %s:%s" % host
	elif numeric == INFO_PONGNC:
		ipaddy = rawip2ip(r_packet[13:17])
		port = unpack('H', r_packet[17:19])[0]
		host = (ipaddy, port)
		if host in config['hosts']:
			config['hosts'].remove(host)
		config['hosts'].append(host)
		if config['loglevel'] > 1 and for_me(identifiers, r_packet):
			print "Received info pong NC: %s:%s" % host
	return

def handle_genny_echo(config, identifiers, r_packet):
        if len(r_packet) < 13:
                if config['loglevel'] > 1:
                        print "Info packet too small %s: " % conn + \
                                `r_packet`
                payload = pack('B', ECHO_ERROR) + "Packet was too small."
                respond(config, identifiers, r_packet[:12], GENNY_ECHO, payload)
		return
        numeric = unpack('B', r_packet[12])[0]
	if numeric in (ECHO_BROAD, ECHO_BROADNC):
		payload = pack('B', ECHO_REPLYNC) + r_packet[13:]
		respond(config, identifiers, r_packet[:12], GENNY_ECHO, payload)
	elif numeric == ECHO_REPLY:
		if for_me(identifiers, r_packet) and config['loglevel'] > 0:
			print "Echo received: %s" % `r_packet[13:]`
	elif numeric == ECHO_REPLYNC:
		if for_me(identifiers, r_packet) and config['loglevel'] > 0:
			print "Echo received NC: %s" % `r_packet[13:]`
	return

def handle_genny_daytime(config, identifiers, r_packet):
        if len(r_packet) < 13:
                if config['loglevel'] > 1:
                        print "Daytime packet too small %s: " % conn + \
                                `r_packet`
                payload = pack('B', DAYTIME_ERROR) + "Packet was too small."
                respond(config, identifiers, r_packet[:12], GENNY_DAYTIME, \
			payload)
		return
	numeric = unpack('B', r_packet[12])[0]
	if numeric in (DAYTIME_ASK, DAYTIME_ASKNC):
		payload = pack('B', DAYTIME_STATENC) + ctime(time())
		respond(config, identifiers, r_packet[:12], GENNY_DAYTIME, \
			payload)
	elif numeric == DAYTIME_STATE:
		if for_me(identifiers, r_packet) and config['loglevel'] > 0:
			print "Daytime received: %s" % `r_packet[13:]`
	elif numeric == DAYTIME_STATENC:
		if for_me(identifiers, r_packet) and config['loglevel'] > 0:
			print "Daytime received NC: %s" % `r_packet[13:]`
	return

def handle_genny_hostname(config, identifiers, r_packet):
        if len(r_packet) < 13:
                if config['loglevel'] > 1:
                        print "Hostname packet too small %s: " % conn + \
                                `r_packet`
                payload = pack('B', HOSTNAME_ERROR) + "Packet was too small."
                respond(config, identifiers, r_packet[:12], GENNY_HOSTNAME, \
                        payload)
		return
        numeric = unpack('B', r_packet[12])[0]
	if numeric in (HOSTNAME_ASK, HOSTNAME_ASKNC):
		payload = pack('B', HOSTNAME_STATENC) + gethostname()
		respond(config, identifiers, r_packet[:12], GENNY_HOSTNAME, \
			payload)
	elif numeric == HOSTNAME_STATE:
		if for_me(identifiers, r_packet) and config['loglevel'] > 0:
			print "Hostname received: %s" % `r_packet[13:]`
	elif numeric == HOSTNAME_STATENC:
		if for_me(identifiers, r_packet) and config['loglevel'] > 0:
			print "Hostname received NC: %s" % `r_packet[13:]`
	return

def control(config, conn, routing_header, payload):
	if not routing_header:
		if conn.last_foo:
			routing_header = conn.last_foo
		else:
			routing_header = '\0'*10
	elif len(routing_header) != 10:
		raise TypeError, "routing_header must be ten bytes or false"
	rawservice = pack('H', GENNY_CONTROL)
	#print "routing_header -> %s, rawservice -> %s, payload -> %s" % \
	#	(`routing_header`, `rawservice`, `payload`)
	s_packet = routing_header + rawservice + payload
	conn.send(s_packet)
	return

