/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Interface to the admin functionality
 */

/*
 *  Save preferences
 */
void admin_exit (void);
void admin_load (void);
void admin_save (void);


/*
 *  General settings
 */
gchar *admin_get_downloaddir    (void);
gint   admin_get_linkspeed      (void);
gint   admin_get_maintaincount  (void);

void   admin_set_downloaddir    (gchar *downloaddir,
				 gint   update_interface);
void   admin_set_linkspeed      (gint   linkspeed,
				 gint   update_interface);
void   admin_set_maintaincount  (gint   maintaincount,
				 gint   update_interface);


/*
 *  Statistics
 */
extern gulong totalhosts;
extern gulong totalfiles;
extern gulong totalbytes;
extern gulong servedfiles;
extern gulong servedbytes;
extern gulong downloadfiles;
extern gulong downloadbytes;


void admin_set_statistic (gchar  *statlabel,
			  gulong *oldvalue,
			  gulong  addition);

/*
 *  Server Files
 */

typedef struct _ServerFile
{
  gchar  *filename;
  gint32  size;
} ServerFile;

void admin_file_add    (gchar      *filename);
void admin_file_remove (ServerFile *file);


/*
 *  Server Nodes
 */

#define DEFAULT_MAINTAIN_COUNT   20
#define DEFAULT_BALANCE_INTERVAL 10000

typedef enum
{
  NODE_DISCONNECTED,
  NODE_FAILURE,
  NODE_CONNECTING,
  NODE_INBOUND,
  NODE_OUTBOUND
} NodeStatus;

typedef struct _ServerNode
{
  gchar      *server;
  gint32      port;
  NodeStatus  status;
  gint32      files;
  gulong      bytes;
} ServerNode;

void admin_node_balance     (void);
ServerNode* admin_node_add         (ServerNode *node);
ServerNode* admin_node_find (ServerNode *node);
void admin_node_connect     (ServerNode *node);
void admin_node_disconnect  (ServerNode *node);
void admin_node_delete      (ServerNode *node);
void admin_node_update      (ServerNode *node,
			    NodeStatus  status);

