/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "globals.h"
#include "search.h"
#include "download.h"
#include "admin.h"


void
on_administrative1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  /*  Create the admin dialog  */
  if (admin == NULL)
    admin = create_admin ();

  /* Show the admin dialog  */
  gtk_widget_show (admin);
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  admin_exit ();
}


void
on_search1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkNotebook *notebook;

  //  Fetch the notebook widget
  notebook = (GtkNotebook *) lookup_widget (GTK_WIDGET (menuitem), 
					    "main_notebook");
  gtk_notebook_set_page (notebook, 0);
}


void
on_downloads1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkNotebook *notebook;

  //  Fetch the notebook widget
  notebook = (GtkNotebook *) lookup_widget (GTK_WIDGET (menuitem), 
					    "main_notebook");
  gtk_notebook_set_page (notebook, 1);
}


void
on_monitor1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkNotebook *notebook;

  //  Fetch the notebook widget
  notebook = (GtkNotebook *) lookup_widget (GTK_WIDGET (menuitem), 
					    "main_notebook");
  gtk_notebook_set_page (notebook, 2);
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog2;

  /*  Create the admin dialog  */
  dialog2 = create_dialog2 ();
  gtk_widget_show (dialog2);
}


gboolean
on_app_destroy_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_exit (0);
  return FALSE;
}


gboolean
on_app_delete_event                    (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  admin_exit ();
  return FALSE;
}


void
on_button9_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *addfiles;

  /*  Create the add files dialog  */
  addfiles = create_addfiles ();
  gtk_widget_show (addfiles);
}


void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  /*  Get the selection dialog  */
  GtkFileSelection *filesel = (GtkFileSelection *)
    lookup_widget (GTK_WIDGET (button), "addfiles");

  /*  Send the file(s) to the file list  */
  admin_file_add (gtk_file_selection_get_filename (filesel));

  /*  Hide the file selection dialog  */
  gtk_widget_hide (GTK_WIDGET (filesel));
}


void
on_cancel_button1_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (lookup_widget (GTK_WIDGET (button), "addfiles"));
}


gboolean
on_searchentry_key_press_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  GtkEntry *entry = GTK_ENTRY (widget);
  gchar *message = NULL;

  if (event->keyval == GDK_Return)
    {
      message = gtk_entry_get_text (entry);
      search_search (message);
      gtk_entry_select_region (entry, 0, strlen (message));
    }

  return FALSE;
}


void
on_searchbutton_clicked                (GtkButton       *button,
					gpointer         user_data)
{
  GtkEntry *entry = GTK_ENTRY (lookup_widget (app, "searchentry"));
  gchar *message = gtk_entry_get_text (entry);
  search_search (message);
  gtk_entry_select_region (entry, 0, strlen (message));
}


void
on_ok_button2_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  /*  Get the selection dialog  */
  GtkFileSelection *filesel = (GtkFileSelection *)
    lookup_widget (GTK_WIDGET (button), "downloaddir");

  /*  Send the file(s) to the file list  */
  admin_set_downloaddir (gtk_file_selection_get_filename (filesel), 1);

  /*  Hide the file selection dialog  */
  gtk_widget_hide (GTK_WIDGET (filesel));
}


void
on_cancel_button2_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (lookup_widget (GTK_WIDGET (button), "downloaddir"));
}


gboolean
on_hostentry_key_press_event           (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  return FALSE;
}


void
on_downloadbutton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected search results from the 
   *   listbox and send them to the search download function
   */
  GtkCList     *clist;
  GtkNotebook  *notebook;
  GList        *selection;
  SearchResult *result;
  gint          row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "searchclist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      result = (SearchResult *) gtk_clist_get_row_data (clist, row);
      search_download (result);
    }

  /*  Unselect the selected items  */
  gtk_clist_unselect_all (clist);

  gtk_clist_thaw (clist);

  /*  Fetch the notebook widget and switch page  */
  notebook = (GtkNotebook *) lookup_widget (app, "main_notebook");
  gtk_notebook_set_page (notebook, 1);
}


void
on_clearbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected search results from the 
   *   listbox and send them to the search download function
   */
  GtkCList     *clist;
  GList        *selection;
  DownloadFile *dfile;
  gint          row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "cdlist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      dfile = (DownloadFile *) gtk_clist_get_row_data (clist, row);
      download_clear (dfile);
    }

  /*  Unselect the selected items  */
  gtk_clist_unselect_all (clist);

  gtk_clist_thaw (clist);
}


void
on_abortbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected search results from the 
   *   listbox and send them to the search download function
   */
  GtkCList     *clist;
  GList        *selection;
  DownloadFile *dfile;
  gint          row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "adlist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      dfile = (DownloadFile *) gtk_clist_get_row_data (clist, row);
      download_abort (dfile);
    }

  /*  Unselect the selected items  */
  gtk_clist_unselect_all (clist);

  gtk_clist_thaw (clist);
}


void
on_disconnectbutton_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected hosts from the connected
   *   listbox and send them to the disconnect
   */
  GtkCList   *clist;
  GList      *selection;
  ServerNode *node;
  gint        row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "chostlist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      node = (ServerNode *) gtk_clist_get_row_data (clist, row);
      admin_node_disconnect (node);
    }

  /*  Unselect the selected items  */
  gtk_clist_unselect_all (clist);

  gtk_clist_thaw (clist);
}


void
on_connectbutton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected hosts from the connected
   *   listbox and send them to the disconnect
   */
  GtkCList   *clist;
  GList      *selection;
  ServerNode *node;
  gint        row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "khostlist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      node = (ServerNode *) gtk_clist_get_row_data (clist, row);
      admin_node_connect (node);
    }

  /*  Unselect the selected items  */
  gtk_clist_unselect_all (clist);

  gtk_clist_thaw (clist);
}


void
on_deletebutton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected hosts from the connected
   *   listbox and send them to the disconnect
   */
  GtkCList   *clist;
  GList      *selection;
  ServerNode *node;
  gint        row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "khostlist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      node = (ServerNode *) gtk_clist_get_row_data (clist, row);
      admin_node_delete (node);
    }

  gtk_clist_thaw (clist);
}


void
on_addhostbutton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkEntry *hostentry = GTK_ENTRY (lookup_widget (admin, "hostentry"));
  GtkEntry *portentry = GTK_ENTRY (lookup_widget (admin, "portentry"));
  gchar *server = gtk_entry_get_text (hostentry);
  gchar *port_str = gtk_entry_get_text (portentry);
  gint   port = (strlen (port_str) == 0) ? 6346 : atoi (port_str);
  ServerNode *new_node = NULL;

  if (strlen (server) == 0 ||
      strlen (port_str) > 5 ||
      port < 0 || port >= (1 << 16))
    {
      gtk_widget_show (create_hosterror ());
      g_print ("error add host: %s:%d\n", server, port);
      return;
    }

  new_node = g_new (ServerNode, 1);
  new_node->server = g_strdup (server);
  new_node->port = port;
  new_node->status = NODE_DISCONNECTED;

  admin_node_add (new_node);
  gtk_entry_set_text (hostentry, "");
  gtk_entry_set_text (portentry, "");
}


void
on_removefilesbutton_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  /*
   *  Get the list of selected hosts from the connected
   *   listbox and send them to the disconnect
   */
  GtkCList   *clist;
  GList      *selection;
  ServerFile *file;
  gint        row;

  /*  Get the selection list  */
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (button), "filelist"));
  gtk_clist_freeze (clist);

  selection = clist->selection;

  while (selection != NULL)
    {
      row = GPOINTER_TO_INT (selection->data);
      selection = g_list_next (selection);

      file = (ServerFile *) gtk_clist_get_row_data (clist, row);
      admin_file_remove (file);
    }

  gtk_clist_thaw (clist);
}


void
on_downloaddirbutton_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *setdir;

  /*  Create the add files dialog  */
  setdir = create_downloaddir ();
  gtk_widget_show (setdir);
}


void
on_speedcomboentry_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkEntry *entry = GTK_ENTRY (lookup_widget (admin, "speedcomboentry"));
  gchar    *linkspeed = gtk_entry_get_text (entry);
  gint      speed = 0;

  if (strcmp (linkspeed, "2.4 kbps") == 0)
    speed = 2400;
  else if (strcmp (linkspeed, "9.6 kbps") == 0)
    speed = 9600;
  else if (strcmp (linkspeed, "14.4 kbps") == 0)
    speed = 14400;
  else if (strcmp (linkspeed, "28.8 kbps") == 0)
    speed = 28800;
  else if (strcmp (linkspeed, "56.0 kbps") == 0)
    speed = 56000;
  else if (strcmp (linkspeed, "128.0 kbps") == 0)
    speed = 128000;
  else if (strcmp (linkspeed, "384.0 kbps") == 0)
    speed = 384000;
  else if (strcmp (linkspeed, "1500.0 kbps") == 0)
    speed = 1500000;
  else if (strcmp (linkspeed, "10000.0 kbps") == 0)
    speed = 10000000;
  else if (strcmp (linkspeed, "100000.0 kbps") == 0)
    speed = 100000000;

  admin_set_linkspeed (speed, FALSE);
}


gboolean
on_downloadentry_focus_out_event       (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  GtkEntry *entry = GTK_ENTRY (lookup_widget (admin, "downloadentry"));
  gchar    *downloaddir = gtk_entry_get_text (entry);

  /*  Send the file(s) to the file list  */
  admin_set_downloaddir (downloaddir, FALSE);

  return FALSE;
}


gboolean
on_admin_delete_event                  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_destroy (widget);
  admin = NULL;
  return FALSE;
}


gboolean
on_admin_destroy_event                 (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_destroy (widget);
  admin = NULL;
  return FALSE;
}


void
on_adminokbutton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  admin_save ();
  gtk_widget_hide (admin);
}


gboolean
on_maintaincountentry_focus_out_event  (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  GtkEntry *entry = GTK_ENTRY (lookup_widget (admin, "maintaincountentry"));
  gchar    *mc_str = gtk_entry_get_text (entry);
  gint      mc = atoi (mc_str);

  /*  Send the file(s) to the file list  */
  admin_set_maintaincount (mc, FALSE);

  return FALSE;
}

