/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "stream_config.h"

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <sys/param.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <dirent.h>
#include <assert.h>
#include <glib.h>


#include "stream_sched.h"
#include "stream_inet.h"
#include "stream_std.h"

#include "textutils.h"
#include "admin.h"
#include "search.h"
#include "monitor.h"
#include "download.h"
#include "log.h"
#include "hexdump.h"

#define FREE(func,x) (func)(x), (x) = NULL

typedef struct {
	int state;
#define DL_RCVHDR  0
#define DL_RCVDATA 1

	int content_length;
	int rcvbytes;
	FILE* fp;

	char wbuf[2048];
	char rbuf[16384];

	/*  ui data  */
	void *ui_data;
} dl_privdata;

void dl_cb
(int _s, short _events,
 char* _rbuf, int _rbytes,
 char* _wbuf, int _wbytes,
 void* _privdata) {
        DownloadFile *dfile;
	stream_host*  cli = (stream_host*) _privdata;
	dl_privdata*  privdata = (dl_privdata*) cli->privdata;

	/*  Get the ui data  */
	dfile = (DownloadFile *) privdata->ui_data;

	if (_events & STREAMCONNECT) {
		privdata->state = DL_RCVHDR;

		stream_sched_post_write(cli->s, privdata->wbuf, strlen(privdata->wbuf));
		stream_sched_post_read(cli->s, privdata->rbuf, sizeof(privdata->rbuf) - 1);
	}

	if (_events & STREAMHUP)
		goto killconn;

	if (_events & STREAMOUT) {
	}

	if (_events & STREAMIN) {
		switch (privdata->state) {
			case DL_RCVHDR:
				_rbuf[_rbytes] = '\0';
				if (!strstr(_rbuf, "\r\n\r\n")) {
					if (_rbytes == sizeof(privdata->rbuf) - 1)
						goto killconn;
					return;
				}

				if (!strstr(privdata->rbuf, "200 OK\r\n"))
					download_update (dfile, DOWNLOAD_ERROR);

				{
				char* content_length;

				if (!(content_length = strstr(privdata->rbuf, "Content-length:")) &&
				    !(content_length = strstr(privdata->rbuf, "Content-Length:"))) {
					download_update (dfile, DOWNLOAD_ERROR);
					goto killconn;
				}

				privdata->content_length = atoi(strstr(privdata->rbuf, "Content-length:") + strlen("Content-length:"));
				}

				{
				char* newlines;
				int nbytes;

				newlines = strstr(_rbuf, "\r\n\r\n");

				nbytes = privdata->rbuf + _rbytes - newlines - 4;

				if (fwrite(newlines + 4, 1, nbytes, privdata->fp) != nbytes) {
					download_update (dfile, DOWNLOAD_ERROR);
					goto killconn;
				}

				if (nbytes >= privdata->content_length) {
					download_update (dfile, DOWNLOAD_COMPLETE);
					goto killconn;
				}

				privdata->rcvbytes = nbytes;

				stream_sched_post_read(cli->s, privdata->rbuf, sizeof(privdata->rbuf) - 1);
				privdata->state = DL_RCVDATA;
				break;
			case DL_RCVDATA:
				if (fwrite(privdata->rbuf, 1, _rbytes, privdata->fp) != _rbytes) {
					download_update (dfile, DOWNLOAD_ERROR);
					goto killconn;
				}

				/* Update bytes received */

				privdata->rcvbytes += _rbytes;
				dfile->read += _rbytes;
				download_update (dfile, DOWNLOAD_ACTIVE);

				if (privdata->rcvbytes >= privdata->content_length) {
					download_update (dfile, DOWNLOAD_COMPLETE);
					goto killconn;
				}

				stream_sched_post_read(cli->s, privdata->rbuf, sizeof(privdata->rbuf) - 1);
			}
		}
	}

	return;

killconn:
	stream_host_close(cli);
	fclose(privdata->fp);
	FREE(free, privdata);
}

int network_get_file (char* _dns, int _port, int _index, char* _name, char* _ddir, void *ui_data) {
	char destfile[1024];
	dl_privdata* privdata;
	stream_host* cli;

	privdata = (dl_privdata*) malloc(sizeof(dl_privdata));
	/*  Set the ui data  */
	privdata->ui_data = ui_data;
	sprintf(privdata->wbuf, "GET /get/%d/%s HTTP/1.0\r\n"
							"Connection: Keep-Alive\r\n"
							"Range: bytes=0-\r\n"
							"\r\n",
							_index, _name);

	if (!(cli = stream_cli_new(_dns, _port, privdata, dl_cb))) {
		stream_err("cli create");
		return 1;
	}

	sprintf(destfile, "%s/%s", _ddir, _name);
	privdata->fp = fopen(destfile, "w+");
	if (!privdata->fp)
		return 1;

	return 0;
}
