/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Interface to the search functionality
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>

#include "network.h"
#include "interface.h"
#include "support.h"
#include "globals.h"
#include "search.h"
#include "download.h"

/*
 *  Static variables
 */
uuid current_search_id;

/*
 *  This function is called by the interface when the user
 *   submits a search string for search across the network
 */
void search_search (gchar *search_str)
{
  /*  Clear the search window */
  search_clear ();

  /* 
   *  Send the query to the network
   */
  current_search_id = network_send_query (search_str);
}

/*
 *  This function should be called by the server
 *   to clear the search list
 */
void search_clear (void)
{
  /*
   *  Clear the search file list
   */
  GtkCList *search_list = GTK_CLIST (lookup_widget (app, "searchclist"));
  GtkLabel *search_hits = GTK_LABEL (lookup_widget (app, "searchhitslabel"));

  gtk_clist_clear (search_list);
  gtk_label_set_text (search_hits, "0");
}

/*
 *  This function is called by the server to register the
 *   results of a search with the interface
 */
void search_result (SearchResult *result)
{
  /*
   *  Add the file to the searched files list
   */
  static gchar  hits_str[32];
  GtkCList     *search_list = NULL;
  GtkLabel     *search_hits = NULL;
  gchar        *text[4];
  gchar         server_str[32];
  gchar         size_str[32];
  gchar         speed_str[32];

  if (!uuidcmp(result->search_id, current_search_id))
	return;

  search_list = GTK_CLIST (lookup_widget (app, "searchclist"));
  search_hits = GTK_LABEL (lookup_widget (app, "searchhitslabel"));
  /*  Format column values  */
  text[0] = g_strdup (result->filename);
  sprintf (size_str, "%d", result->size);
  text[1] = g_strdup (size_str);
  sprintf (server_str, "%s:%d", result->server, result->port);
  text[2] = g_strdup (server_str);
  sprintf (speed_str, "%d", result->link_speed);
  text[3] = g_strdup (speed_str);
      
  /*  Insert into clist  */
  gtk_clist_append (search_list, text);
  gtk_clist_set_row_data (search_list, (search_list->rows - 1), result);

  /*  Update the number of search hits  */
  sprintf (hits_str, "%d", search_list->rows);
  gtk_label_set_text (search_hits, hits_str);
}

/*
 *  This function is called by the interface with a list
 *   of search files to fetch from the network
 */
void search_download (SearchResult *result)
{
  download_register (result);
}  


