/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "stream_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <netdb.h>

#include "stream_sched.h"
#include "stream_inet.h"
#include "stream_std.h"

stream_host* stream_cli_new
(char* _name_or_ip, u_int16_t _port, void* _privdata, stream_sched_cb _cb) {
	stream_host* client;
	struct sockaddr_in addr;
	struct hostent* hent;
	int one = 1;
	struct linger linger = { 0, 0 };

	client = (stream_host*) malloc(sizeof(stream_host));

	client->s = socket(AF_INET, SOCK_STREAM, 0);
	if (client->s == -1) {
		stream_err("socket");
		goto bad;
	}
	client->privdata = _privdata;

	if (fcntl(client->s, F_SETFL, O_NONBLOCK) == -1) {
		stream_err("fcntl");
		goto bad1;
	}

	if (setsockopt(client->s, SOL_SOCKET, SO_OOBINLINE, &one, sizeof(one)) == -1) {
		stream_err("setsockopt oobinline");
		goto bad1;
	}
	if (setsockopt(client->s, SOL_SOCKET, SO_KEEPALIVE, &one, sizeof(one)) == -1) {
		stream_err("setsockopt keepalive");
		goto bad1;
	}
	if (setsockopt(client->s, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger)) == -1) {
		stream_err("setsockopt linger");
		goto bad1;
	}

	hent = gethostbyname(_name_or_ip);
	if (!hent)
		addr.sin_addr.s_addr = inet_addr(_name_or_ip);
	else
		memcpy(&addr.sin_addr, hent->h_addr, hent->h_length);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(_port);

	if (connect(client->s, (struct sockaddr*) &addr, sizeof(addr)) == -1) {
		if (errno != EINPROGRESS) {
			stream_err("connect");
			goto bad1;
		}
	}

	if (stream_sched_add_connect_fd(client->s, _cb, client) == -1) {
		stream_err("add_connect_fd");
		goto bad1;
	}

	return client;

bad1:
	close(client->s);
bad:
	free(client);
	return NULL;
}
