/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Interface to the monitor functionality
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "globals.h"

#define MAX_LOG_RESULTS 100

typedef struct _LogMessage
{
  gchar *time;
  gchar *message;
} LogMessage;

/*
 *  This function is called by the server to log a message
 *   to the monitor (ie. as each new search request is received
 */
void log_append (gchar *message)
{
  GtkCList   *clist = GTK_CLIST (lookup_widget (app, "loglist"));
  LogMessage *log;
  struct tm  *tm;
  time_t      timet;
  gchar       time_buf[256];
  gchar      *text[2];

  /*  Check the number of listbox rows  */
  if (clist->rows > MAX_LOG_RESULTS)
    {
      log = (LogMessage *) gtk_clist_get_row_data (clist, clist->rows - 1);

      /*  Remove the last row  */
      gtk_clist_remove (clist, clist->rows - 1);
	  
      /*  Free the result  */
      g_free (log->time);
      g_free (log->message);
      g_free (log);
    }

  log = g_new (LogMessage, 1);

  time (&timet);
  tm = localtime (&timet);
  strftime (time_buf, 256, "%H:%M:%S", tm);
  log->time = g_strdup (time_buf);
  log->message = g_strdup (message);

  text[0] = log->time;
  text[1] = log->message;

  gtk_clist_prepend (clist, text);
  gtk_clist_set_row_data (clist, 0, log);
}



