/*
 *  Gnubile - Gnutella software for Unix
 *  Copyright (C) 2000  Spencer Kimball and Gene Kan
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "stream_config.h"

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "globals.h"
#include "admin.h"
#include "network.h"
#include "stream_sched.h"

/*  Extern declarations  */
extern stream_sched* g_sched;

/*  Global variable definitions  */
GtkWidget *app = NULL;
GtkWidget *admin = NULL;


/*
 *  The idle stream scheduler
 */
gint gnubile_idle_function (gpointer data)
{
  static gint count = 0;

  stream_sched_run(100);

  /*
   *  Finally, attempt to maintain the desired connected host
   *   count (only do this once per minute)
   */
  if (count == 0)
    admin_node_balance ();
  
  count += 1;
  if (count == 1000)
    count = 0;

  return TRUE;
}

unsigned int g_listenport;

/*
 *  The main function
 */
int
main (int argc, char *argv[])
{
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  app = create_app ();
  gtk_widget_show (app);
  admin = create_admin ();

  /*
   *  Initialize the network
   */
  network_init ();

  if (argc > 1) {
    g_listenport = atoi(argv[1]);
    if (network_start_server(g_listenport) != 0) {
	  printf("Port %d unavailable.  Try another port.\n", g_listenport);
	  exit(1);
	}
  }

  /*
   *  Load the user preferences
   */
  admin_load ();

  /*
   *  Add the idle processor
   */
  gtk_idle_add (gnubile_idle_function, NULL);

  /*
   *  Start the main processing loop
   */
  gtk_main ();

  return 0;
}


