/* Josh Pieper, (c) 2000 */

/* This file is distributed under the GPL, see file COPYING for details */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "lib.h"
#include "threads.h"
#include "hash.h"
#include "conf.h"

conf_key_pair configuration_pairs[] = {
  {"auto_clear",      "1"},
  {"auto_download_retry", "60"},
  {"auto_response",   "1"},
  {"autokill_thres",  "10"},
  {"beep_on_all",     "0"},
  {"beep_on_first",   "0"},
  {"cache_path",      ""},
  {"cache_refresh",   "600"},
  {"cache_size",      "15000"},  /* 15 megs.  Should this be measured in M instead of K? */
  {"config_file",     ""},
  {"debug_opts",      "0"},
  {"default_download_cap", "0"},
  {"default_upload_cap", "0"},
  {"download_path",   ""},
  {"download_timeout","60"},
  {"eval_echo",       "1"},
  {"global_bandwidth_cap", "0"},
  {"guid",            "AAAAAAAAAAAAAAAA"},
  {"hidden",          "0"},
  {"hide_pathname",   "0"},
  {"html_enable",     "1"},
  {"html_template",   ""},
  {"http_client_headers", "User-Agent: Gnut " VERSION "\\n"
                          "Referrer: Gnutella file sharing network\\n"
                          "Connection: Keep-Alive\\n" },
  {"leave_turds",     "512"},
  {"local_port",      "5634"},
  {"log_level",      "0"},
  {"mac",             "ADCVFD"},
  {"max_downloads",   "10"},
  {"max_incoming",    "4"},
  {"max_responses",   "1000"},
  {"max_results",     "64"},
  {"max_searches",    "10"},
  {"max_uploads",     "10"},
  {"min_cache_size",  "15"},
  {"min_connections", "4"},
  {"multi_enable",    "0"},
  {"munge",           "1"},
  {"no_rfc_1597",     "0"},
  {"packet_stats",    "0"},
  {"paginate",        "1"},
  {"password",        ""},
  {"password_required","0"},
  {"prompt",          "gnut> "},
  {"play_prog",       "xmms"},
  {"query_guid",      "AAAAAAAAAAAAAAAA"},
  {"redirect",        "0"},
  {"retry_push",      "1"},
  {"search_extensions","*"},
  {"search_log",      ""},
  {"search_min_size", "0"},
  {"share_paths",     ""},
  {"sort_order",      "s-p-"},
  {"speed",           "28"},
  {"stats_format",    "0"},
  {"strict_search",   "1"},
  {"transfer_log",    ""},
  {"ttl",             "7"},
  {"update_clear",    "0"},
  {"update_guid",     "AAAAAAAAAAAAAAAA"},
  {"verbose",         "3"},
  {"wait_after_find", "1"},
  {0, 0}
};

Gnut_Hash *conf_hash=0;

int gc_ttl;
int gc_debug_opts;
int gc_eval_echo;
int gc_hide_pathname;
int gc_no_rfc_1597;
int gc_strict_search;
int gc_search_min_size;
FILE *gc_searchlog = 0;
FILE *gc_transferlog = 0;
int gc_verbose;

uchar conf_hash_struct(void *arg)
{
  conf_key_pair *ckp;
  int i,len;
  uchar ret;

  ckp = arg;
  len = strlen(ckp->ckp_key);
  for (i=0,ret=0; i<len; i++) {
    ret += ckp->ckp_key[i];
  }

  return ret;
}

int conf_compare_struct(void *arg1, void *arg2)
{
  conf_key_pair *ckp1;
  conf_key_pair *ckp2;

  ckp1=arg1;
  ckp2=arg2;

  return strcmp(ckp1->ckp_key, ckp2->ckp_key);
}

/* conf_cache_entry performs two different but related functions.
 *
 * The first is to save certain configuration values which, for
 * performance reasons, must be kept in real global variables. An
 * example is the "TTL" value, which is referred to all the time
 * in the packet parsing and routing code.
 *
 * The other function is to perform actions that must be performed as
 * a consequence of setting a configuration value. An example is
 * the "search_log" setting -- as soon as you set it, the logfile
 * should be opened (if possible), and if you change it, the existing
 * logfile should be closed and a new one opened.
 *
 * conf_cache_entry is called by each of the other conf_routines,
 * so the global variables and immediate actions are always in sync
 * with the configuration variables. */
void conf_cache_entry(char *key)
{
  int val;
  char *sval;
  
  if (strcmp(key, "debug_opts") == 0) {
    val = conf_get_int(key);
    gc_debug_opts = val;
  } else if (strcmp(key, "eval_echo") == 0) {
    val = conf_get_int(key);
    gc_eval_echo = val;
  } else if (strcmp(key, "hide_pathname") == 0) {
    val = conf_get_int(key);
    gc_hide_pathname = val;
  } else if (strcmp(key, "log_level") == 0) {
    val = conf_get_int(key);
    gnut_lib_debug = val;
  } else if (strcmp(key, "no_rfc_1597") == 0) {
    val = conf_get_int(key);
    gc_no_rfc_1597 = val;
  } else if (strcmp(key, "search_log") == 0) {
    if (gc_searchlog) {
      fclose(gc_searchlog);
    }
    sval = conf_get_str(key);
    if (sval && *sval) {
      gc_searchlog = fopen(sval, "a");
      if (gc_searchlog) {
	setvbuf(gc_searchlog, 0, _IONBF, 0);
      }
    }
  } else if (strcmp(key, "strict_search") == 0) {
    val = conf_get_int(key);
    gc_strict_search = val;
  } else if (strcmp(key, "search_min_size") == 0) {
    val = conf_get_int(key);
    gc_search_min_size = val;
  } else if (strcmp(key, "transfer_log") == 0) {
    if (gc_transferlog) {
      fclose(gc_transferlog);
    }
    sval = conf_get_str(key);
    if (sval && *sval) {
      gc_transferlog = fopen(sval, "a");
      if (gc_transferlog) {
	setvbuf(gc_transferlog, 0, _IONBF, 0);
      }
    }
  } else if (strcmp(key, "ttl") == 0) {
    val = conf_get_int(key);
    gc_ttl = val;
  } else if (strcmp(key, "verbose") == 0) {
    val = conf_get_int(key);
    gc_verbose = val;
  }
}

int conf_init()
{
  conf_key_pair *ckp;
  int i;
  
  conf_hash = gnut_hash_new(conf_hash_struct, conf_compare_struct);

  for(i=0; configuration_pairs[i].ckp_key; i++) {
    ckp = (conf_key_pair *) ymaloc(sizeof(conf_key_pair), 342);
    ckp->ckp_key = ystdup(configuration_pairs[i].ckp_key, 466);
    ckp->ckp_val = ystdup(configuration_pairs[i].ckp_val, 467);
    gnut_hash_insert(conf_hash, ckp, 501);
    conf_cache_entry(ckp->ckp_key);
  }
  
  return 0;
}

int conf_get_int(char *key)
{
  conf_key_pair ckp;
  conf_key_pair *ckp_ret;
  
  ckp.ckp_key=key;
  
  ckp_ret=gnut_hash_find(conf_hash, &ckp);
  
  if (!ckp_ret) {
    gd_s(0, "conf_get_int not found ");
    gd_s(0, key);
    gd_s(0, "\n");
    return -1;
  }
  
  return atoi(ckp_ret->ckp_val);
}  

int conf_set_int(char *key, int val)
{
  conf_key_pair ckp;
  conf_key_pair *ckp_ret;

  ckp.ckp_key=key;
  
  ckp_ret=gnut_hash_find(conf_hash, &ckp);
  
  if (!ckp_ret) {
    gd_s(0, "conf_set_int not found ");
    gd_s(0, key);
    gd_s(0, "\n");
    return -1;
  }
  
  if (ckp_ret->ckp_val) {
    fre_str(&(ckp_ret->ckp_val), 397);
  }

  ckp_ret->ckp_val= ymaloc(20, 343);
  sprintf(ckp_ret->ckp_val, "%i", val);

  conf_cache_entry(key);

  return 0;
}

char *conf_get_str(char *key)
{
  conf_key_pair ckp,*ckp_ret;
  
  ckp.ckp_key=key;
  
  ckp_ret=gnut_hash_find(conf_hash, &ckp);
  
  if (!ckp_ret) {
    gd_s(0, "conf_get_str not found ");
    gd_s(0, key);
    gd_s(0, "\n");
    return 0;
  }
  if (!ckp_ret->ckp_val) {
    return 0;
  }
  
  return ckp_ret->ckp_val;
}

int conf_set_str(char *key, char *val)
{
  conf_key_pair ckp;
  conf_key_pair *ckp_ret;
  
  ckp.ckp_key = key;
  
  ckp_ret = gnut_hash_find(conf_hash, &ckp);
  
  if (!ckp_ret) {
    gd_s(0, "conf_set_str not found ");
    gd_s(0, key);
    gd_s(0, "\n");
    return -1;
  }
  if (ckp_ret->ckp_val) {
    fre_str(&(ckp_ret->ckp_val), 398);
  }
  
  if (val) {
    ckp_ret->ckp_val = ystdup(val, 468);
  } else {
    ckp_ret->ckp_val=0;
  }

  conf_cache_entry(key);
  
  return 0;
}

int conf_set_str_len(char *key, char *val, int len)
{
  conf_key_pair ckp, *ckp_ret;
    
  ckp.ckp_key = key;
  
  ckp_ret = gnut_hash_find(conf_hash, &ckp);
  
  if (!ckp_ret) {
    gd_s(0, "conf_set_str_len not found ");
    gd_s(0, key);
    gd_s(0, "\n");
    return -1;
  }
  
  if (ckp_ret->ckp_val) {
    fre_str(&(ckp_ret->ckp_val), 399);
  }
  
  if (val) {
    ckp_ret->ckp_val = (char *) ymaloc(len, 344);
    memcpy(ckp_ret->ckp_val, val, len);
  } else {
    ckp_ret->ckp_val = 0;
  }

  conf_cache_entry(key);

  return 0;
}
