/* Josh Pieper, (c) 2000
   Robert Munafo, (c) 2001

   This file is distributed under the GPL, see file COPYING for details */

#ifndef GNUT_LIB_H
#define GNUT_LIB_H 0

#ifdef WIN32
# define VERSION "0.4.24/win32"
#endif 
 
/* Highest used bugnum so far: 519 */
/* Highest used dqi so far: 0x0075 */

/* close_f is for closing a file. This is close() under both Windows and
 * Unix. */
#define close_f(f) close((f))

#include <sys/types.h>
#ifndef WIN32
# include <sys/socket.h>
# include <netinet/in.h>
#endif

#ifdef DMALLOC
# include <dmalloc.h>
#endif

#ifndef MIN
# define MIN(a,b) (((a < b)) ? (a) : (b))
# define MAX(a,b) (((a > b)) ? (a) : (b))
#endif

#ifndef __bswap_16
# define __bswap_16(x) \
    ((((x) >> 8) & 0xff) | (((x) &0xff) <<8))
#endif
      
#ifndef __bswap_32
# define __bswap_32(x) \
    ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >> 8) | \
    (((x) & 0x0000ff00) << 8) | (((x) & 0x000000ff) << 24))
#endif
                                                
#ifndef GUINT16_TO_LE
# ifdef WORDS_BIGENDIAN
#  define GUINT16_TO_LE(x)  __bswap_16(x)
#  define GUINT32_TO_LE(x)  __bswap_32(x)
#  define GUINT16_FROM_LE(x)  __bswap_16(x)
#  define GUINT32_FROM_LE(x)  __bswap_32(x)
# else
#  define GUINT16_TO_LE(x)  (x)
#  define GUINT32_TO_LE(x)  (x)
#  define GUINT16_FROM_LE(x)  (x)
#  define GUINT32_FROM_LE(x)  (x)
# endif
#endif

#define g_debug undefined_g_debug1 undefined_g_debug2;

#define OCTAL(a,b,c) (((a) << 6) | ((b) << 3) | (c))

#define IS_DIGIT(c) (((c) >= '0') && ((c) <= '9'))

typedef unsigned int   uint32;
typedef int             int32;
typedef unsigned short uint16;
typedef unsigned char  uchar;
#ifndef WIN32
 typedef unsigned long long uint64;
#endif

#ifdef __cplusplus
extern "C" {
#endif

  extern char *path_separator;
  extern char *path_slash;

  extern int gnut_lib_debug;

#define DQ_SIZE 256
  extern uint16 dq[DQ_SIZE];
#define DQT_CHUNK_SIZE 128
#define DQT_CHUNK_NUM 64
#define DQT_CHUNK_MASK 31
  extern uint16 dqt[DQT_CHUNK_SIZE * DQT_CHUNK_NUM];
  extern int32 dqtp[DQT_CHUNK_NUM];
  extern int32 dqp;
  extern uint16 dqtt;

  void dq_init(void);

#define DQ_TID (((int) pthread_self()) & DQT_CHUNK_MASK)
  void dq_start(char *thtype, uint16 dqinum);
  void dq_end(void * retval);

#define dqi(x) (dq[dqp] = (x), dqp = (dqp + 1) % DQ_SIZE, \
		dqtt = DQ_TID, \
		dqt[(dqtt * DQT_CHUNK_SIZE) + dqtp[dqtt]] = (x), \
		dqtp[dqtt] = (dqtp[dqtt] + 1) % DQT_CHUNK_SIZE, 0)
  void dq_dump(char *msg);

#define gnut_mprobe(x) _gnut_mprobe(x)
  int gnut_mprobe(void *);

  char *expand_path(char *);

  void init_leaks(void);
  void reset_leaks(void);
  void check_reset(void);
  void tell_leaks(void);
  void yfre(void **, int tracking_number);
  void fre_v(void **x, int bugnum);
  void fre_strl(char ***x, int bugnum);
  void fre_str(char **x, int bugnum);
  void *ymaloc(int size, int tracking_number);
  char *ystdup(const char *s, int tracking_number);
  void *ycaloc(size_t nmemb, size_t size, int tracking_number);

#ifdef WIN32
#include <windows.h>

#define HAVE_STRTOK_R

  typedef signed __int64 uint64;
  typedef unsigned int uint;

#define stricmp(x,y) strcasecmp(x,y)
#define sleep(x) Sleep(x*1000)
#define usleep(x) Sleep(x)
  char *strtok_r(char *, const char *, char **);
  int getopt(int argc, char *argv[], char *);
  extern char *optarg;
  extern int optind;

#define strncasecmp strnicmp
#define strcasecmp stricmp

#define inet_aton(string, ip) (ip)->s_addr = inet_addr(string)

#define write(sock, buf, len) send((sock), (buf), (len), 0)
#define read(sock, buf, len) recv((sock), (buf), (len), 0)
#define close_s(sock) closesocket(sock)

#define F_SETFL 1
#define O_NONBLOCK 1
  int fcntl(int sock, int, uint);

#define flock(a,b) 0
#else
#define close_s(sock) close(sock)
#endif

  void gd_s(int level, char * str);
  void gd_02x(int level, uchar c);
  void gd_i(int level, int32 x);
  void gd_x(int level, int32 x);
  void gd_li(int level, int32 x);
  void gd_lu(int level, uint32 x);
  void gd_p(int level, void *p);

  char *gnut_strdelimit(char *string, char *delimeters, char new_delim);
  void gnut_strstrip(char *string);
  char *munge_time(int secs);
  char *munge_bytes(uint64 bytes);

  int strlen_an(char *s);
  void crc32_start(uint32 *crc);
  void crc32_add8(uint32 *crc, uchar data);  
  uint32 crc32_string(char *data);
  uint32 crc32_block(void * data, uint32 len);
  void print_ascii(uchar *s);
  void print_asc_n(uchar *s, int len);
  int writestr(int sock, char *str);
  int trycmd(char *cmd, char *pat);
  void strcatlim(char *dest, char *src, int maxlen);

#ifdef __cplusplus
}
#endif

#endif
