/* Josh Pieper, (c) 2000 */

/* This file is distributed under the GPL, see file COPYING for details. */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "lib.h"
#include "queue.h"

void fre_gq(Gnut_Queue **x, int bugnum)
{
  yfre((void **) x, bugnum);
}

Gnut_Queue * gnut_queue_new(int tracking_number)
{
  Gnut_Queue *a;

  a = (Gnut_Queue *) ymaloc(sizeof(Gnut_Queue), tracking_number);
  a->head = 0;
  a->tail = 0;
  a->size = 0;
  return a;
}

void gnut_queue_delete(Gnut_Queue *q, void *d)
{
  Gnut_List *tmp;
  int flag=0;
  
  if (q->tail && d==q->tail->data) {
    flag=1;
  }
  
  q->head = gnut_list_remove(q->head, d);

  if (flag==1) {
    /* we need to reset tail... */
    for (tmp = q->head; tmp; tmp = tmp->next) {
      if (tmp->next == 0) {
	q->tail = tmp;
      }
    }
  }
}  

void gnut_queue_insert(Gnut_Queue *q, void *d, int tracking_number)
{
  Gnut_List *tmp;

  if (!q) {
    gd_s(0, "gnut_queue_insert q=");
    gd_p(0, q);
    gd_s(0, " d=");
    gd_p(0, d);
    gd_s(0, "\n");
    exit(0);
  }
  gd_s(4,"gnut_queue_insert entering q=");
  gd_p(4, q);
  gd_s(4, " d=");
  gd_p(4, d);
  gd_s(4, "\n");

  tmp = (Gnut_List *) ymaloc(sizeof(Gnut_List), tracking_number);
  tmp->data = d;
  tmp->next = 0;
  q->size++;

  gd_s(4, "new size: ");
  gd_i(4, q->size);
  gd_s(4, "\n");

  if (q->head == 0) {
    q->head = q->tail = tmp;
  } else {
    gnut_mprobe(q->tail);
    q->tail->next = tmp;
    q->tail = tmp;
  }
}

/* Remove an item from the end of a queue, fr.ee the queue node but don't
 * fr.ee the data itself */
void * gnut_queue_remove(Gnut_Queue *q)
{
  Gnut_List *gltmp;
  void *ret;
  
  gnut_mprobe(q);
  
  if (!q) {
    gd_s(0, "gnut_queue_remove q=");
    gd_p(0, q);
    gd_s(0, "\n");
    exit(0);
  }
  
  gltmp=q->head;
  
  if (gltmp) {
    gnut_mprobe(gltmp);
    
    q->head=q->head->next;
    if (q->head==0) {
      q->tail=0;
    }
  
    ret=gltmp->data;
    fre_gl(&gltmp, 407);
  
    q->size--;
    return ret;
  }
  /* else */
  return 0;
}

Gnut_List * gnut_queue_list(Gnut_Queue *q)
{
  if (!q) {
    gd_s(0, "gnut_queue_list q=");
    gd_p(0, q);
    gd_s(0, "\n");
    exit(0);
  }
  return q->head;
}

/* Removes all the elements in a queue, but foes not fr.ee the header
   structure *q */
void gnut_queue_fre(Gnut_Queue *q)
{
  Gnut_List *gltmp;
  Gnut_List *gltmp2;

  gd_s(2, "gnut_queue_fre entering q=");
  gd_p(2, q);
  gd_s(2, "\n");

  if (!q) {
    gd_s(0, "q=");
    gd_p(0, q);
    gd_s(0, "\n");
    exit(0);
  }

  for (gltmp = q->head; gltmp; gltmp = gltmp2) {
    gltmp2 = gltmp->next;
    if (gltmp->data) {
      fre_v(&(gltmp->data), 408);
    }
    fre_gl(&gltmp, 409);
  }
  gd_s(2, "gnut_queue_fre returning\n");
}
