/* Josh Pieper, (c) 2000 */

/* This file is distributed under the GPL, see file COPYING for details */

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#ifndef WIN32
#include <unistd.h>
#endif

#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <ctype.h>

#include "hash.h"
#include "lib.h"
#include "queue.h"
#include "share.h"
#include "list.h"


/* here we declare share's global variables... */
int share_num=0;
double share_size=0;
Gnut_List *share_root=0;

#ifndef PTHREADS_DRAFT4
pthread_mutex_t share_mutex=PTHREAD_MUTEX_INITIALIZER;
#else
pthread_mutex_t share_mutex;
#endif

Gnut_Hash *share_cache_hash = 0;
Gnut_Queue *share_cache_queue = 0;

int have_find, have_perl;

void fre_si(share_item **x, int bugnum)
{
  yfre((void **) x, bugnum);
}

void make_lc(char *str)
{
  char *s;

  for(s=str; *s; s++) {
    if ((*s >= 'A') && (*s <= 'Z')) {
      *s = (*s) + 'a' - 'A';
    }
  }
}

/* is_match_tok does a boolean match test on a string (str) using
 * a query string (query). Both are standard C strings. The third
 * parameter "and" should be 0 for a boolean OR match, and 1 for
 * a boolean AND match. */
int is_match_tok(char *query, char *str, int and, int ignorecase)
{
  char *a,*b;
  char *q2;
  char *s2;
  int result;
  char *loc;

  gd_s(5, "ismatch_tok entering\n");

  /* Copy query and str so make_lc and strtok doesn't trash them */
  q2 = ystdup(query, 436);
  s2 = ystdup(str, 437);

  if (ignorecase) {
    make_lc(q2);
    make_lc(s2);
  }

  /* First, break the query string into words ("tokens"). For example,
   * if the query is "john smith .jpg" the tokens will be "john",
   * "smith" and "jpg". */
#ifdef HAVE_STRTOK_R
  a = strtok_r(q2, " \t", &b);
#else
  a = strtok(q2, " \t");
#endif

  if (a == 0) {
    fre_str(&q2, 155);
    fre_str(&s2, 156);
    return 0;
  }

  /* Check each token to see if it's in the string */
  if (and) {
    result = 1;
  } else {
    result = 0;
  }
  while (a) {
    gd_s(5, "ismatch_tok checking for \"");
    gd_s(5, a);
    gd_s(5, "\" in \"");
    gd_s(5, s2);
    gd_s(5, "\" \n");
    loc = strstr(s2, a);
    if (loc == 0) {
      /* Not found */
      if (and) {
        result = 0;
      }
    } else {
      /* Found at position loc */
      int i;

      if (!(and)) {
        result = 1;
      }
      /* "Erase" the matched text, so that two occurrances of the
       * same keyword in query will require two occurrances of that
       * word in the string. Note this affects AND matches only, not
       * OR matches */
      for(i=0; a[i]; i++) {
	if (loc[i]) {
          loc[i] = ' ';
        }
      }
    }
#ifdef HAVE_STRTOK_R
    a = strtok_r(0, " \t", &b);
#else
    a = strtok(0, " \t");
#endif
  }

  if (result == 0) {
    gd_s(5, "ismatch_tok returning failure\n");
  } else {
    gd_s(5, "ismatch_tok returning success\n");
  }

  fre_str(&q2, 157);
  fre_str(&s2, 158);
  return result;
}

/* Gnut_List *share_search(char *query, int max)
 *
 * searches through the shares, and returns a list of up to <max>
 * shares that match the query */
Gnut_List *share_search(char *orig_query, int max)
{
  Gnut_List *nlist=0;
  Gnut_List *ltmp;
  share_item *si;
  share_item *copy;
  char *myquery;
  char *query;
  int count;
  int i,len;
  
  gd_s(3, "share_search entering\n");
  
  query=ystdup(orig_query, 438);
  gnut_strdelimit(query,"*?._<>|[]{}()_=+-/!@#$%^&",' ');
  
  len=strlen(query);
  for (i=0;i<len;i++) {
    query[i]=tolower(query[i]);
  }
  
  count=0;
  myquery=ystdup(query, 439);
  
  for (ltmp=share_root; ltmp; ltmp=ltmp->next) {
    gd_s(4, "share_search ltmp=");
    gd_p(4, ltmp);
    gd_s(4, " myquery=");
    gd_p(4, myquery);
    gd_s(4, " query=");
    gd_p(4, query);
    gd_s(4, "\n");
    /* see if this share matches... */
    si=ltmp->data;
    strcpy(myquery,query);
    
    gnut_mprobe(ltmp);
    gnut_mprobe(myquery);
    gnut_mprobe(query);
    
    if (is_match_tok(myquery, si->upath, 1, 0)) {
      /* we need to make a copy of this node, and
       * insert into the result list... */
      copy=(share_item *) ymaloc(sizeof(share_item), 289);
      
      copy->path=ystdup(si->path, 440);
      copy->upath=ystdup(si->upath, 441);
      copy->fpath=ystdup(si->fpath, 442);
      copy->size=si->size;
      copy->ref=si->ref;
      
      nlist=gnut_list_prepend(nlist, copy);
      
      if ((++count)>max)
	break;
    }
  }
  fre_str(&myquery, 161);
  fre_str(&query, 162);
  
  gd_s(3, "share_search returning\n");
  return nlist;  
} 

/* Gnut_List *share_get_root()
 *
 * returns the share list, used to do ui */
Gnut_List *share_get_root()
{
  return share_root;
}

/* int share_totals(int *num, int *size)
 *
 * returns total number of shared files, and their total size
 * in KB */
int share_totals(int *num, int *size)
{
  gd_s(3,"share_totals entering\n");
  *num=share_num;
  *size = (int) (share_size / 1024.);
  gd_s(3, "share_totals returning share_num=");
  gd_i(3, share_num);
  gd_s(3, " share_size=");
  gd_i(3, ((int) (share_size / 1024.)));
  gd_s(3, " K  *num=");
  gd_i(3, *num);
  gd_s(3, " *size=");
  gd_i(3, *size);
  gd_s(3, "\n");
  return 0;
}

int share_insert(void *data, void *user_data)
{
  share_item *si,*nsi;
  Gnut_List **l;

  l=user_data;
  
  si=data;
  nsi=(share_item *) ymaloc(sizeof(share_item), 290);
  memcpy(nsi,si,sizeof(share_item));
  nsi->path=ystdup(si->path, 443);
  nsi->upath=ystdup(si->upath, 444);
  nsi->fpath=ystdup(si->fpath, 445);
  
  *l=gnut_list_prepend(*l,nsi);
  return 0;
}

Gnut_List * share_copy_list(Gnut_List *l)
{
  Gnut_List *nl=0;
  gd_s(3, "share_copy_list entering\n");
  
  gnut_list_foreach(l,share_insert,&nl);
  
  return nl;
}

/* callback, fr.ees up path in share_item */
int share_delete(void * data, void * user_data)
{
  share_item *si;

  gd_s(3, "share_delete entering\n");

  si=data;
  fre_str(&(si->path), 163);
  fre_str(&(si->upath), 164);
  fre_str(&(si->fpath), 165);
  gd_s(3,"share_delete returning\n");
  return 0;
}

/* int share_clear_list(Gnut_List *l) 
 *
 * clears the share list specified by l */
int share_clear_list(Gnut_List *l)
{
  gd_s(3,"share_clear_list entering\n");

  gnut_list_foreach(l, share_delete, 0);
  gnut_list_fre(l);

  gd_s(3,"share_clear_list returning\n");
  return 0;
}
  
  
/* int share_clear()
 *
 * clears out the list of shares */
int share_clear()
{
  gd_s(3,"share_clear entering\n");
  gnut_list_foreach(share_root, share_delete, 0);
  gnut_list_fre(share_root);
  share_root=0;
  share_num=0;
  share_size=0.;
  gd_s(3,"share_clear returning\n");
  return 0;
}

/* share_item *share_find(int ref)
 *
 * locates the share indexed by ref and returns it */
share_item *share_find(int ref)
{
  Gnut_List *ltmp;
  share_item *si,*si2;

  gd_s(3,"share_find entering\n");

  for (ltmp=share_root; ltmp; ltmp=gnut_list_next(ltmp)) {
    si=ltmp->data;
    if (si->ref==ref) {
      /* we found the correct one, now we'll make a copy
       * of it and return... */
      si2=(share_item *) ymaloc(sizeof(share_item), 291);
      si2->ref=si->ref;
      si2->size=si->size;
      si2->path=ystdup(si->path, 427);
      si2->upath=ystdup(si->upath, 428);
      si2->fpath=ystdup(si->fpath, 429);
      gd_s(3, "share_find returning si2=");
      gd_p(3, si2);
      gd_s(3, "\n");
      return si2;
    }
  }
  gd_s(3,"share_find returning failure\n");
  return 0;
}

#ifdef WIN32
void log_win32_error(char *szFunctionName)
{
  LPVOID lpMsgBuf;
  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_IGNORE_INSERTS,
				0, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
				(LPSTR) &lpMsgBuf, 0, 0);
    
  gd_s(3, "log_win32_error ");
  gd_s(3, szFunctionName);
  gd_s(3, ": ");
  gd_s(3, (LPSTR) lpMsgBuf);
  gd_s(3, "\n");
  LocalFree(lpMsgBuf);
}

/* copies into pszFilePath a version of szPath that is ensured to end
 * in a "\*.*" */

void cleanup_directory_path(char *szPath, char *pszFilePath,
							int *pnFilePathLength)
{
  strcpy(pszFilePath, szPath);
  *pnFilePathLength = strlen(szPath);
  
  /* TODO: if szPath ends in \*.* bail out */
  
  if (L'\\' != *( pszFilePath + *pnFilePathLength-1)) {
    strcat(pszFilePath, TEXT("\\"));
  }
    
  strcat(pszFilePath, TEXT("*.*"));
}

int share_path_len=0;

int share_scan_dir_win32(char *szPath)
{
  char szFilePath[OUR_MAX_PATH+1];
  char szRootFilePath[OUR_MAX_PATH+1];
  ULONG  nFilePathLength;
  DWORD  dwFileAttributes;
  HANDLE  hSearch;
  WIN32_FIND_DATA  Win32FindData;
  share_item *si;
  int i, len;
  
  dwFileAttributes = GetFileAttributesA( szPath );
  if (0xFFFFFFFF == dwFileAttributes)
    return 0;
  
  if (dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
    cleanup_directory_path(szPath, szRootFilePath, &nFilePathLength);
  }
  
  hSearch = FindFirstFile (szRootFilePath, &Win32FindData);
  if (INVALID_HANDLE_VALUE == hSearch) {
    log_win32_error("share_scan_dir_win32");
    return 0;
  }
  
  /* Make sure we ignore the "." and ".." files */
  while (!strcmp(Win32FindData.cFileName, TEXT("."))
    || !strcmp(Win32FindData.cFileName, TEXT("..")))
  {
    if (!FindNextFile(hSearch, &Win32FindData)) {
      log_win32_error("share_scan_dir_win32");
      FindClose(hSearch);
      return 0;
    }
  }
  
  do {
    strcpy(szFilePath, szPath);
    nFilePathLength = strlen( szFilePath );
    if (L'\\' != *(szFilePath + nFilePathLength - 1)) {
      strcat(szFilePath, TEXT("\\"));
    }
    strcat(szFilePath, Win32FindData.cFileName);
    
    if (Win32FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
      gd_s(3, "share_scan_dir_win32: scanning %s \n",szFilePath);
      share_scan_dir_win32(szFilePath);
    } else {
      si = (share_item *)ymaloc(sizeof(share_item), 292);
      si->size=Win32FindData.nFileSizeLow;
      
      share_size += (double) si->size;
      si->ref=++share_num;
      si->path=ystdup(szFilePath+share_path_len+1, 430);
      gnut_strdelimit(si->path,path_slash,'/');
      si->upath=ystdup(szFilePath+share_path_len+1, 431);
      gnut_strdelimit(si->upath,path_slash,'/');
      si->fpath=ystdup(szFilePath, 432);
      
      len=strlen(si->upath);
      
      /* lowercase the path */
      for (i=0;i<len;i++)
		si->upath[i]=tolower(si->upath[i]);
      
      pthread_mutex_lock(&share_mutex);
      share_root=gnut_list_prepend(share_root,si);
      pthread_mutex_unlock(&share_mutex);
    }
  } while (FindNextFile(hSearch, &Win32FindData));

  FindClose(hSearch);
  return 0;
}
#endif

/* int scan.dir(char *path)
 *
 * adds the contents of path to the share_root list */
int share_scan_dir(char *path, int verbose)
{
#ifdef WIN32
  share_path_len = strlen(path);
  return share_scan_dir_win32(path);
#else
  FILE *findfp;
  char *leafname;
  char *lnend;
  char *pathname;
  struct stat st;
  int ret;
  share_item *si;
  int i,len;
  
  gd_s(3, "share_scan_dir entering\n");
  
  pthread_mutex_lock(&share_mutex);

  leafname = (char *) ymaloc(OUR_MAX_PATH, 293);
  pathname = (char *) ymaloc(OUR_MAX_PATH, 294);
  if (have_find) {
    if (verbose) {
      printf("Using 'find' to index shared files...\n");
    }
    sprintf(leafname, "find %s -follow \\( -type f "
	    "! -name \"*.gnut\" -o -type l \\) "
	    "-printf %s",    path,    "\\%P\\\\n\\%p\\\\n");
  } else if (have_perl) {
    if (verbose) {
      printf("Using 'perl' to index shared files...\n");
    }
    sprintf(leafname,
	    "perl -we '"

	    "$r = \"'%s'\"; "  /* $r is root */
	    "%%visited = (); "

	    "sub visit ($) "
	    "{ "
	    "  local ($p, $f, $d, $i); " /* path file device inode */
	    "  local *DIR; "

	    "  $p = shift; "
	    "  opendir DIR, \"$r/$p\"; "
	    "  while ($f = readdir DIR) "
	    "  { "
	    "    next if $f eq \"..\"; "
	    "    ($d, $i) = stat \"$r/$p$f\"; "
	    "    next if $visited{\"$d, $i\"}; "
	    "    $visited{\"$d, $i\"} = 1; "
	    "    next if $f eq \".\"; "
 
	    "    if (-f _) "
	    "    { "
            "      print \"$p$f\\n$r/$p$f\\n\" "
	    "      unless $f =~ /\\.gnut$/; "
	    "    } "
	    "    elsif (-d _) "
	    "    { "
            "      visit(\"$p$f/\"); "
	    "    } "
	    "  } "
	    "} "
	    
	    "visit(\"\");'",
	    path);
  } else {
    printf("NOTE: You cannot 'scan' shared files because you do not have\neither GNU 'find' or 'perl' on your system.\n");
    sprintf(leafname, "echo -n ''");
  }

  gd_s(1, "share_scan_dir about to run: \"");
  gd_s(1, leafname);
  gd_s(1, "\" \n");

  /* Open pipe from command */
  findfp = popen(leafname, "r");
  if (findfp == 0) {
    gd_s(1, "share_scan_dir pipe to find returned error: ");
    gd_i(1, errno);
    gd_s(1, "\n");
    pthread_mutex_unlock(&share_mutex);
    return -1;
  }
  gd_s(1, "share_scan_dir retrieving results from find...\n");
  while (fgets(leafname, OUR_MAX_PATH, findfp)) {
    if (fgets(pathname, OUR_MAX_PATH, findfp)) {
      /* remove trailing eol */
      lnend = leafname + strlen(leafname); lnend--; *lnend = 0;
      pathname[strlen(pathname)-1] = 0;

      lnend -= 5; /* for checking for '.gnut' */

      if (leafname[0] == '.') {
	/* ignore dot-files */
      } else if ((strlen(leafname) > 5)
		 && (lnend[0] == '.')
		 && (lnend[1] == 'g')
		 && (lnend[2] == 'n')
		 && (lnend[3] == 'u')
		 && (lnend[4] == 't')) {
	/* .gnut-files (actually already filtered out by the find command) */
      } else {
	ret = stat(pathname, &st);

	si = (share_item *) ymaloc(sizeof(share_item), 295);

	si->size = st.st_size;
	si->mtime = st.st_mtime;
	share_size += (double) st.st_size;
	share_num++;
	si->ref = share_num;
      
	si->path = ystdup(leafname, 433); /* returned in query reply */
	si->upath = ystdup(leafname, 434); /* used in query match */
	si->fpath = ystdup(pathname, 435); /* real fullpath */

	len = strlen(si->upath);
	for (i=0; i<len; i++) {
	  si->upath[i] = tolower(si->upath[i]);
	}

	share_root = gnut_list_prepend(share_root, si);
      }
    }
  }
  
  pclose(findfp);
  fre_str(&leafname, 166);
  fre_str(&pathname, 167);
  
  pthread_mutex_unlock(&share_mutex);
  
  return 0;
#endif
}

/* my test implementation of search result caching, what I've got is a
 * hash based on the query token, and a queue based on the time of
 * search when a new token is searched for, it is first checked for in
 * the hash, if it exists, then the Gnut_List of shares is used
 * otherwise, a new search is performed, this search result is then
 * appended to the hash and queue, with the oldest query being removed
 * from the queue and the hash, much like in the routing section. It
 * would be a good idea upon a match to move the item back to the
 * front of the list, so that it wouldn't expire as quickly if it was
 * accessed frequently */
uchar share_cache_hash_func(void *d)
{
  uchar b=0;
  int i;
  share_query_results *ss;
  
  ss=d;
  for (i=0; i < strlen(ss->query); i++) {
    b += ss->query[i];
  }
  return b;
}

int share_cache_compare_func(void *a, void *b)
{
  share_query_results *ss,*ss2;

  ss = a;
  ss2 = b;

  return strcmp(ss->query, ss2->query);
}

int share_init()
{
  have_find = trycmd("find --version", "GNU fi");
  have_perl = trycmd("perl -we 'print \"Perl rules!\\n\";'", "rules");

  share_cache_hash = gnut_hash_new(share_cache_hash_func,
				   share_cache_compare_func);
  share_cache_queue = gnut_queue_new(511);
  return 0;
}

Gnut_List *share_cache_run_search(char *token, int max)
{
  share_query_results ss;
  share_query_results *ss2;
  Gnut_List *gl;
  
  ss.query=token;
  
  ss2 = gnut_hash_find(share_cache_hash, &ss);
  if (ss2) {
    /* we hit a query in our cache, so we should return a copy of this
     * list, and possibly move this query up in the queue */
    return share_copy_list(ss2->results);
  }
  /* OK, we didn't have it in the hash, which means we need to
   * run the query, and then insert it into the cache, then return a copy */
  
  gl = share_search(token,max);
  ss2 = (share_query_results *) ymaloc(sizeof(share_query_results), 287);
  ss2->query = ystdup(token, 423);
  ss2->results = share_copy_list(gl);

  gnut_hash_insert(share_cache_hash, ss2, 494);
  gnut_queue_insert(share_cache_queue, ss2, 365);

  return gl;
}

/* take two share lists, and make a new list, not to exceed max num
 * which is the logical AND of the two inputs. */
Gnut_List *share_results_combine(Gnut_List *l1, Gnut_List *l2,int max)
{  
  int i=0;
  Gnut_List *tmp,*tmp2,*prev=NULL;
  Gnut_List *nl=NULL; 
  share_item *si1,*si2;
  share_item *nsi;
  
  tmp2=l2;
  
  for (tmp=l1;tmp;tmp=tmp->next) {
    si1=tmp->data;
    for (;tmp2;tmp2=tmp2->next) {
      si2=tmp2->data;
      if (si2->ref==si1->ref) {
        /* they match, so we'll make a copy, and insert it into
         * the new list */
        nsi=(share_item *) ymaloc(sizeof(share_item), 288);
        memcpy(nsi,si2,sizeof(share_item));
        nsi->path= ystdup(si2->path, 424);
        nsi->upath=ystdup(si2->upath, 425);
        nsi->fpath=ystdup(si2->fpath, 426);
        nl=gnut_list_prepend(nl,nsi);
        
        if ((++i) > max) return nl;
        break;
      }
      if (si2->ref < si1->ref) {
        tmp2=prev;
        break;
      }
      prev=tmp2;
    }
  }
  return nl;
}
