/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Shell.java,v 1.4 2001/05/07 20:22:21 hamada Exp $
 */

package net.jxta.jfc;

import java.awt.*;
import javax.swing.*;

import net.jxta.peergroup.*;
import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.impl.shell.*;
import net.jxta.pipe.*;
import net.jxta.impl.peergroup.*;
import java.util.*;
import java.io.*;
import net.jxta.impl.pipe.*;
import net.jxta.protocol.*;
import net.jxta.id.*;

/**
 * This class implements a JXTA Shell
 */

public class Shell extends ShellApp implements  Runnable {

    static public final String ThisShell = "Shell";
    static public final String ParentShell = "parentShell";
    private ShellEnv env;
    private boolean stopped = false;
    private JTextArea text = null;
    private ShellConsole cons = null;
    private Thread thread = null;
    private ShellEnv parentEnv = null;
    private String shellRootEnvName = null;
    private BufferedReader scriptReader = null;
    private boolean gotParent = false;
    private boolean gotMyOwnFrame = false;
    private String  pipecmd = null;

    public boolean isRootShell() {
	if (gotParent) {
	    return false;
	} else {
	    return true;
	}
    }

    public Shell(JTextArea t){
        text=t;
    }
    public Shell(){
    }

    // Initialization defered until setPeerGroup or init is called.

    public PeerGroup setGroup (PeerGroup thisGroup) {
	PeerGroup oldg = super.setGroup(thisGroup);
	if (oldg != null) return oldg;
	return null;
    }

    public void init(PeerGroup thisGroup) {
    setGroup(thisGroup);
    startApp(null);
    }


    public void initPipe(PeerGroup thisGroup, ShellEnv penv, String cmd) {
        println("Init Shell" + penv);
	setGroup(thisGroup);
	setEnv(penv);
        println("Init Shell" + getEnv());
	pipecmd = cmd;
	startPipeApp( cmd );
    }


    public void run () {
	runShell();
    }

    public void exit () {
	// Remove itself from the parent ShellEnv (GC)
	if (parentEnv != null) {
	    ShellObject old = null;
	    old = parentEnv.get (shellRootEnvName);
	    if (old != null) {
		parentEnv.remove (shellRootEnvName);
	    }
	}
	// Destroy ourself
	if (gotMyOwnFrame) {
	    cons.destroy();
	}
	// Interrupt our parsing thread.
	if (thread != null) {
	    thread.interrupt();
	} else {
	    stopped = true;
	}
    }

    /**
     * This method implements the default input stream (keyboard).
     */
    private void runShell () {

        if (pipecmd != null) {
	  processCmd(pipecmd);
	  return;
	}

	if (scriptReader == null) {
	    println ("=====================================================");
	    println ("========= Welcome to the JXTAShell  Version 1.0 ============");
	    println ("=====================================================");
	    info();
	}

	// check if there is a .jshrc file
        startupFile();

        String cmd = null;
	String name = null;

	while (true) {

	    if (stopped) break;

            if (scriptReader == null) print ("JXTA> ");
            try {
		if (scriptReader == null) {
		    cmd = waitForInput();
		} else {
		    cmd = scriptReader.readLine();
		}
		if (cmd == null) {
		    return;
		}

	    } catch (IOException e) {
		System.err.println("Shell is reconnecting to DefaultInputPipe");
		// This shell has lost its standard InputPipe. Try
		// to reconnect to
		// the special keyboard InputPipe.
		ShellInputPipe ip = ShellInputPipe.getInputPipe();
                while (true) {
		    if (ip.getOwnership()) {
			inputPipe = ip;
		    }
		    try {
			cmd = waitForInput();
		    } catch (IOException e2) {
			//Wait a bit a try again
			try {
			    Thread.sleep(1000);
			} catch (Exception e3) {
			}

			continue;
		    }
		    break;
		}

	    }

            if (cmd.equals(";")) {
		continue;
	    }
            processMultipleCmd (cmd);
	}
    }

    private boolean initScriptFile (String fn) {
	try {
	    scriptReader = new BufferedReader (new FileReader (fn));
	    return true;
	} catch (Exception e) {
	    return false;
	}
    }

    private void startupFile() {
      try {
        BufferedReader scriptReader = new BufferedReader (new FileReader
	   (".jshrc"));
	if (scriptReader == null) return; //nothing to do

	String cmd = scriptReader.readLine();

	while(cmd != null) {
	  processMultipleCmd (cmd);
	  cmd = scriptReader.readLine();
	}
	scriptReader.close();
      } catch (Exception e) {
      }
    }

    public int startApp (String[] args) {

    if (args != null) {
        // Process args
        if (args[0].equals("-s")) {
        cons = new ShellConsole(text, 30, 80);
        gotMyOwnFrame = true;
        }

        if (args[0].equals("-f")) {
        if (args.length != 2) {
            println("usage: Shell -f <filename>");
            return ShellApp.appMiscError;
        }
        if (!initScriptFile (args[1])) {
            println("Shell: cannot access " + args[1]);
            return ShellApp.appMiscError;
        }
        }
    }
	env = getEnv();

	if (env != null) {
	    // This is a child Shell.

	    ShellObject old = null;

	    // Recover the parent env, and duplicate it
	    ShellEnv envRoot = getEnv();
	    if (envRoot != null) {
		env = new ShellEnv(envRoot);

		// Store this Shell into the root environment
		shellRootEnvName = envRoot.createName();
		old = envRoot.add (shellRootEnvName,
                       new ShellObject ("Child Shell", this));

		old = envRoot.get (ThisShell);
		if (old != null) {
		    old = env.add (ParentShell, old);
		}
		parentEnv = envRoot;
	    } else {
		env = new ShellEnv();
	    }
	    setEnv (env);

	    // Store this Shell into its own environment
	    old = env.add (ThisShell, new ShellObject ("Root Shell", this));

	    gotParent = true;

	    if (gotMyOwnFrame) {
		// Create the default InputPipe
		InputPipe ip = null;
		ShellInputPipe defaultInputPipe =
		        new ShellInputPipe (group, cons);
		if ( !defaultInputPipe.getOwnership()) {
		    System.err.println("Shell cannot access ShellInputPipe");
		    return ShellApp.appMiscError;
		}
		old = env.add ("stdin",
			       new ShellObject ("Default InputPipe",
						defaultInputPipe));

		old = env.add ("consin",
			       new ShellObject ("Default Console InputPipe",
						defaultInputPipe));

		ip = setInputPipe (defaultInputPipe);

		// Create the default OutputPipe
		OutputPipe op = null;
		ShellOutputPipe defaultOutputPipe =
                               new ShellOutputPipe (group, cons);
		old = env.add ("stdout",
			       new ShellObject ("Default OutputPipe",
						defaultOutputPipe));

		old = env.add ("consout",
			       new ShellObject ("Default Console OutputPipe",
						defaultOutputPipe));

		op = setOutputPipe (defaultOutputPipe);
	    }

	} else {

	    // That is the case only if there is no parent Shell

	    cons = new ShellConsole(text, 30, 80);
	    gotMyOwnFrame = true;
	    env = new ShellEnv();

	    InputPipe ip = null;

	    ShellObject obj = env.get ("stdin");
	    if (obj == null) {
		// Create the default InputPipe
		ShellInputPipe defaultInputPipe =
                              new ShellInputPipe (group, cons);
		if ( !defaultInputPipe.getOwnership()) {
		    System.err.println("Shell cannot access ShellInputPipe");
		    return ShellApp.appMiscError;
		}

		ShellObject old = env.add ("stdin",
                   new ShellObject ("Default InputPipe", defaultInputPipe));

		old = env.add ("consin",
                   new ShellObject ("Default Console InputPipe",
				    defaultInputPipe));

		ip = setInputPipe (defaultInputPipe);
	    } else {
		try {
		    ip = (InputPipe) obj.getObject();
		} catch (Exception e) {
		    System.err.println("stdin is not an InputPipe");
		    return ShellApp.appMiscError;
		}
	    }

	    OutputPipe op = null;
	    obj = env.get ("stdout");
	    if (obj == null) {
		// Create the default OutputPipe
		ShellOutputPipe defaultOutputPipe =
                                   new ShellOutputPipe (group, cons);
		ShellObject old = env.add ("stdout",
		    new ShellObject ("Default OutputPipe", defaultOutputPipe));
		old = env.add ("consout",
		    new ShellObject ("Default Console OutputPipe",
				     defaultOutputPipe));
		op = setOutputPipe (defaultOutputPipe);
	    } else {
		try {
		    op = (OutputPipe) obj.getObject();
		} catch (Exception e) {
		    System.err.println("stdout is not an OutputPipe");
		    return ShellApp.appMiscError;
		}
	    }

	    obj = env.get ("stdgroup");
	    if (obj == null) {
		ShellObject old = env.add ("stdgroup",
                                  new ShellObject ("Default Group" , group));
	    }

	    // Store this Shell into the environment
	    ShellObject old = env.add (ThisShell,
				       new ShellObject ("Root Shell", this));
	}
	// Start this Shell
	if (gotMyOwnFrame) {

	    thread = new Thread (this, "Root Shell" );
	    // Reset its pipes - this is needed..
	    // the previous line has called the empty constructor
	    thread.start();
	} else {
	    runShell();
	}
	return ShellApp.appSpawned;
    }

   private int startPipeApp( String cmd ) {
     env = getEnv();
     thread = new Thread (this, cmd );
     // Reset its pipes - this is needed..
     // the previous line has called the empty constructor
     thread.start();
     return ShellApp.appSpawned;
   }

    public void stopApp () {
	stopped = true;
    }

    private void loadApp (String appName, String[] args, String var) {
      try {
	Class appClass = Class.forName ("net.jxta.impl.shell.bin." +
                       appName + "." + appName);
        ShellApp app = (ShellApp) appClass.newInstance ();


        // Set up the default environment

        ShellEnv oe = app.setEnv (env);

        PeerGroup og = app.setGroup ((PeerGroup)
		       env.get("stdgroup").getObject());

        InputPipe ip = app.setInputConsPipe ((InputPipe)
                  env.get("consin").getObject());

        OutputPipe op = app.setOutputConsPipe  ((OutputPipe)
                  env.get("consout").getObject());

        ip = app.setInputPipe ((InputPipe)
                  env.get("stdin").getObject());

        op = app.setOutputPipe  ((OutputPipe)
                  env.get("stdout").getObject());

        // start the app

	// Set the variable name for the return value

	app.setReturnVariable (var);

    int result = app.startApp (args);
        if( ( ShellApp.appNoError != result )
	    && (ShellApp.appSpawned != result) ) {
	  //println( "# '" + appName + "' returned error code : " + result );
	  }
       } catch (Exception e) {
	  println ("command not found '" + appName + "'");
	  //println (e.toString());
	  //e.printStackTrace();
	  //println(" ");
      }
    }


  /*
   * Process the <cmd;cmd>* commands
   */

    private void processMultipleCmd (String cmd) {

      StringTokenizer tokens = new StringTokenizer (cmd, ";");

      while (tokens.hasMoreElements())
	processPipeCmd(tokens.nextToken());
    }


  /*
   * Process the <cmd | cmd>* commands
   */

    private void processPipeCmd (String cmd) {

      Vector cmds = new Vector();

      StringTokenizer tokens = new StringTokenizer (cmd, "|");

      while (tokens.hasMoreElements())
	  cmds.addElement(tokens.nextToken());

      if (cmds.size() != 1) { // we have a pipeline here!

	// at the beginning start with stdin and stdout

	PeerGroup pg =(PeerGroup) env.get("stdgroup").getObject();
	InputPipe stdin =(InputPipe) env.get("stdin").getObject();
	OutputPipe stdout = (OutputPipe) env.get("stdout").getObject();

	// these are for building the pipeline

	InputPipe pipein = null;
	OutputPipe pipeout = null;
        InputPipe lastin = null;


	/*
	 * create Shell cmd pipe to link the two
	 */

    Pipe pipes = null;
    PipeAdvertisement padv = null;
    try {

        pipes = pg.getPipe();
        padv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
        PipeAdvertisement.getAdvertisementType() );
        padv.setPipeID( new PipeID( pg.getID() ) );

        pipein = pipes.createInputPipe(padv);
        pipeout = pipes.createOutputPipe(padv,Pipe.NonBlocking, -1);

    } catch (Exception ex) {
    }
	/*
	 * create the environment by cloning the parent. I think
	 * this is right (will see)
	 */

	ShellEnv pipeenv = new ShellEnv(env);

	/*
	 * create the new Shell ENV to run this pipe command
	 */

	Shell pipeShell = new Shell();

	/*
	 * Set the pipeline for the first command and let it go
	 * only stdout since stdin is the right one
	 */

	pipeenv.add ("stdout", new ShellObject ("Default OutputPipe",
						pipeout));

	pipeenv.add ("stdin", new ShellObject ("Default InputPipe",
						stdin));

	pipeenv.add ("stdgroup", new ShellObject ("Default Peer Group",
						  pg));

	pipeShell.initPipe(pg, pipeenv, (String) cmds.elementAt(0));

	/*
	 * update last pipe in pipeline
	 */

	lastin = pipein;

	// The first and last command in the pipe needs to be treated separatly

	for (int i=1; i < cmds.size()-2; i++) {

	  /*
	   * create Shell cmd pipe to link the two
	   */

                try {
                    padv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
                    PipeAdvertisement.getAdvertisementType() );
                    padv.setPipeID( new PipeID( pg.getID() ) );

                    pipein = pipes.createInputPipe(padv);
                    pipeout = pipes.createOutputPipe(padv,Pipe.NonBlocking, -1);
                } catch (Exception ex) {
                }

	  /*
	   * create the environment by cloning the parent. I think
	   * this is right (will see)
	   */

	  pipeenv = new ShellEnv(env);

	  /*
	   * create a new Shell process to run this pipe command
	   */

	  pipeShell = new Shell();

	  /*
	   * Set the pipeline for the first command and let it go
	   * only stdout since stdin is the right one
	   */

	  pipeenv.add ("stdout", new ShellObject ("Default OutputPipe",
						  pipeout));

	  pipeenv.add ("stdin", new ShellObject ("Default InputPipe",
						lastin));

	  pipeenv.add ("stdgroup", new ShellObject ("Default Peer Group",
						    pg));

	  pipeShell.initPipe(pg, pipeenv, (String) cmds.elementAt(i));

	  /*
	   * update last in pipe for the next command
	   */

	  lastin = pipein;

	}

	/*
	 * Set the pipeline for the first command and let it go
	 * only stdout since stdin is the right one
	 */

	env.add ("stdout", new ShellObject ("Default OutputPipe",
						stdout));

	env.add ("stdin", new ShellObject ("Default InputPipe",
					       lastin));

	processCmd((String) cmds.lastElement());

        // need to restore stdin

      } else { // no pipe let's go

	processCmd((String) cmds.firstElement());

      }
    }

    private void processCmd (String cmd) {

	// Is there an "=" in the string ?
	int index = cmd.indexOf ("=");
	String var = null;

	if (index > 0) {
	    // There is a variable assignement. Get the name
	    // of the variable to use.
	    var = cmd.substring (0, index);
	    // Remove any space.
	    int index2 = var.indexOf (" ");
	    if (index2 > 0) {
		var = var.substring (0, index2);
	    }
	    // Remove the left operand and operator from the string
	    cmd = cmd.substring (index + 1);
	}

        StringTokenizer tokens = new StringTokenizer (cmd);
        int size = tokens.countTokens();
        if (size < 1) return;

        String[] args = null;
        if (size > 1) {
	    args = new String [size - 1];
	}

        String app = null;
        try {
          app = tokens.nextToken();
	} catch (Exception e) {
	    // Should not happen
	    return;
	}

        index = 0;
        try {
	    while (tokens.hasMoreElements()) {
		args[index++] = tokens.nextToken();
	    }
	} catch (Exception e) {
	    return;
	}

	loadApp (app, args, var);
    }


public void help() {

    println("NAME");
    println("     Shell  - JXTA Shell command interpreter");
    println(" ");
    println("SYNOPSIS");
    println(" ");
    println("     Shell [-f filename] [-s] ");
    println(" ");
    println("DESCRIPTION");
    println(" ");
	info();
    println(" ");
	println(" ");
	println("Every Shells commands is of the form <cmd>;..;<cmd>. The ';'");
	println("is used as command separator. Multiple commands can be entered");
	println("in one line. Pipelines can be created by combining the pipe stdout");
	println("of a command into the pipe stdin of another command using the");
	println("pipe '|' operator. For example the following command:");
	println(" ");
	println("      JXTA> cat env1 | more");
	println(" ");
	println("Pipe the output of the command 'cat' into the stdin of the command");
	println("'more'. Arbitrary number of commands can be pipelined together within");
	println("a single Shell command. Currently '|' is the only pipe operator");
	println("supported.");
	println(" ");
	println("The '=' operator can be used to assign the value of a command ");
	println("output to an environment variable. For example :");
	println(" ");
	println("      JXTA> myadv = mkadv -p");
	println(" ");
	println("This command creates a new pipe advertisement and stores it in the");
	println("'myadv' environment variable.");
	println(" ");
	println("COMMANDS");
    println(" ");
	println("The Shell provides the following commands:");
	println(" ");
	println(" cat        Display the content of an environment variable");
	println(" ");
	println(" chpgrp     Change the default peer group");
	println(" ");
	println(" env        Display all environment variables ");
	println(" ");
	println(" exit       Exit the Shell ");
	println(" ");
	println(" exportfile Export a Shell variable into an external file");
	println(" ");
	println(" get        Extract a message tag body from a pipe messsage ");
	println(" ");
	println(" groups     Discover and list peer groups ");
	println(" ");
	println(" help       On-line Help about commands");
	println(" ");
	println(" importfile Import an external file into a Shell variable");
	println(" ");
	println(" join       Join a new peer group");
	println(" ");
	println(" leave      Leave a peer group");
	println(" ");
    println(" mkadv      Make a new pipe or peer group advertisement  ");
	println(" ");
	println(" mkmsg      Make a new pipe message");
	println(" ");
	println(" mkpgrp     make a new peer group ");
	println(" ");
	println(" mkpipe     Make a new pipe ");
	println(" ");
	println(" more       browse or page through a Shell object ");
	println(" ");
	println(" peers      Discover and list peers ");
	println(" ");
	println(" put        Put a new tag body into a pipe message");
	println(" ");
	println(" recv       Receive a pipe message");
	println(" ");
	println(" search     Search a shared advertisement in a peer group");
	println(" ");
	println(" send       Send a pipe message");
	println(" ");
	println(" setenv     Set the value of an environment variable");
	println(" ");
	println(" setinfo    Give a symbolic name to a peer");
	println(" ");
	println(" share      Share an advertisement in a peer group");
	println(" ");
	println(" Shell      Fork a new Shell");
	println(" ");
	println(" talk       Instant messaging talk command");
	println(" ");
	println(" whoami     Display peer and peer group information");
	println(" ");
    println("OPTIONS");
    println(" ");
	println("    [-f filename] execute the script file");
	println("    [-s] fork a new Shell console in a new window");
    println(" ");
    println("EXAMPLE");
    println(" ");
    println("    JXTA> Shell -f /home/tra/batch");
    println(" ");
    println("This command executes the commands stored in the Shell script");
	println("file '/home/tra/myfile' in the current Shell environment.");
	println(" ");
    println("ENVIRONMENT");
    println(" ");
	println("The following environment variables are defined by default:");
	println(" consin    = Default Console InputPipe");
	println(" consout   = Default Console OutputPipe");
	println(" stdout    = Default OutputPipe");
	println(" stdin     = Default InputPipe");
	println(" Shell     = Root Shell");
	println(" stdgroup  = Default peer group");
	println(" rootgroup = Default NetPeerGroup");
	println(" ");
    println("FILES");
	println(" ");
	println("  $HOME/.jshrc");
	println(" ");
	println("       a default startup script can be setup that is executed");
	println("       when the Shell is invoked");
    println(" ");
    println("SEE ALSO");
}


public void info() {
    println(" ");
    println("The JXTA Shell provides an interactive environment to the JXTA    ");
    println("platform. The Shell provides basic commands to discover peers and  ");
    println("peergroups, to join and resign from peergroups, to create pipes");
    println("between peers, and to send pipe messages. The Shell provides environment ");
    println("variables that permit binding symbolic names to Jxta platform objects.      ");
    println("Environment variables allow Shell commands to exchange data between ");
	println("themselves. The shell command 'env' displays all defined environment ");
	println("variables in the current Shell session.");
    println(" ");
    println("The Shell creates a Jxta InputPipe (stdin) for reading input from ");
    println("the keyboard, and a Jxta OutputPipe (stdout) to display information");
	println("on the Shell console. All commands executed by the Shell have their");
	println("initial 'stdin' and 'stdout' set up to the Shell's stdin and stdout pipes.");
    println("The Shell also creates the environment variable 'stdgroup' that   ");
    println("contains the current JXTA PeerGroup in which the Shell and commands ");
    println("are executed.");
    println(" ");
	println("A new Shell can be forked within a Shell. The 'Shell -s'");
	println("command starts a new Shell with a new Shell window. The Shell can");
	println("also read a command script file via the 'Shell -f myfile'.");
	println(" ");
    println("A 'help' command is available to list the commands available.");
	println("Type 'help <command>' to get help about a particular command.");
    println("To exit the Shell, use the 'exit' command.");
}

}

