/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ShellConsole.java,v 1.2 2001/04/26 21:50:55 hamada Exp $
 */

package net.jxta.impl.shell;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;

import net.jxta.impl.cm.*;

public class ShellConsole implements Runnable, KeyListener {

  private static final int MAX_LINE = 256;

  private OutputStream opipe;
  private InputStream ipipe;
  private InputStream in;
  private PrintStream out;
  private JTextArea text = null;

  private StringBuffer line = new StringBuffer();
  private int textLength = 0;
  private ShellConsole console = this;
  private Frame f = null;
  private Thread thread = null;

  public ShellConsole(JTextArea t , int rows, int cols) {


      this.text= t;
      Cm cm = new Cm("ShellConfig");
	  text.setFont(new Font("DialogInput", Font.PLAIN,12) );
	  text.setEditable(false);
	  text.addKeyListener (this);
	  opipe = new PipedOutputStream();
	  try {
	      in = new PipedInputStream((PipedOutputStream)opipe);
	  } catch ( IOException e ) {
	  }
	  // Start the thing

	  thread = new Thread(this, "JXTA Console" );
	  thread.start();

  }

    public void destroy() {
	thread.interrupt();

    }

  public void keyPressed(KeyEvent e ) {
    handling(e.getKeyCode(), e.getKeyChar(), e.getModifiers() );
    e.consume();
  }

  public void handling(int val, char ch, int modifiers ) {
    switch (val) {
    case (KeyEvent.VK_BACK_SPACE ):
      if (line.length() > 0) {
        line.setLength(line.length() - 1 );
        text.replaceRange( "", textLength-1, textLength );
        textLength--;
      }
      break;
    case ( KeyEvent.VK_ENTER ):
      readme();
      break;
    case ( KeyEvent.VK_U ):
      if ((modifiers & InputEvent.CTRL_MASK) > 0 ) {
        int len = line.length();
        text.replaceRange( "", textLength-len, textLength );
        line.setLength( 0 );
        textLength = text.getText().length();
      } else
        process(ch);
      break;
    default:
      process(ch);
    }
  }

  private void process(char ch) {
    if ((ch >= ' ') && (ch <= '~')) {
      line.append( ch );
      text.append(String.valueOf(ch));
      text.setCaretPosition(text.getCaretPosition());
      textLength++;
    }
  }

  private void readme() {
    String s;
    if ( line.length() == 0 )
      s = ";\n";
    else {
      s = line +"\n";
    }
    line.setLength( 0 );
    text.append("\n");
    textLength = text.getText().length();
    readLine( s );
    setBlinker( textLength );
  }

  /*
   * set the cursor write
   */

  public void setBlinker( int pos ) {
    text.setCaretPosition( pos );
  }

  private void readLine( String line ) {
    try {
        opipe.write( line.getBytes() );
        opipe.flush();
      } catch ( IOException e ) {
        opipe = null;
        throw new RuntimeException("ShellConsole Error");
      }
  }

  synchronized public void print( String s ) {
    text.append(s);
    textLength = text.getText().length(); // sync for safety
    setBlinker(text.getCaretPosition() + s.length());
  }

  synchronized public void println( String s ) {
    print(s+"\n");
  }

  private void readPipe() throws IOException {
    if (ipipe == null ) { // initial setup
      PipedOutputStream pout = new PipedOutputStream();
      out = new PrintStream( pout );
      ipipe = new PipedInputStream(pout);
    }
    byte [] buff = new byte [MAX_LINE];
    int read;
    while ((read = ipipe.read(buff)) != -1 ) {
      print(new String(buff, 0, read) );
    }
  }

  /*
   * count the end of line
   */

  private int newlines() {
    String s = text.getText();
    int c = 0;
    for(int i=0; i<s.length(); i++)
      if (s.charAt(i) == '\n')
        c++;
    return c;
  }

  public void run() {
    try {
	readPipe();
    } catch ( IOException e ) {
    } catch ( Exception e) {
	return;
    }
  }

  public Reader getIn() {
    return new InputStreamReader(in);
  }

  public PrintStream getOut() { return out; }
  public PrintStream getErr() { return out; }

  public void main() {
  }

  public String read() {
    try {
      char buf[] = new char[MAX_LINE];
      console.getIn().read(buf);
      String cmd = new String(buf);
      int pos = cmd.indexOf('\n'); // end of command
      return cmd.substring(0, pos);
    } catch (Exception ex) {
      return null;
    }
  }

  public void write(String msg) {
    try {
      console.getOut().print(msg);
    } catch (Exception ex) {
      System.err.println ("Error=" + textLength + msg.length());
    }
  }

  public void keyTyped(KeyEvent e) {
  }

  public void keyReleased(KeyEvent e) {
  }

}
