/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: jxtaDemo.java,v 1.4 2001/05/07 20:22:21 hamada Exp $
 */


package net.jxta.jfc;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;


import net.jxta.exception.*;
import net.jxta.peergroup.*;
import net.jxta.document.*;
import net.jxta.exception.*;
import net.jxta.platform.*;
import net.jxta.protocol.*;
import net.jxta.discovery.*;


public class jxtaDemo extends JFrame implements Application {


public jxtaDemo(){

}


   public void init(PeerGroup group, Advertisement adv )
       throws PeerGroupException {


      this.group = group;
      debug= group.getDebug();
      discovery = group.getDiscovery();
      try {

       jxtaMainFrame();
      } catch (Throwable t) {
        if (debug)  t.printStackTrace();
        System.exit(1);
      }

      try {
       peers = discovery.getLocalAdvertisements(discovery.PEER, null, null);
       parsePeers(peers);
       groups = discovery.getLocalAdvertisements(discovery.GROUP, null, null);
       parseGroups(groups);
      } catch (Exception e) {
        // do nothing right now
        e.printStackTrace();

      }

   }

    public int startApp ( String[] str ) {
        return 0;
    }

    public void stopApp () {
    }


private void jxtaMainFrame()
{
        int ypos =10;
        int hgt  =35;
        int wd   =80;

        setTitle(title);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        getContentPane().setLayout(new BorderLayout(0,0));
        setSize(640,480);
        setVisible(false);
        jxtaPanel.setLayout(null);
        getContentPane().add("Center", jxtaPanel);
        jxtaPanel.setBackground(color);
        jxtaPanel.setBounds(0,0,630,470);

        jxtaButton(peerButton,jxtaPanel,"Peers","Discover peers within the current group",
                     15,ypos,wd,hgt);
        peerButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
                 discover();
         }
         });

        jxtaButton(groupButton,jxtaPanel,"Groups","Discover groups within the current group",
                     95,ypos,wd,hgt);
        groupButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
                 groups();
         }
        });

        jxtaButton(importButton,jxtaPanel,"Import","Import a document",
                     175,ypos,wd,hgt);
        importButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
                 importFile();
         }
         });
        jxtaButton(exportButton,jxtaPanel,"Export","Export a document",
                     255,ypos,wd,hgt);
        exportButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
                 importFile();
         }
         });

        jxtaButton(helpButton,jxtaPanel,"Help","Help",
                    335,ypos,wd,hgt);

        jxtaButton(exitButton,jxtaPanel,"Exit","Exit jxta shell",
                     415,ypos,wd,hgt);
        exitButton.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
                 exitJxta();
         }
        });


        // Tabbed Panels
        jxtaPanel.add(jxtaTabbedPane);
        jxtaTabbedPane.setBackground(color);
        jxtaTabbedPane.setBounds(12,40,600,410);

        jxtaTabbedPane.add(ptsp);
        ptsp.setBounds(2,27,439,310);

        peerTable= new JTable(peerData,columnNames);
        peerTable.setEnabled(false);

        ptsp.getViewport().add(peerTable);

        groupTable= new JTable(groupData,columnNames);
        groupTable.setEnabled(false);
        jxtaTabbedPane.add(gtsp);
        gtsp.setBounds(2,27,439,310);
        gtsp.getViewport().add(groupTable);


        // talk panel
        jxtaTabbedPane.add(talkPanel);
        talkPanel.setLayout(null);
        talkPanel.setBackground(color);
        talkPanel.setBounds(2,27,439,310);
        talkPanel.add(myIdent);
        talkPanel.add(remoteIdent);
        remoteLabel.setText("remote identity");
        talkPanel.add(remoteLabel);
        myLabel.setText("my identity");
        talkPanel.add(myLabel);
        myLabel.setBounds(10,9,134,20);
        myIdent.setBounds(10,33,150,26);
        remoteIdent.setBounds(178,33,150,26);
        remoteLabel.setBounds(178,9,150,25);

        jxtaButton(registerButton,talkPanel,"Regis","Register my Identity",
                     10,70,wd,hgt);
        registerButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
                 register();
         }
        });

        jxtaButton(talkButton,talkPanel,"Talk","Talk to remote Identity",
                     90,70,wd,hgt);
        talkButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
                 talk();
         }
        });


        JScrollPane sp1 = new JScrollPane();
        JScrollPane sp2 = new JScrollPane();


        talkPanel.add(sp1);
        talkPanel.add(sp2);
        sp1.setBounds(10,110,420,84);
        sp2.setBounds(10,219,420,80);

        sp1.getViewport().add(remoteTalkTextArea);
        sp2.getViewport().add(localTalkTextArea);

        stsp.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        jxtaTabbedPane.add(stsp);

        Shell shell = new Shell(shellTextArea);
        shell.init(group);
        stsp.getViewport().add(shellTextArea);
        stsp.getViewport().setScrollMode(JViewport.BACKINGSTORE_SCROLL_MODE);
        stsp.setBounds(2,27,580,390);
        shellTextArea.setToolTipText("jxta shell");
        shellTextArea.setRows(20);

        jxtaTabbedPane.setFont(new Font("Default", Font.BOLD,12) );
        jxtaTabbedPane.setTitleAt(0,"Peers");
        jxtaTabbedPane.setTitleAt(1,"Groups");
        jxtaTabbedPane.setTitleAt(2,"Talk");
        jxtaTabbedPane.setTitleAt(3,"Shell");

        addWindowListener(new WindowAdapter() {
          public void windowClosing(WindowEvent e) {
              exitJxta();
          }
         });

         setVisible(true);

   }


    public void jxtaButton ( JButton button, JPanel panel, String label,
                        String tip, int x, int y, int w, int h){
        button.setBackground(color);
        button.setBounds(x,y,w,h);
        button.setHorizontalTextPosition(SwingConstants.CENTER);
        button.setText(label);
        button.setBorderPainted(false);
        button.setToolTipText(tip);
        button.setFocusPainted(false);
        button.setIcon(unpressed);
        button.setPressedIcon(pressed);
        button.setRolloverIcon(rollover);
        button.setFont(new Font("Default", Font.BOLD,11) );
        button.setContentAreaFilled(false);
        button.setMargin(new Insets(0,0,0,0));
        panel.add(button);

    }


     public jxtaDemo(String sTitle)
     {
       this();
       setTitle(sTitle);
       setFont(new Font("Default", Font.PLAIN,12) );

     }

    void exitJxta()
    {
     try {
       int reply = JOptionPane.showConfirmDialog(this,
                                 "Do you really want to exit?",
                                 "jxta Shell UI" ,
                                 JOptionPane.YES_NO_OPTION,
                                 JOptionPane.QUESTION_MESSAGE);
       if (reply == JOptionPane.YES_OPTION)
       {
         this.setVisible(false);
         this.dispose();
         System.exit(0);
       }
      } catch (Exception e) {
     }
    }

   private void parsePeers(Enumeration peers) {

        int i=0;

        while (peers.hasMoreElements() ) {
        PeerGroupAdvertisement adv = (PeerGroupAdvertisement) peers.nextElement();

        String name =  adv.getName();
        String pid  =  adv.getPid();
        String gid  =  adv.getGid();
        String address= ((EndpointAdvertisement)
             adv.getEndpointAdvertisements().get("TCP")).getEndpointAddress();

        peerTable.setValueAt(name,i,0);
        peerTable.setValueAt(pid,i,1);
        peerTable.setValueAt(gid,i,2);
        peerTable.setValueAt(address,i,3);
        i++;

     }

   }


   private void parseGroups(Enumeration groups) {

        int i=0;
        if (groups!=null)
        while (groups.hasMoreElements() ) {
        PeerGroupAdvertisement adv = (PeerGroupAdvertisement) groups.nextElement();

        String name =  adv.getName();
        String pid  =  adv.getPid();
        String gid  =  adv.getGid();
        String address= ((EndpointAdvertisement)
             adv.getEndpointAdvertisements().get("TCP")).getEndpointAddress();

        groupTable.setValueAt(name,i,0);
        groupTable.setValueAt(pid,i,1);
        groupTable.setValueAt(gid,i,2);
        groupTable.setValueAt(address,i,3);
        i++;

     }

   }

    public void discover (){

      try {
        discovery.getRemoteAdvertisements(null,discovery.PEER, null, null,10);
        try {
            wait(3000);
        } catch (Exception e){}
       peers = discovery.getLocalAdvertisements(discovery.PEER, null, null);
       parsePeers(peers);
      } catch (Exception e) {
        // do nothing right now
        e.printStackTrace();

      }

    }

    public void groups (){

      try {
        discovery.getRemoteAdvertisements(null,discovery.GROUP, null, null,10);
        try {
            wait(3000);
        } catch (Exception e){}
       groups = discovery.getLocalAdvertisements(discovery.GROUP, null, null);
       parseGroups(groups);
      } catch (Exception e) {
        // do nothing right now
        e.printStackTrace();

      }
    }

    private void importFile (){
        String filename = getFileName ("Import");
        if (filename !=null)
           System.out.println(" FileName is "+filename);
    }
    private void exportFile (){
        String filename = getFileName ("Export");
        if (filename !=null)
           System.out.println(" FileName is "+filename);
    }



    public String getFileName (String label) {
     JFileChooser chooser = new JFileChooser();
     int returnVal = chooser.showDialog(jxtaDemo.this, label);
     if(returnVal == JFileChooser.APPROVE_OPTION) {
       return (chooser.getSelectedFile().getName());
     } else return null;

    }

    public void talk (){

      String my=myIdent.getText();
      String remote=remoteIdent.getText();
    // System.out.println("I "+my+" Will attempt to connect to "+remote);
    }

    public void register (){

      String my=myIdent.getText();
     // System.out.println("I "+my+" Will attempt to connect to "+remote);
    }

    private void debug (String str) {
    if (debug) {
        System.out.println ("[jxtaDemo] " + str);
    }
    }



   // Color color= new Color(203,180,208);
    Color color= new Color(163,161,195);

    JPanel jxtaPanel    = new JPanel();
    JPanel talkPanel    = new JPanel();

    JTabbedPane jxtaTabbedPane = new JTabbedPane();
    JScrollPane ptsp = new JScrollPane();
    JScrollPane gtsp = new JScrollPane();
    JScrollPane stsp = new JScrollPane();


    JButton peerButton     = new JButton();
    JButton groupButton    = new JButton();
    JButton importButton   = new JButton();
    JButton exportButton   = new JButton();
    JButton exitButton     = new JButton();
    JButton helpButton     = new JButton();
    JButton talkButton     = new JButton();
    JButton registerButton = new JButton();

    JTextArea remoteTalkTextArea = new JTextArea();
    JTextArea localTalkTextArea  = new JTextArea();
    JTextArea peerTextArea       = new JTextArea();
    JTextArea groupTextArea      = new JTextArea();
    JTextArea  shellTextArea      = new JTextArea();

    JTable peerTable;
    JTable groupTable;

    JTextField myIdent     = new JTextField();
    JTextField remoteIdent = new JTextField();

    JLabel remoteLabel  = new JLabel();
    JLabel myLabel      = new JLabel();

    ImageIcon unpressed = new ImageIcon(ClassLoader.getSystemResource("net/jxta/jfc/images/button1.png"));
    ImageIcon   pressed = new ImageIcon(ClassLoader.getSystemResource("net/jxta/jfc/images/button1_pressed.png"));
    ImageIcon  rollover = new ImageIcon(ClassLoader.getSystemResource("net/jxta/jfc/images/button1_rollover.png"));
    String[] columnNames = {"Name",
                            "Peer ID",
                            "Group ID",
                            "Address"};

   Object [][] peerData = new Object [250] [4];
   Object [][] groupData= new Object [250] [4];

   private String Attribute;
   private String Value;
   private static String title= "jxta Shell UI";
   private PeerGroup      group;
   private Discovery discovery=null;
   private Enumeration peers=null;
   private Enumeration groups=null;
   private boolean debug = false;


}


