/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Discover.java,v 1.15 2001/05/25 17:52:05 kuldeep Exp $
 */

package net.jxta.instantp2p;

import java.io.IOException;

import net.jxta.protocol.*;
import net.jxta.endpoint.*;
import net.jxta.discovery.*;
import net.jxta.peergroup.*;
import net.jxta.exception.*;
import java.util.*;

/**
 * Discovers peers and peergroups
 *
 * @version
 * @since       JXTA 1.0
 */
public class Discover {

    private Discovery       disco = null;
    private PeerGroup       peerGroup;
    private final int       THRESHOLD = 2;
    private int             threshold;

    public Discover ( PeerGroup group ) {
	peerGroup = group;
	threshold = THRESHOLD;
	discoverService ();

	//getRemoteGroups ( threshold );
	//getRemoteAdvertisements ( null, null, null );

    }

    public Discovery getDiscovery () {
        return disco;
    }

    public void setPeerGroup ( PeerGroup group ) {
	if ( peerGroup.equals ( group))
	    return;
	peerGroup = group;
	discoverService ();
    }

    public PeerGroup getPeerGroup () {
	return peerGroup;
    }

    public int setThreashold ( int value ) {
	int oldValue = threshold;
	threshold = value;
	return oldValue;
    }

    private void discoverService () {

	if ( peerGroup == null )
	    return;

	disco = peerGroup.getDiscovery();
	if ( disco != null ) {
	    getRemotePeers ( threshold );
	}
	else {
	    System.out.println ( "Discover: disco is null ");
	}
    }

    public Enumeration getGroups () {
	return getGroups ( null, null, null );
    }

    public Enumeration getGroups ( String peerID ) {
	return getGroups ( peerID, null, null );
    }


    public Enumeration getGroups (String peerID,
				  String attribute, String value ) {
	getRemoteGroups ( peerID, attribute, value, threshold );
	return discover ( peerID, disco.GROUP, attribute, value, threshold );
    }


    public Enumeration getPeers () {
	return getPeers ( null, null, null );
    }

    public Enumeration getPeers ( String peerID ) {
	return getPeers ( peerID, null, null );
    }


    public Enumeration getPeers ( String peerID,
				  String attribute, String value ) {
	return discover ( peerID, disco.PEER, attribute, value, threshold );
    }

    public Enumeration getAdvertisements () {
	return getAdvertisements ( null, null, null );
    }

    public Enumeration getAdvertisements ( String peerID ) {
	return getAdvertisements ( peerID, null, null );
    }

    public Enumeration getAdvertisements ( String peerID,
				       String attribute, String value ) {
	getRemoteAdvertisements ( peerID, attribute, value );
	return discover ( peerID, disco.ADV, attribute, value, threshold );
    }


    public void getRemotePeers ( int threshold ) {
	discoverRemote ( null, disco.PEER, null, null, threshold );
    }

    public void getRemoteGroups ( int threshold ) {
	discoverRemote ( null, disco.GROUP, null, null, threshold );
    }

    public void getRemoteGroups ( String peerID, String attribute, 
				  String value,int threshold ) {
	discoverRemote ( peerID, disco.GROUP, attribute, value, threshold );
    }

    public void getRemoteAdvertisements ( String peerID, String attribute,
					String value ) {
	discoverRemote ( peerID, disco.ADV, attribute, value, threshold );
    }

    private void discoverRemote ( String peerID, int type, String attribute,
					String value, int threshold ) {

	disco.getRemoteAdvertisements( peerID, type,
				       attribute, value, threshold );
    }

    public void flush() {
      
        try {
            disco.flushAdvertisements( null, Discovery.PEER );
        } catch ( IOException e ) {
            e.printStackTrace();
        }

        try {
	    disco.flushAdvertisements( null, Discovery.GROUP );
        } catch ( IOException e ) {
            e.printStackTrace();
        }
    }

    private Enumeration discover ( String peerID, int type, String attribute,
				   String value, int threshold ) {
	
	Enumeration res;

	try {
	    res = disco.getLocalAdvertisements( type, attribute, value );
	} catch ( Exception e ) {
	    e.printStackTrace();
	    return null;
	}
	return res;
    }
}
