/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: GroupManager.java,v 1.2 2001/05/17 23:14:44 kuldeep Exp $
 *
 */

package net.jxta.instantp2p;

import java.io.File;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;

import net.jxta.exception.PeerGroupException;
import net.jxta.peergroup.PeerGroup;
import net.jxta.protocol.PeerGroupAdvertisement;

import net.jxta.share.*;

public class GroupManager {

    public static final String IP2P_DIRNAME = "IP2P";
    private static GroupManager instance = null;

    private Hashtable cmsTable = new Hashtable();
    private Hashtable searchTable = new Hashtable();

    public GroupManager() {
    }

    public CMS getCMS(PeerGroup group) {
        // check if the cms is already started
        CMS cms = (CMS)cmsTable.get(getShortName(group));

        if (null == cms) {
            // create a new cms for this peergroup
            cms = new CMS();

            try {
                cms.init(group, group.getAdvertisement());
            } catch (PeerGroupException e) {
	      e.printStackTrace ();
            }

            cms.startApp(new File(getGroupDir(group), CMS.DEFAULT_DIR));
        }

        if (null != cms) {
            cmsTable.put(getShortName(group), cms);
        }

        return cms;
    }

    public Search getSearch(PeerGroup group) {
        // check if the Search object is already started
        Search search = (Search)searchTable.get(getShortName(group));

        if (null == search) {
            // create a new cms for this peergroup
            search = new Search(group, new File(getGroupDir(group), Search.DEFAULT_DIR));
        }

        if (null != search) {
            searchTable.put(getShortName(group), search);
        }

        return search;
    }

    public static GroupManager getInstance() {
        if (null == instance) {
            instance = new GroupManager();
        }

        return instance;
    }

    private static File getGroupDir(PeerGroup pg) {
        File dir = null;

        // make sure the root directory exists
        File rootDir = new File(IP2P_DIRNAME);
        if (!rootDir.exists()) {
            rootDir.mkdir();
        }

        dir = new File(rootDir, getShortName(pg));
        if (!dir.exists()) {
            dir.mkdir();
        }

        return dir;
    }

    private static String getShortName(PeerGroup pg) {
        URL url = null;
        try {
            url = new URL(pg.getID().toString());
        } catch (MalformedURLException e) {
            e.printStackTrace();
            return null;
        }

        return Integer.toHexString((url.getHost()).hashCode());
    }
}
