/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Messaging.java,v 1.9 2001/06/03 05:08:02 kuldeep Exp $
 */

package net.jxta.instantp2p.desktop;

import java.awt.*;
import java.awt.event.*;
import net.jxta.widget.*;
import net.jxta.exception.*;
import net.jxta.peergroup.*;
import net.jxta.platform.*;
import net.jxta.instantp2p.*;

public class Messaging extends Panel implements MessageBoard, ActionListener,
    ItemListener {
    private PeerGroup group;         // default peergroup
    private TextArea  messageBoard;  // message board for all

    private TextField sendBox;       // field to type in message
    private Chat chat;

    private Button login, logout;
    private Checkbox securedBox = new Checkbox ( "Secured", false );
    private boolean  securedSession = false;
    ChatProposal  chatDialog = null;
    boolean acceptProp = false;

    Frame parentFrame;

    public Messaging ( PeerGroup group, String chatString, PeerGroupPanel pgPanel ) {
	this.group = group;
	parentFrame = InstantP2P.getFrame ();

        setLayout( new BorderLayout());

        Panel buttonPanel = new Panel();
        setBackground ( new Color ( 158, 158, 204 ));
        setForeground ( Color.black );
        buttonPanel.setLayout( new FlowLayout());

	if ( chatString.equals ( "Chat" )) {
            login = new Button ( "Start Chat" );
            logout = new Button ( "End Chat" );
	    buttonPanel.add( securedBox );
	    securedBox.addItemListener ( this );
	}
	else {
            login = new Button ( "Join Chat" );
            logout = new Button ( "Leave Chat" );
	}

	logout.setEnabled ( false );
        login.addActionListener ( this );
        logout.addActionListener ( this );

	buttonPanel.add( login );
        buttonPanel.add( logout );

	messageBoard = new TextArea  ( "", 10, 40, 
				       TextArea.SCROLLBARS_VERTICAL_ONLY  );
	messageBoard.setBackground ( Color.white );
	messageBoard.setForeground ( Color.black );
        messageBoard.setEditable( false );

        sendBox      = new TextField( 40 );
	sendBox.setBackground ( Color.white );
	sendBox.setForeground ( Color.black );
        sendBox.addActionListener ( this );

	add( buttonPanel, "North" );
	Border messageBoardBorder = new Border ( messageBoard, "Received Messages");
	add( messageBoardBorder, "Center" );
	Border sendBoxBorder = new Border ( sendBox, "Send Message");
	sendBoxBorder.setBackground ( new Color ( 102, 102, 153 ));
	sendBoxBorder.setForeground ( Color.white );
	add( sendBoxBorder, "South" );

	// This is quick and dirty. Need to implement in an OO manner down the road
	if ( chatString.equals ( "Chat" )) 
	     chat = new Chat ( group, this );
	else
	     chat = new GroupChat ( group, this );
	pgPanel.addPeerListener ( chat );
    }

    public void actionPerformed ( ActionEvent evt ) {

	if ( evt.getSource() == sendBox ) {
            String message = sendBox.getText();
            chat.sendMessageToPeers ( message );
	    sendBox.setText("");
        }        
	else if ( evt.getSource() == login ) {
	    login.setEnabled ( false);
	    securedBox.setEnabled ( false );
	    chat.login( InstantP2P.getPeerName (), securedSession );
	    logout.setEnabled ( true );
	    sendBox.requestFocus ();
        }
        else if ( evt.getSource() == logout ) {
	    logout.setEnabled ( false );
	    chat.logout();
	    login.setEnabled ( true );
	    securedBox.setEnabled ( true );
        }
    }

    public void itemStateChanged ( ItemEvent evt ) {
	securedSession = ( evt.getStateChange() == ItemEvent.SELECTED ) ;
    }

    public void error ( String str ) {
        displayMessage ( "ERROR> " + str + "\n" );
    }

    public void debug_MSG ( String str ) {
	//        displayMessage ( "DEBUG> " + str + "\n" );
    }

    public void info ( String str ) {
        displayMessage ( str );
    }

    public void displayMessage ( String str ) {
	synchronized (this) {
	    messageBoard.append ( str );
	}
    }

    public boolean displayDialog ( String name, String pipeID ) {
    
       if ( chatDialog == null) {
            chatDialog = new ChatProposal( parentFrame );
           
            chatDialog.addWindowListener(new java.awt.event.WindowAdapter() {
                public void windowClosing(java.awt.event.WindowEvent e) {
                    dialogWindowClosing();
                }
            });
        }
        chatDialog.setText ( name, pipeID );

	if ( acceptProp )
	  securedBox.setState ( acceptProp );
	  
	return acceptProp;
    }

    private void dialogWindowClosing() {
        if ( chatDialog != null) {
            chatDialog.setVisible(false);
        }
    }

    class ChatProposal extends Dialog implements ActionListener {

        Button accept;
        Button reject;
        TextArea text;

        public ChatProposal ( Frame frame) {
            super( frame, "Chat Proposal", true );
	    setSize ( 400, 300 );

            setBackground(Color.lightGray);
            setForeground( Color.black );

            setLayout(new BorderLayout());
      
            Panel buttonPanel = new Panel ();

            text = new TextArea ( "", 10, 40, 
				 TextArea.SCROLLBARS_NONE  );

            accept = new Button( "Accept" );
            reject = new Button( "Reject" );

            buttonPanel.add( accept );
            buttonPanel.add( reject );

            add ( text, "Center" );
            add ( buttonPanel, "South" );

            accept.addActionListener( this );
            reject.addActionListener( this );
	}

        public void setText ( String name, String pipeID ) {
	    text.setText ( "" );
	    text.append ( name + " - " + pipeID );
	    text.append ( "\n" );
	    text.append ( "is proposing a chat on secured channel\n" );
	    text.append ( "\n" );
	    text.append ( "Click on Accept to accept the proposal," );
	    text.append ( "else click on Reject\n" );
	    text.append ( "\n" );
	    text.append ( "WARNING: if accepted, you would loose your current Secured session\n");
	    setVisible ( true );
        }

        public void doLayout() {
            super.doLayout();

            Dimension pd = parentFrame.getSize();
            Dimension d = getSize();
            if (d.width == 0 || d.height == 0) {
                Point p = parentFrame.getLocationOnScreen();
                int x = p.x + (pd.width-d.width)/2;
                int y = p.y + (pd.height-d.height)/2;

                setLocation(new Point(x, y));
                setSize(getPreferredSize());
            }
        }
      
        public void actionPerformed(ActionEvent evt) {
            // System.out.println("action "+evt);
            if ( evt.getSource().equals( accept )) {
                dialogWindowClosing();
		acceptProp = true;
            } else if ( evt.getSource().equals( reject )) {
                dialogWindowClosing();
		acceptProp = false;
            } 
        }
    }
}
