/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: PeerGroupPanel.java,v 1.12 2001/06/02 05:38:11 kuldeep Exp $
 *
 */

package net.jxta.instantp2p.desktop;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import net.jxta.widget.*;
import net.jxta.instantp2p.*;

import net.jxta.discovery.*;
import net.jxta.exception.*;
import net.jxta.peergroup.*;
import net.jxta.protocol.*;

public class PeerGroupPanel extends Panel implements ActionListener, 
  ItemListener {

    private PeerGroupManager pgMgr = null;
    private Frame parentFrame = null;
    
    private List groupList = new List(6);
    private List peerList = new List(6);
    
    private Vector groups = new Vector( 50 );
    private Vector peers = new Vector( 100 );
    private Vector myGroups = new Vector ( 10 );
    private Vector myGroupAdvs = new Vector ( 10 );
    private Vector tempGroups = new Vector ();
    private Vector listeners = new Vector();
    
    private NewGroupDialog newGroupDialog = null;
    
    private Choice         searchChoice;
    private TextField      searchField;
    private Button         searchButton;

    private final String   SEARCHALLGROUPS = "All Groups";
    private final String   SEARCHMYGROUPS  = "My Groups";
    private final String   SEARCHAGROUP    = "A Group Name";
    private final String   SELECTITEM      = "Select ...";

    private Cursor waitCursor = new Cursor ( Cursor.WAIT_CURSOR );
    private Cursor currCursor = null;

    public PeerGroupPanel( PeerGroup group ) {
	try {
	    pgMgr = new PeerGroupManager ( group );
	} catch ( Exception e ) {
	    return ;
	}
	
	myGroups.addElement ( group );
	myGroupAdvs.addElement ( group.getAdvertisement ());

        parentFrame = InstantP2P.getFrame ();

        GridBagLayout gridLayout = new GridBagLayout();
        setLayout(gridLayout);

        GridBagConstraints c = new GridBagConstraints();

	Panel buttonPanel = new Panel ();
        buttonPanel.setBackground ( new Color ( 158, 158, 204 ));
        buttonPanel.setForeground ( Color.black );
	buttonPanel.setLayout ( new FlowLayout ());

	searchChoice = new Choice();
	searchChoice.add ( SELECTITEM );
	searchChoice.add ( SEARCHALLGROUPS );
	searchChoice.add ( SEARCHMYGROUPS );
	searchChoice.add ( SEARCHAGROUP );
	searchChoice.addItemListener ( this );
	buttonPanel.add ( searchChoice ) ;
	
	searchField = new TextField ( 25 );
	searchField.setBackground ( Color.white );
	searchField.setForeground ( Color.black );
	searchField.setEnabled ( false );
	searchField.addActionListener ( this );
	buttonPanel.add ( searchField );

	searchButton = new Button ( "Group Search" );
	searchButton.setEnabled ( false );
	searchButton.addActionListener ( this );
	buttonPanel.add ( searchButton );
	
        c.anchor = GridBagConstraints.NORTHWEST;
        c.insets = new Insets (2,2,2,2);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = 1;
       	c.weightx = 1.0;
        c.weighty = 0.0;

        gridLayout.setConstraints(buttonPanel, c);
        add(buttonPanel, c);
			       
        c.anchor = GridBagConstraints.NORTHWEST;
        c.insets = new Insets(2,2,2,2);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = GridBagConstraints.RELATIVE;
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.gridheight = GridBagConstraints.REMAINDER;
       	c.weightx = 0.5;
        c.weighty = 1.0;

	groupList.setBackground ( Color.white );
	groupList.setForeground ( Color.black );
	groupList.addItemListener ( this );
        Border groupListBorder = new Border ( groupList, "Peer Groups" );
        gridLayout.setConstraints(groupListBorder, c);
        add(groupListBorder, c);

        c.anchor = GridBagConstraints.NORTH;
        c.insets = new Insets(2,2,2,2);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = GridBagConstraints.RELATIVE;
        c.gridy = GridBagConstraints.RELATIVE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = GridBagConstraints.REMAINDER;
       	c.weightx = 0.5;
        c.weighty = 1.0;

	peerList.setBackground ( Color.white );
	peerList.setForeground ( Color.black );

	peerList.addItemListener ( this );
        Border peerListBorder = new Border (peerList, "Peers");
        gridLayout.setConstraints(peerListBorder, c);
        add(peerListBorder, c);
	
    }

    public void addPeerListener ( PeerListener listener ) {
	listeners.addElement( listener );
    }
    
    public void removePeerListener ( PeerListener listener ) {
	listeners.removeElement ( listener );
    }
 
    public void itemStateChanged(ItemEvent evt) {
        // System.out.println("PeerGroupPanel.itemStateChanged()"+evt);
      if ( searchChoice.getSelectedItem ().equals ( SELECTITEM ) )
	  searchChoice.select ( SEARCHALLGROUPS );

	toggleButtons ( false );
	if ( evt.getSource().equals ( groupList )) {
	    int idx = groupList.getSelectedIndex();
	    if (idx != -1) {
		searchPeers ( idx );
	    }
	}
	else if (  evt.getSource().equals ( peerList )) {
	    int idx = peerList.getSelectedIndex();
	    if ( idx != -1 )
		processPeerChangedEvent (( PeerGroupAdvertisement ) 
					  peers.elementAt ( idx ));
	}
	else if ( evt.getSource().equals ( searchChoice )) {

	    if ( searchChoice.getSelectedItem ().equals ( SEARCHAGROUP ) ) {
		searchField.setEnabled ( true );
		searchButton.setEnabled ( true );
		searchField.requestFocus ();
		if ( groupList.getItemCount () > 0 )
		    groupList.removeAll ();
		if ( peerList.getItemCount () > 0 )
		    peerList.removeAll ();
	    }
	    else {
		searchField.setEnabled ( false );
		searchButton.setEnabled ( false );

		if ( searchChoice.getSelectedItem ().equals ( SEARCHALLGROUPS ) ) {
		    updateGroupList();   
		}       
		else if ( searchChoice.getSelectedItem ().equals ( SEARCHMYGROUPS )) {
		    displayMyGroups ();
		}
		
		else {
		    if ( groupList.getItemCount () > 0 )
			groupList.removeAll ();
		    if ( peerList.getItemCount () > 0 )
			peerList.removeAll ();
		}
	    }
	    processGroupChangedEvent ( null );

	}

	toggleButtons ( true );
    }
    
    public void actionPerformed(ActionEvent evt) {
        //System.out.println("PeerGroupPanel.actionPerformed()"+evt);

      if ( searchChoice.getSelectedItem ().equals ( SELECTITEM ) )
	  searchChoice.select ( SEARCHALLGROUPS );

	toggleButtons ( false );
	if ( evt.getSource ().equals ( searchButton) || 
	     evt.getSource ().equals ( searchField) ) {
	    searchGroups ();
	} 
	else if ( evt.getActionCommand ().equals ( "Join Group" )) {
	    int idx = groupList.getSelectedIndex();
	    PeerGroupAdvertisement tmpadvt = null;
	    if ( idx != -1 ) {
		try {
		    if ( searchChoice.getSelectedItem ().equals ( SEARCHALLGROUPS ) )
			tmpadvt = 
			    joinGroup (( PeerGroupAdvertisement) groups.elementAt( idx ));
		    if ( searchChoice.getSelectedItem ().equals ( SEARCHAGROUP ) )
			tmpadvt = 
			    joinGroup (( PeerGroupAdvertisement) tempGroups.elementAt( idx ));
		    if ( tmpadvt == null ) {
			String [] strs = new String [2];
			strs[0] = "Error in joining Group";
			strs[1] = "Please try later";
			InstantP2P.getDialog().setText ( strs );
		    }
		    else 
			refreshMyGroups ();
		} catch ( Throwable t ) {
		    //t.printStackTrace ();
		    toggleButtons ( true );
		    String [] strs = new String [2];
		    strs[0] = "Error in joining Group";
		    strs[1] = "Please try later";
		    InstantP2P.getDialog().setText ( strs );
		    processGroupChangedEvent ( null );
		    return;
		}
	    }
	}
	else if ( evt.getActionCommand ().equals ( "Add Group" )) {
	    newGroupDialog();
	}
        else if ( evt.getActionCommand ().equals ( "Leave Group" )) {
	    int idx = groupList.getSelectedIndex();
	    if ( idx != -1 )
	        leaveGroup ( idx );
	}
	else if  ( evt.getActionCommand ().equals ( "Refresh" )) {
	    refresh ();
	}
	toggleButtons ( true );
    }

    private void refresh () {
        processGroupChangedEvent ( null );
        processPeerChangedEvent ( null );

	// first reload all the groups, if All groups selected
	if ( searchChoice.getSelectedItem ().equals ( SEARCHALLGROUPS ) ) {
	    updateGroupList();
	} 	      

	// reload peers in the selected group
	int idx = groupList.getSelectedIndex();
	if (idx != -1) {
	  searchPeers ( idx );
	}
    }

    private synchronized void leaveGroup ( int index ) {
        
        if ( index == 0 ) {
	    String [] strs = new String [2];
            strs[0] = "You have tried leaving NetPeerGroup";
            strs[1] = "You can't leave this World group";
	    InstantP2P.getDialog().setText ( strs );
        }
	else if ( index < myGroups.size () ) {
	    myGroups.removeElementAt ( index );
	    myGroupAdvs.removeElementAt ( index );
	    groupList.remove ( index );
	    refreshMyGroups ();
	}
    }

    private void refreshMyGroups () {
        searchChoice.select ( SEARCHMYGROUPS );
	displayMyGroups ();
	groupList.select ( groupList.getItemCount() -1 ); 
	updatePeerList( ( PeerGroupAdvertisement ) 
			myGroupAdvs.elementAt ( myGroupAdvs.size() - 1 ));
	processGroupChangedEvent (( PeerGroup) 
		  myGroups.elementAt ( myGroups.size() - 1 ));

    }

    private void toggleButtons ( boolean flag ) {

	if ( !flag ) {
	    currCursor = getCursor ();
	    parentFrame.setCursor ( waitCursor );
	}
	else parentFrame.setCursor ( currCursor );

	searchChoice.setEnabled ( flag );
    }

    private void searchGroups () {
	String groupName = searchField.getText ();
	if (!( groupName == null && groupName.equals ( "" ))) {
	    
	    if (!( listGroupsByName ( groupName )))
		StatusPanel.displayStatus ( groupName + " not found. try Later" ) ;
	}
	
	StatusPanel.displayStatus ( "Searching Groups: Done" );
    }


    private void searchPeers ( int index ) {

        String item = searchChoice.getSelectedItem();
	int memberIndex = -1;

        if ( item.equals ( SEARCHALLGROUPS )) {
	    memberIndex = updatePeerList( ( PeerGroupAdvertisement ) groups.elementAt ( index ));
        } else if ( item.equals ( SEARCHMYGROUPS )) {
	    memberIndex = updatePeerList( ( PeerGroupAdvertisement ) myGroupAdvs.elementAt ( index ));
        } else {
	    if ( tempGroups != null ) {
		memberIndex = updatePeerList( ( PeerGroupAdvertisement ) tempGroups.elementAt ( index ));
	    }
	}
	if ( memberIndex >= 0 )
	    processGroupChangedEvent (( PeerGroup) 
				      myGroups.elementAt ( memberIndex ));
	
    }

    private void processGroupChangedEvent ( PeerGroup group ) {
	// notify listener that a the Vector has changed
	for ( int i = 0; i < listeners.size(); i++ ) {
	    (( PeerListener ) listeners.elementAt(i)).groupChanged( group );
	}
    }

    private void processPeerChangedEvent ( PeerGroupAdvertisement peerAdv ) {
	// notify listener that a the Vector has changed
	for ( int i = 0; i < listeners.size(); i++ ) {
	    (( PeerListener ) listeners.elementAt(i)).peerChanged( peerAdv );
	}
    }

    private int alreadyAMember (Vector vector, PeerGroupAdvertisement pgAdv ) {
	int i;
	for ( i = 0; i < vector.size(); i++ ) {
	    if ( pgAdv.getGid().equals ( ((PeerGroupAdvertisement) vector.elementAt(i)).getGid()))
		return i;
	}	
	return -1;
    }


    private PeerGroupAdvertisement joinGroup ( PeerGroupAdvertisement pgAdv ) {
	PeerGroup group;

	if ( alreadyAMember ( myGroupAdvs, pgAdv ) != -1 ) {
	    StatusPanel.displayStatus ( "You are Already a Member of " + pgAdv.getName());
	     return pgAdv;
	}

        if ("net.jxta.instantp2p.InstantP2PApp".equals(pgAdv.getAppCode())) {
            // join the peergroup and set it as the current
	    group = pgMgr.joinGroup( pgAdv );
        } else {
            // This PeerGroup is not a sharing PeerGroup
            // Start it outside of the InstantP2P Group Tree
            group = PeerGroupFactory.newPeerGroup();
	    
            if ( group == null ) 
		return null;

            // initialize the peergroup

            try {
                group.init( pgMgr.getRootPeerGroup(), pgAdv );
            } catch (PeerGroupException e) {
                //e.printStackTrace();
                return null;
            }
	    //	    group.startApp(null);
        }

	if ( group != null ) {
	    myGroups.addElement ( group );
	    myGroupAdvs.addElement ( pgAdv );
	    return pgAdv;
	}
	return null;
    }


    private void createPeerGroup ( String name ) {
	PeerGroupAdvertisement pgAdvt;
	toggleButtons ( false );
	try {
	    pgAdvt = pgMgr.createGroup ( name );
	    
	    if ( pgAdvt != null ) {
		pgMgr.publishGroup ( pgAdvt );
		myGroups.addElement ( pgMgr.joinGroup ( pgAdvt ));
		myGroupAdvs.addElement ( pgAdvt );
		//		updateGroupList ();
		refreshMyGroups ();
	    }
	    
	} catch ( PeerGroupException pge ) {
	    //pge.printStackTrace ();
	    StatusPanel.displayStatus ( name + " group couldn't be created ");
	    toggleButtons ( true );
	    return ;
	}
	toggleButtons ( true );
    }


    private void updateGroupList () {

        Enumeration enum = null;

	if ( peerList.getItemCount () > 0 )
	    peerList.removeAll();

	if ( groupList.getItemCount () > 0 )
	  groupList.removeAll();

        enum = pgMgr.getGroups();

        while ( enum != null && enum.hasMoreElements()) {
            PeerGroupAdvertisement pgAdv = 
		( PeerGroupAdvertisement ) enum.nextElement();
	    if ( alreadyAMember ( groups, pgAdv ) == -1 ) {
	      groups.addElement( pgAdv );
	    }
        }
	for ( int i = 0; i < groups.size(); i ++ ) {
	    PeerGroupAdvertisement pgAdv = 
	       ( PeerGroupAdvertisement ) groups.elementAt(i);
	    groupList.add( pgAdv.getName());
	}
    }

    private void displayMyGroups () {

	if ( peerList.getItemCount () > 0 )
	    peerList.removeAll();
	
	if ( groupList.getItemCount () > 0 )
	    groupList.removeAll();
	
	for ( int i = 0; i < myGroupAdvs.size (); i ++ ) {
	    PeerGroupAdvertisement peerGroup = ( PeerGroupAdvertisement ) myGroupAdvs.elementAt ( i );
	    groupList.add( peerGroup.getName());
	}
    }

    private boolean listGroupsByName ( String groupName ) {

        Enumeration enum = null;

	if ( peerList.getItemCount () > 0 )
	    peerList.removeAll();

	if ( groupList.getItemCount () > 0 )
	    groupList.removeAll();

	tempGroups.removeAllElements ();

        enum = pgMgr.getGroupByName ( groupName );
	int groupCount = 0;
        while (enum != null && enum.hasMoreElements()) {
            PeerGroupAdvertisement pgAdvt = (PeerGroupAdvertisement) enum.nextElement();
            groupList.add(pgAdvt.getName());
	    groupCount++;
	    tempGroups.addElement ( pgAdvt );
        }
	return ( groupCount > 0 );
    }

    private int updatePeerList ( PeerGroupAdvertisement pgAdv ) {
        // check if the PeerGroup is an InstantP2P PeerGroup
      
	if ( peerList.getItemCount() > 0)
	    peerList.removeAll();

	int index = alreadyAMember ( myGroupAdvs, pgAdv );
	if ( index != -1 ) {
	    StatusPanel.displayStatus ( "Searching for peers in " + pgAdv.getName());
	}
	else {
	    StatusPanel.displayStatus ( "You are not a member of " + pgAdv.getName());
	    String [] strs = new String [4];
            strs[0] = "You are not a member of " + pgAdv.getName();
            strs[1] = "To access peers: ";
            strs[2] = "Click on \"Group\" on the Menu Bar";
            strs[3] = "Click on \"Join Group\"";
	    InstantP2P.getDialog().setText ( strs );
	    return index;
	}
	
	pgMgr.checkDiscoverResults( (PeerGroup) myGroups.elementAt ( index )) ;
	StatusPanel.displayStatus ( "Discovering peers.... DONE");
	
	Enumeration enum = null;
	
	peers.removeAllElements ();

	enum = pgMgr.getPeers ();
	while (enum != null && enum.hasMoreElements()) {
	    PeerGroupAdvertisement peerAdv =
		( PeerGroupAdvertisement ) enum.nextElement();
	    peerList.add( peerAdv.getPeerName()+ " - "+ getEndpoints( peerAdv ));
	    peers.addElement( peerAdv );	    
	}
	return index;
    }


    private String getEndpoints(PeerGroupAdvertisement adv) {
        String endpoint = "";

        Hashtable endpoints = adv.getEndpointAdvertisements();
        Enumeration enum = endpoints.elements();
        while (enum.hasMoreElements()) {
            EndpointAdvertisement epAdv = (EndpointAdvertisement)enum.nextElement();

            endpoint = endpoint + "   "+epAdv.getEndpointAddress();
	    break;  // hack to stop adding http:.... stuff
        }

        return endpoint;
    }

    private void newGroupDialog() {
        if (newGroupDialog == null) {
            newGroupDialog = new NewGroupDialog();

       		newGroupDialog.addWindowListener(new java.awt.event.WindowAdapter() {
       		    public void windowClosing(java.awt.event.WindowEvent e) {
                    dialogWindowClosing();
                }
            });
        }

        newGroupDialog.setVisible(true);
    }

    private void dialogWindowClosing() {
        newGroupDialog.setVisible(false);
    }
    
    class NewGroupDialog extends Frame implements ActionListener {
        TextField nameField;
        Button ok;
        Button cancel;
        
        public NewGroupDialog() {
            super("Get New Group Name");
            setBackground(Color.lightGray);

            setLocation(new java.awt.Point(100, 100));
            setSize(new java.awt.Dimension(450, 100));

            setLayout(new BorderLayout());
            Panel textPanel = new Panel ();
            textPanel.setLayout(new FlowLayout());
            Panel buttonPanel = new Panel ();

            nameField = new TextField(30);
            ok = new Button("ok");
            cancel = new Button("cancel");

            textPanel.add(new Label("New Peer Group Name"));
            textPanel.add(nameField);

            buttonPanel.add(ok);
            buttonPanel.add(cancel);

            add (textPanel, "Center");
            add (buttonPanel, "South");

            nameField.addActionListener(this);
            ok.addActionListener(this);
            cancel.addActionListener(this);
        }

        public void doLayout() {
            super.doLayout();
	    
            Dimension pd = parentFrame.getSize();
            Dimension d = getSize();
            Point p = parentFrame.getLocationOnScreen();
            int x = p.x + (pd.width-d.width)/2;
            int y = p.y + (pd.height-d.height)/2;

            setLocation(new Point(x, y));
	    
            setSize(getPreferredSize());
        }

        public void actionPerformed(ActionEvent evt) {
            // System.out.println("action "+evt);
            if (evt.getSource().equals(ok) || evt.getSource().equals(nameField)) {
		if (!(nameField.getText().equals(""))) {
		    dialogWindowClosing();
		    createPeerGroup(nameField.getText());
		}
            } else if (evt.getSource().equals(cancel)) {
                dialogWindowClosing();
            }
        }
    }

    private void showPeerGroupAdvertisement(PeerGroupAdvertisement adv) {
        Frame frame = new Frame(adv.getName());
        TextArea text = new TextArea();
        frame.add(text);
        frame.setSize(400, 300);
	/*
        Point p = parentFrame.getLocationOnScreen();
        frame.setLocation(p.y+10, p.y+10);
	*/
	//TBD        text.setText(adv.prettyDisplay());

	frame.addWindowListener(new TextInfoWindowAdapter(frame));

	frame.setVisible(true);
    }

    class TextInfoWindowAdapter extends WindowAdapter {
        Frame frame = null;

        public TextInfoWindowAdapter(Frame frame) {
            this.frame = frame;
        }

        public void windowClosing(WindowEvent e) {
            if (frame != null) {
                frame.setVisible(false);
                frame.dispose();
                frame = null;
            }
        }
   }
}
