/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RemoteContentTab.java,v 1.17 2001/06/03 05:08:02 kuldeep Exp $
 */

package net.jxta.instantp2p.desktop;

import java.lang.reflect.InvocationTargetException;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import net.jxta.widget.*;
import net.jxta.instantp2p.*;

import net.jxta.share.*;

import net.jxta.protocol.*;
import net.jxta.peergroup.PeerGroup;
import net.jxta.platform.Application;

public class RemoteContentTab extends Panel implements ActionListener, 
    ItemListener, PeerListener, SearchListener {

    private static final int DefaultBufferSize = 64*1024;

    private InstantP2P iP2PApp = null;
    private Frame parentFrame = null;

    private PeerGroup curGroup = null;
    private Search search = null;

    private TextField searchField = new TextField(30);
    private Button searchButton = new Button("Search");
    private List contentList = new List();

    private GridBagLayout gridLayout = new GridBagLayout();

    private SearchResult[] searchResults = null;
    private Vector getRequests = new Vector();

    private SaveContentDialog saveContentDialog = null;
    private Cursor waitCursor = new Cursor ( Cursor.WAIT_CURSOR );
    private Cursor currCursor = null;

    private boolean wantToView = false;

    public RemoteContentTab(InstantP2P app, PeerGroupPanel pgPanel ) {
        this.iP2PApp = app;

        parentFrame = iP2PApp.getFrame();
        setBackground ( new Color ( 158, 158, 204 ));
        setForeground ( Color.black );
        setLayout(gridLayout);
        setComponentPlacement();
        pgPanel.addPeerListener ( this );

        searchButton.addActionListener(this);
        searchField.addActionListener(this);
        contentList.addItemListener(this);
    }

    private void setComponentPlacement() {
        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(2,2,2,2);
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = GridBagConstraints.RELATIVE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = 1;
       	c.weightx = 1.0;
        c.weighty = 0.0;

        Panel searchPanel = new Panel();
        searchPanel.add(searchField);
	searchField.setBackground ( Color.white );
	searchButton.setEnabled ( false );
	searchField.setEnabled ( false );

        searchPanel.add(searchButton);
	//Border searchBorder = new Border (searchPanel, new Label("Search"), Border.ALIGN_LEFT);
        gridLayout.setConstraints(searchPanel, c);
        add(searchPanel, c);

        c.anchor = GridBagConstraints.NORTHWEST;
        c.insets = new Insets(2,2,2,2);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = GridBagConstraints.RELATIVE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        //c.gridheight = 5;
        c.gridheight = GridBagConstraints.REMAINDER;
       	c.weightx = 1.0;
        c.weighty = 1.0;

        Border contentListBorder = new Border (contentList, new Label("Content Search Results"), Border.ALIGN_LEFT);
        gridLayout.setConstraints(contentListBorder, c);
        add(contentListBorder, c);
	contentList.setBackground ( Color.white );
	contentList.setForeground ( Color.black );
    }

     private void toggle ( boolean flag ) {

	if ( !flag ) {
	    currCursor = getCursor ();
	    parentFrame.setCursor ( waitCursor );
	}
	else parentFrame.setCursor ( currCursor );

	searchButton.setEnabled ( flag );
	searchField.setEnabled ( flag );
    }

    public void actionPerformed(ActionEvent evt) {
        // System.out.println("RemoteContentTab.actionPerformed() "+evt);

        if (null == search) {
            return;
        }

        if (evt.getSource().equals(searchButton) ||
	     evt.getSource().equals(searchField)) {
	    toggle ( false );
            search.search(searchField.getText());
            //XXX may be we do not need this call
	    updateContentList();
	    toggle ( true );
        }
    }

    public void itemStateChanged(ItemEvent evt) {
        // System.out.println("RemoteContentTab.itemStateChanged() "+evt);

        if (null == search) {
            return;
        }

        if (evt.getSource().equals(contentList)) {
            int idx = contentList.getSelectedIndex();
            if (idx != -1 &&  null != searchResults) {
                // get the content
	        ContentAdvertisement cAdv = searchResults[idx].contentAdv;
		if ( cAdv.getType() != null ) {
                    // get the mime type
                    MimeTable mimeTable = MimeTable.getDefaultMimeTable();
		    MimeInfo mInfo = mimeTable.get(cAdv.getType());

		    // can not display the content, save it
		    saveContentDialog( searchResults[idx], 
				       cAdv.getName(), mInfo );
		} else {
                    // can not display the content, save it
		    saveContentDialog( searchResults[idx], 
				       cAdv.getName(), null );
		}
            }
        }
    }

    public void groupChanged(PeerGroup group) {
        curGroup = group;

	if ( contentList.getItemCount () > 0 )
	    contentList.removeAll();

        if (null != group) {
	    searchButton.setEnabled ( true );
            searchField.setEnabled ( true );
            // cancel the search and remove the listener 
            if (null != search) {
                search.cancel();
                search.removeListener(this);
            }
            search = GroupManager.getInstance().getSearch(group);
            search.addListener(this);
        }
    }

    public void peerChanged ( PeerGroupAdvertisement pgAdv ) {
        // do nothing on Peer selection
    }

    public void searchUpdate() {
        updateContentList();
        repaint();
    }

    private synchronized void updateContentList() {

        if (search == null) {
            return ;
        }

        // Display the results
	if ( contentList.getItemCount () > 0 )
	    contentList.removeAll();

        searchResults = search.getResults();
	for (int i=0; i<searchResults.length; i++) {
            contentList.add(searchResults[i].contentAdv.getName());
        }
    }

    private void updateRequestList() {

        Enumeration enum = getRequests.elements();
        while (enum.hasMoreElements()) {
            GetRequestor request = (GetRequestor)enum.nextElement();
	    //System.out.println (request.getPercentDone()+"%   "+
	    //  request.getContentAdvertisement().getName());
        }
    }

    class GetRequestor extends GetContentRequest {
        public SearchResult searchResult = null;
        String url = null;

        public GetRequestor(PeerGroup pg, SearchResult res, File tmpFile) throws InvocationTargetException {
            super(pg, res.pipeAdv, res.contentAdv.getContentId(), tmpFile);
            searchResult = res;
	    url = tmpFile.getAbsolutePath ();
        }

        public ContentAdvertisement getContentAdvertisement() {
            return searchResult.contentAdv;
        }

        public void notifyDone() {
	    StatusPanel.displayStatus( getContentAdvertisement().getName() + " 100 % " );
	    if ( wantToView ) {
	        showURL ( url );
		wantToView = false;
	    }
        }

        public void notifyFailure() {
	    StatusPanel.displayStatus( getContentAdvertisement().getName() + " failed to download" );
        }

        public void notifyUpdate(int percentage) {
   	    StatusPanel.displayStatus( getContentAdvertisement().getName() + percentage + "% ");
	}
    }

    private synchronized void getContent(SearchResult result, String fileName ) {
        // System.out.println("RemoteContentTab.getContent()");

        File tmpFile = new File( fileName);

        try {
            GetRequestor request = new GetRequestor(curGroup, result, tmpFile);
            getRequests.addElement(request);
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }

        updateRequestList();
    }

    private void showURL ( String url ) {
        String[] command = null;
	Runtime rt = Runtime.getRuntime ();

	if ( File.separator.equals ( "/" )) {
	  // unix platform
	  command = new String [3];
	  command[0] = "netscape";
	  command[1] = "-remote";
	  command[2] =  "openURL(" + url + ")";
	}
	else if ( File.separator.equals ( "\\" )) {
	  command = new String [2];
	  command[0] = "viewfile";
	  command[1] =  url;
	}
	try {
	    Process process = rt.exec ( command );
	} catch ( IOException e) {
	    //e.printStackTrace ();
	    // need to Handle browser case when netscape is not in path
	    String [] strs = new String [1];
	    strs[0] = "Mime Type not supported as yet";
	    InstantP2P.getDialog().setText ( strs );
	}
    }

    private void showContent( SearchResult result ) {
        GetRequestor requestor = null; 

        // System.out.println("RemoteContentTab.showContent() "+status);
	ContentAdvertisement cAdv = result.contentAdv;

        File rootDir = new File(GroupManager.IP2P_DIRNAME);
        if (!rootDir.exists()) {
            rootDir.mkdir();
        }

        File tmpDir = new File(rootDir, "tmp");
        if (!tmpDir.exists()) {
            tmpDir.mkdir();
        }

	String name = cAdv.getName ();
        String ext = "";
	int index = name.lastIndexOf('.');
	if ( index != -1 )
	  ext = name.substring ( name.lastIndexOf('.')  );
        File tmpFile = new File(tmpDir, Integer.toHexString(new UUID().hashCode()) + ext );
        try {
            requestor = new GetRequestor(curGroup, result, tmpFile);
            getRequests.addElement(requestor);
        } catch (InvocationTargetException e) {
	   String [] str = new String [1];
	   str[0] = "Error in getting contents...";
	   InstantP2P.getDialog().setText ( str );
	  //e.printStackTrace();
	  return;
        }

        //updateRequestList();
	wantToView = true;
	//	showURL ( tmpFile.getAbsolutePath() );	
    }

    private void saveContent(File contentFile, String path) {

        FileInputStream ip = null;
        FileOutputStream op = null;

        try {
            ip = new FileInputStream(contentFile);
            op = new FileOutputStream(path);
        } catch (FileNotFoundException e) {
            StatusPanel.displayStatus("Failed to open file" );
            //e.printStackTrace();
            return ;
        } catch (IOException e) {
            StatusPanel.displayStatus("Failed to open file" );
            //e.printStackTrace();
            return ;
        }

        try {
            byte[] buffer = new byte [DefaultBufferSize];

            while (ip.available() > 0) {
                int res = ip.read(buffer);
                op.write(buffer, 0, res);
            }
        } catch (Exception e) {
            StatusPanel.displayStatus( "Failed to write file" );
            //e.printStackTrace();
        }

        try {
            ip.close();
            op.close();
        } catch (Exception e) {
	  //System.err.println("Failed to close file");
	  //e.printStackTrace();
        }
    }

    private void saveContentDialog(SearchResult result, String name, MimeInfo mInfo) {
        if (saveContentDialog == null) {
            saveContentDialog = new SaveContentDialog(result, name, mInfo);
           
            saveContentDialog.addWindowListener(new java.awt.event.WindowAdapter() {
                public void windowClosing(java.awt.event.WindowEvent e) {
                    dialogWindowClosing();
                }
            });
        }
	else saveContentDialog.setResult ( result );

        saveContentDialog.setVisible(true);
    }

    private void dialogWindowClosing() {
        if (saveContentDialog != null) {
            saveContentDialog.setVisible(false);
        }
    }

    class SaveContentDialog extends Frame implements ActionListener {
        TextField path;
        Button browse;
        Button ok;
        Button cancel;
        Button view;
        String name;
        SearchResult searchResult;

        public SaveContentDialog( SearchResult result, String name, MimeInfo mInfo) {
            super("save content");
            setBackground(Color.lightGray);

            searchResult = result;
            this.name = name;

            setLayout(new BorderLayout());
            Panel textPanel = new Panel ();
            textPanel.setLayout(new FlowLayout());
            Panel buttonPanel = new Panel ();

            path = new TextField(30);
            browse = new Button("browse");
            ok = new Button("ok");
            cancel = new Button("cancel");
            view = new Button("view");
            view.setEnabled(false);

            textPanel.add(new Label("Path"));
            textPanel.add(path);
            textPanel.add(browse);

            buttonPanel.add(ok);
            buttonPanel.add(cancel);
            buttonPanel.add(view);

            add (textPanel, "Center");
            add (buttonPanel, "South");

            path.addActionListener(this);
            browse.addActionListener(this);
            ok.addActionListener(this);
            cancel.addActionListener(this);
            view.addActionListener(this);

	    //    if ((mInfo != null ) && ("text/plain".equals(mInfo.getType()))) {
                view.setEnabled(true);
		// }
        }

        public void setResult ( SearchResult result ) {
	    searchResult = result;
	}

        public void doLayout() {
            super.doLayout();

            Dimension pd = parentFrame.getSize();
            Dimension d = getSize();
            if (d.width == 0 || d.height == 0) {
                Point p = parentFrame.getLocationOnScreen();
                int x = p.x + (pd.width-d.width)/2;
                int y = p.y + (pd.height-d.height)/2;

                setLocation(new Point(x, y));
                //setSize(getPreferredSize());
		pack ();
            }
        }
      
        public void actionPerformed(ActionEvent evt) {
	    //System.out.println("action "+evt);
            if (evt.getSource().equals(ok) || evt.getSource().equals(path)) {
	        if ( path.getText().equals ( "" ))
		     return;
                dialogWindowClosing();
		getContent ( searchResult, path.getText());
            } else if (evt.getSource().equals(cancel)) {
                dialogWindowClosing();
            } else if (evt.getSource().equals(view)) {
                dialogWindowClosing();
		showContent( searchResult );
            } else if (evt.getSource().equals(browse)) {
                FileDialog fd = new FileDialog(this, "Save Content", FileDialog.SAVE);
                setEnabled(false);
                fd.setFile(name);
                fd.show();
                setEnabled(true);
                
                String fileName = fd.getDirectory()+fd.getFile();
                // System.out.println("browse path: "+fileName);

                if (fd.getDirectory() != null) {
                    dialogWindowClosing();
		    getContent ( searchResult, path.getText());
                }
            }
        }
    }
      
      class RequestListMouseAdapter extends MouseAdapter {
	public void mouseReleased(MouseEvent e) {
        }
      }

      class contentListMouseAdapter extends MouseAdapter {
	public void mouseReleased(MouseEvent e) {
	  if (e.isPopupTrigger()) {
	    //updateRequestList();
	  }
        }
    }
}
