/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: InstantP2P.java,v 1.10 2001/06/03 05:08:41 kuldeep Exp $
 *
 */

package net.jxta.instantp2p.pda;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import net.jxta.widget.*;
import net.jxta.instantp2p.*;

import net.jxta.peergroup.PeerGroup;
import net.jxta.exception.PeerGroupException;
import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.platform.Application;
import net.jxta.document.Advertisement;

public class InstantP2P implements Application {

    private static Frame iP2PFrame;

    private PeerGroupPanel peerGroupPanel = null;
    private RemoteContentTab remoteContentTab = null;
    private LocalContentTab localContentTab = null;
    private Messaging groupChat = null;
    private Messaging chat = null;

    private MenuItem saveAs, exit;
    private MenuItem prefs, about;
    private MenuItem addGroup, joinGroup, leaveGroup, refresh;
    private static String myName = null;
    private static String myPassword = null;
    private PeerGroup group;
    private InstantP2P thisapp ;
    private static CommonDialog dialog;

    public InstantP2P() {
    }

    public void init ( PeerGroup group, Advertisement adv ) 
      throws PeerGroupException {
        this.group = group;
        thisapp = this;
	myName = group.getAdvertisement().getPeerName();

        Thread app = new InstantP2PMain ();
        app.setPriority ( Thread.MAX_PRIORITY );
        app.start ();
    }

    public int startApp ( String[] str ) {
        return 0;
    }

    public void stopApp () {
    }

    public PeerGroupAdvertisement getAdvertisement() {
        return group.getAdvertisement();
    }

    public static String getPeerName () {
        return myName;
    }

    public static CommonDialog getDialog () {
        return dialog;
    }

    public static Frame getFrame() {
        return iP2PFrame;
    }

    class InstantP2PMain extends Thread implements ActionListener {
        Panel appPanel;

	public InstantP2PMain () {
	
	iP2PFrame = new Frame("InstantP2P");
        iP2PFrame.setBackground( new Color ( 102, 102, 153 ));
        iP2PFrame.setForeground( Color.white );

	iP2PFrame.setSize (230, 310);

	//login ();
	layoutComponents ();
	addMenuBar ();
	dialog = new CommonDialog ( iP2PFrame );
    }
    
    public void actionPerformed ( ActionEvent event ) {
        MenuItem item = ( MenuItem ) event.getSource ();
	
	if ( item == exit ) {
	    exitInstantP2P ();
	}
	else if ( item == about ) {
	    String [] str = new String [1];
	    str[0] = "InstantP2P Version: " + Version.version;
	    dialog.setText ( str );
	}else {
  	    CardLayout cl = ( CardLayout )( appPanel.getLayout());
            cl.show( appPanel, ( String )event.getActionCommand());
        }
    }

    private void login () {
        LoginDialog dialog = new LoginDialog(iP2PFrame, "InstantP2P Login");
        dialog.setVisible(true);
    }


    private void addMenuBar () {
	// Menu Bars over here

	MenuBar menubar = new MenuBar ();
	Menu fileMenu = new Menu ( "File" );
	Menu editMenu = new Menu ( "Edit" );
	Menu navMenu = new Menu ( "Navigation" ); 
	Menu grpMenu = new Menu ( "Group" ); 	
	Menu helpMenu = new Menu ( "Help" );
 
        menubar.add ( fileMenu );
        menubar.add ( editMenu );
	menubar.add ( navMenu );
	menubar.add ( grpMenu );
	menubar.add ( helpMenu );

	exit = new MenuItem ( "Exit" );
	saveAs = new MenuItem ( "Save As" );
	prefs = new MenuItem ( "Preferences" );
	prefs.setEnabled ( false );
	addGroup = new MenuItem ( "Add Group" );
	joinGroup = new MenuItem ( "Join Group" );
	leaveGroup = new MenuItem ( "Leave Group" );
	refresh = new MenuItem ( "Refresh" );

	about = new MenuItem ( "About InstantP2P" );

	// Navigation menu
        MenuItem findMenu = new MenuItem ( "Search" );
        MenuItem shareMenu = new MenuItem ( "Share" );
        MenuItem chatMenu = new MenuItem ( "Chat" );
        MenuItem groupChatMenu = new MenuItem ( "Group Chat" );
        chatMenu.addActionListener ( this );
        groupChatMenu.addActionListener ( this );
        shareMenu.addActionListener ( this );
        findMenu.addActionListener ( this );

	navMenu.add ( findMenu );
	navMenu.addSeparator ();
	navMenu.add ( shareMenu );
	navMenu.addSeparator ();
	navMenu.add ( chatMenu );
	navMenu.addSeparator ();
	navMenu.add ( groupChatMenu );
	
	addGroup.addActionListener ( peerGroupPanel );
	joinGroup.addActionListener ( peerGroupPanel );
	leaveGroup.addActionListener ( peerGroupPanel );
	refresh.addActionListener ( peerGroupPanel );

	exit.addActionListener ( this );
	about.addActionListener ( this );

	fileMenu.add ( exit );
	editMenu.add ( prefs );

	grpMenu.add ( addGroup );
	grpMenu.addSeparator ();
	grpMenu.add ( joinGroup );
	grpMenu.addSeparator ();
	grpMenu.add ( leaveGroup );
	grpMenu.addSeparator ();
	grpMenu.add ( refresh );

	helpMenu.add ( about );

	iP2PFrame.setMenuBar ( menubar );
    }

    private void layoutComponents () {

	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	iP2PFrame.setLayout(gridbag);
	
        peerGroupPanel = new PeerGroupPanel( group );
	remoteContentTab = new RemoteContentTab( thisapp, peerGroupPanel );
	localContentTab = new LocalContentTab( thisapp, peerGroupPanel );
	groupChat = new Messaging( group, "GroupChat", peerGroupPanel );
	chat = new Messaging( group, "Chat", peerGroupPanel );

	c.anchor = GridBagConstraints.NORTH;	
	c.insets = new Insets ( 0, 0, 2, 0 );
	c.gridx = 0;

	c.fill = GridBagConstraints.BOTH;
	
	c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 0.5;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridheight = 6;//GridBagConstraints.RELATIVE;
	gridbag.setConstraints( peerGroupPanel, c);
	iP2PFrame.add ( peerGroupPanel );

	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;	
	c.insets = new Insets ( 0, 2, 2, 2 );	
	c.gridy = GridBagConstraints.RELATIVE;
	c.gridheight = GridBagConstraints.REMAINDER;	

	appPanel = new Panel ();
	appPanel.setLayout ( new CardLayout());

	appPanel.add ("Search", remoteContentTab);
	appPanel.add ("Share", localContentTab);
	appPanel.add ("Chat", chat);
	appPanel.add ("Group Chat", groupChat);
	appPanel.setBackground ( new Color ( 158, 158, 204 ));
        appPanel.setForeground ( Color.black );
	gridbag.setConstraints(appPanel, c);
	iP2PFrame.add (appPanel);

	StatusPanel status = new StatusPanel () ;

	iP2PFrame.addWindowListener(new java.awt.event.WindowAdapter() {
	    public void windowClosing(java.awt.event.WindowEvent e) {
		exitInstantP2P();
	    }
	});

	iP2PFrame.setVisible(true);
    }

    private void setLoginName ( String loginName, String password ) {
        myName = loginName;
        myPassword = password; // not being used now

        if ( loginName != null && !(loginName.equals (""))) {
            PeerGroupAdvertisement adv = getAdvertisement();
            adv.setPeerName(myName);
        }
    }

    private void exitInstantP2P () {
        iP2PFrame.setVisible(false);
        iP2PFrame.dispose();
        System.exit(0);
    }

    class LoginDialog extends Dialog implements ActionListener {
        TextField login;
        TextField password;
        Button ok;
        Button cancel;

        public LoginDialog( Frame frame, String title ) {
            super ( frame, title );
            setModal ( true );

            setBackground(Color.lightGray);
            setForeground(Color.black);

            setLocation(new java.awt.Point( 100, 100 ));
            setSize(new java.awt.Dimension( 200, 100 ));
	    
            GridBagLayout gridbag = new GridBagLayout ();
            GridBagConstraints c = new GridBagConstraints ();
            setLayout(gridbag);

            c.weightx = 0.1;
            c.weighty = 0.0;
            c.anchor = GridBagConstraints.NORTH;
            c.fill = GridBagConstraints.NONE;
            c.gridwidth = GridBagConstraints.RELATIVE;
            c.gridheight = GridBagConstraints.RELATIVE;
            c.insets = new Insets (2, 2, 2, 2);

            Label loginLabel = new Label ("User Name: ") ;
            gridbag.setConstraints( loginLabel, c);
            add ( loginLabel );

            c.gridwidth = GridBagConstraints.REMAINDER;

            login = new TextField( 15 );
            gridbag.setConstraints(login, c);
            add (login);

            c.gridwidth = GridBagConstraints.RELATIVE;

            Label passwordLabel = new Label ( "Password: " );
	    // TBD enable this field once we have password feature
	    passwordLabel.setEnabled ( false );
            gridbag.setConstraints( passwordLabel, c );
            add ( passwordLabel );

            c.gridwidth = GridBagConstraints.REMAINDER;

            password = new TextField( 15 );
            gridbag.setConstraints(password, c);
            add (password);

	    c.anchor = GridBagConstraints.NORTHEAST;
            c.gridwidth = GridBagConstraints.RELATIVE;
            c.gridheight = GridBagConstraints.REMAINDER;

            ok = new Button( "OK" );
            gridbag.setConstraints(ok, c);
            add (ok);

	    c.anchor = GridBagConstraints.NORTH;
            c.gridwidth = GridBagConstraints.REMAINDER;
            c.gridheight = GridBagConstraints.REMAINDER;

            cancel = new Button( "Cancel" );
            gridbag.setConstraints( cancel, c);
            add ( cancel );

            login.addActionListener( this );
            password.addActionListener( this );
            ok.addActionListener( this );
            cancel.addActionListener( this );
    
            addWindowListener( new java.awt.event.WindowAdapter () {
                public void windowClosing( java.awt.event.WindowEvent e ) {
                    dialogWindowClosing();
                }
            });
	    login.requestFocus ();
        }

        public void actionPerformed( ActionEvent evt ) {
            // System.out.println( "action "+evt );
            if (evt.getSource().equals( ok ) || evt.getSource().equals( login )) {
                setLoginName( login.getText(), password.getText() );
                dialogWindowClosing();
            } else if ( evt.getSource().equals( cancel )) {
                dialogWindowClosing();
                exitInstantP2P ();
            }
        }

        private void dialogWindowClosing() {
            setVisible( false );
	    dispose ();
        }
    }
    }
}
