/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: LocalContentTab.java,v 1.13 2001/06/03 05:09:21 kuldeep Exp $
 */

package net.jxta.instantp2p.pda;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import net.jxta.widget.*;
import net.jxta.instantp2p.*;
import net.jxta.share.*;

import net.jxta.protocol.*;
import net.jxta.peergroup.PeerGroup;

public class LocalContentTab extends Panel 
        implements ActionListener, ItemListener, PeerListener  {
    private InstantP2P iP2PApp = null;
    private Frame parentFrame = null;

    final static int ITEMS = 5;
    List contentList = new List(ITEMS);
    Button addButton = new Button("Add Content");
    Button removeButton = new Button("Remove Content");

    GridBagLayout gridLayout = new GridBagLayout();

    ContentManager contentManager = null;
    Content[] content = null;

    public LocalContentTab(InstantP2P app, PeerGroupPanel pgPanel ) {
        this.iP2PApp = app;

        setBackground ( new Color ( 158, 158, 204 ));
        setForeground ( Color.black );

        parentFrame = iP2PApp.getFrame();
        pgPanel.addPeerListener ( this );

        setLayout(gridLayout);
        setComponentPlacement();

        contentList.addItemListener(this);
        addButton.addActionListener(this);
        removeButton.addActionListener(this);
    }

    public void groupChanged ( PeerGroup group ) {
	if ( contentList.getItemCount () > 0 )
	    contentList.removeAll();        
	if (null != group) {
	    addButton.setEnabled ( true );
	    removeButton.setEnabled ( true );
            CMS cms = GroupManager.getInstance().getCMS(group);
            contentManager = cms.getContentManager();
            loadContent();
        }
    }

    public void peerChanged(PeerGroupAdvertisement pgAdv) {
        // nothing needs to be done when the peer changes
    }

    private void setComponentPlacement() {
        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(0,2,0,2);
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.gridy = GridBagConstraints.RELATIVE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = 1;
       	c.weightx = 1.0;
        c.weighty = 0.0;

        Panel buttonPanel = new Panel();
        buttonPanel.add(addButton);
        buttonPanel.add(removeButton);
	addButton.setEnabled ( false );
	removeButton.setEnabled ( false );
        //Border buttonBorder = new Border (buttonPanel, new Label("Operations"), Border.ALIGN_LEFT);
        gridLayout.setConstraints(buttonPanel, c);
        add(buttonPanel, c);

        c.anchor = GridBagConstraints.NORTHWEST;
        c.insets = new Insets(0,2,0,2);
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = GridBagConstraints.RELATIVE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = GridBagConstraints.RELATIVE;
       	c.weightx = 1.0;
        c.weighty = 1.0;

	Border contentListBorder = new Border (contentList, new Label("Shared Content"), Border.ALIGN_LEFT);
        gridLayout.setConstraints(contentListBorder, c);
        add(contentListBorder, c);
	contentList.setBackground ( Color.white );
	contentList.setForeground ( Color.black );
    }

    public void itemStateChanged(ItemEvent evt) {
        // System.out.println("LocalContentTab.itemStateChanged()"+evt);

        if (contentManager == null) {
            return ;
        }

        if (evt.getSource().equals(contentList)) {
            // show the content info
            int idx = contentList.getSelectedIndex();
            if (idx != -1) {
                showContentInfo(content[idx].getContentAdvertisement());
            }
        }
    }

    public void actionPerformed(ActionEvent evt) {
        // System.out.println("LocalContentTab.actionPerformed() "+evt);
        if (contentManager == null) {
            return ;
        }

        if (evt.getSource().equals(addButton)) {
            // Share new content
            addContentDialog();
        } else if (evt.getSource().equals(removeButton)) {
            // Share remove selected content
            int idx = contentList.getSelectedIndex();
            if (idx != -1) {
                try {
                    contentManager.unshare(content[idx]);
                } catch (IOException e) {
                    // XXX REMIND Should display error dialog here
                    e.printStackTrace();
                }
                loadContent();
           }
        }
    }

    private void showContentInfo(ContentAdvertisement cAdv) {

        String [] strs = new String [4];
        strs[0] = "Name: "+cAdv.getName();
        strs[1] = "ContentId: "+cAdv.getContentId();
	strs[2] = "Length: "+cAdv.getLength();
        if (cAdv.getType() != null) {
            strs[3] = "Type: "+cAdv.getType();
        }
        if (cAdv.getDescription() != null) {
            strs[3] = "Description: "+cAdv.getDescription();
        }
	InstantP2P.getDialog().setText ( strs );
    }

    private void loadContent() {
        content = contentManager.getContent();
	if ( contentList.getItemCount () > 0 )
	  contentList.removeAll();
        if (content != null) {
            for (int i=0; i<content.length; i++) {
                contentList.add(content[i].getContentAdvertisement().getName());
            }
        }
    }

    private void addContent(String path) {
        if ( path.equals ( "") ) {
	    return;
        }
        try {
            contentManager.share(new File(path));
        } catch (IOException e) {
            // XXX REMIND Should display error dialog here
            e.printStackTrace();
        }

        loadContent();
    }

    private void addContentDialog() {
      FileDialog fd = new FileDialog( parentFrame, "Select File");
      fd.setBackground ( Color.white );
      fd.setForeground ( Color.black );
      setEnabled(false);
      fd.setSize ( 200, 280 );
      fd.show();
      setEnabled(true);
      
      if ( fd.getFile() != null && fd.getDirectory() != null) {
	  String fileName = fd.getDirectory() + fd.getFile();
	  addContent(fileName);
      }
    }
}
