/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Border.java,v 1.3 2001/05/25 00:23:02 kuldeep Exp $
 */

package net.jxta.widget;

import java.awt.*;

public class Border extends Panel {
    private Label          title;
    private Rectangle      border;
    private Component      component;
    private int            alignment;
    private int            thickness;
    private boolean        engravedIn;

    public static int ALIGN_CENTER = 0;
    public static int ALIGN_LEFT = 1;
    public static int ALIGN_RIGHT = 2;
    public static int DEFAULT_THICKNESS =2;

    public Border(Component component, String title) { 
        this(component, new Label(title, Label.CENTER), ALIGN_CENTER);
    }

    public Border(Component component, String title, int alignment) {
        this(component, new Label(title, Label.CENTER), alignment);
    }

    public Border(Component component, Label title, int alignment) {

	this.component  = component;
        this.title      = title;
        this.alignment  = alignment;
        thickness  = DEFAULT_THICKNESS; // default value
        engravedIn = true;   // default value

        border = new Rectangle (thickness, thickness);

        GridBagLayout gbl = new GridBagLayout ();
        GridBagConstraints gbc = new GridBagConstraints ();

        setLayout (gbl);
        gbc.gridwidth = GridBagConstraints.REMAINDER;

        if (alignment == ALIGN_CENTER) {
            gbc.anchor = GridBagConstraints.NORTH;
        }
        else if (alignment == ALIGN_RIGHT) {
            gbc.anchor = GridBagConstraints.NORTHEAST;
            gbc.insets = new Insets (0, 0, 0, 5);
        }
        else if (alignment == ALIGN_LEFT) {
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.insets = new Insets (0, 5, 0, 0);
        }

        gbl.setConstraints (title, gbc);
        add(title);

        gbc.insets = new Insets (0, 5, 5, 5);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;

        gbl.setConstraints (component, gbc);
        add(component);
    }

    public void setEngraved (boolean engraved) {
        this.engravedIn = engraved;
    }

    public void setThickness ( int thickness) {
        this.thickness = thickness;
    }

    public int getThickness () {
        return thickness;
    }

    public void paint(Graphics g) { 
	Color topLeft = SystemColor.controlShadow; 
	Color botRight = SystemColor.controlLtHighlight; 

        if (engravedIn)
            paintEngraved (g, topLeft, botRight);
        else
            paintEngraved (g, botRight, topLeft);
    }

    private void paintEngraved (Graphics g, Color topLeft, Color botRight) {
	Dimension size = getSize();

        int w = border.width - thickness;
        int h = border.height - thickness;
        int x = border.x;
        int y = border.y;

        g.setColor (topLeft);
        for (int i = 0; i < thickness/2; i++) 
            g.drawRect ( x+i, y+i, w, h);

        g.setColor (botRight);
        for (int i = thickness/2; i < thickness; i++)
            g.drawRect ( x+i, y+i, w, h);
	super.paint(g);  // ensures lightweights are painted
    }

    public void setSize(int w, int h) {
	setBounds (getLocation().x, getLocation().y, w, h);
    }

    public void setBounds(int x, int y, int w, int h) {
        super.setBounds (x, y, w, h);

        FontMetrics fm = title.getFontMetrics (title.getFont());
        int top = getInsets().top + fm.getAscent();
        Dimension size = getSize ();
        border.setBounds (0, top, size.width - 1, size.height - top);
    }

    protected String paramString() {
        return super.paramString() + ",border=" + 
               border.toString() + ",thickness=" + thickness;
    }
}
