/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Rule.java,v 1.2 2001/05/25 00:23:02 kuldeep Exp $
 */

package net.jxta.widget;

import java.awt.*;

public class Rule extends Component {
    static boolean EMBOSSEDIN  = true;
    static boolean EMBOSSEDOUT = false;

    static private int DEFAULT_LENGTH         = 100; // length in percentage
    static private int DEFAULT_THIKNESS       = 2;   // thickness of Rule

    public Rule () {
        this (DEFAULT_THIKNESS, DEFAULT_LENGTH, EMBOSSEDIN);
    }

    public Rule (int thickness, int ruleLength, boolean embossedIn) {
	this.thickness = thickness;
	this.embossedIn = embossedIn;
        setRuleLength(ruleLength);
	setSize (thickness, thickness);
    }


    public void setRuleLength (int ruleLength) {
        if (ruleLength < 0 || ruleLength >100)
           this.length = 1.0;
        else
           this.length = (double)ruleLength/100.0;
    }

    public void paint ( Graphics g) {
	Color lighter = SystemColor.controlLtHighlight; 
//g.getColor().darker().darker();
	Color darker = SystemColor.controlShadow; 
//g.getColor().brighter().brighter();

	Dimension size = getSize();
        width = size.width;
        height = size.height;

	if (embossedIn) {
	    if (width > height) 
		horizontalRule (g, width, height, darker, lighter);
	    else
		verticalRule (g, width, height, darker, lighter);
	}
	else {
	    if (width > height) 
		horizontalRule (g, width, height, lighter, darker);
	    else
		verticalRule (g, width, height, lighter, darker);
	}
    }

    private void horizontalRule (Graphics g, int width, int height, 
                 Color topColor, Color botColor) {

        int y = (height - thickness) / 2;
        int adjustedWidth = (int)((double)width*length);
        int x1 = (width - adjustedWidth)/2;
        int x2 = x1 + adjustedWidth;

	g.setColor (topColor);
	while (y < height/2){
	    g.drawLine (x1, y, x2, y);
            y++;
        }

	g.setColor (botColor);
        y = height/2;

	while (y < (height + thickness) / 2) {
	    g.drawLine (x1, y, x2, y);
            y++;
        }
    }

    private void verticalRule (Graphics g, int width, int height, 
                 Color topColor, Color botColor) {

        int x = (width - thickness) / 2;
        int adjustedHeight = (int)((double)height*length);
        int y1 = (height - adjustedHeight)/2;
        int y2 = y1 + adjustedHeight;

	g.setColor (topColor);

	while (x < width/2) {
	    g.drawLine (x, y1, x, y2);
            x++;
        }

	g.setColor (botColor);
        x = width/2;

	while (x < ((width + thickness) / 2)) {
	    g.drawLine (x, y1, x, y2);
            x++;
        }
    }

    public Dimension getMinimumSize () {
	return new Dimension (thickness, thickness);
    }

    public Dimension getPreferredSize () {
	return new Dimension (thickness, thickness);
    }

    private boolean embossedIn;
    private int thickness;
    private double length;
    private int width;
    private int height;
}
