#!/usr/bin/perl -w
###############################################################################
#
# Freenet Key Directory v1.0.0
#
# Copyright (C) 1999, 2000 Steven Hazel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# see http://freenet.sourceforge.net/ for more info on Freenet
#
###############################################################################

use strict;
use keyindex;
use URI::Escape;
use CGI qw(:standard);
use LWP::Simple;


# Set this to point to whatever Freenet web proxy you want your users
# to request from by default
my $DEFAULT_URLPREFIX = "http://localhost:8081/";

# "INDEXFILE" is a relative path to the INDEXFILE set in keyindex.pl
# If you're not running a local keyindex, set this value to '', and
# a default keyindex URL will be used instead.
my $INDEXFILE = 'keyindex';

# This is the URL where you keep the little folder and file images
# We default to getting them out of Freenet, heh.
my $IMAGEURL = "http://localhost:8081/";

# That's it, you shouldn't need to configure anything below this line.

my $change = param('change') || "";
if ($change eq "true") {
print "Content-type: text/html\n\n";
print << "EOF";
<form 
action=./ 
method=get>
<table>
<tr>
<td>
You can browse any remote keyindex by giving its URL here:
</td>
<td>
<input type=textarea name=keyindex>
</td>
</tr><tr>
<td>
Keys listed in this directory will link to the URL prefix given here:
</td>
<td>
<input type=textarea name=urlprefix>
</td>
</tr><tr>
<td>
<input type=submit value=\"Change Settings\">
</td>
</tr>
</table>
</form>
EOF
exit;
}


my $keyindex = param('keyindex') || "";
my $search = param('search') || "";
my $urlprefix = param('urlprefix') || $DEFAULT_URLPREFIX;

my @files = ();
my @dirs = ();
my @lines = ();
my @results = ();
my $junk;
my $line = "";
my $prevline = "";
my $urikey = "";
my $uriline = "";

my $term = "";
my @possibles = ();
my $fail = 0;
my $psearch = "";

my $key = "";
my $pkey = "";

if (defined($ENV{'PATH_INFO'})) {
  $key = $ENV{'PATH_INFO'};
  $key =~ tr/+/ /;
  $key =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $key =~ s/^\/*//;
}
  
$pkey = $key;
$pkey =~ s/\&/\&amp\;/g;
$pkey =~ s/\</\&lt\;/g;
$pkey =~ s/\>/\&gt\;/g;

$psearch = $search;
$psearch =~ s/\&/\&amp\;/g;
$psearch =~ s/\</\&lt\;/g;
$psearch =~ s/\>/\&gt\;/g;

print "Content-type: text/html\n\n";
print "<HEAD>\n";
print "<TITLE>/$pkey directory</TITLE>\n";
print "</HEAD>\n";
print "<H2>Current directory is /$pkey</H2>\n";

if ($psearch ne "") {
  print "Currently browsing keys matching \"$psearch\".<br>\n";
} else {
  print "Currently browsing all keys.<br>\n";
}
print "<form action=./ method=get><input type=textarea name=search>\n";
if ($keyindex ne "") {
  print "<input type=hidden name=keyindex value=\"$keyindex\">";
}
if ($urlprefix ne $DEFAULT_URLPREFIX) {
  print "<input type=hidden name=urlprefix value=\"$urlprefix\">";
}
print "<input type=submit value=\"Search\"></form>\n";

print "<HR>\n";

print "Keyindex URL: $keyindex<br>\n";
print "Link URL Prefix: $urlprefix<br>\n";
print "<form action=./ method=get>\n";
print "<input type=hidden name=change value=true>\n";
print "<input type=submit value=\"Change Settings\">\n";
print "</form>\n";

print "<HR>\n";


if (($INDEXFILE eq "") or ($keyindex ne "")) {
  if ($keyindex eq "") {
    $keyindex = "http://azrael.dyn.cheapnet.net/cgi-bin/keyindex/";
  }
  @lines = split(/\n/, get($keyindex));
} else {
  @lines = keyindex::get_keys($INDEXFILE);
}
@lines = sort(@lines);
for $line (@lines) {
  $line =~ s/^\s*//;
  $line =~ s/\s*$//;

  $line =~ s/\&/\&amp\;/g;
  $line =~ s/\</\&lt\;/g;
  $line =~ s/\>/\&gt\;/g;
}

if ($search eq "") {
  for $line (@lines) {
    if ($line =~ m/^\Q$pkey\E/) {
      push(@results, $line);
    }
  }
} else {
  for $line (@lines) {
    if ($line =~ m/^\Q$pkey\E/) {
      for $term (split(/\s/, $search)) { 
	if ((not ($term =~ m/^\+/))
	    and (not ($term =~ m/^\-/))) {
	  if ($line =~ m/$term/i) {
	    push(@possibles, $line);
	    last;
	  }
	}
      }
    }
  }

  for $line (@possibles) {
    $fail = 0;
    for $term (split(/\s/, $search)) { 
      if ($term =~ m/^\+/) {
	$term =~ s/^\+(.*)/$1/;
	if (not ($line =~ m/$term/i)) {
	  $fail = 1;
	  last;
	}
      } elsif ($term =~ m/^\-/) {
	$term =~ s/^\-(.*)/$1/;
	if ($line =~ m/$term/i) {
	  $fail = 1;
	  last;
	}
      }
    }

    if ($fail == 0) {
      push(@results, $line);
    }
    
  }

}

if  ($key ne "") {
  print "<img src=\"$IMAGEURL/top.gif\" alt=\"[DIR]\"> ";
  print "<a href=\"";
  for $junk (split("/", $key)) {
    print "../";
  }
  if ($keyindex ne "") {
    print "?keyindex=$keyindex";
  }
  if ($search ne "") {
    print "?search=$search";
  }
  if ($urlprefix ne $DEFAULT_URLPREFIX) {
    print "?urlprefix=$urlprefix";
  }
  print "\">Top</a><br><br>\n";
  
  print "<img src=\"$IMAGEURL/back.gif\" alt=\"[DIR]\"> ";
  print "<a href=\"../";
  if ($keyindex ne "") {
    print "?keyindex=$keyindex";
  }
  if ($search ne "") {
    print "?search=$search";
  }
  if ($urlprefix ne $DEFAULT_URLPREFIX) {
    print "?urlprefix=$urlprefix";
  }
  print "\">Previous Directory</a><br><br>\n";
}


for $line (@results) {
  $line =~ s/^\Q$pkey\E//;

  if (($line =~ m/\//) 
      and (not ($line =~ m/\/\//))          # keys with double slashes
      and (not ($line =~ m/^\//))           # keys that start with a /
      and (not ($line =~ m/\/$/))
      and (not ($line =~ m/^\.+$/))
      and (not ($line =~ m/^\.+\//))
      and (not ($line =~ m/\/\.+$/))
      and (not ($line =~ m/\/\.+\//))) {
    ($line, $junk) = split(/\//, $line);
    push(@dirs, "$line/");
  } else {
    if ((not ($line =~ m/\/\//)) or ($key eq "")) {
      push(@files, $line);
    }
  }
}

@files = sort(@files);
@dirs = sort(@dirs);


$urikey = uri_escape($pkey);

# print the directories
$prevline = "";
for $line (@dirs) {
  if ($line ne $prevline) {
    print "<img src=\"$IMAGEURL/folder.gif\" alt=\"[DIR]\"> ";
    print "<a href=\"";
    $uriline = uri_escape($line);
    $uriline =~ s/\?/\%3F/g;
    print $uriline;
    if ($keyindex ne "") {
      print "?keyindex=$keyindex";
    }
    if ($search ne "") {
      print "?search=$search";
    }
    if ($urlprefix ne $DEFAULT_URLPREFIX) {
      print "?urlprefix=$urlprefix";
    }
    print "\"><tt>$line</tt></a><br>\n";
    $prevline = $line;
  }
}

$prevline = "";
for $line (@files) {
  if ($line ne $prevline) {
    print "<img src=\"$IMAGEURL/file.gif\" alt=\"[   ]\"> ";
    print "<a href=\"";
    print $urlprefix;
    if (($line =~ m/^\.+$/) or     # keys that are just dots
	($line =~ m/^\.+\//) or    # keys that begin with dots and a slash
	($line =~ m/\/\.+$/) or    # keys that end with a slash and some dots
	($line =~ m/\/\.+\//) or   # keys with dots between slashes
	($line =~ m/^\//)) {       # keys that begin with a slash
      print "?key=";
    }
    if ($key ne "") {
      print "$urikey";
    }
    $uriline = uri_escape($line);
    $uriline =~ s/\?/\%3F/g;       # escape ?s
    print $uriline;
    if ($keyindex ne "") {
      print "?keyindex=$keyindex";
    }
    print "\"><tt>$line</tt></a><br>\n";
    $prevline = $line;
  }
}
