// Jason Rohrer
// ClientFileRequest.java

/**
*
*	Client to client file transfer request
*
*	Created 5-11-2000
*	Mods:
*		Jason Rohrer	8-27-2000	Added a toString() method.
*/

package konspire.client;

import konspire.common.Message;
import konspire.common.FileDescriptor;

/**
 * Message sent from one client to another to request a file.
 *
 * @author Jason Rohrer
 */
public class ClientFileRequest implements Message {

	protected FileDescriptor f;

	protected long bytesToSkip;	// for resumed file downloads
	
	
	/**
	 * Construct ClientFileRequest.
	 *
	 * @param fDes 			file descriptor of file to request
	 * @param skipBytes 	bytes to skip in file when starting transfer
	 *  (for resumed transfers).
	 *
	 */
	public ClientFileRequest( FileDescriptor fDes, long skipBytes ) {
		f = fDes;
		bytesToSkip = skipBytes;
		}


	/**
	 * Get FileDescriptor contained in this request.
	 *
	 * @return descriptor of file contained in this request
	 *
	 */
	public FileDescriptor getFileDescriptor() {
		return f;
		}
	
	
	/**
	 * Get number of bytes to skip in file when starting transfer
	 *
	 * @return number of bytes to skip in file when starting transfer
	 *
	 */
	public long getBytesToSkip() {
		return bytesToSkip;
		}
		
		
		
	// overrides the Object toString() method	
	public String toString() {
		return "ClientFileRequest: " + f + ", skipping " + bytesToSkip + " bytes";
		}	
		
	}
