// Jason Rohrer
// ClientSettingsFile.java

/**
*
*	Object for sending client settings to file.
*
*	Created 7-4-2000
*	Mods:
*		Jason Rohrer	8-27-2000	Added JLog logging support.
*		Jason Rohrer	9-29-2000	Made mClientSettings and writeToFile()
*									public to support saving all settings
*									before exiting (and to avoid writing
*									wrapper methods in this class for
*									every individual setting).
*/


package konspire.client;

import java.io.*;
import konspire.common.log.AppLog;
import konspire.common.ErrorHandler;

/**
 * Object for sending client settings to file.  Contains default settings for client.
 *
 * @author Jason Rohrer
 */
public class ClientSettingsFile {

	
	private String mFileName = "ClientSettings.dat";
	
	public ClientSettings mClientSettings = new ClientSettings();

	
	/**
	 * Construct a new ClientSettingsFile and read settings from file
	 *
	 */
	public ClientSettingsFile() {
		readFromFile();
		}
	
	
	/**
	 * Set files directory.
	 *
	 * @param inFilesDirector the new files directory
	 *
	 */
	public void setFilesDirectory( File inFileDirectory ) {
	
		//mClientSettings.mFilesDirectory = inFileDirectory;
		
		writeToFile();
		}
	
	/**
	 * Get files directory.
	 *
	 * @return the files directory
	 *
	 */
	public File getFilesDirectory() {
	
		//return mClientSettings.mFilesDirectory;
		return new File( "files" );
		}
		
	
	/**
	 * Write settings to file.
	 *
	 */	
	public void writeToFile() {
		try {
			DataOutputStream fileOutput = new DataOutputStream (
							new FileOutputStream( mFileName ) );
			
			ObjectOutputStream objOutStream 
					= new ObjectOutputStream( fileOutput );
			
			// write the settings object to file
			objOutStream.writeObject( mClientSettings );
			
			objOutStream.flush();
			objOutStream.close();
			fileOutput.close();
			}
		catch( IOException e ) {
			AppLog.error( "writing settings to file " + mFileName + " failed",
				true );
			ErrorHandler.report( e );
			}
		}
		
	/**
	 * Read settings from file.
	 *
	 */	
	private void readFromFile() {
		File testExistsFile = new File( mFileName );
		if( ! testExistsFile.exists() ) {
			writeToFile();
			}
		
		try {
			DataInputStream fileInput = new DataInputStream (
							new FileInputStream( mFileName ) );
			
			ObjectInputStream objInStream 
					= new ObjectInputStream( fileInput );
			
			try {
				Object inObject = objInStream.readObject();

				if( inObject instanceof ClientSettings ) {
					mClientSettings = (ClientSettings)inObject;
					}
				objInStream.close();			
				fileInput.close();		
				}
			catch( ClassNotFoundException ex ) {
				AppLog.error( "Unknown object read from settings file: " + mFileName );
				ErrorHandler.report( ex );
				fileInput.close();
				}	
			}
		catch( IOException e ) {
			// don't "print also" for read exceptions,
			// since these will be common when starting up the first time
			AppLog.error( "Reading settings from file " + mFileName + " failed." );
			ErrorHandler.report( e );
			}
		}		 
	}


