// Jason Rohrer
// FileDirectoryDialog.java


/**
*
*	Dialog box for selecting file directories
*
*	Created 9-29-2000
*	Mods:
*		Jason Rohrer	10-3-2000	Changed so that it uses non-deprecated
*									setVisible( false ) instead of hide().
*/


package konspire.client;

import konspire.common.Vector;
import konspire.common.FileDirectoryPath;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.io.File;

/**
 * Dialog box for selecting file sharing directories.
 * This is a blocking dialog box.
 *
 * @author Jason Rohrer
 */
public class FileDirectoryDialog extends JDialog {

	JButton mOkButton = new JButton( "ok" );
	
	Frame mParent;;
	
	/**
 	 * Constructs a <code>FileDirectoryDialog</code>.
	 *
	 * Dialog shows itself, modifies inPath according to user
	 * actions, and then closes itself, unblocking inParent.
	 *
	 * @param inPath the directory path to edit.
	 * @param inParent the parent <code>Frame</code> to block.
	 */
	public FileDirectoryDialog( FileDirectoryPath inPath, Frame inParent ) {
		super( inParent, "select files directories", true );
		
		mParent = inParent;
		
		setSize( 400, 300 );
		WindowUtils.centerWindow( this );
		
		getContentPane().setLayout( 
			new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ) );
		
		
		getContentPane().add( new DownloadDirectoryPanel( inPath ) );
		
		getContentPane().add( new UploadDirectoriesTable( inPath ) );
		
		JPanel okButtonPanel = new JPanel();
		okButtonPanel.setLayout( new FlowLayout( FlowLayout.RIGHT ) );
		
		okButtonPanel.add( mOkButton );
		
		getContentPane().add( okButtonPanel );
		
		mOkButton.addActionListener( new ButtonActionListener() );

		show();
		}
	
	
	/**
	 * Listener for OK button events.
	 */
	private class ButtonActionListener implements ActionListener {
		
		public void actionPerformed( ActionEvent inAE ) {
			if( inAE.getSource() == mOkButton ) {
				setVisible( false );
				}
			}
		}
	
	
	/**
	 * Panel that controls download directory part of file directory path.
	 */
	private class DownloadDirectoryPanel extends EtchedTitledJPanel {
		
		FileDirectoryPath mPath;
		
		JTextField mDirectoryField = new JTextField();
		
		JButton mModifyButton =  new JButton( "modify..." );
		
		/**
		 * Constructs a <code>DownloadDirectoryPanel</code>.
		 *
		 * @param inPath the file directory path to manipulate.
		 */
		public DownloadDirectoryPanel( FileDirectoryPath inPath ) {
			super( "download directory" );
			
			mPath = inPath;
			
			super.setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
			
			mDirectoryField.setText(
				mPath.getDownloadDirectory().getAbsolutePath() );
			mDirectoryField.setEditable( false );
			
			super.add( mDirectoryField );
			
			JPanel buttonPanel = new JPanel();
			buttonPanel.setLayout( new FlowLayout( FlowLayout.RIGHT ) );
			
			buttonPanel.add( mModifyButton );
			
			super.add( buttonPanel );
			
			mModifyButton.addActionListener( new ButtonListener() );
			}
		
		/**
		 * Listener for modify button events.
		 */
		private class ButtonListener implements ActionListener {
			public void actionPerformed( ActionEvent inAE ) {
				if( inAE.getSource() == mModifyButton ) {
					String startingDirectoryName = 
						mPath.getDownloadDirectory().getParent();
					
					File startingDirectory = null;
					if( startingDirectoryName != null ) {
						startingDirectory = new File( startingDirectoryName );
						}
						
					File selectedDir =
						FileGuiUtils.launchDirectorySelector( startingDirectory, 
							mParent );
					// make sure a file was actually selected		
					if( selectedDir != null ) {
						mPath.setDownloadDirectory( selectedDir );
						mDirectoryField.setText(
							mPath.getDownloadDirectory().getAbsolutePath() );
						}
					}
				}
			}
			
			
		} // end DownloadDirectoryPanel class
	/**
	 * Table that displays upload directory paths.
	 */ 
	private class UploadDirectoriesTable extends ListTablePanel {

		String tableTitle;

		Client parent;

		private String mActionButtonText = "remove";
		private String mTableTitle = "upload directories";
	
		private FileDirectoryPath mPath;

		private JButton mAddButton = new JButton( "add..." );
		
		/**
		 * Constructs an <code>UploadDirectoriesTable</code>.
		 *
		 * @param inPath the file directory path to manipulate.
		 */
		public UploadDirectoriesTable( FileDirectoryPath inPath ) {
			super( new FileDirectoryPathRenderer() );

			
			
			setTableTitle( mTableTitle );
			setActionButtonText( mActionButtonText );

			setActionKey( KeyEvent.VK_DELETE );
			
			mPath = inPath;
			
			// add each directory in upload path to table
			Vector uploadDirectories = mPath.getUploadDirectories();
			int numItems = uploadDirectories.size();
			for( int i=0; i<numItems; i++ ) {
				// abort each transfer thread
				File chosenPath =
			 		(File)( uploadDirectories.elementAt(i) );
				addTableItem( chosenPath );
				}
				
			addAuxillaryButton( mAddButton );
			mAddButton.addActionListener( new ButtonListener() );	
			}	

		/**
		 * Listener for add button events.
		 */
		private class ButtonListener implements ActionListener {
			public void actionPerformed( ActionEvent inAE ) {
				if( inAE.getSource() == mAddButton ) {
					File selectedDir =
						FileGuiUtils.launchDirectorySelector( (File)null, 
							mParent );
					// make sure a file was actually selected		
					if( selectedDir != null ) {
						mPath.addUploadDirectory( selectedDir );
						addTableItem( selectedDir );
						}
					}
				}
			}


		// Overrides ListTablePanel's action( Vector ) method
		public void action( java.util.Vector inSelectedItems ) {
			int numItems = inSelectedItems.size();
			for( int i=0; i<numItems; i++ ) {
				// abort each transfer thread
				File chosenDirectory =
			 		(File)( inSelectedItems.elementAt(i) );
				mPath.removeUploadDirectory( chosenDirectory );
				}
			}
		} // end of class UploadDirectoriesTable

	} // end of class FileDirectoryDialog
	
	
	
