// Jason Rohrer
// FileDirectoryPathRenderer.java

/**
*
*	GUI List cell renderer for local file directory path
*   ( renders a File in a JList)
*
*	Created 9-29-2000
*	Mods:
*/
package konspire.client;

import konspire.common.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import java.awt.Color;

import java.awt.Component;
import java.awt.FlowLayout;

import java.io.File;

/**
 * Class that renders a <code>List</code> cell representation of a File
 * directory path.
 *
 * @author Jason Rohrer
 */
class FileDirectoryPathRenderer implements ListCellRenderer {
	
	// implements the ListCellRenderer getListCellRendererComponent method
	public Component getListCellRendererComponent( JList inList, Object inValue,
		int inIndex, boolean inIsSelected, boolean inCellHasFocus ) {
	 	
		JPanel listItemPanel = new JPanel();
			
		if ( inIsSelected ) {
			listItemPanel.setBackground( inList.getSelectionBackground() );
			listItemPanel.setForeground( inList.getSelectionForeground() );
			}
		else {
			listItemPanel.setBackground( inList.getBackground() );
			listItemPanel.setForeground( inList.getForeground() );
	 		}
	 	
		listItemPanel.setEnabled( inList.isEnabled() );
		listItemPanel.setFont( inList.getFont() );
	 	
	 	
	 	if( inValue instanceof File ) {
	 		
	 		File file = (File)inValue;
	 		
			String pathString = file.getAbsolutePath();
			
	 		listItemPanel.setLayout( new BoxLayout( listItemPanel, BoxLayout.X_AXIS ) );
	 		
			JPanel pathPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
			
			pathPanel.setBackground( listItemPanel.getBackground() );
			pathPanel.setForeground( listItemPanel.getForeground() );
			
			JLabel pathLabel = new JLabel( pathString );
	 		
			pathLabel.setForeground( listItemPanel.getForeground() );
			pathLabel.setFont( listItemPanel.getFont() );

	 		pathPanel.add( pathLabel );
			
			listItemPanel.add( pathPanel );
			
			listItemPanel.setBorder( new LineBorder( Color.lightGray, 1 ) );
	 		} 
	 	
		return listItemPanel;
		}
	}
