// Jason Rohrer
// FileGuiUtils.java


/**
*
*	Utitilies for file manipulation and selection GUIs.
*
*	Created 9-29-2000
*	Mods:
*/

package konspire.client;

import java.awt.*;
import javax.swing.*;
import java.io.File;

public class FileGuiUtils {

	/**
     * Launches a dialog to select a directory.  Returns only after directory
     * is selected, or user cancels operation.
     *
     * @param inDirectory the directory to display by default when dialog opens
	 * @param inParent <code>Frame</code> to block.
	 *
     * @return file selected, or null if user cancels operation
	 */
	public static  File launchDirectorySelector( File inDirectory, 
		Frame inParent ) {
		//System.out.println( "Launching directory dialog." );
		//FileDirectoryDialog dialog = new FileDirectoryDialog( new
		//	FileDirectoryPath(), inParent );
		//System.out.println( "Returning from launch." );
		
		JFileChooser chooser;
		if( inDirectory != null ) {
			chooser = new JFileChooser( inDirectory );
			}
		else {
			chooser = new JFileChooser();	// else use default directory
			}
		chooser.setFileSelectionMode( chooser.DIRECTORIES_ONLY );
		
		int returnVal = chooser.showOpenDialog( inParent );
		if( returnVal == JFileChooser.APPROVE_OPTION ) {
			return chooser.getSelectedFile();
			}
		else return null;
		
		}

    /**
     * Launches a dialog to select a location to save a file.  Returns only after file
     * is selected, or user cancels operation.
     *
     * @param inDirectory the directory to display by default when dialog opens
     * @param inParent <code>Frame</code> to block.
	 *
     * @return file selected, or null if user cancels operation
	 */
	public static File launchSaveAs( File inDirectory, Frame inParent ) {
		JFileChooser chooser;
		if( inDirectory != null ) {
			chooser = new JFileChooser( inDirectory );
			}
		else {
			chooser = new JFileChooser();	// else use default directory
			}
		chooser.setFileSelectionMode( chooser.FILES_ONLY );
	
		int returnVal = chooser.showSaveDialog( inParent );
		if( returnVal == JFileChooser.APPROVE_OPTION ) {
	   		return chooser.getSelectedFile();
	   		}
	   	else return null;
		}
	}
