// Jason Rohrer
// FileTransferInit.java

/**
*
*	Client to client message for initializing (or denying) a file transfer
*
*	Created 5-11-2000
*	Mods:
*/

package konspire.client;

import konspire.common.Message;
import konspire.common.FileDescriptor;

/**
 * Client to client message for initializing (or denying) a file transfer
 *
 * @author Jason Rohrer
 */
public class FileTransferInit implements Message {

	boolean accepted;
	int blockSize;
	String status;
	
	/**
	 * Constructs a <code FileTransferInit</code>.
	 *
	 * @param acc true iff request accepted
	 * @param stat status message justifying accept or reject
	 * @param blockSz size of file blocks to send across network
	 */
	public FileTransferInit( boolean acc, String stat, int blockSz ) {
		accepted = acc;
		blockSize = blockSz;
		status = stat;
		}
	
	/**
	 * Gets whether transfer request has been accepted.
	 *
	 * @return true iff tranfer request accepted
	 */
	public boolean getAccepted() {
		return accepted;
		}
	
	/**
	 * Gets block size for network file transfer.
	 *
	 * @return size of blocks for network file transfer
	 */
	public int getBlockSize() {
		return blockSize;
		}
	
	/**
	 * Gets a status message about accept or reject.
	 *
	 * @return a message justifying the accpet or reject
	 */
	public String getStatus() {
		return status;
		}
		
	}