// Jason Rohrer
// GuiMessage.java

/**
*
*	Class with static methods for displaying messages to user
*
*	Created 6-2-2000
*	Mods:
*		Jason Rohrer	8-23-2000	Changed to implement UserMessageHandler
*									interface.
*		Jason Rohrer	8-27-2000	Changed to be swing thread-safe.
*/

package konspire.client;

import javax.swing.JOptionPane;
import java.awt.Component;

import konspire.common.UserMessageHandler;

import konspire.common.log.AppLog;
import konspire.common.ErrorHandler;
import javax.swing.SwingUtilities;
import java.lang.reflect.InvocationTargetException;

/**
 * Collection of static methods for displaying messages to user.
 *
 * @author Jason Rohrer
 */
public class GuiMessage implements UserMessageHandler {
	
	static Component parent;
	
	/**
	 * Sets parent component for messages.  This component is blocked 
	 * until message has been acknowledged by user.
	 *
	 * @param par parent component to be blocked by message
	 */
	public static void setParent( Component par ) {
		parent = par;
		}
		
	/**
	 * Displays a GUI dialog representation of a <code>String</code> message.
	 *
	 * @param s the message to display
	 */
	public static void show( String s ) {
		// all locals used in anonymous inner class must be final
		final String finalMessage = s;
		
		// anonymous class that wraps calls to JOptionPane methods
		Runnable showMessageWrapper = new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog( parent, finalMessage, "note:", JOptionPane.WARNING_MESSAGE );
				}
			};
		// only need to use invokeAndWait if we're not the 
		// EventDispatch thread
		if( !SwingUtilities.isEventDispatchThread() ) {
			//try {
				//SwingUtilities.invokeAndWait( showMessageWrapper );
				SwingUtilities.invokeLater( showMessageWrapper );
			/*	}
			catch( InterruptedException inIE ) {
				AppLog.error( "Interrupted while waiting for event dispatch"
					+ " thread to display a JOptionPane." );
				ErrorHandler.report( inIE );
				}
			catch( InvocationTargetException inITE ) {
				ErrorHandler.report( inITE );
				}
			*/
			}
		else {
			showMessageWrapper.run();
			}
		}
		
	
	
	// implements UserMessageHandler showMessage( String ) method
	public void showMessage( String inMessage ) {
		show( inMessage );
		}
		
	}
