// Jason Rohrer
// ListChoiceEvent.java

/**
*
*	Event generated by a ClickableJList when a group of its 
*   items are chosen (for example, double clicked ).
* 
*	Created 8-11-2000
*	Mods:
*		Jason Rohrer	8-11-2000 	Changed to work with a group
*									of selected items.
*		Jason Rohrer 	8-16-2000	Changed add() to addElement() to support 
*								java 1.1
*/

package konspire.client;

import java.util.Vector;

/**
 * Event generated by a <code>ClickableJList</code>
 * when a group of its items are chosen (for example, double clicked ).
 * <p>
 * Note that this class only uses the even pattern and doesn't
 * actually subclasse <code>java.awt.event</code>.
 *
 * @author Jason Rohrer
 */
public class ListChoiceEvent {

	private Vector mChosenItems;



	/**
	 * Constructs a <code>ListChoiceEvent</code> with a vector
	 * of items.
	 *
	 * @param inChosenItems a <code>Vector</code> of the 
	 *  list items chosen when this event was generated.
	 */
	public ListChoiceEvent( Vector inChosenItems ) {
		mChosenItems = inChosenItems;	
		}
	
	
	/**
	 * Constructs a <code>ListChoiceEvent</code> with an array
	 * of items.
	 *
	 * @param inChosenItems an array of the list items 
	 *  chosen when this event was generated.
	 */
	public ListChoiceEvent( Object inChosenItems[] ) {
		int numItems = inChosenItems.length;
		mChosenItems = new Vector( numItems );
		for( int i=0; i<numItems; i++ ) {
			mChosenItems.addElement( inChosenItems[i] );
			}
		}
	
	
	/**
	 * Gets the list items chosen when this event was generated.
	 *
	 * @return a <code>Vector</code> of list items chosen 
	 *  when this event was generated.
	 */
	public Vector getChosenItems() {
		return mChosenItems;
		}
	
	
	}
