// Jason Rohrer
// SearchResultsTable.java

/**
*
*	Table of search results (FileDescriptorGUI's), with buttons to start downloads
*
*	Created 5-11-2000
*	Mods:
*		Jason Rohrer	6-1-2000	Changed to filter results so that files on this
*									host don't appear
*		Jason Rohrer	6-14-2000	Removed filtering against files on this host.
*									This functionality has been moved server-side
*									to ensure that search results are comprehensive.
*		Jason Rohrer	7-31-2000	Complete overhaul.
*									Switched from using a JTable to using a JList.
*		Jason Rohrer	8-3-2000	Changed layout of panel to GridBagLayout to
*									make sure table is as large as possible.
*		Jason Rohrer 	8-11-2000	Added support for double-clickable results list. 
*		Jason Rohrer	8-13-2000	Changed to subclass ListTablePanel to
*									provide all scrolling list functionality.
*		Jason Rohrer	8-16-2000	Changed get() to elementAt() to support 
*									java 1.1
*/

package konspire.client;

import konspire.common.SearchResults;
import konspire.common.FileDescriptor;
import konspire.common.Host;

import konspire.common.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import javax.swing.*;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dimension;

/**
 * Table of search reulsts.  Contains a <code>ClickableJList</code> that
 * displays each <code>FileDescriptor</code> in the result set.
 *
 * @author Jason Rohrer
 */
public class SearchResultsTable extends ListTablePanel {

	private String tableTitle = "search results";
	
	private DownloadController parent;
	
	private String mActionButtonText = "download";
	

	
	
	/**
	 * Construct a <code>SearchResultsTable</code>
	 *
	 * @param par parent download controler responsible for downloading files
	 * selected by user.
	 */
	public SearchResultsTable( DownloadController par ) {
		super( new FileDescriptorRenderer() );
		
		setTableTitle( tableTitle );
		setActionButtonText( mActionButtonText );
		
		parent = par;
		}
		
	/**
	 * Displays search results.
	 *
	 * @param sR results to display
	 */
	public void displayResults( SearchResults sR ) {
		
		Vector fileListData = sR.getFileVector();
		String searchTerms[] = sR.getSearchTerms();
		
		if( searchTerms.length > 0 ) {
			String searchString = searchTerms[0];
			
			for( int i=1; i<searchTerms.length; i++ ) {
				searchString += ", " + searchTerms[i];
				}
			setSubtitle( searchString );	// display search terms in title of table
			}
		
		setTableItems( fileListData );
		
		}	// end displayResults()
		
		
		
	// Overrides ListTablePanel's action( Vector ) method
	public void action( java.util.Vector inSelectedItems ) {
		int numItems = inSelectedItems.size();
		for( int i=0; i<numItems; i++ ) {
			// pass on to client
			parent.startDownload( (FileDescriptor)( inSelectedItems.elementAt(i) ) );
			}
		}
	
	
	
	/**
	 * Set title displayed after "search results: ".  Usually used to display
	 * string that was matched to produce the current results.
	 *
	 * @param subtitle title to display after main panel title
	 */	
	private void setSubtitle( String subtitle ) {
		if( subtitle.length() == 0 ) {
			// remove subtitle
			setTableTitle( tableTitle );
			}
		else {
			// add/change subtitle
			setTableTitle( tableTitle + ": " + subtitle );
			}
		repaint();
		}	
		
		
	}
