// Jason Rohrer
// StatePanel.java

/**
*
*	Bordered panel for system state information 
*
*	Created 5-11-2000
*	Mods:
*		Jason Rohrer	6-1-2000	Added number of clients to stats displayed.
*									Replaced pairs of separate labels each with 
*									a LabeledFieldJLabel.
*		Jason Rohrer	7-3-2000	Changed to use UnitsPrefixFormat to format total file size.
*/

package konspire.client;

import konspire.common.Host;
import konspire.common.ToClientMessage;

import konspire.common.UnitsPrefixFormat;
import konspire.common.BaseTwoUnitsPrefixFormat;

import javax.swing.*;

import java.awt.FlowLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Panel that displays system state information.
 *
 * @author Jason Rohrer
 */
public class StatePanel extends EtchedTitledJPanel {
	
	String baseUnitsString = "B";
	
	JLabel connectionLabel;
	JLabel connection;
	
	JLabel onlineLabel;
	
	LabeledFieldJLabel clientsLabel;
	LabeledFieldJLabel filesLabel;
	JLabel bytesLabel;
	
	int addressWidth = 15;
	int portWidth = 5;
	
	JTextField seedServerAddress;
	JTextField seedServerPort;
	
	JLabel addressLabel;
	JLabel portLabel;
	
	JButton addSeedButton;

	JPanel connectionPanel;
	JPanel onlinePanel;
	
	JPanel seedServerPanel;
	JPanel seedServerButtonPanel;
	
	ButtonActionListener buttonListen;
	
	String title = "server state";
	
	Client parent;
	
	/**
	 * Constructs a <code>StatePanel</code>.
	 *
	 * @param par parent <code>Client</code> for this panel
	 */
	public StatePanel( Client par ) {
		
		setTitle( title );
		
		parent = par;
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		connectionPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		
		onlinePanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		
		seedServerPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		
		seedServerButtonPanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		
		connectionLabel = new JLabel( "live server connection: " );		
		connection = new JLabel( "none" );
		
		onlineLabel = new JLabel( "global online: " );
		
		clientsLabel = new LabeledFieldJLabel( " clients, ", "0", false );
		filesLabel = new LabeledFieldJLabel( " files, ", "0", false );
		bytesLabel = new JLabel( "0 " + baseUnitsString );

		seedServerAddress = new JTextField( addressWidth );
		seedServerPort = new JTextField( portWidth );
		
		addressLabel = new JLabel( "address:" );
		portLabel = new JLabel( "port:" );
		
		addSeedButton = new JButton( "seed server" );
			
		connectionPanel.add( connectionLabel );
		connectionPanel.add( connection );
		
		onlinePanel.add( onlineLabel );
		
		onlinePanel.add( clientsLabel );
		
		onlinePanel.add( filesLabel );
		
		onlinePanel.add( bytesLabel );
		
		seedServerPanel.add( addressLabel );
		seedServerPanel.add( seedServerAddress );
		
		seedServerButtonPanel.add( portLabel );
		seedServerButtonPanel.add( seedServerPort );
		
		seedServerButtonPanel.add( addSeedButton );
		
		add( connectionPanel );
		add( onlinePanel );
		add( seedServerPanel );
		add( seedServerButtonPanel );
		
		
		buttonListen = new ButtonActionListener();
		addSeedButton.addActionListener( buttonListen );
		}
	
	/**
	 * Sets host listed as current server connection.
	 *
	 * @param s server to list as current server connection
	 */	
	public void setLiveServer( Host s ) {
		String serverName = "none";
		if( s != null ) {
			serverName = s.toString();
			seedAreaSetEnabled( false );
			}
		else {
			// enable add seed button if not connected
			seedAreaSetEnabled( true );
			}
			
		connection.setText( serverName );
		}

	/**
	 * Updates system statistics with information from a new message
	 * received by client from server.
	 *
	 * @param m message to client containing stats
	 */
	public void setOnlineStats( ToClientMessage m ) {
		clientsLabel.setValue( "" + m.getNumHosts() );
		filesLabel.setValue( "" + m.getNumFiles() );
		
		UnitsPrefixFormat prefixFormat = new BaseTwoUnitsPrefixFormat();
		String bytesString = prefixFormat.convertUnitsPrefix( + m.getNumBytes(), 
									baseUnitsString );
		
		
		bytesLabel.setText( bytesString );
		}
	
	
	/**
	 * Listener for button actions.
	 */
	private class ButtonActionListener implements ActionListener {
		public void actionPerformed( ActionEvent e ) {
			if( e.getSource() == addSeedButton ) {
				
				int port;
				
				try{
					port = Integer.valueOf(seedServerPort.getText()).intValue();
					}
				catch( NumberFormatException nfe ) {
					return;			// bad number format
					}
				
				Host s = new Host( seedServerAddress.getText(), port );
				
				seedAreaSetEnabled( false );
				parent.trySeed( s );
				
				}
			}
		}
	
	/**
	 * Sets whether seed server input area is enabled.
	 *
	 * @param enabled true iff seed area is enabled
	 */
	private void seedAreaSetEnabled( boolean enabled ) {
		addSeedButton.setEnabled( enabled );
		seedServerPort.setEnabled( enabled );
		seedServerAddress.setEnabled( enabled );
		addressLabel.setEnabled( enabled );
		portLabel.setEnabled( enabled );
		}
	
	
	}