// Jason Rohrer
// TransferTable.java

/**
*
*	Table that displays status information (and user controls) for 
*	currently active file transfer threads
*
*	Created 5-11-2000
*	Mods:
*		Jason Rohrer	6-14-2000	Altered table row height to improve readability
*									of progress bars.
*		Jason Rohrer	8-13-2000	Changed to subclass ListTablePanel (instead
*									of ScrollTablePanel).  This results in
*									almost a complete GUI overhaul.
*		Jason Rohrer	8-16-2000	Changed get() to elementAt() to support 
*									java 1.1
*/

package konspire.client;


import javax.swing.JButton;
import javax.swing.JProgressBar;
import java.awt.event.KeyEvent;

/**
 * Table that displays status information (and user controls) for 
 * currently active file transfer threads
 *
 * @author Jason Rohrer
 */
public abstract class TransferTable extends ListTablePanel {

	String tableTitle;
	
	Client parent;
	
	private String mActionButtonText = "cancel/clear";
	
	/**
	 * Constructs a <code>TransferTable</code>.  Protected
	 * becase <code>TransferTable</code> is abstract.
	 *
	 * @param title title to be displayed on table panel
	 */
	protected TransferTable( String title ) {
		super( new TransferThreadRenderer() );
		
		tableTitle = title;
		
		setTableTitle( tableTitle );
		setActionButtonText( mActionButtonText );
		
		setActionKey( KeyEvent.VK_DELETE );
		}


	/**
	 * Adds an UNSTARTED thread to the table.<BR>
	 * Thread will be started by this function call.<BR>
	 * Thread MUST remove itself when done.<BR>
	 *
	 * @param transThread thread to add to table
	 *
	 * @return true iff thread was added successfully
	 */
	protected boolean addThread( TransferThread transThread ) {
		addTableItem( transThread );
		transThread.setTable( this );
		transThread.start();
		return true;
		}	
	
	
	
	// Overrides ListTablePanel's action( Vector ) method
	public void action( java.util.Vector inSelectedItems ) {
		int numItems = inSelectedItems.size();
		for( int i=0; i<numItems; i++ ) {
			// abort each transfer thread
			TransferThread chosenThread =
			 	(TransferThread)(inSelectedItems.elementAt(i) );
			
			chosenThread.abortTransfer();
			}
		}
		
	}
