// Jason Rohrer
// TransferThreadRenderer.java

/**
*
*	GUI List cell renderer for a TransferThread ( renders TransferThreadss in a JList)
*
*	Created 8-13-2000
*	Mods:
*/
package konspire.client;

import konspire.common.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import java.awt.Color;

import java.awt.Component;
import java.awt.FlowLayout;


/**
 * Class that renders a <code>List</code> cell representation of a
 * TransferThread.
 *
 * @author Jason Rohrer
 */
class TransferThreadRenderer implements ListCellRenderer {
	
	// implements the ListCellRenderer getListCellRendererComponent method
	public Component getListCellRendererComponent( JList inList, Object inValue,
		int inIndex, boolean inIsSelected, boolean inCellHasFocus ) {
	 	
		JPanel listItemPanel = new JPanel();
			
		if ( inIsSelected ) {
			listItemPanel.setBackground( inList.getSelectionBackground() );
			listItemPanel.setForeground( inList.getSelectionForeground() );
			}
		else {
			listItemPanel.setBackground( inList.getBackground() );
			listItemPanel.setForeground( inList.getForeground() );
	 		}
	 	
		listItemPanel.setEnabled( inList.isEnabled() );
		listItemPanel.setFont( inList.getFont() );
	 	
	 	
	 	if( inValue instanceof TransferThread ) {
	 		
	 		TransferThread transferThread = (TransferThread)inValue;
	 		FileDescriptor fileDescriptor = transferThread.getFileDescriptor();
	 		
	 		UnitsPrefixFormat prefixFormat = new BaseTwoUnitsPrefixFormat();
	 		
	 		
	 		listItemPanel.setLayout( new BoxLayout( listItemPanel, BoxLayout.X_AXIS ) );
	 		
			JPanel namePanel = new JPanel( new FlowLayout( FlowLayout.RIGHT ) );
			JPanel sizePanel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
			
			namePanel.setBackground( listItemPanel.getBackground() );
			namePanel.setForeground( listItemPanel.getForeground() );
			sizePanel.setBackground( listItemPanel.getBackground() );
			sizePanel.setForeground( listItemPanel.getForeground() );
			
			
			JLabel nameLabel = new JLabel( fileDescriptor.getName() );
	 		
	 		String sizeString = prefixFormat.convertUnitsPrefix( fileDescriptor.getSize(), 
	 																"B" );
	 		JLabel sizeLabel = new JLabel( sizeString );
	 		
			nameLabel.setForeground( listItemPanel.getForeground() );
			sizeLabel.setForeground( listItemPanel.getForeground() );
			
			nameLabel.setFont( listItemPanel.getFont() );
			sizeLabel.setFont( listItemPanel.getFont() );
			
			
	 		namePanel.add( nameLabel );
			
	 		// add both prog bar and size to one panel
			sizePanel.add( transferThread.getProgressBar() );
			sizePanel.add( sizeLabel );
			
			//listItemPanel.add( progressPanel );
			listItemPanel.add( sizePanel );
			listItemPanel.add( namePanel );
			listItemPanel.setBorder( new LineBorder( Color.lightGray, 1 ) );
			
	 		} 
	 	
		return listItemPanel;
		}
	}
