// Jason Rohrer
// ClientRequestConnect.java

/**
*
*	Message from client requesting one of server's dedicated sockets
*
*	Created 5-4-2000
*	Mods:
*		Jason Rohrer	7-4-2000	Changed so that Request contains
*									address of client requesting connection.
*									Thus, server can determine whether client
*									knows its own correct local address.
*/

package konspire.common;


/**
 * Message from a <code>Client</code> to a <code>Server</code> requesting one of the
 * <code>Server</code>'s dedicated sockets.
 * <p>
 * If the <code>Client</code> connection is accepted, the <code>Client</code> can then
 * use the dedicated socket to send service requests and receive responses.
 *
 * @author Jason Rohrer
 */
public class ClientRequestConnect implements Message {

	// nothing in this class
	// just a message to let server know that client wants to connect
	
	// added client address to this request so that server
	//   can verify that client determined its own address correctly
	
	
	private Host clientAddress;
	
	/**
	 * Constructs a <code>ClientRequestConnect</code>.
	 *
	 * @param inClientAddress address of the <code>Client</code>
	 *  as determined by the <code>Client</code>.  May be incorrect if 
	 *  <code>Client</code> is behind some sort of firewall.
	 */
	public ClientRequestConnect( Host inClientAddress ) {
		clientAddress = inClientAddress;
		}
	
	/**
	 * Gets the address of the <code>Client</code>
	 *  as determined by the <code>Client</code>.  May be incorrect if 
	 *  <code>Client</code> is behind some sort of firewall.
	 *
	 * @return the suspected address of the <code>Client</code>
	 */
	public Host getClientAddress() {
		return clientAddress;
		}
	
	}