// Jason Rohrer
// ErrorHandlingThread.java

/**
*
*	Thread class that catches errors thrown by the run method and passes
*	them to the error handling framework.
*
*	Created 8-23-2000
*	Mods:
*/

package konspire.common;


/**
 * Thread class that catches <code>Throwable</code>s thrown by the run
 * methods of all subclasses.  <code>Throwables</code> are reported to the
 * error handling framework.
 * <p>
 * Note that all constructors simply wrap <code>Thread</code> constructors,
 * so no explicit Javadoc is included for them.
 *
 * @author Jason Rohrer
 */
public abstract class ErrorHandlingThread extends Thread {
	
	
	
	/**
	 * @see Thread#Thread( Runnable )
	 */
	protected ErrorHandlingThread( Runnable inRunnable ) {
		super( inRunnable );
		}
	
	
	
	/**
	 * @see Thread#Thread( String )
	 */
	protected ErrorHandlingThread( String inName ) {
		super( inName );
		}
	
	
	
	/**
	 * @see Thread#Thread( Runnable, String )
	 */
	protected ErrorHandlingThread( Runnable inRunnable, String inName ) {
		super( inRunnable, inName );
		}
	
	
	
	/**
	 * @see Thread#Thread( ThreadGroup, Runnable, String )
	 */
	protected ErrorHandlingThread( ThreadGroup inGroup, 
		Runnable inRunnable, String inName ) {
		
		super( inGroup, inRunnable, inName );
		}



	/**
	 * @see Thread#Thread( ThreadGroup, Runnable )
	 */
	protected ErrorHandlingThread( ThreadGroup inGroup, Runnable inRunnable ) {
		super( inGroup, inRunnable );
		}



	/**
	 * @see Thread#Thread( ThreadGroup, String )
	 */
	protected ErrorHandlingThread( ThreadGroup inGroup, String inName ) {
		super( inGroup, inName );
		}
		


	/**
	 * Method overrided from <code>Thread</code> class.  Made final
	 * so that no subclasses can bypass our error handling code.
	 */
	public final void run() {
		try {
			runMethod();
			}
		catch( Throwable inThrowable ) {
			ErrorHandler.report( inThrowable );
			}
		}
	
	
	
	/**
	 * Method that must be implemented by all non-abstract subclasses.
	 * This is where thread run code should go.
	 * <p>
	 * This method is executed by the parent thread inside a try
	 * and catch so that all throwables thrown by the method are
	 * caught and handled by the error handling framework.
	 */ 
	public abstract void runMethod();
	
	
	}
