// Jason Rohrer
// FileDescriptor.java

/**
*
*	Descriptor of a file
*
*	Created 5-4-2000
*	Mods:
*		Jason Rohrer	6-1-2000	Added file path (to support scanned subdirectories).
*		Jason Rohrer	6-1-2000	Added support for setting host after construction
*									to allow for fixing bad local addresses.
*/

package konspire.common;

import java.io.Serializable;


/**
 * Pointer to a file on a network file host.
 *
 * @author Jason Rohrer
 */
public class FileDescriptor implements java.io.Serializable {
	
	
	
	/**
	 * Host that is sharing this file.
	 */
	protected Host fileLocation;
	
	
	
	/**
	 * Name of this file.
	 */
	protected String fileName;
	
	
	
	/**
	 * Host filesystem path to this file.
	 */
	protected String filePath;
	
	
	
	/**
	 * Size of this file in bytes.
	 */
	protected long fileSize;	
	
	
	
	/**
	 * Constructs a <code>FileDescriptor</code>.
	 *
	 * @param h the host sharing this file.
	 * @param name the name of this file.
	 * @param path the host filesytem path to this file.
	 * @param fSize the size of this file in bytes.
	 */
	public FileDescriptor( Host h, String name, String path, long fSize ) {
		fileLocation = h;
		fileName = name;
		filePath = path;
		fileSize = fSize;
		}
	
	
	
	/**
	 * Gets the host that is sharing this file.
	 *
	 * @return the host that is sharing this file.
	 */
	public Host getHost() {
		return fileLocation;
		}
	
	
	
	/**
	 * Sets the host that is sharing this file.
	 *
	 * @param inFileLocation the host that is sharing this file.
	 */
	public void setHost( Host inFileLocation ) {
		fileLocation = inFileLocation;
		}
	
	
	
	/**
	 * Gets the name of this file.
	 *
	 * @return the name of this file.
	 */
	public String getName() {
		return fileName;
		}
	
	
	
	/**
	 * Gets the host filesytem path to this file.
	 *
	 * @return the host filesytem path to this file.
	 */
	public String getPath() {
		return filePath;
		}
	
	
	
	/**
	 * Gets the size of this file in bytes.
	 *
	 * @return the size of this file in bytes.
	 */
	public long getSize() {
		return fileSize;
		}
	
	
	
	// overrides the Object equals( Object ) method	
	public boolean equals( Object o) {
		if( o instanceof FileDescriptor ) {
			FileDescriptor other = (FileDescriptor) o;
			if(  fileLocation.equals( other.getHost() ) && 
					fileName.equals( other.getName() ) &&
					fileSize == other.getSize() ) {
				return true;
				}
			}
		return false;
		}	



	// overrides the Object toString() method	
	public String toString() {
		return fileLocation.toString() + " : " + fileName + " (" + fileSize + "B)";
		}

	}