// Jason Rohrer
// FileDirectoryPath.java


/**
*
*	Class that holds file sharing directory paths.
*
*	Created 9-29-2000
*	Mods:
*/

package konspire.common;

import java.io.File;

import konspire.common.Vector;

import java.io.Serializable;

/**
 * Container for file sharing directory paths.
 * Contains path one download/upload directory and an arbitrary
 * number of upload-only directories.
 *
 * @author Jason Rohrer
 */
public class FileDirectoryPath implements Serializable {

	private File mDownloadDirectory = new File( "files" );
	private Vector mUploadDirectories = new Vector();
	
	
	/**
	 * Sets the download directory.
	 *
	 * @param inFile the new download directory.
	 *
	 * @return true if the directory was set successfully ( will fail if
	 *   inFile is not a directory, for example ).
	 */
	public boolean setDownloadDirectory( File inFile ) {
		if( inFile.isDirectory() ) {
			mDownloadDirectory = inFile;
			return true;
			}
		return false;
		}
	
	/**
	 * Gets the download directory.
	 *
	 * @return the download directory.
	 */
	public File getDownloadDirectory() {
		return mDownloadDirectory;
		}
	
	
	/**
	 * Adds an upload directory.
	 *
	 * @param inFile the directory to add.
	 *
	 * @return true if the directory was added successfully ( will fail if
	 *   inFile is not a directory, for example, or if directory
	 *   already exists in list ).
	 */
	public boolean addUploadDirectory( File inFile ) {
		if( inFile.isDirectory() ) {
			if( !mUploadDirectories.contains( inFile ) ) {
				mUploadDirectories.addElement( inFile );
				return true;
				}
			}
		return false;
		}
		
		
	/**
	 * Removes an upload directory.
	 *
	 * @param inFile the directory to remove.
	 */
	public void removeUploadDirectory( File inFile ) {
		mUploadDirectories.removeElement( inFile );
		}
		
		
	/**
	 * Gets the upload directories.
	 *
	 * @return a <code>Vector</code> of upload directories.
	 */
	public Vector getUploadDirectories() {
		return mUploadDirectories;
		}


	}
