// Jason Rohrer
// Host.java

/**
*
*	Pointer to an internet host
*
*	Created 5-2-2000
*	Mods:
*/

package konspire.common;

import java.io.Serializable;


/**
 * Address and port of an internet host that is accepting connections.
 *
 * @author Jason Rohrer
 */
public class Host implements java.io.Serializable {

	private String address;
	
	private int port;
	
	/**
	 * Constructs a new <code>Host</code>.
	 *
	 * @param addr internet address of the host
	 * @param prt the port that the host is listening on 
	 */
	public Host( String addr, int prt ) {
		address = addr;
		port = prt;
		}
	
	/**
	 * Gets the internet address of this <code>Host</code>.
	 *
	 * @return the internet address of this <code>Host</code>
	 */
	public String getAddress() {
		return address;
		}
	
	/** 
	 * Gets the port that this <code>Host</code> is listening on.
	 *
	 * @return the port that this <code>Host</code> is listening on
	 */
	public int getPort() {
		return port;
		}
	
	/**
	 * Sets the internet address of this <code>Host</code>.
	 *
	 * @param newAddress the internet address of this <code>Host</code>
	 */
	public void setAddress( String newAddress ) {
		address = newAddress;
		}	
	
	/** 
	 * Sets the port that this <code>Host</code> is listening on.
	 *
	 * @param p the port that this <code>Host</code> is listening on
	 */
	public void setPort( int p ) {
		port = p;
		}
		
	
	// overrides the Object equals( Object ) method
		
	public boolean equals( Object o ) {
		if( o instanceof Host ) {
			Host other = (Host) o;
			if( address.equals( other.getAddress() ) && port == other.getPort() ) {
				return true;
				}
			}
		return false;
		}
	
	
	// overrides the Object toString() method
		
	public String toString() {
		return address + ":" + port;
		}	
		
	}