// Jason Rohrer
// MessageSender.java

/**
*
*	Class for sending reliable socket messages on non-maintained sockets
*
*	Created 5-2-2000
*	Mods:
*/


package konspire.common;

import java.io.*;
import java.net.*;

/**
 * Class for sending reliable messages on non-maintained sockets.
 * <p>
 * Each send method call creates a <code>Socket</code>, sends the <code>Message</code>,
 * and then closes the <code>Socket</code>.
 *
 * @author Jason Rohrer
 */
public class MessageSender {
	
	/**
	 * Sends a single <code>Message</code> to a specified <code>Host</code>.
	 *
	 * @param m the <code>Message</code> to send
	 * @param h the <code>Host</code> to send the message to
	 *
	 * @return true iff send completes successfully
	 */
	public boolean sendMessage( Message m, Host h ) {
		try {
			try {
				Socket sock = new Socket( h.getAddress(), h.getPort() );

				DataOutputStream output = new DataOutputStream( sock.getOutputStream() );
				
				DataInputStream input = new DataInputStream( sock.getInputStream() );

								
				ObjectOutputStream objStream = new ObjectOutputStream( output );
				ObjectInputStream objInStream = new ObjectInputStream( input );
				
				
				objStream.writeObject( m );
				
				objStream.flush();
				
				objStream.close();
				
				output.flush();
				
				output.close();
				sock.close();
				
				return true;		// message sent successfully
				}
			catch( UnknownHostException e ) {
				return false;
				}
			}
		catch( IOException e ) {
			return false;
			}
		}
			
	}