// Jason Rohrer
// SearchResults.java

/**
*
*	Message from server responding to a search query
*
*	Created 5-4-2000
*	Mods:
*		Jason Rohrer	5-11-2000	Added search terms to results message.
*/

package konspire.common;


import konspire.common.Vector;

/**
 * Message from <code>Server</code> to <code>Client</code> containing search
 * results in response to a <code>SearchQuery</code> message.
 *
 * @author Jason Rohrer
 */
public class SearchResults extends ToClientMessage {


	// vector of FileDescriptors
	private Vector fileVector;
	
	private String[] searchTerms;
	
	// note that the server database of file names is in all
	// lower case.  Upper case strings won't find matches in the database
	
	
	/**
	 * Constructs a <code>SearchResults</code> message.
	 *
	 * @param fileV a <code>Vector</code> of <code>FileDescriptors</code> for
	 *  files that match the terms in the corresponding <code>SearchQuery</code>
	 * @param q the <code>SearchQuery</code> that this <code>SearchResults</code>
	 *  message corresponds to
	 */
	public SearchResults( Vector fileV, SearchQuery q ) {
		fileVector = fileV;
		searchTerms = q.getSearchTerms();
		}
	
	/**
	 * Gets a <code>Vector</code> of <code>FileDescriptors</code> for files
	 * that match the terms in the corresponding <code>SearchQuery</code>.
	 *
	 * @return a <code>Vector</code> of <code>FileDescriptors</code> for matching files
	 */
	public Vector getFileVector() {
		return fileVector;
		}
	
	
	/**
	 * Gets the search terms that produced these <code>SearchResults</code>.
	 *
	 * @return array of search terms that produced these results
	 */
	public String[] getSearchTerms() {
		return searchTerms;
		}			

	}