// Jason Rohrer
// ServerGoingDown.java

/**
*
*	Message containing information about a dying (or dead) server
*
*	Created 5-4-2000
*	Mods:
*		Jason Rohrer	8-20-2000	Added a toString() method.
*/

package konspire.common;


/**
 * Message containing information about a dying (or dead) <code>Server</code>.
 * <p>
 * This message is sent by a <code>Server</code> that is about to go down,
 * and the receiving <code>Server</code> propagates the message.
 * <p>
 * This message is also propagated by a <code>Server</code> that discovers
 * that another <code>Server</code> is no longer responding (socket connection
 * broken, etc.). 
 *
 * @author Jason Rohrer
 */
public class ServerGoingDown implements Message {

	private Host serverAddress;
	
	
	/**
	 * Constructs a <code>ServerGoingDown</code> message.
	 *
	 * @param serverAddr address of <code>Server</code> that is going down (or is down)
	 */
	public ServerGoingDown( Host serverAddr ) {
		serverAddress = serverAddr;
		}
	
	/**
	 * Gets the address of the dying (or dead) <code>Server</code>
	 *
	 * @return the address of the dying (or dead) <code>Server</code>
	 */
	public Host getServerAddress() {
		return serverAddress;
		}
	
	
	// overrides the Object toString() method
	public String toString() {
		return "Server going down: " + serverAddress;
		}
		
	}
