// Jason Rohrer
// StringPrintWriter.java

/**
*
*	Message propagated by a server when one of its clients disconnects.
*
*	Created 8-20-2000
*	Mods:
*/

package konspire.common;

import java.io.PrintWriter;
import java.io.StringWriter;


/**
 * A <code>PrintWriter</code> that prints to a <code>String</code>.
 * Automatic line flushing is enabled by default.
 * <p>
 * Note that the implementation is very ugly right now in attempt
 * to support Java1.1.  In 1.1, the <code>Writer</code> is not accessible
 * to subclasses because it is private.  Thus, we can't replace the 
 * writer after calling the superclass constructor, nor can we simply call the 
 * superclass constructor and pass in our own member writer (because we
 * can't access member variables before calling the superclass constructor).
 * So, this implementation keeps a member <code>PrintWriter</code> and 
 * has wrapper functions for every <code>PrintWriter</code> function.
 * <p>
 * This putrid implementation will change once we're doing full-blown 1.2,
 * because 1.2 makes the underlying <code>Writer</code> accessible to 
 * subclasses.
 *
 * @author Jason Rohrer
 */
public class StringPrintWriter extends PrintWriter {
	
	
	private StringWriter mStringWriter = new StringWriter();
	private PrintWriter mPrintWriter = new PrintWriter( mStringWriter, true );


	/**
	 * Constructs a <code>StringPrintWriter</code>.
	 */
	public StringPrintWriter() {
		// call the super class constructor, but we won't
		// actually use the superclass for anything... until we
		// move completely to Java 1.2
		super( new StringWriter(), true );
		}
	
	
	
	/**
	 * Gets the buffer's current contents as a <code>String</code>.
	 *
	 * @return the buffer's current contents as a <code>String</code>.
	 */ 
	public String toString() {
		return mStringWriter.toString();
		}

	
	// These methods all override and wrap PrintWriter methods
	
	public boolean checkError() {
		return mPrintWriter.checkError();
		}
	public void close() {
		mPrintWriter.close();
		}
	public void flush() {
		mPrintWriter.flush();
		}
	public void print( boolean inValue ) {
		mPrintWriter.print( inValue );
		}

	public void print( char inValue ) {
		mPrintWriter.print( inValue );
		}
	public void print( char[] inValue ) {
		mPrintWriter.print( inValue );
		}
	public void print( double inValue ) {
		mPrintWriter.print( inValue );
		} 
	public void print( float inValue ) {
		mPrintWriter.print( inValue );
		}
	public void print( int inValue ) {
		mPrintWriter.print( inValue );
		} 
	public void print( long inValue ) {
		mPrintWriter.print( inValue );
		}
	public void print( Object inValue ) {
		mPrintWriter.print( inValue );
		}
	public void print( String inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println() {
		mPrintWriter.println();
		}
	public void println( boolean inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println( char inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println( char[] inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println( double inValue ) {
		mPrintWriter.println( inValue );
		} 
	public void println( float inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println( int inValue ) {
		mPrintWriter.println( inValue );
		} 
	public void println( long inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println( Object inValue ) {
		mPrintWriter.println( inValue );
		}
	public void println( String inValue ) {
		mPrintWriter.println( inValue );
		}

	// don't worry about wrapping protected methods like setError() 

	public void write( char[] inValue ) {
	 	mPrintWriter.write( inValue );
		}
	public void write( char[] inValue, int inStart, int inEnd ) {
		mPrintWriter.write( inValue, inStart, inEnd );
		}
	public void write( int inValue ) {
	 	mPrintWriter.write( inValue );
		}
	public void write( String inValue ) {
	 	mPrintWriter.write( inValue );
		}
 	public void write( String inValue, int inStart, int inEnd ) {
		mPrintWriter.write( inValue, inStart, inEnd );
		}
	
	// end of wrapper methods

	}
