// Jason Rohrer
// ToClientMessage.java

/**
*
*	Messages that are sent to client.
*	Contain update information about global system state.
*
*	Created 5-9-2000
*	Mods:
*		Jason Rohrer	7-23-2000	Made this class abstract.
*/


package konspire.common;

/**
 * Abstract superclass for all messages sent from a <code>Server</code> to a <code>Client</code>.
 * <BR>
 * Contains information about global system state so that <code>Client</code> will
 * be updated with this information whenever it receives a message from a <code>Server</code>.
 *
 * @author Jason Rohrer
 */
public abstract class ToClientMessage implements Message {
	
	private long numHosts = -1;
	private long numFiles = -1;
	private long numBytes = -1;
	
	
	/**
	 * Sets the global system statistics that will be carried by this message.
	 *
	 * @param nHosts the total number of hosts in the system
	 * @param nFiles the total number of files in the system
	 * @param nBytes the total number of bytes in the system
	 */
	public void setStats( long nHosts, long nFiles, long nBytes ) {
		numHosts = nHosts;
		numFiles = nFiles;
		numBytes = nBytes;
		}
	
	/**
	 * Getes the total number of hosts statistic from this message.
	 *
	 * @return the total number of hosts in the system
	 */
	public long getNumHosts() {
		return numHosts;
		}
	
	/**
	 * Getes the total number of files statistic from this message.
	 *
	 * @return the total number of files in the system
	 */
	public long getNumFiles() {
		return numFiles;
		}
	
	/**
	 * Getes the total number of bytes statistic from this message.
	 *
	 * @return the total number of bytes in the system
	 */
	public long getNumBytes() {
		return numBytes;
		}

	}