// Jason Rohrer
// Vector.java

/**
*
*	Extends Java 1.1 Vector to provide some 1.2 functionality
*
*	Created 5-5-2000
*	Mods:
*/


package konspire.common;

//import java.util.Vector;
import java.util.Enumeration;


/**
 * Extends Java 1.1 <code>Vector</code> to provide some 1.2 functionality.  
 * <p>
 * Will be replaced
 * permanently by the 1.2 <code>Vector</code> implementation (or java.util.ArrayList) once
 * every konspire target platform has access to Java 1.2.
 *
 *
 * @author Jason Rohrer
 */
public class Vector extends java.util.Vector {


	public Vector() {
		super();
		}
	
	/**
	 * Constructs a <code>Vector</code> that starts with elements from a starting <code>java.util.Vector</code>.
	 *
	 * @param start starting point <code>Vector</code> for this new <code>Vector</code><br>
	 * constructed <code>Vector</code> will contain all elements from <code>start</code>
	 */ 
	public Vector( java.util.Vector start ) {
		super();
		
		addAll( start );
		}
	
	
	/**
	 * Constructs a <code>Vector</code> using standard <code>java.util.Vector</code> constructor
	 * parameters.
	 *
	 * @param startSize suggested starting capacity of <code>Vector</code>
	 */
	public Vector( int startSize ) {
		super( startSize );
		}
		
	/**
	 * Adds all elements from a <code>java.util.Vector</code> to this <code>Vector</code>.
	 *
	 * @param v source <code>java.util.Vector</code> for all elements to be added
	 */
	public synchronized boolean addAll( java.util.Vector v ) {
		Enumeration e = v.elements();
		
		while( e.hasMoreElements() ) {
			addElement( e.nextElement() );
			}
		return true;
		}
		
	
	/**
	 * Takes the intersection of this <code>Vector</code> with a <code>java.util.Vector</code>.
	 *
	 * @param v source <code>java.util.Vector</code> for all elements to be retained
	 *
	 * @return true iff this <code>Vector</code> changes as a result
	 */
	public synchronized boolean retainAll( java.util.Vector v ) {
		Enumeration e = elements();
		
		boolean returnVal = false;
		
		while( e.hasMoreElements() ) {
			Object o = e.nextElement();
			
			// remove all elements not also contained in v
			if( ! v.contains( o ) ) {
				removeElement( o );
				// restart enumeration after remove
				// YUCK... but only safe way to do this with 1.1
				// Enumeration no longer functions properly after the underlying
				// class has been altered by the remove
				e = elements();		
				returnVal = true;	
				}
			}
		return returnVal;	
		}	

	/**
	 * Extracts elements from this <code>Vector</code> in the form of an array.
	 *
	 * @return array containing all elements from this <code>Vector</code>
	 */
	public synchronized Object[] toArray() {
		Object[] result = new Object[elementCount];
		System.arraycopy(elementData, 0, result, 0, elementCount);
		return result;
		}
	
	}
	