// Jason Rohrer
// LogManager.java

/**
*
*	Modified from original LogManager by Todd Lauinger for use by konspire.
*
*	Created 8-23-2000
*	Mods:
*		Jason Rohrer	8-23-2000	Changed instance method names and
*									added static methods with the same
*									functionality.
*/


package konspire.common.log;

import java.io.*;
import java.util.*;
import java.text.*;

/*
 * LogManager is a concrete class which manages all of the logs in the
 * local system.  Anything that a particular log cannot do of itself,
 * LogManager is responsible for.
 *
 * @author Todd Lauinger
 * @version $Revision: 1.4 $
 *
 * @see RemoteLogManager
 */
public class LogManager extends Vector {

	protected static final String DEFAULT_APPLICATION_NAME = "App";

	protected static LogManager singleton = new LogManager();

	protected String applicationName = DEFAULT_APPLICATION_NAME;

	// Protect the singleton by scoping the constructor!
	protected LogManager() {
	}

	//
	// Class accessor which answers the singleton instance of the
	// LogManager for this image.
	//
	public static LogManager getInstance() {

		return singleton;
	}

	/**
	 * Return the application name string that the logs pertain to
	 */
	public String getInstanceApplicationName() {
		return applicationName;
	}

	/**
	 * Set the application name string that the logs pertain to
	 */
	public void setInstanceApplicationName(String appName) {
		applicationName = appName;
	}



	// Since this class is a singleton, we might as well have
	// static class methods as well.
	
	/**
	 * Return the application name string that the logs pertain to.
	 */
	public static String getApplicationName() {
		return singleton.getInstanceApplicationName();
	}



	/**
	 * Set the application name string that the logs pertain to.
	 */
	public static void setApplicationName( String inAppName ) {
		singleton.setInstanceApplicationName( inAppName );
	}

}


