package konspire.common.log;

import java.util.*;
import java.rmi.*;
import java.rmi.server.*;

/**
 * RemoteLogManager is the RMI interface to
 * the RemoteLogManagerImpl class so that RMI clients can 
 * control the application's logs.
 *
 * @author Todd Lauinger
 * @version $Revision: 1.3 $
 *
 * @see RemoteLogManagerImpl
 * @see LogManager
 */
public interface RemoteLogManager extends Remote {

    /**
     * Retrieves all of the names of the logs registered in
     * the virtual machine.
     * 
     * @exception java.rmi.RemoteException
     */
	public String[] getLogNames() throws RemoteException;

    /**
     * Gets the current logging level of a particular log.
	 *
	 * @param logName The name of the log as a string
     */
	public int getLoggingLevel(String logName) throws RemoteException;
	
    /**
     * Resets the current logging level of a particular log.
	 *
	 * @param logName The name of the log as a string
     */
	public void setLoggingLevel(String logName, int loggingLevel) throws RemoteException;

    /**
     * Gets the current log field separator of a particular log.
	 *
	 * @param logName The name of the log as a string
     */
	public String getLogFieldSeparator(String logName) throws RemoteException;
	
    /**
     * Resets the current log field separator of a particular log.
	 *
	 * @param logName The name of the log as a string
     */
	public void setLogFieldSeparator(String logName, String logFieldSeparator) throws RemoteException;
	
	/**
     * Returns all of the names of the logging mechanisms currently
     * active for the log given as input.
	 *
	 * @param logName The name of the log as a string
     */
    public String[] getLoggingMechanismNames(String logName) throws RemoteException;

    /**
     * Adds a logging mechanism to the log given.
	 *
	 * @param logName The name of the log as a string
     */
    public void addLoggingMechanism(String logName, String mechanismName) 
        throws RemoteException;

    /**
     * Removes a logging mechanism from the log given.
	 *
	 * @param logName The name of the log as a string
     */
    public void removeLoggingMechanism(String logName, String mechanismName) 
        throws RemoteException;    

	/**
	 * Adds a message string to the log if the log's level is
	 * currently set at or above the loggingLevel passed in.
	 *
	 * @param logName The name of the log as a string
	 *
	 * @param logString The string to write to all logging mechanisms
	 *
	 * @param loggingLevel The level at which to log the string.
	 */
	public void logString(String logName, String logString, int loggingLevel) 
	    throws RemoteException;

}
