package konspire.common.log;

import java.io.*;
import java.util.*;
import java.text.*;

/**
 * This logging mechanism is shared among all of the different logs
 * which attempt to use it, instead of creating a new log file for every
 * log (which the "Unique" logging mechanisms do).
 *
 * @author Todd Lauinger
 * @version $Revision: 1.3 $
 */
public class SharedFileLoggingMechanism extends AbstractFileLoggingMechanism {

//
// Class (static) section
//

	protected static SharedFileLoggingMechanism singleton =
		new SharedFileLoggingMechanism();

	//
	// Class instance accessor which answers the singleton instance of the
	// concrete subclass.
	//
	public static AbstractLoggingMechanism getInstance() {

		return singleton;
	}


	/**
	 * Shared logging needs an additional qualifier on each message
	 * logged, prefix the message with the log name.
	 */
	public String loggingPrefix(Log log, int loggingLevel) {

		return (log.getName() + log.getLogFieldSeparator() + 
		        super.loggingPrefix(log, loggingLevel)        );
	}

}

