// Jason Rohrer
// PropMessage.java

/**
*
*	Message wrapper for a propagating message
*
*	Created 5-2-2000
*	Mods:
*		Jason Rohrer	8-23-2000	Added JLog logging.
*/


package konspire.server;

import konspire.common.Message;
import konspire.common.Host;

import konspire.common.log.AppLog;


/**
 * Message wrapper for a propagating message.
 *
 * @author Jason Rohrer
 */
public class PropMessage implements Message {
	
	Host serverList[];
	boolean markers[];		// have servers been propagated to yet?
	
	Message m;
	
	/**
	 * Constructs a <code>PropMessage</code>.
	 *
	 * @param sList <code>Host</code>s this message should be propagated to
	 * @param in_m message to be wrapped in this <code>PropMessage</code>
	 */
	public PropMessage( Host sList[], Message in_m ) {
		
		m = in_m;
		
		serverList = sList;
		
		if( serverList != null ) {
			markers = new boolean[serverList.length];
			for( int i=0; i<serverList.length; i++ ) {
				markers[i] = false;
				}
			}
		else {
			markers = null;
			}
		}
	
	
	/**
	 * Gets the message wrapped in this <code>PropMessage</code>. 
	 *
	 * @return the message wrapped in this <code>PropMessage</code>
	 */
	public Message getMessage() {
		return m;
		}
	
	/**
	 * Marks a <code>Host</code> in this message's list as a <code>Host</code> that has
	 * already received the message	
	 *
	 * @param s <code>Host</code> to mark
	 */
	public void markServer( Host s ) {
		if( serverList != null ) {
			for( int i=0; i<serverList.length; i++ ) {
				if( serverList[i].equals(s) ) {
					markers[i] = true;
					}
				}
			}
		}
	
	
	/**
	 * Gets a <code>Host</code> that has not yet received this message.
	 *
	 * @return a <code>Host</code> that has not yet received this message
	 */ 
	public Host getUnmarked() {
		if( serverList == null ) {
			AppLog.error( "Server list empty on search for unmarked", true );
			return null;
			}
		
		// pick a random starting index
		// this should avoid propagations that always go in same order
		
		int startIndex = (int)( Math.random() * ( serverList.length ) );
		
		startIndex = startIndex % serverList.length;
		
		int currentIndex = startIndex;

		do {
			if( !markers[currentIndex] ) {
				AppLog.info( "Found unmarked:" + 
					serverList[currentIndex].toString() );
				return serverList[currentIndex];
				}
			currentIndex = (currentIndex + 1) % serverList.length;
			} while( currentIndex != startIndex ); 
			
		// if we got here, no unmarked hosts remain
		AppLog.info( "All found were marked (num hosts = " +
			serverList.length + ")" );
		return null;
		}


	}	// end class PropMessage
