// Jason Rohrer
// StringTreeFileDescriptor.java

/**
*
*	Descriptor of a file indexed by a StringTree
*
*	Created 5-4-2000
*	Mods:
*		Jason Rohrer	6-2-2000	Added support for file descriptors with paths
*/


package konspire.server.database;

import konspire.common.Host;
import konspire.common.FileDescriptor;


/**
 * Descriptor of a file indexed by a <code>StringTree</code>.
 * This wrapper allows the node indexing the <code>FileDescriptor</code>
 * to be accessed directly, facilitating the removal of a file
 * from the tree. 
 *
 * @author Jason Rohrer
 */
public class StringTreeFileDescriptor extends FileDescriptor {

	protected StringTreeNode node;
	
	/**
	 * Constructs a <code>StringTreeFileDescriptor</code>.
	 *
	 * @param f descriptor indexed in the tree
	 * @param n node where descriptor is indexed in the tree
	 */
	public StringTreeFileDescriptor( FileDescriptor f, StringTreeNode n ) {
		super( f.getHost(), f.getName(), f.getPath(), f.getSize() );
		node = n;
		}
	/**
	 * Gets the node where the file descriptor is inserted in the tree.
	 *
	 * @return the node where the descriptor is inserted in the tree
	 */	
	public StringTreeNode getNode() {
		return node;
		}
	
	/**
	 * Gets descriptor indexed in the tree.
	 *
	 * @return the descriptor indexed in the tree
	 */	
	public FileDescriptor toFileDescriptor() {
		return new FileDescriptor( fileLocation, fileName, filePath, fileSize );
		}
	
	// overrides Object equals( Object ) method
		
	public boolean equals( Object o ) {
		if( o instanceof StringTreeFileDescriptor ) {
			StringTreeFileDescriptor other = (StringTreeFileDescriptor) o;
			if( node == other.getNode()  && 
				toFileDescriptor().equals( other.toFileDescriptor() ) ) {
				
				return true;
				}
			}
		return false;
		}	
	
	}
	
