
#include <stdlib.h>
#include <stdio.h>

#include "share.h"

// Pick up global constant arrays
#define ONCE
#include "share_local.h"

#ifdef PYTHON_MODULE
#include "Python.h"
#endif

// Initialize parameters that don't change for a given
// packet count/segment count/field size
//
int init_params(int num_msg_pkts, int num_extra_pkts, int field_size,
  int num_segments, SIZE_PARAMS *these_params) {

  u_int32 cmask = 1 << field_size;
  int i;

  if(field_size > 15) {
#ifdef PYTHON_MODULE
    PyErr_SetString(PyExc_ValueError, "Field size cannot exceed 15");
#else
    do_error("Field size cannot exceed 15", __FILE__, __LINE__);
#endif
    return -1;
  }

  these_params->gfexp = field_size;
  these_params->mpkt_count = num_msg_pkts;
  these_params->rpkt_count = num_extra_pkts;
  these_params->tpkt_count = num_msg_pkts + num_extra_pkts;
  these_params->pkt_len = field_size * num_segments * sizeof(u_int32);

  if(these_params->tpkt_count > (1 << (field_size - 1))) {
#ifdef PYTHON_MODULE
    PyErr_SetString(PyExc_ValueError,
            "Too many total packets for field size");
#else
    do_error("Too many total packets for field size", __FILE__, __LINE__);
#endif
    return -1;
  }

  these_params->seg_count = num_segments;
  these_params->topbit = 1 << (field_size - 1);
  these_params->MultFieldMax = (1 << field_size) - 1;

  // Allocate memory for lookup tables
  these_params->FieldEltToExp = (u_int32 *) calloc((1 << field_size), sizeof(u_int32));
  these_params->ExpToFieldElt = (u_int32 *) calloc((1 << field_size) + field_size, 
    sizeof(u_int32));

  if((these_params->FieldEltToExp == 0) || 
     (these_params->ExpToFieldElt == 0)) {
#ifdef PYTHON_MODULE
    PyErr_SetString(PyExc_MemoryError, "Couldn't allocate lookup tables");
#else
    do_error("Calloc failed", __FILE__, __LINE__);
#endif
    if (these_params->FieldEltToExp) free(these_params->FieldEltToExp);
    if (these_params->ExpToFieldElt) free(these_params->ExpToFieldElt);
    return -1;
  }

  // Initialize lookup tables based on irreducible polynomial
  // in POLYMASK
  these_params->ExpToFieldElt[0] = 1;
  for (i=1; i < these_params->MultFieldMax + field_size - 1; i++) {
    these_params->ExpToFieldElt[i] = these_params->ExpToFieldElt[i-1] << 1;
    if (these_params->ExpToFieldElt[i]&cmask) {
      these_params->ExpToFieldElt[i] ^= POLYMASK[field_size];
    }
  }

  // FieldEltToExp is the inverse of ExpToFieldElt
  these_params->FieldEltToExp[0] = -1; 
  for (i=0; i < these_params->MultFieldMax; i++) {
        these_params->FieldEltToExp[these_params->ExpToFieldElt[i]] = i;
  }

  return 0;
}
