
#ifndef _SHARE_H
#define _SHARE_H

typedef unsigned int u_int32;
typedef unsigned char u_int8;

// structure used to define a given block sharing scheme
//
typedef struct size_params_struct {
  u_int32 gfexp; //  matrix ops are done in GF(2^gfexp)
  u_int32 mpkt_count; // number of message packets
  u_int32 rpkt_count; // number of redundant packets
  u_int32 tpkt_count; // total packets 
  u_int32 seg_count; // number of segments per packet 

  // packet size in bytes = gfexp * seg_count * sizeof(u_int32)
  u_int32 pkt_len; // length of packet

  // Lookup tables used to convert between exponent and
  // field element form 
  u_int32 *FieldEltToExp;
  u_int32 *ExpToFieldElt; 

  // Largest vaule in the multiplicative field.
  u_int32 MultFieldMax; 

  // highest order bit in GF(2^gfexp)
  u_int32 topbit;

} SIZE_PARAMS;

// the structure used to create shares of a Diaspora block
typedef struct share_struct {
  u_int32 *message;  // the block to be shared
  u_int32 *shares; // the created shares
  SIZE_PARAMS *sp; // the parameters for sharing it
} SHARE;

// the structure used to re-assemble shares of a Diaspora block
typedef struct rebuild_struct {
  int rec_packets; // number of packets received so far
  u_int32 *rec_table; // list of packet indexes received
  u_int8 *rec_mark; // checklist of packets received

  u_int32 *pre_block;  // the pre-decoding block
  u_int32 *post_block;  // the post-decoding block

  SIZE_PARAMS *sp;
  u_int8 done;
} REBUILD;

// function prototypes
int init_params(int num_msg_pkts, int num_extra_pkts, int field_size,
int num_segments, SIZE_PARAMS *these_params);
int make_shares(SHARE *s);
int share_in(u_int32 pkt_index, char *packet, u_int32 packet_sz, REBUILD *rb);
int rebuild_block(REBUILD *rb);
void do_error(const char *msg, const char *file, const int line);

#else
#endif
