// Takes an incoming packet, and does the "right thing" with it
// Notices if it has enough packets to reconstruct the message,
// and triggers message rebuilding. 
//

#include <stdio.h>
#include "share.h"
#include "share_local.h"

#ifdef PYTHON_MODULE
#include "Python.h"
#endif

int share_in(u_int32 pkt_index, char *packet, u_int32 packet_sz, REBUILD *rb) {
  u_int32 *pre_block_ptr; 
 
  if(packet_sz != rb->sp->pkt_len) {
#ifdef PYTHON_MODULE
    PyErr_SetString(PyExc_ValueError, "Wrong packet size");
#else
    do_error("wrong packet size", __FILE__, __LINE__);
#endif
    return -1;
  }

  if(pkt_index >= rb->sp->rpkt_count) {
#ifdef PYTHON_MODULE
    PyErr_SetString(PyExc_ValueError, "Bad packet index");
#else
    do_error("bad packet index", __FILE__, __LINE__);
#endif
    return -1;
  }

  // pre_block is a dense collection of the received shares
  // we add another share to it here
  //
  pre_block_ptr = &(rb->pre_block[packet_sz * rb->rec_packets / 
     sizeof(u_int32)]);

  memcpy(pre_block_ptr, packet, packet_sz);

  // only increase packet received count if packet is really new
  // rec_table stores a list of received shares (in order
  // received)
  //
  if(rb->rec_mark[pkt_index] == 0) {
    rb->rec_table[rb->rec_packets] = pkt_index;
    rb->rec_mark[pkt_index] = 1;
    rb->rec_packets++;  
    if(rb->rec_packets == rb->sp->mpkt_count) {
      /* rebuild the block, stopping on any errors */
      if (rebuild_block(rb))
        return -1;
    }
  }
  
  return 0;
}
