

#include "share.h"
#include "share_local.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

int main() {
  int retval;

  SIZE_PARAMS sp_struct;
  SHARE share_struct;
  REBUILD rebuild_struct;

  u_int32 num_msg_pkts = 8;
  u_int32 num_extra_pkts = 16;
  u_int32 field_size = 8;
  u_int32 num_segments = 512;
  u_int32 msize, esize;
  u_int32 rand_share, share_off;
  int i;
  char used_shares[100]; // XXX keep track of shares already used
  struct timeval tp ;
  struct timezone tzp;

  gettimeofday(&tp, &tzp);
  srand48(tp.tv_usec);

  printf("Starting share test\n");
  retval = init_params(num_msg_pkts, num_extra_pkts, field_size, 
    num_segments, &sp_struct);  

  share_struct.sp = &sp_struct;

  /* calculate total message size in bytes */
  msize = num_msg_pkts * field_size * num_segments * sizeof(u_int32);
  printf("message size is %d\n", msize);

  share_struct.message = (u_int32 *) malloc(msize);

  /* calculate total extra packet size in bytes */
  esize = num_extra_pkts * field_size * num_segments * sizeof(u_int32);
  printf("extra packet size is %d\n", esize);

  /* clear shares */
  share_struct.shares = (u_int32 *) malloc(esize);
  memset(share_struct.shares, 0, esize);

  for(i = 0; i < (msize/sizeof(u_int32)); i++) {
    share_struct.message[i] = i;
  }

  make_shares(&share_struct);

  // initialize REBUILD
  memset(&rebuild_struct, 0, sizeof(rebuild_struct));
  rebuild_struct.rec_table = (u_int32 *) calloc(num_extra_pkts, sizeof(u_int32));
  rebuild_struct.rec_mark = (u_int8 *) calloc(num_extra_pkts, sizeof(u_int8));
  rebuild_struct.pre_block = (u_int32 *) calloc(esize, 1);
  rebuild_struct.post_block = (u_int32 *) calloc(msize, 1);
  rebuild_struct.sp = &sp_struct;
  
  printf("starting to send random packets to rebuild\n");
  printf("pkt_len is %d\n", sp_struct.pkt_len);
  fflush(stdout);
  i = 0;
  bzero(used_shares, sizeof(used_shares));
  while(!rebuild_struct.done) {
    // pick an unused share
    do {
      rand_share = mrand48() % num_extra_pkts;
    } while (used_shares[rand_share]);
    used_shares[rand_share] = 1;  // mark this share as used
    printf("GPS using share %d.\n", rand_share);
    share_off = rand_share * sp_struct.pkt_len;
    share_in(rand_share, (char *) &(share_struct.shares[share_off / 4]), 
      sp_struct.pkt_len, &rebuild_struct);
    ++i;
  }

  printf("GPS rebuilt from %d shares (no duplicates allowed).\n", i);

  for(i = 0; i < (msize/sizeof(u_int32)); i++) {
    if(share_struct.message[i] != rebuild_struct.post_block[i]) {
      printf("%d does not match %d\n", share_struct.message[i],
        rebuild_struct.post_block[i]);
    } else {
      printf("message at %d is %d\n", i, rebuild_struct.post_block[i]);
    }
  }

  return 0;
}
