# This module encapsulates UI Presentation.

### Imports:
# Standard modules:
import types
import os
import string
import time
import urllib
import sys
import copy
import stat
import cgi

# Mojo Modules:
import debug
import confutils
import contentlib
import mojobootpage
import DocumentTemplate
from MojoConstants import true, false
from VersionNumber import VersionNumber
import mojostd
import mojometric
import idlib
import mojoutil

class PresentationError(StandardError):
    pass

class DuplicateChecker:
    def __init__(self):
        self.__dups = {}

    def register(self, value):
        value = `value`
        if self.__dups.has_key(value):
            return false
        else:
            self.__dups[value] = None
            return true
        
class PseudoModule:
    def __init__(self, dict):
            self.dict = dict
    def __getattr__(self, name):
            return self.dict[name]
pseudomodule = PseudoModule(locals())

def file_exists(path):
    return os.path.exists(path)

def format_time(seconds=None):
    """
    Whenever we want to present a time to the user, always use this function
    for consistency.  If we decide to change the format we can simply change it here.
    """
    if seconds is None:
        seconds = time.time()
    # TODO really we should make the date format string configurable to match users arcane preferences
    #return time.strftime("%Y-%m-%d %H:%M:%S %Z", time.localtime(seconds))
    return time.strftime("%B %d, %I:%M%p", time.localtime(seconds))
    
def format_search_result(descdict={}):
    """
    Take the xml content type description for the search and the description dictionary of the item returned
    and format for a smaller search return table. Do not show empty fields, and bundle the "less important" 
    fields into one table row. 
    Return 2 dictionaries. One of fields with each having their own row, one with the fields that are bundled.
    """
    # XXX this is a big hack.  info on which fields are short vs long for display needs to be added
    # to the xml content type description itself.
    own_row=['Title','Description','MojoID','Performer',
                'TITLE','AUTHOR','SUBJECT', 'ADD. AUTHOR', 'NOTES',
                'Author(s)', 'Subject', 'Description/Keywords', 'ISBN Number', 
                'Platforms', ]
    row={}
    for key in descdict.keys():
    	if key in ["__DINODE__","__MOJOID__"] :
    		continue
        if key in own_row:
            row[key]=1
        else:
            row[key]=0
    return row

def get_counter(value=0):
    return mojoutil.Counter(value=0)
 
def get_config_value(templatestr):
    """
    Takes a string from a template file and translates it into an access into confutils.confman.dict.
    The translation is done as follows:
    A '.' (period) or '__' (double underscore) signifies a "classification separator".
    A '___' (triple underscore) signifies a space.
    If any key ends with the letters PATH os.path.expandvars will be called on the result
    For example:  "PATH__CGI-BIN_DIR" resolves to os.path.expandvars(confutils.confman.dict["PATH"]["CGI-BIN_DIR"])
    For example:  "MAX_VERBOSITY" resolves to confutils.confman.dict["MAX_VERBISOTY"]
    """
    value = confutils.confman.dict # Signifies an error or nonexistence of a value.

    templatestr = string.replace(templatestr, '___', ' ')   # ___s are really spaces
    templatestr = string.replace(templatestr, '__', '.')    # __s are really .s

    call_expandvars = None
    for classification in string.split(templatestr, '.'):
        if classification[:len('PATH')] == 'PATH':
            call_expandvars = 1
        try:
            value = value[classification]
        except KeyError:
            return None
        
    if type(value) == types.StringType :
        if call_expandvars:
            return os.path.expandvars(value)
        else:
            return value
    else:
        return None

def get_flat_confman_dict(confdict=None, appendstr='', _prefixstr=''):
    """
    Returns a flattened dictionary containing confman keys/subdicts
    flattened out with '__' seperating subdict names.  This is used for
    the web form config UI.  For example the key 'PATH__CGI-BIN_DIR'
    will map to confman.dict['PATH']['CGI-BIN_DIR'].

    @param confdict defaults to confutils.confman.dict
    @param appendstr is a string to append to all returned dictionary keys (such as _DEFAULT)
    @param _prefixstr is used internally for recursion.
    """
    if confdict is None:
        confdict = confutils.confman.dict
    flatdict = {}
    for key,value in confdict.items():
        flatkey = _prefixstr + key
        # turn all spaces in flatkey into ___
        flatkey = string.replace(flatkey, ' ', '___')
        if type(value) == types.DictType:
            flatdict.update(get_flat_confman_dict(confdict=value, appendstr=appendstr, _prefixstr=flatkey+'__'))
        else:
            # A hack because the dtml doesn't use confman.is_true_bool:
            if value == "true":
                value = "yes"
            elif value == "false":
                value = "no"
            flatdict[flatkey + appendstr] = value
    return flatdict

def set_config_value(templatestr, value, addnewkey=0, dict=None):
    """
    See get_config_value.

    If addnewkey is false, a non-existent key will not be added, and
    will silently fail.

    dict defaults to confman.dict if not specified
    """
    templatestr = string.replace(templatestr, '___', ' ')   # ___s are really spaces
    templatestr = string.replace(templatestr, '__', '.')    # __s are really .s

    classifications = string.split(templatestr, '.')
    dictkeys = classifications[:-1]
    key = classifications[-1]
    
    if dict is None:
        dict = confutils.confman.dict # Signifies an error or nonexistence of a value.

    if hasattr(types, 'UnicodeType') and type(value) == types.UnicodeType:
        value = str(value)

    for dictkey in dictkeys:
        if type(dict) != types.DictType:
            raise PresentationError, "Could not set config value, because <%s> did not refer to dict." % `dict`
        elif not dict.has_key(dictkey):
            raise PresentationError, "Could not set config value, because non-most-specific key <%s> did not exist." % `dictkey`
        else:
            dict = dict[dictkey]
        
    if type(dict) == types.DictType :
        if not addnewkey and not dict.has_key(key):
            return
        dict[key] = value
        # XXX HACK this forces computation of the other block prices if the
        # user uses the UI to configure their block server block prices.
        debug.stderr.write("dictkeys %s\nkey %s\nvalue %s\n", args=(dictkeys, key, value), v=6)
        if len(dictkeys) > 2 and dictkeys[:2] == ['BLOCKSERVER', 'PRICE_MAPS'] :
            assert float(value) >= 0
            if dictkeys[2] == 'RETRIEVE' or dictkeys[2] == 'PUBLISH' :
                dict[key] = str(mojostd.round_to_positive_int_with_min(value))
                debug.stderr.write("Saving new %s prices for all block sizes\n", args=(dictkeys[2],), v=2)
                onekbprice = float(value)
                dict.update( confutils.gen_per_kb_price_dict(onekbprice) )
            # Hack to keep block server prices for "HAVE BLOCK" and "HAVE BLOCK LIST" equal:
            if dictkeys[2] == 'HAVE BLOCK':
                confutils.confman.dict['BLOCKSERVER']['PRICE_MAPS']['HAVE_BLOCK_LIST'] = str(mojostd.round_to_positive_int_with_min(value))
    else:
        raise PresentationError, "Could not set config value, because value <%s> did not refer to dict." % `dict`
    
def cdesc_to_table(typedefrep, ctype, cdesc, style={}):
    s={
        'keywrapper' : '<b>%s</b>',
        'valuewrapper' : '<font size=-1>%s</font>',
        }
    s.update(style)
    return recursive_dict_to_html_table(cdesc, orderedkeys=typedefrep.get_ordered_type_fieldname_list(ctype), style=s)

def cut_off_dirs(path):
    return os.path.basename(path)

def recursive_dict_to_html_table(dict, sortfunc=cmp, orderedkeys=None, style={}, depth=0):
    """
    Returns a string representing html rows which represent the dict in a human readable context.

    If orderedkeys, it is a list of keys that should be taken from the dict (in order).
    """
    rows = []
    if not orderedkeys:
        orderedkeys = dict.keys()
        orderedkeys.sort(sortfunc)
    for key in orderedkeys:
        if not dict.has_key(key):
            continue
        value = dict[key]
        if type(value) == types.DictType:
            value = recursive_dict_to_html_table(value, sortfunc, style, depth+1)
        rows.append('''\
%(depth)s<td %(td)s %(keytd)s>%(keywrapper)s</td>

%(depth)s<td %(td)s %(valuetd)s>
%(depth)s\t%(valuewrapper)s
%(depth)s</td>
''' % {
            'depth' : '\t' * (depth + 2),
            'td' : (style.get('td', '')),
             'keywrapper' : style.get('keywrapper', '%s') % key,
             'valuewrapper' : style.get('valuewrapper', '%s') % str(value)
            })

    rowformat='%(depth)s<tr %(tr)s>\n%(depth)s\t%%s\n%(depth)s</tr>' % {
            'depth' : '\t' * (depth + 1),
            'tr' : style.get('tr', ''),
            }
    rows = map(lambda s, rf=rowformat: rf % s, rows)
    return '''\
%(depth)s<table %(table)s>
%(depth)s\t%(rows)s
%(depth)s</table>
''' % {
        'table' : style.get('table', ''),
        'rows' : string.join(rows, '\n'),
        'depth' : '\t' * depth,
        }

def is_checked(bool):
    if bool:
        return "checked"
    else:
        return ""


def show_conf_value(value):
	mapping = { 'yes' : 'On',
		'no' : 'Off',}
	return mapping.get(value, value)

def _debug_dump(obj):
    return cgi.escape(`obj`)

# This is a global list of alternate paths that should be searched for DTML files
# before confman['PATH']['WEB_TEMPLATE_DIR'] is searched.
Additional_DTML_Paths = []
def name_to_dtmlpath(filename):
    """
    Given a dtml template name such as 'status' return the full pathname to the appropriate dtml file to render.
    """
    for path in Additional_DTML_Paths:
        fullpath = os.path.join(path, filename + '.dtml')
        if os.path.isfile(fullpath):
            return fullpath
    return os.path.join(confutils.confman.getpath(['PATH', 'WEB_TEMPLATE_DIR']), filename + '.dtml')


### Presentation Layer API:
# For use by the Presentation Layer.
def show(filename, variables, **kwargs):
    """
    Return a string representation of template filename.
    variables should be a VariableManager object specific to the current http request.

    other keyword arguments are passed directly to the dtml parser for use as dtml variables
    """
    fullpath = name_to_dtmlpath(filename)
    mtime = os.stat(fullpath)[stat.ST_MTIME]
    if not _show_cache.has_key(filename) or _show_cache[filename][0] != mtime:
        # If CACHE_DTML is false, we simply refresh the cache with each call.
        # (This is handy if you are editing DTML files and you want to see the changes without restarting the Broker.)
        
        # If it's not in the cache, add it:
        _show_cache[filename] = (mtime, DocumentTemplate.File(fullpath))
        
    if variables is not None:
        # XXX HACK this code is very ugly and non-obvious to follow what it does since it is messing around directly with the
        # dictionary defining what is in the global scope of this module.  Doing the wrong thing easily leads to infinite
        # recursion crashes.
        ourpseudomodule = PseudoModule(copy.copy(pseudomodule.dict))  # make our own pseudomodule derived from the global one
        ourpseudomodule.dict['variables'] = variables
        # replace the show method with one that defaults to using the VariableManager passed to us
        # so the .dtml files don't need:  <dtml-var expr="Presentation.show('spam_and_eggs', Presentation.variables)">
        def show_with_these_variables(filename=filename, variables=variables, **kwargs):
            return apply(show, (filename, variables), kwargs)  # use show() from the global pseudomodule
        ourpseudomodule.dict['show'] = show_with_these_variables

    dtml_mapping = {}
    dtml_mapping.update(kwargs)
    dtml_mapping.update({'Presentation':ourpseudomodule, 'os':os, 'string':string, 'urllib':urllib})

    dtml_mapping['debug'] = _debug_dump

    # import some commonly needed python builtin functions.  Do -NOT- import the __builtins__ directly,
    # that causes conflicts with python's __builtin__._ and dtml's use of _.  In some interpreter setups.
    dtml_mapping.update({'int': int, 'None': None, 'len': len, 'str': str, 'dir': dir, 'filter': filter, 'map': map})
    return _show_cache[filename][1](mapping=dtml_mapping)

_show_cache = {} # maps filename to a two-tuple (mtime, DocumentTemplate.File instance).


class VariableManager:
    """
    Encapsulates presentation variables.
    """
    def __init__(self, typedefrep, privacycode=None, maps={}, _initialized_updatemap=None):
        """
        self.updatemap is be updated with maps.
        """
        self.__typedefrep = typedefrep
        self.__privacycode = privacycode

        self.__searchdups = {}

        # The "updatemap" dict maps presentation variables to
        # either a value, or a means of calculating a value.
        # The keys must be strings.

        # The value for each key is of this format:
        # (type, source) where type is a string
        # equal to either "value" or "function".

        # If type is "value" then source is returned
        # to the presentation layer.

        # If type is "function" then source is a
        # two-length tuple containing a function
        # and a tuple of arguments.
        def generate_baseurl(self=self):
            if self.__privacycode:
                return '/%s/' % self.__privacycode
            else :
                return ''
        def url_config_pathname(p):
            return urllib.pathname2url(os.path.expandvars(p))

        if _initialized_updatemap:
            self.updatemap = _initialized_updatemap
            return

        self.updatemap = {
            "colors" : ("value",
                        {"MOJO BLUE" : '"#0099cc"',
                         "DARK BLUE" : '"#000066"',
                         "NAV ON" : '"#ffffff"',
                         "NAV OFF" :  '"#0099cc"',
                         "LIGHT BLUE" : '"#6699cc"'}
                        ),
            "configvalues" : ("value", confutils.confman.dict),
            "configvaluedefaults" : ("value", confutils.confdefaults),
            "knownctypes" : ("function", (self.__typedefrep.known_type_list, ())),
            "typedefrep" : ("value", self.__typedefrep),
            "contentlib" : ("value", contentlib),
            "webroot" : ("function", (url_config_pathname, (confutils.confman.dict['PATH']['WEBROOT_DIR'],)) ),
            "version" : ("value", VersionNumber(confutils.BROKER_VERSION_STR)),
            "bootpage_version" : ("function", (mojobootpage.get_bootpage_version, ()) ),
            "errorstring" : ("value", None),
            "intropagepath" : ("function", (url_config_pathname, (confutils.confman.dict['PATH']['INTRO_PAGE_v2'],)) ),
            "readmepath" : ("function", (url_config_pathname, (confutils.confman.dict['PATH']['README_FILE_v2'],)) ),
            "faqpath" : ("function", (url_config_pathname, (confutils.confman.dict['PATH']['FAQ_FILE'],)) ),
            "quickstartpath" : ("function", (url_config_pathname, (confutils.confman.dict['PATH']['QUICKSTART'],)) ),
            "autorefresh" : ("function", (lambda cu : cu['UI_REFRESH_RATE'], (confutils.confman.dict,))),
            "ctypes" : ("value", typedefrep.typedef_ordered_map),
            "pythonversionstr" : ("value", sys.version),
            "mojometer" : ("value", mojometric.mojometer),
            "mojometer_color" : ("value", mojometric.mojometer_color),
            "get_digit_filenames" : ("value", mojometric.get_digit_filenames),
            "platform": ("value", confutils.platform),
            "browser" : ("value", "unknown"),   # this is set by each request handler
            # These variables are set below:
            "baseurl" : ("function", (generate_baseurl, ()) ),
            "token_account_id" : ("value", "*unknown: broker still starting*" ),          
            "broker_id" : ("value", "*unknown: broker still starting*" ),          
            }
        
        # Note: These updatemap keys should be set before each calling of show:
        # state - The state of the user interface represented as a string.
        # userinput - User input as a dict.

        # Values that only need to be caluculated once:
        
        # Specific values that were passed in at runtime:
        self.updatemap.update(maps)

    def prepare_servicemtm_values(self, servicemtm, tokenservermtm):
    	# for values that change after start of broker
        self.updatemap.update({           
            "mojo" : ("function", (servicemtm._pt.get_amount_in_purse, () )),
            "outgoingoffers" : ("function", (servicemtm._keeper.get_total_mojo_we_owe_others, () )),
            "incomingoffers" : ("function", (servicemtm._keeper.get_total_mojo_others_owe_us, () )),
            "token_account_id" : ("value", idlib.to_ascii(tokenservermtm.get_id()) ),                      
            "broker_id" : ("value", idlib.to_ascii(servicemtm.get_id()) ),                      
            })
        debug.stderr.write('Initialized servicemtm-related values.\n', vs="Presentation", v=5)
        
    def clone(self):
        """
        Return a new VariableManager instance class that contains all
        of the state from this one in its own copy of self.updatemap.
        """
        return VariableManager(typedefrep=self.__typedefrep, _initialized_updatemap=copy.copy(self.updatemap))

    def get(self, presentationvariable, default=None):
        """
        This returns an appropriate value according to the semantics
        of self.updatemap.

        Arguments:
        presentationvariable - A valid key in self.updatemap.
        """
        if not self.updatemap.has_key(presentationvariable):
            return default
        vartype, source = self.updatemap[presentationvariable]
        if vartype == 'value':
            return source
        elif vartype == 'function':
            func, args = source
            return apply(func, args)
        else:
            raise PresentationError, 'Unknown presentation variable type "%s".' % vartype
        # Unreachable code.

    def set(self, presentationvariable, source, vartype="value"):
        """
        Use this to change values.

        Arguments:
        presentationvariable - A valid key for self.updatemap.
        source - A valid source which follows the semantics of updatemap.
        vartype - A valid type for updatemanp (must be "value" or "function").
        """
        if not (vartype == "value" or vartype == "function"):
            raise PresentationError, 'Cannot set presentation variable, unknown type "%s".' % vartype
        if vartype == "function":
            func, args = source
            if not (type(func) == types.FunctionType and type(args) == types.TupleType):
                raise PresentationError, 'Cannot set presentation variable, incorrect function source format %s.' % str(source)
        self.updatemap[presentationvariable] = (vartype, source)

    def mapping(self, keylist, mapping={}):
        for key in keylist:
            mapping[key] = self.get(key)
        return mapping

    
    def set_dict_key(self, d, key, value):
        d[key] = value

        
    def __repr__(self):
        return '<Presentation.VariableManager instance>: %s' % `self.__typedefrep`
