#!/usr/bin/env python
#
#  Copyright (c) 2001 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
#
# A Mojo Nation (TM) archive publisher and publishing library.
#

# standard modules
from cStringIO import StringIO
import whrandom # only for randomizing the order we read lists
import threading
from Queue import Queue, Empty
from time import time

# our modules
import debug
import makesplit
from ClientEGTPlib import publish_blob
import mojoutil
import MetaTrackerLib
from confutils import confman
from human_readable import hr
import idlib
import idmasklib
import mojosixbit
import sharemap
import SEXP
import MojoTransaction
import pricelib
from DataTypes import *

from PublishSession import PublishSession

class Failure(StandardError) : pass

MAX_CONCURRENT_PUTS = 8  # XXX this should be configurable

def find_publishing_agents(MTM, get_broadcasters_only=false) :
    """
    Returns a list of tuples containing publishing agent ids and their
    metainfo dicts.
    """
    agents_id_map = MetaTrackerLib.get_publishing_agents(MTM, get_broadcasters_only=get_broadcasters_only)

    # XXX call business logic to sort/filter the list in order of preference
    return agents_id_map.items()


def find_publish_blob_servers(MTM, blob_id) :
    """
    Returns a list of blob servers that should be willing to publish
    a blob with the given Id.
    """
    # XXX fine tune how many bits we specify in this search mask (it
    # should probably be semi-dynamic based on some info about how fine
    # of a mask range servers are speaking these days)  That way we'll
    # get the most out of our meta tracker response without giving
    # away too much info about what we're looking for.
    search_id_mask = idmasklib.IdMask(mask=idlib.to_binary(blob_id), bits=14)

    blobservers_id_map = MetaTrackerLib.get_publishing_blob_servers(MTM, blob_id_mask=search_id_mask, min=5)

    # XXX call business logic to sort/filter the list in order of preference
    return blobservers_id_map.items()



locations_template = ListMarker({'blob id' : UNIQUE_ID, 'server id' : BINARY_SHA1, 'bytes' : INTEGER})

########################################################################
if __name__ == '__main__' :
    run()

