#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
#
__cvsid = '$Id: BandwidthThrottler.py,v 1.7 2001/02/02 07:09:13 greg Exp $'


### standard modules
from time import time

### our modules
from MojoConstants import true, false, DEBUG_MODE
import debug


class BandwidthThrottler:
    """
    Very simple class to keep track of how much bandwidth you're using.  Curently we have one
    instance for in- and one for out-.
    """
    def __init__(self, Kbps, throttle=false, granularity=10):
        """
        @param Kbps max throughput in kilobits/second
        @param throttle `true' if and only if comms should be throttled when more than `Kbps'
            bandwidth has been used (averaged over the last `granularity' seconds)
        @param granularity how many seconds long are the periods
        """
        self._maxbytes = long(((Kbps / 8.0) * 1024.0) * granularity)
        self._throttle = throttle
        self._granularity = granularity 
        self._used = 0
        self._lasttick = time()

    def used(self, bytes):
        """
        @param bytes the number of bytes (not bits!) you just used
        """
        if time() > (self._lasttick + self._granularity):
            if DEBUG_MODE:
                debug.mojolog.write("BandwidthThrottler measuring: %s bytes in %s seconds (%s Kbit/s)\n", args=(self._used, "%0.0f" % (time() - self._lasttick), "%0.3f" % (((self._used * 8.0) / 1024.0) / self._granularity)), v=7, vs="TCPCommsHandler") ### for faster operation, comment this line out.  --Zooko 2000-12-11

            self._lasttick = time()
            self._used = bytes
            return

        self._used = self._used + bytes

        if (DEBUG_MODE) and (self._throttle) and (self._used >= self._maxbytes):
            debug.mojolog.write("BandwidthThrottler maxed out: %s bytes in %s seconds (%s Kbit/s); throttling\n", args=(self._used, "%0.0f" % (time() - self._lasttick), "%0.3f" % (((self._used * 8.0) / 1024.0) / self._granularity)), v=7, vs="TCPCommsHandler") ### for faster operation, comment this line out.  --Zooko 2000-12-11

    def is_maxed(self):
        if time() > (self._lasttick + self._granularity):
            if DEBUG_MODE:
                debug.mojolog.write("BandwidthThrottler measuring: %s bytes in %s seconds (%s Kbit/s)\n", args=(self._used, "%0.0f" % (time() - self._lasttick), "%0.3f" % (((self._used * 8.0) / 1024.0) / self._granularity)), v=7, vs="TCPCommsHandler") ### for faster operation, comment this line out.  --Zooko 2000-12-11

            self._lasttick = time()
            self._used = 0
            return

        return (self._throttle) and (self._used >= self._maxbytes)
