#!/usr/bin/env python
#
#  Copyright (c) 2001 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
__cvsid = '$Id: BidStrategy.py,v 1.11 2001/06/08 01:49:06 greg Exp $'


# standard modules
import whrandom

# our modules
from confutils import confman
import debug
import mojoutil

# Here are the default settings for client bidding strategy:

# Want to see how bidding higher or lower effects behaviour?  Twiddle this knob
# here: (GREG BROILES, look here)
mojoutil.setdefault(confman, "MO_HURRY_FACTOR", 5)

if confman.has_key("MO_ADD_NOISE_PERCENT"):
    del confman["MO_ADD_NOISE_PERCENT"]

# This subtracts a random amount to your bid.
mojoutil.setdefault(confman, "MO_SUB_NOISE_PERCENT", 50)

# This is the premium you'll offer for your favourite servers.
mojoutil.setdefault(confman, "MO_PREMIUM_FOR_PREFERRED_SERVERS_PERCENT", 20)

# Here are specific weightings for specific kinds of message.  Basically, you
# should probably just think of this as "hurry" or "don't hurry" until we see
# behaviour better.  Any more fine tuning is perhaps wasted effort.
mojoutil.setdefault(confman, "MO_PUT_BLOB", 5)
mojoutil.setdefault(confman, "MO_DO_YOU_HAVE_BLOBS_DOWNLOADING", 20)
mojoutil.setdefault(confman, "MO_DO_YOU_HAVE_BLOBS_CONTENT_VERIFYING", 2)
mojoutil.setdefault(confman, "MO_DO_YOU_HAVE_BLOBS_BLOCK_WHOLESALING", 2)
mojoutil.setdefault(confman, "MO_HELLO", 6)
mojoutil.setdefault(confman, "MO_GOODBYE", 1)
mojoutil.setdefault(confman, "MO_RETRIEVE_MESSAGES", 20)
mojoutil.setdefault(confman, "MO_ARE_THERE_MESSAGES", 10)
mojoutil.setdefault(confman, "MO_CONTENT_TRACKER_SUBMIT", 3)
mojoutil.setdefault(confman, "MO_CONTENT_TRACKER_LOOKUP", 10)
mojoutil.setdefault(confman, "MO_DOWNLOAD_CONTENT_TYPES", 1)
mojoutil.setdefault(confman, "MO_GET_BAD_BLOCK_LIST", 1)
mojoutil.setdefault(confman, "MO_REQUEST_BLOB_DOWNLOADING", 20)
mojoutil.setdefault(confman, "MO_REQUEST_BLOB_WHOLESALING", 1)
mojoutil.setdefault(confman, "MO_LOOKUP_CONTACT_INFO", 5)
mojoutil.setdefault(confman, "MO_LIST_SERVERS", 5) # There should really be a more urgent and less urgent version of this, but I don't want to fix it in MetaTrackerLib right now.  --Zooko 2001-06-01
mojoutil.setdefault(confman, "MO_STORE_LOCATION_INFORMATION", 5)

def choose_bid(mtype, mtm, counterparty_id=None, preferenceno=0):
    """
    @param preferenceno 0 if this was your first choice for this service, 1 if it was your second choice, etc.  We'll add a 20% higher bid for the most preferred servers, dropping off to a 3% premium for the 20th favourite.
    """
    base = long(confman.get(mtype, 10)) * long(confman.get("MO_HURRY_FACTOR", 5))
    # Add some noise so that we can tell how loaded a server is even if all clients are running with default values.
    noise = whrandom.random() * (float(confman.get("MO_SUB_NOISE_PERCENT", 50.0)) / 100)
    boostforpref = (float(confman.get("MO_PREMIUM_FOR_PREFERRED_SERVERS_PERCENT", 20.0)) / 100) * (0.9 ** preferenceno)
    bid = base * (1.0 - noise + boostforpref)
    bid = long(max(0, min(bid, mtm.get_total_mojo())))
    debug.mojolog.write("BidStrategy: mtype: %s, counterparty_id: %s, base: %s, noise: %s, boostforpref: %s, bid: %s\n", args=(mtype, counterparty_id, base, noise, boostforpref, bid,), v=6, vs="BidStrategy")
    return bid
