#
#  Copyright (c) 2001 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
__cvsid = '$Id: BitmapDownloadMonitor.py,v 1.6 2001/05/26 00:27:01 greg Exp $'

import types

# This is the Python Imaging Library available from http://www.pythonware.com/products/pil/
# version 1.1.1 was used to write this code.
from PIL import Image
# this import line is needed for gordons installer to be sure to find these
from PIL import GifImagePlugin, JpegImagePlugin, PngImagePlugin

# our modules
import DownloadMonitor
import debug
import MojoConstants
from confutils import confman

# 24-bit RGB color values for the images
grey =  0x666666
blue =  0xaa0000
algae = 0x887722
red =   0x0000cc
green = 0x00cc00
darkgreen = 0x009900
cyan =  0xcccc00
magenta = 0xcc00cc
black = 0
white = 0xffffff

UNKNOWN_PIECE_COLOR =     blue
POSSIBLE_PIECE_COLOR =    algae
FAILED_PIECE_COLOR =      red
COMPLETED_PIECE_COLOR =   green
#DISCARDED_PIECE_COLOR =   darkgreen   # NOTE: turn this on if you want to see when SingleReassembly objects are destroyed
DISCARDED_PIECE_COLOR =   green

UNKNOWN_BLOCK_COLOR = blue
STARTED_BLOCK_COLOR = white
FOUND_BLOCK_COLOR = darkgreen
COMPLETED_BLOCK_COLOR = green

class BitmapPieceMonitor(DownloadMonitor.PieceMonitor):
    ##### Interface to be used by anything wanting a bitmap representation of this piece #####
    def get_overall_color(self):
        if self.completed:
            if self.discarded:
                fillcolor = DISCARDED_PIECE_COLOR
            else:
                fillcolor = COMPLETED_PIECE_COLOR
        elif not self.possible:
            fillcolor = FAILED_PIECE_COLOR
        elif self.possible == DownloadMonitor.Unknown:
            fillcolor = UNKNOWN_PIECE_COLOR
        else:
            fillcolor = POSSIBLE_PIECE_COLOR
        return fillcolor

    def get_image_or_color(self, sizetuple):
        fillcolor = self.get_overall_color()
        if fillcolor in (UNKNOWN_PIECE_COLOR, POSSIBLE_PIECE_COLOR):
            completed = len(self.completed_sharenums)
            if completed > 0:
                if completed >= self.num_shares_needed:
                    return COMPLETED_PIECE_COLOR
                im = Image.new('RGB', sizetuple, fillcolor)
                w, h = im.size
                pixels_per_block = h / float(self.num_shares_needed)
                # fill from the bottom up
                toppixel = int(h - completed*pixels_per_block)
                im.paste(COMPLETED_PIECE_COLOR, (0, toppixel, w, h))
                return im
        return fillcolor

class DetailedBitmapPieceMonitor(BitmapPieceMonitor):
    def get_border_color(self):
        if self.completed:
            if self.discarded:
                color = DISCARDED_PIECE_COLOR
            else:
                color = COMPLETED_PIECE_COLOR
        elif not self.possible:
            color = FAILED_PIECE_COLOR
        elif self.possible == DownloadMonitor.Unknown:
            color = UNKNOWN_PIECE_COLOR
        else:
            color = POSSIBLE_PIECE_COLOR
        return color

    border_width = 2
    coords_tables = {}  # keyed on (sizetuple, total_shares) -> maps to a precomputed shnum coordinate tuple table
    def get_image_or_color(self, sizetuple):
        width, height = sizetuple[0] - (2*self.border_width), sizetuple[1] - (2*self.border_width)  # allow for our border
        coords = self.coords_tables.get((sizetuple, self.total_shares))
        if coords is None:
            # only compute the coordinate table for a given piece size & total_shares once.
            coords = []
            blocks_across = float(4)
            blocks_high = round(self.total_shares/blocks_across)
            for shnum in xrange(self.total_shares):
                col = int(shnum % blocks_across)
                x1 = int(round((width / blocks_across) * col))
                x2 = int(round((width / blocks_across) * (col+1)))
                row = int((shnum - col) / blocks_across)
                y1 = int(round((height / blocks_high) * row))
                y2 = int(round((height / blocks_high) * (row+1)))
                coords.append( (x1, y1, x2, y2) )
            self.coords_tables[(sizetuple, self.total_shares)] = coords

        bordercolor = self.get_overall_color()
        borderIm = Image.new('RGB', sizetuple, bordercolor)
        im = Image.new('RGB', (width, height), UNKNOWN_BLOCK_COLOR)
        # fill in individual found blocks
        for blocknum in self.found_sharenums.keys():
            if not self.completed_sharenums.has_key(blocknum) and not self.started_sharenums.has_key(blocknum):
                im.paste(FOUND_BLOCK_COLOR, coords[blocknum])
        # fill in individual started but not completed blocks
        for blocknum in self.started_sharenums.keys():
            if not self.completed_sharenums.has_key(blocknum):
                im.paste(STARTED_BLOCK_COLOR, coords[blocknum])
        # fill in individual completed blocks
        for blocknum in self.completed_sharenums.keys():
            im.paste(COMPLETED_BLOCK_COLOR, coords[blocknum])
        borderIm.paste(im, (self.border_width, self.border_width))
        return borderIm

class BitmapDownloadMonitor(DownloadMonitor.DownloadMonitor):
    _public_methods_ = []
    _public_methods_.extend(DownloadMonitor.DownloadMonitor._public_methods_)   # allow COM access
    _public_attrs_ = []
    _public_attrs_.extend(DownloadMonitor.DownloadMonitor._public_attrs_)
    def __init__(self, name, piecelist, num_shares_needed, piecesacross=32, piecesize=(14, 20), piecespacing=(2, 2)):
        assert 0 < piecesacross < 8000
        assert type(piecesize) is types.TupleType and len(piecesize) == 2
        assert type(piecespacing) is types.TupleType and len(piecespacing) == 2
        if confman.is_true_bool(['DOWNLOAD_MONITOR', 'SHOW_TOO_MUCH_DETAIL']):
            piecesacross = piecesacross / 2
            piecesize = (piecesize[0]*2, piecesize[1]*2)
            piecemonitor = DetailedBitmapPieceMonitor
            self.toomuchdetail = 1
        else:
            piecemonitor = BitmapPieceMonitor
            self.toomuchdetail = 0
        DownloadMonitor.DownloadMonitor.__init__(self, name, piecelist, num_shares_needed, piecemonitor)
        self.piecesacross = piecesacross
        self.piecesize = piecesize
        self.piecespacing = piecespacing

    ##### Interface to be used by anything wanting a bitmap representation of this piece #####
    _public_methods_.append('get_image_size')
    def get_image_size(self):
        """returns a 2-tuple of width, height"""
        numpieces = len(self._piece_order)
        pieceshigh = numpieces / (self.piecesacross+1) + 1
        piecesacross = min(numpieces, self.piecesacross)
        imagesize = ( piecesacross*(self.piecesize[0]+self.piecespacing[0])+self.piecespacing[0],
                      pieceshigh*(self.piecesize[1]+self.piecespacing[1])+self.piecespacing[1]    )
        return imagesize

    def get_image(self, bgcolor=0xcc9900):
        """Returns a PIL Image object"""
        numpieces = len(self._piece_order)
        pieceshigh = numpieces / (self.piecesacross+1) + 1
        piecesacross = min(numpieces, self.piecesacross)
        imagesize = ( piecesacross*(self.piecesize[0]+self.piecespacing[0])+self.piecespacing[0],
                      pieceshigh*(self.piecesize[1]+self.piecespacing[1])+self.piecespacing[1]    )
        im = Image.new('RGB', imagesize, bgcolor)
        debug.mojolog.write('imagesize %s [%sx%s] im = %s\n', args=(imagesize, piecesacross, pieceshigh, im), v=5, vs='BitmapDownloadMonitor')
        row = 0
        col = 0
        for row in xrange(pieceshigh):
            for col in xrange(piecesacross):
                piecenum = row*piecesacross + col
                piece_img_x = self.piecespacing[0] + (col*(self.piecesize[0]+self.piecespacing[0]))
                piece_img_y = self.piecespacing[1] + (row*(self.piecesize[1]+self.piecespacing[1]))
                #debug.mojolog.write('piecenum %s row, col = %s, %s\n', args=(piecenum, row, col), v=2, vs='BitmapDownloadMonitor')
                im.paste(self._piece_order[piecenum].get_image_or_color(self.piecesize), (piece_img_x, piece_img_y, piece_img_x+self.piecesize[0], piece_img_y+self.piecesize[1]))
                numpieces = numpieces - 1
                if numpieces <= 0:
                    return im
        return im
    
    _public_methods_.append('write_image')
    def write_image(self, file, format="gif", bgcolor=0xcc9900):
        """
        @param file is a filename or an open file object implementing the seek, tell and write calls.
        """
        im = self.get_image(bgcolor=bgcolor)
        im.save(file, format)
    
    # TODO for possible use in a windows GUI, investigate the ImageWin module and using ImageWin.Dib?
    # http://www.pythonware.com/library/pil/handbook/imagewin.htm


def test_basic_and_get_image():
    bdm = BitmapDownloadMonitor(name="test", num_shares_needed=4, piecelist=[{'pieceid': 'A'*27, 'length': 42, 'shares': {0:'A'*27}}])
    im = bdm.get_image()

def test_edge_case_not_too_tall():
    piecelist = [
        {'pieceid': 'a'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'b'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'c'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'd'*20, 'length': 42, 'shares': {0:'A'*20}},
    ]
    bdm = BitmapDownloadMonitor(name="test", num_shares_needed=4, piecelist=piecelist, piecesize=(16, 30), piecespacing=(1, 1), piecesacross=4)
    im = bdm.get_image()
    assert im.size == ((16+1)*4 + 1, (30+1)*1 + 1), "incorrect resulting image size %d,%d" % im.size

def test_edge_case_not_too_tall_2():
    piecelist = [
        {'pieceid': 'a'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'b'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'c'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'd'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'e'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'f'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'g'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'h'*20, 'length': 42, 'shares': {0:'A'*20}},
    ]
    bdm = BitmapDownloadMonitor(name="test", num_shares_needed=4, piecelist=piecelist, piecesize=(16, 30), piecespacing=(1, 1), piecesacross=4)
    im = bdm.get_image()
    assert im.size == ((16+1)*4 + 1, (30+1)*2 + 1), "incorrect resulting image size %d,%d" % im.size

def test_piece_spacing():
    piecelist = [
        {'pieceid': 'a'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'b'*20, 'length': 42, 'shares': {0:'A'*20}},
    ]
    bdm = BitmapDownloadMonitor(name="test", num_shares_needed=4, piecelist=piecelist[:1], piecesize=(16, 30), piecespacing=(3, 10))
    im = bdm.get_image()
    assert im.size == ((16+3)*1 + 3, (30+10)*1 + 10), "incorrect resulting image size %d,%d" % im.size
    bdm = BitmapDownloadMonitor(name="test", num_shares_needed=4, piecelist=piecelist, piecesize=(16, 30), piecespacing=(3, 10), piecesacross=1)
    im = bdm.get_image()
    assert im.size == ((16+3)*1 + 3, (30+10)*2 + 10), "incorrect resulting image size %d,%d" % im.size

def _disabled_test_show():
    # NOTE: leave this disabled by default, it launches an image viewer (not suitable for automatic testing)
    piecelist = [
        {'pieceid': 'a'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'b'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'c'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'd'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'e'*20, 'length': 42, 'shares': {0:'A'*20}},
        {'pieceid': 'f'*20, 'length': 42, 'shares': {0:'A'*20}},
    ]
    bdm = BitmapDownloadMonitor(name="test", num_shares_needed=4, piecelist=piecelist, piecesize=(24, 40), piecesacross=4, piecespacing=(2, 2))
    bdm._piece_order[0].possible = DownloadMonitor.true
    bdm._piece_order[0].completed = DownloadMonitor.true
    bdm._piece_order[1].possible = DownloadMonitor.true
    bdm._piece_order[2].possible = DownloadMonitor.true
    bdm._piece_order[2].completed_sharenums = {0:0, 3:3, 2:2}
    bdm._piece_order[3].possible = DownloadMonitor.false
    bdm._piece_order[3].discarded = DownloadMonitor.true
    bdm._piece_order[4].possible = DownloadMonitor.true
    bdm._piece_order[4].discarded = DownloadMonitor.true
    bdm._piece_order[4].completed = DownloadMonitor.true
    bdm._piece_order[4].discarded = DownloadMonitor.true
    im = bdm.get_image(bgcolor=0xcc9900)
    im.show()


mojo_test_flag = 1
def run():
    import RunTests
    RunTests.runTests("BitmapDownloadMonitor")

if __name__ == '__main__':
    run()
