#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
__cvsid = '$Id: BlockServerHandicapper.py,v 1.8 2001/06/07 17:52:26 zooko Exp $'


# standard modules
import math
import time

# our modules
import MetaTrackerLib
import MojoHandicapper
import confutils
import debug
import idlib
import idmasklib
import mojoutil



PENALTY_FOR_MASK_EXCLUSION=10000

# prefer servers with narrower ranges
def compute_handicap(counterparty_id, metainfo, message_type, message_body):
    """
    for msgtype in ('request blob', 'put blob')

    @returns 0-20, or 10000 if mask excludes blob;  Note that default bitmask is 7.
    """
    if message_type not in ('do you have blobs', 'request blob', 'put blob'):
        return 0 # no preference

    # this block of code is a modification of the correction suggested by donwulff on mojonation-devel
    bits = 0
    for service in metainfo.get('services', [{}]):
        if (service is not None) and (service.get('type') == 'blob server'):
            for mask in service.get('mask list', []):
                bits = min(20, min(bits, int(mask.get('bits', 0))))
            break # assume there is only one 'blob server' service (as is proper and as the rest of the code does)

    if message_type == 'do you have blobs':
        return (20 - bits) * 10  # prefer more specialized servers (those with fewer bits in their bitmask)

    if message_type in ('request blob', 'put blob',):
        blobid = message_body.get('blob id', '')

        if idlib.is_sloppy_id(blobid):
            blobidmask = idmasklib.IdMask(mask=idlib.canonicalize(blobid, 'block'), bits=160)
            if not MetaTrackerLib.blobserver_covers_mask_list(metainfo, [blobidmask]):
                # return MojoHandicapper.DISQUALIFIED
                # It's possible that the block server actually _does_ have the block, but he has changed his mask, or he is doing some kind of publishing trickery.
                return PENALTY_FOR_MASK_EXCLUSION

            # XXX this assumes only one mask in masklist  --Zooko 2000-10-04
            return (20 - bits) * 100  # prefer more specialized servers (those with fewer bits in their bitmask)

    return 0 # no preference

