#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

from PriorityList import PriorityList
import debug
true = 1
false = 0

class BlockThrottler:
    """
    Does block purchasing

    Does not check the cache - that should be done by the caller

    Limits the number of parallel block downloads to a fixed number

    The BlockFinder must have been called previously, or purchasing will probably fail
    """

    def __init__(self, doq, max_concurrent):
        self.doq = doq
        self.max_concurrent = max_concurrent
        # contains (server id, block id)
        self.current_downloads = []
        # contains reassemblers
        self.downloads = PriorityList()

    def do_download(self, reassembly):
        assert self.doq.is_currently_doq()
        if reassembly.is_finished():
            debug.mojolog.write("%s finished or cancelled, forgetting\n", args=(reassembly,), v=3, vs="BlockThrottler")
            self.downloads.forget(reassembly)
        else:
            self.downloads.set(reassembly, reassembly.get_priority())
            self._start_single(reassembly)

    def download_done(self):
        assert self.doq.is_currently_doq()
        self._start_new()

    def _start_new(self):
        next = self.downloads.highest()
        while next is not None and len(self.current_downloads) < self.max_concurrent:
            self._start_single(next)
            next = self.downloads.after(next)

    def _start_single(self, reassembly):
        while len(self.current_downloads) < self.max_concurrent:
            if not reassembly.start_next(self.current_downloads):
                return

mojo_test_flag = 1

if __name__ == '__main__':
    RunTests.runTests('BlockThrottler')

