#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

import exceptions
import cStringIO


import SingleThreader


# just for testing
import random
import array 
import RunTests


class Error(exceptions.StandardError):
    pass

class DumbBuf(SingleThreader.SingleThreader):
    def __init__(self, size=8192):
        SingleThreader.SingleThreader.__init__(self)
        self._buf = ''

    def read(self):
        return self._buf
    
    def chop(self, num):
        self._buf = self._buf[num:]

    def len(self):
        return len(self._buf)

    def append(self, str):
        self._buf = self._buf + str


class CircBuf(SingleThreader.SingleThreader):
    def __init__(self, size=8192):
        """
        @param size the number of bytes that can be stored in the buffer; (note that one more byte will be allocated for internal use)
        """
        SingleThreader.SingleThreader.__init__(self)

        self._size = size + 1
        self._buf = cStringIO.StringIO()

        # When writeptr == readptr, the buffer is empty.
        # Note: this means that you can only store size - 1 characters...
        self._writeptr = 0
        self._readptr = 0
        self._used = 0

    def read(self):
        """
        @returns a string containing all of the available data
        """
        self._enter()
        try:
            self._buf.seek(self._readptr)
            if self._writeptr > self._readptr:
                return self._buf.read(self._writeptr - self._readptr)
            else:
                str = self._buf.read()
                self._buf.seek(0)
                str = str + self._buf.read(self._writeptr)
                return str
        finally:
            self._exit()
    
    def chop(self, num):
        self._enter()
        try:
            self._readptr = (self._readptr + num) % self._size
            self._used = self._used - num
        finally:
            self._exit()

    def len(self):
        self._enter()
        try:
            return self._used
        finally:
            self._exit()

    def append(self, str):
        self._enter()
        try:
            self._buf.seek(self._writeptr)
            lenstr = len(str)
            if (lenstr + self._used) > self._size:
                raise "CircBuffer full!"
            self._used = self._used + lenstr

            newwriteptr = self._writeptr + lenstr
            if newwriteptr < self._size:
                # Write it.
                self._buf.write(str)
                self._writeptr = self._writeptr + lenstr
            else:
                # Write it in two chunks.
                self._buf.write(str[:(self._size - self._writeptr)])
                self._buf.seek(0)
                self._buf.write(str[(self._size - self._writeptr):])
                self._writeptr = newwriteptr - self._size
        finally:
            self._exit()


def _help_test_append_and_read_CircBuf(leng):
    c = CircBuf(leng)

    b = ''
    for i in range(leng):
        b = b + chr(random.randint(0, 255))

    i = 0
    while i < leng:
        oldi = i
        i = random.randint(oldi, leng)
        c.append(b[oldi:i])

    newb = c.read()

    assert newb == b, "newb: %s, b: %s" % (`newb`, `b`)
   

def test_CircBuf_a_lot():
    for i in range(100):
        _help_test_append_and_read_CircBuf(i)
    

# Generic stuff
NAME_OF_THIS_MODULE="Buf"

mojo_test_flag = 1

def run():
    RunTests.runTests(NAME_OF_THIS_MODULE)

#### this runs if you import this module by itself
if __name__ == '__main__':
    run()


