# These handicappers are related to Content meta-info/descriptions.

# Mojo Modules:
import contentlib
import mencode

def keyword_specialization_handicapper(cpid, metainfo, mtype, mbody):
    """
    This handicapper only applies to content searches.

    It compares the hash of each white-space delimited word in a search
    query with the counterparty id (cpid).  The end-result, we hope is
    that content tracking becomes robustly distributed.
    """
    if not mtype in ('content tracker lookup', 'content tracker submit'):
        return 0 # no preference

    wordscores = [] # This stores words, mencodes, hashes, scores, etc...

    # Create field/word dicts:
    for field, sentence in contentlib.content_xml_to_dict(mbody['XML data']).items():
        for word in string.split(sentences):
            # We store both the field and the word so that
            # "Title: 128" is different than "Encoding Speed: 128".
            wordscores.append({field : word})

    # mencode the values
    wordscores = map(mencode.mencode, wordscores)

    # hash the mencoded strings
    wordscores = map(lambda s : sha.new(s).digest(), wordscores)

    # count the common substring length (in bits)
    wordscores = map(
        lambda a, b, : mojoutil.common_substring_length(a,b),
        wordscores,
        (cpid,) * len(wordscores)
        )

    # square root (many common keywords is a better score than a single
    # super specialized keyword)
    wordscores = map(math.sqrt, wordscores)

    # "flip" the values from a merit score to a demerit handicap:
    # 8**2 == 64 bits  (matches greater than 64 bits are flattened...
    # this is a very unlikely event.)
    wordscores = map(lambda v : max(8.0 - v, 0), wordscores)

    # return the sum:
    return reduce(lambda a, b : a+b, wordscores, 0)
