#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

import RunTests
from copy import copy

class CounterHash:
    def __init__(self):
        # maps key to int
        self.counts = {}
        self.zero_counts = []

    def get_count(self, key):
        if not self.counts.has_key(key):
            return 0
        return self.counts[key]

    def start_counting(self, key):
        if not self.counts.has_key(key):
            self.counts[key] = 0
            self.zero_counts.append(key)

    def decrement_count(self, key):
        if not self.counts.has_key(key) or self.counts[key] == 0:
            return
        self.counts[key] = self.counts[key] - 1
        if self.counts[key] == 0:
            self.zero_counts.append(key)
    
    def decrement_count_and_delete_if_zero(self, key):
        if not self.counts.has_key(key):
            return
        if self.counts[key] == 0:
            del self.counts[key]
            self.zero_counts.remove(key)
        else:
            self.counts[key] = self.counts[key] - 1
            if self.counts[key] == 0:
                del self.counts[key]

    def set_count_to_zero_if_present(self, key):
        if self.counts.has_key(key) and self.counts[key] > 0:
            self.counts[key] = 0
            self.zero_counts.append(key)
    
    def start_counting_and_increment(self, key):
        if self.counts.has_key(key):
            if self.counts[key] == 0:
                self.zero_counts.remove(key)
            self.counts[key] = self.counts[key] + 1
        else:
            self.counts[key] = 1

    def is_counting(self, key):
        return self.counts.has_key(key)

    def delete(self, key):
        if self.counts.get(key) == 0:
            self.zero_counts.remove(key)
        if self.counts.has_key(key):
            del self.counts[key]

    def get_all_zero_counts(self):
        return copy(self.zero_counts)

def test_operation():
    t = CounterHash()
    assert t.pop_zero_count() == None
    assert not t.is_counting('a')
    t.start_counting('a')
    assert t.is_counting('a')
    assert t.pop_zero_count() == 'a'
    assert not t.is_counting('a')
    assert t.pop_zero_count() == None
    assert not t.is_counting('a')

    t.start_counting('a')
    assert t.is_counting('a')
    t.increment_count('a')
    assert t.is_counting('a')
    assert t.pop_zero_count() == None
    t.decrement_count('a')
    assert t.is_counting('a')
    assert t.pop_zero_count() == 'a'
    assert not t.is_counting('a')
    assert t.pop_zero_count() == None
    assert not t.is_counting('a')
    t.start_counting_and_increment('b')
    assert t.is_counting('b')
    assert t.pop_zero_count() == None
    t.increment_count('b')
    t.decrement_count('b')
    t.decrement_count('b')
    assert t.pop_zero_count() == 'b'
    assert not t.is_counting('b')
    assert t.pop_zero_count() == None
    t.start_counting_and_increment('c')
    t.increment_count('c')
    t.decrement_count_and_delete_if_zero('c')
    t.decrement_count_and_delete_if_zero('c')
    assert t,pop_zero_count() == None
    assert not t.is_counting('c')

mojo_test_flag = 1

if __name__ == '__main__':
    RunTests.runTests('CounterHash')









