#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

from human_readable import hr
from MojoErrors import BadFormatError
import types
import mojosixbit
from human_readable import hr
from idlib import is_sloppy_id, is_canonical_uniq, is_mojosixbitencoded_id
import MojoConstants
from MojoConstants import true, false


def NONEMPTY(thing, verbose):
    pass

def ANY(thing, verbose):
    if thing is None:
        raise BadFormatError, "any can't be none"

def STRING(thing, verbose):
    if type(thing) != types.StringType:
        raise BadFormatError, "not a string"

def BOOLEAN(thing, verbose):
    if thing != 'true' and thing != 'false':
        raise BadFormatError, "not 'true' or 'false'"

def INTEGER(thing, verbose):
    if (type(thing) != types.StringType) and (type(thing) != types.IntType) and (type(thing) != types.LongType):
        raise BadFormatError, "not an integer (not a string, int or long)"

    if type(thing) == types.StringType:
        try:
            num = long(thing)
        except ValueError:
            raise BadFormatError, "not an integer"

        canonicalstr = `num`
        # remove any annoying python 1.5.2 Ls (1.6 doesn't do this)
        if canonicalstr[-1] == 'L':
            canonicalstr = canonicalstr[:-1]

        if canonicalstr != thing:
            raise BadFormatError, "not an integer (not canonical)"

        if long(canonicalstr) != num:
            raise BadFormatError, "not an integer (not canonical)"

def NON_NEGATIVE_INTEGER(thing, verbose):
    if (type(thing) != types.StringType) and (type(thing) != types.IntType) and (type(thing) != types.LongType):
        raise BadFormatError, "not an integer (not a string, int or long)"

    if type(thing) != types.StringType:
        if thing < 0:
            raise BadFormatError, "not a non-negative integer (not non-negative)"
    else:
        try:
            num = long(thing)
        except ValueError:
            raise BadFormatError, "not an integer"

        canonicalstr = `num`
        # remove any annoying python 1.5.2 Ls (1.6 doesn't do this)
        if canonicalstr[-1] == 'L':
            canonicalstr = canonicalstr[:-1]

        if canonicalstr != thing:
            raise BadFormatError, "not an integer (not canonical)"

        if long(canonicalstr) != num:
            raise BadFormatError, "not an integer (not canonical)"

        if num < 0:
            raise BadFormatError, "not a non-negative integer (not non-negative)"

def ASCII_ARMORED_DATA(thing, verbose):
    if type(thing) != types.StringType:
        raise BadFormatError, "not proper ascii-armored data - not a string"
    try:
        if not len(mojosixbit.a2b(thing)) > 0:
            raise BadFormatError, 'zero-length strings are rejected by ascii armored data match'
    except mojosixbit.Error, reason:
        raise BadFormatError, str(reason)
    ### !!!!! XXXXXX need to make this canonical!  --Zooko 2000-08-20

def UNIQUE_ID(thing, verbose):
    if type(thing) != types.StringType or not is_sloppy_id(thing):
        raise BadFormatError, "not an unique id"

def ASCII_ID(thing, verbose):
    if type(thing) != types.StringType or not is_mojosixbitencoded_id(thing):
        raise BadFormatError, "not an ascii id"

def is_canonical_mod_val(thing):
    if type(thing) is not types.StringType:
        return false
    if MojoConstants.DEBUG_MODE:
        # This is to enable testing with 82 byte pubkeys while still having some hardcoded 128 byte pubkeys..  --Zooko 2000-09-12.
        return (len(thing) == MojoConstants.SIZE_OF_MODULAR_VALUES) or (len(thing) == 128)

    return len(thing) == MojoConstants.SIZE_OF_MODULAR_VALUES

def MOD_VAL(thing, verbose):
    if type(thing) != types.StringType:
        raise BadFormatError, "not proper modval - not a string"
    try:
        if not is_canonical_mod_val(mojosixbit.a2b(thing)):
            raise BadFormatError, "not a proper modval"
    except mojosixbit.Error:
        raise BadFormatError, "not a proper modval - not even proper ascii-encoded data"

def BINARY_SHA1(thing, verbose):
    if type(thing) != types.StringType:
        raise BadFormatError, "not proper binary sha1 - not a string"
    if not is_canonical_uniq(thing):
        raise BadFormatError, "not a sha1 value - it does not have a length of 20" 

def FLOAT(thing, verbose):
    if type(thing) != types.StringType:
        raise BadFormatError, "not a float (not a string)"
    
    try:
        num = float(thing)
    except ValueError:
        raise BadFormatError, "not a float"

    canonicalstr = `num`

    if canonicalstr != thing:
        raise BadFormatError, "not a float (not canonical)"

    if float(canonicalstr) != num:
        raise BadFormatError, "not a float (not canonical)"

def NON_NEGATIVE_FLOAT(thing, verbose):
    if type(thing) != types.StringType:
        raise BadFormatError, "not a non-negative float (not a string)"
    
    try:
        num = float(thing)
    except ValueError:
        raise BadFormatError, "not a non-negative float"

    canonicalstr = `num`

    if canonicalstr != thing:
        raise BadFormatError, "not a non-negative float (not canonical)"

    if float(canonicalstr) != num:
        raise BadFormatError, "not a float (not canonical)"

    if num < 0.0:
        raise BadFormatError, "not a non-negative float (not non-negative)"

FLOAT_OR_INTEGER = [FLOAT, INTEGER]

class OptionMarker :
    def __init__(self, template) :
        self.template = template
    def __repr__(self):
        return "OptionMarker: <%s>" % hr(self.template)

def AndMarker(templs):
    """
    Require the thing to match all templates.
    """
    def func(thing, verbose, templs = templs):
        for templ in templs:
            inner_check(thing, templ, verbose)
    return func

def NotMarker(template):
    def func(thing, verbose, template = template):
        try:
            inner_check(thing, template, verbose)
        except BadFormatError:
            return
        raise BadFormatError, "got match when should not have"
    return func

NOT_PRESENT = OptionMarker(NotMarker(NONEMPTY))

def ListMarker(template):
    def func(thing, verbose, template = template):
        if type(thing) != types.ListType and type(thing) != types.TupleType:
            raise BadFormatError, 'not a list'
        if verbose:
            try:
                i = 0
                while i < len(thing):
                    inner_check(thing[i], template, 1)
                    i = i + 1
            except BadFormatError, reason:
                reason = 'mismatch at index ' + str(i) + ': ' + str(reason)
                raise BadFormatError, reason
        else:
            for i in thing :
                inner_check(i, template, 0)
    return func

def is_template_matching(thing, templ):
    try:
        checkTemplate(thing, templ)
    except BadFormatError:
        return false
    return true
       
def checkTemplate(thing, templ):
    """
    throws BadFormatError if the thing does not match the template
    """
    try:
        inner_check(thing, templ, false)
        return
    except BadFormatError, reason:
        pass
    try:
        inner_check(thing, templ, true)
    except BadFormatError, reason:
        reason = 'failed template check because: (' + str(reason) + ') template was: (' + hr(templ) + ') target was: (' + hr(thing) + ')'
        raise BadFormatError, reason

def inner_check(thing, templ, verbose):
    # The following isn't really used right now, but I'm leaving the commented-out code for evidence.  --Zooko 2001-06-07
    # if isinstance(thing, mencode.PreEncodedThing):
    #     thing = thing.getvalue()

    if type(templ) == types.FunctionType or type(templ) == types.MethodType:
        templ(thing, verbose)
    elif type(templ) == types.DictType:
        if not type(thing) == types.DictType:
            raise BadFormatError, 'target is not a dict'

        for key in templ.keys():
            if not thing.has_key(key):
                if not isinstance(templ[key], OptionMarker) :
                    if verbose:
                        reason = "lacks required key: (" + hr(key) + ")"
                        raise BadFormatError, reason
                    else:
                        raise BadFormatError, "lacks required key"
            else:
                try:
                    if isinstance(templ[key], OptionMarker) :
                        inner_check(thing[key], templ[key].template, verbose)
                    else :
                        inner_check(thing[key], templ[key], verbose)
                except BadFormatError, reason:
                    if verbose:
                        reason = 'mismatch in key (' + hr(key) + '): ' + str(reason)
                        raise BadFormatError, reason
                    else:
                        raise
    elif type(templ) == types.StringType:
        if type(thing) != types.StringType:
            raise BadFormatError, "no match - target is not a string"
        if thing != templ:
            if verbose:
                raise BadFormatError, "strings (" + thing + ') and (' + templ + ') do not match'
            else:
                raise BadFormatError, "strings do not match"
    elif templ == 0 or templ == -1 or templ == 1:
        if type(thing) is not types.LongType and type(thing) is not types.IntType:
            raise BadFormatError, 'expected int'
        if templ == 0:
            if thing < 0:
                raise BadFormatError, 'template called for non-negative value'
        elif templ == -1:
            return
        else:
            assert templ == 1
            if thing <= 0:
                raise BadFormatError, 'template called for strictly positive value'
    elif type(templ) == types.ListType or type(templ) == types.TupleType:
        if verbose:
            failure_reason = 'did not match any of the ' + str(len(templ)) + ' possible templates;'
            index = -1
            for i in templ:
                try:
                    index = index + 1
                    inner_check(thing, i, verbose)
                    return
                except BadFormatError, reason:
                    failure_reason = failure_reason + ' failed template at index ' + str(index) + ' because (' + str(reason) + ')'
            raise BadFormatError, failure_reason
        else:
            for i in templ:
                try:
                    inner_check(thing, i, verbose)
                    return
                except BadFormatError:
                    pass
        raise BadFormatError, "did not match any possible templates"
    elif templ is None:
        if thing is not None:
            raise BadFormatError, 'expected None'
    else:
        assert false, "bad template - " + hr(templ)

def test_not_on_rejection():
    checkTemplate('a', NotMarker('b'))

def test_not_on_acceptance():
    try:
        checkTemplate('a', NotMarker('a'))
        return 0
    except BadFormatError:
        return 1

def test_uses_function():
    spam = []
    checkTemplate(3, lambda thing, verbose, spam = spam: spam.append(thing))
    assert len(spam) == 1 and spam[0] == 3

def test_none_passes_none():
    checkTemplate(None, None)

def test_none_rejects_string():
    try:
        checkTemplate('spam', None)
    except BadFormatError:
        return 1

def test_check_positive_accepts_positive():
    checkTemplate(2, 1)

def test_check_positive_rejects_zero():
    try:
        checkTemplate(0, 1)
    except BadFormatError:
        return 1
    
def test_check_positive_rejects_negative():
    try:
        checkTemplate(-2, 1)
    except BadFormatError:
        return 1

def test_check_int_passes():
    checkTemplate(3, -1)
    checkTemplate(-2, -1)
    checkTemplate(0, -1)

def test_check_int_fails_string():
    try:
        checkTemplate('spam', -1)
    except BadFormatError:
        return 1    

def test_check_nonnegative_accepts_positive():
    checkTemplate(2, 0)
    
def test_check_nonnegative_accepts_zero():
    checkTemplate(0, 0)

def test_check_nonnegative_rejects_negative():
    try:
        checkTemplate(-2, 0)
    except BadFormatError:
        return 1

# checks both items of a list passing
def test_check_list_pass() :
    template = ListMarker({"a" : ANY})
    thing = [{"a" : "a"},{"a" : "a"}]
    checkTemplate(thing, template)

# checks the second item of a list failing
def test_check_list_fail() :
    template = ListMarker({"a" : ANY})
    thing = [{"a" : "a"},{"b" : "a"}]
    try :
        checkTemplate(thing, template)
        return 0
    except BadFormatError :
        return 1

def test_check_option_accepts_nonexistent() :
    template = {'spam' : OptionMarker('eggs')}
    thing = {}
    checkTemplate(thing, template)
    
def test_check_option_rejects_exists_with_none() :
    template = {'spam' : OptionMarker('eggs')}
    thing = {'spam' : None}
    try :
        checkTemplate(thing, template)
    except BadFormatError :
        return 1
    
def test_check_option_accepts_valid() :
    template = {'spam' : OptionMarker('eggs')}
    thing = {'spam' : 'eggs'}
    checkTemplate(thing, template)
    
def test_check_option_rejects_invalid() :
    template = {'spam' : OptionMarker('eggs')}
    thing = {'spam' : 'bacon'}
    try :
        checkTemplate(thing, template)
    except BadFormatError :
        return 1

# checks formatting against a non-list
def test_check_bad_list() :
    template = ListMarker({"a" : ANY})
    thing = 'spam'
    try :
        checkTemplate(thing, template)
        return 0
    except BadFormatError :
        return 1

# use list of matches none of which match (although combined pieces do)
def test_check_multiple_fail() :
    template = (
        {"a" : ANY, "b" : ANY},
        {"c" : ANY, "d" : ANY}
        )
    thing = {"a" : "spam", "c" : "eggs"}
    try :
        checkTemplate(thing, template)
        return 0
    except BadFormatError :
        pass
    
# use list of templates the first of which matches
def test_check_multiple_pass_first() :
    template = (
        {"a" : ANY, "b" : ANY},
        {"c" : ANY, "d" : ANY}
        )
    thing = {"a" : "spam", "b" : "eggs"}
    try :
        checkTemplate(thing, template)
        return
    except BadFormatError :
        return 0
    
# use list of two templates the second of which matches
def test_check_multiple_pass_second() :
    template = (
        {"a" : ANY, "b" : ANY},
        {"c" : ANY, "d" : ANY}
        )
    thing = {"c" : "spam", "d" : "eggs"}
    try :
        checkTemplate(thing, template)
        return
    except BadFormatError :
        return 0
    
# doesn't match first depth but does second
def test_check_multiple_in_second_depth_pass() :
    template = {"a" : ({"c" : ANY}, {"d" : ANY})}
    thing = {"a" : {"d" : "spam"}}
    try :
        checkTemplate(thing, template)
        return
    except BadFormatError :
        return 0
    
# matches first depth but not second
def test_check_multiple_in_second_depth_fail() :
    template = {"a" : ({"c" : ANY}, {"d" : ANY})}
    thing = {"a" : {"a" : "spam"}}
    try :
        checkTemplate(thing, template)
        return 0
    except BadFormatError :
        return 1
    
mojo_test_flag = 1

def run():
    import RunTests
    RunTests.runTests(["DataTypes"])

if __name__ == '__main__':
    run()

