#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

import mojoutil
import CommStrat


import exceptions


 # Useful for testing -- it has a handy "push incoming message in the bottom" function.

# Note that "ids" are just not used in this case -- these two dummies always send messages to one another, regardless of what id you ask them to send it.


def create_two_linked_dummies():
    d1 = DummyCommsHandler(idlib.new_random_uniq())
    d2 = DummyCommsHandler(idlib.new_random_uniq())

    d1.link_to_other_dummy(d2)
    d2.link_to_other_dummy(d1)

    return (d1, d2)


class DummyCommsHandler:
    def __init__(self, myid):
        self._myid = myid

    def link_to_other_dummy(self, otherdummy):
        self._otherdummy = otherdummy

    def send_msg(self, counterparty_id, msg):
        """
        Non-blocking.
        """
        self._otherdummy.push_incoming_message_in_the_bottom(self._myid, msg)

    def start_listening(self, inmsg_handler_func, listenport=None):
        """
        Start trying to get incoming messages which you will then pass to `inmsg_handler_func()'.
        """
        self._upward_inmsg_handler = inmsg_handler_func

    def push_incoming_message_in_the_bottom(self, counterparty_id, msg):
        self._upward_inmsg_handler(counterparty_id, msg, commstrat=CommStrat.Dummy())

    def use_comm_strategy(self, counterparty_id, strategy, orig_cs=None) :
        return # Whatever, dude!

    def get_comm_strategy(self):
        return CommStrat.Dummy() # Whatever, dude!

    def get_id(self):
        return self._myid
