#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
# This library provides a common library for all meta/id tracker
# lookups.  It caches the response values.
#
__cvsid = '$Id: MetaTrackerLib.py,v 1.106.2.1 2001/06/11 03:29:10 zooko Exp $'

# standard modules
import string
import types
import traceback
import copy
import time
import traceback
import whrandom
import threading
import cPickle
pickle = cPickle
import types

# our modules
import BidStrategy
import Cache
import CommStrat
import MojoKey
import MojoConstants
from MojoConstants import true, false
import MojoErrors
import MojoMessage
import MojoTransaction
import OurMessages
import SEXP
from confutils import confman
import debug
from human_readable import  hr
import idlib
import idmasklib
from idmasklib import IdMask, IdMaskError
import mencode
import mojobootpage
import mojosixbit
import mojoutil
import pricelib

# if you use this during a test, be sure to set it back to none afterwards.  please use
# a try: finally: to do this!
_fake_root_meta_info_for_testing = None

DEFAULT_TIMEOUT = 60 # how many seconds to wait

LONG_TIMEOUT = DEFAULT_TIMEOUT * 3 # how many seconds to wait when waiting for a response from the metatracker

def find_root_meta_trackers(MTM, max=None) :
    """
    Returns the known root meta trackers in random order.  This is pretty slow

    doesn't block
    """
    def dictToLists(thedict):
        if type(thedict) == type({}):
            if type(thedict['connection strategies']) == type({}) :
                thedict['connection strategies'] = SEXP.dictToList(thedict['connection strategies'])
            if type(thedict['services']) == type({}) :
                thedict['services'] = SEXP.dictToList(thedict['services'])
        return thedict
    
    if _fake_root_meta_info_for_testing:
        debug.stderr.write('Using _fake_root_meta_info_for_testing.\n')
        return [_fake_root_meta_info_for_testing]
    
    retval = []
    trackerlist = None
    try:
        trackerlist = copy.copy(confman.dict['MULTI_ROOT_ID_TRACKER_CONTACT_INFO'])
        if type(trackerlist) == type(""):
            trackerlist = eval(trackerlist)
    except KeyError:
        pass

    if trackerlist != None:
        for rootinfo in trackerlist:
            rootinfo = dictToLists(rootinfo)
            retval.append((idlib.string_to_id(mencode.mencode(rootinfo['connection strategies'][0]['pubkey'])), rootinfo))
    else:
        rootinfo = copy.copy(confman.dict["ROOT_ID_TRACKER_CONTACT_INFO"])
        rootinfo = dictToLists(rootinfo)
        retval.append((idlib.string_to_id(mencode.mencode(rootinfo['connection strategies'][0]['pubkey'])), rootinfo))

    assert len(retval) > 0, "No Metatrackers found."

    if max == None:
        max = len(retval)

    # XXX This is a crazy place to put this, but I don't see where else to put it...  --Zooko 2001-05-05
    for k, v in retval:
        if not PartyInfo_id_cache.has_key(k):
            PartyInfo_id_cache[k] = v

    # right now we are returning metatrackers in random order
    return mojoutil.shuffleList(retval)[:max]

def find_meta_trackers(MTM, root=0, min=1, max=None) :
    """
    Returns a list of tuples containing (id, meta info dict) for the meta trackers.

    @dontblock The NewParallelBlockFetcher depends on this function to never block.
    """
    return find_root_meta_trackers(MTM, max)

    # don't do this until we implement better multi meta tracker support
    # XXX meta_trackers = __get_any_meta_info(MetaTracker_id_cache, __lookup_meta_trackers, MTM, min=min, max=max)
    # XXX call business logic here to sort/prune this list based on our preferences
    # XXX return meta_trackers.items()


__cache_timeout = int(confman.dict["METATRACKERLIB"]["CACHE_INFO_TIMEOUT"])
if __cache_timeout < 600 :
    __cache_timeout = 600
    debug.stderr.write("MetaTrackerLib CACHE_INFO_TIMEOUT too small, raising to a minimum of 600\n")
    confman.dict["METATRACKERLIB"]["CACHE_INFO_TIMEOUT"] = str(__cache_timeout)
    confman.save()
debug.stderr.write("MetaTrackerLib cache timeout: %d seconds\n" % __cache_timeout)

# XXX base the expiretime reported by the meta tracker we used as the expiretime for each entry (requires
# implementing a StatsCache variant that allows items to be inserted with a non-zero age so they timeout
# sooner).  -greg
# Please do not access these directly without a good reason.  Use the
# get_foo_servers() methods below, they're what most callers really want.
PartyInfo_id_cache = Cache.StatsCache(maxitems=5000, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})
BlobServer_id_cache = Cache.StatsCache(maxitems=1000, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})
DinodeTracker_id_cache = Cache.StatsCache(maxitems=500, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})
ContentTracker_id_cache = Cache.StatsCache(maxitems=500, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})
MetaTracker_id_cache = Cache.StatsCache(maxitems=500, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})
RelayServer_id_cache = Cache.StatsCache(maxitems=500, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})
PublicationAgent_id_cache = Cache.StatsCache(maxitems=500, autoexpireinterval=300, autoexpireparams={'maxage':__cache_timeout})

# a negative cache of ids that we failed to lookup very recently (in the last 240 seconds)
# (It is possible for an id to be both in here and in a known-party cache -- you only look at this cache if you have already failed to find the id in your known-party caches.)
UnknownParty_id_cache = Cache.StatsCache(maxitems=1000, autoexpireinterval=30, autoexpireparams={'maxage':240})

def metainfo_dict_to_id(info_dict):
    return idlib.make_id(CommStrat.dict_to_strategy(info_dict['connection strategies'][0])._pubkey)

def create_hello_response_from_hellobody_for_testing(hellobody) :
    """mangle a hello message into the meta tracker response format needed to store it in the cache(s)"""
    cached = {
        'result': 'success',
        'connection strategies': copy.deepcopy(hellobody['connection strategies']),
    }
    services = hellobody.get('services')
    if services is not None:
        cached['services'] = copy.deepcopy(services)
    return cached

def stuff_hello_into_cache_for_testing(hellobody) :
    """mangle a hello message into the meta tracker response format needed to store it in the cache(s)"""
    cache_meta_tracker_responses([ create_hello_response_from_hellobody_for_testing(hellobody) ])


def cache_meta_tracker_responses(meta_tracker_response_list) :
    """
    Given a list of responses returned from a meta/id tracker lookup,
    index the response into our caches of known meta tracking info.

    @precondition `meta_tracker_response_list' must be a list.: type(meta_tracker_response_list) == types.ListType: "meta_tracker_response_list: %s" % hr(meta_tracker_response_list)
    """
    assert type(meta_tracker_response_list) == types.ListType or type(meta_tracker_response_list) == types.TupleType, "`meta_tracker_response_list' must be a list." + " -- " + "meta_tracker_response_list: %s" % hr(meta_tracker_response_list)

    for party_info in meta_tracker_response_list :
        if not party_info.get('connection strategies') :
            continue  # useless party info, skip it
        if len(party_info['connection strategies']) < 1 :
            continue  # that's useless

        first_connstrat = CommStrat.dict_to_strategy(party_info['connection strategies'][0])
        if not isinstance(first_connstrat, CommStrat.Crypto) :
            continue  # non-crypto conn strat?  useless
        party_key = first_connstrat._pubkey
        party_keyid = idlib.make_id(first_connstrat._pubkey, 'broker')
        # make sure the rest are crypto strategies with the same key id as well
        try:
            for c in party_info['connection strategies'][1:] :
                tmpcs = CommStrat.dict_to_strategy(c)
                if not isinstance(tmpcs, CommStrat.Crypto) :
                    raise KeyError
                if not idlib.equal(idlib.make_id(tmpcs._pubkey, 'broker'), party_keyid):
                    raise KeyError
        except (MojoErrors.MojoMessageError, MojoErrors.BadFormatError, KeyError), le:
            debug.stderr.write('ERROR: bad connection strategy, not caching: %s, le: %s\n', args=(party_info, le,))
            continue

        if not MojoKey.publicRSAKeyForCommunicationSecurityIsWellFormed(party_key):
            debug.stderr.write('ERROR: bad RSA key for communications, not caching: %s\n', args=(party_info,))
            continue
        if not MojoKey.publicRSAKeyIsSane(party_key):
            debug.stderr.write('ERROR: insane RSA key, not caching: %s\n', args=(party_info,))
            continue

        # invalid sequence number, don't use it
        if party_info.has_key('sequence num'):
            if type(party_info['sequence num']) != types.LongType or type(party_info['sequence num']) != types.IntType:
                del party_info['sequence num']

        # only update our metainfo cache if the received metainfo has a higher
        # sequence number than our current entry for this counterparty.
        # (XXX note that we need to implement metainfo signing/signature checking before
        # relying on this in a gossip-based metainfo environment to avoid DoS attacks due
        # to poisoned metainfo; until then it works fine for now with trusted metatrackers)
        if PartyInfo_id_cache.has_key(party_keyid):
            if type(PartyInfo_id_cache.get(party_keyid)) != types.DictType:
                debug.mojolog.write("WARNING: Zooko doesn't understand how this could ever happen unless there is a bug in StatsCache, but his logs say it is happening. -- party_keyid: %s, PartyInfo_id_cache.get(party_keyid): %s, PartyInfo_id_cache: %s, UnknownParty_id_cache.get(party_keyid): %s\n", args=(party_keyid, PartyInfo_id_cache.get(party_keyid), PartyInfo_id_cache, UnknownParty_id_cache.get(party_keyid),), v=4, vs="debug")

            current_seq_num = PartyInfo_id_cache.get(party_keyid, {}).get('sequence num', 0)
            new_seq_num = party_info.get('sequence num', 0)
            if current_seq_num > new_seq_num:
                debug.mojolog.write('received old metainfo for %s, not updating cache.  current seq: %s, received seq: %s\n', args=(party_keyid, current_seq_num, new_seq_num), v=2, vs='MetaTrackerLib')
                continue

        # save the insertion time for use later if we want to auto-uncache "old" entries if we
        # have reason to think that they're stale
        # Commengint this out -- it isn't used at the moment, and it causes problems as I try to use the info stored here in a message to someone else and mencode barfs when it tries to mencode the float insertion time.  --Zooko 2001-05-05
        # party_info['MetaTrackerLib cache insertion time'] = time.time()

        # store it in our mega keyid -> contact info cache
        PartyInfo_id_cache[party_keyid] = party_info

        # index the info by the potentially useful services provided
        if party_info.has_key('services') :
            for service in party_info['services']:
                if type(service) is not type({}) :
                    debug.stderr.write("ERROR: metainfo services list contained a non-dict entry: %s\n", args=(service,))
                    continue  # bad service listing, don't cache it
                try:
                    servtype = service['type']
                    if servtype == "blob server" :
                        MojoMessage.checkTemplate(service, OurMessages.HELLO_SERVICE_BLOB_SERVER_TEMPL)
                        BlobServer_id_cache[party_keyid] = party_info
                    elif servtype == "content tracker" :
                        MojoMessage.checkTemplate(service, OurMessages.HELLO_SERVICE_CONTENT_TRACKER_TEMPL)
                        ContentTracker_id_cache[party_keyid] = party_info
                    elif servtype == "dinode tracker" :
                        MojoMessage.checkTemplate(service, OurMessages.HELLO_SERVICE_DINODE_TRACKER_TEMPL)
                        DinodeTracker_id_cache[party_keyid] = party_info
                    elif servtype == "meta tracker" :
                        MojoMessage.checkTemplate(service, OurMessages.HELLO_SERVICE_META_TRACKER_TEMPL)
                        MetaTracker_id_cache[party_keyid] = party_info
                    elif servtype == "relay server v2" :
                        MojoMessage.checkTemplate(service, OurMessages.HELLO_SERVICE_RELAY_SERVER_TEMPL)
                        RelayServer_id_cache[party_keyid] = party_info
                    elif servtype == "publication agent" :
                        MojoMessage.checkTemplate(service, OurMessages.HELLO_SERVICE_PUBLICATION_AGENT_TEMPL)
                        PublicationAgent_id_cache[party_keyid] = party_info
                except (KeyError, MojoErrors.MojoMessageError, MojoErrors.BadFormatError), le:
                    debug.stderr.write("WARNING: bad %s service entry for %s: %s\n", args=(service.get('type'), party_keyid, le.args))
                    continue  # bad service listing, don't cache it

def post_outcome_caching_func(widget, outcome, failure_reason=None):
        if (not failure_reason) and (outcome) and ((type(outcome) is types.ListType) or (type(outcome) is types.TupleType)):
            cache_meta_tracker_responses(outcome)


# update this each time we auto-reload the bootpage so that we don't do it too often.
_last_bootpage_query_time = time.time()


def nonblocking_get_contact_info(MTM, id, callback, timeout=LONG_TIMEOUT, force_metatracker_info_tuple_list=None, fromcacheonly=None, notes=None):
    """
    Just like `get_contact_info()', but the name and parameters remind you that it doesn't block.  ;-)

    #XXXX put callback on DoQ instead of calling back?  --Zooko 2000-10-09
    """
    get_contact_info(MTM, id, callback=callback, timeout=timeout, force_metatracker_info_tuple_list=force_metatracker_info_tuple_list, fromcacheonly=fromcacheonly, notes=notes)

def neg_contact_info(id):
    list_of_caches = [
        PartyInfo_id_cache,
        BlobServer_id_cache,
        DinodeTracker_id_cache,
        ContentTracker_id_cache,
        MetaTracker_id_cache,
        RelayServer_id_cache,
        PublicationAgent_id_cache,
    ]
    for cache in list_of_caches:
        if cache.has_key(idlib.to_binary(id)):
            del cache[idlib.to_binary(id)]
        if cache.has_key(idlib.to_ascii(id)):  # this shouldn't be needed but is here until that's verified
            del cache[idlib.to_ascii(id)]

def get_contact_info(MTM, id, callback=None, timeout=LONG_TIMEOUT, force_metatracker_info_tuple_list=None, fromcacheonly=false, recursiondepth=0, notes = None):
    """
    Get contact info for id.

    @param callback if specified, it is an outcome_func for `MojoTransactionManager.initiate()'.  This function always
        returns None immediately if a callback was specified.  The `outcome' argument will match
        the 'lookup contact info response' template defined in OurMessages.py.
    @param timeout is the time to wait for a response before returning None.
    @param force_metatracker_info_tuple_list forces the lookup to use the given meta tracker(s)
        instead of calling find_meta_trackers().
    @param fromcacheonly prevents us from performing a lookup, we only return what we find in
        our cache (or None)
    @param recursiondepth is used internally to tell when we are recursively trying other metatrackers.

    @return an info_dict describing contact and service info for id.  Returns None if no info could be found.
    @return None if your answer (if any) will come via callback or a timeout occurred.

    @precondition `MTM' must not be None.: MTM is not None
    @precondition `id' must be an id.: idlib.is_sloppy_id(id): "id: %s" % hr(id)
    """
    assert MTM is not None, "precondition: `MTM' must not be None."
    assert idlib.is_sloppy_id(id), "precondition: `id' must be an id." + " -- " + "id: %s" % hr(id)

    id = idlib.to_binary(id)

    unknown_id = UnknownParty_id_cache.get(id, None)
    info_dict = PartyInfo_id_cache.get(id, None)

    if not info_dict and not fromcacheonly and not unknown_id:
        msgtype = 'lookup contact info'
        msgbody = { 'key id': idlib.to_ascii(id) }

        if force_metatracker_info_tuple_list :
            mlist = force_metatracker_info_tuple_list
        else :
            mlist = find_meta_trackers(MTM, max=int(confman.dict['METATRACKERLIB']['MAX_TRACKERS_TO_QUERY_FOR_ID']))
            # NOTE: we're not sorting as they aren't competing yet; we want them in random order for load balancing
            #mlist = MTM.get_handicapper().sort_by_preference(mlist, msgtype, msgbody)

        # the id and contact info (prices!) of the meta tracker we'll ask first
        meta_tracker_id, meta_tracker_infodict = mlist[0]
        if idlib.equal(meta_tracker_id, id):
            debug.mojolog.write("Not trying to ask %s for %s's contact info.\n", args=(meta_tracker_id, id,), v=3, vs="MetaTrackerLib")
            if len(mlist) > 1:
                debug.mojolog.write("Trying next meta tracker.\n", v=3, vs="MetaTrackerLib")
                get_contact_info(MTM=MTM, id=id, callback=callback, timeout=timeout, force_metatracker_info_tuple_list=mlist[1:], fromcacheonly=false, recursiondepth=recursiondepth+1, notes = notes)
                return

        fixedprice = pricelib.meta_tracker_lookup_contact_info_price(meta_tracker_infodict, msgbody)
        if fixedprice is not None:
            # Remove this once we've grandfathered-out most of the fixed-price MTs...  --Zooko 2001-06-01
            mojooffer = fixedprice
        else:
            mojooffer = BidStrategy.choose_bid("MO_LOOKUP_CONTACT_INFO", MTM)
      
        if callback :
            # define our own intermediate outcome_func callback to cache the response before calling the users
            def our_outcomefunc_for_caching(widget, outcome, failure_reason=None, callback=callback, MTM=MTM, id=id, timeout=timeout, remaining_mlist=mlist[1:], recursiondepth=recursiondepth, notes = notes):
                if (not failure_reason) and (outcome) and (outcome.get('result') != "failure"):
                    try:
                        # cache the response, the callback below should get it from the cache
                        cache_meta_tracker_responses([outcome])
                    except:
                        debug.stderr.write("get_contact_info caught an exception in our_outcomefunc_for_caching()\n", v=1, vs="MetaTrackerLib")
                        traceback.print_exc(file=debug.mojolog)
                if failure_reason or (outcome and outcome.get('result') == "failure"):
                    # if there are other meta trackers, ask them instead
                    if remaining_mlist:
                        debug.mojolog.write("meta tracker lookup of %s failed, trying next meta tracker\n", args=(id,), v=3, vs="MetaTrackerLib")
                        get_contact_info(MTM=MTM, id=id, callback=callback, timeout=timeout, force_metatracker_info_tuple_list=remaining_mlist, fromcacheonly=false, recursiondepth=recursiondepth+1, notes = notes)
                        return

                    debug.mojolog.write("WARNING: couldn't find contact info for %s, after querying %s metatracker(s)\n", args=(id, recursiondepth+1,), v=2, vs="MetaTrackerLib")
                    # temporarily mark this id as unknown (to prevent hitting the meta tracker 100s of times
                    # in a row with the same failing query)
                    UnknownParty_id_cache[id] = id

                    # None of the metatrackers worked, time to quit.
                    if failure_reason:
                        # if it was a comms failure as opposed to a meta tracker saying "lookup failed", try refreshing our metatracker contact info
                        global _last_bootpage_query_time
                        if _last_bootpage_query_time + float(confman.dict["METATRACKERLIB"]["MIN_TIME_BETWEEN_BOOTPAGE_RELOADS"]) > time.time() :
                            debug.stderr.write("Communication with metatracker(s) failed, checking bootpage for new contact info.\n")
                            _last_bootpage_query_time = time.time()
                            mojobootpage.load_and_use(doQ=MTM._doQ)
                            # XXX I think we should re-try the query now...   --Zooko 2000-08-31
                            # XXX if we do that we need to make sure we don't get stuck in an infinite retry loop -greg
                try:
                    if notes is None:
                        callback(widget=widget, outcome=outcome, failure_reason=failure_reason)
                    else:
                        callback(widget=widget, outcome=outcome, failure_reason=failure_reason, notes = notes)
                except:
                    debug.mojolog.write("Got exception calling callback %s in MetaTrackerLib.get_contact_info:\n", args=(callback,), v=2, vs="ERROR")
                    traceback.print_exc(file=debug.mojolog)
        
            MTM.initiate(meta_tracker_id, msgtype, msgbody, mojooffer, outcome_func=our_outcomefunc_for_caching, post_timeout_outcome_func=post_outcome_caching_func, timeout=timeout)
            return
        else:
            # caller wants to block until they have the info, okay, we'll setup a
            # callback to set an event and call ourselves in non-blocking mode using it.
            done_event = threading.Event()
            res = {}
            def wait_for_result(widget, outcome, failure_reason, res=res, done_event=done_event):
                if type(outcome) == type({}) and outcome.get('result') == 'success' and not failure_reason:
                    res.update(outcome)
                done_event.set()

            get_contact_info(MTM=MTM, id=id, callback=wait_for_result, timeout=timeout, force_metatracker_info_tuple_list=force_metatracker_info_tuple_list, fromcacheonly=fromcacheonly, notes = notes)

            MTM._doQ.dissociate()
            debug.mojolog.write("calling wait() in get_contact_info()\n", v=5)
            # don't put a timeout in this wait, the MojoTransaction timeouts will take care of it
            while not done_event.isSet():
                # wait in short bursts as waits with a timeout are more portably interruptable allowing processes to shutdown properly
                done_event.wait(2)
            if res:
                info_dict=res
            debug.mojolog.write("done with wait() in get_contact_info()\n", v=5)

    unknown_id = UnknownParty_id_cache.get(id, None)
    info_dict = PartyInfo_id_cache.get(id, None)

    # fake a callback when we just found the info in our cache w/o asking the meta tracker
    if callback:
        if not unknown_id:
            info_dict['result'] = "success" # NOTE: it is bad form to mutate someone else's dict, but this is just too harmless and easy.  --Zooko 2000-09-01
        else:
            info_dict = {'result': "failure"}
        if notes is None:
            MTM._doQ.add_task(callback, kwargs={'widget': MojoTransaction.Widget(MTM.get_id()), 'outcome': info_dict})
        else:
            MTM._doQ.add_task(callback, kwargs={'widget': MojoTransaction.Widget(MTM.get_id()), 'outcome': info_dict, 'notes': notes})
    else:
        return info_dict

def get_blob_servers_and_call_back(MTM, callback, blob_id_mask_list=None, min=8, max=None, publishing_only=false, doremotequery="yourchoice"):
    """
    Gets info for blob servers, either from cache or by asking metatrackers.  Returns
    immediately and then later calls `callback()' with the result. 

    @param `blob_id_mask_list' a list of instances of IdMask;  It gets information only about
        those blob servers that cover all of the masks in the list.  If you want information
        about all blob servers, pass an empty list.
    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"

    @callback `callback' with one argument, a dict mapping keys ids to their full meta info dicts
    
    @precondition `blob_id_mask_list' must be a list.: (blob_id_mask_list is None) or (type(blob_id_mask_list) == types.ListType) or (type(blob_id_mask_list) == types.TupleType): "blob_id_mask_list: %s" % hr(blob_id_mask_list)
    """
    assert (blob_id_mask_list is None) or (type(blob_id_mask_list) == types.ListType) or (type(blob_id_mask_list) == types.TupleType), "precondition: `blob_id_mask_list' must be a list." + " -- " + "blob_id_mask_list: %s" % hr(blob_id_mask_list)

    if blob_id_mask_list is None:
        blob_id_mask_list = []

    def blobid_mask_filter_func(infotuple, blob_id_mask_list=blob_id_mask_list, publishing_only=false):
        # If the filter func took a metainfodict instead of a tuple of (serverid, metainfodict), then we could just pass `blobserver_covers_mask_list()' instead of defining this func.  --Zooko 2000-10-08
        return blobserver_covers_mask_list(infotuple[1], blob_id_mask_list, publishing_only)

    if publishing_only :
        lookup_func_that_calls_back = __lookup_publishing_blob_servers_and_call_back
    else :
        lookup_func_that_calls_back = __lookup_blob_servers_and_call_back

    # XXX Special case to get around semantic mismatch with metatrackers:  --Zooko 2000-10-09
    # XXXX add `num_bits()' to IdMask..  --Zooko 2000-10-09
    # XXXXX put masklist back

    __get_any_meta_info_and_call_back(BlobServer_id_cache, lookup_func_that_calls_back, MTM, callback, filter_func=blobid_mask_filter_func, masklist=blob_id_mask_list, min=min, max=max, doremotequery=doremotequery)


# The is the function that behaves as expected for the working "OldSchoolBlockFetcher", do -not- disable it;
# make your own if you need different semantics. -greg 12-oct-2000
def get_blob_servers(MTM, blob_id_mask, min=15, max=None, publishing_only=0, doremotequery="yourchoice"):
    """
    @param blob_id_mask specifies a minimum range of blob ids that any returned server must support
    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"

    @returns a dict mapping key ids to their full meta info dicts
    """
    def blobid_mask_filter_func(infotuple, blob_id_mask=blob_id_mask, publishing_only=false):
        # If the filter func took a metainfodict instead of a tuple of (serverid, metainfodict), then we could just pass `blobserver_covers_mask()' instead of defining this func.  --Zooko 2000-10-08
        return blobserver_covers_mask(infotuple[1], blob_id_mask, publishing_only)

    if publishing_only :
        lookup_func = __lookup_publishing_blob_servers
    else :
        lookup_func = __lookup_blob_servers

    result = __get_any_meta_info(BlobServer_id_cache, lookup_func, MTM, filter_func=blobid_mask_filter_func, masklist=[blob_id_mask], min=min, max=max, doremotequery=doremotequery)
    # idstring = ''
    # for server_id, other_stuff in result.items():
    #     idstring = idstring + ' <' + mojosixbit.b2a(server_id) + '>'
    # print 'SERVERINFO mask = ' + str(blob_id_mask) + ' servers =' + idstring + ' publishing_only = ' + str(publishing_only)
    return result


def get_publishing_blob_servers(MTM, blob_id_mask, min=8, max=None) :
    """
    @param blob_id_mask specifies a minimum range of blob ids that any returned server must support
    Returns a dict mapping key ids to their full meta info dicts
    """
    return get_blob_servers(MTM, blob_id_mask=blob_id_mask, min=min, max=max, publishing_only=1)


def get_publishing_blob_servers_and_call_back(MTM, callback, blob_id_mask, min=8, max=None) :
    """
    @param blob_id_mask specifies a minimum range of blob ids that any returned server must support
    Returns a dict mapping key ids to their full meta info dicts
    """
    return get_blob_servers_and_call_back(MTM, callback, blob_id_mask_list=[blob_id_mask], min=min, max=max, publishing_only=1)


def get_content_trackers(MTM, min=1, max=None, doremotequery="yourchoice"):
    """
    Returns a dict mapping key ids to their full meta info dicts

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    return __get_any_meta_info(ContentTracker_id_cache, __lookup_content_trackers, MTM, min=min, max=max, doremotequery=doremotequery)

def get_content_trackers_and_call_back(MTM, callback, min=1, max=None, doremotequery="yourchoice"):
    """
    Returns a dict mapping key ids to their full meta info dicts

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    return __get_any_meta_info_and_call_back(ContentTracker_id_cache, __lookup_content_trackers_and_call_back, MTM, callback=callback, min=min, max=max, doremotequery=doremotequery)

def get_relay_servers(MTM, min=1, max=None, doremotequery="yourchoice"):
    """
    Returns a dict mapping key ids to their full meta info dicts

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    return __get_any_meta_info(RelayServer_id_cache, __lookup_relay_servers, MTM, min=min, max=max, doremotequery=doremotequery)

def get_relay_servers_and_call_back(MTM, callback, min=1, max=None, doremotequery="yourchoice"):
    """
    Returns a dict mapping key ids to their full meta info dicts

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    return __get_any_meta_info_and_call_back(RelayServer_id_cache, __lookup_relay_servers_and_call_back, MTM, callback=callback, min=min, max=max, doremotequery=doremotequery)

def get_publishing_agents(MTM, min=1, max=None, get_broadcasters_only=None, filter_func=None, doremotequery="yourchoice"):
    """
    Returns a dict mapping key ids to their full meta info dicts

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    def debug_filter(infotuple, f=filter_func):
        debug.mojolog.write("[A] pub tracker info %s\n", args=(infotuple,), v=6, vs='MetaTrackerLib')
        if callable(f):
            return f(infotuple)
        else:
            return 1
    def filter_broadcasters(infotuple, outer_filter_func=filter_func):
        debug.mojolog.write("[B] pub tracker info %s\n", args=(infotuple,), v=6, vs='MetaTrackerLib')
        services = infotuple[1].get('services', [])
        for service_info in services:
            if service_info['type'] == 'publication agent':
                if service_info.get('allows store location information') != 'yes':
                    return None
                if callable(outer_filter_func):
                    outer_result = outer_filter_func(infotuple)
                    return outer_result
                return 1
    if get_broadcasters_only:
        ourfilter = filter_broadcasters
    else:
        ourfilter = debug_filter
        #ourfilter = filter_func
    return __get_any_meta_info(PublicationAgent_id_cache, __lookup_publication_agents, MTM, min=min, max=max, filter_func=ourfilter, doremotequery=doremotequery)

def get_publishing_agents_and_call_back(MTM, callback, min=1, max=None, get_broadcasters_only=None, filter_func=None, doremotequery="yourchoice"):
    """
    Calls callback with a dict mapping key ids to their full meta info dicts

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    def debug_filter(infotuple, f=filter_func):
        debug.mojolog.write("[A] pub tracker info %s\n", args=(infotuple,), v=6, vs='MetaTrackerLib')
        if callable(f):
            return f(infotuple)
        else:
            return 1
    def filter_broadcasters(infotuple, outer_filter_func=filter_func):
        debug.mojolog.write("[B] pub tracker info %s\n", args=(infotuple,), v=6, vs='MetaTrackerLib')
        services = infotuple[1].get('services', [])
        for service_info in services:
            if service_info['type'] == 'publication agent':
                if service_info.get('allows store location information') != 'yes':
                    return None
                if callable(outer_filter_func):
                    outer_result = outer_filter_func(infotuple)
                    return outer_result
                return 1
    if get_broadcasters_only:
        ourfilter = filter_broadcasters
    else:
        ourfilter = debug_filter
        #ourfilter = filter_func
    return __get_any_meta_info_and_call_back(PublicationAgent_id_cache, __lookup_publication_agents_and_call_back, MTM, callback=callback, min=min, max=max, filter_func=ourfilter, doremotequery=doremotequery)


_lookupfunc_last_call_time_map = {}
def __get_any_meta_info_and_call_back(cacheobj, lookupfunc_that_calls_back, MTM, callback, filter_func=None, masklist=None, min=1, max=None, timeout=LONG_TIMEOUT, doremotequery="yourchoice"):
    """
    Does the same thing as `__get_any_meta_info()', but it returns immediately and then later
    calls `callback()' with the result.

    If it is using cached results, then the callback will happen ASAP (on the DoQ).  If it has
    to query a metatracker, then the callback will happen after it gets the response from the
    metatracker.  In case of transaction error, a callback will happen with a null value.  If
    there is currently an outstanding request for this information with this metatracker, it
    will just wait until the answer to the request arrives rather than beginning another,
    redundant, transaction.

    This last feature is actually implemented inside
    `__list_specified_servers_and_call_back()'.

    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    # We'll call this after getting results, or ASAP (from the DoQ) if we think we have enough cached info.
    def filter_results_and_call_back(widget=None, outcome=None, failure_reason=None, callback=callback, cacheobj=cacheobj, filter_func=filter_func, max=max, min=min, lookupfunc_that_calls_back=lookupfunc_that_calls_back):
        """
        @param outcome the latest results;  Actually we completely ignore this param and just
            query the cacheobj for the results we want.

        @precondition `callback' is callable.: callable(callback): "callback: %s" % hr(callback)
        """
        assert callable(callback), "precondition: `callback' is callable." + " -- " + "callback: %s" % hr(callback)

        _lookupfunc_last_call_time_map[lookupfunc_that_calls_back] = time.time() # record the time of this call

        # Get info from the cache again, hopefully we have enough now
        # that we've looked some up.
        info_list = cacheobj.items()
        if filter_func:
            info_list = filter(filter_func, info_list)

        resultdict = {}
        if max is None or len(info_list) <= max:
            # return all values from info_list
            for keyid, cachedinfo in info_list:
                resultdict[keyid] = cachedinfo
        else:
            # return a random selection by rotating the list first rather than just using the first "max" items.
            rotation = whrandom.randint(0, len(info_list)-1)
            info_list = info_list[rotation:] + info_list[:rotation]
            for keyid, cachedinfo in info_list[:max]:
                resultdict[keyid] = cachedinfo

        mojoutil.callback_wrapper(callback, args=(resultdict,))

    def remote_query(MTM=MTM, lookupfunc_that_calls_back=lookupfunc_that_calls_back, filter_results_and_call_back=filter_results_and_call_back, masklist=masklist, timeout=timeout):
        # TODO make this loop through all meta trackers returned by find_meta_trackers until one of them returns the info.

        if lookupfunc_that_calls_back == __lookup_meta_trackers:
            # if we're here to lookup a meta tracker, use a root tracker
            meta_tracker_tuple_list = find_root_meta_trackers(MTM)
        else:
            meta_tracker_tuple_list = find_meta_trackers(MTM, max=1) # XXX picking only one metatracker here.  --Zooko 2000-10-08

        if meta_tracker_tuple_list:
            meta_tracker_id, meta_tracker_infodict = meta_tracker_tuple_list[0]  # XXX Depending on only one metatracker here.  --Zooko 2000-10-08
            fixedprice = pricelib.meta_tracker_list_servers_price(meta_tracker_infodict, msgbody={})
            if fixedprice is not None:
                # Remove this once we've grandfathered-out most of the fixed-price MTs...  --Zooko 2001-06-01
                mojooffer = fixedprice
            else:
                mojooffer = BidStrategy.choose_bid("MO_LIST_SERVERS", MTM)

            lookupfunc_that_calls_back(MTM, meta_tracker_id, callback=filter_results_and_call_back, mojooffer=mojooffer, masklist=masklist, timeout=timeout)
        else:
            # XXX this is a bad thing for the operation of the system, how should we handle it?
            debug.stderr.write("ERROR: no meta trackers are useful to further the given request!\n", v=0, vs="ERROR")

    if doremotequery == "forceremotequery":
        remote_query()
        return

    last_call_time = _lookupfunc_last_call_time_map.get(lookupfunc_that_calls_back)
    info_list = cacheobj.items()
    if filter_func:
        info_list = filter(filter_func, info_list)

    # ask a meta/id tracker if we don't have enough info in our cache and we haven't already queried recently.
    if (doremotequery == "forcenoremotequery") or (len(info_list) >= min) or \
       ( (len(info_list) >= int(confman.dict["METATRACKERLIB"]["MIN_CACHED_RESULTS_TO_PREVENT_RETRY_LOOKUP_v4"])) or \
         (last_call_time and (time.time() - last_call_time) < int(confman.dict["METATRACKERLIB"]["SECONDS_BEFORE_ALLOWING_A_RETRY_LOOKUP_v4"])) \
       ):
        debug.stderr.write("NOTE: skipping call to %s due to recent cached results. (min %s, last call time %s)\n", args=(lookupfunc_that_calls_back, min, last_call_time), v=8)
        MTM._doQ.add_task(filter_results_and_call_back, args=(None,))
        return
    else:
        remote_query()
        return

def __get_any_meta_info(cacheobj, lookupfunc, MTM, filter_func=None, masklist=None, min=1, max=None, timeout=LONG_TIMEOUT, doremotequery="yourchoice"):
    """
    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    return __inner_get_any_meta_info(cacheobj, lookupfunc, MTM, filter_func=filter_func, masklist=masklist, min=min, max=max, timeout=timeout, doremotequery=doremotequery)

def __inner_get_any_meta_info(cacheobj, lookupfunc, MTM, filter_func=None, masklist=None, min=1, max=None, timeout=LONG_TIMEOUT, doremotequery="yourchoice"):
    """
    Return a dict of cached or looked up meta info using the supplied cacheobj and lookup func.
    Don't return more than max, and attempt to lookup more using the given meta tracker if we have less than min cached.

    Only info about keyid causing a true return value from the optional filter_func parameter will be returned.
    filter_func should expect a tuple as a parameter containing: (keyid, meta_info_dict)

    @param masklist should be None or a list of IdMasks to send to the meta tracker to limit its response size
    @param `doremotequery' is one of "yourchoice", "forcenoremotequery" or "forceremotequery"
    """
    info_list = cacheobj.items()
    if filter_func :
        info_list = filter(filter_func, info_list)

    # ask a meta/id tracker if we don't have enough info in our cache
    if len(info_list) < min :
        # TODO make this loop through all meta trackers returned by find_meta_trackers until one of them returns the info.

        if lookupfunc == __lookup_meta_trackers :
            # if we're here to lookup a meta tracker, use a root tracker
            meta_tracker_tuple_list = find_root_meta_trackers(MTM)
        else :
            meta_tracker_tuple_list = find_meta_trackers(MTM, max=1)

        # if we've done this lookup recently and we already have enough results to choose from
        # don't bother with another lookup yet as its not likely to bring anything new.
        if doremotequery == "forcenoremotequery":
            skip_lookup = true
        elif doremotequery == "forceremotequery":
            skip_lookup = false
        else:
            last_call_time = _lookupfunc_last_call_time_map.get(lookupfunc)
            if len(info_list) >= int(confman.dict["METATRACKERLIB"]["MIN_CACHED_RESULTS_TO_PREVENT_RETRY_LOOKUP_v4"]) or (
                     last_call_time and (time.time() - last_call_time) < int(confman.dict["METATRACKERLIB"]["SECONDS_BEFORE_ALLOWING_A_RETRY_LOOKUP_v4"]) 
                ) :
                skip_lookup = true
            else:
                skip_lookup = false

        if skip_lookup :
            debug.stderr.write("NOTE: skipping call to %s due to recent cached results.\n", args=(lookupfunc,), v=8)
            pass
        elif not meta_tracker_tuple_list :
            # XXX this is a bad thing for the operation of the system, how should we handle it?
            debug.stderr.write('ERROR: no meta trackers are useful to further the given request!\n')
        else :
            meta_tracker_id, meta_tracker_infodict = meta_tracker_tuple_list[0]  # XXX
            fixedprice = pricelib.meta_tracker_list_servers_price(meta_tracker_infodict, msgbody={})
            if fixedprice is not None:
                # Remove this once we've grandfathered-out most of the fixed-price MTs...  --Zooko 2001-06-01
                mojooffer = fixedprice
            else:
                mojooffer = BidStrategy.choose_bid("MO_LIST_SERVERS", MTM)

            lookupfunc(MTM, meta_tracker_id, mojooffer=mojooffer, masklist=masklist, timeout=timeout)

            # XXX please just make me call __get_any_meta_info_and_call_back...  --Zooko 2001-06-03
            _lookupfunc_last_call_time_map[lookupfunc] = time.time() # record the time of this call

            # Get info from the cache again, hopefully we have enough now
            # that we've looked some up.
            info_list = cacheobj.items()
            if filter_func :
                info_list = filter(filter_func, info_list)

    resultdict = {}
    if max is None or len(info_list) <= max :
        # return all values from info_list
        for keyid, cachedinfo in info_list :
            resultdict[keyid] = cachedinfo
    else :
        # return a random selection by rotating the list first rather than just using the first "max" items.
        rotation = whrandom.randint(0, len(info_list)-1)
        info_list = info_list[rotation:] + info_list[:rotation]
        for keyid, cachedinfo in info_list[:max] :
            resultdict[keyid] = cachedinfo

    return resultdict

# This maps from tuples of (meta_tracker_id, msgtypestr, masklist) to a list of functions.
# Each function will be called as soon as soon as the information comes in.
_waiters = {}

# This maps from tuples of (meta_tracker_id, msgtypestr) to a list of mask lists, such that you can find all keys in `_waiters' by using (meta_tracker_id, msgtypestr, masklist).
_outstanding_masklists = {}

# Note lists must be tuplified before being used as keys.  Also note IdMasks must be converted to mencoded mojodict representation to be used as keys.

def __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, msgtypestr, callback, masklist=None, timeout=LONG_TIMEOUT):
    """
    Callback with a list of msgtypestr lookup responses known by the given meta/id
    tracker. the responses received will be added to our cache.

    If there is an outstanding request with this metatracker to list this type of server
    (including masklist), then we do not initiate a new (redundant) transaction sending a new
    query, but instead add `callback' to the list of callbacks that will get called when that
    transaction completes.

    Note that the outcome passed to the callback may include hits that don't match the masklist.
    The `filter_func' is supposed to take care of this, which you passed to `get_any_meta_info_and_call_back'.

    @precondition `msgtypestr' must be a "list something" message type.: msgtypestr[:len('list ')] == 'list ': "msgtypestr: %s" % hr(msgtypestr)
    @precondition `masklist' must be a sequence.: (type(masklist) == types.TupleType) or (type(masklist) == types.ListType): "masklist: %s" % hr(masklist)
    """
    assert msgtypestr[:len('list ')] == 'list ', "precondition: `msgtypestr' must be a \"list something\" message type." + " -- " + "msgtypestr: %s" % hr(msgtypestr)
    assert (masklist is None) or (type(masklist) == types.TupleType) or (type(masklist) == types.ListType), "precondition: `masklist' must be a sequence." + " -- " + "masklist: %s" % hr(masklist)

    if not masklist:
        masklist = ()
    else:
        masklist = tuple(masklist)

    # Now if there is already an outstanding request that will satisfy, then we'll just register as listening for it.
    # (An empty mask list is satisfied by _any_ request.)

    for outstandingmasklist in _outstanding_masklists.get((meta_tracker_id, msgtypestr,), []):
        if idmasklib.mask_list_includes(includer=outstandingmasklist, includee=masklist):
            # debug.mojolog.write("MetaTrackerLib: queueing redundant query.  meta_tracker_id: %s, msgtypestr: %s, outstandingmasklist: %s, masklist: %s, callback: %s\n", args=(meta_tracker_id, msgtypestr, outstandingmasklist, masklist, callback,), v=5, vs="debug")
            omlmd = tuple(map(lambda i: mencode.mencode(i.get_mojodict()), outstandingmasklist))
            assert _waiters.has_key((meta_tracker_id, msgtypestr, omlmd,)), "Internal error: consistency violation.  _waiters: %s must always have keys as indicated by _outstanding_masklists: %s.\n" % (hr(_waiters), hr(_outstanding_masklists))
            _waiters[(meta_tracker_id, msgtypestr, omlmd,)].append(callback)
            return

    # If we get here then there is no currently outstanding request to answer our question, so we'll make a new entry on the local maps (`_waiters' and `_outstanding_masklists') and initiate a transaction!
    # debug.mojolog.write("MetaTrackerLib: initiating new query.  meta_tracker_id: %s, msgtypestr: %s, masklist: %s, callback: %s\n", args=(meta_tracker_id, msgtypestr, masklist, callback,), v=5, vs="debug")
    mojodictmasklist = tuple(map(lambda i: mencode.mencode(i.get_mojodict()), masklist))

    ml = _outstanding_masklists.get((meta_tracker_id, msgtypestr,), [])
    ml.append(masklist)
    _outstanding_masklists[(meta_tracker_id, msgtypestr,)] = ml

    fl = _waiters.get((meta_tracker_id, msgtypestr, mojodictmasklist,), [])
    fl.append(callback)
    _waiters[(meta_tracker_id, msgtypestr, mojodictmasklist,)] = fl
      
    msgbody = {}
    msgbody['filter mask list'] = map(IdMask.get_mojodict, masklist)

    def cacheem_and_call_all_waiters(widget, outcome, failure_reason=None, meta_tracker_id=meta_tracker_id, msgtypestr=msgtypestr, masklist=masklist, mojodictmasklist=mojodictmasklist):
        debug.mojolog.write("MetaTrackerLib: got result: widget: %s, outcome: %s, failure_reason: %s, meta_tracker_id: %s, msgtypestr: %s, masklist: %s\n", args=(widget, outcome, failure_reason, meta_tracker_id, msgtypestr, masklist,), v=5, vs="debug")
        if (not failure_reason) and (outcome) and ((type(outcome) is types.ListType) or (type(outcome) is types.TupleType)):
            cache_meta_tracker_responses(outcome)
        else:
            outcome = []

        assert len(_waiters[(meta_tracker_id, msgtypestr, mojodictmasklist,)]) > 0, "_waiters: %s, widget: %s, outcome: %s, failure_reason: %s, meta_tracker_id: %s, msgtypestr: %s, masklist: %s" % (hr(_waiters), hr(widget), hr(outcome), hr(failure_reason), hr(meta_tracker_id), hr(msgtypestr), hr(masklist),)
        for waiterfunc in _waiters[(meta_tracker_id, msgtypestr, mojodictmasklist,)]:
            debug.mojolog.write("MetaTrackerLib: calling waiterfunc: waiterfunc: %s, widget: %s, outcome: %s, failure_reason: %s, meta_tracker_id: %s, msgtypestr: %s, masklist: %s\n", args=(waiterfunc, widget, outcome, failure_reason, meta_tracker_id, msgtypestr, masklist,), v=5, vs="debug")
            waiterfunc(widget, outcome)

        ml = _outstanding_masklists[(meta_tracker_id, msgtypestr)]
        ml.remove(masklist)
        if len(ml) == 0:
            debug.mojolog.write("MetaTrackerLib: removing masklist list widget: %s, outcome: %s, failure_reason: %s, meta_tracker_id: %s, msgtypestr: %s, masklist: %s\n", args=(widget, outcome, failure_reason, meta_tracker_id, msgtypestr, masklist,), v=5, vs="debug")
            del _outstanding_masklists[(meta_tracker_id, msgtypestr)]

        del _waiters[(meta_tracker_id, msgtypestr, mojodictmasklist,)]

    MTM.initiate(meta_tracker_id, msgtypestr, msgbody, mojooffer, outcome_func=cacheem_and_call_all_waiters, post_timeout_outcome_func=post_outcome_caching_func, timeout=LONG_TIMEOUT, retry=false)

def __list_specified_servers(MTM, meta_tracker_id, mojooffer, msgtypestr, masklist=None, timeout=LONG_TIMEOUT) :
    """
    Return a list of msgtypestr lookup responses known by the given meta/id tracker.
    the responses received will be added to our cache.

    @precondition `msgtypestr' must be a "list something" message type.: msgtypestr[:len('list ')] == 'list ': "msgtypestr: %s" % hr(msgtypestr)
    """
    assert msgtypestr[:len('list ')] == 'list ', "precondition: `msgtypestr' must be a \"list something\" message type." + " -- " + "msgtypestr: %s" % hr(msgtypestr)

    outcomeholder = [None]
    doneflag = threading.Event()
    def outcome_func(widget, outcome, failure_reason=None, doneflag=doneflag, outcomeholder=outcomeholder):
        outcomeholder[0] = outcome
        doneflag.set()

    __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, msgtypestr, callback=outcome_func, masklist=masklist, timeout=timeout)
    doneflag.wait()
    return outcomeholder[0]

def __lookup_blob_servers_and_call_back(MTM, meta_tracker_id, mojooffer, masklist, callback, timeout=LONG_TIMEOUT) :
    """
    Does the same thing as `__lookup_blob_servers()', but it returns
    immediately and then later calls `callback()' with the result.
    """
    # XXX masklist is removed here for now so that we only do general "tell me some blob servers" queries
    masklist=None
    __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, 'list blob servers', callback=callback, masklist=masklist, timeout=timeout)

def __lookup_blob_servers(MTM, meta_tracker_id, mojooffer, masklist, timeout=LONG_TIMEOUT) :
    """
    Return a list of blob servers known by the given id tracker.
    the responses received will be added to our cache.
    """
    # XXX masklist is removed here for now so that we only do general "tell me some blob servers" queries
    masklist=None
    return __list_specified_servers(MTM, meta_tracker_id, mojooffer, 'list blob servers', masklist=masklist, timeout=timeout)

def __lookup_publishing_blob_servers_and_call_back(MTM, meta_tracker_id, mojooffer, masklist, callback, timeout=LONG_TIMEOUT) :
    # XXX masklist is removed here for now so that we only do general "tell me some blob servers" queries
    masklist=None
    return __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, 'list publishing blob servers', callback=callback, masklist=masklist, timeout=timeout)

def __lookup_publishing_blob_servers(MTM, meta_tracker_id, mojooffer, masklist, timeout=LONG_TIMEOUT) :
    """
    Return a list of blob publishing servers known by the given id tracker.
    the responses received will be added to our cache.
    """
    # XXX masklist is removed here for now so that we only do general "tell me some publishing blob servers" queries
    masklist=None
    return __list_specified_servers(MTM, meta_tracker_id, mojooffer, 'list publishing blob servers', masklist=masklist, timeout=timeout)


def __lookup_content_trackers(MTM, meta_tracker_id, mojooffer, masklist, timeout=LONG_TIMEOUT) :
    """
    Return a list of content trackers known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers(MTM, meta_tracker_id, mojooffer, 'list content trackers', masklist=masklist, timeout=timeout)

def __lookup_content_trackers_and_call_back(MTM, meta_tracker_id, mojooffer, masklist, callback, timeout=DEFAULT_TIMEOUT) :
    """
    Return a list of content trackers known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, 'list content trackers', callback=callback, masklist=masklist, timeout=timeout)


def __lookup_meta_trackers(MTM, meta_tracker_id, mojooffer, masklist, timeout=LONG_TIMEOUT) :
    """
    Return a list of meta trackers known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers(MTM, meta_tracker_id, mojooffer, 'list meta trackers', masklist=masklist, timeout=timeout)


def __lookup_relay_servers(MTM, meta_tracker_id, mojooffer, masklist, timeout=LONG_TIMEOUT) :
    """
    Return a list of relay servers known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers(MTM, meta_tracker_id, mojooffer, 'list relay servers v2', masklist=masklist, timeout=timeout)

def __lookup_relay_servers_and_call_back(MTM, meta_tracker_id, mojooffer, masklist, callback, timeout=LONG_TIMEOUT) :
    """
    Return a list of relay servers known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, 'list relay servers v2', callback=callback, masklist=masklist, timeout=timeout)

def __lookup_publication_agents(MTM, meta_tracker_id, mojooffer, masklist, timeout=LONG_TIMEOUT) :
    """
    Return a list of publication agents known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers(MTM, meta_tracker_id, mojooffer, 'list publication agents', masklist=masklist, timeout=timeout)

def __lookup_publication_agents_and_call_back(MTM, meta_tracker_id, mojooffer, masklist, callback, timeout=LONG_TIMEOUT) :
    """
    Return a list of publication agents known by the given meta/id tracker.
    the responses received will be added to our cache.
    """
    return __list_specified_servers_and_call_back(MTM, meta_tracker_id, mojooffer, 'list publication agents', callback=callback, masklist=masklist, timeout=timeout)


def get_blobserver_mask_list(metainfodict):
    """
    returns a list of IdMask objects for the block server described by metainfodict.
    """
    assert type(metainfodict) == type({})
    if not metainfodict.has_key('services'):
        # No services?  then there is no mask list.
        # (while debugging, raise an error to make sure this doesn't happen)
        if MojoConstants.DEBUG:
            raise RuntimeError, "metainfodict had no services: %s" % hr(metainfodict)
        return []
    # extract the 'blob server' services dict
    blobservice = filter(lambda s: type(s) == type({}) and s.get('type') == 'blob server', metainfodict['services'])[0]
    blob_server_mask_list = map(lambda d: IdMask(mojodict=d), blobservice['mask list']) 
    return blob_server_mask_list
    

def blobserver_covers_mask_list(metainfodict, blob_id_mask_list, publishing_only=false):
    """
    @returns `true' if and only if every bitstring covered by `blob_id_mask_list' is covered by
        the blobserver

    @precondition `blob_id_mask_list' must be a list.: (type(blob_id_mask_list) == types.ListType) or (type(blob_id_mask_list) == types.TupleType): "blob_id_mask_list: %s" % hr(blob_id_mask_list)
    """
    assert (type(blob_id_mask_list) == types.ListType) or (type(blob_id_mask_list) == types.TupleType), "precondition: `blob_id_mask_list' must be a list." + " -- " + "blob_id_mask_list: %s" % hr(blob_id_mask_list)

    try:
        if not metainfodict.has_key('services'):
            # No services?  Ok, then it doesn't cover it!
            # XX or we could raise exception here.  That old dilemma-picky or flexible?
            if MojoConstants.DEBUG:
                raise "metainfodict had no services: %s" % hr(metainfodict)
            return false

        # remove None's from metainfodict first  XXX how are they getting there?
        metainfodict['services'] = filter(lambda s: s != None, metainfodict['services'])
        # extract the 'blob server' services dict
        if len(metainfodict.get('services',[])) < 1:
            debug.mojolog.write("WARNING: a metainfodict had an empty services list in `blobserver_covers_mask_list()'.  Is this ok?  metainfodict: %s\n", args=(metainfodict,), v=5, vs="debug")
            return false

        blobservice = filter(lambda s: s.get('type') == 'blob server', metainfodict['services'])[0]
        if publishing_only and string.lower(blobservice['publishing allowed']) == 'no' :
            return false
        blob_server_mask_list = map(lambda dict: IdMask(mojodict=dict), blobservice['mask list']) 
        return idmasklib.mask_list_includes(includer=blob_server_mask_list, includee=blob_id_mask_list)
    except (AttributeError, TypeError, KeyError, SEXP.Error, IdMaskError):
        debug.stderr.write("WARNING: MetaTrackerLib skipping unparsable info: %s\n", args=(metainfodict,))
        traceback.print_exc(file=debug.stderr)
    return false

def blobserver_covers_mask(metainfodict, blob_id_mask, publishing_only=false) :
    """
    @returns `true' if and only if the blobserver described by `metainfodict' covers the id mask

    @precondition `blob_id_mask' must be an IdMask.: isinstance(blob_id_mask, IdMask): "blob_id_mask: %s" % hr(blob_id_mask)
    """
    assert isinstance(blob_id_mask, IdMask), "precondition: `blob_id_mask' must be an IdMask." + " -- " + "blob_id_mask: %s" % hr(blob_id_mask)

    try:
        # remove None's from metainfodict first  XXX how are they getting there?
        metainfodict['services'] = filter(lambda s: s != None, metainfodict['services'])
        # extract the 'blob server' services dict
        blobservice = filter(lambda s: s.get('type') == 'blob server', metainfodict['services'])[0]
        if publishing_only and string.lower(blobservice['publishing allowed']) == 'no' :
            return false
        blob_server_maskdict_list = blobservice['mask list']
        # search all server blob_id masks
        for maskdict in blob_server_maskdict_list :
            blob_server_mask = IdMask(mojodict=maskdict)
            #debug.stderr.write('BlobMask: want %s, checking against %s\n' % (repr(blob_id_mask), repr(blob_server_mask)))
            if blob_id_mask.is_subset_of(blob_server_mask) :
                #debug.stderr.write('BlobMask: matched!\n')
                return true
    except (AttributeError, TypeError, KeyError, SEXP.Error, IdMaskError):
        debug.stderr.write("WARNING: MetaTrackerLib skipping unparsable info: %s\n", args=(metainfodict,))
        traceback.print_exc(file=debug.stderr)
    return false



def save_partyinfo_to_disk(filename=None):
    """attempts to save PartyInfo_id_cache to disk, raises no errors, but logs one if the attempt fails"""
    if not filename:
        filename = confman.getpath(["METATRACKERLIB", "SAVED_CACHE_FILE"])
    debug.mojolog.write("Saving cache to file %s\n", args=[filename], v=2, vs='MetaTrackerLib')
    list_of_info = PartyInfo_id_cache.values()
    try:
        rawstr = pickle.dumps(list_of_info)
        f = open(filename, "w")
        f.write(rawstr)
        f.close()
    except (pickle.PicklingError, pickle.UnpicklingError, IOError), e:
        debug.mojolog.write("Got %s attempting to save metainfo cache to %s\n", args=[e, filename], vs='ERROR')

def load_partyinfo_from_disk(filename=None):
    """attempts to load PartyInfo_id_cache from disk, raises no errors, but logs one if the attempt fails"""
    if not filename:
        filename = confman.getpath(["METATRACKERLIB", "SAVED_CACHE_FILE"])
    list_of_info = []
    debug.mojolog.write("Loading cache from file %s\n", args=[filename], v=2, vs='MetaTrackerLib')
    try:
        f = open(filename, "r")
        rawstr = f.read()
        f.close()
        list_of_info = pickle.loads(rawstr)
        cache_meta_tracker_responses(list_of_info)
    except (pickle.PicklingError, pickle.UnpicklingError, IOError), e:
        debug.mojolog.write("Got %s attempting to load metainfo cache from %s\n", args=[e, filename], vs='ERROR')
    debug.mojolog.write("%s entries loaded\n", args=[len(list_of_info)], v=3, vs='MetaTrackerLib')


contact_cache = None
contact_cache_preencoded = None
last_contact_cache_update = time.time()
refresh_delay = max(60, int(confman.dict["METATRACKERLIB"]["TIME_BETWEEN_EXTRA_CONTACT_REFRESH"]))
num_contacts = int(confman.dict["METATRACKERLIB"]["NUM_EXTRA_CONTACTS"])
cids_extrametainfo_sent_to_map = {}  # keyed on binary cid indicating we've sent the contact_cache to that counterparty

def get_a_few_random_contacts(time=time.time, _empty_list_encoding=mencode.mencode([])):
    global contact_cache
    global contact_cache_preencoded
    global last_contact_cache_update
    global cids_extrametainfo_sent_to_map 
    global refresh_delay
    global num_contacts
    if num_contacts <= 0:
        contact_cache = []
        contact_cache_preencoded = _empty_list_encoding
    elif not contact_cache or ((time() - last_contact_cache_update) > refresh_delay):
        list_of_info = PartyInfo_id_cache.values()
        
        if not list_of_info:
            contact_cache = []
            contact_cache_preencoded = mencode.PreEncodedThing(contact_cache)
        else:
            list_of_info = mojoutil.shuffleList(list_of_info)
            contact_cache = list_of_info[:num_contacts]
            contact_cache_preencoded = mencode.PreEncodedThing(contact_cache)
        cids_extrametainfo_sent_to_map.clear()  # new extrametainfo, clear the map
        last_contact_cache_update = time()
        debug.mojolog.write("reselecting random extra metainfo to send to others\n", v=3, vs="MetaTrackerLib")
    return contact_cache

def get_preencoded_extra_metainfo_for(cid, time=time.time):
    """
    Returns an mencode.PreEncodedThing object containing extra metainfo to include on a
    message to counterparty cid or None if we should not send any at this time.
    """
    assert idlib.is_binary_id(cid), "cid must be a binary counterparty id"
    global last_contact_cache_update
    global refresh_delay
    global cids_extrametainfo_sent_to_map 
    if cids_extrametainfo_sent_to_map.has_key(cid) and ((time() - last_contact_cache_update) < refresh_delay):
        return None
    cids_extrametainfo_sent_to_map[cid] = None  # flag that we've recently sent info to this cid
    get_a_few_random_contacts()
    global contact_cache_preencoded
    return contact_cache_preencoded

