#
# A simplified function for reading windows registry keys
#

import _winreg

def get_registry_setting(key, name, _topkey=_winreg.HKEY_CURRENT_USER):
    """
    Checks HKEY_CURRENT_USER followed by HKEY_LOCAL_MACHINE for a value
    with the given name stored under the given backslash seperated key

    Only supports string values.
    """
    try:
        regkey = _winreg.OpenKey(_topkey, key)
    except WindowsError:
        raise KeyError, (key, "registry key not found")

    sublen, vallen, timestamp = _winreg.QueryInfoKey(regkey)
    for validx in xrange(vallen):
        keyname, value, keytype = _winreg.EnumValue(regkey, validx)
        if keyname == name and keytype == _winreg.REG_SZ:
            return value

    if _topkey != _winreg.HKEY_LOCAL_MACHINE:
        return get_registry_setting(key, name, _topkey=_winreg.HKEY_LOCAL_MACHINE)

    raise KeyError, (key, name, "registry setting not found")

def set_registry_setting(key, name, data, _topkey=_winreg.HKEY_CURRENT_USER):
    """
    Sets a registry setting.
    
    Only supports string values.
    """
    try:
        regkey = _winreg.OpenKey(_topkey, key, 0, _winreg.KEY_SET_VALUE)
    except WindowsError:
        raise KeyError, (key, "registry key not found")

    try:
        _winreg.DeleteValue(regkey, name)
    except:
        pass

    _winreg.SetValueEx(regkey, name, 0, _winreg.REG_SZ, data)

