import tripledescbc
import randsource
import HashRandom
import mojosixbit
from sha import sha


class InvalidPassphrase(StandardError):
    pass


# encrypts plaintext with a passphrase and returns a
# dict containing two keys ['iv', 'ctext'], data is sixbit encoded
# cyphertext includes a hash of the plaintext to detect an invalid passphrase
def encrypt(passphrase, plaintext):
    sr = HashRandom.SHARandom(sha(passphrase).digest())
    key = sr.get(24)
    iv = randsource.get(8)
    plaintext = sha(plaintext).digest() + plaintext
    ctext = tripledescbc.new(key).encrypt(iv, plaintext)
    ndict = {'iv' : mojosixbit.b2a(iv), 'ctext' : mojosixbit.b2a(ctext)}
    return ndict


# takes a dictionary with two keys ('iv', 'ctext') and returns the plaintext
# raises an exception if the wrong passphrase is entered
# note: if the ciphertext isn't in the format created by the above
# encrypt() function, then it will still raise the InvalidPassphrase
# exception!
def decrypt(passphrase, cdict):
    iv = mojosixbit.a2b(cdict['iv'])
    ctext = mojosixbit.a2b(cdict['ctext'])
    sr = HashRandom.SHARandom(sha(passphrase).digest())
    key = sr.get(24)
    rawptext = tripledescbc.new(key).decrypt(iv, ctext)
    hash, ptext = rawptext[:20], rawptext[20:]
    if(hash != sha(ptext).digest()):
        raise InvalidPassphrase()
    return ptext


class TooManyTries(StandardError):
    pass

## uses raw_input to capture a passphrase and decrypt ctext using the
## above decryption functions
## this function is temporary and is unlikely to be used in production
## RETURNS:  plaintext, passphrase
def acquirePhraseAndDecrypt(ctext):
    gotPhrase = 0
    tries = 0
    while not gotPhrase:
        tries = tries + 1
        if tries > 3:
            raise TooManyTries()

        # get a passphrase
        pphrase = raw_input("Enter your passphrase (NOT HIDDEN): ")

        # try to decrypt the key
        try:
            plain = decrypt(pphrase, ctext)
            gotPhrase = 1
        except InvalidPassphrase, e:
            print("The passphrase you entered was invalid.")
    return plain, pphrase
