#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

from confutils import confman, integerizePrice
from bsddb3 import db, dbobj
from CleanLogDb import CleanLogDbEnv
import RunTests


MINS_BETWEEN_DB_CHECKPOINTS = 5


# XXX check templates throughout this file
class PayLarsTrackerHandlers :
    def __init__(self, dbdir, recover=true) :
        self.db_env = CleanLogDbEnv()
        if recover:
            recoverflag = db.DB_RECOVER
        else:
            recoverflag = 0

        try:
            self.db_env.open(dbdir, db.DB_CREATE | db.DB_INIT_MPOOL | db.DB_INIT_LOCK | db.DB_THREAD | db.DB_INIT_LOG | db.DB_INIT_TXN | recoverflag)
        except db.DBError, dbe:
            # XXX HACK sometimes trying again after one open with recover works
            self.db_env.open(dbdir, db.DB_CREATE | db.DB_INIT_MPOOL | db.DB_INIT_LOCK | db.DB_THREAD | db.DB_INIT_LOG | db.DB_INIT_TXN | recoverflag)

        # maps block id to author description
        self._block_to_author = dbobj.DB(self.db_env)
        self._block_to_author.open('issued tokens', db.DB_BTREE, db.DB_CREATE | db.DB_THREAD)

        self.announced_services = {
            'type' : 'paylars tracker',
            'store author information price' : integerizePrice(confman.dict["PAYLARSTRACKERHANDLERS"]["PRICES"]["STORE"]),
            'get author information price' : integerizePrice(confman.dict["PAYLARSTRACKERHANDLERS"]["PRICES"]["RETRIEVE"]),
        }
    
    def __del__(self) :
        if hasattr(self, '_block_to_author') and self._block_to_author is not None :
            self._block_to_author.close()
        if hasattr(self, 'db_env') and self.db_env is not None :
            self.db_env.nosyncerror_txn_checkpoint(0)
            self.db_env.close()

    def compute_cost_store_author_info(self, widget, message_type, message_body) :
        return int(self.announced_services['store author information price'])

    def compute_cost_get_author_info(self, widget, message_type, message_body) :
        return int(self.announced_services['get author information price'])

    def get_handlers_dict(self) :
        dict = {}
        dict['store author information'] = (self.store_author_information, self.compute_cost_store_author_info)
        dict['get author information'] = (self.get_author_information, self.compute_cost_get_author_info)
        return dict
        
    def get_services_dict(self) :
        return self.announced_services

    def store_author_information(self, widget, message_body) :
        self.db_env.nosyncerror_txn_checkpoint(MINS_BETWEEN_DB_CHECKPOINTS)
        trans = self.db_env.txn_begin()
        try :
            blockid = message_body['block id']
            info = message_body['author information']
            oldval = self._block_to_author.get(blockid, trans, db.DB_RMW)
            if oldval is not None :
                if oldval == info :
                    return {'result' : 'success'}
                else :
                    return {'result' : 'failure', 'reason' : 'other information already posted'}
            self._block_to_author.put(message_body['block id'], message_body['author information'], trans)
            trans.commit()
            trans = None
            return {'result' : 'success'}
        finally :
            if trans is not None :
                trans.abort()
        
    def get_author_information(self, widget, message_body) :
        val = self._block_to_author.get(message_body['block id'])
        if val is None :
            return {'result' : 'failure'}
        return {'result' : 'success', 'information' : val}

def test_cover_get_handlers_dict() :
    handlers = PayLarsTrackerHandlers(RunTests.get_next_dir_name())
    assert handlers.get_handlers_dict() is not None

def test_cover_get_services_dict() :
    handlers = PayLarsTrackerHandlers(RunTests.get_next_dir_name())
    assert handlers.get_services_dict() is not None

def test_returns_failure_for_unstored_thing() :
    handlers = PayLarsTrackerHandlers(RunTests.get_next_dir_name())
    assert handlers.get_author_information(None, {'block id' : 'aoentshuaeu'})['result'] == 'failure'

def test_returns_failure_for_different_information_PayLarsed_previously() :
    handlers = PayLarsTrackerHandlers(RunTests.get_next_dir_name())
    assert handlers.store_author_information(None, {'block id' : 'spam', 'author information' : 'eggs'})['result'] == 'success'
    assert handlers.store_author_information(None, {'block id' : 'spam', 'author information' : 'eggs2'})['result'] == 'failure'
    
def test_returns_success_for_same_information_PayLarsed_previously() :
    handlers = PayLarsTrackerHandlers(RunTests.get_next_dir_name())
    assert handlers.store_author_information(None, {'block id' : 'spam', 'author information' : 'eggs'})['result'] == 'success'
    assert handlers.store_author_information(None, {'block id' : 'spam', 'author information' : 'eggs'})['result'] == 'success'

def test_succeeds_at_PayLars_and_retrieve() :
    handlers = PayLarsTrackerHandlers(RunTests.get_next_dir_name())
    assert handlers.store_author_information(None, {'block id' : 'spam', 'author information' : 'eggs'})['result'] == 'success'
    r = handlers.get_author_information(None, {'block id' : 'spam'})
    assert r['result'] == 'success'
    assert r['information'] == 'eggs'

mojo_test_flag = 1

if __name__ == '__main__' :
    RunTests.runTests('PayLarsTrackerHandlers')





