#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
__cvsid = '$Id: PerformanceHandicapper.py,v 1.20 2001/02/12 02:54:04 greg Exp $'

# standard modules
import math
import time

# our modules
import confutils
import mojoutil
import debug


class GenericLatencyHandicapper:
    def __init__(self, counterparties, msgtype_list, latency_statname_list, base_mult_confstr):
        assert 0, "this base example class shouldn't be instantiated"
        self.counterparties = counterparties
        # the message typea this handicapper will apply to
        self.message_types = msgtype_list
        # the name of the latency stats stored in the counterparty object to consider
        self.latency_statnames = latency_statname_list
        # the name of the confutils.confman.dict["COUNTERPARTY"] key for the base multiplier value
        self.base_mult_confstr = base_mult_confstr
        # default latency when none
        self.default = 0.0

    def __call__(self, counterparty_id, metainfo, message_type, message_body):
        if message_type not in self.message_types :
            return 0
        scalingfactor = float(confutils.confman.dict['COUNTERPARTY'][self.base_mult_confstr])
        cpobj = self.counterparties.get_counterparty_object(counterparty_id)
        statlist = map(cpobj.get_custom_stat, self.latency_statnames)
        statlist = filter(lambda x: x is not None, statlist)
        if len(statlist) > 0:
            avg_latency = float(reduce(lambda p,q: p+q, statlist)) / len(statlist)
            if avg_latency < 0:
                debug.mojolog.write("avg_latency for stats %s less than zero!\n" % `self.latency_statnames`, v=3, vs=self.__class__.__name__)
                avg_latency = self.default
        else:
            avg_latency = self.default
        return {'additive': avg_latency*scalingfactor}

class DoYouHaveBlobsLatencyHandicapper(GenericLatencyHandicapper) :
    """
    for msgtype == 'do you have blobs'

    @returns average *recent* latency in seconds times 500, or 500 if no recent information is available
    """
    def __init__(self, counterparties) :
        self.counterparties = counterparties
        self.message_types = ('do you have blobs',)
        self.latency_statnames = ('do you have blobs latency',)
        self.base_mult_confstr = "BASE_LATENCY_HANDICAP_MULT"
        self.default = 1.0   # XXX tune this, the default determines if you handicap for or against new counterparties

class RequestBlobThruputHandicapper(GenericLatencyHandicapper) :
    """
    for msgtype in ('do you have blobs', 'request blob')

    @returns average thruput in bytes/second
    """
    def __init__(self, counterparties) :
        self.counterparties = counterparties
        self.message_types = ('do you have blobs', 'request blob',)
        self.latency_statnames = ('request blob bytes/sec',)

        self.base_mult_confstr = "BASE_THRUPUT_HANDICAP_MULT_v2"
        cs_in = float(confutils.confman.dict['CONN_SPEED_IN'])  # in kbits/sec
        self.default = cs_in*8192   # XXX tune this. this assumes a default of 1/10th the users configured connection in speed

class PutBlobThruputHandicapper(GenericLatencyHandicapper) :
    """
    for msgtype == 'put blob'

    @returns average thruput in bytes/second
    """
    def __init__(self, counterparties) :
        self.counterparties = counterparties
        self.message_types = ('put blob',)
        self.latency_statnames = ('put blob bytes/sec',)

        self.base_mult_confstr = "BASE_THRUPUT_HANDICAP_MULT_v2"
        cs_in = float(confutils.confman.dict['CONN_SPEED_OUT'])  # in kbits/sec
        self.default = cs_in*8192   # XXX tune this. this assumes a default of the users configured connection out speed

