#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
__cvsid = '$Id: RelayHandicapper.py,v 1.6 2000/12/12 08:52:45 greg Exp $'

# TODO another useful handicapper to implement here file would be to figure out
# how much extra it will cost to send the message to this person via relay


from confutils import confman
from MojoHandicapper import DISQUALIFIED


# this was lowered from 5000 to 100 now that we have reasonably fast relay and metainfo included
# on messages when it has changed, relay isn't nearly so much of a burden to serve behind anymore
PENALTY_FOR_RELAY = 100


class UseRelayLastForSpeedHandicapper:
    """
    This handicapper penalizes servers operating behind Relay since they
    can be *damn* slow to return results compared to people with direct
    IP addresses.

    Better handicappers along these lines could be created, but this should
    help the user experience for now.
    """
    def __init__(self):
        pass

    def compute_handicap(self, counterparty_id, metainfo, message_type, message_body):
        """
        for all msgtypes

        @returns a penalty if counterparty is behind relay
        """
        handicap = None
        for csdict in metainfo['connection strategies'] :
            try:
                lowerstrattype = csdict['lowerstrategy']['comm strategy type']
                if lowerstrattype == 'relay' :
                    # handicap severely if all their commstrats are relay
                    if handicap is None :
                        if confman.get('KLOOGE_NO_RELAY', 'no') in ('yes', 'true'):
                            handicap = DISQUALIFIED
                        else:
                            handicap = PENALTY_FOR_RELAY
                elif lowerstrattype == 'TCP' :
                    # don't handicap if they have a TCP CommStrat
                    handicap = 0
                    break
            except KeyError:
                pass
        
        if handicap is None :
            handicap = 0
        return handicap
