import RunTests
import types

class ShortcutsDict:
    def __init__(self):
        # maps string to string, ShortcutsDict, or (ShortcutsDict, key)
        # (tuples represent shortcuts)
        self.dict = {}
        self.alias = None

    def add_shortcut(self, key, other, other_key = None):
        if self.alias is not None:
            self.alias.add_shortcut(key, other, other_key)
            return
        if other_key is None:
            other_key = key
        assert type(key) is types.StringType, 'only supports string keys'
        assert type(other_key) is types.StringType, 'only supports string keys'
        old = self.dict[key]
        if old is not None:
            assert type(old) is types.TupleType
            assert old == (other, other_key)
            return
        self.shortcuts[key] = (other, other_key)
        val = self.dict.get(key)
        self.dict[key] = (other, other_key)
        if val is None:
            return
        if type(val) is types.StringType:
            other[other_key] = val
        else:
            otherval = other[other_key]
            assert type(otherval) is not types.StringType, "can't shortcut a dict onto a string"
            otherval._map_to(val)

    def _map_to(self, other):
        self.alias = other
        for key, value in self.dict.items():
            if type(value) == types.StringType:
                other[key] = value
            elif type(value) == types.

    def get(self, key, default = None):
        if self.alias is not None:
            return self.alias.get(key, default)
        assert type(key) == types.StringType, 'only supports string keys'
        dict = self.dict
        if self.shortcuts.has_key(key):
            dict, key = self.shortcuts[key]
        if not dict.has_key(key):
            return default
        return dict[key]

    def __len__(self):
        if self.alias is not None:
            return len(self.alias)
        num = 0
        for value in self.dict.values():
            if type(value) is types.InstanceType and len(value) == 0:
                continue
            num = num + 1
        for otherdict, otherkey in self.shortcuts.values():
            value = otherdict.get(otherkey)
            if type(value) is types.InstanceType and len(value) == 0:
                continue
            num = num + 1
        return num

    def has_key(self, key):
        if self.alias is not None:
            return self.alias.has_key(key)
        dict = self.dict
        if self.shortcuts.has_key(key):
            dict, key = self.shortcuts[key]
        value = dict.get(key)
        if type(value) is types.InstanceType and len(value) == 0:
            return 0
        return value is not None

    def keys(self):
        if self.alias is not None:
            return self.alias.keys()
        result = []
        for key in self.dict.keys():
            if self.has_key(key):
                result.append(key)
        return result

    def values(self):
        if self.alias is not None:
            return self.alias.values()
        result = []
        for key, value in self.dict.items():
            if self.has_key(key):
                result.append(value)
        return result

    def items(self):
        if self.alias is not None:
            return self.alias.items()
        result = []
        for key, value in self.dict.items():
            if self.has_key(key):
                result.append((key, value))
        return result

    def __getitem__(self, key):
        if self.alias is not None:
            return self.alias[key]
        assert type(key) is types.StringType, 'only supports string keys'
        if not self.dict.has_key(key):
            self.dict[key] = ShortcutsDict()
        return self.dict[key]

    def __setitem__(self, key, value):
        if self.alias is not None:
            self.alias[key] = value
            return
        assert type(key) == types.StringType, 'only supports string keys'
        assert type(value) == types.StringType, 'can only set string values, to set dicts use implicit creation with lookup'
        assert type(self.dict.get(key)) is not types.InstanceType, 'changing a value from a dict to a string is not allowed, to make accesses of the old dict safe'
        self.dict[key] = value

    def clear(self):
        if self.alias is not None:
            self.alias.clear()
            return
        for key in self.keys():
            del self[key]

    def __delitem__(self, key):
        if self.alias is not None:
            del self.alias[key]
            return
        assert type(key) == types.StringType, 'only supports string keys'
        old = self.dict.get(key)
        if old is None:
            return
        if type(old) is types.InstanceType:
            old.clear()
        else:
            del self.dict[key]

mojo_test_flag = 1

def test_pruning_and_reappearance():
    x = ShortcutsDict()
    x['a']['b'] = 'c'
    y = x['a']
    assert len(x) == 1
    assert len(y) == 1
    assert x.has_key('a')
    assert y.has_key('b')
    assert y['b'] == 'c'
    assert y.get('b') == 'c'
    assert x.get('a') is y
    del y['b']
    assert len(x) == 0
    assert len(y) == 0
    assert not x.has_key('a')
    assert not y.has_key('b')
    assert x.get('a') is None
    assert y.get('b') is None
    y['b'] = 'd'
    assert y['b'] == 'd'
    assert y.get('b') == 'd'
    assert len(x) == 1
    assert len(y) == 1
    assert x['a'] is y
    assert x.get('a') is y
    del x['a']
    assert len(x) == 0
    assert len(y) == 0
    assert not x.has_key('a')
    assert not y.has_key('b')
    assert x.get('a') is None
    assert y.get('b') is None
    y['b'] = 'd'
    assert y['b'] == 'd'
    assert y.get('b') == 'd'
    assert len(x) == 1
    assert len(y) == 1
    assert x['a'] is y
    assert x.get('a') is y

def test_keys():
    x = ShortcutsDict()
    x['a']
    x['b'] = 'spam'
    x['c']['d'] = 'eggs'
    del x['c']
    x['e']['f'] = 'green'
    r = x.keys()
    r.sort()
    assert r == ['b', 'e']

def test_del_nonexistant():
    x = ShortcutsDict()
    del x['a']

def test_asserts_on_change_from_dict_to_string():
    x = ShortcutsDict()
    x['a']
    try:
        x['a'] = 'spam'
        return 0
    except AssertionError:
        return 1

def test_has_key():
    x = ShortcutsDict()
    x['a']
    x['b'] = 'spam'
    x['c']['d'] = 'eggs'
    del x['c']
    x['e']['f'] = 'green'
    assert not x.has_key('a')
    assert x.has_key('b')
    assert not x.has_key('c')
    assert x.has_key('e')

def test_len():
    x = ShortcutsDict()
    x['a']
    x['b'] = 'spam'
    x['c']['d'] = 'eggs'
    del x['c']
    x['e']['f'] = 'green'
    assert len(x) == 2   

def test_clear():
    x = ShortcutsDict()
    x['a']
    x['b'] = 'spam'
    x['c']['d'] = 'eggs'
    del x['c']
    x['e']['f'] = 'green'
    x.clear()
    assert len(x) == 0

def test_values():
    x = ShortcutsDict()
    x['a']
    x['b'] = 'spam'
    x['c']['d'] = 'eggs'
    del x['c']
    x['e']['f'] = 'green'
    r = x.values()
    if type(r[0]) == types.StringType:
        assert r[0] == 'spam'
        assert r[1]['f'] == 'green'
    else:
        assert r[1] == 'spam'
        assert r[0]['f'] == 'green'

def test_items():
    x = ShortcutsDict()
    x['a']
    x['b'] = 'spam'
    x['c']['d'] = 'eggs'
    del x['c']
    x['e']['f'] = 'green'
    r = x.items()
    r.sort()
    assert r[0] == ('b', 'spam')
    assert r[1][0] == 'e'
    assert r[1][1]['f'] == 'green'

def test_get_with_default():
    x = ShortcutsDict()
    assert x.get('a', 2) == 2
    x['a'] = 'b'
    assert x.get('a', 2) != 2
    x['b']
    assert x.get('b', 3) == 3
    x['b']['c'] = 'd'
    assert x.get('b', 1) != 1

if __name__ == '__main__':
    RunTests.runTests('ShortcutsDict')





