#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
# A system test suite for MojoNation.
#
# Birthdate: 2000-06-16
__cvsid = '$Id: SystemTest.py,v 1.3 2000/10/03 01:25:42 zooko Exp $'

### Imports:
# Standard Modules:
import threading, whrandom, os, sys, time, cStringIO, types, string, traceback
from urlparse import urlparse

from confutils import confman
import debug
import archive
from human_readable import hr
import randsource
import mojoutil
import publish
import MojoTransaction
import unsplit

### Globals:
identifier = 'system_test__'

class SystemTestException(Exception):
    """
    If this is raised, the given test will fail, but testing will continue.
    If a general exception is raised, the test will fail, and testing will stop.
    """
    pass

class ChildProcessManager:
    def __init__(self):
        self.procdata = []

    def spawn_process(self, path, args = (), confdir = apply(os.path.join, mojoutil.get_tempfile_name(pre = 'systemtest'))) :
        write('Starting external command %s.' % path)
        pid = system(command = path, args = args, confdir = confdir)
        self.procdata.append({
            'pid' : pid,
            'confdir' : confdir})

    def kill_items(self, signal=15):
        for pd in self.procdata :
            pid = pd['pid']
            write('Killing pid %d (with signal %d).' % (pid, signal))
            try :
                os.kill(pid, signal)
                write('Waiting for pid %d.' % pid)
                os.waitpid(pid, 0)
            except OSError, description:
                print 'desc = ' + str(description)
            os.system('rm -rf ' + pd['confdir'])

### Functions:
# Tools:
def write(output, newline='\n', colorcode=debug.COLORCODE_BLUE):
    """
    Writes hilighted output.  This is used to visually filter out output we are not as interested in (like other thread output).
    """
    sys.stdout.write(debug.hilight(output, colorcode))
    if newline:
        sys.stdout.write(newline)
    sys.stdout.flush()

def system(command, args, confdir) :
    """
    Executes commands in a sub-process.  Fork'n'exec's so there's no waiting.

    @param command type must be StringType.
    @param args type must be a TupleType or ListType.

    @return Returns the PID of the child.
    """
    assert type(command) == types.StringType, 'command "%s" must be a StringType.' % str(command)
    assert type(args) in (types.TupleType, types.ListType), 'args "%s" must be a TupleType or a ListType.' % str(args)

    pid = os.fork()
    if pid == 0 :
        # We are the child
        os.environ['EVILCONFDIR'] = confdir
        os.execvp(command, (command,) + tuple(args) + ('# mojotest',))
        raise 'AAAAAAAAAAAAAAAAAAAAAAAAAAAAUGH!'
    return pid

# Tests:
def system_test__publish_and_retrieve(comq):
    # comq.spawn_process(os.path.join(os.environ["EVILDIR"], "IdTracker"))
    # time.sleep(5)
    # comq.spawn_process(os.path.join(os.environ["EVILDIR"], "BlockServer"))
    # time.sleep(2)
    # comq.spawn_process(os.path.join(os.environ["EVILDIR"], "PublishingAgent.py"))
    # time.sleep(2)

    write('Creating Files of Random Data:')
    randomfiles = {}
    for n in range(3):
        filename = apply(os.path.join, mojoutil.get_tempfile_name('', "system_test.pubnret.", ".data"))
        write('Creating file "%s"' % filename, None)
        randomfiles[filename] = mojosixbit.b2a(randsource.get(whrandom.randint(3, 100)))
        write('.', None)
        file = open(filename, 'w')
        write('.', None)
        file.write(randomfiles[filename])
        write('.', None)
        file.flush()
        write('.', None)
        file.close()
        write('. - Done.')

    write('Creating archive of the random files', None)
    arcname = apply(os.path.join, mojoutil.get_tempfile_name('', "system_test.pubnret.", ".mojoarc"))
    write('in "%s"' % arcname, None)
    arc = archive.Creator(compression=archive.CTYPE_NONE)
    write('.', None)
    arc.queuefiles(randomfiles.keys())
    write('.', None)
    arc.writeto(arcname)
    write('. - Done.')

    write('Creating new test MTM', None)
    listenportstr = string.upper(confman.dict["TRANSACTION_MANAGER_LISTEN_PORT"])
    if listenportstr:
        listenport = int(listenportstr)
    else :
        listenport = None
    pickyport = confman.dict["TRANSACTION_MANAGER_PICKY_PORT"]

    TM = MojoTransaction.MojoTransaction(
            listenport=listenport,
            handler_funcs={},
            announced_service_dicts=[],
            pickyport=pickyport
            )
    write('. -Done.')

    write('Publishing archive', None)
    (arcid, dinode) = publish.publish_archive(MTM=TM,
        arcfile=open(arcname), arcsize=os.path.getsize(arcname),
        statusfile=sys.stdout, verbose=0)
    write('. -Done.')

    write('Retrieving archive "%s" with dinode "%s"' % (arcid, mojosixbit.b2a(dinode)))
    archivefile = unsplit.ReassemblerSplitFile(TM=TM, dinode=dinode)
    write('. - Done.')
    write('Reading files from archive:')
    arc = archive.Reader(archivefile)
    for randomfile in randomfiles.keys():
        write('Extracting file "%s"' % randomfile, None)
        memfile = cStringIO.StringIO()
        write('.', None)
        arc.seekpart(randomfile)
        write('.', None)
        arc.extract(memfile)
        write('.', None)
        if memfile.getvalue() != randomfiles[randomfile]:
            description = 'Retrieved file, names "%s", not equal to published file.\nPublished File:\n%s\nRetrieved File:\n%s\n'
            description = description % (randomfile, randomfiles[randomfile], memfile.getvalue())
            raise SystemTestException, description
        else:
            write('. - Done, file contents unchanged!') 
    comq.kill_items()

def system_test__token_withdrawal(comq):
    # comq.spawn_process(os.path.join(os.environ["EVILDIR"], 'IdTracker'))
    # time.sleep(5)
    # comq.spawn_process('python', args=('-c', 'import TokenServer\nts = TokenServer.TokenServer()\n'))
    # time.sleep(2)

    # Start the Paytool and withdraw 2000 mojo:
    write('Starting the Paytool')
    import Paytool
    mtm = MojoTransaction.MojoTransactionManager(
            handler_funcs={},
            announced_service_dicts=[] )
    pt = Paytool.Paytool(mtm)
    flag = threading.Event()
    def callback(widget, msgbody, flag=flag):
        flag.set()
    pt.withdraw(2000)

    write('Waiting 5 seconds to get the mojo.')
    flag.wait(5)
    mymojo = pt.get_amount_in_purse()
    if mymojo == 2000 :
        raise SystemTestException, "The withdrawal was successful, despite a bank balance of 0."
    if mymojo != 0 :
        raise SystemTestException, "I have %d mojo in my purse, but I should have 0." % mymojo

    write("Withdrawal of non-existent funds handled correctly.")

    comq.kill_items()

### Main Procedure:
def run(testfuncs = filter(lambda funcname : funcname[:len(identifier)] == identifier, dir()) ) :
    comq = ChildProcessManager()
    results = {}

    write("Running System Tests:")
    for funcname in testfuncs:
        write('\n\n\n-<0>- Test "%s":\n\n\n' % funcname)
        try:
            exec "%s(comq)" % funcname
            write('Test "%s" Passed.' % funcname)
            results[funcname] = 'Passed.'
        except SystemTestException, description:
            write('\nTest "%s" Failed, because: %s\n\n\n' % (funcname, description))
            tracebacklist = traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)
            tracebacklist.insert(0, string.join(['Known Failure:\n']))
            results[funcname] = string.join(tracebacklist, '')
        except Exception, description:
            write('\nTest "%s" Failed, because: %s\nSee Results Summary below.' % (funcname, description))
            tracebacklist = traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)
            tracebacklist.insert(0, string.join(['UNKNOWN Failure:\n']))
            results[funcname] = string.join(tracebacklist, '')
    write("Done Testing.\n\n\n")

    write("Cleaning up lingering processes:")
    time.sleep(2)
    comq.kill_items(9)
    write("Done cleaning up child processes.\n\n\n")
    #write("Cleaning up those *really* annoying daemon threadlike beasts...")
    #try:
    #    while 1:
    #        pid, exitstatus = os.wait()
    #        write('Cleaned up pid %d which had exit status %d.' % (pid, exitstatus))
    #except OSError:
    #    write('Done cleaning up the *really* annoying daemon threadlike beasts.')

    write("Results Summary:", colorcode=debug.COLORCODE_GREEN)
    for testname in results.keys():
        write('%s: %s' % (testname, results[testname]), colorcode=debug.COLORCODE_GREEN)

if __name__ == '__main__' :
    run()

