#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
__cvsid = '$Id: UnreliableHandicapper.py,v 1.17 2001/06/08 00:50:42 greg Exp $'

# standard modules
import whrandom

# our modules
from MojoHandicapper import DISQUALIFIED
import confutils
import debug
from human_readable import hr
import idlib
import mojoutil
from MojoHandicapper import DISQUALIFIED

class UnreliableHandicapper:
    def __init__(self, counterparties, our_id):
        self.counterparties = counterparties
        self.our_id = our_id

    def __call__(self, counterparty_id, metainfo, message_type, message_body):
        """
        for all msgtypes

        @returns 1200 times a counterparties unreliability factor.
            disqualifies highly unreliable counterparties most of the time.
        """
        if idlib.equal(counterparty_id, self.our_id):
            return 0  # no handicap for us, we have high self esteem
        else:
            cpty = self.counterparties.get_counterparty_object(counterparty_id)
            reliability = cpty.get_custom_stat("reliability", 1.0)

        if message_type == 'put blob':
            if reliability < 0.2:
                if whrandom.random() > reliability:
                    # don't bother with these guys this time...
                    return DISQUALIFIED
                else:
                    debug.mojolog.write('Handicap allowing %s with reliability %s another chance for %s\n', args=(counterparty_id, reliability, message_type), v=8, vs='UnreliabilityHandicapper')

        unreliability = 1.0 - reliability

        handicap = unreliability * 1200 # XXX tune me
        return handicap

