#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

# standard modules
import os.path
import string

# our modules
import confutils
import localblockstore
import debug

# A list of BlockStore objects that block servers use and that clients
# will look & cache in before going elsewhere to get a block.
# NOTE: the 1024*1000 computation for megabytes should cover the
# overhead used by the index database and directory structure by
# erring on the safe side.
if string.lower(confutils.confman.dict["BLOCKSERVER"]["USE_MULTIPLE_STORES"]) in ("yes", "true"):
    debug.mojolog.write("Using Multiple BlockStores as configured\n", vs='blockstorelib')
    _tmp_bslist = []
    store_config_list = confutils.confman.getlist(["BLOCKSERVER", "STORAGE_SPACE_LIST"])
    num = 0
    for sc in store_config_list:
        if string.lower(sc['READONLY']) in ("yes", "true"):
            _bsinstance = localblockstore.ReadOnlyBLBlockStore
        else:
            _bsinstance = localblockstore.BLBlockStore
        _tmp_bslist.append( _bsinstance(path=os.path.expandvars(sc['PATH']), maxspace=1024*1000*long(sc['MAX_MEGABYTES']), depth=int(sc['SUBDIRDEPTH']), name="block server storage #%02d" % num) )
        num = num + 1
    if len(_tmp_bslist) < 0:
        debug.mojolog.write("No BlockStores configured!\n", vs="ERROR")
    if len(_tmp_bslist) > 20:
        debug.mojolog.write("Using %s seperate BlockStores, that's a lot!\n", vs="WARNING")
    debug.mojolog.write("Using %s BlockStores\n", args=(len(_tmp_bslist),), v=2, vs='blockstorelib')
    BLOCK_STORES = [ localblockstore.MultiBlockStore(_tmp_bslist) ]
else:
    BLOCK_STORES = [ localblockstore.BLBlockStore(path=os.path.expandvars(confutils.confman.dict["PATH"]["BLOCK_SERVER_DIR"]), maxspace=1024*1000*long(confutils.confman.dict["BLOCKSERVER"]["MAX_MEGABYTES_OF_DISK_TO_USE"]), recoverdb=(confutils.confman.dict["BLOCKSERVER"]["AUTO_RECOVER_DB"]=="true"), name="block server storage") ]

# if a downloaded block cache has been configured, set it up
size_for_downloaded_block_cache = 1000*1024*long(confutils.confman.dict["DOWNLOADED_BLOCK_DISK_CACHE_MEGABYTES"])
if size_for_downloaded_block_cache > 0 :
    DOWNLOAD_CACHE = localblockstore.BlockStore(path=os.path.expandvars(confutils.confman.dict["PATH"]["DOWNLOAD_CACHE_DIR"]), maxspace=size_for_downloaded_block_cache, recoverdb=((confutils.confman.dict["BLOCKSERVER"]["AUTO_RECOVER_DB"]=='true') or (confutils.confman.dict["BLOCKSERVER"]["AUTO_RECOVER_DB"]=='yes')), name="download cache")
else :
    DOWNLOAD_CACHE = None


def clean_shutdown():
    global BLOCK_STORES
    if BLOCK_STORES:
        for b in BLOCK_STORES:
            b.shutdown()
    global DOWNLOAD_CACHE
    if DOWNLOAD_CACHE:
        DOWNLOAD_CACHE.shutdown()
