import os
import sys
import string
import re
# distutils is included with python 1.6 and later, go grab a copy if you use python 1.5.2.
from distutils.core import setup, Extension


if os.environ.has_key('CRYPTOPP_DIR'):
    cryptoppdir = os.environ['CRYPTOPP_DIR']
    if not os.path.isdir(cryptoppdir) :
        raise SystemExit, "Your CRYPTOPP_DIR environment variable is incorrect."
else:
    raise SystemExit, "Your CRYPTOPP_DIR environment variable must be set."


extra_inc_dirs = []
library_dirs = []
libraries = []

# XXX this hack because the distutils included with python 1.6 accepts
# but doesn't support the define_macros parameter to Extension() seen
# below.  It does support the CFLAGS environment variable.  Getting a
# later version of distutils (presumably the one included with python 2.0?)
# will hopefully make this unnecessary.
if sys.platform == 'win32':
    # vc++ complained to me and told me to add /GX so I did
    os.environ['CFLAGS'] = '/DPYTHON_MODULE /DWIN32 /GX'
    extra_inc_dirs.append(os.environ['EXTSRCWINDIR'] + r'\Python-1.6\PC')
    library_dirs.extend([os.path.join(sys.prefix, 'PCbuild'), os.path.join(cryptoppdir, 'Release')])
    libraries.append('cryptlib-mojo')
else:
    # win32 isn't using crypto++ 4.0 yet so definining CRYPTOPP_40 is necessary when we want 4.0
    os.environ['CFLAGS'] = '-DPYTHON_MODULE'
    if not re.search('(OpenBSD|irix)', sys.platform, re.I) and \
       (not re.search('egcs', os.popen("gcc --version 2>&1").read())) and \
          not os.environ.get('USE_CRYPTOPP_32', ""):
        # Use Crypto++ 4.0 unless we're on OpenBSD where it appears to still cause C++ namespace & linking errors as a python module
	# Or on Irix where Irix's CC and g++ can't seem to handle the massive c++ template f*** of 4.x
        # Also, crypto++ 4.0 causes redhat 6.1's egcs compiler to barf (internal compiler error)
        os.environ['CFLAGS'] = os.environ['CFLAGS'] + ' -DCRYPTOPP_40'
    library_dirs.extend([cryptoppdir])
    libraries.extend(['cryptopp', 'stdc++'])

# On FreeBSD we -finally- found that -lgcc was the magic needed linker flag to
# prevent the "undefined symbol: __pure_virtual" problem when attempting to
# import the evilcryptopp.so module.
if re.search('bsd', sys.platform, re.I):
    libgcc_dir = os.path.dirname( os.popen("gcc -print-libgcc-file-name").readline()[:-1] )
    library_dirs.append(libgcc_dir)
    libraries.append('gcc')

if os.environ.has_key('CRYPTOPP_DIR'):
    cryptoppdir = os.environ['CRYPTOPP_DIR']
    if not os.path.isdir(cryptoppdir) :
        raise SystemExit, "Your CRYPTOPP_DIR environment variable is incorrect."
else:
    raise SystemExit, "Your CRYPTOPP_DIR environment variable must be set."

setup ( name = "evilcryptopp",
        version = "1.0.0",
        author = "Autonomous Zone Industries",
        author_email = "mojonation-devel@lists.sourceforge.net",
        url = "http://sourceforge.net/projects/mojonation/",
        ext_modules = [
            Extension(
                "evilcryptopp",
                ["modval.cpp", "../wrappedrsa.cpp", "../randsource_methods.cpp", "evilcryptopp.cpp", "tripledescbc.cpp", "randsource.cpp"],
                # The BERKELEYDB_DIR environment variable is required
                include_dirs=["../", cryptoppdir]+extra_inc_dirs,
                library_dirs=library_dirs,
                libraries=libraries,
            )
        ],
      )

