#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

import exceptions
import types


def recursive_dict_update(a, b):
    '''This is much like a.update(b), except when a value in b is a dict, it
    recurses.  Thus it acts "deep" instead of "shallow".'''
    for k, v in b.items():
        if type(v) == types.DictType:
            try:
                try:
                    recursive_dict_update(a[k], v)
                except KeyError:
                    # a[k] is not defined yet, so:
                    a[k] = {}
                    recursive_dict_update(a[k], v)
            except exceptions.StandardError, le:
                le.args = (le.args, "in `recursive_dict_update(%s[%s], %s)" % (`a`, `k`, `v`))
                raise le
        else:
            a[k] = v
    return


def add_long_to_dict(dict, key, val):
    """
    If the key doesn't appear in dict then it is created with value 0 (before addition).
    """
    old = dict.get(key)
    if not old:
        dict[key] = val
    else:
        dict[key] = old + val
    

def add_long_to_dict_TEMP_DB(dict, key, val):
    """
    If the key doesn't appear in dict then it is created with value 0 (before addition).
    """
    old = long(dict.get(str(key)))
    if not old:
        dict[str(key)] = str(val)
    else:
        dict[str(key)] = str(old + val)
    

def subtract_long_from_dict(dict, key, val):
    """
    If the value goes down to 0 then the key is removed from the dict.

    If the value goes down to less than zero then an exception is raised.
    """
    old = dict.get(key)
    if not old:
        raise "bad usage"
    else:
        new = old - val
        if new == 0:
            del dict[key]
        else:
            dict[key] = new
    

mojo_test_flag = 1

#### generic stuff
def run():
    import RunTests
    RunTests.runTests(["dictutil"])
    
#### this runs if you import this module by itself
if __name__ == '__main__':
    run()

