#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
### standard modules
import sys
import string
import sha
import time
import os

if sys.platform == 'win32':
    # this is part of the win32all python package, get it from:
    # http://www.activestate.com/Products/ActivePython/win32all.html
    import win32api

### our modules
import randsource
import debug
import MojoConstants


def randinit():
    if MojoConstants.USE_REAL_RANDOM_SEED:
        # TODO add entropy gathering for other OSes
        if sys.platform == "win32" :
            # Anyone know good ways to gather more starting entropy on windows? 
            shabits = sha.sha()
            shabits.update(str(win32api.GetCursorPos()))
            shabits.update(str(time.time()))
            shabits.update(sys.exec_prefix)
            shabits.update(str(time.time()))
            shabits.update(str(win32api.GetCursorPos()))
            shabits.update(str(os.environ))
            shabits.update(str(win32api.GetCursorPos()))
            shabits.update(str(time.time()))
            shabits.update(str(win32api.GetCurrentProcessId()))
            shabits.update(str(sys.dllhandle))
            randsource.add(shabits.digest(), 160)
        elif string.find(sys.platform, "linux") >= 0 :
            urandomdata = open('/dev/urandom', 'rb').read(20)
            randsource.add(urandomdata, len(urandomdata)*8)
        elif string.find(string.lower(sys.platform), "bsd") >= 0 :
            urandomdata = open('/dev/urandom', 'rb').read(20)
            randsource.add(urandomdata, len(urandomdata)*8)
        else :
            debug.stderr.write('WARNING: no good random entropy source known found for this OS\n')
            randsource.add(sha.sha( sys.platform + sys.version + str(time.time()) ).digest(), 160)

    else:
        debug.stderr.write("WARNING: NOT using a real random seed!  Instead, I'm using the number ZERO, for testing purposes.  This is not secure.\n")
        randsource.add("00000000000000000000", 160)
