#!/usr/bin/env python
#
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#

import string
import copy
import os
import re
import sys
import time


import modval
import randsource
import SEXP
import MojoKey
from MojoConstants import true, false
import mojoutil
import MojoConstants
import idlib
import mesgen


TEMPDIR=os.path.join(os.path.join(os.sep, "var"), "tmp")

def make_all_keys_with_silly_abbrevs(lengthinbytes, pubexp, seed="12345678901234567890", commsortokensigning="comms"):
    """
    Caution: this can take a long time!

    Uses the keys in "/tmp/foo.comms.key" and puts new ones into "/tmp/foo.comms.key".

    @param seed should be at least 20 bytes

    @precondition `commsortokensigning' must be either "comms" or "tokensigning".: (commsortokensigning == "comms") or (commsortokensigning == "tokensigning")
    """
    assert (commsortokensigning == "comms") or (commsortokensigning == "tokensigning"), "`commsortokensigning' must be either \"comms\" or \"tokensigning\"."

    sillyabbs = make_three_letter_syllables()

    numgenned = 0
    announced = false
    fnames = os.listdir(TEMPDIR)
    for fname in fnames:
        (base, ext) = os.path.splitext(fname)
        if ext == ".key":
            numgenned = numgenned + 1
            (base, innerext) = os.path.splitext(base)
            if base in sillyabbs:
                if not announced:
                    sys.stdout.write("Not regenerating already generated keys: <" + base + ">")
                    announced = true
                else:
                    sys.stdout.write(", <" + base + ">")
                sillyabbs.remove(base)


    sys.stdout.write("\n")
    if announced:
        sys.stdout.write("Number already generated: " + `numgenned` + "\n")

    sys.stdout.write("Number remaining to generate: " + `len(sillyabbs)` + "\n")

    while len(sillyabbs) > 0:
        testabb = None
        while not testabb in sillyabbs:
            testmv = modval.new_random(lengthinbytes, pubexp)
            if commsortokensigning == "comms":
                testsexp = MojoKey.makePublicRSAKeyForCommunicating(testmv)
            else:
                testsexp = MojoKey.makePublicRSAKeyForVerifyingTokenSignatures(testmv)

            testabb = idlib.id_to_abbrev(idlib.string_to_id(testsexp))

            # time.sleep(1) # Give my CPU in imp a chance to cool down.  --Z
        
        newfname = os.path.join(TEMPDIR, testabb + "." + commsortokensigning + "." + "key")
        f = open(newfname, "w")
        f.write(`(testsexp, testmv.get_private_key_encoding(), mojosixbit.b2a(idlib.string_to_id(testsexp)), idlib.id_to_abbrev(idlib.string_to_id(testsexp)))`)
        f.close()

        sillyabbs.remove(testabb)
        print "Generated new key: <" + testabb + ">, number remaining to generate: " + `len(sillyabbs)`


_3lws = [ "Abe", "Abo", "Abu", "ace", "act", "Ada", "add", "ado", "ads", "aft", "age", "ago", "aid", "ail", "aim", "air", "ale", "Ali", "all", "Amy", "and", "Ann", "ant", "any", "ape", "apt", "arc", "are", "ark", "arm", "art", "ash", "ask", "asp", "ass", "ate", "awe", "awl", "aye", "bad", "bag", "bah", "ban", "bar", "bat", "bay", "bed", "bee", "beg", "Ben", "bet", "bib", "bid", "big", "bin", "bit", "boa", "bob", "bog", "boo", "bow", "box", "boy", "bra", "bud", "bug", "bum", "bun", "bus", "but", "buy", "bye", "cab", "cam", "can", "cap", "car", "cat", "caw", "cod", "cog", "con", "coo", "cop", "cot", "cow", "cry", "cub", "cue", "cup", "cut", "dad", "dam", "Dan", "Dar", "day", "DEC", "Dee", "Del", "den", "Des", "dew", "did", "die", "dig", "dim", "din", "dip", "Dis", "Doc", "doe", "dog", "don", "dot", "Dow", "dry", "dub", "dud", "due", "dug", "dye", "ear", "eat", "ebb", "eel", "egg", "ego", "eke", "elf", "Eli", "elk", "elm", "Ely", "end", "Eng", "era", "ere", "erg", "err", "Eva", "Eve", "ewe", "eye", "fag", "fan", "FAQ", "far", "fat", "fed", "fee", "fen", "few", "fib", "fig", "fin", "fir", "fit", "fix", "Flo", "flu", "fly", "fob", "foe", "fog", "for", "fox", "fro", "fry", "fun", "fur", "gab", "gad", "gag", "gap", "gas", "gay", "gel", "gem", "get", "gig", "Gil", "gin", "gnu", "Goa", "god", "got", "gum", "gun", "Gus", "gut", "guy", "had", "hag", "Hal", "ham", "Han", "hap", "has", "hat", "hay", "hem", "hen", "her", "hew", "hex", "hey", "hid", "him", "hip", "his", "hit", "hoe", "hog", "Hom", "hop", "hot", "how", "hub", "hue", "hug", "huh", "hum", "Hun", "hut", "Ian", "Ibn", "ice", "icy", "Ida", "Ike", "ill", "imp", "ink", "inn", "ion", "Ira", "ire", "irk", "Ito", "its", "ivy", "jab", "jam", "jar", "jaw", "jay", "Jed", "jet", "Jew", "jig", "Jim", "job", "Joe", "jog", "Jon", "jot", "joy", "jug", "jut", "Kay", "ken", "key", "kid", "Kim", "kin", "kit", "lab", "Lac", "lad", "lag", "Lao", "lap", "law", "lax", "lay", "led", "lee", "leg", "Len", "Leo", "let", "Lev", "Lew", "lid", "lie", "Lin", "lip", "lit", "Liz", "Los", "lot", "Lou", "low", "Loy", "Mac", "mad", "Mae", "man", "Mao", "map", "mat", "Max", "May", "Meg", "Mel", "men", "met", "mew", "mid", "mix", "mob", "Moe", "moo", "mop", "mow", "mud", "mug", "nab", "nag", "Nan", "nap", "Nat", "nay", "Ned", "net", "new", "nil", "nip", "nod", "non", "nor", "not", "now", "nun", "nut", "oaf", "oak", "oar", "oat", "odd", "ode", "off", "oft", "ohm", "oil", "old", "one", "opt", "orb", "ore", "Orr", "Ott", "our", "out", "owe", "owl", "own", "pad", "pal", "Pam", "pan", "par", "pat", "paw", "pay", "Paz", "pea", "peg", "pen", "pep", "per", "pet", "pew", "phi", "pie", "pig", "pin", "pip", "pit", "ply", "pod", "Poe", "pop", "pot", "pox", "pro", "pry", "pub", "pun", "pup", "pus", "put", "qua", "quo", "Rae", "rag", "ram", "ran", "rap", "rat", "raw", "ray", "red", "Rex", "rho", "rib", "rid", "rig", "rim", "Rio", "rip", "rob", "rod", "roe", "Ron", "rot", "row", "Roy", "rub", "rue", "rug", "rum", "run", "rut", "rye", "sad", "sag", "Sal", "Sam", "San", "Sao", "sap", "sat", "saw", "sax", "say", "sea", "see", "set", "sew", "sex", "she", "Shu", "shy", "sip", "sir", "sit", "six", "ski", "sky", "sly", "sob", "sod", "Sol", "son", "sow", "soy", "spa", "spy", "Sri", "Stu", "sub", "sue", "sum", "sun", "Sus", "tab", "tag", "tan", "tap", "tar", "tau", "tax", "tea", "Ted", "ten", "TeX", "Tex", "the", "tie", "Tim", "tin", "tip", "tit", "toe", "Tom", "ton", "too", "top", "tow", "toy", "try", "tub", "tug", "two", "ugh", "Uri", "urn", "use", "Uzi", "van", "vat", "VAX", "vex", "via", "Vic", "vie", "vow", "wag", "wan", "war", "was", "wax", "way", "web", "wee", "Wei", "wet", "who", "why", "wig", "win", "wit", "woe", "won", "woo", "xxx", "yea", "yes", "yet", "yon", "you", "Yuh", "Zan", "Zen", "Zoe", "zoo", "bax", "bux", "cox", "dex", "dox", "dux", "hax", "Hix", "kix", "lax", "lix", "lox", "lux", "mox", "nex", "nix", "pax", "pex", "pix", "rax", "rox", "sax", "sox", "sux", "tix", "tux", "wix", "FBI", "CIA", "NSA", "IRS", "INS", "puk", "ixL", "ick", "haQ", "OB1" ]


_2lws = [ "ad", "ah", "Al", "am", "an", "as", "at", "ax", "be", "by", "De", "do", "Ed", "et", "ex", "Fe", "go", "ha", "he", "hi", "if", "in", "Io", "is", "it", "Jo", "me", "mu", "my", "no", "of", "oh", "on", "or", "ox", "Oz", "pi", "Po", "re", "so", "to", "up", "us", "we", "Wu" ]

CONSONANTS=['b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z']
VOWELS=['a', 'e', 'i', 'o', 'u'] 

CAPCONSONANTS=['B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'R', 'S', 'T', 'V', 'W', 'X', 'Y', 'Z']
CAPVOWELS=['A', 'E', 'I', 'O', 'U']

BOTHCONSONANTS=['b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z', 'B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'R', 'S', 'T', 'V', 'W', 'X', 'Y', 'Z']
BOTHVOWELS=['a', 'e', 'i', 'o', 'u', 'A', 'E', 'I', 'O', 'U']

ALLLETTERS=['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z' ]
CAPALLLETTERS=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' ]
BOTHALLLETTERS=['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z' , 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' ]

NUMERALS=['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']


def make_three_letter_words():
    """
    More recognizable than the output of `make_three_letter_syllables()'.
    """
    myres = copy.copy(_3lws)

    for num in NUMERALS:
        myres.append(num + num + num)
        myres.append("00" + num)
       
       
    return myres


def make_three_letter_syllables():
    """
    More numerous than the output of `make_three_letter_words()'.
    """

    myres = make_three_letter_words()

    for one in BOTHCONSONANTS:
        for two in VOWELS:
            for three in CONSONANTS:
                myres.append(one + two + three)

    for one in BOTHVOWELS:
        for two in CONSONANTS:
            for three in VOWELS:
                myres.append(one + two + three)

    for one in CAPCONSONANTS:
        for two in CAPVOWELS:
            for three in CAPCONSONANTS:
                myres.append(one + two + three)

    for one in CAPVOWELS:
        for two in CAPCONSONANTS:
            for three in CAPVOWELS:
                myres.append(one + two + three)
       
    return myres


def make_three_letter_syllables_plus_numerals():
    """
    More numerous than the output of `make_three_letter_words()'.
    """

    myres = make_three_letter_words()

    for twoletword in _2lws:
        for num in NUMERALS:
            myres.append(num + twoletword)
            myres.append(twoletword + num)

    for one in BOTHCONSONANTS:
        for two in VOWELS:
            for three in CONSONANTS:
                myres.append(one + two + three)

    for one in BOTHVOWELS:
        for two in CONSONANTS:
            for three in VOWELS:
                myres.append(one + two + three)

    for one in CAPCONSONANTS:
        for two in CAPVOWELS:
            for three in CAPCONSONANTS:
                myres.append(one + two + three)

    for one in CAPVOWELS:
        for two in CAPCONSONANTS:
            for three in CAPVOWELS:
                myres.append(one + two + three)

    for let in ALLLETTERS: # Note: don't want ambiguities by including caps.
        for numone in NUMERALS:
            for numtwo in NUMERALS:
                myres.append(let + numone + numtwo)
                myres.append(numone + numtwo + let)
       
    return myres


def run():
    if len(sys.argv) > 1:
        commsortokensigning = sys.argv[1]
    else:
        commsortokensigning = "comms"

    seed = repr(time.time()) + repr(time.time()) + repr(time.time())


    print seed
        
    make_all_keys_with_silly_abbrevs(MojoConstants.SIZE_OF_MODULAR_VALUES, mesgen.HARDCODED_RSA_PUBLIC_EXPONENT, seed=seed, commsortokensigning=commsortokensigning)
    pass



#### this runs if you import this module by itself

if __name__ == '__main__':
    run()

