#! /usr/bin/env python

# This is an extension mod for the Mojo Nation Broker which provides a python tkinter
# basic interface.
#
# Requirements:
# Python 2.0 (-for the "webbrowser" module.)
# Tkinter
# Mojo Nation Broker

# Standard Python modules:
import os
import re
import time
import threading # For the GUI event thread.
import webbrowser
import traceback

# TkInter modules:
import Tkinter
import tkFont
from Tkconstants import *
import ScrolledText
import Dialog

# Mojo Nation Modules:
import MojoMod

### Constants:
DEFAULT_MAX_LOG_LENGTH = 256 * 1024 # Log Length in characters.
BGCOLOR='#0099cc'
MENUBGCOLOR='#000066'
ACTIVE_LINK_COLOR='#aa0022'
LOG_ENTRY_REGEX = re.compile(r"^DEBUG( \((?P<AUTHOR>[^\)]*)\))?: \[(?P<TIMESTAMP>[^\]].*)\]: (?P<MESSAGE>.*)$")

def main():
    print "This can only be used as a Mojo Nation extension module currently..."

class MojoModHook (MojoMod.MojoModHook):
    def __init__(self, broker):
        MojoMod.MojoModHook.__init__(self, broker)

        self.close_called = 0
        self.interface = BrokerInterface(broker, baseName="BrokerTk")
        self.gui_thread = threading.Thread(target=self.interface.mainloop)
        self.gui_thread.setDaemon(1)
        self.log('Launching Tkinter GUI thread.\n')
        self.gui_thread.start()

    def close(self):
        if not self.close_called:
            self.log('Closing BrokerTk interface.\n')
            self.interface.after(300, self.interface.shutdown)
            self.close_called = 1
        else:
            self.interface.destroy()

class LogMixin:
    def __init__(self, broker):
        self.broker = broker
    def log(self, *args):
        apply(self.broker.mojolog.write, args, {'vs': 'BrokerTk'})
        
class LinkLabel (Tkinter.Label, LogMixin):
    def __init__(self, master, broker, relurl, **kw):
        apply(Tkinter.Label.__init__, (self, master), kw)
        LogMixin.__init__(self, broker)

        self.url = 'http://localhost:%d%s%s' % (self.broker.mojohttpd.globaldict['port'], self.broker.mojohttpd.Presentation.variables.get('baseurl'), relurl)

        self.bind('<Enter>', lambda event, self=self : self.config(relief=RAISED))
        self.bind('<Leave>', lambda event, self=self : self.config(relief=FLAT))
        self.bind('<Button-1>', self.link_action)

    def link_action(self, e):
        oldcolor = self.cget('fg')
        self.config(relief=SUNKEN, fg=ACTIVE_LINK_COLOR)
        self.update_idletasks()
        self.broker.mojohttpd.launch_browser(self.url, newthread=0, newwindow=0)
        self.config(relief=FLAT, fg=oldcolor)

class BrokerInterface (Tkinter.Tk, LogMixin):
    def __init__(self, broker, *args, **kw):
        apply(Tkinter.Tk.__init__, (self,) + args, kw)
        LogMixin.__init__(self, broker)
        
        self.title('Mojo Nation Broker')
        self.config(bg=BGCOLOR)

        # Non-GUI Attributes:
        self.broker = broker
        self.log("GUI Interface instantiated.\n")

        # Widgets:
        self.mainframe = Tkinter.Frame(self, bg=BGCOLOR, borderwidth=2, relief=SUNKEN)
        #self.menuframe = Tkinter.Frame(
        #    self.mainframe, bg=MENUBGCOLOR, borderwidth=0, relief=FLAT)

        self.logwidget = LogWidget(self, self.broker, borderwidth=2, relief=SUNKEN)

        self.logoimg = Tkinter.PhotoImage(file=os.path.join(self.broker.confman.getpath(('PATH','WEBROOT_DIR')), 'images/m_logoweb.gif'))
        self.logo = LinkLabel(self.mainframe, self.broker, '/', image=self.logoimg, bg=BGCOLOR)
        #self.titlelabel = Tkinter.Label(self.mainframe, text="BrokerTk", font=("Arial", 14, "bold"), bg=BGCOLOR)
        self.mm = MojoMeter(self.mainframe, self.broker, bg=BGCOLOR)

        self.statuslink = LinkLabel(self.mainframe, self.broker, '/status', text='log', bg=BGCOLOR)
        self.searchlink = LinkLabel(self.mainframe, self.broker, '/search', text='search/download', bg=BGCOLOR)
        self.trackerslink = LinkLabel(self.mainframe, self.broker, '/trackers.dtml', text='trackers', bg=BGCOLOR)
        self.publishlink = LinkLabel(self.mainframe, self.broker, '/publish', text='publish', bg=BGCOLOR)
        #self.modslink = LinkLabel(self.mainframe, self.broker, '/mods', text='mods', bg=BGCOLOR)
        self.configurelink = LinkLabel(self.mainframe, self.broker, '/configure', text='preferences', bg=BGCOLOR)
        self.helplink = LinkLabel(self.mainframe, self.broker, '/help', text='help', bg=BGCOLOR)
        self.exitlink = Tkinter.Label(self.mainframe, text='exit', bg=BGCOLOR)

        # Event/Protocol handlers:
        self.protocol("WM_DELETE_WINDOW", self.close_dialog)
        self.exitlink.bind('<Enter>', lambda event, self=self : self.exitlink.config(relief=RAISED))
        self.exitlink.bind('<Leave>', lambda event, self=self : self.exitlink.config(relief=FLAT))
        self.exitlink.bind('<Button-1>', lambda event, self=self : self.close_dialog())
        
        # Layout:
        self.mainframe.pack(side=TOP, fill=BOTH, expand=1)
        self.logwidget.pack(side=BOTTOM, fill=X, expand=0)

        self.logo.grid(row=0, column=0, rowspan=2, sticky=W)
        #self.titlelabel.grid(row=0, column=1, sticky=W + E)
        self.mm.grid(row=1, column=1, columnspan=3, sticky=W)

        self.searchlink.grid(row=0, column=2)
        self.trackerslink.grid(row=0, column=3)
        self.publishlink.grid(row=0, column=4)
        self.statuslink.grid(row=0, column=5)
        #self.modslink.grid(row=0, column=5)
        self.configurelink.grid(row=0, column=6)
        self.helplink.grid(row=0, column=7)
        self.exitlink.grid(row=0, column=8)

    def mainloop(self):
        try:
            return Tkinter.Tk.mainloop(self)
        except:
            self.broker.mojolog.write("Exception from GUI mainloop:\n", vs='BrokerTk')
            traceback.print_exc(file=self.broker.mojolog)
                
    def close_dialog(self):
        d = Dialog.Dialog(self, {'title': 'Quit Broker',
                          'text': 'Do what?',
                          'bitmap': Dialog.DIALOG_ICON,
                          'default': 0,
                          'strings': ('Shutdown', 'Cancel')})
        if d.num == 0:
            self.shutdown()
        
    def withdraw(self):
        """This overridden method calls withdraw on the log window, and then itself."""
        self.logwidget.tbtl.withdraw()
        Tkinter.Tk.withdraw(self)
        
    def reload(self):
        self.log('Reloading.\n')
        self.withdraw()
        self.broker.doq.add_task(self.broker.modman.reload, args=('BrokerTk',), delay=0)

    def stop(self):
        self.withdraw()
        self.broker.doq.add_task(self.broker.modman.stop, args=('BrokerTk',), delay=0)

    def shutdown(self):
        self.log('Shutdown.\n')
        self.withdraw()
        self.broker.exit()

class MojoMeter (Tkinter.Frame, LogMixin):
    def __init__(self, master, broker, **kw):
        apply(Tkinter.Frame.__init__, (self, master), kw)
        LogMixin.__init__(self, broker)
        
        self.imgpath = None

        self.text = Tkinter.Label(self, text="You've Got Good Mojo", bg=self.cget('bg'))
        self.text.pack(side=LEFT)


class LogWidget (Tkinter.Frame, LogMixin):
    """
    This widget is a single line status-bar widget which also controlls a ScrolledText window.
    As it is written to, the lines are displayed in both windows; replacing the current line in the status bar,
    and appending to the text window.

    Clicking on the status-bar widget toggles the visibility of the ScrolledText window.
    """
    def __init__(self, master, broker, *_args, **_kw):
        apply(Tkinter.Frame.__init__, (self, master) + _args, _kw)
        LogMixin.__init__(self, broker)
        
        self.broker.mojolog.filters.append(self.append_log)

        self.logq = []
        self.timestamp = 0
        self.maxlength = DEFAULT_MAX_LOG_LENGTH 
        self.length = 0
        
        # Widgets:
        self.toggle = Tkinter.Button(self, text='MojoLog', relief=RAISED, command=self._toggle_textbox_state)
        self.blurbbox = Tkinter.Text(self, height=1, bd=1, relief=FLAT)
        self.tbtl = Tkinter.Toplevel(self)
        self.maxlengthframe = Tkinter.Frame(self.tbtl)
        #self.tbtl.transient(self) # This seems the "expected" thing to do, but I hate this style of UI.
        self.maxlengthlabel = Tkinter.Label(self.maxlengthframe, text="Max Log Display Size (Kilobytes):")
        self.maxlengthinput = Tkinter.Text(self.maxlengthframe, height=1, bd=1, relief=SUNKEN)
        self.textbox = ScrolledText.ScrolledText(self.tbtl)

        # Event binding:
        #self.blurbbox.bind("<Button-1>", lambda event, self=self : self._toggle_textbox_state())
        self.blurbbox.unbind("<Enter>") # Don't change the mouse pointer to a text cursor.
        self.tbtl.protocol("WM_DELETE_WINDOW", self._toggle_textbox_state)
        self.maxlengthinput.bind('<Return>', self.set_maxlength)

        # Layout:
        self.toggle.pack(side=LEFT)
        self.blurbbox.pack(fill=BOTH, expand=1)
        self.tbtl.withdraw() # Hide the textbox initially.
        self.maxlengthlabel.pack(side=LEFT, expand=0)
        self.maxlengthinput.pack(side=LEFT, expand=1, fill=X)
        self.maxlengthframe.pack(side=TOP, expand=0, fill=X)
        self.textbox.pack(side=TOP, expand=1, fill=BOTH)

        # Start the log updates:
        self.master.after(1000, self.update_log_loop)

    def __del__(self):
        self.broker.mojolog.filters.remove(self.append_log)

    def destroy(self):
        self.broker.mojolog.filters.remove(self.append_log)
        Tkinter.Frame.destroy(self)
        
    def _toggle_textbox_state(self):
        if self.tbtl.state() in ['normal', 'zoomed']:
            self.tbtl.withdraw()
            self.toggle.config(relief=RAISED)
        else:
            self.maxlengthinput.delete("1.0", END)
            self.maxlengthinput.insert(END, str(self.maxlength/1024))
            self.tbtl.deiconify()
            self.toggle.config(relief=SUNKEN)
            
    def set_maxlength(self, event):
        try:
            self.maxlength = int(self.maxlengthinput.get("1.0", END)) * 1024
        except ValueError:
            pass
        self.maxlengthinput.delete("1.0", END)
        self.maxlengthinput.insert(END, str(self.maxlength/1024))
        return "break"

    def append_log(self, output):
        self.logq.append(output)
        return output # this is a DebugStream filter.

    def update_log_loop(self):
        while len(self.logq) > 0:
            self.update_log(self.logq.pop(0))
        self.master.after(500, self.update_log_loop)

    def update_log(self, output):
        # Chop down the memory used by the textbox:
        newlength = len(output)
        self.length += newlength

        self.textbox.config(state=NORMAL)
        if self.length > self.maxlength:
            # Delete as many _lines_ as we add.  If the output is huge and has few newlines, we could get
            # way over MAX_LOG_LENGTH!
            endindex = self.textbox.index("1.0 + %d chars" % newlength)
            endindex = self.textbox.index("%s.end" % endindex.split('.', 1)[0]) 
            self.textbox.delete('1.0', endindex)
            self.length -= newlength

        # Append the new log output to the textbox:
        self.textbox.insert(END, output)
        self.textbox.see(END)
        self.textbox.config(state=DISABLED)

        # If enough time has passed, update the blurbbox:
        if time.time() - self.timestamp > 1:
            # Mangle the log output:
            m = LOG_ENTRY_REGEX.match(output)
            if m is not None:
                d = m.groupdict()
                blurb = d['MESSAGE']
                if d.get('AUTHOR') is not None:
                    blurb = d['AUTHOR'] + ': ' + blurb

                self.blurbbox.config(state=NORMAL)
                self.blurbbox.delete('1.0', END)
                self.blurbbox.insert(END, blurb)
                self.blurbbox.config(state=DISABLED)
                self.timestamp = time.time()
    
if __name__ == '__main__':
    main()
