#!/usr/bin/env python
#
#  Copyright (c) 2001 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.
#
# This MojoMod implements a silly PingPong client for testing comms
# between two brokers at maximum speed.
#
__cvsid = '$Id: PingPongTest.py,v 1.1 2001/06/04 19:08:27 zooko Exp $'

### standard modules
import sys
import whrandom

# other mojo nation modules
import MojoMod
import idlib
import OurMessages


class EchoService:
    def __init__(self, broker):
        self.broker = broker
        self.receive_funcs = {
            'echo': (self.handle_echo, self.price_callback_for_echo),
            }

        self.broker.ServicesMTM.add_handler_funcs(self.receive_funcs)

    def shutdown(self):
        # unregister our handler functions and announced services with the MTM
        self.broker.ServicesMTM.remove_handler_funcs(self.receive_funcs.keys())
        self.broker = None

    def price_callback_for_echo(self, widget, incoming_type, incoming_body) :
        return 0

    def handle_echo(self, widget, msgbody):
        return msgbody


########################################################################
class MojoModHook(MojoMod.MojoModHook):
    def __init__(self, broker):
        self.mojolog = broker.mojolog
        self.broker = broker
        self.started = 0

        self.broker.add_post_startup_task(self._startup)

    def _startup(self):
        self.mojolog.write("Starting\n", vs="PingPong")

        self.echo_service = EchoService(self.broker)
        self.broker.echo_service = self.echo_service

        getnext = 0
        self.pingee = None
        for arg in sys.argv:
            self.mojolog.write('processing arg %s\n', args=(arg,), vs='PingPong')
            if getnext:
                self.pingee = arg
                break
            if arg == '--ping':
                getnext = 1
        
        self.started = 1

        if self.pingee:
            if not idlib.is_ascii_id(self.pingee):
                raise RuntimeError, (self.pingee, "bad pingee specified")
            self.mojolog.write("pingee set to %s, commencing in 30 seconds\n" % self.pingee, vs="PingPong")
            self.broker.doq.add_task(self.start_pinging, args=(self.pingee,), delay=30)
        else:
            self.mojolog.write("no pingee specified, just handling echos\n", vs='PingPong')
        
        OurMessages.templs['echo'] = OurMessages.ANY
        OurMessages.templs['echo response'] = OurMessages.ANY

    def close(self):
        if not self.started:
            return
        self.started = 0
        self.broker.echo_service = None
        if self.echo_service:
            self.mojolog.write("Stopping\n", vs='PingPong')
            self.echo_service.shutdown()
            self.echo_service = None

    def start_pinging(self, brokerid, numatonce=3):
        for x in xrange(numatonce):
            if not self.started:
                return
            self.broker.ServicesMTM.initiate(brokerid, 'echo', {}, whrandom.randint(0,10), outcome_func=self._handle_echo_response, post_timeout_outcome_func=self._handle_echo_response, notes=brokerid, timeout=20)

    def _handle_echo_response(self, widget, outcome, failure_reason, notes=None):
        if failure_reason is None:
            if whrandom.randint(0, 4) == 4:
                # "slowly" exponentially increase the number of echos we have running
                self.start_pinging(notes, numatonce=2)
            else:
                self.start_pinging(notes, numatonce=1)

