#!/usr/bin/env python
#
# An http proxy server which takes all requests to mojonation.net 
# and sends them to your mojonation broker as appropriate.
#
# based on a combination of code from the following sources:
#   the medusa tutorial at http://www.nightmare.com/medusa/index.html
#   munchy.py by Neil Schemenauer <nascheme@enme.ucalgary.ca>
#
__cvsid = '$Id: AsyncMojoProxy.py,v 1.13 2000/11/14 23:56:30 nejucomo Exp $'

#####################################################################
# Start user configuration
#####################################################################

# default port if none specified on command line
PORT = 8000
MOJO_NATION_SERVER_PORT = 4004

# debugging level,
#   0 = no debugging
#   1 = access debugging
#   2 = full debugging
#   3 = really full debugging
DEBUG_LEVEL = 2
ALLOW_DEBUG_ON_WIN32 = None

# the address to bind the server to
ADDR_TO_BIND_TO = '127.0.0.1'

#####################################################################
# End of user configuration
#####################################################################


import sys
import os
import time
import string
import re

import BaseHTTPServer
import urlparse
import mimetools
from stat import ST_MTIME
from cStringIO import StringIO

import socket
import asyncore
import asynchat


###############################################################################
def log(s, v=1, args=None):
    if v <= DEBUG_LEVEL:
        if args:
            sys.stdout.write(s % args)
        else:
            sys.stdout.write(s)

def dummylog(s, v=1, args=None):
    pass

if sys.platform == 'win32' and not ALLOW_DEBUG_ON_WIN32:
    log = dummylog


# hostnames we wish to check for a redirect to your local broker
# (we run a real-world server on all of these so that people without
# the mojo nation software will be directed to our site for info and
# instructions on how to quickly download and install it)
hosts_to_check_re = re.compile(r'^(www\.|)(mojonation|mojo-nation)\.(com|net|org)', re.I)

# note: this will match with or without a privacy code
paths_to_redirect_re = re.compile(r'^(/[A-Za-z0-9_-]{12,27}|)/(id|mojoid|dinode|broker|nexus|station)', re.I)

# this is used to get info about the running broker from its intropage
extract_brokerinfo_re = re.compile(r'-- Port: (?P<port>\d+) PrivacyCode: (?P<privacycode>\S+) --', re.I)

# we remove all referer headers matching this
referer_to_strip_re = re.compile(r'(www\.|)mojonation\.(com|net|org)(/[A-Za-z0-9_-]{12,27}|)/(id|mojoid|dinode|broker|nexus|station)', re.I)

Intropage_Filename = None
Last_Intropage_Stat = {ST_MTIME:None}  # XXX this dict is a hack so the if in get_privacy_code_and_port works ;)
Current_Privacy_Code = ""
Current_Broker_Port = MOJO_NATION_SERVER_PORT
Current_Broker_Address = "127.0.0.1"   # the mojonation broker runs on localhost by default


###############################################################################
def get_privacy_code_and_port() :
    """
    Checks if intropage.html has changed.  If so, it rereads the privacy code from it.

    Returns a tuple: (privacycode, broker port)
    """
    global Last_Intropage_Stat
    global Current_Privacy_Code
    global Current_Broker_Port
    try:
        newstat = os.stat(Intropage_Filename)
    except OSError:
        log('error getting status of intropage %s\n' % Intropage_Filename, 1)
        Last_Intropage_Stat = {ST_MTIME:None}
        return Current_Privacy_Code, Current_Broker_Port

    # only re-read the file if it has been modified
    if Last_Intropage_Stat[ST_MTIME] != newstat[ST_MTIME]:
        try:
            ipfile = open(Intropage_Filename, "r")
            m = extract_brokerinfo_re.search(ipfile.readline())
            if m :
                Current_Privacy_Code = '/' + m.group('privacycode')
                Current_Broker_Port = int(m.group('port'))
            Last_Intropage_Stat = newstat
        except:
            log('error reading intropage %s\n' % Intropage_Filename, 1)
    
    return Current_Privacy_Code, Current_Broker_Port


###############################################################################
class AsyncProxyError(StandardError): pass


###############################################################################
class AsyncHTTPProxySender(asynchat.async_chat):
    def __init__(self, receiver, id, host, port):
        asynchat.async_chat.__init__(self)
        self.receiver = receiver
        self.id = id
        self.set_terminator(None)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.host = host
        self.port = port
        try:
            self.connect( (host, port) )
        except socket.error, e:
            log('(%d) XXX %s\n' % (self.id, e))
            self.receiver.sender_connection_error(e)
            self.close()
            return

    def handle_connect(self):
        log('(%d) S handle_connect\n' % self.id, 3)
        try:
            if sys.platform != 'win32':
                self.socket.recv(0)  # check for any socket errors during connect
            self.receiver.sender_is_connected()
        except socket.error, e:
            log('(%d) OOO %s\n' % (self.id, e))
            if hasattr(self, 'receiver'):
                self.receiver.sender_connection_error(e)
            self.close()
            return
        log('(%d) sender connected\n' % self.id, 2)

    def return_error(self, e):
        log('(%d) sender got socket error: %s\n', args=(self.id, e), v=2)
        if isinstance(e, socket.error) and type(e.args) == type(()) and len(e.args) == 2:
            e = e.args[1]  # get the error string only
        self.receiver.error(404, 'Error connecting to <em>%s</em> on port <em>%d</em>: <b>%s</b>' % (self.host, self.port, e), response=str(e))
        self.close()

    def collect_incoming_data(self, data):
        if DEBUG_LEVEL >= 3:
            log('==> (%d) %s\n', args=(self.id, repr(data)), v=3)
        else:
            log('==> (%d) %d bytes\n', args=(self.id, len(data)), v=2)
        self.receiver.push(data)

    def handle_close(self):
        log('(%d) sender closing\n' % self.id, v=2)
        self.receiver.close_when_done()
        del self.receiver  # break circular reference
        self.close()

    def log(self, message):
        log('(%d) sender: %s\n', args=(self.id, message,), v=1)
    
    def log_info (self, message, type='info'):
        if __debug__ or type != 'info':
            log('%s: %s' % (type, message))
    

###############################################################################
class AsyncHTTPProxyReceiver(asynchat.async_chat):
    channel_counter = [0]

    def __init__(self, server, (conn, addr)):
        self.id = self.channel_counter[0]  # used during log calls
        try:
            self.channel_counter[0] = self.channel_counter[0] + 1
        except OverflowError:
            self.channel_counter[0] = 0
        asynchat.async_chat.__init__(self, conn)
        self.set_terminator('\n')
        self.server = server
        self.buffer = StringIO()

        # in the beginning there was GET...
        self.found_terminator = self.read_http_request
    
    def collect_incoming_data(self, data):
        self.buffer.write(data)
    
    def push_incoming_data_to_sender(self, data):
        if DEBUG_LEVEL >= 3:
            log('<== (%d) %s\n', args=(self.id, repr(data)), v=3)
        else:
            log('<== (%d) %d bytes\n', args=(self.id, len(data)), v=2)
        self.sender.push(data)

    #### to be used as a found_terminator method
    def read_http_request(self):
        request = self.buffer.getvalue()
        self.buffer = StringIO()

        log('%s - %s\n', args=(time.ctime(time.time()), request), v=0)

        # client-originated shutdown hack:
        if string.strip(request) == 'quit':
            log('External quit command received.\n')
            # On pre python 2.0 this will raise a NameError,
            # but asyncore will handle it well.  On 2.0, it
            # will cause asyncore to exit.
            raise asyncore.ExitNow

        try:
            self.method, self.url, self.protocol = string.split(request)
            self.method = string.upper(self.method)
        except:
            self.error(400, "Can't parse request")
        if not self.url:
            self.error(400, "Empty URL")
        if self.method not in ['CONNECT', 'GET', 'HEAD', 'POST', 'PUT']:
            self.error(501, "Unknown request method (%s)" % self.method)
        if self.method == 'CONNECT':
            self.netloc = self.url
            self.scheme = 'https'
            self.path = ''
            params, query, fragment = '', '', ''
        else:
            # split url into site and path
            self.scheme, self.netloc, self.path, params, query, fragment = urlparse.urlparse(self.url)
            if string.lower(self.scheme) != 'http':
                self.error(501, "Unknown request scheme (%s)" % self.scheme)
        # find port number
        if ':' in self.netloc:
            self.host, self.port = string.split(self.netloc, ':')
            self.port = string.atoi(self.port)
        else:
            self.host = self.netloc
            if self.method == 'CONNECT':
                self.port = 443  # default SSL port
            else:
                self.port = 80
        path_match = paths_to_redirect_re.match(self.path)
        # redirect mojonation.net urls to the local broker as appropriate
        if hosts_to_check_re.match(self.host) and path_match :
            self.original_host_and_port = (self.host, self.port)
            self.host = Current_Broker_Address
            privacycode, self.port = get_privacy_code_and_port()
            if len(path_match.group(1)) == 0 :   # if no privacy code was supplied, add one
                # TODO now we need some way of authenticating the user in here if we're really
                # going to support automatic adding of the privacy code like this.  <ugh..>
                self.path = privacycode + self.path
            log("redirecting to mojo broker as %s\n" % self.path, 2)
            # put this header in so that the broker knows we came via the proxy
            self.reveal_mojoproxy = 1
        else :
            self.original_host_and_port = None
            self.reveal_mojoproxy = 0
        self.path = urlparse.urlunparse(('', '', self.path, params, query, fragment))

        self.rawheaders = StringIO()  # a "file" to read the headers into for mimetools.Message
        self.found_terminator = self.read_http_headers

    #### to be used as a found_terminator method
    def read_http_headers(self):
        header = self.buffer.getvalue()
        self.buffer = StringIO()
        if header and header[0] != '\r':
            self.rawheaders.write(header)
            self.rawheaders.write('\n')
        else:
            # all headers have been read, process them
            self.rawheaders.seek(0)
            self.mimeheaders = mimetools.Message(self.rawheaders)
            if self.reveal_mojoproxy:
                self.mimeheaders['mojoproxy'] = 'yes'
            if (self.method == 'POST' or self.method == 'PUT') and not self.mimeheaders.has_key('content-length'):
                self.error(400, "Missing Content-Length for %s method" % self.method)
            self.length = int(self.mimeheaders.get('content-length', 0))
            del self.mimeheaders['accept-encoding']
            del self.mimeheaders['proxy-connection']
            # strip off referer from urls we don't want referer headers on
            referer = self.mimeheaders.get('referer', 0)
            if referer and referer_to_strip_re.search(self.mimeheaders['referer']) :
                # IMHO, we should -always- do this but unfortunately some
                # stupid web sites probably depend on it.  -greg
                del self.mimeheaders['referer']
            if self.port == 80:
                self.mimeheaders['host'] = self.host
            else:
                # XXX hmm.. is the host header ever supposed to have the port on it?
                # Shouldn't the server know that already?  (I ask because some don't
                # like seeing a :80 on it [such as sourceforge.net])
                self.mimeheaders['host'] = self.host + ':' + str(self.port)

            # determine the next hop (another proxy or the remote host) and open a connection
            http_proxy = os.environ.get('http_proxy')
            if not http_proxy :
                http_proxy = os.environ.get('HTTP_PROXY')
            # if we're chaining to another proxy, modify our request to do that
            if http_proxy and self.host != Current_Broker_Address :
                scheme, netloc, path, params, query, fragment = urlparse.urlparse(http_proxy)
                if string.lower(scheme) == 'http' :
                    log('using next http proxy: %s\n' % netloc, 2)
                    # set host and port to the proxy
                    if ':' in netloc:
                        self.host, self.port = string.split(netloc, ':')
                        self.port = string.atoi(self.port)
                    else:
                        self.host = netloc
                        self.port = 80
                    # replace the path within the request with the full URL for the next proxy
                    self.path = self.url

            # create a sender connection to the next hop
            self.sender = AsyncHTTPProxySender(self, self.id, self.host, self.port)

            # send the request to the sender (this is its own method so that the sender can trigger
            # it again should its connection fail and it needs to redirect us to another site)
            self.push_request_to_sender()
    
    def push_request_to_sender(self):
        request = '%s %s HTTP/1.0\r\n%s\r\n' % (self.method, self.path, string.join(self.mimeheaders.headers, ''))

        if self.original_host_and_port:
            log('(%d) sending request to local mojonation broker:\n' % self.id, v=2)
        elif http_proxy:
            log('(%d) sending request to the next http proxy:\n' % self.id, v=2)
        else :
            log('(%d) sending request to server:\n' % self.id, v=2)
        log(request, v=2)

        # send the request and headers on through to the next hop
        self.sender.push(request)

        # no more formatted IO, just pass any remaining data through
        self.set_terminator(None)

        # buffer up incoming data until the sender is ready to accept it
        self.buffer = StringIO()
    
    def sender_is_connected(self):
        """
        The sender calls this to tell us when it is ready for more data
        """
        log('(%d) R sender_is_connected()\n' % self.id, v=3)
        # sender gave is the OK, give it our buffered data and any future data we receive
        self.push_incoming_data_to_sender(self.buffer.getvalue())
        self.buffer = None
        self.collect_incoming_data = self.push_incoming_data_to_sender
    
    def sender_connection_error(self, e):
        log('(%d) R sender_connection_error(%s) for %s:%s\n' % (self.id, e, self.host, self.port), v=2)
        # if this was a redirected request and the redirection failed, create a new sender to send it to
        # the original host and port.
        if self.original_host_and_port:
            self.sender = AsyncHTTPProxySender(self, self.id, self.host, self.port)
            self.push_request_to_sender()
            return
        if isinstance(e, socket.error) and type(e.args) == type(()) and len(e.args) == 2:
            e = e.args[1]  # get the error string only
        self.error(404, 'Error connecting to <em>%s</em> on port <em>%d</em>: <b>%s</b>' % (self.host, self.port, e), response=str(e))

    def handle_close(self):
        log('(%d) receiver closing\n' % self.id, v=2)
        if hasattr(self, 'sender'):
            # self.sender.close() should be fine except for PUT requests?
            self.sender.close_when_done()
            del self.sender  # break circular reference
        self.close()
    
    def show_error(self, code, body, response=None):
        if not response:
            response = BaseHTTPServer.BaseHTTPRequestHandler.responses[code][0]
        self.push("HTTP/1.0 %s %s\r\n" % (code, response))
        self.push("Server: Mojo Nation Proxy\r\n")
        self.push("Content-type: text/html\r\n")
        self.push("\r\n")
        self.push('<html><head>\n<title>%d %s</title>\n</head>\n'
                '<body>\n%s\n</body>\n</html>' % (code, response, body))

    def error(self, code, body, response=None):
        self.show_error(code, body, response=response)
        if hasattr(self, 'sender'):
            self.sender.handle_close()
            del self.sender  # break circular reference
        self.close()
        # asyncore.poll() catches this  (XXX shouldn't need this?)
        #raise AsyncProxyError, (code, body)

    def log(self, message):
        log('(%d) receiver: %s\n', args=(self.id, message,), v=1)

    def log_info (self, message, type='info'):
        if __debug__ or type != 'info':
            log('%s: %s' % (type, message))


###############################################################################
class AsyncHTTPProxyServer(asyncore.dispatcher):
    def __init__(self, port):
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.set_reuse_addr()
        self.ouraddr = (ADDR_TO_BIND_TO, port)
        log('Starting proxy on %s port %d\n' % self.ouraddr, 1)
        self.bind(self.ouraddr)
        self.listen(5)

    def handle_accept(self):
        AsyncHTTPProxyReceiver(self, self.accept())
    
    def log(self, message):
        log('server: %s\n', args=(message,), v=1)


###############################################################################
def kill_external_proxy():
    """
    Kills all external instances of MojoProxy owned by this user.

    This is a wrapper for the platform specific version.
    """
    log("Stopping external proxies:\n")
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        try:
            s.connect(('localhost', 8000))
            s.send('quit\r\n')
        finally:
            s.close()
    except:
        log('Could not connect to locahost 8000, oh well...\n')
    
def kill_external_proxy_win32():
    import win32api, win32pdhutil, win32con
    # XXX This code was taken from the Python1.6 Windows distribution.

    # Change suggested by Dan Knierim, who found that this performed a
    # "refresh", allowing us to kill processes created since this was run
    # for the first time.
    try:
       	win32pdhutil.GetPerformanceAttributes(
            'Process','ID Process','mojoproxy')
    except:
        pass

    try:
        pids = win32pdhutil.FindPerformanceAttributesByName('mojoproxy')
    except:
        log("Not enough win32pdh library calls supported on this OS\n")
        log("older mojoproxy processes will not be automatically killed.\n")
        return

    # If _my_ pid in there, remove it!
    try:
        pids.remove(win32api.GetCurrentProcessId())
    except ValueError, e:
        log("Strange, this proxy wasn't in the process list.\n")
        pass

    if pids:
        for pid in pids:
            log("Attempting to terminate pid "+str(pid)+"...\n")
            handle = win32api.OpenProcess(win32con.PROCESS_TERMINATE, 0, pid)
            win32api.TerminateProcess(handle,0)
            win32api.CloseHandle(handle)
    else:
        log("No external proxies found.\n")
    
if __name__ == '__main__':
    # Kill the other mojo proxies:
    kill_external_proxy()
    if len(sys.argv) > 1 and sys.argv[1] == '--kill':
        log("Exiting.\n")
        raise SystemExit

    global Intropage_Filename
    if len(sys.argv) > 2 :  # 2rd param: the location of the intropage file containing the privacy code
        Intropage_Filename = sys.argv[2]
    else :
        # defaults; we really expect these to come on the command line
        if sys.platform == 'win32':
            Intropage_Filename = r"\Program Files\Mojo Nation\config\broker\intropage.html"
        else:
            Intropage_Filename = os.path.expandvars("${HOME}/.mojonation/broker/intropage.html")
    log('Using intropage %s to obtain your privacy code\n' % Intropage_Filename, 1)

    # get the port if specified
    if len(sys.argv) >= 2:
        PORT = int(sys.argv[1])

    if len(sys.argv) > 3 :  # 3th param: the next-step HTTP proxy can specified here (overrides the environment variable)
        os.environ['http_proxy'] = sys.argv[3]
    # display which proxy we're using
    http_proxy = os.environ.get('http_proxy')
    if not http_proxy :
        http_proxy = os.environ.get('HTTP_PROXY')
    
    if http_proxy :
        log("Next hop proxy: %s\n" % http_proxy, 1)
        scheme, netloc, path, params, query, fragment = urlparse.urlparse(http_proxy)
        # set host and port to the proxy
        if ':' in netloc:
            host, port = string.split(netloc, ':')
            port = string.atoi(port)
        else:
            host = netloc
            port = 80
        # do a basic checks to prevent a proxy loop
        if port == PORT and (((host == '127.0.0.1' or string.lower(host) == 'localhost') and ADDR_TO_BIND_TO == '') or string.lower(host) == string.lower(ADDR_TO_BIND_TO)):
            raise SystemExit, "Next hop proxy cannot be the MojoProxy itself"

    ps = AsyncHTTPProxyServer(PORT)
    log("Starting service...\n")
    asyncore.loop()

