#!/usr/bin/env python

# Publish a file and optionally a description to Mojo Nation.
# This spends some of your Mojo.
# For usage info, run it with no arguments.

# NOTE: This command has changes significantly as of release 0.90.

# Changes include:
# - A completely different command line interface (sorry to you script writers!).
# - No need for std in.  Everything is command-line oriented.
#   (This script is no longer interactive.)
# - Only one file can be published per call.  This is in order to mirror UI changes.

# Wishlist:
# I personally would like to see a way to publish multiple files.
# A good way to do this is to allow a section for common description (of all files
# being published), and then allow a mechanism for specifying specific descriptions
# for each file, something like:
# cmdpub.py 'Description=My own original music compilation' song1.mp3 'Title=Something pretentious and poetic' song2.mp3 'Title=Yet another song'

# I'd also like as many content description fields as possible to be automatically
# filled in.  IE: file size, format, etc...

import os, re, sys, string, urllib

# Mojo Nation specific:
import snarfprivacycode
import Describe
import contentlib
import confutils


### Constants:
DEBUG=None
USAGE_STRING = """
Usage:
%(command)s [-v] file [content-type] [(name=value)...]

Arguments:
-v           - Verbose output.
file         - The file to publish.
content-type - The complete content type string.  (Remember to quote/escape.)
               If none is supplied, the file will be automatically scraped for meta-data
               and it will be automatically filed into a hopefully-appropriate category
               (JPEGs into "Image", mp3s into "Audio", etc.)
               If "NONE" (deprecated) or "Private" (recommended) is supplied, then
               the file will be published without submitting a content description.  (The
               only way to retrieve it is with the Dinode URL.)
name=value   - Content description fields and values.

Example:
%(command)s my_story.txt 'Generic' 'Description=A story I wrote about Mojo Nation.'

""" % {'command' : os.path.basename(sys.argv[0])}


def publish_file(file, ctype = None, description = {}):
    """
    Publishes a single file with content description as per the 0.90 UI.

    @precondition file - a string, the path of a file to publish.; can be relative
    @precondition ctype - a string, the content type of this file, or:
        None, 'NONE' or 'Private' (case insensitive) meaning publish no content description.
    @precondition description - a dict, follows the convention of
        cgi.FieldStorage, that being string keys mapped to lists of strings.
        It is the description of the file.  If ctype is None or "NONE",
        description should be {}.
    """
    if (ctype == None) or (string.lower(ctype) == "private"):
        ctype = "NONE"

    (port, privacy_code) = snarfprivacycode.snarfprivacycode()
    # absolutize the path:
    file = os.path.abspath(file)
    
    base = "http://localhost:%s/%s/broker/publish" % (`port`, str(privacy_code))
    formdict = {
        'activity_start' : 'Publish',
        'ctype' : ctype,
        'file' : file,
        }

    formdict.update(description)

    print "formdict: %s" % `formdict`

    formstring = urllib.urlencode(formdict)

    fullurl = base + '?' + formstring
    debug("Submission URL:\n%s" % fullurl)

    # Publish!
    try:
        httpgetconnection = urllib.urlopen(fullurl)
    except IOError, e:
        if e.__dict__.has_key('strerror') and e.strerror[0] == 111:
            print "A connection to the server could not be made.  Make sure the Mojo Broker is running."
            raise SystemExit
        else:
            raise
    debug(httpgetconnection.read())

    # In the future we will need to make new getrequests to get incremental updates!
    print "Okay, now your publish has been added to your broker's activity list.  Sorry, there is currently no command-line accessible way to find out if/when the publish succeeds."

def debug(output):
    if DEBUG:
        print output
    
def run(argv):
    # Defaults:
    ctype, description = None, {}
        
    if len(argv) == 1:
        print USAGE_STRING
        raise SystemExit

    if argv[1] == '-v':
        global DEBUG
        DEBUG=1
        del argv[1]

    file = argv[1]

    if len(argv) > 2:
        ctype = argv[2]
        for nvpair in argv[3:]:
            name, value = string.split(nvpair, '=', 1)
            description[name] = value

    typedefrep=contentlib.ContentTypeDefRepository(
        confutils.confman.getpath(['PATH', 'CONTENT_TYPEDEF_DIR']))

    ctype, description = Describe.describe(file, ctype, description, typedefrep = typedefrep)
    publish_file(file, ctype, description)

#### this runs if you import this module by itself
if __name__ == '__main__':
    run(sys.argv)


