#!/usr/bin/env python
#
#  This script should convert annoying html-like format DTML tags into
#  the newer easier to read within html %()[] format tags.  It worked
#  for all of mojo nation's pretty complex dtml based UI, I can't
#  guarantee that it works everywhere though.
#
# Warning: don't use python 2.1b1 to run this, its re module is broken
# and will generate the wrong results.
#

import re
import sys
import os
import copy

start_convert_re = re.compile("<dtml(?P<tagtype>[A-Za-z-]*)(?P<tagbody>\s*[^>]*?)>", re.I)
end_convert_re = re.compile("</dtml(?P<tagtype>[A-Za-z-]*)>", re.I)

if len(sys.argv) != 2:
    print "usage:\n  %s filename" % sys.argv[0]
    sys.exit(1)

infilename = sys.argv[1]
assert os.path.isfile(infilename), "input file %s must be a valid file" % infilename
backup_infilename = infilename + '.saved'

print 'reading', infilename
inputdata = open(infilename, 'rb').read()
print 'backing up', infilename, 'to', backup_infilename
os.rename(infilename, backup_infilename)

dtml = copy.copy(inputdata)

# convert simple dtml-var tags
dtml, numchanges = re.subn(r"(?im)<dtml-var\s+(?P<tagbody>[a-z_0-9-]+?(\s[^>]+|))\s*>", r"%(\g<tagbody>)s", dtml)
print numchanges, "simple vars corrected"
# convert dtml-var expr= tags
dtml, numchanges = re.subn(r"(?im)<dtml-(?P<tagtype>var)\s+(expr=|)(?P<tagbody>((\".+?\")|(\'.+?\'))(\s[^>]+|)\s*)>", r"%(\g<tagtype> expr=\g<tagbody>)s", dtml)
print numchanges, "var tags with expressions corrected"
# convert dtml-call expr= tags
dtml, numchanges = re.subn(r"(?im)<dtml-(?P<tagtype>call)\s+(expr=|)(?P<tagbody>((\".+?\")|(\'.+?\'))\s*)>", r"%(\g<tagtype> expr=\g<tagbody>)!", dtml)
print numchanges, "call tags with expressions corrected"
# convert starting in if and elif tags that need expr=
dtml, numchanges = re.subn(r"(?im)<dtml-(?P<tagtype>in|if|elif|unless)(?P<space>\s*)(expr=|)(?P<tagbody>((\".+?\")|(\'.+?\'))\s*)>", r"%(\g<tagtype>\g<space>expr=\g<tagbody>)[", dtml)
print numchanges, "starting in/if/elif/unless tags with expressions corrected"

### this one is the catch-all that can cause screwed up conversions leading to 'invalid format character [' if any of the above regexes
### missed something.
# convert the remaining starting tags
dtml, numchanges = re.subn(r"(?im)<dtml-(?P<tagtype>[a-z]*)(?P<tagbody>\s*[^>]+?\s*)>", r"%(\g<tagtype>\g<tagbody>)[", dtml)
print numchanges, "starting in/if/elif/let dtml tags corrected"
# convert the ending tags
dtml, numchanges = re.subn(r"(?im)</dtml-(?P<tagtype>[a-z]*)\s*>", r"%(\g<tagtype>)]", dtml)
print numchanges, "ending html style dtml tags corrected"

print 'writing output to', infilename
outfile = open(infilename, 'wb')
outfile.write(dtml)

