#! /usr/bin/env python
#
# This updates CVS checkouts, then tar-bzips them, then publishes them to Mojo Nation!
# This script is unix- (probably GNU/Linux-) specific.
# This assumes "cmdpub.py" is on your path.
#
# I use it like this: mn-cvs-pub.py ~/MojoPublishing/prep/*
#
# ~/MojoPublishing/prep is full of CVS checkouts.  Each directory has a .mn-cvs-desc file
# which contains a python dict definition which describes this package.


import os
import sys
import time
import string


CVS_COMMAND = "cd %s; cvs up -PAd"
TARBALL_COMMAND = "cd %s; tar -cIf %s %s"
PUB_COMMAND = "cmdpub.py %s 'Software - Source Code' %s"

def main(argv=()):
    for cvsdir in argv[1:]:
        print "Working on directory:", cvsdir
        cvs_update(cvsdir)
        tarball = make_tarball(cvsdir)
        describe_and_publish(tarball, cvsdir)

def cvs_update(cvsdir):
    run_command(CVS_COMMAND % cvsdir)

def make_tarball(dir):
    tarballpath = "%s/%s_%s.tar.bz2" % (get_tmp_dir(), base_name(dir)[1], get_date_stamp())
    run_command(TARBALL_COMMAND % (base_name(dir)[0], tarballpath, base_name(dir)[1]))
    return tarballpath

def run_command(cmd, acceptable_statuses=()):
    print "Executing:", cmd
    status = os.system(cmd)
    if status and (status not in acceptable_statuses):
        raise SystemExit, "Command: %s\nExit Status: %d\n" % (cmd, status)

def describe_and_publish(tarball, cvsdir):
    run_command(PUB_COMMAND % (tarball, make_description(cvsdir)))

def make_description(cvsdir):
    # Default description:
    descdict = {'Archive Format' : 'tar archive with bzip2 compression'}

    # If the cvs directory has a description file, eval it.  If should be a dict which updates descdict:
    descfile = os.path.join(cvsdir, '.mn-cvs-desc')
    if os.path.isfile(descfile):
        descdict.update(eval(open(descfile).read()))

    # Save the generated description, for easy customization:
    f = open(descfile, 'w')
    f.write(repr(descdict))
    f.close()

    descdict['Version'] = descdict['Version'] + (' - Date: %s' % get_date_stamp())
    desc = []
    for key, value in descdict.items():
        desc.append("'%s=%s'" % (key, value))
    return string.join(desc)

def get_tmp_dir():
    tmpdir = os.environ.get('TMPDIR')
    if tmpdir is not None:
        return tmpdir
    tmpdir = os.environ.get('TMP_DIR')
    if tmpdir is not None:
        return tmpdir
    return '/var/tmp' # Default 

def get_date_stamp():
    return time.strftime("%Y-%m-%d", time.localtime(time.time()))

def base_name(path):
    if path[-1] == '/':
        path = path[:-1]
    return os.path.split(path)

if __name__ == '__main__':
    main(sys.argv)
