#!/usr/bin/env python
#  Copyright (c) 2000 Autonomous Zone Industries
#  This file is licensed under the
#    GNU Lesser General Public License v2.1.
#    See the file COPYING or visit http://www.gnu.org/ for details.

# Query content trackers and download content randomly

# This script doesn't scale elegantly.
# This script is for testing.

# You probably don't really want to waste your Mojo downloading content at random
# but if you're 18 and own the computer then you can do pretty much whatever makes you happy.

import os
import re
import string
import sys
import urllib
import debug
import thread
import time
import whrandom

from confutils import confman
from human_readable import hr
from snarfprivacycode import snarfprivacycode
from contentlib import ContentTypeDefRepository

SEARCH_INTERVAL = 6 * 60 * 60 # six hours
lastSearch = 0

ctypes = ContentTypeDefRepository(dirname=os.path.expandvars(confman.dict["PATH"]["CONTENT_TYPEDEF_DIR"])).known_type_list()

port, privacyCode = snarfprivacycode()

dinodePat = re.compile(r"Dinode/[-_A-Za-z0-9]+/")

# takes a dinode "/Dinode/xxxxxxxxx"
def fetchDinode(dinode):
    debug.stderr.write('Fetching Dinode: %s\n' % hr(dinode))
    urllib.urlretrieve("http://localhost:%s/%s/%s" % (port, privacyCode, dinode), '/dev/null')
    debug.stderr.write('Done fetching Dinode: %s\n' % hr(dinode))

# returns a new list with duplicates removed
def uniqueList(list):
    # is there a better way to removed duplicates from a list?
    dict = {}
    for i in list:
        dict[i] = 0
    return dict.keys()

# returns a new list with the items shuffled
# this isn't all that efficient (especially on space)
def shuffleList(list):
    l = list[:] # make a copy so nothing unexpected happens to the caller
    shuffled = []
    length = len(l)
    for i in range(length):
        x = whrandom.randint(0, (length - 1) - i)
        shuffled.append(l[x])
        del l[x]
    return shuffled

## does as earch on ctype and returns a list of Dinodes for fetching
def searchCtype(ctype):
    debug.stderr.write("Searching content type: %s\n" % ctype)
    url = 'http://localhost:%s/%s/broker/search_query?%s' % (port, privacyCode, urllib.urlencode({'ctype' : ctype}))
    html = urllib.urlopen(url).read()
    dinodes = dinodePat.findall(html)
    debug.stderr.write("Got %s dinodes.\n" % len(dinodes))
    return dinodes

def run():
    while 1:
        # search all the CTypes and create a big list of dinodes
        dinodes = []
        for type in ctypes:
            dinodes = dinodes + searchCtype(type)

        # remember when we last searched
        lastSearch = time.time()
        # remove duplicates
        dinodes = uniqueList(dinodes)
        # shuffle the list
        dinodes = shuffleList(dinodes)

        debug.stderr.write("Preparing to fetch from list of %s unique dinodes.\n" % len(dinodes))
        while (time.time() - lastSearch < SEARCH_INTERVAL) and len(dinodes) > 0:
            try:
                dinode = dinodes.pop()
                fetchDinode(dinode)
            except IndexError:
                pass
            
if __name__ == "__main__":
    run()
