#!/usr/bin/env python
#
# Makes an HTTP 1.0 GET request and dumps everything, including HTTP headers
# to stdout.
#
# This version uses a main function and the "if __name__ == '__main__'" trick.
# You can run this like the other version from the commandline, but you can
# also use it inside other python programs, like this:
# >>> import rawhttpget2
# >>> rawhttpget2.main('http://www.google.com', sys.stderr)

import sys
import socket

def main(argv, outfile=sys.stdout):
    url = sys.argv[1]

    # Cut off the protocol specifier in the url:
    if url[:7] == 'http://':
        url = url[7:]

    if url.find('/') != -1:
        host, request = url.split('/', 1)
    else:
        host = url
        request = ''

    if host.find(':') != -1:
        host, port = host.split(':')
        port = int(port) # change the string to an integer.
    else:
        port = 80 # The standard http port.

    address = (host, port) # The normal format of a TCP/IP address in python.

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(address)
    s.send('GET /%s HTTP/1.0\r\n\r\n' % request)

    response = s.recv(1024)
    while response:
        outfile.write(response)
        response = s.recv(1024)

    s.close()

if __name__ == '__main__':
    main(sys.argv)
