/*
 * $Id: Advertisement.java,v 1.11 2001/06/08 17:19:00 bondolo Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

package net.jxta.document;

import java.lang.IllegalStateException;

/**
 * This abstract class defines an <b>advertisement</b>. Advertisements are core
 * JXTA objects that are used to advertise a Peer, a PeerGroup, a Service, or a
 * PipeService. Advertisements provide a platform independent representation of
 * a core platform objects that can be exchanged between different platform
 * implementations (Java, C).
 *
 * <p>Each advertisement holds a document that represents the
 * advertisement. Advertisements are typically represented as a text
 * document (XML). The getDocument method of the class enables to get a
 * representation of the advertisement into different format via the
 * requested MimeMediaType. Typical mime types are "text/xml" or
 * "txt/plain" that generate different text representations for the document.
 *
 * <p>Advertisements are created via an advertisement factory to hide their
 *   implementation.
 *
 * @see net.jxta.document.AdvertisementFactory
 * @see net.jxta.document.Document
 * @see net.jxta.document.MimeMediaType
 * @see net.jxta.peergroup.PeerGroup
 * @see net.jxta.protocol.PeerAdvertisement
 * @see net.jxta.protocol.PeerGroupAdvertisement
 * @see net.jxta.protocol.PipeAdvertisement
 *
 * @version $Revision: 1.11 $
 * @since JXTA 1.0
 **/
public abstract class Advertisement {
    
    /**
     * Returns the identifying type of this Advertisement.
     *
     * @return String the type of advertisement
     *
     * @since JXTA 1.0
     */
    public static String getAdvertisementType() {
        throw new RuntimeException(
        "Advertisement : sub-class failed to override getAdvertisementType" );
    }
    
    /**
     * Write advertisement into a document. asMimeType is a mime media-type
     * specification and provides the form of the document which is being
     * requested. Two standard document forms are defined. "text/plain" encodes
     * the document in "pretty-print" format for human viewing and "text/xml"
     * which provides an XML format.
     *
     *  @param asMimeType  MimeMediaType format representation requested
     *  @return Document   the document to be used in the construction
     *  @throws IllegalStateException the Advertisement cannot be converted to
     *  a document because it is incomplete
     *  @exception Exception error generating the document
     *
     *  @since JXTA 1.0
     */
    public abstract Document getDocument( MimeMediaType asMimeType ) 
        throws IllegalStateException, Exception;
}
