/*
 * $Id: StructuredTextDocument.java,v 1.6 2001/06/08 17:16:28 bondolo Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

package net.jxta.document;

/**
 * StructuredTextDocument is a convenience sub-interface of StructuredDocument
 * for the manipulating text based documents such as XML. Whereas
 * StructuredDocument manipulates elements as abstract Objects,
 * StructuredTextDocument manipulates the keys and values of Elements as Strings.
 *
 * @see         net.jxta.document.Document
 * @see         net.jxta.document.TextDocument
 * @see         net.jxta.document.TextElement
 * @see         net.jxta.document.StructuredTextDocument
 * @see         net.jxta.document.StructuredDocumentFactory
 * @see         net.jxta.document.MimeMediaType
 *
 * @version     $Revision: 1.6 $
 * @since JXTA 1.0
 */

public interface StructuredTextDocument extends StructuredDocument,
TextDocument, TextElement {
    
    /**
     * Create a new element without value
     *
     * @param   name    The name of the element to be created.
     * @return  TextElement The new element.
     *
     * @since JXTA 1.0
     */
    
    public abstract TextElement createElement(String name);
    
    /**
     * Create a new element with value
     *
     * @param   name    The name of the element to be created.
     * @param   value   The value of the element to be created.
     * @return  TextElement The new element.
     *
     * @since JXTA 1.0
     */
    
    public abstract TextElement createElement(String name, String value);
}
