/************************************************************************
 *
 * $Id: Message.java,v 1.3 2001/05/23 00:28:20 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS  OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.endpoint;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

/** 
 * Message defines the interface of messages sent or received to or
 * from a endpoint and pipes.  Applications are expected to implement this
 * class in order to exchange messages.
 *
 * @see  net.jxta.endpoint.EndpointAddress
 * @see  net.jxta.pipe.InputPipe
 * @see  net.jxta.pipe.OutputPipe
 * @see  net.jxta.pipe.Pipe
 *
 * @version     $Revision: 1.3 $
 * @since JXTA 1.0 
 */

public interface Message {

  /**
   * Push a chunk of data into the message.
   *
   * @param name is the name of the chunk of data
   * @param inputStream is an inputStream to the data to be pushed.
   *
   * @version     $Revision: 1.3 $
   * @since JXTA 1.0
   */

  public void push (String name, InputStream inputStream)
		throws IOException;
  
  /**
   * Gets a chunk of data associated with the name
   *
   * @param name contains the name of the chunk of data to get
   * @return InputStream an inputStream to the chunk of data
   *
   * @version     $Revision: 1.3 $
   * @since JXTA 1.0
   */

  public InputStream pop (String name) throws IOException;

  /**
   * Get the size of a particular tag's data in the message.
   *
   * @param atag the name of the tag whose size to get
   * @return the size of the data in the message for the specified tag
   *
   * @version     $Revision: 1.3 $
   * @since JXTA 1.0
   */

  public int getSize(String atag);

  /**
   * Returns an Enumeation of all the available chunks of data in the message
   *
   * @return an Enumeation of all the available chunks of data in the message
   *
   * @version     $Revision: 1.3 $
   * @since JXTA 1.0
   */

  public Enumeration getNames();
  
  /**
   * Duplicates the message. This duplicate must be a real copy
   *
   * @return Message a Message that is a copy of the original message
   *
   * @version     $Revision: 1.3 $
   * @since JXTA 1.0
   */

  public Message dup() throws IOException;
  

}



