/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: PeerInfo.java,v 1.3 2001/05/23 00:28:21 lomax Exp $
 */

package net.jxta.peer;

import java.io.IOException;
import java.util.Enumeration;
import net.jxta.service.Service;
import net.jxta.protocol.ServiceAdvertisement;
import net.jxta.protocol.PeerInfoAdvertisement;

/**
 * This interface implements the peer information and management 
 * service that is used for monitoring and registering peer services.
 *
 * @see         net.jxta.document.Advertisement
 * @see         net.jxta.service.Service
 * @see         net.jxta.protocol.PeerInfoAdvertisement
 *
 * @version     $Revision: 1.3 $
 * @since       JXTA  1.0
 */

public interface PeerInfo extends Service {
    
    /**
     * Get PeerInfo from a remote peer.
     *
     * @param peer Address of a peer, or portal to connect to, if
     * address is null 
     */
    public void getRemotePeerInfo(String peer);

    /**
     * Get PeerInfo from local cache.
     *
     * @param peer Address of a peer, or portal to connect to, if
     * address is null 
     */
    public Enumeration getLocalPeerInfo(String peer) 
        throws IOException;

   /**
    * Retrieve Peer Information Advertisement
    *
    * @return PeerInfoAdvertisement
    *
    * @version     $Revision: 1.3 $
    * @since       JXTA  1.0
    */
    
    public PeerInfoAdvertisement getPeerInfo();
    
    /**
     * Flush cached advertisements.
     *
     * @param id peerId of peer whose locally cached
     * PeerInfoAdvertisement is to be deleted. If null, locally cached
     * PeerInfoAdvertisements of <em>all</em> peers are deleted.
     *
     * @see         net.jxta.protocol.PeerInfoAdvertisement
     * @version     $Revision: 1.3 $
     * @since       JXTA  1.0
     */
    public void flushAdvertisements(String id) 
        throws IOException;

   /**
    * Register a Service in the Peer Advertisement
    * service adversement describes a specific service that is running
    * on the peer (i.e. LDAP, etc)
    *
    * @param advertisement service advertisement to be registered
    * @exception IOException if registeration fails
    *
    * @version     $Revision: 1.3 $
    * @since       JXTA  1.0
    */
    
    public void registerService(ServiceAdvertisement advertisement)
      throws IOException;
    
   /**
    * unRegisters a prevously registered service
    *
    * @param advertisement service advertisement to be unregistered
    * @exception IOException if the un-registeration fails
    *
    * @version     $Revision: 1.3 $
    * @since       JXTA  1.0
    */
    
    public void unRegisterService( ServiceAdvertisement advertisement)
      throws IOException;
    
}
