/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Application.java,v 1.6 2001/06/19 03:16:00 jice Exp $
 */

package net.jxta.platform;

import net.jxta.peergroup.PeerGroup;
import net.jxta.document.Advertisement;
import net.jxta.exception.PeerGroupException;

/**
 * This interface defines a JXTA peer group application. All JXTA
 * applications need to implement this interface. Jxta Applications are
 * given their initialization parameters via the init() method rather
 * than a non-default constructor.
 *
 * <p>Applications are passed the peer group they are created within. From
 * the peergroup object, applications can acccess all the peer group
 * services.  Each application will be passed the peergroup they are
 * started in. PeerGroups themselves are subclasses of JXTA Applications, as
 * well as Services.
 * The initial root peer group is known as the World Peer Group and is
 * implemented by an object of class Platform, a subclass of PeerGroup.
 * The "parent" PeerGroup of the World Peer Group is null.
 *
 * <p>Each peer group defines a default application that is started as part
 * the peer group initialization process, if the PeerGroup's startApp()
 * method is invoked. The Default application is specified by the PeerGroup's
 * advertisement and can be the JXTA Shell or any other JXTA Application, as
 * specified by the group's advertisment.
 *
 * @see net.jxta.peergroup.PeerGroup
 * @see net.jxta.service.Service
 * @see net.jxta.protocol.PeerGroupAdvertisement
 *
 * @version $Revision: 1.6 $
 * @since JXTA 1.0
 */

public interface Application {
    
    /**
     *  Initialize the application passing it its peer group and advertisement.
     *
     *  @param group PeerGroup this application is started from
     *  @param adv The advertisement for this application
     *
     *  @exception PeerGroupException failure to initialize this application.
     *
     * @since   JXTA 1.0
     */
    public void init(PeerGroup group, Advertisement adv)
    throws PeerGroupException;
    
    /**
     *  Some applications will wait for start() being called, before proceeding
     *  beyond a certain point. That's also the opportunity to supply
     *  arguments.
     *
     *  @params args An array of Strings forming the parameters for this
     *  application.
     *
     *  @returns int status indication.
     */
    public int startApp(String[] args);
    
    /**
     * One can ask an application to stop.
     * The application cannot be forced to comply, but in the future
     * we might be able to deny it access to anything after some timeout.
     */
    public void stopApp();
}
