/*
 * $Id: PipeAdvertisement.java,v 1.11 2001/05/23 00:28:22 lomax Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

package net.jxta.protocol;

import net.jxta.document.Advertisement;
import net.jxta.pipe.PipeID;

/**
 * This class defines the PipeService Advertisement
 *
 * <p>The default behavior of this abstract class is to simple be a place
 * holder for the PipeService Id and its name. No XML document is processed. That
 * has to be done in the class that implements PipeAdvertisement.  Pipes are
 * named with their XML advertisement. The generation of the PipeService
 * Advertisements is let to the application. A Jxta PipeService advertisement is
 * an XML document (non validated) which contains at least the following
 * structure (but can be extended).
 *
 * <pre>
 * &lt;?xml version="1.0"?&gt;
 * &lt;JxtaPipe&gt;
 *   &lt;id&gt; UUID &lt;/id&gt;
 *   &lt;name&gt; optional symbolic name than can be used by
 *      any search engine &lt;/name&gt;
 * &lt;/JxtaPipe&gt;
 *</pre>
 *
 * <p>Note that other tags can be provided by the application for its own
 * profit. Those extra tags are not interpreted by the PipeService Service.
 *
 * <p>The UUID needs to be a unique String (unique in time and space).
 * The optional tool package net.jxtax.pipe provides helper classes to
 * build PipeService Advertisement (net.jxtax.pipe.PipeAdv);
 *
 * @version $Revision: 1.11 $
 * @since JXTA 1.0
 */

public abstract class PipeAdvertisement extends Advertisement {
    
  String name = null;
  private PipeID pipeId = null;
    
  /**
   * get the pipe type
   *
   *  @return String  PipeAdvertisement type
   *
   *  @version $Revision: 1.11 $
   *  @since JXTA 1.0
   */
  
  public static String getAdvertisementType() {
    return "jxta:PipeAdvertisement" ;
  }
    
  /**
   * get the pipe id
   *
   *  @return String  PipeService id
   *
   *  @version $Revision: 1.11 $
   *  @since JXTA 1.0
   */
  
  public PipeID getPipeID () {
    return pipeId;
  }
  
  /**
   * set the pipe Id
   *
   *  @return String  PipeId
   *
   *  @version $Revision: 1.11 $
   *  @since JXTA 1.0
   */

  public void setPipeID (PipeID id) {
    pipeId = id;
  }
  
  /**
   * get the symbolic name associated with the pipe
   *
   * @return String the name field.
   *
   * @version $Revision: 1.11 $
   * @since JXTA 1.0
   */

  public String getName() {
    return name;
  }
  
  /**
   * set the symbolic name associated with the pipe
   *
   *  @param n the name this pipe adv should have.
   *
   *  @version $Revision: 1.11 $
   *  @since JXTA 1.0
   */
  
  public void setName (String n) {
    name = n;
  }
  
}



