/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ResolverQueryMsg.java,v 1.12 2001/05/23 00:28:22 lomax Exp $
 */
package net.jxta.protocol;

import net.jxta.document.Document;
import net.jxta.document.MimeMediaType;

/**
 * This class defines the Generic ResolverService message "Query"
 * The default behavior of this abstract class is simply  a
 * place holder for the generic resolver query fields. note that no XML
 * processing is performed by this class. The processing of the message is left
 * to the class extending this class.
 *
 * <p>This message is part of the PeerGroup ResolverService Protocol
 *
 * <pre>
 * &lt;?xml version="1.0"?&gt;
 * &lt;ResolverQuery&gt;
 *   &lt;handlername&gt; name &lt;/handlername&gt;
 *   &lt;credentialServiecUri&gt; uri &lt;/credentialServiecUri&gt;
 *   &lt;credentialToken&gt; token &lt;/credentialToken&gt;
 *   &lt;queryid&gt; id &lt;/queryid&gt;
 *   &lt;query&gt; query &lt;/query&gt;
 * &lt;/ResolverQuery&gt;
 * </pre>
 *
 * Note: queryid is unique to the originating node only, it can be
 * utilized to match queries to responses
 *
 * Tag  $Name:  $
 * @see net.jxta.protocol.ResolverResponseMsg
 *
 * @version $Revision: 1.12 $
 * @since JXTA 1.0
 */

public abstract class ResolverQueryMsg {

    private String handlername=null;
    private String credentialServiecUri=null;
    private String credentialToken=null;
    private String srcPeerId = null;

    //XXX we have to expose this field due to access violation of private field
    //need a better way in abstracting the id, and retrival

    public  int queryid=0;
    private String     query=null;



    /**
     * All messages have a type (in xml this is !doctype)
     * which identifies the message
     * @return String "jxta:ResolverQuery"
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public static String getAdvertisementType() {
        return "jxta:ResolverQuery" ;
    }

    /**
     * returns the handlername
     * @return String handlername name
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public String getHandlerName() {
	return handlername;
    }

    /**
     * returns the credentialServiecUri name
     * @return String credentialServiecUri name
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public String getCredentialServieUri() {
	return credentialServiecUri;
    }

    /**
     * returns  the credentialToken
     * @return String value of the credentialToken
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public String getCredentialToken() {
	return credentialToken;
    }

    /**
     * returns queryid  value
     * @return int queryid value
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public int getQueryId() {
	return queryid;
    }

    /**
     * returns the query
     * @return String value of the query
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public String getQuery() {
	return query;
    }

    /**
     * set the handlername
     * @param name handler name
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public void setHandlerName(String name) {
	this.handlername=name;
    }

    /**
     * set the credentialServiecUri
     * @param uri string representing credentialServiecUri
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public void setCredentialServiceUri(String uri) {
	this.credentialServiecUri=uri;
    }

    /**
     * set the value of the credential Token
     * @param token string representing the credential Token
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public void setCredentialToken(String token) {
	this.credentialToken=token;
    }

    /**
     * set the query id. Each query has a unique id.
     * @param id int id
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public void setQueryId(int id) {
	queryid=id;
    }

    /**
     * set the Query string
     * @param Query string representing the query
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public void setQuery(String Query) {
	this.query=Query;
    }

    /**
     * Set the source of the query
     * @param src is a String containing the peerid of the source
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public void setSrc (String p) {
	srcPeerId = p;
    }

    /**
     * Returns the source of the query
     * @return String the peerid of the source of the query
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public String getSrc () {
	return srcPeerId;
    }

    /**
     * Write advertisement into a document. asMimeType is a mime media-type
     * specification and provides the form of the document which is being
     * requested. Two standard document forms are defined. 'text/text' encodes
     * the document in a form nice for printing out and 'text/xml' which
     * provides an XML format.
     *
     * @param asMimeType mime-type requested representation for the returned document
     * @return Document document representing the advertisement
     * @exception Eception error creating the document representing the message.
     *
     * @version $Revision: 1.12 $
     * @since JXTA 1.0
     */

    public abstract Document getDocument ( MimeMediaType asMimeType ) 
      throws Exception;
  
}



