/************************************************************************
 *
 * $Id: RendezVousMonitor.java,v 1.9 2001/05/23 00:28:22 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.rendezvous;

import net.jxta.peer.PeerID;
import net.jxta.document.Advertisement;

/** 
 * This interface must be implemented by the program that controls
 * RendezVous on a peer that uses them.
 *
 * <p>Any peer in a peer group can become a rendezvous for a peer group. A
 * rendezvous provides a long-range discovery mechanism where members of a
 * peer group can find themselves. Rendezvous are advertised as part of the
 * peer group advertisement of a peer group.
 *
 * @see net.jxta.rendezvous.RendezVousManager
 * @see net.jxta.rendezvous.RendezVous
 * @see net.jxta.discovery.Discovery
 * @see net.jxta.protocol.PeerGroupAdvertisement
 *
 * @version $Revision: 1.9 $
 * @since JXTA 1.0
 */

public interface RendezVousMonitor {

    /**
     * This method is called each time a RendezVous peer is not
     * reachable anymore.
     *
     * @param peer is the PeerId of the RendezVous peer.
     *
     * @version $Revision: 1.9 $
     * @since JXTA 1.0
     */

    public void lostConnection (PeerID peer);

    /**
     * This method is called by the RendezVous service in order
     * to provide advertisement about other RendezVous peers.
     * This is usefull when a RendezVous peer wants to balance its load
     * to other RendezVous
     *
     * @param adv add a new advertisement of a rendezvous
     *
     * @version $Revision: 1.9 $
     * @since JXTA 1.0
     */

    public void newRendezVous (Advertisement adv);

    /**
     * This method is called by the RendezVous service to notify the
     * monitor that a new RendezVous has accepted the connection.
     *
     * @param peer is the PeerId of the new connect RendezVous peer.
     * @param lease is the time in millisecond that the RendezVous is
     * ready to serve.
     *
     * @version $Revision: 1.9 $
     * @since JXTA 1.0
     */

    public void isConnected (PeerID peer, long lease);
}



