/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Service.java,v 1.6 2001/06/18 21:17:56 jice Exp $
 */

package net.jxta.service;

import net.jxta.platform.Application;
import net.jxta.document.Advertisement;

/**
 * This interface implements the peer group service interface. Peer groups
 * are formed as a collection of peers that have agreed upon a common set of
 * services. The JXTA platform provides a predefined set of services to build
 * peer groups
 *
 * <ul type-disc> 
 *
 * <li><p><b>DiscoveryService</b> service: The service to discover new peers and peer groups
 *
 * <li><p><b>ResolverService</b> service: The service to resolve query requests
 * (PipeService location and search for advertisement) within a peer group. The
 * services of a peer groups are defined in the <code>peer group
 * advertisement </code> defines each of the group services (membership,
 * discovery, resolver, etc).
 *
 * <li><p><b>PipeService</b> service: The service to create input and output pipes,
 * and send and receive messages.
 *
 * <li><p><b>MembershipService</b> service: The service to apply and join a peer group
 *
 *</ul>
 *
 * <p>The Platform PeerGroup is built using the core services. Every other
 * peer group created can used the core services, extend them or add new
 * ones to form new kinds of peer groups. User can create new peer groups
 * with they own set of customized services.
 *
 * <p>Every service is defined by a <b>ServiceAdvertisement</b>. The Service
 * advertisement provides the necessary information to uniquely identify
 * the service and to contact its associated service(s).
 *
 * <p>When a new peer is joining a peer group an instance of all the
 * services associated with the new peer group is created on this peer
 * dynamically. Not all services need to be resident on a peer. A peer can
 * dynamically load a new service from the peer it contacted to join the 
 * new peer group (via the PeerGroupAdvertisement of this group).
 *
 * <p> Each service is a Application, so the Service interface extends the Application
 * interface.
 *
 * @see         net.jxta.peergroup.PeerGroup
 * @see         net.jxta.platform.Application
 * @see         net.jxta.membership.Membership
 * @see         net.jxta.discovery.Discovery
 * @see         net.jxta.resolver.Resolver
 * @see         net.jxta.pipe.Pipe
 * @see         net.jxta.protocol.PeerGroupAdvertisement
 * @see         net.jxta.protocol.ServiceAdvertisement
 *
 * @version $Revision: 1.6 $
 * @since JXTA 1.0 
 */
public interface Service extends Application {

    /**
     * Service objects are not manipulated directly to protect usage
     * of the service. A Service interface is returned to access the service
     * methods.
     *
     * @return Service public interface of the service
     *
     * @since JXTA 1.0 
     */
    public Service getInterface();

    /**
     * Returns the advertisment for this service.
     *
     * @return Advertisement the advertisement.
     *
     * @since JXTA 1.0 
     */
    public Advertisement getAdvertisement();
}
