/************************************************************************
 *
 * $Id: Cm.java,v 1.32 2001/06/18 20:18:15 hamada Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.cm;

import java.util.*;
import java.io.*;
import net.jxta.document.*;

import net.jxta.peergroup.*;

/**
 * This class implements a limited document storing mechanism
 * intended to provide a mean for DiscoveryService, ResolverService, PipeResolver
 * and Cache services to search and exchange documents over the group.
 *
 * Only Core Services are intended to use this mechanism, the security
 * relies on the generic PeerGroup security service.
 *
 * XXX: this need to be optimized
 */


public final class Cm {

    public static boolean debug = false;

    public static final String RootDirBase    = "cm";
    public static final String PublicDirName  = "public";
    public static final String PrivateDirName = "private";
    public static final String TmpDirName     = "tmp";

    private static final Random random = new Random();


    private String RootDirName = null;


    /**
     * Convenience method; creates a dir if does not exist.
     */
    private static void initDir (String dirName)
	throws IOException {

	File dir = null;
	try {
	    dir = new File (dirName);
	} catch (RuntimeException e) {
	    // That should not happen
        if (debug) System.err.println("Cm failed to open the directory " + dirName);
	    throw new IOException();
	}
	try {
	    if (!dir.exists()) {
		// We need to create the directory
		if (!dir.mkdirs()) {
		    // Cannot create the directory
            if (debug) System.err.println("Cm cannot create directory "
				       + dirName);
		    throw new IOException();
		}
	    }
	} catch (Exception e) {
       if (debug) System.err.println("Cm cannot access directory " + dirName);
	    throw new IOException();
	}
	return;
    }

    /**
     * Makes a temporary storage area with a unique name.
     */
    public Cm () {
	this(createTmpName());
    }



    public Cm (String areaName) {
    setDebug();
	RootDirName = RootDirBase + File.separatorChar + areaName;

	// Initialize directories
	try {
	    initDir (RootDirName + File.separatorChar + PrivateDirName);
	    initDir (RootDirName + File.separatorChar + PublicDirName);
	    initDir (RootDirName + File.separatorChar + TmpDirName);
	} catch (Exception e) {
	    // Not much we can do...
	}
    }


    /**
     * Creates a new folder. Note that this function is idempotent.
     *
     * @param f contains the name of the new folder
     * @throws IOException when the folder creation failed.
     */
    public void createFolder (String f) throws IOException {

	initDir (RootDirName + File.separatorChar + f);
    }

    /**
     * Generates a random file name
     *
     * @return String a random file name
     */
    public static String createTmpName() {
	return "cm" + Integer.toString(random.nextInt());
    }


    private void setDebug() {
        FileInputStream inconf = null;
        Properties prop = null;

        try {
            inconf = new FileInputStream("jxtaConfig");
            prop = new Properties();
            prop.load(inconf);
        } catch (Exception  ex1) {
            prop = null;
        }
        try {
            if (inconf != null) inconf.close();
        } catch (Exception e) {
        }

        if (prop != null &&
            prop.getProperty("Debug", "false").equals("true")) {
            debug = true;
            debug("Turning debugging ON.");
        }
    }

    /**
     * Generates a random file name using doc hashcode
     *
     * @param  doc to hash to generate a unique name
     * @return String a random file name
     */
    public static String createTmpName(StructuredDocument doc) {
      int hash = 0;

      try {
	StringWriter out = new StringWriter();
	((StructuredTextDocument) doc).sendToWriter( out );
	hash = out.toString().hashCode();
	out.close();
      } catch (Exception ex) {
      }

      return "cm" + Integer.toString(hash);
    }


    /**
     * Remove a file
     *
     * @param dn contains the name of the folder
     * @param fn contains the name of the file
     * @throws IOException when removing failed
     */
    public void remove (String dn, String fn)
	throws IOException {

	String filename = RootDirName + File.separatorChar +
	  dn + File.separatorChar + fn;

	File file = null;
	try {
	    file = new File (filename);
	    boolean res = file.delete();
	} catch (Exception e) {
	    debug("Cm cannot remove the file " + filename);
	    throw new IOException();
	}
    }

    /**
     * Tests if a file does exist
     *
     * @param dn contains the name of the folder
     * @param fn contains the name of the file
     * @throws IOException when removing failed
     */
    public boolean exists (String dn, String fn)
	throws IOException {

	String filename = RootDirName + File.separatorChar +
	  dn + File.separatorChar + fn;

	File file = null;
	try {
	    file = new File (filename);
	    return file.exists();
	} catch (Exception e) {
	    debug("Cm cannot test the file " + filename);
	    throw new IOException();
	}
    }

    /**
     * Restore a saved file
     *
     * @param dn contains the name of the folder
     * @param fn contains the name of the file
     * @return StructuredDocument containing the file
     * @throws IOException when reading was not possible
     */

    public StructuredDocument restore (String dn, String fn)
	throws IOException {

	File file = null;
	String filename = RootDirName + File.separatorChar
	  + dn + File.separatorChar + fn;

	debug ("restore document from " + filename);

	InputStream ip = null;
	StructuredDocument doc = null;

	try {
        file = new File (filename);
	    ip = new FileInputStream (file);
	    doc = StructuredDocumentFactory.newStructuredDocument
		(new MimeMediaType ("text/xml"), ip);
	    ip.close();
	    return doc;
	} catch (Exception e) {
	    throw new IOException();
	}
    }


    public byte[] restoreBytes (String dn, String fn)
	throws IOException {

	File file = null;
	String filename = RootDirName + File.separatorChar + dn
	  + File.separatorChar + fn;
	InputStream ip = null;
	byte[] buffer = null;

	try {
        file = new File (filename);
	    ip = new FileInputStream (file);
	    int size = ip.available();
	    buffer = new byte[size];
	    int got = 0;
	    while (got < size) {
		got += ip.read (buffer, got, size - got);
	    }
	    ip.close();
	    return buffer;
	} catch (Exception e) {
	    throw new IOException();
	}
    }


    /**
     * Saves a StructuredDocument into a folder
     *
     * @param dn contains the name of the folder
     * @param fn contains the name of the file
     * @param doc contains the StructuredDocument to save
     * @throws IOException when saving failed.
     **/
    public void save (String dn, String fn, StructuredDocument doc)
	throws IOException {

	String filename = RootDirName + File.separatorChar + dn +
	  File.separatorChar + fn;

	debug ("save document in " + filename);

	File file = null;
	try {
	    file = new File (filename);
	} catch (RuntimeException e) {
	    debug("Cm cannot write doc");
	    if (debug) e.printStackTrace();
	    return;
	}

	FileOutputStream op = null;
	try {
	    op = new FileOutputStream (file);
	} catch (Exception e) {
	    debug("Cm cannot write doc");
	    if (debug) e.printStackTrace();
	    return;
	}

	try {
	    doc.sendToStream( op );
	    op.close();
	} catch (Exception e) {
	    debug("Cm cannot write doc");
	    if (debug) e.printStackTrace();
	}
    }


    public void saveBytes (String dn, String fn, byte[] buffer)
	throws IOException {

	String filename = RootDirName + File.separatorChar + dn +
	  File.separatorChar + fn;

	File file = null;
	try {
	    file = new File (filename);
	} catch (RuntimeException e) {
	    debug("Cm cannot write doc");
	    if (debug) e.printStackTrace();
	    return;
	}

	FileOutputStream op = null;
	try {
	    op = new FileOutputStream (file);
	} catch (Exception e) {
	    debug("Cm cannot write doc");
	    if (debug) e.printStackTrace();
	    return;
	}

	try {
	    op.write (buffer);
	    op.close();
	} catch (Exception e) {
        e.printStackTrace();
	    debug("Cm cannot write doc");
	}
    }


    /**
     * Gets the list of all the files into the given folder
     *
     * @param dn contains the name of the folder
     * @return String[] an array of Strings containing the name of the files
     */
    public String[] getFileNames(String dn) {

	if (dn == null) {
	    return null;
	}
	try {
	    File dir = new File (RootDirName
				 + File.separatorChar
				 + dn);

	    String[] list = dir.list();
	    return list;
	} catch (Exception e) {
	    return null;
	}
    }


    /**
     * Returns the name of the oldest file in a given directory
     *
     * @param dn contains the name of the folder
     * @return String returns the name of the oldest file
     */
    public String getOldestFile (String dn) {

	if (dn == null) {
	    debug ("   no folder");
	    return null;
	}
	try {
	    File dir = new File (RootDirName
				 + File.separatorChar
				 + dn);

	    String[] list = dir.list();
	    if (list == null) {
		debug ("   no file in folder");
		return null;
	    }
	    long oldestTime = 0;
	    String oldest = null;
	    File file = null;
	    long time = 0;

	    for (int i = 0; i < list.length; ++i) {
		try {
		    file = new File (RootDirName +
				     File.separatorChar +
				     dn +
				     File.separatorChar +
				     list [i]);
		    if (file == null) {
			continue;
		    }
		    time = file.lastModified();
		    if (oldestTime == 0) {
			// This is the first one
			oldestTime = time;
			oldest = list [i];
			continue;
		    }
		    if (time < oldestTime) {
			// This one is more recent
			oldest = list [i];
			oldestTime = time;
			continue;
		    }
		} catch (Exception e1) {
		    debug ("getOldest failed " + e1);
		}
	    }
	    return oldest;

	} catch (Exception e) {
	    return null;
	}
    }


    /**
     * Returns true if the directory exists
     *
     * @param dn contains the name of the folder
     * @return boolean
     */
    public boolean isFolder (String dn) {

	if (dn == null) {
	    return false;
	}
	try {
	    File dir = new File (RootDirName
				 + File.separatorChar
				 + dn);
	    return dir.exists();
	} catch (Exception e) {
	    return false;
	}
    }

    /**
     * Returns true if the directory exists
     *
     * @param dn contains the name of the folder
     * @return boolean
     */
    public void cleanupFolder (String dn) throws IOException {

	if (dn == null) {
	    return;
	}
	debug (" cleaning up " + dn);
	String dirname = RootDirName + File.separator + dn;
	try {
	    String[] files = getFileNames (dn);
	    if (files == null) {
		debug ("  no files");
		return;
	    }
	    for (int i = 0; i < files.length; ++i) {
		File file = new File (dirname + File.separator + files[i]);
		if (file == null) {
		    debug ("   cannot get the file");
		    continue;
		}
		try {
		    file.delete();
		} catch (Exception e1) {
		    debug ("  failed removing " + files[i] + " : " + e1);
		    continue;
		}
	    }
	} catch (Exception e) {
	    debug ("   failed " + e);
	}
    }


    /**
     * Search and recovers documents that contains at least
     * a mathing pair of tag/value.
     *
     * @param dn contains the name of the folder on which to perform the search
     * @param tag contains the name of the tag on which to search on
     * @param value contains the value to search on.

     * @return Enumeration an enumeration of all the documents that
     * have been found.
     */

    public Enumeration search (String dn, String tag, String value) {

	String[] files = getFileNames(dn);

	if (files == null) {
	    return null;
	}

	Vector res = new Vector();
	int index = 0;

	for (index = 0; index < files.length; ++index) {
	    if (searchFile (dn, files[index], tag, value)) {
            debug("found a hit "+files[index]);
		res.addElement (files[index]);
	    }
	}
	return res.elements();
    }

    private boolean searchFile (String dn,
				String fn,
				String tag,
				String value) {
     boolean wildcard = false;
        StructuredDocument doc = null;
        try {
            doc = restore (dn, fn);
        } catch (Exception e) {
	    debug ("searchFile failed restoring doc: " + e);
            return false;
        }

        if (doc == null) {
	    debug ("searchFile failed restoring doc: doc is null");
            return false;
        }

        try {
            Element e = null;
            Enumeration enum = doc.getChildren();
            if ((enum == null) || !enum.hasMoreElements()) {
             debug ("searchFile failed restoring doc: doc has no element");
             return false;
            }
            while (enum.hasMoreElements()) {
                e = (Element) enum.nextElement();
                if (e.getKey().equals(tag)) {
                    Object val = e.getValue();

                    if( value.getClass().isInstance( val ) ) {

                        if (value.indexOf("*") !=-1){
                          wildcard=true;
                          value=value.substring(0,value.indexOf("*"));
                          // '*' by itself not allowed
                          if (value.length() == 0 ) return false;
                        }
                        else  wildcard=false;

                        // xxxxx  convert both values to upper case prior to making
                        // the comparison

                        String uVal   = ((String)   val).toUpperCase();
                        String uValue = ((String) value).toUpperCase();
                        if(wildcard) {
                         if ( uVal.indexOf(uValue) >= 0) {
                            return true;
                         }
                        } else {
                         if ( uVal.equals(uValue)) {
                            return true;
                         }
                        }
                    }
                }
            }
        } catch (Exception e) {
	    debug ("searchFile has failed: " + e);
            return false;
        }
        return false;
    }

    /**
     * Publish a document that can be searched by other peers.
     *
     * @param doc is a StructuredDocument to be published
     */
    public void publish (StructuredDocument doc) {
	publish (doc, createTmpName());
    }

    public void publish (StructuredDocument doc, String fn) {

	try {
	    save (PublicDirName, fn, doc);
	} catch (Exception e) {
	   debug("Cm: Cannot publish document");
	}
    }

    /**
     * Search and recovers documents that contains at least
     * a mathing pair of tag/value.
     *
     * @param tag contains the name of the tag on which to search on
     * @param value contains the value to search on.

     * @return Enumeration an enumeration of all the documents that
     * have been found.
     */

    public Enumeration searchLocal (String dn,
				    String tag,
				    String value) {

	debug ("searchLocal starts:");
	debug ("     folder = " + dn);
	debug ("     tag    = " + tag);
	debug ("     value  = " + value);

	String[] files = getFileNames(dn);

	if (files == null) {
	    debug ("searchLocal: no file in the folder");
	    return null;
	}

	Vector res = new Vector();
	int index = 0;

	for (index = 0; index < files.length; ++index) {
	    if (searchFile (dn, files[index], tag, value)) {
		res.addElement (files[index]);
	    }
	}
	if (res.size() == 0) {
	    debug ("searchLocal no result");
	    return null;
	}
	return res.elements();
    }

    public FileInputStream getInputStream(String dn, String fn)
	throws IOException
    {
	return new FileInputStream(RootDirName
					       + File.separatorChar
					       + dn
					       + File.separatorChar
					       + fn);
    }

    private void debug (String str) {
	if (debug) {
	    System.out.println ("[Cm] " + str);
	}
    }

}



