/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: CodeDescriptor.java,v 1.2 2001/05/23 01:00:17 lomax Exp $
 */

package net.jxta.impl.config;

import net.jxta.document.*;
import java.util.Enumeration;


/**
 * Simple class for an "implementation descriptor", which is just a fancy
 * way for saying a concrete classname together with a URI where the
 * class can be found. Instances of this class are immutable.
 * 
 */
public class CodeDescriptor {
    private String mClassname;
    private String mCodeURI;

    /** Constructs a a new CodeDescriptor object. **/
    public CodeDescriptor( String aClassname, String aCodeURI ) {
        mClassname = aClassname;
        mCodeURI = aCodeURI;
    }

    /** Returns the classname for the code **/
    public String getClassname() {
        return mClassname;
    }

    /** Returns the URI for the code **/
    public String getCodeURI() {
        return mCodeURI;
    }

    /** adds the classname and uri as elements. aNode is the node
    for this CodeDescriptor **/
    public void writeToXML( StructuredTextDocument aDoc, Element aNode ) {
        Element child = aDoc.createElement( "classname", mClassname );
        aNode.appendChild( child );
        child = aDoc.createElement( "uri", mCodeURI );
        aNode.appendChild( child );
        return;
    }

    /** Creates the descriptor from an XML element **/
    public static CodeDescriptor newFromXML( TextElement aElement ) {        
        Enumeration enum = aElement.getChildren();
        if ( enum == null ) return null;
        String cn=null;
        String uri=null;
        while ( enum.hasMoreElements() ) {
            TextElement e = (TextElement) enum.nextElement();
            String elname = e.getName();
            if ( elname.equals( "classname" ) )
                cn = e.getTextValue();
            else if ( elname.equals("uri") )
                uri = e.getTextValue();
        }
        return new CodeDescriptor( cn, uri );
    }
}
