/*
 * $Id: PlainTextDocument.java,v 1.8 2001/06/08 23:03:23 bondolo Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

package net.jxta.impl.document;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.lang.String;
import java.util.Enumeration;
import java.util.Vector;

import java.io.IOException;

import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredDocumentFactory;
import net.jxta.document.StructuredTextDocument;
import net.jxta.document.TextElement;
/**
 * This class is an implementation of the StructuredDocument interface using
 * simple text
 * @version     $Revision: 1.8 $
 * @since       JXTA 1.0
 */
public class PlainTextDocument extends PlainTextElement implements StructuredTextDocument {
    
    private static final MimeMediaType [] myTypes = {
        new MimeMediaType( "Text", "Plain" )
    };
    
    private MimeMediaType mimeType = null;
    
    /**
     *  Returns the MIME Media types supported by this this Document per 
     *  {@link http://www.ietf.org/rfc/rfc2046.txt}. 
     *
     *  Jxta does not currently support the 'Multipart' or 'Message' media types.
     *
     *  @return An array of MimeMediaType objects containing the MIME Media Type
     *  for this Document.
     *
     */
    public static MimeMediaType [] getSupportedMimeTypes() {
        return( myTypes );
    }

    /** Creates new PlainTextDocument */
    public PlainTextDocument( final MimeMediaType mimeType, final String type ) {
        super( null, type );
        doc = this;
        parent = this;
        
        this.mimeType = mimeType;
    }
    
    /** Creates new PlainTextDocument */
    public PlainTextDocument( final MimeMediaType mimeType, final InputStream stream ) {
        super( null, null );
        doc = this;
        parent = this;
        
        this.mimeType = mimeType;
        
        // FIXME bondolo@jxta.org  20010219    We don't do anything here....
    }
    
    /**
     * get Type
     */
    public MimeMediaType getMimeType() {
        return myTypes[0];
    }
    
    
    public String getRootType() {
        return name;
    }
    
    /**
     * create a new element without value
     */
    public net.jxta.document.Element createElement( Object key ) {
        return createElement( key, null );
    }
    
    /**
     * create a new element with value
     */
    public net.jxta.document.Element createElement( Object key, Object val) {
        if( !String.class.isAssignableFrom( key.getClass() ) )
            throw new ClassCastException( key.getClass().getName() + " not supported by createElement." );
        
        if( (null != val) && !String.class.isAssignableFrom( val.getClass() ) )
            throw new ClassCastException( val.getClass().getName() + " not supported by createElement." );
        
        return (net.jxta.document.Element) new PlainTextElement( this, (String) key, (String) val );
    }
    
    /**
     * create a new element without value
     */
    public TextElement createElement( String name ) {
        return (TextElement) new PlainTextElement( this, name );
    }
    
    /**
     * create a new element with value
     */
    public TextElement createElement(String name, String val) {
        return (TextElement) new PlainTextElement( this, name, val );
    }
    
    public InputStream getStream() {
        // XXX  bondolo@jxta.org 20010307    Should be using a pipe
        StringWriter  stringOut = new StringWriter();
        
        printNice( stringOut, 0, true );
        
        return new ByteArrayInputStream( stringOut.toString().getBytes() );
    }
    
    public void sendToStream( OutputStream stream ) throws IOException {
        Writer out = new BufferedWriter(new OutputStreamWriter(stream));
        
        sendToWriter( out );
        out.flush();
    }
    
    public Reader getReader() {
        // XXX  bondolo@jxta.org 20010307    Should be using a pipe
        StringWriter  stringOut = new StringWriter();
        
        printNice( stringOut, 0, true );
        
        return new StringReader( stringOut.toString() );
    }
    
    public void sendToWriter( Writer stream ) throws IOException {
        printNice( stream, 0, true );
    }
}
