/************************************************************************
 *
 * $Id: EndpointRouterMessage.java,v 1.11 2001/05/23 01:00:18 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/


package net.jxta.impl.endpoint;

import java.net.*;
import java.io.*;
import java.util.*;
import net.jxta.document.*;


/**
 * This class defines the wire format of HTTP messages.
 *
 */

public class EndpointRouterMessage {

    private StructuredDocument doc = null;
    public static final String SrcTag = "Src";
    public static final String DestTag = "Dest";
    public static final String TTLTag = "TTL";
    public static final String GatewayTag = "Gateway";
    private Vector gateways = null;
    private String srcPeer = null;
    private String destPeer = null;
    private int    TTL = 0;
    public static final String Name = "JxtaEndpointRouter";
    
    public EndpointRouterMessage () {
	gateways = new Vector();
    }

    public EndpointRouterMessage (InputStream ip) {
	doc = createDoc (ip);
	parse();
    }

    public byte[] getBytes() {

	if (doc == null) {
	    return null;
	}
	try {
	    InputStream ip = doc.getStream();
	    byte[] buffer = new byte [ip.available()];
	    ip.read (buffer);
	    return buffer;
	} catch (Exception e) {
	    return null;
	}
    }

    public InputStream getInputStream () {

	doc = createDoc();
	if (doc == null) {
	    System.err.println("EndpointRouterMessage cannot create doc");
	} 

	Element e = null;
	if (srcPeer != null) {
	    e = doc.createElement (SrcTag, srcPeer);
	    doc.appendChild (e);
	}

	if (destPeer != null) {
	    e = doc.createElement (DestTag, destPeer);
	    doc.appendChild (e);
	}

	if (TTL != 0) {
	    e = doc.createElement (TTLTag, Integer.toString(TTL));
	    doc.appendChild (e);
	}

	if (gateways != null) {
	    for (int i=0; i < gateways.size(); ++i) {
		try {
		    e = doc.createElement (GatewayTag,
                                           (String) gateways.elementAt(i));
		    doc.appendChild (e);
		} catch (Exception ee) {
		    continue;
		}
	    }
	}

	try {
	    return doc.getStream();
	} catch (Exception ee) {
	    return null;
	}
    }

    private void parse () {

	Enumeration enum = null;
	Vector tmp = new Vector();
	TextElement e = null;
	
	enum = doc.getChildren();
	if ((enum == null) || (!enum.hasMoreElements())) {
	    return;
	}

	while (enum.hasMoreElements()) {
	    try {
		e = (TextElement) enum.nextElement();
		if (e.getName().equals (SrcTag)) {
		    srcPeer = e.getTextValue();
		    continue;
		}

		if (e.getName().equals (DestTag)) {
		    destPeer = e.getTextValue();
		    continue;
		}

		if (e.getName().equals (TTLTag)) {
		    TTL = Integer.parseInt (e.getTextValue());
		    continue;
		}

		if (e.getName().equals (GatewayTag)) {
		    try {
			tmp.addElement(e.getTextValue());
		    } catch (Exception ee) {
			continue;
		    }
		    continue;
		}
	    } catch (Exception ee) {
		continue;
	    }
	}
		
	// build a smaller vector;
	gateways = new Vector (tmp.size());
	for (int i=0; i < tmp.size(); ++i) {
	    try {
		gateways.addElement (tmp.elementAt (i));
	    } catch (Exception ee) {
		continue;
	    }
	}
    }
    
    public void setSrcPeer (String p) {
	srcPeer = new String (p);
    }

    public String getSrcPeer () {
	return srcPeer;
    }


    public void setDestPeer (String p) {
	destPeer = new String (p);
    }

    public String getDestPeer () {
	return destPeer;
    }

    public void setTTL (int t) {
	TTL = t;
    }

    public int getTTL () {
	return TTL;
    }

    public void setGateways (Vector v) {
	gateways = (Vector ) v.clone();
    }

    public Vector getGateways () {
	return gateways;
    }

    private StructuredDocument createDoc() {
        try {
            return StructuredDocumentFactory.newStructuredDocument (
            new MimeMediaType ("text/xml"), "jxta:EndpointRouterMessage");
        }
        catch ( Exception everything ) {
            return null;
        }
    }

    private StructuredDocument createDoc(InputStream ip) {
        try {
            return StructuredDocumentFactory.newStructuredDocument (
            new MimeMediaType ("text/xml"), ip);
        }
        catch ( Exception everything ) {
            return null;
        }
   }

}








