/************************************************************************
 *
 * $Id: TransportProtocol.java,v 1.9 2001/05/23 01:00:18 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.endpoint;
import net.jxta.endpoint.*;

import java.io.*;

/**
 * This interface defines the API that must implement all the
 * Transport Protocols Messangers
 */

public interface TransportProtocol {

    /**
     * Creates and return an EndpointMessanger for sending Message's
     *
     * @param dest EndpointAddress of the destination
     * @param type type of the messanger as defined in the class Endpoint
     * @return an EndpointMessanger.
     */
    public EndpointMessanger getMessanger (EndpointAddress dest,
					   String destPeerId,
					   int type)

	throws IOException;



    /**
     * Propagates a Message on this TransportProtocol
     *
     * @param msg the Message to be propagated
     * @param serviceName is a String containing the name of the service
     * @param serviceParams is a String containing the parameters associated
     * to the service. The combination of serviceName and serviceParams
     * is the what must have been registered to the Endpoint's demux.
     * @param prunePeer peer which should not receive the propagated message
     * @param msg the Message to be propagated
     */
    public void propagate (Message msg,
			   String serviceName,
			   String serviceParams,
			   String prunePeer) throws IOException;

    /**
     * closes this TransportProtocol.
     */
    public void close();

    /**
     * Initialise this instance of the TransportProtocol.
     *
     * XXX: this method prototype will need to be revisited
     * when the configuration framework will be completed.
     */
    public void init(Endpoint p);

    /**
     * Returns a String containing the canonical name of this
     * protocol, as it appears in an EndpointAddress.
     *
     * @return a String containing the name of the protocol
     */
    public String getProtocolName();

    /**
     * Returns an EndpointAddress of the local endpoint
     * manage by the TransportProtocol
     *
     * @return an EndpointAddress containing the local address
     */
    public EndpointAddress getLocalAddress();

    /**
     * Returns true if the Transport Protocol is fast. The notion
     * of being fast is relative, but any mailbox type of protocols
     * should return false.
     */
    public boolean isFast ();

    /**
     * Returns true if the Transport Protocol can establish
     * connection to the remote host (like TCP).
     */
    public boolean isConnected();

    /**
     * Returns true if the target address is reachable. Otherwise
     * returns false.
     */
    public boolean ping (EndpointAddress addr);
}
