/************************************************************************
 *
 * $Id: HttpMessage.java,v 1.12 2001/05/23 01:00:19 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/


package net.jxta.impl.endpoint.http;

import java.net.*;
import java.io.*;
import java.util.*;
import net.jxta.document.*;
import net.jxta.impl.cm.*;



/**
 * This class defines the wire format of HTTP messages.
 *
 */

public class HttpMessage {

    private String doc = null;
    public static final String CodeTag = "Code";
    public static final String ContentTag = "Msg";


    public static final int Ok = 1;
    public static final int Failed = 2;
    public static final int Empty = 3;
    public static final int Content = 4;

    // This constructor is used to send a reply containing
    // only a return code.

    public HttpMessage (int code) {

	doc =  "<" + CodeTag + ">" + Integer.toString (code) +
	    "</" + CodeTag + ">";
    }

    // This constructor is used to create a HttpMessage off a
    // a String (receiving side).
    // Note that the given buffer is expected to not be altered
    // by the caller, in order to avoid an extra copy.

    public HttpMessage (String buffer) {
	doc = buffer;
    }


    // This constructore is used to send a cmd with a content
    // stored in the CM
    public HttpMessage (String dn, String fn) {

	byte[] content = null;

	try {
	    // First get the file.
	    content = HttpTransport.cm.restoreBytes (dn, fn);
	    if (content == null) {
		return;
	    }
	    doc = new String (content);
	} catch (Exception e) {
	    return;
	}
    }

    public HttpMessage (String dn, String fn, byte[] buffer) {

	doc =  "<" + CodeTag + ">" + Integer.toString (Content) +
	    "</" + CodeTag + ">";
	doc += "<" + ContentTag + ">\n" + new String (buffer) +
	    "</" + ContentTag + ">";

	try {
	    HttpTransport.cm.saveBytes (dn, fn, doc.getBytes());
	} catch (Exception ee) {
	}
    }

    public HttpMessage (byte[] buffer) {

	doc = new String (buffer);
    }

    public byte[] getBytes() {

	if (doc == null) {
	    return null;
	}
	return doc.getBytes();
    }

    public byte[] getData() {

	if (doc == null) {
	    return null;
	}

	int indexFirst = doc.indexOf ("<" + ContentTag + ">");
	int indexLast = doc.lastIndexOf ("</" + ContentTag + ">");

	if ((indexFirst == -1) || (indexLast == -1)) {
	    // Invalid message
	    return null;
	}
	return doc.substring (indexFirst + ContentTag.length() + 2,
			      indexLast).getBytes();
    }
}








