/************************************************************************
 *
 * $Id: TcpNonBlockingMessanger.java,v 1.17 2001/06/10 02:24:41 bondolo Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/


package net.jxta.impl.endpoint.tcp;

import java.net.*;
import java.io.*;
import java.util.*;
import net.jxta.endpoint.*;
import net.jxta.impl.endpoint.*;


/**
 * A TransportProtocolLink allows to send TransportMessage's onto
 * its associated TransportProtocol.
 *
 * @version     $Revision: 1.17 $
 * @since       JXTA 1.0
 */

public class TcpNonBlockingMessanger implements EndpointMessanger {

    public boolean debugFlag = false;

    private static int id = 0;

    private EndpointAddress dstAddress = null;
    private EndpointAddress srcAddress = null;
    private InetAddress inetAddress = null;
    private int port = 0;
    private Header header = null;
    private TcpTransport proto = null;
    private Socket socket = null;

    public TcpNonBlockingMessanger (EndpointAddress addr,
				    String peer,
				    TcpTransport p)
                     throws IOException {

	// Not much has to be done: the connection with the remote node
	// will happen only when the data will be sent.
	debug ("Constructor");
	debug ("   addr = " + addr.toString());
	debug ("   peer = " + peer);

	header = new Header();
	proto = p;
	
	dstAddress = new Address (addr.toString());
	srcAddress = proto.getLocalAddress();
	try {
	    String tmp = addr.getProtocolAddress();
	    int portIndex = tmp.indexOf (":");
	    if (portIndex == -1) {
		throw new IOException ();
	    }
	    port = Integer.valueOf (tmp.substring (portIndex + 1)).intValue();
	    inetAddress = InetAddress.getByName (tmp.substring (0, portIndex));
	    debug ("Constructor using: " + inetAddress.toString());
	} catch (Exception e) {
	    System.out.println(e);
	    throw new IOException();
	}

	if (!openSocket()) {
	    throw new IOException();
	}
    }


    private boolean openSocket () {

	debug ("openSocket for " + inetAddress.toString());
	// Open a socket with the destination and send the message
	socket = null;

	try {
	    socket = new Socket (inetAddress, port);
	    return true;
	} catch (Exception e) {
	    debug ("openSocket: cannot open socket with " + inetAddress.toString());
	    return false;
	}
    }


    /**
     * Send a TransportMessage onto that link.
     *
     * @param message TransportMessage to be sent.
     */
    public void sendMessage (Message message) throws IOException {

	if (dstAddress == null) {
	    debug ("sendMessage: no destination address");
	    throw new IOException();
	}
	// Set the message with the appropriate src and dest address
	((MessageImpl) message).setDestAddress (dstAddress);
	((MessageImpl) message).setSrcAddress (srcAddress);

	debug ("sendMessage");
	debug ("   from = " + srcAddress.toString());
	debug ("     to = " + dstAddress.toString());

	// Build the protocol header
	// Allocate a buffer to contain the message and the header

	InputStream inputStream = ((MessageImpl) message).getInputStream();
	int msgSize = inputStream.available();

	byte[] buffer = new byte [msgSize + header.length];

	// First build the header
	header.cmd = Header.UNICAST;

	// XXX: This part could probably be initialized only once. To be fixed.
	header.srcAddr = proto.usingInterface.getAddress();

	header.srcPort = proto.serverSocketPort;

	header.option = Header.NONBLOCKING;
	header.size = msgSize;

	header.buildForNetwork (buffer, 0);

	// Store the message into the buffer. It is expected to be able to
	// store the message in one read.

	int tmpIndex = header.length;
	int res = 0;

	while (inputStream.available () > 0) {
	    res = inputStream.read ();
	    if (res == -1) {
		debug ("  no data to send");
		break;
	    }
	    buffer [tmpIndex++] = (byte) res;
	}

	// Open a socket with the destination and send the message
	if (socket == null) {
	    debug ("   socket is null");
	    if (!openSocket()) {
		debug ("  cannot get socket");
		throw new IOException();
	    }
	}


	OutputStream outputStream = null;
	int cnt = 0;
	byte[] rep = new byte[1];

	while (cnt < 2) {
	    try {
		outputStream = socket.getOutputStream();

		// Send the message
		outputStream.write (buffer);
		outputStream.flush();

                // If the remote peers allows keep alive, it returns 1
                // otherwise, it either returns 0, or simply closes the socket.
		try {
		    socket.setSoTimeout(5000);
		} catch (Exception ex) {
		    debug ("sendMessage failed with " + ex);
		}
		InputStream ip = socket.getInputStream();
                ip.read (rep);
		if (rep[0] != 1) {
		    // Remote does not want to keep the connection alive.
		    outputStream.close();
		    socket.close();
		    socket = null;
		}
		return;

	    } catch (Exception e) {
		// The connection has failed. Try to reconnect once
		// before failing.
		++cnt;
		debug ("   socket has failed. Get a new connection " + e);
		if (openSocket()) {
		    debug ("   got new socket");
		    try {
			outputStream = socket.getOutputStream();
		    } catch (Exception e2) {
			debug ("   outputStream failed " + e);
		    }
		    continue;
		} else {
		    // We can't. Just fail.
		    try {
			if (socket != null) {
			    debug ("   giving up");
			    socket.close();
			    socket = null;
			}
		    } catch (Exception e2) {
			debug ("  failed opening socket " + e2);
			throw new IOException();
		    }
		}
	    }
	}
	debug ("sendMessage failed - exit");
    }

    /**
     * Close this TransportProtocolLink
     */
    public void close() {

	if (socket != null) {
	    try {
		socket.close();
	    } catch (Exception e) {
	    }
	}
    }

    private void debug (String str) {
	if (debugFlag) {
	    System.out.println ("[TcpNonBlockingMessanger] " + str);
	}
    }
}
