/************************************************************************
 *
 * $Id: TcpTransport.java,v 1.33 2001/06/10 02:24:41 bondolo Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.endpoint.tcp;

import java.net.*;
import java.io.*;
import java.util.*;

import net.jxta.endpoint.*;
import net.jxta.protocol.*;

import net.jxta.impl.endpoint.*;
import net.jxta.impl.protocol.*;

import net.jxta.util.*;

/***
 * This class implements the TCP Transport Protocol
 */

public class TcpTransport implements TransportProtocol, Runnable {
    
    public boolean       debug = false;
    
    public ServerSocket     unicastSocket;
    InetAddress             usingInterface;
    private  InetAddress  propagateInetAddress;
    private  int          propagatePort;
    private  int          propagateSize;
    public   int          serverSocketPort;
    
    private Thread              multicastThread = null;
    private Thread              unicastThread = null;
    
    private Socket              inputSocket = null;
    
    private MulticastSocket     multicastSocket = null;
    
    public Endpoint             endpoint = null;
    private Header header = new Header();
    private Address localAddress = null;
    private EndpointAdvertisement epadv= null;
    private Address mAddress = null;
    private String              localSubnet = null;
    
    /**
     * This is a portion of the API which might not be useful when
     * the configuration part of the JXTA platform will be completed.
     * XXX: to revisit.
     */
    
    public static int unicastPortNb = 9001;
    public static String multicastAddress = "224.0.1.85";
    public static int multicastPortNb = 1234;
    public static int multicastPacketSize = 8192;
    public static String publicAddress = null;
    
    private static final long PingDelay = 2000; // 2 seconds
    
    private IncomingUnicastThreads unicastThreads = null;
    public  static final int DefaultNbOfUnicastThreads = 1;
    public  static final int MaxNbOfUnicastThreads = 10;
    public  static final int MaxNbOfUnicastKeepAliveThreads = 4;
    public  static final int MaxKeepAliveDelay = 5 * 60 * 1000; // 5 Minutes
    
    private IncomingMulticastThreads multicastThreads = null;
    public  static final int DefaultNbOfMulticastThreads = 0;
    public  static final int MaxNbOfMulticastThreads = 5;
    public  static final int MaxNbOfPendingSockets = 10;
 
    public TcpTransport(EndpointAdvertisement transport) {
        
        epadv = transport;
        TCPAdv tcpadv = (TCPAdv) transport.getTransportAdvertisement();
        unicastPortNb = new Integer(tcpadv.getPort()).intValue();
        multicastAddress = tcpadv.getMulticastAddr();
        multicastPortNb = new Integer(tcpadv.getMulticastPort()).intValue();
        multicastPacketSize =
        new Integer(tcpadv.getMulticastSize()).intValue();
        
        debug("Configuring TCP Transport");
        debug("UnicastPort:"+ unicastPortNb);
        debug("MulticastAddr:"  + multicastAddress);
        debug("MulticastPort:"  + multicastPortNb);
        debug("MulticastPacketSize:"  + multicastPacketSize);
        publicAddress = tcpadv.getPublicAddress();
        if ((publicAddress != null) && !publicAddress.equals("TBD")) {
            debug("Public address:"  + publicAddress);
        } else {
            publicAddress = null;
        }
        mAddress = new Address();
        mAddress.setProtocolName("tcp");
        mAddress.setProtocolAddress(multicastAddress);
    }
    
    public int getPort() {
        return serverSocketPort;
    }
    
    public InetAddress getMcastAddr() {
        return propagateInetAddress;
    }
    
    public int getMcastPort() {
        return propagatePort;
    }
    
    public int getMcastSize() {
        return propagateSize;
    }
    
    /**
     * Initialization of the TcpTransport (called by Platform)
     */
    
    public void init(Endpoint ep) {
        
        int port = unicastPortNb;
        String multicastAddr = multicastAddress;
        int multicastPort = multicastPortNb;
        int multicastSize = multicastPacketSize;
        
        endpoint = ep;

        debug = ep.debugFlag;

        unicastSocket = null;
        
        unicastThreads = new IncomingUnicastThreads(this, DefaultNbOfUnicastThreads);
        multicastThreads = new IncomingMulticastThreads(this, DefaultNbOfMulticastThreads);
        
        /* Open the incoming socket. */
        try {
            unicastSocket = new ServerSocket(port);
        } catch (IOException e1) {
            System.out.println ("Cannot create ServerSocket on port " + port +
            ": " + e1.toString());
            
            System.out.println("The socket is already in use (perhaps by another running " +
            "instance of the JXTA Platform)");
            
            throw new RuntimeException( "Could not create transport" );
            
        } catch (SecurityException e2) {
            debug("Cannot create ServerSocket on port " + port +
            ": " + e2.toString());
        }
        
        // First create a listening thread waiting for incoming message
        // on serverSocket
        
        serverSocketPort = port;
        
        // Builds the localAddress
        localAddress = new Address();
        try {
            localAddress.setProtocolName("tcp");
            if ((publicAddress == null) || (publicAddress.equals("TBD"))) {
                usingInterface = InetAddress.getLocalHost();
            } else {
                usingInterface = InetAddress.getByName( publicAddress );
            }
            localAddress.setProtocolAddress( usingInterface.getHostAddress() + 
            ":" + port);
            epadv.setEndpointAddress(localAddress.toString());
        } catch (Exception e) {
            // Not much can be done here. Just print out an error
            System.out.println("TcpTransport init: " + e);
            return;
        }
        
        debug( "Using Interface : " +  usingInterface.getHostAddress() );

        unicastThread = new Thread(this,
        "TcpTransport:unicast server thread" );
        unicastThread.start();
        
        // Create the multicast input socket
        propagatePort = multicastPort;
        propagateSize = multicastSize;
        try {
            propagateInetAddress = InetAddress.getByName(multicastAddr);
        } catch (Exception e) {
            debug(e.toString());
        }
        
        ServerSocket propagateSocket = null;
        
        multicastSocket = null;
        
        try {
            multicastSocket = new MulticastSocket(propagatePort);
            multicastSocket.setInterface( usingInterface );
            multicastSocket.joinGroup(propagateInetAddress);
            debug( "Multicast : " + propagateInetAddress +
            ":" + propagatePort +
            " on interface " +
            multicastSocket.getInterface().getHostAddress() );
        } catch (Exception e) {
            debug(e.toString());
        }
        
        multicastThread = new Thread(this,
        "TcpTransport:multicast server thread" );
        multicastThread.start();
    }
    
    public TcpTransport() {
    }
    
    public void run() {
        
        Thread current = Thread.currentThread();
        
        if (current.equals(unicastThread)) {
            runUnicastServer();
        }
        
        if (current.equals(multicastThread)) {
            runMulticastServer();
        }
    }
    
    public void runUnicastServer() {
        
        Socket socket;
        for (;;) {
            try {
                socket = unicastSocket.accept();
            } catch (IOException e1) {
                debug("ServerSocket.accept() on port " +
                serverSocketPort +
                " has failed with: " + e1.toString());
                
                continue;
            } catch (SecurityException e2) {
                debug("ServerSocket.accept() on port " +
                serverSocketPort +
                " has failed : " + e2.toString());
                continue;
            }
            debug("runUnicastServer: received a connection");
            processReceivingSocket(socket);
        }
    }
    
    public void runMulticastServer() {
        
        byte[] buffer;
        
        for (;;) {
            buffer = new byte [propagateSize];
            DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
            try {
                multicastSocket.receive(packet);
            } catch (Exception e) {
                debug("runMulticastServer " + e);
                break;
            }
            processReceivingBuffer(buffer);
        }
    }
    
    public void processMulticast(byte[] buffer) {
        
        debug("processMulticast starts");
        if (buffer == null) {
            debug("   no buffer");
            return;
        }
        // Get the header
        header.initFromNetwork(buffer, 0);
        
        try {
            if( header.size > buffer.length )
                throw new IOException( "size from header is larger than buffer" );
            
            
            MessageImpl msg = new MessageImpl(buffer,
            Header.length,
            header.size);
            
            // Give the message to the Endpoint Manager
            debug("  handing message to Endpoint");
            endpoint.demux(msg);
        } catch (IOException e) {
            debug("   discard message - exception " + e);
            // Just discard the message. Multicast are not reliable
            return;
        }
    }
    
    public void runReceive(Socket socket, boolean keep) {
        
        debug("runReceive starts");
        if (socket == null) {
            debug("   no socket");
            return;
        }
        
        InputStream inputStream = null;
        int size = 0;
        
        try {
            inputStream = socket.getInputStream();
            if (inputStream == null) {
                debug("   no inputStream");
                socket.close();
                return;
            }
        } catch (Exception e) {
            debug("  failed to create inputStream");
            try {
                socket.close();
            } catch ( Exception ignored ) {
            }
            return;
        }
        
        while (true) {
            try {
                // Get the header
                byte[] headerBuffer = new byte [Header.length];
                
                socket.setSoTimeout(MaxKeepAliveDelay);
                size = inputStream.read(headerBuffer, 0, header.length);
                if( -1 == size ) {
                    debug( "   EOF on inputsteam. Closing socket" );
                    inputStream.close();
                    socket.close();
                    return;
                }
                
                if (size != header.length) {
                    // We assume that the header is transmitted in one chunk
                    // Discard message
                    debug("   wrong header size= " + size +
                    " expected= " + header.length);
                    inputStream.close();
                    socket.close();
                    return;
                }
                
                header.initFromNetwork(headerBuffer, 0);
                
                size = 0;
                int tmpSize = 0;
                byte[] buffer = new byte [header.size];
                while (size < header.size) {
                    try {
                        tmpSize = inputStream.read(buffer,
                        size,
                        header.size - size);
                    } catch (Exception e) {
                        // The stream is broken. Discard the message
                        debug("   inputStream is broken");
                        inputStream.close();
                        socket.close();
                        return;
                    }
                    if (tmpSize == -1) {
                        // No more bytes to receive. This is an error. Discard the
                        // message.
                        debug("  no more bytes to receive - incomplete message");
                        inputStream.close();
                        socket.close();
                        return;
                    }
                    size += tmpSize;
                }
                
                if (size != header.size) {
                    debug("    header size is invalid");
                    inputStream.close();
                    socket.close();
                    return;
                }
                
                // Send keepalive info
                try {
                    OutputStream op = socket.getOutputStream();
                    if (op != null) {
                        byte[] rep = new byte[1];
                        if (keep) {
                            debug("   keep alive");
                            // Keep alive ok
                            rep[0] = 1;
                        } else {
                            // Keep alive not ok
                            debug("   not keep alive");
                            rep[0] = 0;
                        }
                        op.write(rep);
                        op.flush();
                    }
                    else
                        debug("Cannot send keepalive");
                } catch (Exception e) {
                    // Not fatal
                    debug("Cannot send keepalive ok");
                }
                
                // Give the message to the endpoint manager
                debug("   handing incoming message to Endpoint");
                MessageImpl msg = new MessageImpl(buffer, 0, header.size);
                
                endpoint.demux(msg);
                
                if (!keep) {
                    // No keep alive. Close the socket and return
                    debug("Not keepalive. close socket");
                    inputStream.close();
                    socket.close();
                    return;
                }
            } catch (Exception e) {
                debug("runReceiveError " + e);
                return;
            }
        }
    }
    
    /**
     * Creates and return an EndpointMessanger
     */
    public EndpointMessanger getMessanger(EndpointAddress dst,
    String peer,
    int type)
    throws IOException {
        
        // XXX: the following is a work around in order to
        //      strip out peers that advertise a local IP address
        //      as their Endpoint (127.0.0.1)
        // lomax@jxta.org
        
        String addr = dst.getProtocolAddress();
        if (addr.indexOf("127.0.0.1") >= 0) {
            debug("getMessanger is trying to get 127.0.0.1 - discard");
            return null;
        }
        
        switch (type) {
            case Endpoint.NonBlocking:
                return new TcpNonBlockingMessanger(dst, peer, this);
            case Endpoint.HandChecking:
                // XXX: not implemented yet
                debug("HandCheck is not yet implemented on TCP");
                throw new IOException();
            default:
                debug("TcpTransport: unknown type");
                throw new IOException();
        }
    }
    
    
    /**
     * Propagates a TransportMessage on this TransportProtocol
     *
     * @param message the TransportMessage to be propagated
     * @param pName the name of a service
     * @param pParams parameters for this message.
     * @param prunePeer ????
     * @exception IOException   thrown if the message could not be sent for some reason.
     */
    public synchronized void propagate(Message message,
    String pName,
    String pParams,
    String prunePeer) throws IOException {
        
        
        debug("propagate");
        debug("prunepeer = " + prunePeer);
        
        // First build the destination and source addresses
        mAddress.setServiceName(pName);
        mAddress.setServiceParameter(pParams);
        
        ((MessageImpl) message).setDestAddress(mAddress);
        ((MessageImpl) message).setSrcAddress(localAddress);
        
        
        // Allocate a buffer to contain the message and the header
        InputStream inputStream = ((MessageImpl) message).getInputStream();
        // FIXME   20010418 MJD available() does not always return the length of streams!
        int msgSize = inputStream.available();
        
        if( (msgSize + header.length) > propagateSize )
            // Something odd happened. Discard the message
            throw new IOException( "Message discarded : " + 
              "message was larger than propagate size (" +
              (msgSize + header.length) + ") bytes" );
        
        byte[] buffer = new byte [msgSize + header.length];
        
        // First build the header
        header.cmd = Header.PROPAGATE;
        
        // XXX: This part could probably be initialized only once. To be fixed.
        header.srcAddr = usingInterface.getAddress();
        
        header.srcPort = serverSocketPort;
        header.size = msgSize;
        
        header.buildForNetwork(buffer, 0);
        
        // Store the message into the buffer. It is expected to be able to
        // store the message in one read.
        
        int tmpSize = inputStream.read(buffer, header.length, msgSize);
        if (tmpSize != msgSize) {
            // Something odd happened. Discard the message
            throw new IOException( "Message discarded : could not read whole message from stream" );
        }
        
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length,
        propagateInetAddress,
        propagatePort);
        if (multicastSocket == null) {
            // The multicast socket needs first to be allocated.
            try {
                multicastSocket = new MulticastSocket(propagatePort);
                multicastSocket.joinGroup(propagateInetAddress);
            } catch (Exception e) {
                debug(e.toString());
            }
        }
        try {
            multicastSocket.send(packet);
        } catch (Exception e) {
            debug(e.toString());
        }
    }
    
    
    public String getProtocolName() {
        return "tcp";
    }
    
    /**
     * closes this TransportProtocol.
     */
    public void close() {
        // XXX: is there anything to do ? To be checked
    }
    
    public EndpointAddress getLocalAddress() {
        return localAddress;
    }
    
    public boolean isFast() {
        return true;
    }
    
    public boolean isConnected() {
        return true;
    }
    
    //
    // Ping a remote peer to see if it is reachable
    // XXX: this should be improved by actually really sending a ping
    // request and verifying that the remote address is really the same
    // reamote peer.
    // XXX: to be fixed. lomax@jxta.org
    
    protected class TcpDoPing implements Runnable {
        
        private TcpDoPingHandler handler = null;
        private EndpointAddress addr = null;
        
        public TcpDoPing(EndpointAddress a, TcpDoPingHandler h) {
            
            addr = a;
            handler = h;
            
            Thread th = new Thread(this, "TcpDoPing");
            th.start();
        }
        
        public void cancel() {
            handler = null;
        }
        
        private void signal(boolean v) {
            if (handler != null) {
                handler.signal(v);
            }
        }
        
        public void run() {
            
            Socket socket;
            InetAddress inetAddress = null;
            int port = 0;
            
            try {
                String tmp = addr.getProtocolAddress();
                int portIndex = tmp.indexOf(":");
                if (portIndex == -1) {
                    signal(false);
                }
                port = Integer.valueOf(tmp.substring(portIndex + 1)).intValue();
                inetAddress = InetAddress.getByName(tmp.substring(0, portIndex));
                
            } catch (Exception e) {
                signal(false);
            }
            if (inetAddress == null) {
                signal(false);
            }
            
            try {
                OutputStream outputStream = null;
                socket = new Socket(inetAddress, port, usingInterface, 0 );
                outputStream = socket.getOutputStream();
                outputStream.close();
                socket.close();
                signal(true);
            } catch (Exception e) {
                try {
                    debug("Ping [" + addr.toString() + "] exception " + e);
                } catch (Exception e2) {
                    debug("Ping exception " + e);
                }
                signal(false);
            }
        }
    }
    
    protected class TcpDoPingHandler {
        
        private long delay = 0;
        private boolean status = false;
        private Thread thread = null;
        
        public TcpDoPingHandler(long d) {
            delay = d;
            thread = Thread.currentThread();
        }
        
        public boolean waitForPing() {
            
            try {
                Thread.sleep(delay);
                thread = null;
                return false;
            } catch (Exception e) {
                Thread.interrupted();
                return status;
            }
        }
        
        public void signal(boolean s) {
            status = s;
            if (thread != null) {
                thread.interrupt();
            }
        }
    }
    
    
    public  boolean ping(EndpointAddress addr) {
        
        try {
            debug("ping to " + addr.toString());
        } catch (Exception e) {
            debug("ping cannot print address because: " + e);
        }
        
        // XXX: the following is a work around in order to
        //      strip out peers that advertise a local IP address
        //      as their Endpoint (127.0.0.1)
        // lomax@jxta.org
        
        try {
            String tmpAddr = addr.getProtocolAddress();
            if (tmpAddr.indexOf("127.0.0.1") >= 0) {
                debug("   pinging localhost - discard");
                return false;
            }
        } catch (Exception e) {
            debug("  failed " + e);
            return false;
        }
        
        TcpDoPingHandler handler = new TcpDoPingHandler(PingDelay);
        TcpDoPing pinger = new TcpDoPing( addr, handler);
        boolean res =  handler.waitForPing();
        debug("ping returns " + res);
        return res;
    }
    
    private void debug(String str) {
        if (debug) {
            System.out.println("[TcpTransport] " + str);
        }
    }
    
    
    public class IncomingUnicastThreads implements Runnable {
        
        protected Vector sockets = null;
        protected TcpTransport tp = null;
        private   int waitingThreads = 0;
        private   int nbOfThreads = 0;
        private   int nbOfKeepAlive = 0;
        
        public IncomingUnicastThreads(TcpTransport tp, int initialNumber) {
            
            this.tp = tp;
            sockets = new Vector();
            
            for (int i = 0; i < initialNumber; ++i) {
                ++nbOfThreads;
                Thread t = new Thread(this,
                "Incoming Unicast Thread " + i);
                t.start();
            }
        }
        
        public void run() {
            
            Socket socket = null;
            while (true) {
                socket = waitForSocket();
                if (socket == null) {
                    // That means that this thread must die
                    decThreads();
                    return;
                }
                boolean keep = doesKeepAlive();
                tp.runReceive(socket, keep);
                if (keep) {
                    stopKeepAlive();
                }
            }
        }
        
        synchronized private void decThreads() {
            --nbOfThreads;
        }
        
        synchronized protected Socket waitForSocket() {
            
            Socket socket = null;
            
            while (true) {
                
                if (sockets.size() > 0) {
                    try {
                        socket = (Socket) sockets.elementAt(0);
                    } catch (Exception e) {
                        // Strange error. XXX: to handle better
                        debug("waitForSocket error: " + e);
                    }
                    if (socket != null) {
                        try {
                            sockets.removeElementAt(0);
                        } catch (Exception e) {
                            debug("waitForSocket error: " + e);
                        }
                        return socket;
                    }
                    // We should not get here.
                    debug("waitForSocket cannot get incoming socket");
                }
                
                // No more socket to consume... Just wait.
                // Check if this thread must die or wait for a new socket
                if (mustDie()) {
                    debug("waitForSocket: killing thread");
                    return null;
                }
                try {
                    ++waitingThreads;
                    wait();
                    --waitingThreads;
                } catch (Exception e) {
                }
            }
        }
        
        synchronized private boolean mustDie() {
            
            if ((nbOfThreads > MaxNbOfUnicastThreads) ||
            (waitingThreads > 1)) {
                return true;
            }
            return false;
        }
        
        synchronized private boolean doesKeepAlive() {
            
            if (nbOfKeepAlive >= MaxNbOfUnicastKeepAliveThreads) {
                return false;
            }
            ++nbOfKeepAlive;
            return true;
        }
        
        synchronized private void stopKeepAlive() {
            --nbOfKeepAlive;
        }
        
        
        synchronized protected void newSocket(Socket socket) {
            
            if (sockets.size() > MaxNbOfPendingSockets) {
                // Too many request. Just drop it.
                try {
                    socket.close();
                } catch (Exception e) {
                    debug ("newSocket socket.close failed with " + e);
                }
                debug ("newSocket: too many incoming request, dropped incoming message");
                return;
	    }

            sockets.addElement(socket);
            // Check if we need to allocate a new Thread
            if (waitingThreads == 0) {
                ++nbOfThreads;
                // Allocate a new thread.
                debug("newSocket: create a new Thread");
                Thread t = new Thread(this,
                "Incoming Unicast Thread ");
                t.start();
            }
            try {
                notify();
            } catch (Exception e) {
            }
        }
    }
    
    
    
    void processReceivingSocket(Socket socket) {
        
        unicastThreads.newSocket(socket);
    }
    
    
    public class IncomingMulticastThreads implements Runnable {
        
        protected Vector buffers = null;
        protected TcpTransport tp = null;
        private   int waitingThreads = 0;
        private   int nbOfThreads = 0;
        
        public IncomingMulticastThreads(TcpTransport tp, int initialNumber) {
            
            this.tp = tp;
            buffers = new Vector();
            
            for (int i = 0; i < initialNumber; ++i) {
                ++nbOfThreads;
                Thread t = new Thread(this,
                "Incoming Multicast Thread " + i);
                t.start();
            }
        }
        
        public void run() {
            
            byte[] buffer = null;
            while (true) {
                buffer = waitForBuffer();
                if (buffer == null) {
                    // That means that this thread must die
                    decThreads();
                    return;
                }
                tp.processMulticast(buffer);
            }
        }
        
        synchronized private void decThreads() {
            --nbOfThreads;
        }
        
        synchronized protected byte[] waitForBuffer() {
            
            byte[] buffer = null;
            
            while (true) {
                
                if (buffers.size() > 0) {
                    try {
                        buffer = (byte[]) buffers.elementAt(0);
                    } catch (Exception e) {
                        // Strange error. XXX: to handle better
                        debug("waitForBuffer error: " + e);
                    }
                    if (buffer != null) {
                        try {
                            buffers.removeElementAt(0);
                        } catch (Exception e) {
                            debug("waitForBuffer error: " + e);
                        }
                        return buffer;
                    }
                    // We should not get here.
                    debug("waitForBuffer cannot get incoming buffer");
                }
                
                // No more socket to consume... Just wait.
                // Check if this thread must die or wait for a new socket
                if (mustDie()) {
                    debug("waitForBuffer: killing thread");
                    return null;
                }
                try {
                    ++waitingThreads;
                    wait();
                    --waitingThreads;
                } catch (Exception e) {
                }
            }
        }
        
        synchronized private boolean mustDie() {
            
            if ((nbOfThreads > MaxNbOfMulticastThreads) ||
            (waitingThreads > 0)) {
                return true;
            }
            return false;
        }
        
        synchronized protected void newBuffer(byte[] buffer) {
            
            buffers.addElement(buffer);
            // Check if we need to allocate a new Thread
            if (waitingThreads == 0) {
                ++nbOfThreads;
                // Allocate a new thread.
                debug("newBuffer: create a new Thread");
                Thread t = new Thread(this,
                "Incoming Multicast Thread ");
                t.start();
            }
            try {
                notify();
            } catch (Exception e) {
            }
        }
    }
    
    void processReceivingBuffer(byte[] buffer) {
        
        multicastThreads.newBuffer(buffer);
    }
    
}
