/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: UUIDFactory.java,v 1.2 2001/05/23 01:00:19 lomax Exp $
 */

package net.jxta.impl.id;

import java.util.Random;
//import java.security.SecureRandom;

/**
 * A <code>UUIDFactory</code> generates a UUID id. UUIDs are used to uniquely
 * identify objects such as peers, peer groups and pipes.
 *
 * @see         net.jxta.id.UUID
 * @see         net.jxta.id.ID
 * @see         net.jxta.id.IDFactory
 *
 * @version     $Revision: 1.2 $
 * @since       JXTA  1.0
 */

public class UUIDFactory {

    /**
     * We have to catch exceptions from construct of SecureRandom so we
     * have to init it inline.  
     */

    private static boolean inited = false;

    /**
     * random number generator for UUID generation.
     */

    private static Random randNum = null;

    /**
     * 	128-bit buffer for use with randNum
     */

    private static byte[] randBuf16 = new byte[16];

    /**
     * Returns a new UUID value. This member must be synchronized because it
     * makes use of shared internal state.
     *
     * @return UUID returns a version 4 IETF varient random UUID.
     *
     * @version     $Revision: 1.2 $
     * @since       JXTA  1.0
     */

    synchronized public static UUID newUUID() {
      if (!inited) {
	try {
	  //randNum = SecureRandom.getInstance( "SHA1PRNG" );
	  throw new Exception();
	  }
	catch( Exception everything ) {
	  randNum = new Random();
	  }
	inited = true;
	}

      randNum.nextBytes(randBuf16);
      randBuf16[6] &= 0x0f;
      randBuf16[6] |= 0x40; /* version 4 */
      randBuf16[8] &= 0x3f;
      randBuf16[8] |= 0x80; /* IETF variant */
      randBuf16[10] &= 0x3f;
      randBuf16[10] |= 0x80; /* multicast bit */

      long mostSig = 0;
      for (int i = 0; i < 8; i++) {
        mostSig = (mostSig << 8) | (randBuf16[i] & 0xff);
        }

      long leastSig = 0;
      for (int i = 8; i < 16; i++) {
        leastSig = (leastSig << 8) | (randBuf16[i] & 0xff);
        }

      return new UUID(mostSig, leastSig);
    }
}
