/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaLoader.java,v 1.4 2001/05/23 01:00:19 lomax Exp $
 */

package net.jxta.impl.loader;

import java.io.*;
import java.util.Hashtable;

/*
 * This class is a sample loader used to load class
 */

public class JxtaLoader extends ClassLoader {

  private String dir;
  private Hashtable Classes = new Hashtable();

  public JxtaLoader(String dir) {
    this.dir = dir;
  }
    // Make a stub for a version that uses URL, so that code that load
    // services can be written properly, even if it works only for classes
    // that do not need download.
  public Class loadClass(String name, String URL, boolean resolve)
     throws ClassNotFoundException
    {
	try {
	    return loadClass(name, resolve);
	} catch (ClassNotFoundException e) {
	    if (URL != null && ! URL.equals(""))
		System.err.println("Warning; download not supported yet");
	    throw new ClassNotFoundException(name);
	}
    }

  public Class loadClass(String name, boolean resolve)
     throws ClassNotFoundException
    {
      try {
	Class newClass = (Class)Classes.get(name);
	if (newClass == null) {
	  try {
	    newClass = findSystemClass(name);
	    if (newClass != null)
	      return newClass;
	  } catch (ClassNotFoundException e) {
	  }

	byte[] buf = bytesForClass(name);
	newClass = defineClass(name, buf, 0, buf.length);
	Classes.put(name, newClass);
	}
	if (resolve)
	  resolveClass(newClass);
	return newClass;
      } catch (IOException e) {
	throw new ClassNotFoundException(e.toString());

      }
    }

  /**
   * load a class file from the load class directory
   */

  protected byte[] bytesForClass(String name)
    throws IOException, ClassNotFoundException {

      FileInputStream in = new FileInputStream(dir + name.replace('.',
			       File.separatorChar) + ".java");
      int length = in.available();
      if (length == 0)
	throw new ClassNotFoundException(name);
      byte[] buf = new byte[length];
      in.read(buf);
      return buf;
  }
}
