/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ConfigDialog.java,v 1.17 2001/05/30 18:22:45 kuldeep Exp $
 */

package net.jxta.impl.peergroup;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import java.io.*;

import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.protocol.ServiceAdvertisement;
import net.jxta.impl.protocol.HTTPAdv;
import net.jxta.impl.protocol.TCPAdv;
import net.jxta.impl.protocol.EndpointAdv;


// A few widgets.

class PanelGBL extends Panel {
    protected Insets insets = new Insets(0, 0, 0, 0);
    GridBagLayout lay = new GridBagLayout();

    private static GridBagConstraints constrLabel = new GridBagConstraints();
    static {
        constrLabel.gridwidth = GridBagConstraints.REMAINDER;
        constrLabel.gridheight = 1;
        constrLabel.gridx = 0;
        constrLabel.gridy = GridBagConstraints.RELATIVE;
        constrLabel.weightx = 1;
        constrLabel.weighty = 1;
        constrLabel.anchor = GridBagConstraints.NORTHWEST;
        constrLabel.fill = GridBagConstraints.HORIZONTAL;
    }

    public PanelGBL(String label) {
        super();
        setLayout(lay);
        add(new Label(label, Label.LEFT), constrLabel);
    }
    public PanelGBL() {
        super();
        setLayout(lay);
    }
    public Insets getInsets() {
        return insets;
    }
}


class BorderPanelGBL extends PanelGBL {

    public static final int NONE = 0;
    public static final int RAISED = 1;
    public static final int LOWERED = 2;
    public static final int GROOVE = 3;
    public static final int BUMP = 4;

    int style = GROOVE;
    String title;
    int ascent = 0;
    int descent = 0;
    int leading = 0;
    int titleWidth = 0;
    GridBagLayout lay = new GridBagLayout();

    public BorderPanelGBL(String title) {
        super();
        this.title = title;
    }
    public BorderPanelGBL(String title, String advice) {
        super(advice);
        this.title = title;
    }
    public BorderPanelGBL(String title, String advice, int s) {
        super(advice);
        this.title = title;
        if ((s < NONE) && (s > BUMP)) return;
        if ((s == RAISED) || (s == LOWERED)) this.title = null;
        style = s;
    }
    private void checkMetrics() {
        Font font = getFont();
        if ((title == null) || (font == null)) {
            ascent = 2;
        } else {
            FontMetrics fmetrics = getFontMetrics(font);
            ascent = fmetrics.getAscent();
            descent = fmetrics.getDescent();
            leading = fmetrics.getLeading();
            titleWidth = fmetrics.stringWidth(title);
        }
        insets = new Insets(descent + ascent + leading + 2, 7, 7, 7);
    }
    public Insets getInsets() {
        checkMetrics();
        return insets;
    }
    private void paintLowered(Graphics g) {
        checkMetrics();
        if (ascent == 0) return;

        Dimension d = getSize();

        g.setColor(Color.black);
        g.drawRect(1, ascent - 2, d.width - 4, d.height - ascent);
        g.setColor(Color.white);
        g.drawRect(2, ascent - 1, d.width - 4, d.height - ascent);
        g.setColor(getBackground());
        g.drawRect(2, ascent - 1, d.width - 5, d.height - ascent - 1);
    }

    private void paintRaised(Graphics g) {
        checkMetrics();
        if (ascent == 0) return;

        Dimension d = getSize();

        g.setColor(Color.white);
        g.drawRect(1, ascent - 2, d.width - 4, d.height - ascent);
        g.setColor(Color.black);
        g.drawRect(2, ascent - 1, d.width - 4, d.height - ascent);
        g.setColor(getBackground());
        g.drawRect(2, ascent - 1, d.width - 5, d.height - ascent - 1);
    }

    private void paintGroove(Graphics g) {
        checkMetrics();
        if (ascent == 0) return;

        Dimension d = getSize();
        g.setColor(Color.black);
        g.drawRect(1, ascent - 2, d.width - 4, d.height - ascent);
        g.setColor(Color.white);
        g.drawRect(2, ascent - 1, d.width - 4, d.height - ascent);
        
        g.setColor(Color.black);
        g.clearRect(10, 0, titleWidth+6, descent+ascent+leading + 1);
        g.drawString(title, 12, ascent + 1);
        g.setColor(Color.white);
        g.drawString(title, 13, ascent + 2);
        
        // Work around a bug of at least the awt implem I'm using.
        // A few wild pixels appear on that line during drawstring.
        g.clearRect(0,0,d.width,1);
    }

    private void paintBump(Graphics g) {
        checkMetrics();
        if (ascent == 0) return;

        Dimension d = getSize();
        g.setColor(Color.white);
        g.drawRect(1, ascent - 2, d.width - 4, d.height - ascent);
        g.setColor(Color.black);
        g.drawRect(2, ascent - 1, d.width - 4, d.height - ascent);
        
        g.setColor(Color.white);
        g.clearRect(10, 0, titleWidth+6, descent+ascent+leading + 1);
        g.drawString(title, 12, ascent + 1);
        g.setColor(Color.black);
        g.drawString(title, 13, ascent + 2);
        
        // Work around a bug of at least the awt implem I'm using.
        // A few wild pixels appear on that line during drawstring.
        g.clearRect(0,0,d.width,1);
    }

    public void paint(Graphics g) {
        switch (style) {
        case GROOVE:
            paintGroove(g);
            break;
        case BUMP:
            paintBump(g);
            break;
        case RAISED:
            paintRaised(g);
            break;
        case LOWERED:
            paintLowered(g);
        default:
        }
        super.paint(g);
    }
}

class PagesPanel extends Panel implements ActionListener {
    Panel buttons;
    Panel pages;
    CardLayout l;

    public void actionPerformed(ActionEvent e) {
        l.show(pages, e.getActionCommand());
    }

    public PagesPanel() {
        super(new BorderLayout());
        l = new CardLayout();
        pages = new Panel(l);
        buttons = new Panel(new FlowLayout(FlowLayout.LEFT, 0, 0));
        add(pages, BorderLayout.CENTER);
        add(buttons, BorderLayout.NORTH);
    }
    public PanelGBL addPage(String buttonName, String comment) {
        BorderPanelGBL p = new BorderPanelGBL(buttonName, comment,
                                              BorderPanelGBL.RAISED);
        pages.add(p, buttonName);
        Button b = new Button(buttonName);
        buttons.add(b);
        b.addActionListener(this);
        return p;
    }
}


public class ConfigDialog
    extends Frame
    implements ItemListener, MouseListener
{
    Frame mainFrame = this;

    PeerGroupAdvertisement peerGroupAdv;
    EndpointAdv httpEp;
    EndpointAdv tcpEp;
    HTTPAdv httpAdv;
    TCPAdv tcpAdv;
    ServiceAdvertisement resolverAdv = null;

    GridBagConstraints stdConstr;
    GridBagConstraints centerConstr;
    GridBagConstraints centerLastConstr;
    GridBagConstraints fillConstr;
    GridBagConstraints leftLineConstr;
    GridBagConstraints midLineConstr;
    GridBagConstraints rightLineConstr;
    GridBagConstraints leftPanelConstr;
    GridBagConstraints rightPanelConstr;


    Label   helpLabel;
    IdPanel idPanel;
    Checkbox debug;
    HttpPanel httpPanel;
    TcpPanel tcpPanel;
    RdvPanel rdvPanel;
    RouterPanel routerPanel;

    Button ok;
    Button cancel;


    class HostPortPanel extends PanelGBL implements ItemListener {


        public Checkbox  useMe;
        public TextField host;
        public TextField port;

        Label addressLabel;
        ItemListener listener;

        public void setState(boolean state) {
            useMe.setState(state); // sometimes redundant but not always.
            host.setEnabled(state);
            port.setEnabled(state);
            addressLabel.setEnabled(state);
        }

        public boolean getState() {
            return useMe.getState();
        }

        public void itemStateChanged(ItemEvent e) {
            setState(useMe.getState());
            listener.itemStateChanged(e);
        }

        public void setEnabled(boolean enabled) {
            super.setEnabled(enabled);
        }

        HostPortPanel(ItemListener l,
                      String checkLabel,
                      String addrLabel,
                      String defaultHost, String defaultPort,
                      boolean defaultState) {

            super();

            listener = l;
            useMe = new Checkbox(checkLabel, null, defaultState);
            host = new TextField(defaultHost, 20);
            port = new TextField(defaultPort, 8);
            addressLabel = new Label(addrLabel);

            add(useMe, stdConstr);
            add(addressLabel, leftLineConstr);
            add(host, midLineConstr);
            add(port, rightLineConstr);

            setState(defaultState);
            useMe.addItemListener(this);
        }
    }

    class AutoLoader extends Dialog implements ActionListener, MouseListener {

        HostListPanel list;
        Button load;
        Button quit;
        TextField urlField;
        Label helpLabel;

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseClicked(MouseEvent e) {
            helpLabel.setForeground(Color.black);
            helpLabel.setText("Edit the url below and click \"load\".");
        }

        private void loadList() {
            BufferedReader l = null;
            try {
                URL url = new URL(urlField.getText());
                l =
                  new BufferedReader(new InputStreamReader(url.openStream()));

                String s;
                while ((s = l.readLine()) != null) {
                    list.addItem(s);
                }
            } catch (MalformedURLException e) {
                helpLabel.setForeground(Color.red.darker());
                helpLabel.setText("This URL is malformed.");
            } catch (IOException e) {
                helpLabel.setForeground(Color.red.darker());
                helpLabel.setText("This URL is not accessible.");
            }
        }

        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == quit) {
                dispose();
                return;
            }
            loadList();
        }

        public AutoLoader(Frame owner, HostListPanel lst, String defaultUrl) {
            super(owner, "Load list from URL", true);
            list = lst;
            urlField = new TextField(defaultUrl, 50);
            helpLabel = new Label("Edit the url below and click \"load\".");
            helpLabel.setBackground(new Color(220, 220, 220));
            helpLabel.setForeground(Color.black);
            load = new Button(" Load ");
            quit = new Button(" Quit ");
            Panel okPanel = new Panel();
            okPanel.add(load);
            okPanel.add(quit);
            add(helpLabel, BorderLayout.NORTH);
            add(urlField, BorderLayout.CENTER);
            add(okPanel, BorderLayout.SOUTH);

            helpLabel.addMouseListener(this);
            load.addActionListener(this);
            quit.addActionListener(this);

            pack();
            show();
        }
    }


    class HostListPanel extends PanelGBL
        implements ActionListener, ItemListener {

        public Checkbox useMe;
        public TextField host;
        public TextField port;
        public Button insert;
        public Button remove;
        public Button auto;

        String defaultUrl = "";
        java.awt.List list;
        Label listLabel;

        ItemListener listener;

        public void setState(boolean state) {
            if (useMe != null) useMe.setState(state);
            list.setEnabled(state);
            host.setEnabled(state);
            port.setEnabled(state);
            insert.setEnabled(state);
            remove.setEnabled(state);
            listLabel.setEnabled(state);
        }

        public boolean getState() {
            return useMe.getState();
        }

        public void itemStateChanged(ItemEvent e) {
            if (e.getSource() == useMe) {
                if (useMe != null) setState(useMe.getState());
                listener.itemStateChanged(e);
            }
        }

        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == insert) {
                if (addItem(host.getText() + ":" + port.getText())) {
                    host.setText("");
                    port.setText("");
                }
            } else if (e.getSource() == remove) {
                int[] sel = list.getSelectedIndexes();
                int i = sel.length;
                while (i-- > 0) {
                    list.remove(sel[i]);
                }
            } else {
                AutoLoader a = new AutoLoader(mainFrame, this, defaultUrl);
            }
        }


        public HostListPanel(String lstLabel, String defUrl) {
            this(null, null, lstLabel, defUrl, true);
        }

        public HostListPanel(ItemListener l, String checkLabel,
                             String lstLabel,
                             String defUrl,
                             boolean defaultState) {

            super();
            defaultUrl = defUrl;
            listener = l;
            host = new TextField("", 16);
            port = new TextField("", 4);
            insert = new Button(" + ");
            remove = new Button(" - ");
            auto = new Button("load");

            list = new java.awt.List(3, true);
            listLabel = new Label(lstLabel);

            PanelGBL p1 = new PanelGBL();
            PanelGBL p2 = new PanelGBL();

            GridBagConstraints c1 =
                (GridBagConstraints) leftLineConstr.clone();
            c1.insets = new Insets(0,0,0,0);

            p1.add(host, c1);
            p1.add(port, rightLineConstr);
            p1.add(list, fillConstr);
            p2.add(insert, fillConstr);
            p2.add(remove, fillConstr);
            p2.add(auto, fillConstr);

            GridBagConstraints c2 =
                (GridBagConstraints) rightLineConstr.clone();
            c2.anchor = GridBagConstraints.NORTHWEST;
            c2.insets = new Insets(4,2,2,8);

            if ((listener != null) && (checkLabel != null)) {
                useMe = new Checkbox(checkLabel, null, defaultState);
                add(useMe, stdConstr);
                useMe.addItemListener(this);
            }

            add(listLabel, stdConstr);
            add(p1, c1);
            add(p2, c2);

            setState(defaultState);
            insert.addActionListener(this);
            remove.addActionListener(this);
            auto.addActionListener(this);
        }

        public boolean addItem(String item) {
            int i = item.indexOf(':');
            if (i == -1) return false;
            try {
                String ips = item.substring(0, i);
                String ports = item.substring(i + 1);
                int port = Integer.parseInt(ports);
                if (port < 0 || port > 65535) return false;
                list.add(ips.trim() + ":" + port);
            } catch (Exception e) {
                return false;
            }
            return true;
        }

        public String[] getItems() {
            return list.getItems();
        }
    }

    class HttpPanel extends BorderPanelGBL implements ItemListener {

        public Checkbox useMe;
        public HostPortPanel proxyPanel;

        ItemListener listener;

        public void setState(boolean state) {
            useMe.setState(state);
            proxyPanel.setEnabled(state);
        }

        public boolean getState() {
            return useMe.getState();
        }

        public void itemStateChanged(ItemEvent e) {
            setState(useMe.getState());
            listener.itemStateChanged(e);
        }

        public HttpPanel(ItemListener listener,
                  String defaultHttpPort,
                  String defaultProxyName,
                  String defaultProxyPort,
                  boolean defaultState,
                  boolean proxyOn) {

            super("HTTP settings",
                  "Enable if behind a firewall/NAT and no local RendezVous.");

            this.listener = listener;

            useMe = new Checkbox("Enabled", null, defaultState);
            proxyPanel =
                new HostPortPanel(this,
                                  "Use a proxy server (Often needed if behind "
                                  + "firewall)",
                                  "Proxy address",
                                  defaultProxyName,
                                  defaultProxyPort,
                                  proxyOn);

            add(useMe, stdConstr);
            add(proxyPanel, fillConstr);
            setState(defaultState);

            useMe.addItemListener(this);
        }
    }

    class TcpPanel extends BorderPanelGBL implements ItemListener {

        public Checkbox useMe;

        public TextField port;
        public Choice ips;

        ItemListener listener;

        public void setState(boolean state) {
            useMe.setState(state);
            port.setEnabled(state);
            ips.setEnabled(state);
        }

        public boolean getState() {
            return useMe.getState();
        }

        public void itemStateChanged(ItemEvent e) {
            setState(useMe.getState());
            listener.itemStateChanged(e);
        }

        public TcpPanel(ItemListener listener,
                 String defaultTcpPort, boolean defaultState) {

            super("TCP settings", "Enable if direct LAN connection.");

            this.listener = listener;

            ips = new Choice();
            try {
                String hostName = InetAddress.getLocalHost().getHostName();
                InetAddress[] all = InetAddress.getAllByName(hostName);
                int i = all.length;
                while (i-->0) ips.addItem(all[i].getHostAddress()); 
            } catch (Exception e) {
                defaultState = false;
            }

            useMe = new Checkbox("Enabled", null, defaultState);
            port = new TextField(defaultTcpPort, 8);

            add(useMe, leftLineConstr);
            add(ips, midLineConstr);
            add(port, rightLineConstr);

            setState(defaultState);
            useMe.addItemListener(this);
        }
    }

    class IdPanel extends PanelGBL {

        public TextField peerName;
        public TextField hostName;

        IdPanel(String defaultHost, String defaultPeer) {

            super();

            hostName = new TextField(defaultHost, 20);
            peerName = new TextField(defaultPeer, 20);

            add(new Label("Peer Name"), leftLineConstr);
            add(peerName, midLineConstr);
            add(new Label("(Mandatory)"), rightLineConstr);
            add(new Label("Host Name"), leftLineConstr);
            add(hostName, midLineConstr);
            add(new Label("(Mandatory)"), rightLineConstr);
        }
    }

    class RdvPanel extends BorderPanelGBL {
        public Checkbox isRdv;
        public HostListPanel http;
        public HostListPanel tcp;

        RdvPanel(String title, boolean isRendezVous) {
            super(title);
            isRdv = new Checkbox("Act as a Rendezvous", null,
                                 isRendezVous);
            http =
                new HostListPanel("Available HTTP rendez-vous",
                             "http://platform.jxta.org/java/httpRdvsProd.txt");

            tcp = new HostListPanel("Available TCP rendez-vous", "");

            add(isRdv, stdConstr);
            add(tcp, leftPanelConstr);
            add(http, rightPanelConstr);
        }
    }

    class RouterPanel extends BorderPanelGBL implements ItemListener {

        public HostPortPanel local;
        public HostListPanel remote;
        ItemListener listener;

        
        public void itemStateChanged(ItemEvent e) {

            // Cannot be a router and need one in the same time.
            // We handle that one more or less as a checkbox group. If you try
            // to turn one on, the other goes off. The difference is: both
            // may be off.

            if (   routerPanel.local.getState()
                   && routerPanel.remote.getState()) {
                // Figure out what the user is clicking to decide what to
                // to turn off.
                if (e.getSource() == routerPanel.remote.useMe) {
                    routerPanel.local.setState(false);
                } else {
                    routerPanel.remote.setState(false);
                }
            }

            listener.itemStateChanged(e);
        }

        public RouterPanel(ItemListener l,
                           String title,
                           String defaultName, String defaultPort,
                           boolean serverDefault, boolean routerDefault) {

            super(title);
            listener = l;
            local = new HostPortPanel(this,
                                      "Act as a router (Both TCP and HTTP "
                                      + "must be enabled.)",
                                      "Public address",
                                      defaultName,
                                      defaultPort,
                                      serverDefault);
            remote =
                new HostListPanel(this,
                               "Use a router "
                               + "(Required if behind firewall/NAT)",
                               "Available routers",
                               "http://platform.jxta.org/java/routersProd.txt",
                               routerDefault);
            add(local, stdConstr);
            add(remote, stdConstr);
        }
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
        helpLabel.setForeground(Color.black);
        helpLabel.setText("See \"http://shell.jxta.org/index.html\" for config help");
    }


    public void itemStateChanged(ItemEvent e) {

        boolean routerPossible =
            (   tcpPanel.getState()
             && httpPanel.getState());

        // A remore router makes sense only if http is on.
        routerPanel.remote.setEnabled(httpPanel.getState());

        // Rendez-vous make sense only if their respective transports are on.
        rdvPanel.http.setEnabled(httpPanel.getState());
        rdvPanel.tcp.setEnabled(tcpPanel.getState());

        // Can be a router only if both transports are on.
        routerPanel.local.setEnabled(routerPossible);

        // A proxy does not make sense if there's no router.
        if (httpPanel.getState()) {
            httpPanel.proxyPanel.setEnabled(routerPanel.remote.getState());
        }
    }


    public ConfigDialog (PeerGroupAdvertisement pgadv){

        super("JXTA HTTP Configurator");

        stdConstr = new GridBagConstraints();
        stdConstr.gridwidth = GridBagConstraints.REMAINDER;
        stdConstr.gridheight = 1;
        stdConstr.gridx = 0;
        stdConstr.gridy = GridBagConstraints.RELATIVE;
        stdConstr.fill = GridBagConstraints.NONE;
        stdConstr.weightx = 1;
        stdConstr.anchor = GridBagConstraints.NORTHWEST;
        stdConstr.insets = new Insets(0,0,0,0);

        fillConstr = (GridBagConstraints) stdConstr.clone();
        fillConstr.fill = GridBagConstraints.HORIZONTAL;

        centerConstr = (GridBagConstraints) stdConstr.clone();
        centerConstr.anchor = GridBagConstraints.NORTH;

        centerLastConstr = (GridBagConstraints) centerConstr.clone();
        centerLastConstr.weighty = 1;

        midLineConstr = new GridBagConstraints();
        midLineConstr.gridwidth = 1;
        midLineConstr.gridheight = 1;
        midLineConstr.fill = GridBagConstraints.NONE;
        midLineConstr.anchor = GridBagConstraints.EAST;
        midLineConstr.weightx = 0;

        rightLineConstr = (GridBagConstraints) midLineConstr.clone();
        rightLineConstr.gridwidth = GridBagConstraints.REMAINDER;
        rightLineConstr.weightx = 1;
        rightLineConstr.anchor = GridBagConstraints.WEST;

        leftLineConstr = (GridBagConstraints) midLineConstr.clone();
        leftLineConstr.gridx = 0;
        leftLineConstr.weightx = 0;
        leftLineConstr.anchor = GridBagConstraints.WEST;
        leftLineConstr.insets = new Insets(0,20, 0, 20);

        GridBagConstraints fillInsetConstr =
            (GridBagConstraints)fillConstr.clone();
        fillInsetConstr.insets = new Insets(5,5,5,5);

        leftPanelConstr = (GridBagConstraints)fillConstr.clone();
        leftPanelConstr.gridwidth = 1;

        rightPanelConstr = (GridBagConstraints)fillConstr.clone();
        rightPanelConstr.gridwidth = GridBagConstraints.REMAINDER;;
        rightPanelConstr.gridx = GridBagConstraints.RELATIVE;


        peerGroupAdv = pgadv;
        Hashtable endpoints = pgadv.getEndpointAdvertisements();
        httpEp = (EndpointAdv) (endpoints.get("HTTP"));
        tcpEp = (EndpointAdv) (endpoints.get("TCP"));

        Vector rdvs = null;
        try {
            Hashtable servs = pgadv.getServiceAdvertisements();
            resolverAdv =
                (ServiceAdvertisement) servs.get("jxta.service.resolver");
            rdvs = resolverAdv.getParams();
        } catch (Exception e) {
        }

        tcpAdv = (TCPAdv) tcpEp.getTransportAdvertisement();
        httpAdv = (HTTPAdv) httpEp.getTransportAdvertisement();

        // Read-in the adv as it is now.
        boolean serverDefault = httpAdv.getServerEnabled();
        boolean proxyDefault = httpAdv.getProxyEnabled();
        boolean routerDefault = httpAdv.getRouterEnabled();

        String defaultServerName = httpAdv.getServer();
        String defaultProxyName = httpAdv.getProxy();
        Vector defaultRouters = httpAdv.getRouters();

        int index;
	String defaultServerPort = "7900";
	String defaultProxyPort = "8080";
	if ( defaultServerName != null ) {
	  index = defaultServerName.indexOf(":");
	  defaultServerPort = defaultServerName.substring(index + 1);
	  defaultServerName = defaultServerName.substring(0, index);
	}

	if ( defaultProxyName != null ) {
	  index = defaultProxyName.indexOf(":");
	  defaultProxyPort = defaultProxyName.substring(index + 1);
	  defaultProxyName = defaultProxyName.substring(0, index);
	}

        boolean tcpDefault = tcpEp.getEnabled();
        boolean httpDefault = httpEp.getEnabled();

        String pn = pgadv.getPeerName();
        if (pn == null) pn = "";

        helpLabel = new Label("See \"http://shell.jxta.org/index.html\" for config help", Label.CENTER);
        helpLabel.setBackground(new Color(220, 220, 220));
        helpLabel.setForeground(Color.black);

        idPanel = new IdPanel(defaultServerName, pn);
        debug = new Checkbox("Debug mode", null, pgadv.getDebug());
        
        tcpPanel = new TcpPanel(this, tcpAdv.getPort(), tcpDefault);
        httpPanel = new HttpPanel(this, httpAdv.getPort(),
                                  defaultProxyName,
                                  defaultProxyPort,
                                  httpDefault,
                                  proxyDefault);

        routerPanel = new RouterPanel(this, "Routing",
                                      defaultServerName,
                                      defaultServerPort,
                                      serverDefault,
                                      routerDefault);


        if (defaultRouters != null) {

            Enumeration e = defaultRouters.elements();
            while (e.hasMoreElements()) {
                routerPanel.remote.addItem((String) e.nextElement());
            }
        }

        rdvPanel = new RdvPanel("Rendezvous Settings",
                                pgadv.getIsRendezvous());

        if (rdvs != null) {
            Enumeration e = rdvs.elements();
            while (e.hasMoreElements()) {
                String s = (String) e.nextElement();
                if (s.startsWith("tcp://")) {
                    rdvPanel.tcp.addItem(s.substring(6));
                    continue;
                }
                if (s.startsWith("http://")) {
                    rdvPanel.http.addItem(s.substring(7));
                }
            }
        }

        ok = new Button("  OK  ");
        cancel = new Button("Cancel");

        routerPanel.remote.setEnabled(httpPanel.getState());

        routerPanel.local.setEnabled(   tcpPanel.getState()
                              && httpPanel.getState());


        rdvPanel.http.setEnabled(httpPanel.getState());
        rdvPanel.tcp.setEnabled(tcpPanel.getState());

        Panel okPanel = new Panel();
        okPanel.add(ok);
        okPanel.add(cancel);

        PagesPanel pages = new PagesPanel();
        PanelGBL basicsPanel = pages.addPage("basics",
                                             "Basic settings");

        PanelGBL advancedPanel = pages.addPage("advanced",
                                               "Experienced Users Only");

        advancedPanel.add(rdvPanel, fillInsetConstr);
        advancedPanel.add(routerPanel, fillInsetConstr);

        GridBagConstraints centerWConstr =
            (GridBagConstraints) centerConstr.clone();
        centerWConstr.weighty = 1;

        basicsPanel.add(idPanel, centerWConstr);
        
        basicsPanel.add(debug, centerConstr);
        basicsPanel.add(tcpPanel, fillInsetConstr);
        basicsPanel.add(httpPanel, fillInsetConstr);
        

        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        add(helpLabel, fillConstr);
        add(pages, fillInsetConstr);
        add(okPanel, centerLastConstr);

        ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    saveValues();
                }
            });

        cancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    System.exit(0);
                }
            });

        helpLabel.addMouseListener(this);

        pack();
        show();

        untilDone();

        dispose();
    }

    boolean done = false;

    synchronized boolean untilDone() {
        try {
            while (! done) wait();
        } catch (Exception e) {
        }
        return (done);
    }

    synchronized void beDone() {
        done = true;
        notify();
    }

    private boolean verifyInput () {

        if (idPanel.hostName.getText().equals("")
            ||idPanel.peerName.getText().equals("")) {

            helpLabel.setForeground(Color.red.darker());
            helpLabel.setText("Host name and peer name are required.");
            return false;
        }

        if ((httpPanel.getState() == false)
            && (tcpPanel.getState() == false)) {

            helpLabel.setForeground(Color.red.darker());
            helpLabel.setText("At least one of TCP or HTTP must be enabled.");
            return false;
        }

        int p1, p2, p3, p4;
        try {
            p1 = Integer.parseInt(httpPanel.proxyPanel.port.getText());
            //            p2 = Integer.parseInt(remoteRouterPanel.port.getText());
            p3 = Integer.parseInt(routerPanel.local.port.getText());
            p4 = Integer.parseInt(tcpPanel.port.getText());
        } catch (Exception e) {
            helpLabel.setForeground(Color.red.darker());
            helpLabel.setText("Port number must be integers between 0 and "
                              + "65535");
            return false;
        }
        if (p1 > 65535 || /*p2 > 65535 ||*/ p3 > 65535 || p4 > 65535
            || p1 < 0 || /*p2 < 0 ||*/ p3 < 0 || p4 < 0) {
            helpLabel.setForeground(Color.red.darker());
            helpLabel.setText("Port number must be integers between 0 and "
                              + "65535");
            return false;
        }

        if (httpPanel.getState() && routerPanel.remote.getState()) {
            String[] routers = routerPanel.remote.getItems();
            String[] httpRdvs = rdvPanel.http.getItems();

            if (routers.length == 0) {
                helpLabel.setForeground(Color.red.darker());
                helpLabel.setText("Use of external routers is ON, but the list"
                                  + " of routers is empty.");
                return false;
            }
            if (httpRdvs.length == 0) {
                helpLabel.setForeground(Color.red.darker());
                helpLabel.setText("Use of external routers is ON, but there"
                                  + " is no rendezvous.");
                return false;
            }
            int i = httpRdvs.length;
            while (i-->0) {
                int j = routers.length;
                while (j-->0) {
                    if (httpRdvs[i].equals(routers[j])) return true;
                }
            }
            helpLabel.setForeground(Color.red.darker());
            helpLabel.setText("The http routers and rendez-vous lists must "
                              + "have an intersection.");

            return false;
        }
        return true;
    }

    private void saveValues () {

        if (! verifyInput())return;
        
        try {

            httpEp.setEnabled(httpPanel.getState());

            // FIXME: this statement is for backwards compat only.
            httpAdv.setPort(routerPanel.local.port.getText());

            if (httpPanel.proxyPanel.host.getText().equals("")) {
                httpAdv.setProxy(null);
            } else {
                httpAdv.setProxyEnabled(httpPanel.proxyPanel.getState());
                httpAdv.setProxy(httpPanel.proxyPanel.host.getText()
                                 + ":"
                                 + httpPanel.proxyPanel.port.getText());
            }
            String[] routers = routerPanel.remote.getItems();

            if (routers.length == 0) {
                httpAdv.setRouters(null);
            } else {
                httpAdv.setRouterEnabled(routerPanel.remote.getState());
                Vector routersVect = new Vector();
                int i = routers.length;
                while (i-->0) routersVect.addElement(routers[i]);
                httpAdv.setRouters(routersVect);
            }


            tcpEp.setEnabled(tcpPanel.getState());
            tcpAdv.setPort(tcpPanel.port.getText());
            tcpAdv.setPublicAddress(tcpPanel.ips.getSelectedItem());

            // If you turn off either tcp or http we have to loose your
            // otherwise preferrence for being a router, because httpTransport
            // does not know weither tcp is on or off, so we have to tell
            // it explicitly what to do. Alternatively, we could create an
            // extra flag to remember your preferrence...later.

            // if there's nothing interresting inthere, do not save it.
            if (routerPanel.local.host.getText().equals("")) {
                httpAdv.setServer(null);
            } else {
                httpAdv.setServerEnabled(httpPanel.getState()
                                         && tcpPanel.getState()
                                         && routerPanel.local.getState());

                httpAdv.setServer(routerPanel.local.host.getText()
                                  + ":"
                                  + routerPanel.local.port.getText());
            }
            httpAdv.setPropagateEnabled(false); // obsolete

            peerGroupAdv.setIsRendezvous(rdvPanel.isRdv.getState());
            peerGroupAdv.setPeerName(idPanel.peerName.getText());
            peerGroupAdv.setDebug(debug.getState());

            if (resolverAdv != null) {
                // We do not want to dump any address that the user has left
                // in the list, so we just re-order them: If http is on,
                // then it comes first, otherwise tcp addresses come first.
                int i;
                String[] rdvs;
                Vector params = new Vector();
                if (httpPanel.getState()) {
                    rdvs = rdvPanel.http.getItems();
                    i = rdvs.length;
                    while (i-- > 0) {
                        params.addElement("http://" + rdvs[i]);
                    }
                }
                rdvs = rdvPanel.tcp.getItems();
                i = rdvs.length;
                while (i-- > 0) {
                    params.addElement("tcp://" + rdvs[i]);
                }
                if (! httpPanel.getState()) {
                    rdvs = rdvPanel.http.getItems();
                    i = rdvs.length;
                    while (i-- > 0) {
                        params.addElement("http://" + rdvs[i]);
                    }
                }
                resolverAdv.setParams(params);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        beDone();
    }
}



