/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Configurator.java,v 1.12 2001/06/14 07:25:50 bondolo Exp $
 */

package net.jxta.impl.peergroup;

import java.net.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.impl.protocol.ServiceAdv;
import net.jxta.impl.protocol.HTTPAdv;
import net.jxta.impl.protocol.TCPAdv;
import net.jxta.impl.protocol.EndpointAdv;

import net.jxta.peer.*;
import net.jxta.peergroup.*;
import net.jxta.resolver.*;
import net.jxta.exception.*;
import net.jxta.credential.*;
import net.jxta.document.*;

public class Configurator {

    static String advName = "PlatformPeerGroup";
    static String configName = "PlatformConfig";

    PeerGroupAdvertisement advertisement = null;

    private void loadAdvertisement(String fileName) {
	FileInputStream advStream = null;
        File file = null;
	try {
            file = new File(fileName);
	    advStream = new FileInputStream(file);

	    advertisement = (PeerGroupAdvertisement)
	        AdvertisementFactory.newAdvertisement (
                             new MimeMediaType( "text/xml" ), advStream );
	    System.out.println("Recovered " + fileName);
	} catch (FileNotFoundException e) {
	    System.out.println("No Existing " + fileName);
	} catch (Exception e) {
	    System.out.println("Failed to Recover " + fileName);
	    e.printStackTrace();
            try {
                // delete that bad file.
                file.delete();
            } catch (Exception  ex1) {
                System.err.println("Could not remove "
                                   + fileName
                                   + ". Removed it by hand before retrying");
                System.exit(1);
            }

	    advertisement = null;
	}
	try {
	    if (advStream != null) advStream.close();
	} catch(Exception e) {
	}
    }

    private void saveAdvertisement(String fileName) {

        // Save the adv as input for future reconfiguration
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(fileName);
            Document aDoc =
                advertisement.getDocument( new MimeMediaType( "text/xml" ) );
            aDoc.sendToStream( out );
        } catch (Exception e) {
            System.out.println(e.toString());
            e.printStackTrace();
            System.err.println("Warning: could not save to " + fileName);
        }
        try {
            out.close();
        } catch (Exception e) {
        }
    }

    //
    // Create a new configuration, or heal an existing one.
    //
    private void fixAdvertisement() {
        if (advertisement == null) {

            try {
                advertisement = (PeerGroupAdvertisement)
                    AdvertisementFactory.newAdvertisement(
                            PeerGroupAdvertisement.getAdvertisementType() );

            } catch ( InvocationTargetException failed ) {
                System.err.println(
                                   "Cannot create a new Platform "
                                   + "Advertisement. Cause : "
                                   + failed.getTargetException().toString() );
                System.exit(1);
            }

	    advertisement.setDebug(false);
            //    advertisement.setVersion("JXTA1.0");
            advertisement.setName (advName);
            advertisement.setIsRendezvous (false);
            advertisement.setKeywords("Platform PeerGroup");
	    advertisement.setPid(
               new PeerID(PeerGroupID.worldPeerGroupID).toString());
            advertisement.setGid(PeerGroupID.worldPeerGroupID.toString());
        }

	/*
	 * create the default Service advertisement
	 */

	Hashtable services = advertisement.getServiceAdvertisements();
        if (services == null) {
            services = new Hashtable();
            advertisement.setServiceAdvertisements(services);
        }

        ServiceAdv resolv = (ServiceAdv) services.get("jxta.service.resolver");
        ServiceAdv disco = (ServiceAdv) services.get("jxta.service.discovery");
        ServiceAdv pipe = (ServiceAdv) services.get("jxta.service.pipe");
        ServiceAdv pip = (ServiceAdv) services.get("jxta.service.peerinfo");
        ServiceAdv rdv = (ServiceAdv) services.get("jxta.service.rendezvous");
        ServiceAdv member =
            (ServiceAdv) services.get("jxta.service.membership");

        if (resolv == null) {
            Vector resolverParams = new Vector();
            resolverParams.addElement("http://209.25.154.239:6002");
            resolverParams.addElement("http://63.115.99.228:6002");
           resolverParams.addElement("http://jxta.clip2.com:6002");


            resolv = new ServiceAdv("jxta.service.resolver",
                                    "JXTA1.0",
                                    "resolver",
                                    null,
                                    resolverParams,
                                    "http:/www.jxta.org/download/jxta.jar",
                                    "sun.com",
                                    "TBD",
                                    "net.jxta.impl.resolver.ResolverService");

            services.put("jxta.service.resolver", resolv);
        }


        if (disco == null) {
            disco = new ServiceAdv("jxta.service.discovery",
                                   "JXTA1.0",
                                   "discovery",
                                   null,
                                   new Vector(),
                                   "http:/www.jxta.org/download/jxta.jar",
                                   "sun.com",
                                   "TBD",
                                   "net.jxta.impl.discovery.DiscoveryService");

            services.put("jxta.service.discovery", disco);
        }


        if (pipe == null) {
            pipe = new ServiceAdv("jxta.service.pipe",
                                  "JXTA1.0",
                                  "pipe",
                                  null,
                                  new Vector(),
                                  "http:/www.jxta.org/download/jxta.jar",
                                  "sun.com",
                                  "TBD",
                                  "net.jxta.impl.pipe.PipeService");
            
            services.put("jxta.service.pipe", pipe);
        }

        if (pip == null) {
            pip = new ServiceAdv("jxta.service.peerinfo",
                                 "JXTA1.0",
                                 "peerinfo",
                                 null,
                                 new Vector(),
                                 "http:/www.jxta.org/download/jxta.jar",
                                 "sun.com",
                                 "TBD",
                                 "net.jxta.impl.peer.PeerInfoService");
             
            services.put("jxta.service.peerinfo", pip);
        }

        if (rdv == null) {
            rdv = new ServiceAdv("jxta.service.rendezvous",
                                 "JXTA1.0",
                                 "rendezvous",
                                 null,
                                 new Vector(),
                                 "http:/www.jxta.org/download/jxta.jar",
                                 "sun.com",
                                 "TBD",
                                 "net.jxta.impl.rendezvous.RendezVousImpl");

            services.put("jxta.service.rendezvous", rdv);
        }

        if (member == null) {
            member =
                new ServiceAdv("jxta.service.membership",
                               "JXTA1.0",
                               "Membership",
                               null,
                               new Vector(),
                               "http:/www.jxta.org/download/jxta.jar",
                               "sun.com",
                               "TBD",
                               "net.jxta.impl.membership.NullMembership");

            services.put("jxta.service.membership", member);
        }

	/*
	 * Create the default TCP endpoint
	 */

	Hashtable endpoints =  advertisement.getEndpointAdvertisements();
        if (endpoints == null) {
            endpoints = new Hashtable();
            advertisement.setEndpointAdvertisements(endpoints);
        }

        EndpointAdv tcpEp = (EndpointAdv) (endpoints.get("TCP"));
        TCPAdv tcpAdv = null;
        if (tcpEp == null) {
            tcpAdv = new TCPAdv("TCP", "9701", "224.0.1.85", "1234",
                                "16384", "TBD");
            tcpEp = new EndpointAdv("TCP", "TCP", "TBD", tcpAdv);
            endpoints.put("TCP", tcpEp);
        } else {
            tcpAdv = (TCPAdv) tcpEp.getTransportAdvertisement();
            if (tcpAdv == null) {
                tcpAdv = new TCPAdv("TCP", "9701", "224.0.1.85", "1234",
                                    "8129", "TBD");
                tcpEp.setTransportAdvertisement(tcpAdv);
            }
        }

        EndpointAdv httpEp = (EndpointAdv) (endpoints.get("HTTP"));
        HTTPAdv httpAdv = null;
        if (httpEp == null) {
            httpAdv = new HTTPAdv("http", "9700", "TBD");
            httpAdv.setRouterEnabled(true);
            httpAdv.setProxyEnabled(false);
            httpAdv.setServerEnabled(false);
            httpEp =  new EndpointAdv("HTTP", "HTTP", "TBD", httpAdv);
            endpoints.put("HTTP", httpEp);
        } else {
            httpAdv = (HTTPAdv) httpEp.getTransportAdvertisement();
            if (httpAdv == null) {
                httpAdv = new HTTPAdv("http", "9700", "TBD");
                httpEp.setTransportAdvertisement(httpAdv);
            }
        }


        String hn = "";
        try {
            hn = InetAddress.getLocalHost().getHostName();
        } catch (Exception e) {
            hn = "";
        }

        if (httpAdv.getServer() == null) {
            httpAdv.setServer(hn + ":" + "9700");
            httpAdv.setServerEnabled(false);
        }

        if (httpAdv.getProxy() == null) {
            httpAdv.setProxy("myProxy.myDomain:8080");
            httpAdv.setProxyEnabled(false);
        }

        if (httpAdv.getRouters() == null) {
            Vector r = new Vector();
            r.addElement("209.25.154.239:6002");
            r.addElement("63.115.99.228:6002");
            r.addElement("jxta.clip2.com:6002");
            httpAdv.setRouters(r);
            httpAdv.setRouterEnabled(true);
        }


        ServiceAdv app = (ServiceAdv) advertisement.getApp();
        if (app == null) {
            app = new ServiceAdv(
                                 "jxta.service.InitialApp", "JXTA1.0",
                                 null,
                                 null,
                                 new Vector(),
                                 "http:/www.jxta.org/download/jxta.jar",
                                 "sun.com", "TBD",
                                 "net.jxta.impl.peergroup.StartNetPeerGroup");
            advertisement.setApp(app);
        }

        ServiceAdv grp = (ServiceAdv) advertisement.getGroupImpl();
        if (grp == null) {
            grp = new ServiceAdv(
                                 "jxta.service.Group", "JXTA1.0",
                                 null,
                                 null,
                                 new Vector(),
                                 "http:/www.jxta.org/download/jxta.jar",
                                 "sun.com", "TBD",
                                 getClass().getName());
            advertisement.setGroupImpl(grp);
        }
    }

    class PosRandom extends Random {
        public int nextPos() { return next(31); }
    }

    public Configurator() {
        
        loadAdvertisement(advName);

        if (advertisement != null) {
            return;
        }

        loadAdvertisement(configName);
        fixAdvertisement();

        try {
            ConfigDialog ignore = new ConfigDialog(advertisement);
        } catch (Throwable t) {
            BufferedReader in
                = new BufferedReader(new InputStreamReader(System.in));

            try {
                while (in.ready()) in.readLine();
            } catch (Exception e) {
            }
            System.out.println("The window-based configurator does not seem"
                               + " to be usable.");
            System.out.print("Do you want to stop and edit the current "
                             + "configuration ? [no]: ");
            String answer = "no";
            try {
                answer = in.readLine();
            } catch (Exception e) {
            }

	    // this will cover all the cases of the answer yes, while
	    // allowing non-gui/batch type scripts to load the platform
	    // see platform issue #45
	    if("yes".equalsIgnoreCase(answer)){		
                saveAdvertisement(configName);
                System.out.println("Exiting; edit the file " + configName
                                   + " and then, launch JXTA again.");
                System.exit(0);
            } else {
                System.out.println("Using the current configuration.");
            }
        }

        saveAdvertisement(configName);

        // Make sure that the rendez-vous and router lists
        // are randomly ordered.

        // In addition, reduce http rendez-vous and routers to their
        // intersection, in order to accomodate a current limitation.
        // This accounts for most of the junk below.
        // Note that if http routers are not required then all rdvs are good.

        ServiceAdv resolver = null;
        Vector rdvs = null;
        try {
            Hashtable servs = advertisement.getServiceAdvertisements();
            resolver = (ServiceAdv) (servs.get("jxta.service.resolver"));
            rdvs = resolver.getParams();
        } catch (Exception e) {
        }

        HTTPAdv httpAdv = null;
        EndpointAdv httpEp = null;
        Vector routers = null;
        try {
            httpEp = (EndpointAdv)
                advertisement.getEndpointAdvertisements().get("HTTP");

            httpAdv = (HTTPAdv) httpEp.getTransportAdvertisement();
                
            routers = httpAdv.getRouters();
        } catch (Exception e) {
        }

        // Just do simplify the handling of these corner cases.
        if (routers == null) routers = new Vector();
        if (rdvs == null) rdvs = new Vector();

        Vector intRouters = new Vector();
        Vector intRdvs = new Vector();
        int intersectSz = 0;
        PosRandom random = new PosRandom();

        Enumeration eRdvs = rdvs.elements();

        while (eRdvs.hasMoreElements()) {

            String rdv = (String) eRdvs.nextElement();

            // This elimination process is only for HTTP rdvs.
            // Tcp rdvs are letf alone; they're added at the end
            if (rdv.startsWith("tcp://")) continue;
            rdv = rdv.substring(7);
            Enumeration eRouters = routers.elements();

            // If routing is not enabled, we do not have to look.
            // all rdvs are good (although probably not needed, but that's
            // not our decision to make).
            boolean found = true;
            if (httpAdv.getRouterEnabled()) {
                found = false;
                while (eRouters.hasMoreElements()) {
                    String router = (String) eRouters.nextElement();
                    if (rdv.equals(router)) {
                        found = true;
                        break;
                    }
                }
            }
            if (! found) continue;

            ++intersectSz;
            int here = random.nextPos() % intersectSz;

            intRouters.insertElementAt(rdv, here);
            intRdvs.insertElementAt("http://" + rdv, here);
        }

        // Add tcp rdvs at the begining or the end, dependent on weither
        // http is actually in use or not.
        int front;
        int range = 0;
        if (httpEp.getEnabled()) {
            front = intersectSz;
        } else {
            front = 0;
        }
        eRdvs = rdvs.elements();
        while (eRdvs.hasMoreElements()) {

            String rdv = (String) eRdvs.nextElement();
            if (rdv.startsWith("http://")) continue;
            else {
                ++range;
                intRdvs.insertElementAt(rdv, random.nextPos() % range + front);
            }
        }

        // Replace the two vectors in the adv.
        if (intRouters.size() == 0) intRouters = null;

        httpAdv.setRouters(intRouters);
        resolver.setParams(intRdvs);
    }

    // Get a usable advertisement. It may still need to be pruned and
    // updated before saving.
    public PeerGroupAdvertisement get() {
        return advertisement;
    }

    // Save the "real" advertisement. Always good to call it after
    // it was pruned and updated by the various services, so that
    // the file reflects what is actually in use.
    public void save() {
        saveAdvertisement(advName);
    }

    public void remove() {
        try {
            /*
             * delete the old platform and put the new config
             * as the default one
             */
            File file = new File(advName);
            file.delete();
        } catch (Exception  ex1) {
            System.err.println("Could not clear configuration.");
            System.err.println(
                               "remove "
                               + advName
                               + " by hand before retrying");
            System.exit(1);
        }
    }
}
