/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RefPeerGroup.java,v 1.3 2001/05/23 01:00:20 lomax Exp $
 */



/**
 * This is the base of all peer group implementations in this Reference
 * implementation. There may be various implementations of PeerGroup,
 * but if they do not derive from this, they're not compatible.
 * In a way, this is the interface that all PeerGroups of the reference
 * implementation assume from each other. For now, the only thing a
 * RefPeerGroup has over a PeerGroup is: getEndPoint().
 */

package net.jxta.impl.peergroup;

import java.util.*;
import java.io.*;
import net.jxta.peergroup.*;
import net.jxta.endpoint.*;
import net.jxta.protocol.*;

import net.jxta.impl.endpoint.*;

/*
 * Supplies the PeerGroup APIs that are proper to the reference implementation
 * In this implementation, most of the tasks of the group are performed by
 * a number of service modules, and by the endpoint implementation.
 */

public interface RefPeerGroup extends PeerGroup {

    /*
     * RefPeerGroup have hierarchical relationships.
     */

    /**
     * Ask a group its parent group.
     * Not all group implementations have to tell you, though, and some groups
     * do not have a parent. So, null is a valid result.
     * @return PeerGroup the parent group.
     */
    public RefPeerGroup getParentGroup();

    /**
     * Get the endpoint used by this group.
     * @return Endpoint the endpoint object.
     */
    public Endpoint getEndpoint();

}
