/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: StartNetPeerGroup.java,v 1.30 2001/06/09 22:59:03 bondolo Exp $
 */

/*
 * This code is responsible for configuring and starting the
 * netPeerGroup.
 * Right now it starts a pretty harbitrary peergroup; almost identical to
 * platform. (It actually borrows its adv).
 */

package net.jxta.impl.peergroup;

import java.io.*;
import java.util.*;
import net.jxta.service.*;
import net.jxta.id.*;
import net.jxta.protocol.*;
import net.jxta.discovery.*;
import net.jxta.peergroup.*;
import net.jxta.document.*;
import net.jxta.platform.Application;

import net.jxta.impl.id.UUID;
import net.jxta.impl.id.UUIDFactory;
import net.jxta.impl.protocol.ServiceAdv;

public class StartNetPeerGroup implements Application {

    public int startApp(String[] arg) {
        return 0;
    }
    
    public void stopApp() {
    }

    public void init(PeerGroup pg, Advertisement sadv) {

	// Attempt the recover the NetPeerGroup from a saved adv.
	PeerGroupAdvertisement npgAdv = null;

	// Look for the NetPeerGroup ADV on the net. In theory, there
	// should be some authoritative source for it. That is, whatever
	// advertisement named "NetPeerGroup" that you find, should describe
	// the same group with the same GID. In practive, for now, anyone
	// is an authoritative source, so there might be competing versions
	// seeded by different people who did not find any instance around
	// at some point in time and decided to seed it.


	BufferedReader in
	    = new BufferedReader(new InputStreamReader(System.in));

	// Offer the newbie mode; others know what to do.
	// That is, race against the boot to type "D<enter>"
	// before we run that code below...

	boolean doDisco = false;
	String answer = "";

	try {
	    if (in.ready()) {
		answer = in.readLine();
		if (answer.equals("D")) {
		    doDisco = false;
		}
	    }
	} catch (Exception e) {
	}

	// If the user typed "D", we'll do discovery anyhow.
	// otherwise, we'll try to be smart; if there ever was
	// a NetPeerGroup other than the default instanciated
	// here, we'll assume that the user uses discovery for the
	// netPeerGroup. Otherwise, we'll assume that the user
	// just always uses the hardwired default.

	Vector advs = new Vector();
	boolean first = true;

	Discovery disco = null;
        disco = pg.getDiscovery();

	// Until we find something or the user decides to make a choice.

	try {
	    while (in.ready()) in.readLine();
	} catch (Exception e) {
	}

	int s = 0;
	int current = 0;

	while (true) {
	    try {
		if (in.ready()) {
		    break;
		}
	    } catch (Exception e) {
		break;
	    }

	    Enumeration ae = null;
	    try {
		if (first) {
		    ae = disco.getLocalAdvertisements(Discovery.GROUP
						      , "Name"
						      , "NetPeerGroup");
		} else {
		    disco.getRemoteAdvertisements(null
						  , Discovery.GROUP
						  , "Name"
						  , "NetPeerGroup",10);
		    try {
			wait(5000);
		    } catch (Exception e){
		    }
		    ae = disco.getLocalAdvertisements(Discovery.GROUP
						      , "Name"
						      , "NetPeerGroup");
		}
	    } catch (IOException e){
		// found nothing!  move on
	    }
	    while (ae != null && ae.hasMoreElements()) {
		PeerGroupAdvertisement a =
		    (PeerGroupAdvertisement) ae.nextElement();

		if (a.getName().equals("NetPeerGroup")) {
		    String kw = a.getKeywords();
		    if (kw == null) kw = "";
		    if ((!kw.equals("NetPeerGroup by default"))
			&& a.getPid().equals(pg.getPeerID().toString())) {

			// This user has used the discovery option before
			// therefore he wants to use it.
			doDisco = false;
		    }
		    advs.addElement(a);
		}
	    }
	    // If doDisco is not true the first time around (after
	    // a local discovery), then we don't want it.
	    if (! doDisco) {
		advs.removeAllElements();
		break;
	    }

	    if (first) {
		System.out.println("Looking for NetPeerGroup advertisements. ");
		System.out.println("Press <enter> to stop looking and choose. ");
		first = false;
	    }

	    // Display the last crop
	    s = advs.size();
	    while (current < s) {
		PeerGroupAdvertisement a =
		    (PeerGroupAdvertisement) advs.elementAt(current);

		String self = " ";
		if (a.getPid().equals(pg.getPeerID().toString())) {
		    self = "*";
		}
		String kw = a.getKeywords();
		if (kw == null) kw = "";
		System.out.println(self + current + ": " + kw);
		current++;
	    }
	}

	s = advs.size();
	if (s > 0) {
	    while (true) {
		System.out.println(" " + s + ": None of the above");
		System.out.println("\n* = previously instantiated here\n");
		try {
		    while (in.ready()) in.readLine();
		    System.out.print("Chose which advertisement you want"
					   + " to use: ");
		    answer = in.readLine();
		    int i = Integer.parseInt(answer);
		    if (i == s) break;
		    if (i >= 0 && i <= s) {
			npgAdv = (PeerGroupAdvertisement) advs.elementAt(i);
			break;
		    }
		    i = 0;
		    System.out.println("\n\nFound:");
		    while (i < s) {
			PeerGroupAdvertisement a =
			    (PeerGroupAdvertisement) advs.elementAt(i);

			String self = " ";
			if (a.getPid().equals(pg.getPeerID().toString())) {
			    self = "*";
			}
			String kw = a.getKeywords();
			if (kw == null) kw = "";
			System.out.println(self + i + ": " + kw);
			i++;
		    }
		} catch (Exception e) {
		    break;
		}
	    }
	}

	if (npgAdv != null) {
	    // Good, just start it then.
	    StdPeerGroup g = new StdPeerGroup();
	    g.init(pg, npgAdv);
	    g.initNetworkGroup (); // FIXME: what's that ?
            g.startApp(null);
	    return;
	}


	// Ok, then, maybe we should seed that group.
	// make a new one out of the platform props and the platform's adv.

	// flush
	if (doDisco) {
	    System.out.println("\nNo NetPeerGroup advertisement selected.");

	    try {
		while (in.ready()) in.readLine();

		System.out.print("Do you want to seed the NetPeerGroup ? :");
		answer = in.readLine();

		if (answer.equals("yes")) {
		    System.out.println("Ok, you're the boss.");
		} else {
		    System.out.println("We'll just start a shell here, then.");
		    Application app = (Application)
			((GenericPeerGroup) pg).loader.
			loadClass("net.jxta.impl.shell.bin.Shell.Shell",
				  "",
				  true).newInstance();

		    if (app == null) {
			System.out.println("Error loading Shell");
			System.exit(0);
		    }
		    app.init(pg, null); // FIXME: yuck, null adv.
		    return;
		}

		while (in.ready()) in.readLine();
		System.out.println("Please enter an identifier to sign the"
				   + " advertisement.");
		System.out.print("Your name will be fine. :");
		answer = in.readLine();
	    } catch (Exception e) {
		e.printStackTrace();
		System.exit(1);
	    }
	} else {
	    answer = "default";
	}

	// Ok. Here we go.
	FileInputStream inconf = null;
	Properties prop = null;

	try {
	    inconf = new FileInputStream("jxtaConfig");
	    prop = new Properties();
	    prop.load(inconf);
	    inconf.close();
	} catch (Exception  ex1) {
	    System.err.println("Disaster: No platform default properties.");
	    System.exit(1);
	}

	PeerGroupAdvertisement adv = pg.getAdvertisement();
	StdPeerGroup g = new StdPeerGroup();

	String pgId = "";  // Will create new grp.

	if (! doDisco) {  // Replace it with a hardcoded value.
	    pgId = (new PeerGroupID( new UUID
		(
		 0x5961626164616261L, 0x4A757874614D5047L  // Yabadaba JuxtaNPG
		 )
		)).toString();
	}
	try {
	    npgAdv = (PeerGroupAdvertisement)
		AdvertisementFactory.newAdvertisement(
			   PeerGroupAdvertisement.getAdvertisementType());
	} catch (Exception  ex1) {
	    System.err.println("Disaster: Could not construct "
			       + "Net Peer Group.");
	    System.exit(1);
	}

	npgAdv.setPid ("");
	npgAdv.setGid (pgId);
	npgAdv.setName ("NetPeerGroup");
	npgAdv.setKeywords("NetPeerGroup by " + answer);
	npgAdv.setServiceAdvertisements(adv.getServiceAdvertisements());
	// no private endpoints here.
	npgAdv.setEndpointAdvertisements(new Hashtable());
	npgAdv.setIsRendezvous (pg.isRendezvous());
	npgAdv.setApp(
	   new ServiceAdv("Shell", "JXTA1.0",
                          null,
                          null,
                          new Vector(),
			  prop.getProperty("InitialNetPeerGroupAppCodeURL"),
			  "sun.com", "TBD",
			  prop.getProperty("InitialNetPeerGroupAppCode")));
	npgAdv.setGroupImpl(
	   new ServiceAdv("Group", "JXTA1.0",
                          null,
                          null,
                          new Vector(),
			  "http:/www.jxta.org/download/jxta.jar",
			  "sun.com", "TBD",
			  g.getClass().getName()));

	//
	// start the peergroup and launch the initial
	// application
	//
	g.init(pg, npgAdv);
	g.initNetworkGroup ();
        g.startApp(null);
    }
}
