/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: StdPeerGroup.java,v 1.14 2001/05/23 01:00:20 lomax Exp $
 */



package net.jxta.impl.peergroup;

import java.util.*;
import java.io.*;

import net.jxta.document.Advertisement;

import net.jxta.peer.*;
import net.jxta.endpoint.*;
import net.jxta.peergroup.*;
import net.jxta.protocol.*;
import net.jxta.platform.*;

import net.jxta.impl.discovery.*;
import net.jxta.impl.resolver.*;
import net.jxta.impl.endpoint.*;
import net.jxta.impl.discovery.*;
import net.jxta.impl.pipe.*;
import net.jxta.impl.peergroup.*;

import net.jxta.impl.document.*;
import net.jxta.service.*;
import net.jxta.resolver.*;
import net.jxta.exception.*;
import net.jxta.credential.*;




public class StdPeerGroup extends GenericPeerGroup {

    static {
	try {
	    PeerGroupFactory.setStdPeerGroupClass(Class.forName("net.jxta.impl.peergroup.StdPeerGroup"));
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    private boolean initialized = false;

    public void init(PeerGroup parent, Advertisement gadv)
    {
	if (initialized == true) {
	    System.err.println("You cannot initialize a PeerGroup more"
			       +" than once !");
	    return;
	}
	initialized = true;

        PeerGroupAdvertisement adv = (PeerGroupAdvertisement) gadv;
    
	// Set our endpoint to be that of our parent.
	setEndpoint(((RefPeerGroup) parent).getEndpoint());

	// Set-up the minimal GenericPeerGroup
	super.init(parent, adv);

	try {

            /*
             * Load dynamically all the services from the advertisement
             */

            Hashtable services = adv.getServiceAdvertisements();
            ServiceAdvertisement service = null;

            /*
             * load the ResolverService first
             */

            service = ((ServiceAdvertisement)
                       services.get("jxta.service.resolver"));

            Service s = (Service) loader.loadClass(service.getCode(),
                                                   service.getUri(),
                                                   true).newInstance();
            s.init(this, service);
            addService(s);

            /*
             * Extract the services and load them
             */

	    for (Enumeration e = services.elements(); e.hasMoreElements();) {
                service = (ServiceAdvertisement) e.nextElement();

                if (!service.getName().equals("jxta.service.resolver")) {
                    s = (Service) loader.loadClass(service.getCode(),
                                                   service.getUri(),
                                                   true).newInstance();
                    s.init(this, service);
                    addService(s);
                }
	    }

            // Make sure all the required services are there.
            checkServices();

	} catch(Exception e) {
	    e.printStackTrace();
	    System.out.println("Something botched");
	}
    }


    // No main. This is "applet (sort of)" only, right now
}
