/*
 * $Id: InputPipeImpl.java,v 1.14 2001/05/23 01:00:20 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: InputPipeImpl.java,v 1.14 2001/05/23 01:00:20 lomax Exp $
 */
package net.jxta.impl.pipe;
import java.io.*;

import net.jxta.endpoint.*;
import net.jxta.impl.endpoint.*;
import net.jxta.peergroup.*;
import net.jxta.impl.peergroup.*;
import net.jxta.pipe.*;


/**
 * This class implements the Input PipeService.
 *
 */


public class InputPipeImpl implements InputPipe, EndpointHandler {

    private boolean debug = false;
    private EndpointReceiveQueue queue = null;
    private String pipeId = null;
    private RefPeerGroup myGroup = null;
    private PipeResolver myPipeResolver = null;
    private boolean  closed = false;

    public InputPipeImpl (PeerGroup g,
			  PipeResolver r,
			  PipeID ipId) {

	myGroup = (RefPeerGroup) g;
	pipeId = ipId.toString();
	myPipeResolver = r;

	debug ("Constructor:");
	debug ("    pipeId = " + ipId);

	// Initialize the user queue
	queue = new EndpointReceiveQueue ();

	// Register this InputPipe with the endpoint's general
	// demux mechanism.
	try {
	    myGroup.getEndpoint().registerHandler ("PipeService" + ipId, this);
	} catch (Exception e) {
	    debug ("Constructor failed with " + e);
	    // XXX: something should maybe be done here
	}

	// Register this input pipe
	myPipeResolver.register (pipeId, this);
    }

    /**
     * wait for a message
     *
     * @return Message  message received
     */

    public Message waitForMessage() throws IOException {

	return queue.waitForMessage();
    }

    /**
     * Poll for a message from the pipe
     *
     * @return Message message received
     */
    public Message poll(int timeout) throws IOException {

	return queue.poll(timeout);
    }

    /**
     * close the the pipe
     *
     */
    public synchronized void close() {

	// First remove the queue for the general demux
	// XXX: what to do if unregister fails ?

        boolean res = myGroup.getEndpoint().unregisterHandler (
                                               "PipeService" + pipeId, this);

	// Remove the input pipe from the pipe resolver.
	myPipeResolver.forget (pipeId);

	// Close the queue
	queue.close();
	closed = true;
    }

    protected void finalize () throws Throwable {
	if (!closed) {
	    close();
	}
    }

    // EndpointHandler API

    public void demux (MessageImpl msg) {
	
	// XXX: header check, security and such should be done here
	// before pushing the message onto the queue.

	queue.push (msg);
    }

    private void debug (String str) {
	if (debug) {
	    System.out.println ("[InputPipeImpl] " + str);
	}
    }
}



