/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: DiscoveryQuery.java,v 1.10 2001/05/23 01:00:20 lomax Exp $
 */

package net.jxta.impl.protocol;

import java.io.InputStream;
import java.io.StringWriter;
import java.util.Enumeration;

import java.lang.reflect.InvocationTargetException;

import net.jxta.document.*;
import net.jxta.protocol.*;
import net.jxta.resolver.*;


/**
 * DiscoveryQuery
 *
 * @version $Revision: 1.10 $
 * Tag  $Name:  $
 * @since 1.0
 */
public class DiscoveryQuery extends DiscoveryQueryMsg {


  /**
   * default threshold to limit the number of responses
   * from one peer
   */

    public static int threshold = 10;

 /**
  * Construct a doc from InputStream
  *
  */

  public DiscoveryQuery( InputStream stream ) throws Exception {
        try {
        // We are asked to assume that the message from which this response is constructed is an XML document.
        StructuredTextDocument doc = (StructuredTextDocument)
            StructuredDocumentFactory.newStructuredDocument(
            new MimeMediaType( "text", "xml"), stream );
        readIt( doc );
        }
        catch( InvocationTargetException encaped ) {
            Throwable something = encaped.getTargetException();
            if( Exception.class.isInstance( something ) )
                throw (Exception) something;
            else
                throw encaped;
        }
    }

 /**
  * Construct from a StructuredDocument
  *
  */

  public DiscoveryQuery( TextElement doc ) {

    if( !getAdvertisementType().equals( doc.getName() ) )
      throw new IllegalArgumentException( "Could not construct : " +
       getClass().getName() + "from doc containing a " + doc.getName() );

    readIt( doc );
  }

  public void readIt( TextElement doc ) {

    Enumeration elements = doc.getChildren();
    while (elements.hasMoreElements()) {
       TextElement elem = (TextElement) elements.nextElement();
       if( elem.getName().equals ("type")) {
           type=Integer.parseInt( elem.getTextValue() );
           continue;
       }

       if( elem.getName().equals("threshold")) {
           threshold=Integer.parseInt( elem.getTextValue() );
         continue;
       }

       if( elem.getName().equals("peeradv")) {
         setPeerAdv ( elem.getTextValue() );
         continue;
       }

       if( elem.getName().equals("attr" )) {
         setAttr ( elem.getTextValue() );
         continue;
       }
       if( elem.getName().equals("value" )) {
         setValue ( elem.getTextValue() );
         continue;
       }
     }
  }

 /**
  * Construct a doc from strings
  *
  */

  public DiscoveryQuery(int type, String peeradv,
                           String attr, String value) {

        setDiscoveryType(type);
        setThreshold(threshold);
        setPeerAdv(peeradv);
        setAttr(attr);
        setValue(value);
  }

 /**
  *  return a Document represetation of this object
  */

  public Document getDocument(MimeMediaType asMimeType) throws Exception {
        StructuredDocument adv = (StructuredTextDocument)
            StructuredDocumentFactory.newStructuredDocument( asMimeType,
            getAdvertisementType() );
        Element e;

        e = adv.createElement ("type", Integer.toString(type));
        adv.appendChild (e);
        e = adv.createElement ("threshold", Integer.toString(threshold));
        adv.appendChild (e);
        e = adv.createElement ("peeradv", getPeerAdv());
        adv.appendChild (e);
        e = adv.createElement ("attr",    getAttr());
        adv.appendChild (e);
        e = adv.createElement ("value",   getValue());
        adv.appendChild (e);
        return adv;
  }

 /**
  * return the string representaion of this doc
  *
  */

  public String toString() {
        try {
            StringWriter    out = new StringWriter();
            StructuredTextDocument doc = (StructuredTextDocument) getDocument( new MimeMediaType("text/xml") );
            doc.sendToWriter( out );

            return out.toString();
        }
        catch( Exception e ) {
            return "";
        }
  }


}
