/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: PeerGroupAdv.java,v 1.16 2001/05/23 01:00:20 lomax Exp $
 */

package net.jxta.impl.protocol;

import java.io.*;
import java.util.*;

import java.lang.reflect.InvocationTargetException;

import net.jxta.protocol.*;
import net.jxta.document.*;

/**
 * This class implements the PeerGroupAdvertisement. While being useful,
 * application are not required to use it. However, applications that
 * donot want to use it will still have to provide a class that
 * extends net.jxta.protocol.PeerGroupAdvertisement.
 *
 */

public class PeerGroupAdv extends PeerGroupAdvertisement {
    
    public PeerGroupAdv() {
    }
    
    public PeerGroupAdv( Element root ) throws Exception {
        initialize( root );
    }
    
    public void initialize(Element root) throws Exception {
        
        if( !TextElement.class.isInstance( root ) )
            throw new IllegalArgumentException( getClass().getName() + " only supports TextElement" );
        
        TextElement doc = (TextElement) root;
        
        if( !getAdvertisementType().equals( doc.getName() ) )
            throw new IllegalArgumentException( "Could not construct : "
            + getClass().getName()
            + "from doc containing a "
            + doc.getName() );
        
        // Set up everything empty
        setName( null );
        setKeywords( null );
        setPid( null );
        setGid( null );
        setIsRendezvous(false);
        setApp(null);
        setGroupImpl(null);
        
        // Get local handles to these tables, for convenience.
        Hashtable servs = getServiceAdvertisements();
        Hashtable endps = getEndpointAdvertisements();
        
        // Fill the stuff with what we find in the adv.
        Enumeration elements = doc.getChildren( );

        while ( elements.hasMoreElements() ) {
            TextElement element = (TextElement) elements.nextElement();
            
            // The basics
            if (element.getName().equals("DebugOn")) {
                setDebug(true);
                continue;
            }
            if (element.getName().equals("Name")) {
                setName (element.getTextValue());
                continue;
            }
            if (element.getName().equals("PeerName")) {
                setPeerName (element.getTextValue());
                continue;
            }
            
            if (element.getName().equals("isRendezvous")) {
                
                if (element.getTextValue().equals("true"))
                    setIsRendezvous (true);
                else
                    setIsRendezvous (false);
                continue;
            }
            
            if (element.getName().equals( "Keywords" )) {
                setKeywords ( element.getTextValue() );
                continue;
            }
            
            if (element.getName().equals( "Pid" )) {
                setPid ( element.getTextValue() );
                continue;
            }
            
            if (element.getName().equals( "Gid" )) {
                setGid ( element.getTextValue() );
                continue;
            }
            

            // Endpoints:
            if (element.getName().equals( "Services" )) {
                for( Enumeration eachService = element.getChildren(); eachService.hasMoreElements(); ) {
                    TextElement aService = (TextElement) eachService.nextElement();
                    
                    ServiceAdvertisement sadv = (ServiceAdvertisement) AdvertisementFactory.newAdvertisement(aService);
                    
                    servs.put(sadv.getName(), sadv);
                }
                continue;
            }

            // Endpoints:
            if (element.getName().equals( "Endpoints" )) {
                for( Enumeration eachEndpoint = element.getChildren(); eachEndpoint.hasMoreElements(); ) {
                    TextElement aEndpoint = (TextElement) eachEndpoint.nextElement();
                    
                    EndpointAdvertisement eadv = (EndpointAdvertisement) AdvertisementFactory.newAdvertisement(aEndpoint);
                    endps.put(eadv.getName(), eadv);
                }
                continue;
            }
            
            // Initial app
            if (element.getName().equals( "InitialApp" )) {
                for( Enumeration eachApp = element.getChildren(); eachApp.hasMoreElements(); ) {
                    TextElement aApp = (TextElement) eachApp.nextElement();
                    
                    ServiceAdvertisement sadv = (ServiceAdvertisement) AdvertisementFactory.newAdvertisement(aApp);
                    setApp (sadv);
                }
                continue;
            }
            
            // group Impl
            if (element.getName().equals( "GroupImpl" )) {
                for( Enumeration eachApp = element.getChildren(); eachApp.hasMoreElements(); ) {
                    TextElement aApp = (TextElement) eachApp.nextElement();
                    
                    ServiceAdvertisement sadv = (ServiceAdvertisement) AdvertisementFactory.newAdvertisement(aApp);
                    setGroupImpl (sadv);
                }
                continue;
            }
        }
        // Sometimes this tag is either missing or left blank.
        // in both cases, we get null. Avoid leaving it like that.
        // FIXME: In general we need to make sure that we do not
        // leave nulls in adv object except when it is clearly expected
        // (not names). The best would be to use non-null
        // initial values, and to ignore nulls we get from parsing.
        // ...can wait. In the mean time, here is at least one patch we need.

        if (getPeerName() == null) setPeerName("");
    }
    
    public Document getDocument( MimeMediaType encodeAs ) throws Exception {
        StructuredTextDocument adv = (StructuredTextDocument)
        StructuredDocumentFactory.newStructuredDocument(
        encodeAs, getAdvertisementType() );
        Element e1 = adv.createElement ("Name", getName());
        adv.appendChild (e1);

        Element epn = adv.createElement ("PeerName", getPeerName());
        adv.appendChild (epn);

        if (getDebug()) {
            Element edbg = adv.createElement("DebugOn");
            adv.appendChild(edbg);
        }
        Element e2 = adv.createElement ("Keywords", getKeywords());
        adv.appendChild (e2);
        
        Element e = adv.createElement ("Pid", getPid());
        adv.appendChild (e);
        
        Element e03 = adv.createElement ("Gid", getGid());
        adv.appendChild (e03);
        
        if (getIsRendezvous())
            e = adv.createElement ("isRendezvous","true");
        else
            e = adv.createElement ("isRendezvous","false");
        
        adv.appendChild (e);
        
        /*
         * create all the services
         */
        
        Element e3 = adv.createElement ("Services");
        adv.appendChild (e3);
        
        for (Enumeration p = getServiceAdvertisements().elements();
        p.hasMoreElements();) {
            
            ServiceAdvertisement service = (ServiceAdvertisement) p.nextElement();
            
            StructuredTextDocument serviceDoc = (StructuredTextDocument) service.getDocument( encodeAs );
            
            StructuredDocumentUtils.copyElements( adv, e3, serviceDoc );
        }
        
        /*
         * create all the endpoints
         */
        
        Element e4 = adv.createElement ("Endpoints");
        adv.appendChild (e4);
        
        for (Enumeration p = getEndpointAdvertisements().elements();
        p.hasMoreElements();) {
            
            EndpointAdvertisement endpoint =
            (EndpointAdvertisement) p.nextElement();
            
            StructuredTextDocument endpointDoc = (StructuredTextDocument) endpoint.getDocument( encodeAs );
            
            StructuredDocumentUtils.copyElements( adv, e4, endpointDoc );
        }
        
        /*
         * generate the default application
         */
        Element e5 = adv.createElement ("InitialApp");
        adv.appendChild (e5);
        
        ServiceAdvertisement app = getApp();
        
        StructuredTextDocument appDoc = (StructuredTextDocument) app.getDocument( encodeAs );
        
        StructuredDocumentUtils.copyElements( adv, e5, appDoc );
        
        /*
         * generate the group implementation
         */
        Element e6 = adv.createElement ("GroupImpl");
        adv.appendChild (e6);
        
        ServiceAdvertisement grp = getGroupImpl();
        
        StructuredTextDocument grpDoc = (StructuredTextDocument) grp.getDocument( encodeAs );
        
        StructuredDocumentUtils.copyElements( adv, e6, grpDoc );
        
        return adv;
    }
    
    public String toString() {
        throw new RuntimeException();
    }
}



