/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: PeerInfoAdv.java,v 1.9 2001/06/09 19:00:03 bondolo Exp $
 */

package net.jxta.impl.protocol;

import java.io.InputStream;
import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Enumeration;
import java.lang.reflect.InvocationTargetException;
import net.jxta.protocol.PeerInfoAdvertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.document.Element;
import net.jxta.document.TextElement;
import net.jxta.document.MimeMediaType;
import net.jxta.document.Document;
import net.jxta.document.StructuredTextDocument;
import net.jxta.document.StructuredDocumentFactory;

/**
 * This class implements the PeerInfoAdvertisement. While being useful,
 * application are not required to use it. However, applications that
 * donot want to use it will still have to provide a class that
 * extends net.jxta.protocol.PeerInfoAdvertisement.
 */
public class PeerInfoAdv extends PeerInfoAdvertisement {

    public PeerInfoAdv() {
        super();
    }

    /**
     *  @deprecated Please use the Advertisement factory and accessors
     **/
    public PeerInfoAdv(String spid, String tpid, 
                       int uptime, int timestamp,
                       int itime, int otime,
                       Hashtable itraffic, Hashtable otraffic) {
        super();
        setSourcePid(spid);
        setTargetPid(tpid);
        setUptime(uptime);
        setTimestamp(timestamp);
        setLastIncomingMessageTime(itime);
        setLastOutgoingMessageTime(otime);

        Enumeration ik = itraffic.keys();
        while (ik.hasMoreElements()) {
            String k = (String) ik.nextElement();
            setIncomingTrafficElement(k, 
                                      ((Long) itraffic.get(k)).longValue());
        }

        Enumeration ok = otraffic.keys();
        while (ok.hasMoreElements()) {
            String k = (String) ok.nextElement();
            setOutgoingTrafficElement(k, 
                                      ((Long) otraffic.get(k)).longValue());
        }
    }

    public PeerInfoAdv( Element root ) throws Exception {
        initialize( root );
    }
    
    public void initialize(Element root) {
        
        if( !TextElement.class.isInstance( root ) )
            throw new IllegalArgumentException( getClass().getName() + " only supports TextElement" );

        setSourcePid(null);
        setTargetPid(null);
        setUptime(0);
        setTimestamp(0);
        setLastIncomingMessageTime(0);
        setLastOutgoingMessageTime(0);

        TextElement doc = (TextElement) root;
        
        Enumeration elements;
        elements = doc.getChildren();

        while (elements.hasMoreElements()) {
            TextElement element = (TextElement) elements.nextElement();
            String elementName = element.getName();
            TextElement parent = (TextElement) element.getParent();
            String parentName = parent.getName();

            if (elementName.equals("sourcePid")) {
                setSourcePid(element.getTextValue());
                continue;
            }

            if (elementName.equals("targetPid")) {
                setTargetPid(element.getTextValue());
                continue;
            }

            if (elementName.equals("uptime")) {
                setUptime(Integer.parseInt(element.getTextValue()));
                continue;
            }

            if (elementName.equals("timestamp")) {
                setTimestamp(Integer.parseInt(element.getTextValue()));
                continue;
            }

            if (elementName.equals("lastIncomingMessageAt") &&
                parent != null && parentName.equals("traffic")) {
                setLastIncomingMessageTime
                    (Integer.parseInt(element.getTextValue()));
                continue;
            }

            if (elementName.equals("lastOutgoingMessageAt") &&
                parent != null && parentName.equals("traffic")) {
                setLastOutgoingMessageTime
                    (Integer.parseInt(element.getTextValue()));
                continue;
            }

            if (elementName.equals("in") &&
                parent != null && parentName.equals("traffic")) {
                Enumeration children = element.getChildren();
                while (children.hasMoreElements()) {
                    Element ch = (Element) children.nextElement();
                    setIncomingTrafficElement
                        ((String) ch.getKey(), 
                         ((Long) ch.getValue()).longValue());
                }
                continue;
            }

            if (elementName.equals("out") &&
                parent != null && parentName.equals("traffic")) {
                Enumeration children = element.getChildren();
                while (children.hasMoreElements()) {
                    Element ch = (Element) children.nextElement();
                    setOutgoingTrafficElement
                        ((String) ch.getKey(), 
                         ((Long) ch.getValue()).longValue());
                }
                continue;
            }
        }
    }

    public Document getDocument(MimeMediaType encodeAs) throws Exception {

        StructuredTextDocument doc = (StructuredTextDocument)
            StructuredDocumentFactory.newStructuredDocument
            (encodeAs, getAdvertisementType());
        Element e = null;

        e = doc.createElement("sourcePid", getSourcePid());
        doc.appendChild(e);

        e = doc.createElement("targetPid", getTargetPid());
        doc.appendChild(e);

        e = doc.createElement("uptime", Integer.toString(getUptime()));
        doc.appendChild(e);

        e = doc.createElement("timestamp", 
                              Integer.toString(getTimestamp()));
        doc.appendChild(e);

        Element traffic = doc.createElement("traffic");
        doc.appendChild(traffic);

        Element itraffic = doc.createElement("in");
        traffic.appendChild(itraffic);

        Element itime = doc.createElement
            ("lastIncomingMessageAt",
             Integer.toString(getLastIncomingMessageTime()));
        traffic.appendChild(itime);

        Element otraffic = doc.createElement("out");
        traffic.appendChild(otraffic);

        Element otime = doc.createElement
            ("lastOutgoingMessageAt",
             Integer.toString(getLastOutgoingMessageTime()));
        traffic.appendChild(otime);

        Enumeration ichannels = getIncomingTrafficChannels();
        while (ichannels.hasMoreElements()) {
            String ch = (String) ichannels.nextElement();
            long tr = getIncomingTrafficOnChannel(ch);
            e = doc.createElement(ch, Long.toString(tr));
            itraffic.appendChild(e);
        }

        Enumeration ochannels = getOutgoingTrafficChannels();
        while (ochannels.hasMoreElements()) {
            String ch = (String) ochannels.nextElement();
            long tr = getOutgoingTrafficOnChannel(ch);
            e = doc.createElement(ch, Long.toString(tr));
            otraffic.appendChild(e);
        }

        return doc;
    }

    public String toString() {
        try {
            StringWriter out = new StringWriter();
            StructuredTextDocument doc = (StructuredTextDocument) 
                getDocument(new MimeMediaType("text/xml"));
            doc.sendToWriter(out);

            return out.toString();
        } catch (Exception e) {
            return "";
        }
    }
}



