/*
 * $Id: PipeAdv.java,v 1.3 2001/06/09 19:00:03 bondolo Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

package net.jxta.impl.protocol;

import java.io.*;
import java.net.URL;
import java.util.Enumeration;

import java.lang.reflect.InvocationTargetException;

import net.jxta.document.*;
import net.jxta.endpoint.*;
import net.jxta.id.IDFactory;
import net.jxta.pipe.*;
import net.jxta.protocol.*;


/**
 * This class implements the PipeAdvertisement. While being useful,
 * application are not required to use it. However, applications that
 * donot want to use it will still have to provide a class that
 * extends net.jxta.pipe.PipeAdvertisement.
 *
 * @version $Revision: 1.3 $
 * @since  JXTA 0.1
 *
 */
public class PipeAdv extends PipeAdvertisement {
    
    public PipeAdv() {
    }
    
    /**
     *  @deprecated Please use the Advertisement factory and accessors
     **/
    public PipeAdv(PipeID pipeId) {
        super();
        setPipeID(pipeId);
    }
    
    /**
     *  @deprecated Please use the Advertisement factory and accessors
     **/
    public PipeAdv(PipeID pipeId, String name ) {
        super();
        setPipeID(pipeId);
        setName(name);
    }
    
    public PipeAdv(Element root) throws Exception {
        initialize( root );
    }
    
    public void initialize(Element root) throws Exception {
        
        if( !TextElement.class.isInstance( root ) )
            throw new IllegalArgumentException( getClass().getName() + " only supports TextElement" );
        
        TextElement doc = (TextElement) root;
        
        if( ! doc.getName().equals( getAdvertisementType() ))
            throw new IllegalArgumentException( "Could not construct : "
            + getClass().getName()
            + " from doc containing a '"
            + doc.getName() + "'. Should be : " + getAdvertisementType() );
     
        Enumeration elements = doc.getChildren();
        
        while (elements.hasMoreElements()) {
            TextElement elem = (TextElement) elements.nextElement();
            if( elem.getName().equals("id")) {
                URL pipeID = new URL( elem.getTextValue() );
                setPipeID( (PipeID) IDFactory.fromURL( pipeID ) );
                continue;
            }
            if (elem.getName().equals("name")) {
                setName(elem.getTextValue());
                continue;
            }
        }
    }
    
    
    public Document getDocument( MimeMediaType mediaType ) throws Exception {
        
        StructuredTextDocument adv = (StructuredTextDocument)
        StructuredDocumentFactory.newStructuredDocument(
        mediaType, getAdvertisementType() );
        
        
        TextElement e = adv.createElement("id", getPipeID().toString() );
        adv.appendChild(e);
        
        if (getName() != null) {
            e = adv.createElement("name", getName());
            adv.appendChild(e);
        }
        
        return adv;
    }
}



