/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ServiceAdv.java,v 1.11 2001/06/14 05:42:41 hamada Exp $
 */

package net.jxta.impl.protocol;

import java.io.*;
import java.util.*;

import net.jxta.document.*;
import net.jxta.protocol.*;
import net.jxta.impl.pipe.*;

/**
 * This class implements the ServiceAdvertisement.
 *
 */

public class ServiceAdv extends ServiceAdvertisement {

    public ServiceAdv() {
    }

    /**
     * Make a service Adv.
     *
     * @deprecated Please use the Advertisement factory and accessors
     */
    public ServiceAdv(String name, String version,  String keywords,
    PipeAdvertisement pipe, Vector params, String uri,
    String provider, String security, String code) {
        setName(name);
        setVersion(version);
        setKeywords(keywords);
        setPipe(pipe);
        setParams(params);
        setUri(uri);
        setProvider(provider);
        setSecurity(security);
        setCode(code);
    }

    public ServiceAdv( Element root ) throws Exception {
        initialize( root );
    }

    public void initialize(Element root) {

        if( !TextElement.class.isInstance( root ) )
            throw new IllegalArgumentException( getClass().getName() + " only supports TextElement" );

        TextElement doc = (TextElement) root;

    /*
     * This is a yucky way to check the type; the element can be validly
     * named many things. See FIXME above.
     */
        if( ! doc.getName().equals( "Service" )
        && ! doc.getName().equals( "GroupImpl" )
        && ! doc.getName().equals( "InitialApp" )
        && ! doc.getName().equals( getAdvertisementType() ) )

            throw new IllegalArgumentException( "Could not construct : "
            + getClass().getName()
            + "from doc containing a "
            + doc.getName() );

        Enumeration elements = doc.getChildren();

        // For convenience get the local handle
        Vector params = new Vector();

        while (elements.hasMoreElements()) {
            TextElement elem = (TextElement) elements.nextElement();
            if( elem.getName().equals("Name")) {
                setName( elem.getTextValue() );
                continue;
            }
            if( elem.getName().equals("Version")) {
                setVersion( elem.getTextValue() );
                continue;
            }
            if( elem.getName().equals("Keywords")) {
                setKeywords( elem.getTextValue() );
                continue;
            }

            if( elem.getName().equals("PipeService")) {
                try {
                    for( Enumeration eachPipe = elem.getChildren(); eachPipe.hasMoreElements(); ) {
                        TextElement aPipe = (TextElement) eachPipe.nextElement();

                        PipeAdvertisement pipe = (PipeAdvertisement)
                        AdvertisementFactory.newAdvertisement(aPipe);
                        setPipe(pipe);
                    }
                }
                catch( Exception ignored ) {

                }
                continue;
            }

            if( elem.getName().equals("Params")) {
                params.addElement( elem.getValue() );
                continue;
            }
            if( elem.getName().equals("Uri")) {
                setUri( elem.getTextValue() );
                continue;
            }
            if( elem.getName().equals("Provider")) {
                setProvider( elem.getTextValue() );
                continue;
            }
            if( elem.getName().equals("Security")) {
                setSecurity( elem.getTextValue() );
                continue;
            }
            if( elem.getName().equals("Code")) {
                setCode( elem.getTextValue() );
                continue;
            }
        }
        setParams(params);
    }

    public Document getDocument( MimeMediaType encodeAs ) {
        StructuredTextDocument adv = null;

        try {
            adv = (StructuredTextDocument)
            StructuredDocumentFactory.newStructuredDocument(
            encodeAs, getAdvertisementType() );
        }
        catch ( Throwable everything ) {
            return null;
        }

        Element e1 = adv.createElement("Name", getName());
        adv.appendChild(e1);

        Element e2 = adv.createElement("Version", getVersion());
        adv.appendChild(e2);

        Element e3 = adv.createElement("Keywords", getKeywords());
        adv.appendChild(e3);

        PipeAdvertisement pipe = getPipe();

        Element e4 = adv.createElement("PipeService");
        adv.appendChild(e4);

        if  (pipe != null) {
            try {
            StructuredTextDocument pipeDoc = (StructuredTextDocument) pipe.getDocument( encodeAs );
            StructuredDocumentUtils.copyElements( adv, e4, pipeDoc );
            }
            catch( Exception ignored ) {
            }
        }

        Vector params = getParams();

        if (params != null) {
            for (Enumeration e = params.elements(); e.hasMoreElements();) {
                Element e5 = adv.createElement("Params",
                (String) e.nextElement());
                adv.appendChild(e5);
            }
        }

        Element e6 = adv.createElement("Uri", getUri());
        adv.appendChild(e6);

        Element e7 = adv.createElement("Provider", getProvider());
        adv.appendChild(e7);

        Element e8 = adv.createElement("Security", getSecurity());
        adv.appendChild(e8);

        Element e9 = adv.createElement("Code", getCode());
        adv.appendChild(e9);

        return adv;
    }
}



