/************************************************************************
 *
 * $Id: RdvManager.java,v 1.14 2001/05/23 01:00:21 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.rendezvous;

import net.jxta.peer.*;
import net.jxta.protocol.*;
import net.jxta.document.*;
import net.jxta.rendezvous.*;
import net.jxta.discovery.*;
import net.jxta.peergroup.*;
import net.jxta.impl.util.*;

/** 
 * This class implements a very simple RendezVous manager
 */

public class RdvManager implements RendezVousManager, JxtaTimerHandler {

    private boolean debug = false;
    private PeerGroup group = null;
    private String gId = null;
    private Discovery discovery = null;
    private static final long lease = 30 * 60 * 1000; // 30 minutes
    private JxtaTimer timer = null;
    public  static final long PollDelay = 60 * 60 * 1000; // 60 minutes

    public RdvManager (PeerGroup g) {

	debug = g.getDebug();
        group = g;
    }
    
    /**
     * This is called by the RendezVous service each time a new Peer
     * is requesting a connection. The RendezVous Manager returns zero if
     * the peer is not authorized to be connected, otherwise returns a lease
     * time (in milliseconds) for which the peer is authorized.
     */

    public long requestConnection (Advertisement adv) {

	debug ("requestConnection: got connection request:");
	debug ("    peer = " + ((PeerGroupAdvertisement) adv).getPid());
	debug ("   lease = " + (lease / 60) + " minutes");
	return lease;
    }


    /**
     * This implementation of Rendezvous Manager periodically pools peers and
     * groups advertisements from other peers. 
     */

    public void signal (JxtaTimer t) {

    }

    private void debug (String str) {
	if (debug) {
	    System.out.println ("[RdvManager] group= " + gId);
	    System.out.println ("            " + str);
	}
    }
}



