/************************************************************************
 *
 * $Id: RendezVousImpl.java,v 1.22 2001/05/23 01:00:21 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.rendezvous;

import java.io.*;
import java.util.*;
import java.net.*;

import net.jxta.peer.*;
import net.jxta.id.*;
import net.jxta.protocol.*; 
import net.jxta.rendezvous.*;
import net.jxta.endpoint.*;
import net.jxta.impl.endpoint.*;
import net.jxta.impl.peergroup.*;
import net.jxta.discovery.*;
import net.jxta.service.*;
import net.jxta.document.*;
import net.jxta.peergroup.*;
import net.jxta.util.*;
import net.jxta.impl.util.*;

/** 
 * This class implements the RendezVous service
 */

public class RendezVousImpl implements RendezVous, EndpointHandler {

    public static boolean debug = false;

    private Endpoint endpoint = null;
    private PeerGroup group = null;
    private String    gId = null;
    private Vector   rendezVous = null;
    private Vector   clients = null;
    private RendezVousManager manager = null;
    private RendezVousMonitor monitor = null;
    private boolean isRendezVous = false;
    private boolean isClient = false;
    private boolean endpointCreated = false;
    private  long bootTime = 0;
    private static final long ClientGCDelay = 60 * 60 * 1000; // 1 Hour

    private static final String pName = "JxtaRendezVous";
    private static final String ConnectRequest = "Connect";
    private static final String DisconnectRequest = "Disconnect";
    private static final String ConnectedPeerReply = "ConnectedPeer";
    private static final String ConnectedLeaseReply = "ConnectedLease";
    private static final String ConnectedRdvAdvReply = "RdvAdvReply";
    private static final String PingRequest = "PingRequest";
    private static final String PingReply = "PingReply";
    private static final String RdvAdvReply = "RdvAdv";
    
    private String pParam = null;
    private String localPeerId = null;
    private Advertisement localPeerAdv = null;

    private Discovery discovery = null;
    private JxtaTimer clientGCTimer = null;

    private ServiceAdvertisement advertisement = null;

    public RendezVousImpl () {
    }

    /**
     ** Inner class used to store peers and lease associated to them
     **/

    protected class PeerConnection {

	public String  peer = null;
	public long    lease = 0;
	public boolean isConnected; 

	public PeerConnection (String p, long l) {
	    peer = p;
	    if ((bootTime + l)  < 0) {
		// This is the case when the lease is infinite
		lease = l;
	    } else {
		lease = bootTime + l;
	    }
	    isConnected = true;
	}

	public boolean equals (Object obj) {
	    if (peer.equals ( ((PeerConnection) obj).peer)) {
		return true;
	    } else {
		return false;
	    }
	}
    }


    /**
     * Supply arguments and starts this service if it hadn't started by itself.
     *
     * Currently this service starts by itself and does not expect
     * arguments.
     *
     * @param arg A table of strings arguments.
     * @return int status indication.
     */
    public int startApp(String[] arg) {
        return 0;
    }

    /**
     * Ask this service to stop.
     *
     * This request is currently ignored.
     */
    public void stopApp() {
    }

    /**
     * Service objects are not manipulated directly to protect usage
     * of the service. A Service interface is returned to access the service
     * methods.
     *
     * @return Service public interface of the service
     */

    public Service getInterface() {
        return new RendezVousInterface(this);
    }

    /**
     * Returns the advertisement for that service.
     *
     * @return Advertisement the advertisement.
     *
     * @version $Revision: 1.22 $
     * @since JXTA 1.0
     */
    public Advertisement getAdvertisement() {
        return advertisement;
    }


    /**
     * Init routine
     *
     * @param myGroup The Group associated with this service
     */

    public void init (PeerGroup g, Advertisement sadv) {

	this.debug = g.getDebug();
        this.advertisement = (ServiceAdvertisement) sadv;
        this.group = g;
	this.gId = g.getID().toString();
        endpoint = ((RefPeerGroup) g).getEndpoint();
        discovery = group.getDiscovery();

	bootTime = System.currentTimeMillis();
	pParam = group.getID().toString();
	localPeerId =  group.getPeerID().toString();
	localPeerAdv = group.getAdvertisement();

	debug ("RendezVous is initialized");
    }

    /**
     ** This portion of the API is for the peers that are connected
     ** to a RendezVous peer.
     **/

    /**
     * Add a peer as a new RendezVous point.
     *
     * @param adv the advertisement of the RendezVous peer
     * @param timeout the time to wait to establish the connection
     * @throws IOException when the RendezVous peer is not reachable
     *
     */
    public void connectRendezVous (PeerGroupAdvertisement adv) 
	throws IOException {

	debug ("connectRendezVous with advertisement");
	if (adv == null) {
	    // Sanity check
	    return;
	}

	String peerId = adv.getPid();

	// Try to get an EndpointMessanger to that peer.

	// First publish the advertisement
	if (discovery != null) {
	    try {
		discovery.publish ( adv, Discovery.PEER);
	    } catch (Exception e) {
		debug ("   publish failed with " + e);
	    }
	}

	// Make sure that Endpoint has also the document
	endpoint.publishPeerAdv (adv);

	// Check that the Endpoint can talk to that peer
	if ( !endpoint.addPeerRoute (peerId) ) {
	    throw new IOException();
	}

	EndpointMessanger messanger = null;
	try {
	    messanger = endpoint.getMessanger ( peerId,
						pName,
						pParam,
						Endpoint.NonBlocking);
	    if (messanger == null) {
		throw new IOException();
	    }
	} catch (Exception e) {
		throw new IOException();
	}
	
        connectToRendezVous (messanger);
	return;
    }

    public void connectRendezVous (EndpointAddress addr) 
	throws IOException {

	debug ("connectRendezVous with EndpointAddress");
	if (addr == null) {
	    // Sanity check
	    return;
	}
	
	try {
	    debug ("    to = " + addr.getProtocolName() + "://" + addr.getProtocolAddress());
	} catch (Exception e) {
	}

	String proto = addr.getProtocolName();
	EndpointMessanger messanger = null;

	if (proto.equals("jxta")) {
	    // The rendezvous is a peerid, not a transport address
	    try {
		messanger = endpoint.getMessanger ( "jxta://" + addr.getProtocolAddress(),
						    pName,
						    pParam,
						    Endpoint.NonBlocking);
		if (messanger == null) {
		    throw new IOException();
		}
	    } catch (Exception e) {
		debug ("connecteRendezVous failed " + e);
	    }

	} else {
	    // Check if this address is reachable
	    if (!endpoint.ping (addr)) {
		throw new IOException ();
	    }

	    addr.setServiceName (pName);
	    addr.setServiceParameter (pParam);

	    try {
		messanger = endpoint.getMessanger ( addr,
						    Endpoint.NonBlocking);
		if (messanger == null) {
		    debug ("   no messanger");
		    throw new IOException();
		}
	    } catch (Exception e) {
		debug ("   connectRendezVous failed with " + e);
		throw new IOException();
	    }
	}
	
        connectToRendezVous (messanger);
	return;
    }


    public void reconnectRendezVous (String peer) {

	debug ("reconnectRendezVous to " + peer);
	EndpointMessanger messanger = null;
	try {
	    messanger = endpoint.getMessanger ( peer,
						pName,
						pParam,
						Endpoint.NonBlocking);
	    if (messanger == null) {
		throw new IOException();
	    }

	    connectToRendezVous (messanger);
	} catch (Exception e) {
	    debug ("reconnecteRendezVous failed " + e);
	}
	return;
    }


    /**
     * Remove a RendezVous point.
     *
     * @param rendezVous the PeerId of the RendezVous point to remove
     *
     * @throws IOException if rendezVous is not connected.
     */
    public void removeRendezVous (PeerID rdv) {

	if (!isClient || (rdv == null)) {
	    // Sanity check
	    return;
	}
	
	String peerId = rdv.toString();

	// First try to get an EndpointMessanger to that peer.
	EndpointMessanger messanger = null;
	try {
	    messanger = endpoint.getMessanger ( peerId,
						pName,
						pParam,
						Endpoint.NonBlocking);
	    if (messanger != null) {
		disconnectRendezVous (messanger);
	    }
	} catch (Exception e) {
	}

	removeRdv (peerId);

	if (rendezVous.size() == 0) {
	    // No more rendezVous
	    rendezVous = null;
	    isClient = false;
	}
	return;
    }


    /**
     * Register a notification handler that is called each time a RendezVous
     * peer is not reachable anymore.
     *
     * @param handler a handler that is called each time a RendezVous peer is
     * is not reachable anymore.
     * @return RendezVousMonitor returns, if any, the current RendezVousMonitor
     */
    public synchronized RendezVousMonitor setMonitor (RendezVousMonitor handler) {

	RendezVousMonitor old = monitor;
	monitor = handler;
	return old;
    }


    // XXX: lease is not implemented yet. To be fixed. lomax@jxta.org
    private synchronized void addRdv (String peer, long lease) {
	
	if (peer == null) {
	    return;
	}

	if (!isClient) {
	    // This is the first rendezVous
	    rendezVous = new Vector();
	    isClient = true;
	}

	PeerConnection tmp = new PeerConnection (peer, lease);
	    
	// Check if the peer is already registered.
	if (rendezVous.contains (tmp)) {
	    return;
	}

	rendezVous.addElement (tmp);
	return;
    }

    private synchronized void removeRdv (String peer) {

	if (peer == null) {
	    return;
	}

	PeerConnection tmp = new PeerConnection (peer, 0);
	    
	// Check if the peer is already registered.
	if (!rendezVous.contains (tmp)) {
	    return;
	}

	rendezVous.removeElement (tmp);
	return;
    }


    /**
     * Returns an Enumeration of the PeerID all the RendezVous on which this
     * Peer is currentely connected.
     */

    public synchronized Enumeration listRendezVous () {

	debug ("listRendezVous");
	if (!isClient) {
	    // Sanity check
	    return null;
	}
	
	if ((rendezVous == null) || (rendezVous.size() == 0)) {
	    return null;
	}

	Vector tmp = new Vector();
	String peerId = null;
	PeerConnection pConn = null;
	ID id = null;

	for (int i = 0; i < rendezVous.size(); ++i) {
	    try {
		pConn = (PeerConnection) rendezVous.elementAt (i);
		peerId = pConn.peer;
		id = IDFactory.fromURL (new URL (peerId));
		if (id != null) {
		    tmp.addElement (id);
		}
	    } catch (Exception e) {
	    }
	}
	if (tmp.size() > 0) {
	    return tmp.elements();
	} else {
	    return null;
	}
    }


    
    /**
     ** This portion is for peers that are RendezVous
     **/

    // XXX: lease is not implemented yet. To be fixed. lomax@jxta.org
    private synchronized void addClient (String peer, long lease) {
	
	debug ("addClient starts");
	if (peer == null) {
	    debug ("no peer");
	    return;
	}

	if (!isRendezVous) {
	    debug ("is not a rendezvous");
	    return;
	}

	PeerConnection tmp = new PeerConnection (peer, lease);
	    
	// Check if the peer is already registered.
	if (clients.contains (tmp)) {
	    debug ("   already registered");
	    // Already connected, just upgrade the lease
	    try {
		int index = clients.indexOf (tmp);
		if (index == -1) {
		    // This should not happen
		    debug ("addClient: cannot access PeerConnection for " + peer);
		} else {
		    tmp = (PeerConnection) clients.elementAt (index);
		    long time =  System.currentTimeMillis();
		    if ((time + lease) < 0) {
			// Forever
			tmp.lease = lease;
		    } else {
			tmp.lease = time + lease;
		    }
		    tmp.isConnected = true;
		    debug ("Renewed leased for " + peer);
		}
	    } catch (Exception e) {
		debug ("addClient has failed [1] " + e);
	    }
	    return;
	}

	clients.addElement (tmp);
	debug ("   done");
	return;
    }

    private synchronized void removeClient (String peer) {

	if (peer == null) {
	    return;
	}

	PeerConnection tmp = new PeerConnection (peer, 0);
	    
	// Check if the peer is already registered.
	if (!clients.contains (tmp)) {
	    return;
	}

	clients.removeElement (tmp);
	return;
    }

    /**
     * Sends advertisement about other RendezVous to a given peer
     *
     * @param destPeer is the advertisement of the peer to which to
     * send the RendezVous advertisement.
     * @param rendezVous is the advertisment of a RendezVous peer
     */

    public void sendRendezVousAdv (PeerAdvertisement destPeer,
                                   PeerAdvertisement rdv) {

	// Try to get an EndpointMessanger to that peer.
	// First publish the advertisement
	if (discovery != null) {
	    try {
		discovery.publish ( destPeer, Discovery.PEER);
	    } catch (Exception e) {
	    }
	}

	// Make sure that Endpoint has also the document
	endpoint.publishPeerAdv (destPeer);

	String peer = destPeer.getPid();

	// Check that the Endpoint can talk to that peer
	if ( !endpoint.addPeerRoute (peer) ) {
	    return;
	}

	EndpointMessanger messanger = null;
	try {
	    messanger = endpoint.getMessanger ( peer,
						pName,
						pParam,
						Endpoint.NonBlocking);
	    if (messanger == null) {
		return;
	    }
	} catch (Exception e) {
	    return;
	}
	
	MessageImpl msg = new MessageImpl();

	try {
	    
	    msg.push (RdvAdvReply, rdv.getDocument(
				 new MimeMediaType ("text/xml")).getStream());

	    messanger.sendMessage (msg);
	} catch (Exception e) {
	    return;
	}
    }


    /**
     * Start the local peer as a RendezVous peer. The caller must provide
     * a handler to an authorization manager.
     *
     * @param handler is the RendezVousManager that will be invoked each
     * time a new Peer request to be connected.
     *
     * @throws IOException when a handler has already been connected
     */
    public synchronized void startRendezVous (RendezVousManager handler)
	throws IOException {

	if ((isRendezVous) || (handler == null)) {
	    // We are already a rendez vous. Throw an exception
	    debug ("startRendezvous: already a rendezvous");
	    throw new IOException();
	}

	manager = handler;
	clients = new Vector();

	createLocalEndpoint();

	isRendezVous = true;

	// Start the clients garbage collector
	clientGCTimer = new JxtaTimer (new ClientGC(this), ClientGCDelay, true);

	debug ("RendezVous is started");
	return;
    }


    /**
     * Stop the RendezVous function on the local Peer. All connected Peer are
     * disconnected.
     *
     * @throws IOException if the local peer was not a RendezVous peer.
     */
    public synchronized void stopRendezVous () {

	if (!isRendezVous) {
	    return;
	}
	isRendezVous = false;

	deleteLocalEndpoint();
	
	clients = null;
    }


    /**
     * Returns an Enumeration of PeerID of the peers that are currentely
     * connected.
     */
    public synchronized Enumeration getConnectedPeers() {

	if (!isRendezVous) {
	    // Sanity check
	    return null;
	}
	
	if ((clients == null) || (clients.size() == 0)) {
	    return null;
	}

	Vector tmp = new Vector();
	String peerId = null;
	ID id = null;
	PeerConnection pConn = null;

	for (int i = 0; i < clients.size(); ++i) {
	    try {
		pConn = (PeerConnection) clients.elementAt (i);
		peerId = pConn.peer;
		id = IDFactory.fromURL (new URL (peerId));
		if (id != null) {
		    tmp.addElement (id);
		}
	    } catch (Exception e) {
	    }
	}
	if (tmp.size() > 0) {
	    return tmp.elements();
	} else {
	    return null;
	}
    }



    /**
     ** This portion of the code is common to both RendezVous and Clients
     **
     ** It defines the protocol between both ends
     **/

    private void createLocalEndpoint () throws IOException {

	if (endpointCreated) {
	    return;
	}
	// Enable our Endpoint receiving end
	endpoint.registerHandler (pName + pParam, this);
	endpointCreated = true;
    }


    private void deleteLocalEndpoint () {

	if (!endpointCreated) {
	    return;
	}
	endpoint.unregisterHandler (pName + pParam, this);
	endpointCreated = false;
    }


    private void connectToRendezVous (EndpointMessanger messanger)
	throws IOException {

	debug ("connectToRendezVous");
	if (messanger == null) {
	    debug ("   messanger is null");
	    throw new IOException ();
	}

	createLocalEndpoint();

	MessageImpl msg = new MessageImpl();
	// The request simply includes the local peer advertisement.

	try {
	    InputStream ip = null;
	    Document doc = null;
	    doc = localPeerAdv.getDocument(new MimeMediaType ("text/xml"));
	    ip = doc.getStream();

            msg.push (ConnectRequest, ip);

	    messanger.sendMessage (msg);

	} catch (Exception e) {
	    debug ("   failed with " + e);
	    throw new IOException ();
	}
    }

    private void disconnectRendezVous (EndpointMessanger messanger)
	throws IOException {

	if (messanger == null) {
	    throw new IOException ();
	}

	MessageImpl msg = new MessageImpl();
	// The request simply includes the local peer advertisement.

	try {
	    InputStream ip = null;
	    Document doc = null;
	    doc = localPeerAdv.getDocument(new MimeMediaType ("text/xml"));
	    ip = doc.getStream();

	    msg.push (DisconnectRequest, ip);

	    messanger.sendMessage (msg);

	} catch (Exception e) {
	    throw new IOException ();
	}
    }


    public void demux (MessageImpl msg) {

	debug ("demux");
	Enumeration enum = msg.getNames();
	if ((enum == null) || (!enum.hasMoreElements())) {
	    return;
	}

	String tag = null;
	while (enum.hasMoreElements()) {
	    try {
		tag = (String) enum.nextElement();
		if (tag.equals (ConnectRequest)) {
		    processConnectRequest (msg);
		    continue;
		}

		if (tag.equals (DisconnectRequest)) {
		    processDisconnectRequest (msg);
		    continue;
		}

		if (tag.equals (ConnectedPeerReply)) {
		    processConnectedReply (msg);
		    continue;
		}

		if (tag.equals (RdvAdvReply)) {
		    processRdvAdvReply (msg);
		    continue;
		}

		if (tag.equals (PingRequest)) {
		    processPingRequest (msg);
		    continue;
		}

		if (tag.equals (PingReply)) {
		    processPingReply (msg);
		    continue;
		}
	    } catch (Exception e) {
		break;
	    }
	}
	return;
    }

    private void processConnectRequest (Message msg) {

	debug ("processConnectRequest");

	if ((!isRendezVous) || (manager == null)) {
	    debug ("   not a rendezvous");
	    return;
	}

	InputStream ip = null;
	try {
	    ip = msg.pop (ConnectRequest);
	} catch (Exception e) {
	    debug ("   cannot get tag " + e);
	    return;
	}
	if (ip == null) {
	    debug ("   no InputStream for that tag");
	    return;
	}

	PeerGroupAdvertisement adv = null;
	try {
	    adv = (PeerGroupAdvertisement) 
		AdvertisementFactory.newAdvertisement (new MimeMediaType("text/xml"),
						       ip);

	} catch (Exception e) {
	    debug ("   cannot retrieve advertisment from request");
	    return;
	}

	if (adv == null) {
	    debug ("   advertisement is null");
	    return;
	}

	long lease = manager.requestConnection (adv);
	
	// Send the reply
	if (!sendReply (adv.getPid(), adv, lease)) {
	    debug ("processConnectRequest. Cannot reach client " + adv.getPid());
	    // The client is not reachable. Just drop it
	    return;
	}

	if (lease != 0) {
	    debug ("  addClient");
	    addClient (adv.getPid(), lease);
	}

	return;
    }

    private void processDisconnectRequest (Message msg) {

	if ((!isRendezVous) || (manager == null)) {
	    return;
	}

	InputStream ip = null;
	try {
	    ip = msg.pop (DisconnectRequest);
	} catch (Exception e) {
	    return;
	}
	if (ip == null) {
	    return;
	}

	PeerGroupAdvertisement adv = null;
	try {
	    adv = (PeerGroupAdvertisement) 
		AdvertisementFactory.newAdvertisement (new MimeMediaType ("text/xml"),
						       ip);
	} catch (Exception e) {
	    return;
	}

	if (adv == null) {
	    return;
	}

	removeClient (adv.getPid());
	return;
    }

    private void processConnectedReply (Message msg) {

	debug ("processConnectedReply");
	InputStream ip = null;
	try {
	    ip = msg.pop (ConnectedRdvAdvReply);
	} catch (Exception e) {
	    debug ("   rdv adv failed with " + e);
	}
	if (ip == null) {
	    debug ("   no rendezvous adv");
	} else {

	    Advertisement adv = null;
	    try {
		adv = AdvertisementFactory.newAdvertisement (new MimeMediaType ("text/xml"),
							     ip);
	    } catch (Exception e) {
		debug ("   no advertisement");
	    }

	    if (adv == null) {
		debug ("   no advertisement");
	    } else {
		// Make sure that Endpoint has also the document
		endpoint.publishPeerAdv (adv);

		if (discovery != null) {
		    try {
			discovery.publish ( adv, Discovery.PEER);
		    } catch (Exception e) {
			debug ("   cannot get DiscoveryService");
		    }
		}
	    }
	}

	ip = null;
	try {
	    ip = msg.pop (ConnectedLeaseReply);
	} catch (Exception e) {
	    debug ("   no lease " + e);
	    return;
	}
	if (ip == null) {
	    debug ("   no lease");
	    return;
	}

	long lease = 0;
	try {
	    byte[] buffer = new byte [ip.available()];
	    ip.read (buffer);
	    lease = Long.parseLong (new String (buffer));
	} catch (Exception e) {
	    debug (" leased failed with " + e);
	    return;
	}

	try {
	    ip = msg.pop (ConnectedPeerReply);
	} catch (Exception e) {
	    debug ("    No peer " + e);
	    return;
	}
	if (ip == null) {
	    debug ("   No peer ");
	    return;
	}

	String peer = null;
	try {
	    byte[] buffer = new byte [ip.available()];
	    ip.read (buffer);
	    peer = new String (buffer);
	} catch (Exception e) {
	    debug ("  getting peer failed with " + e);
	    return;
	}

	// Check that we can talk to that peer (we should be able to)
	if ( !endpoint.addPeerRoute (peer) ) {
	    debug ("   cannot find route to RendezVous");
	    debug ("   This is really not normal");
	    return;
	}

	PeerID pId = null;
	try {
	    pId = (PeerID) IDFactory.fromURL (new URL (peer));
	} catch (Exception e) {
	    debug ("   getting peer advertisement failed with " + e);
	    return;
	}

	if (lease == 0) {
	    if (monitor != null) {
		monitor.lostConnection (pId);
	    }
	} else {
	    addRdv (peer, lease);
	    if (monitor != null) {
		monitor.isConnected (pId, lease);
	    }
	}
	return;
    }


    private void processRdvAdvReply (Message msg) {

	if (monitor == null) {
	    return;
	}

	InputStream ip = null;
	try {
	    ip = msg.pop (RdvAdvReply);
	} catch (Exception e) {
	    return;
	}
	if (ip == null) {
	    return;
	}

	Advertisement adv = null;
	try {
	    adv = AdvertisementFactory.newAdvertisement (new MimeMediaType ("text/xml"),
									    ip);
	} catch (Exception e) {
	    return;
	}

	if (adv == null) {
	    return;
	}

	monitor.newRendezVous (adv);
	return;
    }


    private boolean sendReply (String peer, Advertisement adv, long lease) {

	localPeerAdv = group.getAdvertisement();

	debug ("sendReply");
	// Try to get an EndpointMessanger to that peer.
	// First publish the advertisement
	if (discovery != null) {
	    try {
		discovery.publish ( adv, Discovery.PEER);
	    } catch (Exception e) {
		debug ("   cannot get DiscoveryService");
	    }
	}
	// Make sure that Endpoint has also the document
	endpoint.publishPeerAdv (adv);

	// Check that the Endpoint can talk to that peer
	if ( !endpoint.addPeerRoute (peer) ) {
	    debug ("   cannot get route to peer " + peer);
	    return false;
	}

	EndpointMessanger messanger = null;
	try {
	    messanger = endpoint.getMessanger ( peer,
						pName,
						pParam,
						Endpoint.NonBlocking);
	    if (messanger == null) {
		debug ("    no messanger");
		return false;
	    }
	} catch (Exception e) {
	    debug ("   getting messanger failed with " + e);
	    return false;
	}
	
	MessageImpl msg = new MessageImpl();

	ByteArrayInputStream ipPeer =
	    new ByteArrayInputStream (localPeerId.getBytes());
	ByteArrayInputStream ipLease =
	    new ByteArrayInputStream (Long.toString (lease).getBytes());

	try {
	    InputStream ip = null;
	    Document doc = null;
	    doc = localPeerAdv.getDocument(new MimeMediaType ("text/xml"));
	    ip = doc.getStream();

            msg.push (ConnectedRdvAdvReply, ip);

	    msg.push (ConnectedPeerReply, ipPeer);
	    msg.push (ConnectedLeaseReply, ipLease);

	    messanger.sendMessage (msg);
	    debug ("   message sent");
	} catch (Exception e) {
	    debug ("  failed with " + e);
	    return false;
	}
	return true;
    }

    /**
     * This API is provided to the Core implementation in order to
     * propagate a message onto all the clients.
     *
     * @param msg is the message to propagate.
     * @param serviceName is the name of the service
     * @param serviceParam is the parameter of the service
     * @param prunePeer is a peer to prune in the propagation.
     */
    public void propagateToClients (Message msg,
				    String serviceName,
				    String serviceParam,
				    String prunePeer) {

	debug ("propagateToClients");
	if ((!isRendezVous) || (clients == null) || (clients.size() == 0)) {
	    debug ("  no clients");
	    return;
	}

	PeerConnection pConn = null;
	EndpointMessanger messanger = null;
	Message tmpMsg = null;

        // First build the destination and source addresses
	Address mAddress = new Address ();
        mAddress.setServiceName (serviceName);
        mAddress.setServiceParameter (serviceParam);

        ((MessageImpl) msg).setDestAddress (mAddress);
	

	for (int i = 0; i < clients.size(); ++i) {
	    try {
		pConn = (PeerConnection) clients.elementAt (i);
		if ((prunePeer != null) && pConn.peer.equals (prunePeer)) {
		    debug ("   pruning");
		    continue;
		}
		messanger = endpoint.getMessanger ( pConn.peer,
						    serviceName,
						    serviceParam,
						    Endpoint.NonBlocking);
		if (messanger == null) {
		    debug ("   no messanger");
		    return;
		}
		
		tmpMsg = msg.dup();
		((MessageImpl) tmpMsg).setSrcAddress (((MessageImpl) msg).getSrcAddress());
		((MessageImpl) tmpMsg).setDestAddress (mAddress);
		messanger.sendMessage (tmpMsg);
		
	    } catch (Exception e) {
		debug ("   failed with " + e);
	    }
	}
	debug ("   done");
    }

    /**
     * This API is provided to the Core implementation in order to
     * propagate a message onto all the rendez vous.
     *
     * @param msg is the message to propagate.
     * @param serviceName is the name of the service
     * @param serviceParam is the parameter of the service
     * @param prunePeer is a peer to prune in the propagation.
     */
    public void propagateToRendezVous (Message msg,
				       String serviceName,
				       String serviceParam,
				       String prunePeer) {

	debug ("propagateToRendezVous");
	if ((!isClient) || (rendezVous == null) || (rendezVous.size() == 0)) {
	    debug ("  no rendezvous");
	    return;
	}

        // First build the destination and source addresses
	Address mAddress = new Address ();
        mAddress.setServiceName (serviceName);
        mAddress.setServiceParameter (serviceParam);

        ((MessageImpl) msg).setDestAddress (mAddress);
	
	PeerConnection pConn = null;
	EndpointMessanger messanger = null;
	Message tmpMsg = null;

	debug ("    Sending to all rendezvous");
	for (int i = 0; i < rendezVous.size(); ++i) {
	    try {
		pConn = (PeerConnection) rendezVous.elementAt (i);
		if ((prunePeer != null) && pConn.peer.equals (prunePeer)) {
		    debug ("   pruning");
		    continue;
		}
		debug ("    getting messanger for " + pConn.peer);
		messanger = endpoint.getMessanger ( pConn.peer,
						    serviceName,
						    serviceParam,
						    Endpoint.NonBlocking);
		if (messanger == null) {
		    debug ("   no messanger");
		    return;
		}
		
		tmpMsg = msg.dup();
		((MessageImpl) tmpMsg).setDestAddress (mAddress);
		messanger.sendMessage (tmpMsg);
		
	    } catch (Exception e) {
		debug ("   failed with " + e);
	    }
	}
	debug ("     done");
    }

    private void ping (String peer) {

	debug ("ping " + peer);
	EndpointMessanger messanger = null;
	try {
	    messanger = endpoint.getMessanger ( peer,
						pName,
						pParam,
						Endpoint.NonBlocking);
	    if (messanger == null) {
		debug ("    no messanger");
		return;
	    }
	} catch (Exception e) {
	    debug ("   getting messanger failed with " + e);
	    return;
	}
	
	MessageImpl msg = new MessageImpl();

	ByteArrayInputStream pingMsgIp =
	    new ByteArrayInputStream (localPeerId.getBytes());

	try {
	    msg.push (PingRequest, pingMsgIp);

	    messanger.sendMessage (msg);
	    debug ("   message sent");
	} catch (Exception e) {
	    debug ("  failed with " + e);
	}
	return;
    }


    private synchronized void gcClients() {

	debug ("GC CLIENTS starts");
	if (clients == null) {
	    return;
	}
	PeerConnection pConn = null;
	long time = System.currentTimeMillis();

	for (int i = 0; i < clients.size(); ++i) {
	  
	    try {
		pConn = (PeerConnection) clients.elementAt (i);
		if ((!pConn.isConnected) || (pConn.lease < time)){
		    // This client has dropped out. Remove it.
		    // or the leased is over.

		    debug ("GC CLIENT: drop " + pConn.peer);
		    debug ("     lease= " + pConn.lease);
		    debug (" connected= " + pConn.isConnected);

		    clients.removeElementAt (i);
		    // Decrement i because the index has changed
		    --i;
		    continue;
		}
		// Mark the client as disconnected.
		pConn.isConnected = false;
		// Ping the client
		debug ("GC CLIENT pinging " + pConn.peer);
		ping (pConn.peer);
		continue;
	    } catch (Exception e) {
		debug("ping failed " + e);
	    }
	}
	debug ("GC CLIENTS done");
    }

    private synchronized void clientIsConnected (String peer) {

	if (clients == null) {
	    return;
	}
	PeerConnection pConn = null;

	for (int i = 0; i < clients.size(); ++i) {
	  
	    try {
		pConn = (PeerConnection) clients.elementAt (i);
		if (pConn.peer.equals (peer)) {
		    pConn.isConnected = true;
		    return;
		}
	    } catch (Exception e) {
	    }
	}
	return;
    }

    private void processPingReply (Message msg) {

	InputStream ip = null;
	try {
	    ip = msg.pop (PingReply);
	    if (ip == null) {
		return;
	    }
	    byte[] buffer = new byte [ip.available()];
	    ip.read (buffer);
	    String peer =  new String (buffer);
	    clientIsConnected (peer);

	} catch (Exception e) {
	}
    }


    private void processPingRequest (Message msg) {

	EndpointMessanger messanger = null;
	InputStream ip = null;
	try {
	    ip = msg.pop (PingRequest);
	    if (ip == null) {
		return;
	    }
	    byte[] buffer = new byte [ip.available()];
	    ip.read (buffer);
	    String peer =  new String (buffer);
	    // Send a reply
	    try {
		messanger = endpoint.getMessanger ( peer,
						    pName,
						    pParam,
						    Endpoint.NonBlocking);
		if (messanger == null) {
		    debug ("    no messanger");
		    return;
		}
	    } catch (Exception e1) {
		debug ("   getting messanger failed with " + e1);
		return;
	    }
	
	    msg = new MessageImpl();

	    ByteArrayInputStream pingMsgIp =
		new ByteArrayInputStream (localPeerId.getBytes());

	    try {
		msg.push (PingReply, pingMsgIp);

		messanger.sendMessage (msg);
		debug ("   message sent");
	    } catch (Exception e1) {
		debug ("  failed with " + e1);
	    }
	    return;
	} catch (Exception e) {
	}
    }

    public class ClientGC implements JxtaTimerHandler {

	RendezVousImpl rendezvous = null;
	
	public ClientGC (RendezVousImpl r) {
	    rendezvous = r;
	}

	public void signal (JxtaTimer timer) {

	    // First scans all the client to see if there are still
	    // connected (or if their lease is still valid);
	    rendezvous.gcClients();
	}
    }



    private void debug (String str) {
	if (debug) {
	    System.out.println("[RendezVous] group= " + gId);
	    System.out.println("             " + str);
	}
    }
}



