/************************************************************************
 *
 * $Id: RendezVousInterface.java,v 1.3 2001/05/23 01:00:21 lomax Exp $
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *********************************************************************************/

package net.jxta.impl.rendezvous;

import java.util.*;
import java.io.*;
import java.net.*;

import net.jxta.rendezvous.*;
import net.jxta.peergroup.*;
import net.jxta.protocol.*;
import net.jxta.endpoint.*;
import net.jxta.service.*;
import net.jxta.peer.*;
import net.jxta.document.Advertisement;
import net.jxta.exception.PeerGroupException;




/**
 * This class implements the RendezVous interface.
 */

public class RendezVousInterface implements RendezVous {

  RendezVousImpl impl;

    /**
     * The only authorized constructor.
     */
  public RendezVousInterface(RendezVousImpl theRealThing) {
      impl = theRealThing;
  }

    /**
     * returns an interface object that permits to use this
     * service without having access to the real object.
     *
     * Since THIS is already such an object, it returns itself.
     * FIXME: it is kind of absurd to have this method part of the
     * interface but we do not want to define two levels of Service interface
     * just for that.
     * @return Resolver An interface object that implements
     * this service and nothing more.
     */
  public Service getInterface() {
	return this;
  }

    /**
     * Returns the advertisment for that service.
     *
     * @return Advertisement the advertisement.
     *
     * @version $Revision: 1.3 $
     * @since JXTA 1.0 
     */
    public Advertisement getAdvertisement() {
        return impl.getAdvertisement();
    }

    /**
     *  Initialize the application
     *  FIXME: This is meaningless for the interface object;
     * it is there only to satisfy the requirements of the
     * interface that we implement. Ultimately, the API should define
     * two levels of interfaces: one for the real service implementation
     * and one for the interface object. Right now it feels a bit heavy
     * to so that since the only different between the two would be
     * init() and may-be getName().
     * @param g PeerGroup this application is started from
     * @since   JXTA 1.0
     */

    public void init(PeerGroup pg, Advertisement a) {
    }


    /**
     * This is here for temporary class hierarchy reasons.
     * it is ALWAYS ignored. By definition, the interface object
     * protects the real object's start/stop methods from being called
     *
     * @param arg A table of strings arguments.
     * @return int status indication.
     */
    public int startApp(String[] arg) {
        return 0;
    }

    /**
     * This is here for temporary class hierarchy reasons.
     * it is ALWAYS ignored. By definition, the interface object
     * protects the real object's start/stop methods from being called
     *
     * This request is currently ignored.
     */
    public void stopApp() {
    }


    /**
     * Add a peer as a new RendezVous point.
     * If/When the RendezVous accepts the connection, the RendezVous
     * service will invoke the RendezVousMonitor.
     *
     * @param adv the advertisement of the RendezVous peer
     * @exception IOException when the RendezVous peer is not reachable
     */

    public void connectRendezVous (PeerGroupAdvertisement adv) throws IOException {
	impl.connectRendezVous (adv);
    }

    /**
     * Add a peer as a new RendezVous point.
     * If/When the RendezVous accepts the connection, the RendezVous
     * service will invoke the RendezVousMonitor.
     *
     * @param addr EndpointAddress of the rendezvous peer
     * @exception IOException when the RendezVous peer is not reachable
     */

    public void connectRendezVous (EndpointAddress addr) throws IOException {
	impl.connectRendezVous (addr);
    }


    /**
     * Remove a RendezVous point.
     *
     * @param rendezVous the PeerId of the RendezVous point to remove
     */

    public void removeRendezVous (PeerID rendezVous) {
	impl.removeRendezVous (rendezVous);
    }

    /**
     * Register a notification handler that is called each time a RendezVous
     * peer is not reachable anymore.
     *
     * @param handler a handler that is called each time a RendezVous peer is
     * is not reachable anymore.
     * @return RendezVousMonitor returns, if any, the current RendezVousMonitor
     */

    public RendezVousMonitor setMonitor (RendezVousMonitor handler) {
	return impl.setMonitor (handler);
    }

    /**
     * Returns an Enumeration of the PeerID all the RendezVous on which this
     * Peer is currentely connected.
     *
     * @return Enumeration enumeration of RendezVous
     */

    public Enumeration listRendezVous () {
	return impl.listRendezVous();
    }

    /**
     ** This portion is for peers that are RendezVous
     **/

    /**
     * Start the local peer as a RendezVous peer. The caller must provide
     * a handler to an authorization manager.
     *
     * @param handler is the RendezVousManager that will be invoked each
     * time a new Peer request to be connected.
     *
     * @exception IOException when a handler has already been connected
     */

    public void startRendezVous (RendezVousManager handler) throws IOException {
	impl.startRendezVous (handler);
    }


    /**
     * Stop the RendezVous function on the local Peer. All connected Peer are
     * disconnected.
     */

    public void stopRendezVous () {
	impl.stopRendezVous();
    }

    /**
     * Returns an Enumeration of PeerID of the peers that are currentely
     * connected.
     *
     * @return Enumeration enumeration of peers connected to that rendezvous
     */

    public Enumeration getConnectedPeers() {
	return impl.getConnectedPeers();
    }

    /**
     * Sends advertisement about other RendezVous to a given peer
     *
     * @param destPeer is the advertisement of the peer to which to
     * send the RendezVous advertisement.
     * @param rendezVous is the advertisment of a RendezVous peer
     */

    public void sendRendezVousAdv (PeerAdvertisement destPeer,
                                   PeerAdvertisement rendezVous) {
	impl.sendRendezVousAdv (destPeer, rendezVous);
    }
}
