/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaTimer.java,v 1.4 2001/05/23 01:00:21 lomax Exp $
 */

package net.jxta.impl.util;

/**
 * This class implements a simple timer utility.
 *
 * Note that this class is provided only because java.util.Timer was
 * introduced in Java 1.3, and Jxta implementation must be able to run
 * on older Java version (1.1.4).
 *
 * @see net.jxta.util.JxtaTimerHandler
 *
 * @version $Revision: 1.4 $ 
 * @since JXTA 1.0 
 */

public class JxtaTimer implements Runnable {

    private JxtaTimerHandler handler = null;
    private Thread thread = null;
    private long delay = 0;
    private boolean repeat = false;

    /**
     * Timer constructor
     *
     * @param handler is a handler invoked when the timer goes off
     * @param delay is the delay in milliseconds before the timer goes off
     * @param repeat is set to false if the timer is supposes to go off only once
     *  and then die, or set to true if the timer is supposed to go off several time
     *  until it is cancelled.
     *
     * @version $Revision: 1.4 $ 
     * @since JXTA 1.0 
     */      

    public JxtaTimer (JxtaTimerHandler handler, long delay, boolean repeat) {

	this.handler = handler;
	this.delay = delay;
	this.repeat = repeat;

	thread = new Thread (this, "JxtaTimerThread delay= " + delay);
	thread.start();
	Thread.yield();
    }

    /**
     * cancel a pending timer. The timer is destroyed, and the handler is not called.
     *
     * @version $Revision: 1.4 $ 
     * @since JXTA 1.0 
     */

    public void cancel () {

	thread.interrupt();
    }

    /**
     * run method to start the thread timer
     *
     * @version $Revision: 1.4 $ 
     * @since JXTA 1.0 
     */

    public void run () {

	while (true) {
	    try {
		Thread.sleep (delay);
		handler.signal (this);
		if (!repeat) {
		    break;
		}
	    } catch (Exception e) {
		// Cancelled timer
		break;
	    }
	}
    }
}


