/*
 * $Id: DocTest.java,v 1.5 2001/06/08 18:20:13 bondolo Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

import java.io.Writer;
import java.io.StringWriter;
import java.util.Enumeration;

import net.jxta.document.*;
import net.jxta.impl.document.*;

/**
 *
 * @author  mike
 * @version
 */
public final class DocTest {
    
    /** Creates new DocTest */
    public DocTest() {
    }
    
    private String TestStructuredTextDoc( MimeMediaType type ) {
        try {
            final String useDocType = "Test";
            StructuredTextDocument doc = null;
            try {
                doc = (StructuredTextDocument) StructuredDocumentFactory.newStructuredDocument( type, useDocType );
            }
            catch( Throwable thrown ) {
                thrown.printStackTrace();
                return( "Failed : exception thrown during construction!" );
            }
            
            if( null == doc )
                return( "Failed : could not construct object for type : " + type );
            
            String itsType = doc.getName();
            
            if( !useDocType.equals( itsType ) )
                return( "Failed : returned doctype does not equal type document was created with!" );
            
            if( !doc.getName().equals( itsType ) )
                return( "Failed : returned doctype does not equal name of document element" );
            
            TextElement testElement = doc.createElement( "element" );
            
            doc.appendChild( testElement );
            
            try {
                Element firstchild = (Element) doc.getChildren().nextElement();
                
                if( !testElement.equals( firstchild ) )
                    return( "Failed : added a single element, but something else was returned" );
            }
            catch( Exception e ) {
                e.printStackTrace();
                return( "Failed : added a single element, but it was not returned" );
            }
            
            final String useName = "element2";
            final String useValue = "value&<!";
            
            TextElement testElement2 = doc.createElement( useName, useValue );
            
            testElement.appendChild( testElement2 );
            
            String itsName = testElement2.getName();
            if( !useName.equals(itsName) )
                return( "Failed : name of element was not correct after creation" );
            
            String itsValue = testElement2.getTextValue();
            if( !useValue.equals(itsValue) )
                return( "Failed : value of element was not correct after creation. was '" + itsValue + "' should be '" + useValue + "'" );
            
            testElement2 = doc.createElement( "element3", useValue );
            
            testElement.appendChild( testElement2 );
            
            testElement2 = doc.createElement( "element4", "1" );
            
            testElement.appendChild( testElement2 );

            itsValue = testElement2.getTextValue();
            if( !"1".equals(itsValue) )
                return( "Failed : value of element was not correct after creation (length 1)" );

            int count = 0;
            for( Enumeration eachChild = doc.getChildren(); eachChild.hasMoreElements(); count++, eachChild.nextElement() )
                ;
            
            if( 1 != count )
                return( "Failed : Doc didnt have one child" );
            
            count = 0;
            for( Enumeration eachChild = doc.getChildren( "element" ); eachChild.hasMoreElements(); count++, eachChild.nextElement() )
                ;
            
            if( 1 != count )
                return( "Failed : Doc didnt have one child named 'element'" );
            
            count = 0;
            for( Enumeration eachChild = doc.getChildren( "bogus" ); eachChild.hasMoreElements(); count++, eachChild.nextElement() )
                ;
            
            if( 0 != count )
                return( "Failed : Doc shouldnt have had a child named 'bogus'" );
            
            count = 0;
            for( Enumeration eachChild = testElement.getChildren( ); eachChild.hasMoreElements(); count++, eachChild.nextElement() )
                ;
            
            if( 3 != count )
                return( "Failed : element didnt have expected number of children" );
            
            count = 0;
            for( Enumeration eachChild = testElement.getChildren( useName ); eachChild.hasMoreElements(); count++, eachChild.nextElement() )
                ;
            
            if( 1 != count )
                return( "Failed : element didnt have expected number of children named '" +useName + "'" );
            
            // This check also is important for checking that the behaviour of the 
            // tree is correct when there are nodes with the same name as the parent in subtrees.
            
            Element testElement3 = doc.createElement( useName, useValue );
            testElement2.appendChild( testElement3 );
            
            testElement3 = doc.createElement( useName, useValue );
            testElement2.appendChild( testElement3 );
 
            count = 0;
            for( Enumeration eachChild = testElement2.getChildren( useName ); eachChild.hasMoreElements(); count++, eachChild.nextElement() )
                ;
            
            if( 2 != count )
                return( "Failed : element didnt have expected number of children named '" +useName + "'" );

            StructuredDocument likeMe = null;
            
            try {
                likeMe = StructuredDocumentFactory.newStructuredDocument( doc.getMimeType(), doc.getStream() );
            }
            catch( Throwable thrown ) {
                return( "Failed : exception thrown during construction!" );
            }
            
            Writer somewhere = new StringWriter();
            
            ((StructuredTextDocument)doc).sendToWriter( somewhere );
            
            String docAsString = somewhere.toString().trim();

            testElement3 = doc.createElement( useName, docAsString );
            testElement2.appendChild( testElement3 );
            
            String docFromElement = (String) testElement3.getValue();

            if( docAsString.length() != docFromElement.length() )
              return( "Failed : Could not faithfully store stream representation of doc in doc. (lengths dont match)" );

            for( int eachChar = 0; eachChar < docAsString.length(); eachChar++ ) {
                if( docAsString.charAt( eachChar ) != docFromElement.charAt(eachChar) )
                    return( "Failed : Could not faithfully store stream representation of doc in doc. (failed at index: " + eachChar + ")" );
                }

        }
        catch( Exception everything ) {
            everything.printStackTrace();
            return( "Failed : caught an unexpected exception - " + everything.toString() );
        }
        
        return "A-OK";
    }
    
    public static void copyElements( StructuredDocument intoDoc, Element intoElement, Element from ) {
        Object fromKey = from.getKey();
        Object fromValue = from.getValue();
        
        Element newElement = intoDoc.createElement( fromKey, fromValue );
        
        intoElement.appendChild( newElement );
        
        // recurse to add the children.
        for( Enumeration eachChild = from.getChildren(); eachChild.hasMoreElements(); ) {
            Element aChild = (Element) eachChild.nextElement();
            
            copyElements( intoDoc, newElement, aChild );
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
        
        System.err.println ("XMLDoc Test results: " + new DocTest ().TestStructuredTextDoc ( new MimeMediaType("Text/Xml")) );
        
        //System.err.println ("LiteXMLDoc Test results: " + new DocTest ().TestStructuredTextDoc ( new MimeMediaType( "Text/X-xml")) );
        
        System.err.println ("PlainTextDoc Test results: " + new DocTest ().TestStructuredTextDoc ( new MimeMediaType("Text/Plain")) );
    }
    
}
