/*
 * $Id: IDTest.java,v 1.4 2001/05/23 00:28:23 lomax Exp $
 ********************
 *
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 ********************
 */

import java.net.URL;

import net.jxta.id.*;
import net.jxta.codat.CodatID;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peer.PeerID;
import net.jxta.pipe.PipeID;

import net.jxta.impl.id.UUID;
import net.jxta.impl.id.UUIDFactory;

/**
 *
 * @author  mike
 * @version
 */
public final class IDTest extends Object {
    
    /** Creates new IDTest */
    public IDTest() {
    }
    

    private String TestID () {
        try {
            ID first = ID.nullID;
            ID second = ID.nullID;
            ID third;
            String  asString;
            URL     asURL;
            PeerGroupID myPeerGroup;
            
            if( !first.equals(second) )
                return( "Failed : comparison of two IDs failed" );
            
            third = (ID) first.clone();
            
            if( !first.equals(third) )
                return( "Failed : comparison of cloned ID failed" );
            
            if( 0 == first.hashCode() )
                return( "Failed : zero hashcode returned" );
            
            asURL = first.getURL( );
            asString = first.toString( );
            
            if( !asString.equals( asURL.toString() ) )
                return( "Failed : comparison of ID string and string of URL was not the same" );
            
            third = (ID) IDFactory.fromURL( asURL );
            
            if( !first.equals( third ) )
                return( "Failed : result of conversion to URL and back to ID was not equal to original" );
            
            myPeerGroup = first.getPeerGroupID();
            
            if( !first.isSamePeerGroup( third ) )
                return( "Failed : clone of ID is not of same peergroup." );
            
        }
        catch( Exception everything ) {
            return( "Failed : caught an unexpected exception - " + everything.toString() );
        }
        
        return "A-OK";
    }
   
    private String TestCodatID () {
        try {
            CodatID first = new CodatID();
            CodatID second = new CodatID( new PeerGroupID( new UUID( 0x01, 0x01 ) ) );
            CodatID third;
            ID interloper = new PeerID();
            String  asString;
            URL     asURL;
            PeerGroupID myPeerGroup;
            boolean isStatic;
            
            if( !first.equals(first) )
                return( "Failed : comparison of a CodatID against itself failed" );
            
            if( first.equals(second) )
                return( "Failed : comparison of two different CodatIDs should have failed" );
            
            third = (CodatID) first.clone();
            
            if( !first.equals(third) )
                return( "Failed : comparison of cloned ID failed" );
            
            if( first.equals( interloper ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( interloper.equals( first ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( 0 == first.hashCode() )
                return( "Failed : zero hashcode returned" );
            
            asURL = first.getURL( );
            asString = first.toString( );
            
            if( !asString.equals( asURL.toString() ) )
                return( "Failed : comparison of ID string and string of URL was not the same" );
            
            third = (CodatID) IDFactory.fromURL( asURL );
            
            if( !first.equals( third ) )
                return( "Failed : result of conversion to URL and back to ID was not equal to original" );
            
            myPeerGroup = first.getPeerGroupID();
            
            if( !first.isSamePeerGroup( third ) )
                return( "Failed : clone of ID is not of same peergroup." );
            
            if( first.isStatic() )
                return( "Failed : dynamic CodatID did not test as such." );
        }
        catch( Exception everything ) {
            return( "Failed : caught an unexpected exception - " + everything.toString() );
        }
        
        return "A-OK";
    }
    
    private String TestPeerGroupID () {
        try {
            PeerGroupID first = new PeerGroupID();
            PeerGroupID second = new PeerGroupID( );
            PeerGroupID third;
            ID interloper = new PeerID();
            String  asString;
            URL     asURL;
            PeerGroupID myPeerGroup;
            boolean isStatic;
            
            if( !first.equals(first) )
                return( "Failed : comparison of a PeerGroupID against itself failed" );
            
            System.out.println( " world hash code = " + PeerGroupID.worldPeerGroupID.hashCode() );

            if( first.equals(PeerGroupID.worldPeerGroupID) )
                return( "Failed : comparison against worldPeerGroup should have failed" );

            if( first.equals(second) )
                return( "Failed : comparison of two different PeerGroupIDs should have failed" );
            
            third = (PeerGroupID) first.clone();
            
            if( !first.equals(third) )
                return( "Failed : comparison of cloned ID failed" );
            
            if( first.equals( interloper ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( interloper.equals( first ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( 0 == first.hashCode() )
                return( "Failed : zero hashcode returned" );
            
            if( PeerGroupID.worldPeerGroupID.hashCode() == first.hashCode() )
                return( "Failed : hashcode for world group should not have matched." );

            asURL = first.getURL( );
            asString = first.toString( );
            
            if( !asString.equals( asURL.toString() ) )
                return( "Failed : comparison of ID string and string of URL was not the same" );
            
            third = (PeerGroupID) IDFactory.fromURL( asURL );
            
            if( !first.equals( third ) )
                return( "Failed : result of conversion to URL and back to ID was not equal to original" );
            
            myPeerGroup = first.getPeerGroupID();
            
            if( !first.isSamePeerGroup( third ) )
                return( "Failed : clone of ID is not of same peergroup." );
            
            UUID fromFirst = (UUID) first.getGroupUniqueId();
            UUID fromSecond = (UUID) second.getGroupUniqueId();
            
            if( !fromFirst.equals((UUID) third.getGroupUniqueId()) )
                return( "Failed : comparison of UUIDs from an ID and a clone failed" );
            
            if( fromFirst.equals((UUID) second.getGroupUniqueId()) )
                return( "Failed : comparison of UUIDs from an ID and a different ID should have failed." );
        }
        catch( Exception everything ) {
            return( "Failed : caught an unexpected exception - " + everything.toString() );
        }
        
        return "A-OK";
    }
    
    private String TestPeerID () {
        try {
            PeerID first = new PeerID();
            PeerID second = new PeerID( new PeerGroupID( new UUID( 0x01, 0x01 ) ) );
            PeerID third;
            ID interloper = new PeerGroupID();
            String  asString;
            URL     asURL;
            PeerGroupID myPeerGroup;
            boolean isStatic;
            
            if( !first.equals(first) )
                return( "Failed : comparison of a PeerID against itself failed" );
            
            if( first.equals(second) )
                return( "Failed : comparison of two different PeerIDs should have failed" );
            
            third = (PeerID) first.clone();
            
            if( !first.equals(third) )
                return( "Failed : comparison of cloned ID failed" );
            
            if( first.equals( interloper ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( interloper.equals( first ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( 0 == first.hashCode() )
                return( "Failed : zero hashcode returned" );
            
            asURL = first.getURL( );
            asString = first.toString( );
            
            if( !asString.equals( asURL.toString() ) )
                return( "Failed : comparison of ID string and string of URL was not the same" );
            
            third = (PeerID) IDFactory.fromURL( asURL );
            
            if( !first.equals( third ) )
                return( "Failed : result of conversion to URL and back to ID was not equal to original" );
            
            myPeerGroup = first.getPeerGroupID();
            
            if( !first.isSamePeerGroup( third ) )
                return( "Failed : clone of ID is not of same peergroup." );
      }
        catch( Exception everything ) {
            return( "Failed : caught an unexpected exception - " + everything.toString() );
        }
        
        return "A-OK";
    }
    
    private String TestPipeID () {
        try {
            PipeID first = new PipeID();
            PipeID second = new PipeID( new PeerGroupID( new UUID( 0x01, 0x01 ) ) );
            PipeID third;
            ID interloper = new PeerGroupID();
            String  asString;
            URL     asURL;
            PeerGroupID myPeerGroup;
            boolean isStatic;
            
            if( !first.equals(first) )
                return( "Failed : comparison of a PipeID against itself failed" );
            
            if( first.equals(second) )
                return( "Failed : comparison of two different PipeIDs should have failed" );
            
            third = (PipeID) first.clone();
            
            if( !first.equals(third) )
                return( "Failed : comparison of cloned ID failed" );
            
            if( first.equals( interloper ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( interloper.equals( first ) )
                return( "Failed : comparison of different types should have failed" );
            
            if( 0 == first.hashCode() )
                return( "Failed : zero hashcode returned" );
            
            asURL = first.getURL( );
            asString = first.toString( );
            
            if( !asString.equals( asURL.toString() ) )
                return( "Failed : comparison of ID string and string of URL was not the same" );
            
            third = (PipeID) IDFactory.fromURL( asURL );
            
            if( !first.equals( third ) )
                return( "Failed : result of conversion to URL and back to ID was not equal to original" );
            
            myPeerGroup = first.getPeerGroupID();
            
            if( !first.isSamePeerGroup( third ) )
                return( "Failed : clone of ID is not of same peergroup." );
      }
        catch( Exception everything ) {
            return( "Failed : caught an unexpected exception - " + everything.toString() );
        }
        
        return "A-OK";
    }
    
    public static void main (String args[]) {
        System.err.println ("ID Test results: " + new IDTest ().TestID ());
        
        System.err.println ("CodatID Test results: " +  new IDTest ().TestCodatID () );
        
        System.err.println ("PeerGroupID Test results: " +  new IDTest ().TestPeerGroupID () );
        
        System.err.println ("PeerID Test results: " +  new IDTest ().TestPeerID () );

        System.err.println ("PipeID Test results: " +  new IDTest ().TestPipeID () );
    }
    
}
