#ifndef ADDRESS_H
#define ADDRESS_H

#include <string>
#include <strstream>

/**
 * @short This class stores an IP address and port number.
 */
class Address
{
 public:
  /**
   * @short Creates an address out of the own ip and standard port 6346.
   */
  Address();

  /**
   * @short Creates an address out of the given ip and standard port 6346.
   */
  Address(string ip);

  /**
   * @short Creates an address out of the given ip and port.
   */
  Address(string ip, unsigned int port);

  /**
   * @short Creates an address out of the given ip and standard port 6346.
   */
  Address(unsigned long ip);

  /**
   * @short Creates an address out of the given ip and port.
   */
  Address(unsigned long ip, unsigned int port);

  /**
   * @short Returns the IP address as string.
   */
  string       strIP();

  /**
   * @short Returns the port.
   */
  unsigned int  port();

  /**
   * @short Returns the IP address as long.
   */
  unsigned long longIP();

  /**
   * @short Returns the IP address and port number as string.
   * The IP address and port number are separated by a colon.
   */
  string       toString();
  void         setPort(unsigned int port);
  void         setPort(string port);

  string       strPort();

 private:
  string       ip;
  unsigned int uiport;

};

#endif
