#ifndef DOWNLOADTHREAD_H
#define DOWNLOADTHREAD_H

#include <fstream>

#include <qthread.h>
#include <qobject.h>
#include <qdatetime.h>
#include <qtimer.h>

#include "QueryHitEntry.h"

class QSocket;
class QListViewItem;
class QFile;
class DialogExist;
class QtellaSub;

class DownloadThread : public QObject, public QThread
{
  Q_OBJECT

 public:
  DownloadThread(QueryHitEntry qhe, QListViewItem *item, QtellaSub *parent);
  ~DownloadThread();

  virtual void  run();
  int           readState();
  double        readBandwidth();
  int           readRetryTimeout();
  unsigned long bytesRead();
  unsigned long bytesToRead();

  void          abortDownload();
  void          resumeDownload();

  enum { dtIdle,             /*!< Initial state                                     */
	 dtError,            /*!< Error occured.                                    */
	 dtAbort,            /*!< Download aborted by user.                         */
	 dtClosed,           /*!< Connection was closed without any bytes read.     */
	 dtClosedCompleted,  /*!< Connection was closed after finishing download.   */
	 dtClosedUnfinished, /*!< Connection was closed but download not completed. */
	 dtConnecting,       /*!< Try to establish a connection.                    */
	 dtNotFound,         /*!< */
	 dtTimeout,          /*!< */
	 dtBusy,             /*!< */
	 dtConnected,        /*!< Connected.                                        */
	 dtExists,           /*!< File already exists.                              */
	 dtQueued
  };

  //! Pointer to the download item in the download list
  /*!
    This item is deleted automatically when ~DownloadThread() is called.
    \sa ~DownloadThread()
  */
  QListViewItem* _item;

 private:
  string        createFilename(string file, unsigned long size, string host);
  string        findFile(string file, unsigned long size);
  string        findFile(string file, string host);
  string        findFile(string file, unsigned long size, string host);

  void          setState(int);

  //! Timer which manages the update of the status of this download.
  QTimer*        _status_update_timer;

  //! Name of the server.
  string         _client;

  //! Mutex to coordinate access to the current state.
  QMutex         _state_mutex;

  //! Current state of the this download.
  int            _state;

  //! Holds information about the file to be downloaded.
  QueryHitEntry  _qhe;

  //! Size of the file.
  unsigned long  _content_length;

  //! Pointer to the parent.
  QtellaSub*     _parent;

  //! Time when download started.
  QTime          _start_time;

  //! Time when download finished.
  QTime          _end_time;

  //! Buffer to store incoming data.
  string         _buffer;

  //! True if run() method was not invoked yet.
  bool           _first_start;

  //! File to which incoming data is written to.
  QFile*         _f;

  //! Stores the HTML-Header
  string         _header;

  string         complete_path;
  string         incomplete_path;

  QSocket        *sock;
  QTimer         *timer_bandwidth;
  QTimer         *timer_retry;
  int            retry_timeout;
  unsigned long  tmp_read;
  double         bandwidth;

  QTimer         *timeout;

 public slots:
  void           slotReadyRead();
  void           slotConnectionClosed();
  void           slotError(int);
  void           slotConnected();
  void           slotTimerBandwidth();
  void           slotTimeout();
  void           slotRetry();
  void           slotUpdateStatus();
};

#endif
