#include "Address.h"

#include <qhostaddress.h>
#include <qdns.h>
#include <qvaluelist.h>

#include <unistd.h>

Address::Address()
{
  char hostname[50];
  gethostname(hostname, 50);
  char domainname[50];
  getdomainname(domainname, 50);

  strstream str;
  str << hostname << "." << domainname << ends;

  QDns                     dns(str.str());
  QValueList<QHostAddress> list;

  list = dns.addresses();

  if(list.isEmpty())
    ip = "0.0.0.0";
  else
    ip = list.first().toString();

  uiport = 0;
  str.freeze(false);
}

Address::Address(string ip)
{
  string::size_type Idx = ip.find(":");
  if(Idx==string::npos)
    {
      this->ip = ip;
      uiport = 0;
    }
  else
    {
      this->ip = ip.substr(0, Idx);
      uiport = atol((ip.substr(Idx+1)).c_str());
    }
}

Address::Address(string ip, unsigned int port)
{
  this->ip = ip;
  uiport = port;
}

Address::Address(unsigned long ip)
{
  strstream str;

  str << (ip&0xff) << "." << ((ip>>8)&0xff) << "." << ((ip>>16)&0xff)
      << "." << ((ip>>24)&0xff) << ends;

  this->ip = str.str();
  uiport = 0;
  
  str.freeze(false);
}

Address::Address(unsigned long ip, unsigned int port)
{
  strstream str;

  str << (ip&0xff) << "." << ((ip>>8)&0xff) << "." << ((ip>>16)&0xff)
      << "." << ((ip>>24)&0xff) << ends;

  this->ip = str.str();
  uiport = port;

  str.freeze(false);
}

string Address::strIP()
{
  return ip;
}

unsigned long Address::longIP()
{
  std::string::size_type  idx;
  std::string             ip = this->ip;
  unsigned long           lip = 0;
  int                     shift = 0;

  while( (idx = ip.find(".")) != std::string::npos )
    {
      strstream str;
      int       i;
      str << ip.substr(0, idx) << ends;
      str >> i;
      lip |= (i<<shift);
      shift += 8;
      ip = ip.substr(idx+1);
    }

  strstream str;
  int       i;
  str << ip;
  str >> i;
  lip |= i<<shift;

  return lip;
}

void Address::setPort(unsigned int port)
{
  uiport = port;
}

void Address::setPort(string port)
{
  strstream str;
  str << port << ends;
  str >> uiport;
}

unsigned int Address::port()
{
  return uiport;
}

string Address::toString()
{
  strstream str;

  str << ip << ":" << uiport << ends;

  string s;
  str >> s;

  return s;
}

string Address::strPort()
{
  strstream str;
  str << uiport << ends;

  string s = str.str();

  return s;
}
