#include "Connections.h"

#include "QtellaSub.h"
#include "Servent.h"
#include "Pong.h"
#include "Host.h"
#include "QueryHit.h"
#include "Ping.h"
#include "Query.h"
#include "QueryHitEntry.h"
#include "Push.h"
#include "SearchWidget.h"
#include "Search.h"
#include "SharedFile.h"
#include "QueryResult.h"

#include <iostream>
#include <map>
#include <list>
#include <algorithm>
#include <memory>

#include <stdlib.h>
#include <time.h>

#include <qevent.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>

#undef QTELLA_DEBUG

Connections::Connections(QWidget *parent = 0)
{
  this->parent = (QtellaSub*)parent;
  max_servents = ((this->parent)->ui_spinbox_hosts)->value();
  max_hosts_in_hostlist = 300;

  // create unique id

  srand(time(NULL));
  id = "";

  for(int i=0; i<16; i++)
    {
      unsigned char c = rand()%256;
      id += c;
    }
}



void Connections::start()
{
  // create a vector with all hosts in the AutoConnect-List

  #ifdef QTELLA_DEBUG
    cout << "start/" << vServent.size() << endl;
  #endif

  vector<Host> hosts;

  for(int i=0; i<parent->ui_listbox_autoconnect->count(); i++)
    {
      QString qstr = parent->ui_listbox_autoconnect->text(i);
      string  str  = (const char*)qstr;
      hosts.push_back(Host(Address(str), 0, 0));
    }

  if(hosts.empty()) return;

  // connect to hosts

  for(int i=0; i<hosts.size(); i++)
    if(max_servents > vServent.size())
      {
	QListViewItem *item;

	if(vServent.size()==0)
	  item = new QListViewItem(parent->ui_listview_connections);
	else
	  item = new QListViewItem(parent->ui_listview_connections, vServent.back()->_item);

	item->setText(0, hosts[i].address.toString().c_str());
	item->setText(1, "Connecting...");

	Servent *s = new Servent(id, item);
	vServent.push_back(s);
	s->connectToServent(hosts[i].address);
      }

  // now return while Servents are trying to connect to hosts
  // the rest will be done in check_servents()
}



void Connections::check_servents()
{
  unsigned long hosts = 0;

  #ifdef QTELLA_DEBUG
    cout << "0" << endl;
    for(int i=0; i<vServent.size(); i++) cout << "STATE: " << i << ":" << vServent[i]->state() << endl;
  #endif

  for( int i=0; i<vServent.size(); i++)
    switch( vServent[i]->serventState() )
      {
      case Servent::serventConnected:
	{
	  vServent[i]->_item->setText(1, "Connected");

	  // check the vectors of the servent

	  checkVectors(vServent[i]);

	  // update message i/o status in listbox

	  strstream str;
	  str << vServent[i]->_input << " / " << vServent[i]->_output << ends;
	  vServent[i]->_item->setText(2, str.str());
	  str.freeze(false);

	  str.seekp(0, ios::beg);
	  str << vServent[i]->host_list.size() << ends;
	  vServent[i]->_item->setText(3, str.str());
	  str.freeze(false);

	  hosts += vServent[i]->host_list.size();
	}
	break;

      case Servent::serventClosed:
	vServent[i]->_item->setText(1, "Closed");
	break;

      case Servent::serventConnecting:
	vServent[i]->_item->setText(1, "Connecting...");
	break;

      case Servent::serventError:
	vServent[i]->_item->setText(1, "Error");
	break;

      case Servent::serventWaiting:
	vServent[i]->_item->setText(1, "Waiting");
	break;
      }

  #ifdef QTELLA_DEBUG
    cout << "A" << endl;
  #endif

  // delete servent which are in close or error state and increase timeout counter of all servents

  std::vector<class Servent*>::iterator    pos = vServent.begin();

  for(pos=vServent.begin(); pos!=vServent.end(); )
    if((*pos)->serventState()==Servent::serventClosed || (*pos)->serventState()==Servent::serventError)
      {
	// store host list of this servent to listbox
	int c = int();
	list<Host*>::iterator  hpos = (*pos)->host_list.begin();

	while( (++c < 200) && (hpos != (*pos)->host_list.end()) )
	  parent->ui_listbox_hostlist->insertItem( (*(hpos++))->address.toString().c_str(), 0);
	
	delete *pos;
	pos = vServent.erase(pos);
      }
    else
      {
	if( ((*pos)->_timeout > 80) && ((*pos)->serventState() == Servent::serventConnecting) )
	  (*pos)->closeConnection();

	(*pos)->_timeout++;
	++pos;
      }

  #ifdef QTELLA_DEBUG
    cout << "B" << endl;
  #endif

  // connect to other hosts

  while( (vServent.size() < max_servents) && (parent->ui_listbox_hostlist->count() > 0) )
    {
      string str = (const char*)parent->ui_listbox_hostlist->text(0);
      parent->ui_listbox_hostlist->removeItem(0);

      QListViewItem *item;

      if(vServent.size()==0)
	item = new QListViewItem(parent->ui_listview_connections);
      else
	item = new QListViewItem(parent->ui_listview_connections, vServent.back()->_item);

      item->setText(0, str.c_str());
      item->setText(1, "Connecting...");

      vServent.push_back(new Servent(id, item));
      vServent.back()->connectToServent(Address(str));
    }

  // hostlist is empty and maxservents > connected hosts => autoconnect

  if(parent->ui_listbox_hostlist->count()==0 && max_servents>vServent.size()) start();

  #ifdef QTELLA_DEBUG
    cout << "C" << endl;
  #endif

  // number of hosts

  strstream str;
  str << hosts << ends;
  parent->ui_textlabel_hosts->setText(str.str());
  str.freeze(false);
}





void Connections::checkVectors(Servent *s)
{
  s->limitVectors();

  #ifdef QTELLA_DEBUG
    cout << "entering checkVectors" << endl;
  #endif

  s->_list_mutex.lock();

  std::vector<Ping*>::iterator  pos = s->vPing.begin();

  while(pos != s->vPing.end())
    {
      if( ((*pos)->id != id) && ((*pos)->ttl > 1) )
	{
	  for(int k=0; k<vServent.size(); k++)
	    if(vServent[k]!=s) vServent[k]->sendPing((*pos)->ttl-1, (*pos)->hops+1, (*pos)->id);

	  Address ad((const char*)parent->ui_lineedit_ip->text());
	  ad.setPort((const char*)parent->ui_lineedit_listenport->text());

	  s->sendPong((*pos)->hops+1, 0, (*pos)->id, ad, parent->number_shared_files, parent->shared_size/1024);
	  s->pingid_list.push_back((*pos)->id);
	}

      delete *pos;
      ++pos;
    }

  s->vPing.clear();

  s->_list_mutex.unlock();




  //cout << "check pong" << endl;

  s->_list_mutex.lock();

  std::list<Pong*>::iterator  a = s->pong_list.begin();

  while(a != s->pong_list.end())
    {
      std::vector<Servent*>::iterator spos = vServent.end();
      std::vector<Servent*>::iterator k;

      for(k = vServent.begin(); k != vServent.end(); ++k)
	{
	  list<string>::iterator  hpos = (*k)->pingid_list.begin();
	  
	  while( hpos != (*k)->pingid_list.end() )
	    if( *(hpos++) == (*a)->id ) 
	      {
		spos = k;
		break;
	      }
	}
      
      if( spos != vServent.end() )
	(*spos)->sendPong( (*a)->ttl-1, (*a)->hops+1, (*a)->id, (*a)->address, (*a)->files, (*a)->kbytes );
      
      delete *a;
      ++a;
    }
  
  s->pong_list.clear();

  s->_list_mutex.unlock();



  //cout << "check query" << endl;

  s->_list_mutex.lock();

  std::vector<Query*>::iterator  b = s->vQuery.begin();

  while(b != s->vQuery.end())
    {
      if(parent->ui_checkbox_incomingsearch->isChecked())
	{
	  parent->ui_listbox_incomingsearch->insertItem((*b)->search.c_str(), 0);
	  while(parent->ui_listbox_incomingsearch->count() > 40)
	    parent->ui_listbox_incomingsearch->removeItem(40);
	}

      int pos = -1;
      for(int m=0; m<vSearch.size(); m++) if((*b)->id == vSearch[m]->id) pos = m;

      if( pos == -1 )
	{
	  std::vector<Servent*>::iterator  ipos;
	  for(ipos = vServent.begin(); ipos != vServent.end(); ++ipos)
	    if( (*ipos) != s )
	      (*ipos)->sendQuery( (*b)->ttl-1, (*b)->hops+1, (*b)->id, (*b)->min_speed, (*b)->search );

	  s->queryid_list.push_back( (*b)->id );

	  // search query in shared file list
	  std::vector<string>     vstr;
	  std::string             search = (*b)->search;
	  std::string::size_type  idx;

	  transform(search.begin(), search.end(), search.begin(), tolower);

	  while( (idx = search.find_first_of(" ")) != std::string::npos )
	    {
	      vstr.push_back(search.substr(0, idx));
	      search = search.substr(idx+1);
	    }
	  
	  vstr.push_back(search);

	  vector<SharedFile*> v = parent->readSharedFiles();
	  vector<QueryResult> qr;

	  for(int k=0; k<v.size(); k++)  // for each shared file...
	    {
	      bool    found_all = true;
	      string  file = v[k]->file;

	      transform(file.begin(), file.end(), file.begin(), tolower);

	      for(int l=0; l<vstr.size(); l++) // for each word in query...
		if( file.find(vstr[l].c_str()) == std::string::npos ) found_all = false;

	      if(found_all) qr.push_back(QueryResult(k, v[k]->size, v[k]->file));
	    }

	  Address *addr = new Address((const char*)parent->ui_lineedit_ip->text());
	  addr->setPort((const char*)parent->ui_lineedit_listenport->text());

	  int speed[] = {56, 64, 128, 350, 1000, 3000};
	  
	  if(qr.size() > 0)
	    {
	      QueryHit qh((*b)->id,      // id of query
			  (*b)->hops+1,  // ttl
			  0,                     // hops
			  *addr,                 // this address
			  speed[parent->ui_combobox_speed->currentItem()],
			  this->id,              // result id
			  qr);                   // query results

	      s->sendQueryHit( (*b)->hops+1, 0, qh);
	    }

	  delete addr;
	}

      delete *b;
      ++b;
    }

  s->vQuery.clear();

  s->_list_mutex.unlock();



  //cout << "check queryhit" << endl;

  // check queryhit vector

  s->_list_mutex.lock();

  for(int j=0; j<s->vQueryHit.size(); j++)
    {
      int pos = -1;

      for(int m=0; m<vSearch.size(); m++)
	if(s->vQueryHit[j]->id == vSearch[m]->id) pos = m;

      if( pos > -1 )
	{
	  if(vSearch[pos]->active)
	    {
	      strstream str_speed;

	      str_speed << s->vQueryHit[j]->speed << ends;
	      
	      for(int n=0; n<s->vQueryHit[j]->vQueryResult.size(); n++)
		{
		  strstream str_size;
		  strstream str_index;
		  
		  str_size << s->vQueryHit[j]->vQueryResult[n].filesize << ends;
		  str_index << vSearch[pos]->vSearchResults.size() << ends;
		  
		  QListViewItem *item;
		  
		  if(vSearch[pos]->vSearchResults.size()==0)
		    item = new QListViewItem(vSearch[pos]->sw->ui_listview_search);
		  else
		    item = new QListViewItem(vSearch[pos]->sw->ui_listview_search, vSearch[pos]->vSearchResults.back()->item);
		  
		  item->setText(0, str_index.str());
		  item->setText(1, str_speed.str());
		  item->setText(2, s->vQueryHit[j]->vQueryResult[n].filename.c_str());
		  item->setText(3, str_size.str());
		  item->setText(4, s->vQueryHit[j]->address.strIP().c_str());
		  
		  vSearch[pos]->vSearchResults.push_back(new QueryHitEntry(s->vQueryHit[j]->vQueryResult[n].filename,
									s->vQueryHit[j]->vQueryResult[n].filesize,
									s->vQueryHit[j]->vQueryResult[n].index,
									s->vQueryHit[j]->address,
									s->vQueryHit[j]->id_result, item) );

		  str_index.freeze(false);
		  str_size.freeze(false);
		}
	      
	      str_speed.freeze(false);
	    }
	}
      else  // pos = -1
	if(s->vQueryHit[j]->ttl > 1)
	  for(int k=0; k<vServent.size(); k++)
	    {
	      list<string>::iterator  hpos = vServent[k]->queryid_list.begin();

	      while( hpos != vServent[k]->queryid_list.end() )
		if( *(hpos++) == s->vQueryHit[j]->id )
		  vServent[k]->sendQueryHit( s->vQueryHit[j]->ttl-1, s->vQueryHit[j]->hops+1, 
					     *(s->vQueryHit[j]) );
	    }

    }

  while(s->vQueryHit.size() > 0)
    {
      delete s->vQueryHit.back();
      s->vQueryHit.pop_back();
    }

  s->_list_mutex.unlock();



  //cout << "check push" << endl;

  s->_list_mutex.lock();

  std::vector<Push*>::iterator  d = s->vPush.begin();

  while(d != s->vPush.end())
    {
      if( (*d)->id_payload == this->id )
	{
	  // TODO
	}
      else
	if( (*d)->ttl > 1 )
	                                           // ...send push to all other servents
	  for(int k=0; k<vServent.size(); k++)
	    if(vServent[k]->serventState()==Servent::serventConnected && vServent[k]!=s)
	      vServent[k]->sendPush((*d)->ttl-1, (*d)->hops+1, *(*d) );

      delete *d;
      ++d;
    }

  s->vPush.clear();

  s->_list_mutex.unlock();

  #ifdef QTELLA_DEBUG
    cout << "leaving" << endl;
  #endif
}




void Connections::search(string query, SearchWidget *sw)
{
  std::vector<Servent*>::iterator  pos;

  string search_id;

  for(int i=0; i<16; i++)
    {
      unsigned char c = rand()%256;
      search_id += c;
    }

  vSearch.push_back(new Search(search_id, sw));
  for(pos = vServent.begin(); pos != vServent.end(); ++pos) (*pos)->sendQuery(7, 0, search_id, 0, query);
}


string Connections::id2str(string id)
{
  strstream str;

  for(int i=0; i<16; i++)
    {
      char c = id[i];
      int  j = (unsigned char)c;
      str << std::hex << j << "-";
    }
  str << ends;

  string s = str.str();
  str.freeze(false);

  return s;
}


void Connections::slotRemoveHost()
{
  QListViewItem *item = parent->ui_listview_connections->currentItem();

  if(item)
    {
      std::vector<class Servent*>::iterator pos = vServent.begin();

      for(pos; pos != vServent.end(); )
	if( (*pos)->_item == item )
	  {
	    delete *pos;
	    pos = vServent.erase(pos);
	    break;
	  }
	else
	  ++pos;
    }
}



void Connections::slotAddHost()
{
  string hostname = (const char*)parent->ui_lineedit_host->text();
  string port = (const char*)parent->ui_lineedit_port->text();

  Host host = Host(Address(hostname+":"+port));

  QListViewItem *item = 
    new QListViewItem(parent->ui_listview_connections, 
		      host.address.toString().c_str(), "Connecting...");

  Servent *s = new Servent(id, item);
  vServent.push_back(s);
  s->connectToServent(host.address);
  parent->ui_listview_connections->insertItem(item);
}



void Connections::slotMaxServents(int value)
{
  this->max_servents = value;
}


void Connections::slotListClear()
{
  parent->ui_listbox_hostlist->clear();
}


void Connections::slotListConnect()
{
  int i = parent->ui_listbox_hostlist->currentItem();

  if(i!=-1)
    {
      string addr = (const char*)parent->ui_listbox_hostlist->text(i);

      Host host = Host(Address(addr));
      
      QListViewItem *item = 
	new QListViewItem(parent->ui_listview_connections, 
			  host.address.toString().c_str(), "Connecting...");

      Servent *s = new Servent(id, item);
      vServent.push_back(s);
      s->connectToServent(host.address);
      parent->ui_listview_connections->insertItem(item);
    }
}


void Connections::slotListRemove()
{
  int i = parent->ui_listbox_hostlist->currentItem();

  if(i!=-1)
    parent->ui_listbox_hostlist->removeItem(i);
}
