#include "DownloadThread.h"
#include "StringManipulation.h"
#include "QtellaSub.h"

#include <qsocket.h>
#include <qlistview.h>
#include <qfile.h>
#include <qdir.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qlineedit.h>

#include <iomanip>

DownloadThread::DownloadThread(QueryHitEntry qhe, QListViewItem *item, QtellaSub *parent)
  : _parent(parent), _item(item), _qhe(qhe), _state(dtQueued), _content_length(qhe.size)
{
  timeout = new QTimer();
  timer_bandwidth = new QTimer();
  timer_retry = new QTimer();
  sock = new QSocket();

  connect(sock, SIGNAL(readyRead()), this, SLOT(slotReadyRead()));
  connect(sock, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
  connect(sock, SIGNAL(error(int)), this, SLOT(slotError(int)));
  connect(sock, SIGNAL(connected()), this, SLOT(slotConnected()));

  connect(timeout, SIGNAL(timeout()), this, SLOT(slotTimeout()));
  connect(timer_retry, SIGNAL(timeout()), this, SLOT(slotRetry()));
  connect(timer_bandwidth, SIGNAL(timeout()), this, SLOT(slotTimerBandwidth()));

  _client.erase();

  _first_start = true;

  _status_update_timer = new QTimer();
  connect(_status_update_timer, SIGNAL(timeout()), this, SLOT(slotUpdateStatus()));
  _status_update_timer->start(1000);

  _f = NULL;
}


void DownloadThread::run()
{
  setState(dtConnecting);

  if(timer_retry->isActive()) timer_retry->stop();
  if(timeout->isActive()) timeout->stop();
  if(timer_bandwidth->isActive()) timer_bandwidth->stop();

  bandwidth     = 0;
  tmp_read      = 0;
  _buffer.erase();
  _header.erase();
  retry_timeout = 30;

  string host = _qhe.addr.strIP();


  complete_path   = _parent->ui_lineedit_downloaddirectory->text();
  if(complete_path[complete_path.size()-1] != '/') complete_path += "/";
  complete_path += _qhe.filename;

  incomplete_path = _parent->ui_lineedit_incompleted->text();
  if(incomplete_path[incomplete_path.size()-1] != '/') incomplete_path += "/";
  incomplete_path += _qhe.filename;


  QDir d(incomplete_path.substr(0, incomplete_path.find_last_of("/")).c_str());
  
  if(!d.exists())
    {
      cerr << "Directory for incomplete downloads does not exist." << endl;
      setState(dtError);
      timer_retry->start(1000);
      return;
    }

  d.setPath(complete_path.substr(0, complete_path.find_last_of("/")).c_str());

  if(!d.exists())
    {
      cerr << "Directory for completed downloads does not exist." << endl;
      setState(dtError);
      timer_retry->start(1000);
      return;
    }


  if(_first_start)
    {
      _first_start = false;

      QFile file(complete_path.c_str());

      if( file.exists() && _parent->ui_radiobutton_e_abort->isChecked() )
	{
	  _f = new QFile(complete_path.c_str());
	  setState(dtExists);
	  return;
	}

      file.setName(incomplete_path.c_str());

      if( _parent->ui_radiobutton_a_startnew->isChecked() )
	{
	  string s = createFilename(incomplete_path, _qhe.size, _qhe.addr.strIP());
	  _f = new QFile(s.c_str());
	}
      else
	{
	  string s;

	  if( _parent->ui_radiobutton_a_contsize->isChecked() )
	    s = findFile(incomplete_path, _qhe.size);

	  if( _parent->ui_radiobutton_a_conthost->isChecked() )
	    s = findFile(incomplete_path, _qhe.addr.strIP());

	  if( _parent->ui_radiobutton_a_contsizehost->isChecked() )
	    s = findFile(incomplete_path, _qhe.size, _qhe.addr.strIP());
	      
	  if(!s.empty())
	    _f = new QFile(s.c_str());
	  else
	    {
	      string s = createFilename(incomplete_path, _qhe.size, _qhe.addr.strIP());
	      _f = new QFile(s.c_str());
	    }
	}

    }
  else
    if(_f->isOpen()) _f->close();

  if(!_f->isOpen()) _f->open(IO_Append|IO_ReadWrite);
  if(!_f->isOpen())
    {
      cerr << "Cannot open file for writing." << endl;
      slotError(dtError);
      return;
    }

  if(_f->size() > 0) retry_timeout = 15;

  timeout->start(20000, true);
  sock->connectToHost(_qhe.addr.strIP().c_str(), _qhe.addr.port());
}


int DownloadThread::readState()
{
  _state_mutex.lock();
  int i = _state;
  _state_mutex.unlock();

  return i;
}


void DownloadThread::setState(int i)
{
  _state_mutex.lock();
  _state = i;
  _state_mutex.unlock();
}


unsigned long DownloadThread::bytesRead()
{
  if(_f)
    if(_f->exists())
      return _f->size();

  return 0;
}


unsigned long DownloadThread::bytesToRead()
{
  return _content_length - bytesRead();
}


void DownloadThread::slotReadyRead()
{
  char cbuff[32768+10];  // do we really need these 10 bytes?? 
                         // without them I often get an segfault

  int w = sock->readBlock(cbuff, 32768);

  _buffer.append(cbuff, w);
  tmp_read += w;


  if(_header.empty())
    {
      std::string::size_type  pos = _buffer.find("\r\n\r\n");
      
      if( pos != std::string::npos)
	{
	  _header = _buffer.substr(0, pos+4);
	  _buffer = _buffer.substr(pos+4);

	  std::string::size_type  idx1 = _header.find(" ");
	  std::string::size_type  idx2 = _header.find(" ", idx1+1);

	  if( _header.substr(idx1+1, idx2-idx1-1) == "404" ) 
	    {
	      setState(dtNotFound);
	      _buffer.erase();
	    }
	  
	  if( _header.substr(idx1+1, idx2-idx1-1) == "503" ) 
	    {
	      setState(dtBusy);
	      timer_retry->start(1000);
	      _buffer.erase();
	    }
	  
	  if( _header.substr(idx1+1, idx2-idx1-1) == "200" ) 
	    {
	      setState(dtConnected);
	      
	      timer_bandwidth->start(2000);
	      tmp_read = 0;
	      bandwidth = 0;
	    }

	  // get server name
	  
	  if( ((idx1 = _header.find("Server:")) != std::string::npos) && (_client.empty()) )
	    {
	      idx1 += 7;  // point to character after ':'
	      idx1 = _header.find_first_not_of(" ", idx1);
	      idx2 = _header.find("\r\n", idx1);
	      _client =  _header.substr(idx1, idx2-idx1);
	    }
	  
	  // save time
	  
	  _start_time.start();
	}
    }

  if( !_buffer.empty() && !_header.empty() )
    {
      int w = _f->writeBlock(_buffer.c_str(), _buffer.size());
      _buffer.erase(0, w);
    }
}


void DownloadThread::slotConnectionClosed()
{
  while(sock->bytesAvailable() > 0) slotReadyRead();

  _f->writeBlock(_buffer.c_str(), _buffer.size());
  _buffer.erase();
  _f->flush();

  _end_time.start();
  
  if(readState()!=dtBusy)
    if(bytesRead()==_content_length)
      {
	setState(dtClosedCompleted);
	
	_f->close();

	string destination = complete_path;
	int    i = 0;

	if(_parent->ui_radiobutton_e_rename->isChecked() || _parent->ui_radiobutton_e_abort->isChecked())
	  {
	    QFile d(destination.c_str());

	    while(d.exists())
	      {
		strstream s;
		s << complete_path << "." << i++ << ends;
		s >> destination;
		d.setName(destination.c_str());
	      }
	  }

	QFile dest(destination.c_str());

	QDir d;
	if( !d.rename(_f->name(), destination.c_str()) )
	  cerr << "Error in moving file from incomleted path to completed path." << endl;
	else
	  _f->setName(destination.c_str());
      }
    else
      {
	setState(dtClosed);
	timer_retry->start(1000);
      }
}


void DownloadThread::slotError(int error)
{
  setState(dtError);
  timer_retry->start(1000);
}


void DownloadThread::slotConnected()
{
  string         s = _qhe.filename;
  unsigned long  begin = _f->size();
  strstream      str;

  strstream v;
  v << begin << ends;

  str << "GET /get/" << _qhe.index << "/" << s << " HTTP/1.0\r\n"
      << "Range: bytes=" << v.str() << "-\r\n"
      << "User-Agent: Qtella " << VERSION << "\r\n\r\n" << ends;

  s = str.str();

  str.freeze(false);
  v.freeze(false);

  sock->writeBlock(s.c_str(), s.size());
}



void DownloadThread::slotTimerBandwidth()
{
  bandwidth = (double)tmp_read / 2048;  // KB/s
  tmp_read = 0;
}


double DownloadThread::readBandwidth()
{
  return bandwidth;
}


void DownloadThread::slotTimeout()
{
  timeout->stop();

  if(readState()==dtConnecting)
    {
      setState(dtTimeout);
      sock->close();
      timer_retry->start(1000);
    }
}


void DownloadThread::abortDownload()
{
  timeout->stop();
  timer_bandwidth->stop();
  timer_retry->stop();
  sock->close();
  
  if(_f) _f->close();

  setState(dtAbort);
}


void DownloadThread::resumeDownload()
{
  setState(dtQueued);
}


DownloadThread::~DownloadThread()
{
  delete _status_update_timer;
  delete timeout;
  delete timer_bandwidth;
  delete timer_retry;
  delete sock;
  delete _item;

  if(_f)
    {
      if(_f->isOpen()) _f->close();
      delete _f;
    }
}

void DownloadThread::slotRetry()
{
  retry_timeout--;

  if( retry_timeout == 0 )  
    {
      if(timer_retry->isActive()) timer_retry->stop();
      if(timeout->isActive()) timeout->stop();
      if(timer_bandwidth->isActive()) timer_bandwidth->stop();

      setState(dtQueued);
      /*
      if( _parent->readNumberOfDownloads() < _parent->ui_spinbox_maxdownloads->value() )
	run();
      else
	setState(dtQueued);
      */
    }
}


int DownloadThread::readRetryTimeout()
{
  return retry_timeout;
}


string DownloadThread::createFilename(string file, unsigned long size, string host)
{
  time_t t  = time(NULL);

  for(int id = rand()%1000; ; ++id)
    {
      strstream str;
      string    s = file;

      str << ".part_" << t << "_" << size << "_" << host << "_" << id << ends;
      s += str.str();
      str.freeze(false);

      str << ".part_" << t << "_" << size << "_" << host << "_" << id << ends;

      QFile f(s.c_str());

      if(!f.exists()) return s;
    }
}


string DownloadThread::findFile(string file, unsigned long size)
{
  std::string::size_type  idx = file.find_last_of("/");

  string         directory = file.substr(0, idx);
  string         filename  = file.substr(idx+1);
  unsigned long  current_size = 0;

  QDir d(directory.c_str(), (filename+"*").c_str(), QDir::DefaultSort, QDir::Files);

  string s;

  for(int i = 0; i < d.count(); ++i)
    {
      filename = d[i];
      idx = filename.find(".part_");

      if( idx != std::string::npos )
	{
	  idx = filename.find("_", idx+6);
	  std::string::size_type  idx2 = filename.find("_", idx+1);

	  string s_size = filename.substr(idx+1, idx2-idx-1);

	  strstream str;
	  str << s_size << ends;
	  unsigned long str_size;
	  str >> str_size;
	  
	  string s_new = directory + "/" + filename;
	  QFile f(s_new.c_str());

	  if( (str_size == size) && (f.size() > current_size) )
	    {
	      s = s_new;
	      current_size = f.size();
	    }
	}
    }

  return s;
}


string DownloadThread::findFile(string file, string host)
{
  std::string::size_type  idx = file.find_last_of("/");

  string         directory = file.substr(0, idx);
  string         filename  = file.substr(idx+1);
  unsigned long  current_size = 0;

  QDir d(directory.c_str(), (filename+"*").c_str(), QDir::DefaultSort, QDir::Files);

  string s;

  for(int i = 0; i < d.count(); ++i)
    {
      filename = d[i];
      idx = filename.find(".part_");

      if( idx != std::string::npos )
	{
	  idx = filename.find("_", idx+6);
	  idx = filename.find("_", idx+1);
	  std::string::size_type  idx2 = filename.find("_", idx+1);

	  string s_host = filename.substr(idx+1, idx2-idx-1);

	  if( (s_host == host) )
	    {
	      string s_new = directory + "/" + filename;
	      QFile f(s_new.c_str());

	      if( f.size() > current_size )
		{
		  s = s_new;
		  current_size = f.size();
		}
	    }
	}
    }

  return s;
}


string DownloadThread::findFile(string file, unsigned long size, string host)
{
  std::string::size_type  idx = file.find_last_of("/");

  string         directory = file.substr(0, idx);
  string         filename  = file.substr(idx+1);
  unsigned long  current_size = 0;

  QDir d(directory.c_str(), (filename+"*").c_str(), QDir::DefaultSort, QDir::Files);

  string s;

  for(int i = 0; i < d.count(); ++i)
    {
      filename = d[i];
      idx = filename.find(".part_");

      if( idx != std::string::npos )
	{
	  idx  = filename.find("_", idx+6);

	  std::string::size_type  idx2;
	  std::string::size_type  idx3;

	  idx2 = filename.find("_", idx+1);
	  idx3 = filename.find("_", idx2+1);

	  string s_size = filename.substr(idx+1, idx2-idx-1);
	  string s_host = filename.substr(idx2+1, idx3-idx2-1);

	  strstream str;
	  str << s_size << ends;
	  unsigned long str_size;
	  str >> str_size;

	  if( (s_host == host) && (str_size == size) )
	    {
	      string s_new = directory + "/" + filename;
	      QFile f(s_new.c_str());

	      if( f.size() > current_size )
		{
		  s = s_new;
		  current_size = f.size();
		}
	    }
	}
    }

  return s;
}


void DownloadThread::slotUpdateStatus()
{
  static const string status[] = {"Idle", "Error", "Abort", "Closed", "Completed", "Closed",
				    "Connecting", "NotFound", "Timeout", "Busy", "Connected", "Exist",
				    "Queued" };

  string s[7];
  for(int i = 0; i < 7; ++i) s[i] = "";

  s[1] = status[readState()];

  if( (readState()==dtBusy) || (readState()==dtClosed) || (readState()==dtTimeout) || (readState()==dtError) )
    {
      s[1] += " - ";
      strstream str;
      str << readRetryTimeout() << "s" << ends;
      s[1] += str.str();
      str.freeze(false);
    }

  if( readState()==dtClosedCompleted )
    {
      QTime t(0, 0); 
      long  dt = _start_time.secsTo( _end_time );
      s[5] = t.addSecs(dt).toString();

      if(dt>0)
	{
	  double b = bytesRead() / dt / 1024;
	  strstream str;
	  str.setf(ios::fixed);
	  str << std::setprecision(2) << b << " KB/s" << ends;
	  s[4] = str.str();
	  str.freeze(false);
	}
    }

  if( readState()==dtConnected )
    {
      // bandwidth
      strstream str;
      str.setf(ios::fixed);
      str << std::setprecision(2) << readBandwidth() << " KB/s" << ends;
      s[4] = str.str();
      str.freeze(false);

      // calculate time to go
      QTime t; 
      t.start();
      long          dt     = _start_time.secsTo( t );
      unsigned long read   = bytesRead();
      unsigned long toread = bytesToRead();
      unsigned long tos    = 0;
      
      if(read>0)
	tos = toread * dt / read;
      
      t.setHMS(0,0,0);
      s[5] = t.addSecs(tos).toString();
    }

  // progress
  unsigned long long p = 100 * bytesRead() / _content_length;
  strstream str;
  str << p << "%" << ends;
  s[2] = str.str();
  str.freeze(false);

  s[0] = _qhe.filename;
  s[3] = _qhe.addr.strIP();
  s[6] = _client;

  for(int i = 0; i < 7; ++i) _item->setText(i, s[i].c_str());
}
