#include "QtellaSub.h"

#include "Connections.h"
#include "StringManipulation.h"
#include "QueryHitEntry.h"
#include "DownloadThread.h"
#include "Info.h"
#include "SearchWidget.h"
#include "Search.h"
#include "SharedFile.h"
#include "UploadManager.h"

#include <fstream>
#include <algorithm>
#include <iomanip>

#include <qspinbox.h>
#include <qlistbox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qtabwidget.h>
#include <qtabbar.h>
#include <qlabel.h>
#include <qfile.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qcombobox.h>
#include <qradiobutton.h>

#include <signal.h>

#ifdef KDE
#include "ktmainwindow.h"
#endif KDE

/* 
 *  Constructs a QtellaSub which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
QtellaSub::QtellaSub( QWidget* parent,  const char* name, WFlags fl )
  : Qtella( parent, name, fl )
{
  //  setWFlags(Qt::WStyle_Minimize);

  #ifdef KDE
    KPopupMenu *help = new KPopupMenu((KTMainWindow*)this);
  #endif KDE

  #ifdef QT
    QPopupMenu *help = new QPopupMenu(this);
  #endif QT

  help->insertItem("&Info", this, SLOT(slotInfo()));


  #ifdef KDE
    KPopupMenu *file = new KPopupMenu((KTMainWindow*)this);
  #endif KDE

  #ifdef QT
    QPopupMenu *file = new QPopupMenu(this);
  #endif QT

  file->insertItem("&Connect", this, SLOT(connection()));
  file->insertItem("E&xit", this, SLOT(myquit()));


  #ifdef KDE
    menu = new KMenuBar((KTMainWindow*)this);
  #endif KDE

  #ifdef QT
    menu = new QMenuBar(this);
  #endif QT

  menu->insertItem("&File", file);
  menu->insertSeparator();
  menu->insertItem("&Help", help);



  
  strstream str;
  str << "Qtella - " << VERSION << " - http://www.qtella.net" << ends;
  string caption = str.str();
  str.freeze(false);
  setCaption( caption.c_str() );

  connections = new Connections(this);


  timer_connection = new QTimer(this);
  connect(timer_connection, SIGNAL(timeout()), this, SLOT(check_connections()));

  timer_refresh_downloads = new QTimer(this);
  connect(timer_refresh_downloads, SIGNAL(timeout()), this, SLOT(slotRefreshDownloads()));
  timer_refresh_downloads->start(1000);

  connect((QObject*)ui_pushbutton_remove, SIGNAL(pressed()), (QObject*)connections, SLOT(slotRemoveHost()));
  connect((QObject*)ui_spinbox_hosts, SIGNAL(valueChanged(int)), (QObject*)connections, SLOT(slotMaxServents(int)));
  connect((QObject*)ui_pushbutton_listclear, SIGNAL(pressed()), (QObject*)connections, SLOT(slotListClear()));
  connect((QObject*)ui_pushbutton_listconnect, SIGNAL(pressed()), (QObject*)connections, SLOT(slotListConnect()));
  connect((QObject*)ui_pushbutton_listremove, SIGNAL(pressed()), (QObject*)connections, SLOT(slotListRemove()));


  ui_listview_download->setColumnWidthMode(0, QListView::Manual);
  ui_listview_download->setColumnWidthMode(1, QListView::Manual);
  ui_listview_download->setColumnWidthMode(2, QListView::Manual);
  ui_listview_download->setColumnWidthMode(3, QListView::Manual);
  ui_listview_download->setColumnWidthMode(4, QListView::Manual);
  ui_listview_download->setColumnWidthMode(5, QListView::Manual);
  ui_listview_download->setColumnWidthMode(6, QListView::Manual);
  ui_listview_download->setColumnWidth(0, 29*10);
  ui_listview_download->setColumnWidth(1, 10*10);
  ui_listview_download->setColumnWidth(2, 6*10);
  ui_listview_download->setColumnWidth(3, 10*10);
  ui_listview_download->setColumnWidth(4, 7*10);
  ui_listview_download->setColumnWidth(5, 7*10);
  ui_listview_download->setColumnWidth(6, 7*10);
  ui_listview_download->setColumnAlignment(0, AlignLeft);
  ui_listview_download->setColumnAlignment(1, AlignLeft);
  ui_listview_download->setColumnAlignment(2, AlignRight);
  ui_listview_download->setColumnAlignment(3, AlignRight);
  ui_listview_download->setColumnAlignment(4, AlignRight);
  ui_listview_download->setColumnAlignment(5, AlignRight);
  ui_listview_download->setColumnAlignment(6, AlignLeft);
  ui_listview_download->setSorting(-1);
  ui_listview_download->setColumnWidth(0, 30*10);
  
  ui_listview_connections->setColumnWidthMode(0, QListView::Manual);
  ui_listview_connections->setColumnWidthMode(1, QListView::Manual);
  ui_listview_connections->setColumnWidthMode(2, QListView::Manual);
  ui_listview_connections->setColumnWidthMode(3, QListView::Manual);
  ui_listview_connections->setColumnWidth(0, 23*10);
  ui_listview_connections->setColumnWidth(1, 11*10);
  ui_listview_connections->setColumnWidth(2, 11*10);
  ui_listview_connections->setColumnWidth(3, 7*10);
  ui_listview_connections->setColumnAlignment(0, AlignLeft);
  ui_listview_connections->setColumnAlignment(1, AlignLeft);
  ui_listview_connections->setColumnAlignment(2, AlignHCenter);
  ui_listview_connections->setColumnAlignment(3, AlignHCenter);
  ui_listview_connections->setSorting(-1);

  ui_listview_shared->setColumnWidthMode(0, QListView::Manual);
  //ui_listview_shared->setColumnWidthMode(1, QListView::Manual);
  //ui_listview_shared->setColumnWidthMode(2, QListView::Manual);
  //ui_listview_shared->setColumnWidthMode(3, QListView::Manual);
  ui_listview_shared->setColumnAlignment(0, AlignLeft);
  ui_listview_shared->setColumnAlignment(1, AlignRight);
  ui_listview_shared->setColumnAlignment(2, AlignRight);
  ui_listview_shared->setColumnAlignment(3, AlignRight);
  ui_listview_shared->setColumnWidth(0, 46*10);
  //ui_listview_shared->setColumnWidth(1, 6*10);
  //ui_listview_shared->setColumnWidth(2, 6*10);
  //ui_listview_shared->setColumnWidth(3, 9*10);

  ui_listview_uploads->setColumnWidthMode(0, QListView::Manual);
  ui_listview_uploads->setColumnWidthMode(1, QListView::Manual);
  ui_listview_uploads->setColumnWidthMode(2, QListView::Manual);
  ui_listview_uploads->setColumnWidthMode(5, QListView::Manual);
  ui_listview_uploads->setColumnAlignment(0, AlignLeft);
  ui_listview_uploads->setColumnAlignment(1, AlignLeft);
  ui_listview_uploads->setColumnAlignment(2, AlignRight);
  ui_listview_uploads->setColumnAlignment(3, AlignRight);
  ui_listview_uploads->setColumnAlignment(4, AlignRight);
  ui_listview_uploads->setColumnAlignment(5, AlignRight);
  ui_listview_uploads->setColumnWidth(0, 29*10);
  ui_listview_uploads->setColumnWidth(1, 10*10);
  ui_listview_uploads->setColumnWidth(2, 10*10);
  ui_listview_uploads->setColumnWidth(5, 7*10);

  readConfiguration();

  struct sigaction sa;
  sa.sa_handler = sighandler;

  sigaction(SIGPIPE, &sa, NULL);
  //sigaction(SIGSEGV, &sa, NULL);
  //sigaction(SIGABRT, &sa, NULL);

  this->slotRefreshSharedList();



  search_tabs = 0;

  SearchWidget *s = new SearchWidget(this);
  connect(s, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotSearchDoubleClicked(QListViewItem*)));

  ui_tabwidget_search->addTab(s, "");
  ui_tabwidget_search->showPage(s);

  ui_tabwidget_search->removePage(tab);



  Address *addr = new Address();
  addr->setPort((const char*)ui_lineedit_listenport->text());
  ui_lineedit_ip->setText(addr->strIP().c_str());
  ui_lineedit_listenport->setText(addr->strPort().c_str());

  upload_manager = new UploadManager(addr->port(), 0, this);
  if( !upload_manager->ok() ) cerr << "Error: Upload manager." << endl;

  delete addr;
}


void QtellaSub::readConfiguration()
{
  QDir d;
  string homedir = static_cast<string>(d.homeDirPath());
      
  ui_lineedit_downloaddirectory->setText(homedir.c_str());
  ui_lineedit_incompleted->setText(homedir.c_str());

  conffile  = homedir + "/.qtella";
  fstream   f;

  f.open(conffile.c_str(), ios::in );
  if(f)
    {
      while( !f.eof() )
	{
	  char   buffer[1024];
	  f.getline(buffer, 1023);
	  string input = buffer;
	  
	  StringManipulation s(input);
	  
	  s.remove_comments();
	  
	  string s1, s2;
	  s.split(s1, s2, '=');
	  
	  s.assign(s1);
	  s.remove_chars("\n ");
	  s1 = s.str();
	  
	  s.assign(s2);
	  s.remove_chars("\n ");
	  s2 = s.str();
	  
	  transform(s1.begin(), s1.end(), s1.begin(), toupper);
	  
	  if(!s1.empty())
	    {
	      if(s1=="DOWNLOAD")
		ui_lineedit_downloaddirectory->setText(s2.c_str());

	      if(s1=="INCOMPLETE")
		ui_lineedit_incompleted->setText(s2.c_str());
	
	      if(s1=="PORT")
		ui_lineedit_listenport->setText(s2.c_str());

	      if(s1=="SHAREDDIR")
		{
		  vector<string> vString;
		  s.assign(s2);
		  s.split_to_vector(vString, ';');
		  for(int i=0; i<vString.size(); i++)
		    ui_listbox_shareddirectories->insertItem(vString[i].c_str());
		}

	      if(s1=="SPEED")
		{
		  if(s2=="Modem") ui_combobox_speed->setCurrentItem(0);
		  if(s2=="ISDN") ui_combobox_speed->setCurrentItem(1);
		  if(s2=="ISDN2x") ui_combobox_speed->setCurrentItem(2);
		  if(s2=="Cable/DSL") ui_combobox_speed->setCurrentItem(3);
		  if(s2=="T1") ui_combobox_speed->setCurrentItem(4);
		  if(s2=="T3") ui_combobox_speed->setCurrentItem(5);
		}

	      if(s1=="MAXDOWNLOADS")
		{
		  strstream str;
		  int       max_downloads;
		  str << s2 << ends;
		  str >> max_downloads;
		  ui_spinbox_maxdownloads->setValue(max_downloads);
		}

 	      if(s1=="MAXUPLOADS")
		{
		  strstream str;
		  int       max_uploads;
		  str << s2 << ends;
		  str >> max_uploads;
		  ui_spinbox_maxuploads->setValue(max_uploads);
		}

 	      if(s1=="UPLOADBANDWIDTH")
		{
		  strstream str;
		  int       upload_bandwidth;
		  str << s2 << ends;
		  str >> upload_bandwidth;
		  ui_spinbox_upbandwidth->setValue(upload_bandwidth);
		}

	      if(s1=="AUTOCONNECT")
		{
		  vector<string> vString;
		  s.assign(s2);
		  s.split_to_vector(vString, ';');
		  for(int i=0; i<vString.size(); i++)
		    ui_listbox_autoconnect->insertItem(vString[i].c_str());
		}

	      if(s1=="EXISTING_FILES")
		{
		  if(s2=="0") ui_radiobutton_e_overwrite->setChecked(true);
		  else
		    if(s2=="1") ui_radiobutton_e_rename->setChecked(true);
		    else
		      ui_radiobutton_e_abort->setChecked(true);
		}

	      if(s1=="ABORT_DOWNLOADS")
		{
		  if(s2=="0") ui_radiobutton_a_contsize->setChecked(true);
		  else
		    if(s2=="1") ui_radiobutton_a_conthost->setChecked(true);
		    else
		      if(s2=="2") ui_radiobutton_a_contsizehost->setChecked(true);
		      else
			ui_radiobutton_a_startnew->setChecked(true);
		}
	    }
	}
      f.close();
    }
  else  // default configuration
    {
      ui_listbox_autoconnect->insertItem("gnutellahosts.com:6346");
    }
}


/*  
 *  Destroys the object and frees any allocated resources
 */
QtellaSub::~QtellaSub()
{
    // no need to delete child widgets, Qt does it all for us
}


void QtellaSub::connection()
{
  if(!timer_connection->isActive())
    {
      connections->start();
      timer_connection->start(500);
    }
}


void QtellaSub::slotAddHost()
{
  if(!timer_connection->isActive())
    timer_connection->start(500);
  connections->slotAddHost();
}


void QtellaSub::check_connections()
{
  connections->check_servents();
}


void QtellaSub::myquit()
{
  exit(0);
}

void QtellaSub::slotAddDirectory()
{
  string s = (const char*)ui_lineedit_directory->text();
  
  if(!s.empty())
    ui_listbox_shareddirectories->insertItem(s.c_str());

  ui_lineedit_directory->setText("");
}

void QtellaSub::slotRemoveDirectory()
{
  int index = ui_listbox_shareddirectories->currentItem();

  if(index>-1)
    ui_listbox_shareddirectories->removeItem(index);
}

void QtellaSub::slotApplyConfig()
{
  fstream         f;

  f.open(conffile.c_str(), ios::out|ios::trunc);
  if(!f)
    {
      cerr << "Cannot open configuration file. " << conffile << endl;
      return;
    }

  f << "DOWNLOAD=" << (const char*)ui_lineedit_downloaddirectory->text() << endl
    << "INCOMPLETE=" << (const char*)ui_lineedit_incompleted->text() << endl
    << "PORT=" << (const char*)ui_lineedit_listenport->text() << endl
    << "SPEED=" << (const char*)ui_combobox_speed->currentText() << endl
    << "MAXDOWNLOADS=" << ui_spinbox_maxdownloads->value() << endl
    << "MAXUPLOADS=" << ui_spinbox_maxuploads->value() << endl
    << "UPLOADBANDWIDTH=" << ui_spinbox_upbandwidth->value() << endl
    << "SHAREDDIR=";

  for(int i=0; i<ui_listbox_shareddirectories->count(); i++)
    f << (const char*)ui_listbox_shareddirectories->text(i) << ";";

  f << endl;

  f << "AUTOCONNECT=";

  for(int i=0; i<ui_listbox_autoconnect->count(); i++)
    f << (const char*)ui_listbox_autoconnect->text(i) << ";";

  f << endl;

  f << "EXISTING_FILES=";
  if(ui_radiobutton_e_overwrite->isChecked()) f << "0";
  if(ui_radiobutton_e_rename->isChecked()) f << "1";
  if(ui_radiobutton_e_abort->isChecked()) f << "2";
  f << endl;

  f << "ABORT_DOWNLOADS=";
  if(ui_radiobutton_a_contsize->isChecked()) f << "0";
  if(ui_radiobutton_a_conthost->isChecked()) f << "1";
  if(ui_radiobutton_a_contsizehost->isChecked()) f << "2";
  if(ui_radiobutton_a_startnew->isChecked()) f << "3";
  f << endl;

  f.close();
}


void QtellaSub::slotRefreshSharedList()
{
  std::vector<SharedFile*>  old = readSharedFiles();

  number_shared_files = 0;
  shared_size = 0;

  mutex_shared_files.lock();

  vSharedFiles.clear();

  for(int i=0; i<ui_listbox_shareddirectories->count(); i++)
    {
      string directory = (const char*)ui_listbox_shareddirectories->text(i);
      QDir   dir(directory.c_str());

      if(dir.exists())
	{
	  QStringList list = dir.entryList(QDir::Files);

	  for( QStringList::Iterator it = list.begin(); it!=list.end(); ++it )
	  {
	    string filename = (*it).latin1();

	    if(filename!="." && filename!="..")
	      {
		string complete = directory;
		if(complete[complete.size()-1]!='/') complete += "/";
		complete += filename;

		QFile f(complete.c_str());
		
		if(f.exists())
		  {
		    number_shared_files++;
		    QListViewItem *item = new QListViewItem(ui_listview_shared, filename.c_str());

		    item->setText(1, "0");  // uploads
		    item->setText(2, "0");  // requests

		    shared_size += f.size();

		    strstream stream_fsize;
		    stream_fsize << f.size() << ends;
		    string str_fsize = stream_fsize.str();
		    item->setText(3, str_fsize.c_str());

		    vSharedFiles.push_back(new SharedFile(item, filename, directory, f.size()));

		    for(int i = 0; i < old.size(); i++)
		      if( *old[i] == *vSharedFiles.back() )
			{
			  int up = old[i]->uploads;
			  int rq = old[i]->requests;

			  vSharedFiles.back()->uploads = up;
			  vSharedFiles.back()->requests = rq;

			  strstream s1;
			  strstream s2;
			  s1 << up << ends;
			  s2 << rq << ends;
			  string str1 = s1.str();
			  string str2 = s2.str();
			  item->setText(1, str1.c_str());
			  item->setText(2, str2.c_str());
			}
		  }
	      }
	  }

	}
    }

  for(int i=0; i<old.size(); i++) delete old[i];
  mutex_shared_files.unlock();
}



void QtellaSub::downloadItem(QListViewItem *item)
{
  int i = 0;
  int m = 0;

  for(int mp=0; mp<connections->vSearch.size(); mp++)
    for(int ip=0; ip<connections->vSearch[mp]->vSearchResults.size(); ip++)
      if( connections->vSearch[mp]->vSearchResults[ip]->item == item) 
	{
	  m = mp;
	  i = ip;
	}

  string path = (const char*)ui_lineedit_downloaddirectory->text();
  string incomplete = (const char*)ui_lineedit_incompleted->text();

  QListViewItem *item_download = new QListViewItem(ui_listview_download);
  item_download->setText(0, connections->vSearch[m]->vSearchResults[i]->filename.c_str() );
  item_download->setText(3, connections->vSearch[m]->vSearchResults[i]->addr.strIP().c_str() );
  ui_listview_download->insertItem(item_download);

  DownloadThread *dt = new DownloadThread( *(connections->vSearch[m]->vSearchResults[i]), item_download, this );
  vDownloadThreads.push_back(dt);

  if( readNumberOfDownloads() < ui_spinbox_maxdownloads->value() )
    {
      dt->start();
      while( (dt->readState() != DownloadThread::dtQueued) );
    }
}


int QtellaSub::readNumberOfDownloads()
{
  int n = 0;
  std::vector<DownloadThread*>::iterator  pos = vDownloadThreads.begin();

  for( pos; pos != vDownloadThreads.end(); ++pos )
    if( ((*pos)->readState() == DownloadThread::dtConnected) ||
	((*pos)->readState() == DownloadThread::dtConnecting) ) ++n;

  return n;
}


void QtellaSub::slotSearchDoubleClicked(QListViewItem *item)
{
  downloadItem(item);
  Tabular->setCurrentPage(2);
}


void QtellaSub::slotRefreshDownloads()
{
  double bandwidth = 0;

  for(int i = 0; i < vDownloadThreads.size(); i++)
    if(vDownloadThreads[i]->readState() == DownloadThread::dtConnected)
      bandwidth += vDownloadThreads[i]->readBandwidth();
    else
      if(vDownloadThreads[i]->readState() == DownloadThread::dtQueued)
	if( readNumberOfDownloads() < ui_spinbox_maxdownloads->value() )
	  {
	    vDownloadThreads[i]->start();
	    while( (vDownloadThreads[i]->readState() != DownloadThread::dtConnecting) && (vDownloadThreads[i]->readState() != DownloadThread::dtError) );
	  }

  strstream str;
  str.setf(ios::fixed);
  str << std::setprecision(2) << bandwidth << " KB/s" << ends;

  ui_textlabel_bandwidth->setText(str.str());
  str.freeze(false);
}


void QtellaSub::slotSearch()
{
  string query = (const char*)ui_lineedit_search->text();


  if(search_tabs==0)
    {
      SearchWidget *w = (SearchWidget*) ui_tabwidget_search->currentPage();
      w->ui_listview_search->clear();
      ui_tabwidget_search->changeTab(w, query.c_str());
      connections->search(query, w);
    }
  else
    {
      SearchWidget *s = new SearchWidget(this);
      connect(s, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotSearchDoubleClicked(QListViewItem*)));
      ui_tabwidget_search->addTab(s, query.c_str());
      ui_tabwidget_search->showPage(s);
      connections->search(query, s);
    }

  search_tabs++;

}


void QtellaSub::slotClearSearchlist()
{
  SearchWidget *s = (SearchWidget*) ui_tabwidget_search->currentPage();

  for(int i=0; i<connections->vSearch.size(); i++)
    if(connections->vSearch[i]->sw == s) 
      {
	connections->vSearch[i]->active = false;

	for(int j=0; j < connections->vSearch[i]->vSearchResults.size(); j++)
	  delete connections->vSearch[i]->vSearchResults[j];

	break;
      }

  if(search_tabs==1)
    {
      s->ui_listview_search->clear();
      ui_tabwidget_search->changeTab(s, "");
      search_tabs--;
    }

  if(search_tabs>1)
    {
      ui_tabwidget_search->removePage(s);
      delete s;
      search_tabs--;
    }
}


void QtellaSub::slotDownloadAll()
{
  int          m;
  SearchWidget *s = (SearchWidget*) ui_tabwidget_search->currentPage();

  for(int i=0; i<connections->vSearch.size(); i++)
    if(connections->vSearch[i]->sw == s) m = i;

  for(int i=0; i<connections->vSearch[m]->vSearchResults.size(); i++)
    if( connections->vSearch[m]->vSearchResults[i]->item->isSelected() )
      downloadItem(connections->vSearch[m]->vSearchResults[i]->item);

  Tabular->setCurrentPage(2);
}


void QtellaSub::slotAbortDownload()
{
  int i;

  for(i=0; i < vDownloadThreads.size(); i++)
    if( vDownloadThreads[i]->_item->isSelected() ) vDownloadThreads[i]->abortDownload();
}


void QtellaSub::slotResumeDownload()
{
  int i;

  for(i=0; i < vDownloadThreads.size(); i++)
    if( vDownloadThreads[i]->_item->isSelected() ) vDownloadThreads[i]->resumeDownload();
}


void QtellaSub::slotRemoveDownload()
{
  std::vector<DownloadThread*>::iterator  pos;

  for(pos=vDownloadThreads.begin(); pos!=vDownloadThreads.end(); )
    if( (*pos)->_item->isSelected() )
      {
	delete *pos;
	pos = vDownloadThreads.erase(pos);
      }
    else
      ++pos;
}


void QtellaSub::slotAddAutoConnect()
{
  string s = (const char*)ui_lineedit_autoconnect->text();
  
  if(!s.empty())
    ui_listbox_autoconnect->insertItem(s.c_str());

  slotApplyConfig();
  ui_lineedit_autoconnect->setText("");
}


void QtellaSub::slotRemoveAutoConnect()
{
  int index = ui_listbox_autoconnect->currentItem();

  if(index>-1)
    ui_listbox_autoconnect->removeItem(index);

  slotApplyConfig();
}


void QtellaSub::slotInfo()
{
  Info *i = new Info(this, "Info", true);
  i->ui_textlabel_version->setText(VERSION);
  i->exec();
}


void QtellaSub::sighandler(int i)
{
  cout << "signal: " << i << endl;
}


vector<SharedFile*> QtellaSub::readSharedFiles()
{
  mutex_shared_files.lock();
  vector<SharedFile*> v = vSharedFiles;
  mutex_shared_files.unlock();

  return v;
}


void QtellaSub::slotAbortUpload()
{
  upload_manager->abortUpload();
}


void QtellaSub::slotRemoveUpload()
{
  upload_manager->removeUpload();
}


void QtellaSub::slotDeleteSharedFile()
{
  std::vector<SharedFile*>::iterator  pos = vSharedFiles.begin();
  
  for(pos; pos != vSharedFiles.end(); ++pos)
    if( (*pos)->item->isSelected() )
      {
	QFile f( (*pos)->completeFilename().c_str() );
	f.remove();
      }

  slotRefreshSharedList();
}


void QtellaSub::slotClearInactiveDownloads()
{
  std::vector<DownloadThread*>::iterator  pos;

  for(pos=vDownloadThreads.begin(); pos!=vDownloadThreads.end(); )
    if( ((*pos)->readState() != DownloadThread::dtConnecting) && ((*pos)->readState() != DownloadThread::dtConnected) )
      {
	delete *pos;
	pos = vDownloadThreads.erase(pos);
      }
    else
      ++pos;
}


void QtellaSub::slotClearInactiveUploads()
{
  upload_manager->removeInactiveUploads();
}
