#include "Servent.h"

#include "Pong.h"
#include "Ping.h"
#include "Query.h"
#include "Host.h"
#include "QueryHit.h"
#include "Push.h"
#include "Address.h"

#include <string>
#include <vector>

#include <qsocket.h>
#include <qthread.h>
#include <qstring.h>
#include <qlistview.h>


#undef QTELLA_DEBUG

Servent::Servent(string id, QListViewItem *item)
  : _id(id), _item(item), _timeout(0), _input(0), _output(0), _state(serventIdle)
{
  _connect_str     = "GNUTELLA CONNECT/0.4\n\n";
  _response_str    = "GNUTELLA OK\n\n";

  connect(this, SIGNAL(error(int)), this, SLOT(slotError(int)));
  connect(this, SIGNAL(connected()), this, SLOT(slotConnect()));
  connect(this, SIGNAL(readyRead()), this, SLOT(slotReadyRead()));
  connect(this, SIGNAL(connectionClosed()), this, SLOT(slotClosed()));
  connect(this, SIGNAL(delayedCloseFinished()), this, SLOT(slotDelayedClosed()));
}


Servent::~Servent()
{
  for(int i=0; i<pong_list.size(); i++) { delete pong_list.back(); pong_list.pop_back(); }
  for(int i=0; i<vPing.size(); i++)     { delete vPing.back(); vPing.pop_back(); }
  for(int i=0; i<vQuery.size(); i++)    { delete vQuery.back(); vQuery.pop_back(); }
  for(int i=0; i<vQueryHit.size(); i++) { delete vQueryHit.back(); vQueryHit.pop_back(); }
  for(int i=0; i<host_list.size(); i++) { delete host_list.back(); host_list.pop_back(); }
  for(int i=0; i<vPush.size(); i++)     { delete vPush.back(); vPush.pop_back(); }

  if(_item) delete _item;
  
  closeConnection();
}


void Servent::connectToServent(Address address)
{
  #ifdef QTELLA_DEBUG
    cout << "connecting" << endl;
  #endif

  _strip.erase();
  _strip = address.strIP();

  setState(serventConnecting);
  _timeout = 0;
  connectToHost(_strip.c_str(), address.port());
}



void Servent::slotDelayedClosed()
{
  setState(serventClosed);
}


void Servent::closeConnection()
{
  if(state() != QSocket::Idle) 
    {
      close();

      #ifdef QTELLA_DEBUG
        cout << "closing..." << endl;
        while(state() != QSocket::Idle);
        cout << "closed" << "/" << state() << endl;
      #endif
    }

  setState(serventClosed);
}



int Servent::serventState()
{
  _state_mutex.lock();
  int i = _state;
  _state_mutex.unlock();
  return i;
}



void Servent::setState(int state)
{
  _state_mutex.lock();
  _state = state;
  _state_mutex.unlock();
}



void Servent::slotError(int err)
{
  if(state() != QSocket::Idle) close();
  setState(serventError);
}



void Servent::writeConnectString()
{
  sendData(_connect_str);
}



void Servent::slotConnect()
{
  #ifdef QTELLA_DEBUG
    cout << "connected. write connect string." << endl;
  #endif

  setState(serventWaiting);
  writeConnectString();

  #ifdef QTELLA_DEBUG
    cout << "state_c: " << state() << endl;
  #endif
}



void Servent::slotClosed()
{
  #ifdef QTELLA_DEBUG
    cout << "slotClosed" << endl;
  #endif

  setState(serventClosed);
}



string Servent::createHeader(string id, int func, int ttl, int hops, long payload)
{
  string header = id;

  header += static_cast<char>(func);
  header += static_cast<char>(ttl);
  header += static_cast<char>(hops);

  header.append( (char*)&payload, 4 );
  
  return header;
}


  
void Servent::checkHeader(string header, string &id, int& func, int& ttl, int& hops, long& payload)
{
  id       = header.substr(0, 16);
  func     = static_cast<int>((unsigned char)header[16]);
  ttl      = static_cast<int>((unsigned char)header[17]);
  hops     = static_cast<int>((unsigned char)header[18]);

  payload  = *( (long*)(header.substr(19, 4).c_str()) );
}



void Servent::sendPing(int ttl, int hops)
{
  sendPing(ttl, hops, _id);
}



void Servent::sendPing(int ttl, int hops, string id)
{
  string header = createHeader(id, CMD_PING, ttl, hops, 0);

  sendData(header);
}



void Servent::sendPong(int ttl, int hops, string id, Address address, long files, long kbytes)
{
  short unsigned port = address.port();
  long           ip   = address.longIP();

  string header = createHeader(id, CMD_PONG, ttl, hops, 14);

  header.append( (char*)&port, 2);
  header.append( (char*)&ip, 4);
  header.append( (char*)&files, 4);
  header.append( (char*)&kbytes, 4);

  sendData(header);
}



void Servent::sendQuery(int ttl, int hops, string id, int min_speed, string search)
{
  string header = createHeader(id, CMD_QUERY, ttl, hops, search.size()+2+1);

  header.append( (char*)&min_speed, 2 );
  header.append(search);
  header.append("\0", 1);

  sendData(header);
}



void Servent::sendQueryHit(int ttl, int hops, QueryHit queryhit)
{
  short unsigned port = queryhit.address.port();
  long           ip   = queryhit.address.longIP();

  int l = 0;

  // calculate size of payload data

  for(int i=0; i<queryhit.vQueryResult.size(); i++)
    l += 2+4+4+queryhit.vQueryResult[i].filename.size();

  l += queryhit.trailer.size();

  // create header

  string header = createHeader(queryhit.id, CMD_QUERYHIT, ttl, hops, 11+16+l);

  header.append( (char*)&queryhit.hits, 1 );
  header.append( (char*)&port, 2 );
  header.append( (char*)&ip, 4 );
  header.append( (char*)&queryhit.speed, 4 );

  for(int i=0; i<queryhit.vQueryResult.size(); i++)
    {
      header.append( (char*)&queryhit.vQueryResult[i].index, 4 );
      header.append( (char*)&queryhit.vQueryResult[i].filesize, 4 );
      header.append( queryhit.vQueryResult[i].filename );
      header.append("\0\0", 2);
    }

  header.append( queryhit.trailer );
  header.append( queryhit.id_result );

  sendData(header);
}



void Servent::sendPush(int ttl, int hops, Push push)
{
  short unsigned port  = push.address.port();
  unsigned long  ip    = push.address.longIP();
  unsigned long  index = push.index;

  string header = createHeader(push.id_header, CMD_PUSH, ttl, hops, 16+4+4+2);

  header.append( push.id_payload );
  header.append( (char*)&index, 4 );
  header.append( (char*)&ip, 4 );
  header.append( (char*)&port, 2 );

  sendData(header);
}


void Servent::sendData(string buffer)
{
  #ifdef QTELLA_DEBUG
    cout << "sendData" << "/" << state() << "/" << buffer.size() << endl;
  #endif

  if(state() == QSocket::Connection)
    {
      writeBlock(buffer.c_str(), buffer.size());
      _output++;
    }

  #ifdef QTELLA_DEBUG
    flush();
  #endif

  /*
  while( (state()==QSocket::Connection) && (bytesToWrite()>0) )
    {
      cout << "waiting: " << bytesToWrite() << endl;
      sleep(1);
    }
  */

  #ifdef QTELLA_DEBUG
    cout << "written: " << state() << "/" << bytesToWrite() << endl;
    sleep(1);
    cout << "state: " << state() << endl;
  #endif
}


void Servent::limitVectors()
{
  while(pingid_list.size() > 1000) pingid_list.erase(pingid_list.begin());
  while(queryid_list.size() > 1000) queryid_list.erase(queryid_list.begin());

  while(host_list.size() > 2000) 
    {
      delete host_list.front();
      host_list.pop_front();
    }
}



void Servent::slotReadyRead()
{
  long   payload = 0;
  char   cbuff[4096+10];

  #ifdef QTELLA_DEBUG
    cout << "ready read" << endl;
  #endif

  do
    {
      int rd = readBlock(cbuff, 4096);
      _buffer.append(cbuff, rd);

      #ifdef QTELLA_DEBUG
        cout << "read: " << rd;
      #endif
    }
  while(bytesAvailable() > 0);


  if( (serventState() == serventWaiting) && (_buffer.size() >= _response_str.size()) )
    {
      if(_response_str == _buffer.substr(0, _response_str.size()))
	{
	  setState(serventConnected);
	  _buffer.erase(0, _response_str.size());

	  sendPing(7, 0);  // send initial ping
	  
	  #ifdef QTELLA_DEBUG
	    cout << "send initial ping/" << state() << endl;
	  #endif

	  return;
	}
    }

  else

    while( _buffer.size() >= (23+payload) )  // header size
      {
	string id;
	int    function, ttl, hops;

	checkHeader(_buffer.substr(0,23), id, function, ttl, hops, payload);

	if(payload > 20000)  // invalid payload length - disconnect
	  {
	    _buffer.erase();
	    setState(serventError);
	  }
	
	if( _buffer.size() >= (23+payload) )
	  {
	    _buffer.erase(0, 23);  // remove header from payload data

	    _input++;
	    _list_mutex.lock();

	    switch(function)
	      {
	      case CMD_PONG:
		{
		  unsigned long ip     = *((unsigned long*)_buffer.substr(2, 4).c_str());
		  unsigned int  port   = *((unsigned short int*)_buffer.substr(0, 2).c_str());
		  unsigned long files  = *((unsigned long*)_buffer.substr(6, 4).c_str());
		  unsigned long kbytes = *((unsigned long*)_buffer.substr(10,4).c_str());

		  if( id == _id )
		    host_list.push_back(new Host( Address(ip, port), files, kbytes));
		  //else
		    //pong_list.push_back(new Pong(id, ttl, hops, Address(ip, port), files, kbytes));
		}
		break;


	      case CMD_PING:
		vPing.push_back(new Ping(id, ttl, hops));
		pingid_list.push_back(id);
		break;


	      case CMD_QUERY:
		{
		  #ifdef QTELLA_DEBUG
		    cout << "query" << endl;
		  #endif

		  int speed = *((short int*)_buffer.substr(0, 2).c_str());
		  string search = _buffer.substr(2, payload-2);

		  vQuery.push_back(new Query(id, ttl, hops, speed, search));
		  queryid_list.push_back(id);
		}
		break;


	      case CMD_QUERYHIT:
		#ifdef QTELLA_DEBUG
		  cout << "queryhit" << endl;
		#endif

		vQueryHit.push_back(new QueryHit(id, ttl, hops, _buffer.substr(0, payload)));
		break;


	      case CMD_PUSH:
		#ifdef QTELLA_DEBUG
		  cout << "push" << endl;
		#endif

		vPush.push_back(new Push(id, ttl, hops, _buffer.substr(0, payload)));
		break;

	      default:
		setState(serventError);
		payload = 0;
		_buffer.erase();
	      }

	    _list_mutex.unlock();
	    if(payload>0) _buffer.erase(0, payload);
	  }
	
	payload = 0;
	if(_buffer.size()>=23) checkHeader(_buffer.substr(0,23), id, function, ttl, hops, payload);
      }
}
