#include "UploadManager.h"
#include "QtellaSub.h"
#include "UploadThread.h"

#include "qhostaddress.h"
#include "qsocket.h"
#include "qstring.h"
#include "qspinbox.h"
#include "qlistview.h"

#include <unistd.h>

UploadManager::UploadManager(Q_UINT16 port, int backlog = 0, QObject *parent = 0) 
  : QServerSocket(port, backlog, parent, 0)
{
  this->parent = (QtellaSub*) parent;
}


void UploadManager::newConnection(int socket)
{
  UploadThread *ut;
  int          upload_counter = int();

  list<UploadThread*>::iterator  pos = upload_list.begin();

  while( pos != upload_list.begin() )
    if( (*pos)->getState() == UploadThread::Busy )
      {
	delete *pos;
	pos = upload_list.erase(pos);
      }
    else
      ++pos;

  for( pos = upload_list.begin(); pos != upload_list.end(); ++pos )
    if( ((*pos)->getState() == UploadThread::Connected) || ((*pos)->getState() == UploadThread::Start) )
      upload_counter++;

  if(upload_counter == parent->ui_spinbox_maxuploads->value())
    ut = new UploadThread(this, socket, UploadThread::Busy);
  else
    ut = new UploadThread(this, socket, UploadThread::Start);

  upload_list.push_back(ut);
  ut->start();
}


void UploadManager::abortUpload()
{
  std::list<UploadThread*>::iterator  pos;

  for(pos = upload_list.begin(); pos != upload_list.end(); ++pos)
    if( parent->ui_listview_uploads->isSelected( (*pos)->item ) )
      {
	(*pos)->abortUpload();
	break;
      }
}


void UploadManager::removeUpload()
{
  std::list<UploadThread*>::iterator  pos = upload_list.begin();

  while( pos != upload_list.end() )
    if( parent->ui_listview_uploads->isSelected( (*pos)->item ) )
      {
	(*pos)->abortUpload();
	while( (*pos)->running() );
	delete *pos;
	pos = upload_list.erase(pos);
      }
    else
      ++pos;
}


void UploadManager::removeInactiveUploads()
{
  std::list<UploadThread*>::iterator  pos = upload_list.begin();

  while( pos != upload_list.end() )
    if( ((*pos)->getState() != UploadThread::Start) && ((*pos)->getState() != UploadThread::Connected) )
      {
	delete *pos;
	pos = upload_list.erase(pos);
      }
    else
      ++pos;
}
