#include "UploadThread.h"
#include "UploadManager.h"
#include "SharedFile.h"
#include "QtellaSub.h"

#include "qsocket.h"
#include "qlistview.h"
#include "qfile.h"
#include "qdatetime.h"
#include "qspinbox.h"
#include "qhostaddress.h"

#include <iostream>
#include <iomanip>
#include <strstream>

const string UploadThread::statestr[] = {"Busy", "Start", "Connected", "NotFound", 
					 "Finished", "Error", "Abort", "Closed"};

UploadThread::UploadThread(UploadManager *parent, int socket, int state)
  : state(state), parent(parent)
{
  this->socket = new QSocket();
  this->socket->setSocket(socket);

  connect(this->socket, SIGNAL(connectionClosed()), this, SLOT(slotClosed()));

  item = NULL;
}


void UploadThread::run()
{
  string s;

  do
    {
      int timeout = 0;
 
      while( (!socket->canReadLine()) && (timeout < 30) ) 
	{
	  msleep(100);
	  timeout++;
	}
      
      if(timeout>=30)
	{
	  socket->close();
	  setState(Error);
	  return;
	}
      
      string n = (const char*) socket->readLine();
      s += n;
    }
  while(s.find("\r\n\r\n")==std::string::npos);


  if(s.substr(0,3)!="GET")
    {
      socket->close();
      return;
    }

  std::string::size_type  idx = s.find("/get/");
  
  if(idx==std::string::npos)
    {
      socket->close();
      return;
    }

  idx += 5;
  std::string::size_type  idx2 = s.find("/", idx);

  if(idx==std::string::npos)
    {
      socket->close();
      return;
    }

  string sindex = s.substr(idx, idx2-idx);

  strstream a;
  a << sindex << ends;
  a >> index;


  
  idx = s.find(" HTTP", idx2);

  if(idx==std::string::npos)
    {
      socket->close();
      return;
    }
  
  filename = s.substr(idx2+1, idx-idx2-1);



  if( (idx = s.find("User-Agent:")) != std::string::npos )
    {
      idx = s.find_first_not_of(" ", idx+11);
      idx2 = s.find("\r\n", idx);
      client = s.substr(idx, idx2-idx);
    }
  else
    client = "";



  if( (idx = s.find("Range:")) != std::string::npos )
    {
      idx = s.find_first_of("1234567890", idx+6);
      idx2 = s.find("-", idx);
      string srange_start = s.substr(idx, idx2-idx);

      strstream c;
      c << srange_start << ends;
      c >> range_start;

      idx = s.find("\r\n", idx2);
      if( (idx = s.find_first_of("0123456789", idx2+1)) != std::string::npos )
	{
	  idx2 = s.find_first_not_of("0123456789", idx);
	  string srange_end = s.substr(idx, idx2-idx);

	  strstream b;
	  b << srange_end << ends;
	  b >> range_end;
	}
      else range_end = 0;
    }
  else
    {
      range_start = 0;
      range_end = 0;
    }

  if(index < parent->parent->vSharedFiles.size())
    if( parent->parent->vSharedFiles[index]->file == filename )
      {
	parent->parent->vSharedFiles[index]->requests++;
	strstream str;
	str << parent->parent->vSharedFiles[index]->requests << ends;
	parent->parent->vSharedFiles[index]->item->setText(2, str.str());
	str.freeze(false);
      }

  if(getState() == Busy)
    {
      strstream str;
      string    s;
      str << "HTTP/1.0 503 Busy\r\nServer: Qtella " << VERSION << "\r\n\r\n" << ends;
      s = str.str();
      str.freeze(false);
      socket->writeBlock(s.c_str(), s.size());
      socket->close();
      return;
    }

  if(state == Start) startUpload();
}


void UploadThread::startUpload()
{
  string f = parent->parent->vSharedFiles[index]->directory;
  
  if( f[f.size()-1] != '/' ) f += '/';
  f += parent->parent->vSharedFiles[index]->file;

  item = new QListViewItem(parent->parent->ui_listview_uploads, parent->parent->vSharedFiles[index]->file.c_str());
  item->setText(2, socket->peerAddress().toString());

  QFile file(f.c_str());

  if( !file.exists() )
    {
      strstream str;
      string    s;
      str << "HTTP/1.0 404 NOT FOUND\r\nServer: Qtella " << VERSION << "\r\n\r\n" << ends;
      s = str.str();
      str.freeze(false);
      socket->writeBlock(s.c_str(), s.size());
      socket->close();
      setState(NotFound);
      return;
    }

  setState(Connected);

  if(range_end == 0) range_end = file.size() - 1;

  unsigned long towrite = range_end - range_start + 1;

  strstream str;
  str << "HTTP/1.0 200 OK\r\n"
      << "Content-Length: " << file.size() << "\r\n"    // must be file.size() !!
      << "Content-Type: " << "application/binary\r\n"
      << "Server: Qtella " << VERSION << "\r\n"
      << "\r\n" << ends;

  string buffer = str.str();
  str.freeze(false);
  socket->writeBlock(buffer.c_str(), buffer.size());

  if( !file.open(IO_ReadOnly|IO_Raw) )
    {
      setState(Error);
      socket->close();
      return;
    }

  file.at(range_start);

  int    msec   = 0;
  char   cbuff[8192];
  QTime  t;
  int    r = 0;
  int    w;
  int    bytes_written = range_start;
  double b;

  QTime  time_start;
  time_start.start();

  do
    {
      double        bandwidth = parent->parent->ui_spinbox_upbandwidth->value() * 1.024;

      unsigned long bytes_to_read = 1024;
      if(bandwidth > 32) bytes_to_read = 8192;
      if( (range_end+1 - bytes_written) < bytes_to_read ) bytes_to_read = range_end+1 - bytes_written;

      t.restart();
      unsigned long written = 0;
      
      w = 0;
      r = file.readBlock(cbuff, bytes_to_read);
      if(socket->state() == QSocket::Connection) w = socket->writeBlock(cbuff, r);

      bytes_written += w;
      written += w;

      if(w != r)
	{
	  cerr << "Upload error. (" << r << "/" << w << ")" << endl;
	  setState(Error);
	  socket->close();
	  file.close();
	  return;
	}

      while( (socket->bytesToWrite() > 0) && (socket->state() == QSocket::Connection) ) msleep(10);
      while( (b=static_cast<double>(1000*written) / static_cast<double>(t.elapsed()*1024)) > bandwidth )
	msleep(10);

      // bandwidth
      strstream str;
      str.setf(ios::fixed);
      str << std::setprecision(2) << b << " KB/s" << ends;
      item->setText(4, str.str());
      str.freeze(false);

      // progress
      str.seekp(0, ios::beg);
      str << 100 * bytes_written / (range_end+1) << "%" << ends;
      item->setText(3, str.str());
      str.freeze(false);

      // time left
      double u = static_cast<double>(bytes_written-range_start) / static_cast<double>(time_start.elapsed());
      unsigned long sectogo = static_cast<unsigned long>((towrite-(bytes_written-range_start)) / (u*1000));
      QTime time_togo = QTime();
      time_togo = time_togo.addSecs(sectogo);
      item->setText(5, (const char*)time_togo.toString());

    }
  while( ((bytes_written-range_start) < towrite) && (socket->state() == QSocket::Connection) && (getState() != Abort));

  if( (socket->state() == QSocket::Connection) && (socket->bytesToWrite() > 0) && (getState() != Abort) )
    {
      setState(Error);
      socket->close();
      file.close();
      return;
    }

  if(socket->state() == QSocket::Connection) socket->close();

  // final time
  QTime time_end = QTime();
  time_end = time_end.addMSecs(time_start.elapsed());
  item->setText(5, (const char*)time_end.toString());

  // final bandwidth
  b = static_cast<double>((bytes_written-range_start)*1000) / static_cast<double>(time_start.elapsed()*1024);
  str.seekp(0, ios::beg);
  str.setf(ios::fixed);
  str << std::setprecision(2) << b << " KB/s" << ends;
  item->setText(4, str.str());
  str.freeze(false);

  if( (getState() != Abort) && (getState() != Error) )
    {
      setState(Finished);

      parent->parent->vSharedFiles[index]->uploads++;
      str.seekp(0, ios::beg);
      str << parent->parent->vSharedFiles[index]->uploads << ends;
      parent->parent->vSharedFiles[index]->item->setText(1, str.str());
      str.freeze(false);
    }

  file.close();
}


UploadThread::~UploadThread()
{
  if(socket)
    delete socket;
  if(item)
    delete item;
}


int UploadThread::getState()
{
  return state;
}


void UploadThread::slotClosed()
{
  setState(Closed);
}

void UploadThread::abortUpload()
{
  socket->close();
  setState(Abort);
}

void UploadThread::setState(int state)
{
  if( this->state == Finished ) return;

  this->state = state;

  if(item)
    item->setText(1, statestr[state].c_str());
}
