/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: DBAdapterServlet.java,v 1.4 2001/06/12 23:04:45 ddoolin Exp $
 */

package net.jxta.search.dbadapter;

import java.io.*;
import java.sql.*;
import javax.servlet.http.*;
import javax.servlet.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;


/**
 * This servlet adapter provides an example of how
 * to construct a node for answering queries in
 * jxtasearch.  Most of the work should be done
 * in helper classes, the servlet should just dispatch
 * the Request for some action and serve a Response
 * from that action back to the consumer.
 */
public class DBAdapterServlet extends HttpServlet {


    private static boolean debug = false;
    QueryHandler qh;

    public static void setDebug() {
	debug = true;
    }

    public static void clearDebug() {
	debug = false;
    }


    /**
     *
     */
    public void init() throws ServletException {

	System.out.println("Init DBAdapter Servlet");
	qh = new QueryHandler();
    }

    /**
     * The usual suspects, we get the args from a search 
     * request such as from the jxtasearch router.
     * 
     * TODO: Extract all the goodies from this method so that
     * Requests parses itself, etc.  Then call some sort of 
     * action, then grab a Response to format the results of 
     * the action and send it back to the consumer.  In other
     * words, there should not be much code in here.
     */
    public void doPost (
			HttpServletRequest req,
			HttpServletResponse res
			) throws ServletException, IOException
    {

        byte [] inbuf = new byte[4096];
        StringBuffer sb = new StringBuffer(4096);


	InputStream is = req.getInputStream();
	/* This is not currently working, and its pissing 
	 * me off.
	 */
	InputSource isource = new InputSource(is);

	/**
	 * Turning this into a String is one way to do it.
	 * Another way would be to turn the InputStream
	 * into an InputSource, and feed that to the
	 * XMLReader instead.
	 */
        /* FIXME: Embedding all this crap into a boolean 
         * operator (e.g., ()) may be just fine for stevens style
         * unix haXory, but I need to catch an IOException 
         * on the in.read call.  So this needs to be rewritten.
         */
        int readlen = 0;
        while ( (readlen = is.read(inbuf, 0, inbuf.length)) >= 0) {

            sb.append ( new String (inbuf,0,readlen));
	}
        String xmlstring = sb.toString();

        if (debug) {
	    System.out.println("***********************************");
	    System.out.println("From Servlet Adaptor:\n");// + xmlstring);
        }

	/* FIXME: Figure out why the InputSource technique doesn't
	 * work, and make it work.
	 */
        Request request = new Request(xmlstring);

	/* Do something interesting with the database here.*/
   	Result result = qh.handleQuery(request.querytext);
        
	/** In general, the response should take some
         * sort of "action" as an argument.  Or should
	 * be returned from the Queryhandler.
	 */
        Response response = new Response(request,result);


        String response_string = response.getResponseString();
 
        if (debug) {
            System.out.println("Response: " + response_string);
	}


       	PrintWriter pw = res.getWriter();
        pw.print(response_string);       
	pw.close();
    }



    /**
     * jxtasearch does not use doGet, but it might be useful
     * for some other reason.  Here we just have it answer
     * in case its called directly.
     */
    public void doGet (
		       HttpServletRequest req,
		       HttpServletResponse res
		       ) throws ServletException, IOException
    {
	PrintWriter pw = res.getWriter();
	pw.println("JXTA Search Database Servlet Adapter: running.");
	pw.close();
    }




}
