/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: MysqlDBHandler.java,v 1.5 2001/06/13 23:52:46 ddoolin Exp $
 */


/** 
 * database class for mysql, compiles and passes unit test.
 *
 * FIXME: Connection pooling may or may not be idiotic.  If so,
 *        rewrite pooling so I don't look like a moron.  If not,
 *        then maybe I am smarter than the average bear...
 *        (but probably not) -dmd
 */

/* TODO: What happens next in here is a bunch of methods to deal with 
 * using different databases and tables in those databases.  After 
 * all that is written, this class will be bloated and crufty, and 
 * will need to be refactored I am sure.
 */

package net.jxta.search.dbadapter;
import java.io.*;
import java.util.*;
import java.sql.*;


public class MysqlDBHandler extends Properties {

    private static boolean debug = false;
    public boolean use_c_pool = false;
    /** FIXME: Move number_of_connections to the properties file.
     */
    private int number_of_connections = 2;
    private LinkedList connectionqueue;

   /* These were all stashed in a properties file, 
    * which isn't a bad place to put them.  Since this 
    * is a web app, using the web.xml file is probably 
    * a better solution.  Not to mention that the current
    * source tree makes using a property file a pita 
    * for testing.
    */
    private String dbhost = null;
    private String dbinst = null;
    private String dbport = null;
    private String dbuser = null;
    private String dbpass = null;

    /***************  Constructors ***************/

    public MysqlDBHandler()  {

	setDebug();
	setDBHost("localhost");
	setDBPort("3306");
	setDBUser("jxtasearch");
	setDBPass("gonesilent");

	connectionqueue = new LinkedList();
	initializeCQueue(number_of_connections);
    }

    public MysqlDBHandler(String propfile)  {

	this.loadProperties(propfile);
	connectionqueue = new LinkedList();
	initializeCQueue(number_of_connections);
    }  


    /****************  Methods ***************/

    private void initializeCQueue(int num_cons) {

	if (debug)
	    System.out.println("Initializing connection queue: ");


	try {
            // The newInstance() call is a work around for some
            // broken Java implementations
            Class.forName("org.gjt.mm.mysql.Driver").newInstance(); 
	}
	catch (Exception E) {
	    System.err.println("Unable to load driver.");
	    E.printStackTrace();
	}



	for (int i = 0; i < num_cons; i++) {
	    connectionqueue.add(getconn());
	}


        if (debug)
	    System.out.println("done.");

    }  // close InitializeQueue()



    public static void setDebug() {
	debug = true;
    }

    public static void clearDebug() {
	debug = false;
    }


    /** getConnection handles all exceptions internally
     *  instead of throwing.  This should make it a bit easier
     *  to write cleaner code in the classes that invoke it.
     *  Since everything is basically going to the same oracle 
     *  server,  and is called by one class at a time (multipass
     *  algorithm and the Categorizer runs by itself), 
     *  this should not present a big problem.
     */
    public Connection getConnection() {

	if (debug)
	    System.out.println("Fetching connection from pool...");

	if (use_c_pool) {
	    Connection con;
	    try {
		con = (Connection)connectionqueue.removeFirst();
	    } catch (NoSuchElementException e) {
		System.out.println("No more connections in queue.  Fetch more.");
		con = getconn();
	    }
	    return con;
	} else {
	    return getconn();
	}

    }  // close getConnection()


    public void returnConnection(Connection con) {

	/* Maybe add some code here to ensure that the connection
	 * is "valid" whatever that means. The "add" method is 
	 * equivalent to a "push" in this context.  Newer connections
	 * are added to the end of the list, older connections are
	 * popped off the top.
	 */
 	connectionqueue.add(con);

	if (debug) 
	    System.out.println("Connection returned to queue...");

    }  // close returnConnection()



    /** Handles the nuts and bolts of the connection.
     */
    private Connection getconn() {
   
	Connection con = null;

	/* This is all really inefficient here.  Move these to 
         * private class variables, then initialize them in the 
	 * calling method.
	 */
	String dbhost = this.getProperty("hostname");
	String dbinst = this.getProperty("instance");
	String dbport = this.getProperty("port");
	String dbuser = this.getProperty("user");
	String dbpass = this.getProperty("passwd");

        // Format of mysql connection string:
	//jdbc:mysql://[hostname][:port]/dbname[?param1=value1][&param2=value2]...
         
	String URL = "jdbc:mysql://"
	    + dbhost
	    + ":"
	    + dbport
	    + "/jxtasearch?user="
	    + dbuser
            + "&password="
            + dbpass;

	System.out.println("URL: " + URL + ", User: "
			   + dbuser + ", Passwd: " + dbpass);

	try {
	    con = DriverManager.getConnection(URL, dbuser, dbpass);
	    if (debug)
		System.out.println("Connection opened.");
	} catch (Exception e) {
	    System.out.println("We are not connecting here...");
	    e.printStackTrace();    
	}
   
	return con;
   
    }  // close getconn()


    public boolean closeAllConnections() {

	int queuesize = connectionqueue.size();
	int i = 0;
        Connection con;
	boolean closed = false;

	while (i < queuesize) {
	    /* TODO: Go through the java docs and check what
	     * removeFirst is supposed to actually return.
	     */
	    con = (Connection)connectionqueue.removeFirst();
	    closed = this.closeConnection(con);
            if (!closed) {
		break;
	    }
	    i++;
	}
        return closed;
    }


    public boolean closeConnection(Connection con) {
   
	boolean closed = false;
	try {
	    /* TODO: Go through the java docs and check what
	     * removeFirst is supposed to actually return.
	     */
	    con.close();
	    closed = true;
	    if (debug)
		System.out.println("Connection closed.");
	} catch (SQLException sqle) {
	    System.out.println("Can't close the connection.");
	    sqle.printStackTrace();
	}
   
	return closed;

    }  // close closeConnection()




    public void createTable(Connection con, String tablename, String fields) {

	String sqlcmd = "CREATE TABLE " 
	    + tablename
	    + "("
	    + fields
	    + ")";

	if (debug)
	    System.out.println(sqlcmd);


	Statement stmt;
 
	try {
	    stmt = con.createStatement();
	    stmt.executeUpdate(sqlcmd);
	    if (debug)
		System.out.println("Table " + tablename + " created");
	} catch (SQLException sqle) {
	    System.out.println("Unable to create table " + tablename);
	    sqle.printStackTrace();
	}

    }  // close createTable()


    public void setDatabase(String database) {

        Connection conn = getConnection();;

	String entry = "USE " + database;
	String description;

	try {
	    Statement stmt = conn.createStatement();
	    
	    ResultSet rs = stmt.executeQuery(entry);
	    /* FIXME: Find out whether the exception
	     * catches all failures, if not , check the
	     * result set for bogosity.
 	     */
	    stmt.close();
	} catch (SQLException sqle) {
	    System.out.println("Problem with database " + database);
	    sqle.printStackTrace();
	    //description = null;
	}
    }


    public void dropTable(Connection con, String tablename) {

	String sqlcmd = "DROP TABLE " 
	    + tablename;

	if (debug) 
	    System.out.println(sqlcmd);

	Statement stmt;

	try {
	    stmt = con.createStatement();
	    stmt.executeUpdate(sqlcmd);
	    if (debug)
		System.out.println("Table " + tablename + " dropped");
	} catch (SQLException sqle) {
	    System.out.println("Unable to drop table " + tablename);
	    sqle.printStackTrace();
	}
            
    }  // close dropTable()

  
    public void setDBProperties(String dbhost, 
				String dbinst,
				String dbport,
				String dbuser,
				String dbpasswd) {

	this.setProperty("hostname", dbhost);
	this.setProperty("instance", dbinst);
	this.setProperty("port", dbport);

	/* FIXME: If the property is null, we have a problem here.
	 * Need to find a smarter way to do this.
	 * For now, just comment stuff out.
	 */
	this.setProperty("user", dbuser);
	this.setProperty("passwd", dbpasswd);
   
    }  // close setDBProperties()

   
   /* All of these get/sets were written to support 
    * property files, but these parameters need to 
    * change to web.xml files, and the get/set changed
    * to support instance variables.
    */
    public void setDBHost(String dbhost) {

        this.dbhost = dbhost;
	this.setProperty("hostname", dbhost);

    }  // close setDBHost() 


    public void setDBInst(String dbinst) {

        this.dbinst = dbinst;
	this.setProperty("instance", dbinst);

    }  // close setDBInst()

   
    public void setDBPort(String dbport) {

        this.dbport = dbport;
	this.setProperty("port", dbport);

    }  // close setDBPort()


    public void setDBUser(String dbuser) {

        this.dbuser = dbuser;
	this.setProperty("user", dbuser);

    }  // close setDBUser()


    public void setDBPass(String dbpass) {

        this.dbpass = dbpass;
	this.setProperty("passwd", dbpass);

    }  // close setDBPassword()

   
    public String getDBHost(String dbhost) {

	return this.getProperty("hostname");

    }  // close getDBHost() 


    public String getDBInst(String dbinst) {

	return this.getProperty("instance");

    }  // close getDBInst()

   
    public String getDBPort(String dbport) {

	return this.getProperty("port");

    }  // close getDBPort()


    public String getDBUser(String dbuser) {

	return this.getProperty("user");

    }  // close getDBUser()


    public String getDBPass(String dbpass) {

	return this.getProperty("passwd");

    }  // close getDBPassword()


    public void storeProperties(String filename) {

	File propfile = new File(filename);
	storeProps(propfile);

    }  // close storeProperties()


   /* Default file name */
    private void storeProperties() {
  
	File propfile = new File("mysqlprops");
	storeProps(propfile);

    }  // close storeProperties() 


    private void storeProps(File propfile) {

	FileOutputStream fos;

	try {
	    fos = new FileOutputStream(propfile);
	    this.store(fos, "MySql database properties for servlet adapter");
	    fos.close();
	} catch (IOException ioe) {
	    System.out.println(ioe);
	} catch (ClassCastException ce) {
	    System.out.println("Object not a string");
	    ce.printStackTrace();
	}
  
    }  // close storeProps()


    public void loadProperties(String filename) {
 
	File propfile = new File(filename);
	this.loadProps(propfile);

    }  // close loadProperties()


   /* default properties file name */
    private void loadProperties() {
   
	File propfile = new File("mysqlprops");
	this.loadProps(propfile);

    }  // close loadProperties() 

 
    private void loadProps(File propfile) {

	FileInputStream fis;
	try {
	    fis = new FileInputStream(propfile);
	    this.load(fis);
	    fis.close();
	} catch (IOException ioe) {
	    System.out.println("Problem with property input file");
	    ioe.printStackTrace();
	}

    }  // close loadProps()


    /** This method can be called to initialize the dmoz database
     * properties file.
     */
    private static void test() {

	MysqlDBHandler dbh = new MysqlDBHandler();

	/** Ensure that the properties set in this method
	 *  are correct, then run it immediately.
	 */
	setupDefaults(dbh);

	if (TestDBHandler.test(dbh)) {
	    System.out.println("Passed MysqlDBHandler unit tests");
	} else {
	    System.out.println("Failed MysqlDBHandler unit tests");
	}
    }  


    /** Change the properties of the database to reflect the local 
     * machine, then run make DBHandler.main to write these into  
     * the default location in WEB-INF for runtime use.
     */
    private static void setupDefaults(MysqlDBHandler dbh) {

	dbh.setDBHost("127.0.0.1");
	dbh.setDBInst("localhost");
	dbh.setDBPort("3306");
        dbh.setDBUser("jxtasearch");
	dbh.setDBPass("gonesilent");
    } 

 
    public static void main(String [] args) {

	test();
    }  


}  // close class DBHandler






class TestDBHandler {

    public static boolean test(MysqlDBHandler pt) {

	boolean passed = true;

	pt.setDebug();
	pt.use_c_pool = true;

	/* Set the defaults for the local box */
	pt.setDBProperties( pt.getDBHost("hostname"), //"127.0.0.1", 
			    pt.getDBInst("instance"), //"cranksha", 
			    pt.getDBPort("port"),     //"1521", 
			    pt.getDBUser("user"),     //"scott", 
			    pt.getDBPass("passwd"));  //"tiger");
      
	/* Now check out the connection... */
	Connection con = pt.getConnection();

	if (con == null) {
	    passed = false;
	} 

	/* Now we need to test out the connection pooling code...
	 */
      
	/* Flex some muscle here... */
	pt.createTable(con, "BLAH", "SOMENUMBER INTEGER, SOMESTRING VARCHAR(10)");


	if (pt.use_c_pool) {
	    pt.returnConnection(con);
	}


	if (pt.use_c_pool) {
	    pt.getConnection();
	}


	/* Now get rid of the BLAH table and start cleaning up... */
	pt.dropTable(con, "BLAH");


	if (pt.use_c_pool) {
	    pt.returnConnection(con);
	}


	/* and close it... */
	pt.closeConnection(con);
	pt.closeAllConnections();

	return passed;
    }

}
