/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: QueryHandler.java,v 1.2 2001/06/12 23:04:45 ddoolin Exp $
 */


/* The main point of this class is to provide 
 * enough abstraction so that fancy predicate
 * handling can be done before hitting the
 * database.  Ultimately, this should probably
 * be an abstract class that provides just the
 * predicate stuff (whatever other common code
 * code is needed, then other handlers for index
 * searching or database querying can be built
 * from it.
 */


/* FIXME: Methods in this class may need to be
 * synchronized because there will only be one
 * instance of the class running per servlet.
 */
package net.jxta.search.dbadapter;

import java.sql.*;

public class QueryHandler {

    MysqlDBHandler dbh; 

    public QueryHandler () {
        dbh = new MysqlDBHandler();
    }


    /* TODO: Implement.  :) */
    /* This method will probably need a `Predicate'
     * class of some sort as an argument, then return
     * a String or object that can be used to query
     * database.
     */
    public void handlePredicates() {

    }

    /* The argument needs to be turned into a string
     * at some point.  Where and when exactly to do this
     * is not clear at the moment.  The StringBuffer is a
     * consequence of the RequestParser xml parser.
     */
    public Result handleQuery(StringBuffer querytext) {

	Result result = new Result();

	/* There should be no predicates in this string. */
	String query = querytext.toString();

	/* 1. Turn query into UPPER CASE because thats how the
	 *    TERM's are stored in the database table.
	 */
        query = query.toUpperCase();

	/* 2. Collapse adjacent white space into no
	 *    white space at all between terms, again
	 *    because the TERM's are stored with no
	 *    white space.
	 */
        query = collapseWhiteSpace(query);


	// Temporary
	System.out.println("Query after upperspacing and whitespace collapsed:");
	System.out.println(query);


	/* 3. Issue the query(s). */

        Connection conn = dbh.getConnection();

	String entry = "SELECT TITLE,DESCRIPTION,URL FROM rockeng WHERE "
                     + "(TERM = '"
	             + query
	             + "')";
	/*
	String title;
        String description;
	String url;
	*/

	try {
	    ResultSet rs;
	    Statement stmt = conn.createStatement();
	    /* Move this out of here... */
            stmt.executeQuery("use jxtasearch");
	    rs = stmt.executeQuery(entry);
	    if (rs.next()) {
		result.setTitle(rs.getString("TITLE"));
		result.setDescription(rs.getString("DESCRIPTION"));
		result.setURL(rs.getString("URL"));
	    } else {
		result.setTitle(null);
		result.setDescription(null);
		result.setURL(null);
	    }
	    stmt.close();
	} catch (SQLException sqle) {
	    System.out.println("Problem reading table rockeng");
	    sqle.printStackTrace();
	    result.setTitle(null);
	    result.setDescription(null);
	    result.setURL(null);
	}

	dbh.returnConnection(conn);

	/* FIXME: Need to be able to return a url as well as a 
	 * description.
	 */
        return result;
    }


    public static String collapseWhiteSpace(String s) {

	char [] chars = s.toCharArray();
	int charcount = 0;

        for (int i=0; i<chars.length; i++) {
	    if (chars[i] == ' ') {
		continue;
	    } else {
		chars[charcount] = chars[i];
		charcount++;
	    }
	}
	System.out.println("Char count: " + charcount);
 
	return String.valueOf(chars,0,charcount);
    }


    public static boolean test() {

	return TestQueryHandler.test();
    }

    public static void main (String [] args) {

	test();
    }

}




class TestQueryHandler {

    public static boolean test() {

	System.out.println("Query handler unit tests not yet implemented.");
	return false;
    }
}
