/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RequestParser.java,v 1.1.1.1 2001/06/01 16:51:40 jbeatty Exp $
 */

/**
 * Simple SAX parser for handling requests.  This will need to
 * be extended to handle anything other than the simplest of 
 * queries.
 */

package net.jxta.search.dbadapter;
import java.io.*;
import java.util.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;


public class RequestParser  extends DefaultHandler {



    Request request = null;


    int state;
    static final int REQUEST      = 1;
    static final int QUERY        = 2;
    static final int TEXT         = 3;



    public RequestParser(Request r) {
        this.request = r;
    }
    

    public void startElement(String namespaceURI, String localName,
			     String qName, Attributes atts)
    {

	System.out.println("From start element:");

	if (localName.equals("request")) {
	    state = REQUEST;
	    handleRequestAtts(atts);
	} else if (localName.equals("query")) {
	    state = QUERY;
	} else if (localName.equals("text")) {
	    state = TEXT;  
	}

    }  


    public void endElement(String namespaceURI, String localName,
			   String qName) {
	state = 0;
    }  


    /**
     *  Calls the method that extracts the stuff inside of a tag.
     */
    public void characters (char[] ch, int start, int length) {

	String s = new String (ch, start, length);

	System.out.println("From characters:");

	switch (state) {
	case REQUEST:    handleRequest(s);      break; 
	case QUERY:      handleQuery(s);        break;
	case TEXT:       handleText(s);         break;
	default:                                break;
	} 

    }  



    private void handleRequest(String s) {

    }

    private void handleRequestAtts(Attributes atts) {
   
	System.out.println("From Attributes:");

	if (request != null) {
	    request.query_space           = atts.getValue("query-space");
	    request.query_uuid            = atts.getValue("query-uuid");
	    request.query_lifetime        = atts.getValue("query-lifetime");
	    request.max_fanout            = atts.getValue("max-fanout");
	    request.max_hits_per_provider = atts.getValue("max-hits-per-provider");
	    request.flush_after_providers = atts.getValue("flush-after-providers");
	    request.flush_after_ms        = atts.getValue("flush-after-ms");
	}
    }



    private void handleQuery(String s) {
	System.out.println("From handleQuery:");
    }

    private void handleText(String s) {
        request.querytext.append(s);
	System.out.println("From handleText");
    }


    public static boolean test() {
  
        boolean passed = true;
        if(TestRequestParser.test()) {
	    System.out.println("Passed Request unit test");
	} else {
	    System.out.println("Failed Request unit test");
	    passed = false;
	}        
	return passed;

    }

    public static void main (String[] argv) throws IOException,SAXException {

	test();
    }

} 



class TestRequestParser {

    public static boolean test() {

        Request r = new Request();

        r.parse(request);

	/** 
	 * Let's test a couple of the results of the parse and see
	 * what we get...  we could go on and on here, testing every 
	 * element and attribute in the Request, and perhaps we should.
         * Later.
         */
        if (!r.query_space.equals("http://search.jxta.org/text")) {
	    return false;
	}

        if (!r.query_uuid.equals("F417FAE0546511D5A313E163229A8058")) {
	    return false;
	}

        if (r.query_uuid.equals("1-2-3-A-B-C-Do-Re-Mi-w00h00-Old Jackson Five!!!")) {
	    return false;
	}

	return true;
 
    }

    private static String request = "<?xml version='1.0'?>"
	+ "<request xmnls=\"http://search.jxta.org/\""
	+ " query-space='http://search.jxta.org/text'"
	+ " query-uuid='F417FAE0546511D5A313E163229A8058'"
	+ " query-lifetime='10000'"
	+ " max-fanout='2147483647'"
	+ " max-hits-per-provider='2'"
	+ " flush-after-providers='3'"
	+ " flush-after-ms='2000'>"
	+ " <query>"
	+ " <text>china</text>"
	+ " </query>"
	+ " </request>";

}
