/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaDBServerApp.java,v 1.1 2001/06/11 23:23:45 ddoolin Exp $
 */
package net.jxta.search.examples.jxtadb;

import net.jxta.platform.*;
import net.jxta.pipe.*;
import net.jxta.peergroup.*;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.*;
import net.jxta.protocol.*;
import net.jxta.exception.*;
import net.jxta.id.*;
import net.jxta.discovery.*;
import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;

import net.jxta.search.peer.JxtaInfoProviderService;

import java.io.*;

import javax.xml.parsers.*;

public class JxtaDBServerApp implements Application {

    private PeerGroup peerGroup;
    JxtaInfoProviderService svc;

    public void init(PeerGroup peerGroup, Advertisement adv)
    throws PeerGroupException {
	this.peerGroup = peerGroup;
    }

    public int startApp(String[] args) {

	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
	DocumentBuilder builder=null;
	
	try {
	    builder = factory.newDocumentBuilder ();
	} catch (ParserConfigurationException e) {
	    e.printStackTrace ();
	    return -1;
	}


	/*
	try {
	    svc =
		new JxtaInfoProviderService( peerGroup,
					     builder,
					     handler );
            new Thread( svc ).start();

	    if( System.getProperty( "provider.autoregister" ).equals("true"))
		svc.registerWithHubs();

	    if( !System.getProperty( "provider.interactive" ).equals("false"))
		new Thread( new ConsoleController() ).start();

        } catch ( Exception e ) {
            e.printStackTrace();
	    return -1;
        }
	*/
	return 0;
    }
    
    public void stopApp() {
        if( svc != null ) {
	    svc.stop();
	}
    }

    class ConsoleController implements Runnable {
	public void run() {
	    boolean stop = false;
	    while( !stop ) {
		try {
		    System.out.print( "> " );
		    String cmd = readline();
		    if( cmd.equals("stop")) {
			stop=true;
		    } else if( cmd.equals( "register" ) ) {
			svc.registerWithHubs();
		    } else if( cmd.equals( "unregister" ) ) {
			svc.unregisterWithHubs();
		    } else if( !cmd.trim().equals("") ) {
			System.out.println("Available commands: \n");
			System.out.println("stop -- stops the server");
			System.out.println("unregister -- unregisters from hubs");
			System.out.println("register -- registers with hubs");
		    }
		} catch( IOException e ) {
		    // no code intended
		    e.printStackTrace();
		}
	    }
	    System.out.println( "Server is stopping. Please wait for the "+
				"peer to unregister with the hubs." );
	    svc.stop();
	    System.exit(0);
	}

	String readline() throws IOException {
	    boolean eol=false;
	    byte[] buf = new byte[128];
	    int pos=0;
	    int c;
	    while(!eol) {
		c=System.in.read();
		if( c==-1 || c=='\n') {
		    eol=true;
		    break;
		}
		buf[pos]=(byte)c;
		pos++;
	    }
	    return new String( buf, 0, pos);
	}
    }
}
