/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: StockQuoteRequestHandler.java,v 1.3 2001/06/04 04:14:35 jbeatty Exp $
 */
package net.jxta.search.examples.jxtaprovider;

import net.jxta.search.peer.*;

class StockQuoteRequestHandler implements RequestHandler {

    private static StockQuote stockQuoteService =
	new StockQuote();
    
    public String onRequest( String query, String uuid ) {
	Stock stock = stockQuoteService.getQuote( query );
	if( stock == null ) return null;
	
	// note that the stock quote service will give us back a non-null
	// Stock even if the symbol is invalid.  Thus, we throw out
	// responses based on an NA price. TODO: add this intelligence to the
	// stock quote service
	if( stock.getQuote().equals("NA") ) return null;
	
	String title = "Stock quote: Symbol="+stock.getSymbol()+" Price=" +
	    stock.getQuote();//+ " Change="+stock.getPointChange();
	
	System.out.println( "obtained stock: " + title );
	
	StringBuffer response = new StringBuffer();
	String link = "http://quote.yahoo.com/q?s=" + stock.getSymbol() +
	    "&d=c";

	response.append("<?xml version='1.0'?>\n");
	response.append("<responses query-uuid='" +uuid + "'>\n");
	response.append( "<response>");
        response.append(   "<data>");
	response.append(    "<hit><link>");
	response.append(link);
	response.append(    "</link><title>");
	response.append(title);
	response.append(     "</title><detail></detail></hit></data>\n");
	response.append( "</response>" );
	response.append("</responses>\n");
	return response.toString();
    }
}
