/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Request.java,v 1.1.1.1 2001/06/01 16:51:47 jbeatty Exp $
 */

/**
 * FIXME: Java doc
 */

package net.jxta.search.servadapt;

import java.io.*;
import java.util.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

public class Request {


    public Request () {
    }


    public Request (String xmlstring) {
        parse(xmlstring);
    }

    public Request (InputStream is) {
        parse(is);
    }


  /** 
   * FIXME: It's inefficient to take this in as a String 
   * and make it into an InputSource when the original stream
   * was a byte stream, so overload parse to take a byte stream
   * and turn that into an input source to parse.
   */
   public void parse(String request) {

     ByteArrayInputStream bais = new ByteArrayInputStream(request.getBytes());
     this.parse(bais);

   }


   public void parse(InputStream istream) {

       InputSource isource = new InputSource(istream);
       this.parse(isource);

   }


   public void parse (InputSource isource) {

     try {
	XMLReader xml = XMLReaderFactory.createXMLReader ("org.apache.xerces.parsers.SAXParser");
        xml.setContentHandler(new RequestParser(this));
        xml.parse(isource); 
     } catch (Exception e) {
        System.out.println(e);
     }
   }

    public String query_space;
    public String query_uuid;
    public String query_lifetime;
    public String max_fanout;
    public String max_hits_per_provider;
    public String flush_after_providers;
    public String flush_after_ms;



    public static boolean test() {

	boolean passed = true;
        if (TestRequest.test()) {
	    System.out.println("Passed Request unit test");
	} else {
	    System.out.println("Failed Request unit test");
	    passed = false;
	}        
	return passed;
    }

    public static void main(String [] args) {

	test();
    }

}


class TestRequest {


    public static boolean test() {
 
        Request r = new Request(testrequest);


	/** 
	 * Let's test a couple of the results of the parse and see
	 * what we get...  we could go on and on here, testing every 
	 * element and attribute in the Request, and perhaps we should.
         * Later.
         */
        if (!r.query_space.equals("http://search.jxta.org/text")) {
	    return false;
	}

        if (!r.query_uuid.equals("E44ADB40518C11D5A3A88CC6C11CFEEF")) {
	    return false;
	}

	return true;
    }


   static String testrequest =  "<?xml version='1.0'?>"
                      +  " <request xmnls='http://search.jxta.org/'"
                      +  " query-space='http://search.jxta.org/text'"
                      +  " query-uuid='E44ADB40518C11D5A3A88CC6C11CFEEF'"
                      +  " query-lifetime='0'"
                      +  " max-fanout='2147483647'"
                      +  " max-hits-per-provider='0'"
                      +  " flush-after-providers='0'"
                      +  " flush-after-ms='0'>" 
                      +  " <query>"
                      +  " <text>china</text>"
                      +  " </query>"
                      +  " </request>";

}

