/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Response.java,v 1.1.1.1 2001/06/01 16:51:48 jbeatty Exp $
 */

/* Will be used for all the response generation, 
 * formatting, etc that is currently in the 
 * Adaptor servlet.
 */

package net.jxta.search.servadapt;

/**
 * A Response from a JxtaSearch information provider 
 * consists of an xml document with field identifying 
 * the provider, some sort of an icon, the results of
 * a search performed on a query, etc.  This implementation
 * is very simple, and just returns the information in the
 * Request back to the servlet, which sends it back to the 
 * the user application.  The intention here is that
 * a production implementation will use this as a starting
 * point for building classes customized for a particular
 * type of query.
 */
public class Response {


    private String response_string = null;

    /** 
     * FIXME: What needs to be done here is the
     * query and any other useful information
     * stripped out of the request.  Then some sort
     * of "action(s)" on the query needs to be performed.
     */
    public Response (Request r) {

	response_string = formatResponse(r);
    }


    /**
     * @return String response_string is an xml document
     *         that the information provider (i.e., whoever
     *         invoked this class) believes satisfies the
     *         query.
     */
    public String getResponseString () {
	return response_string;
    }


    /** 
     * None of this stuff should change in here, and
     * the values for the provider information in the
     * response should probably be initialized somewhere
     * when the adaptor servlet starts.
     */
    private static String getProvider() {
       
	StringBuffer sb = new StringBuffer(4096);

        sb.append(provider_open);

        sb.append(name_open);
	sb.append("NAME");
        sb.append(name_close);

        sb.append(link_open);
	sb.append("LINK");
        sb.append(link_close);

        sb.append(description_open);
	sb.append("DESCRIPTION");
        sb.append(description_close);

	/** Image */
        sb.append(image_open);
        sb.append(url_open);
	sb.append("IMAGE URL");
        sb.append(url_close);
        sb.append(width_open);
	sb.append("IMAGE WIDTH");
        sb.append(width_close);
        sb.append(height_open);
	sb.append("IMAGE HEIGHT");
        sb.append(height_close);
        sb.append(image_close);

        sb.append(provider_close);

	return sb.toString();
    }


    /**
     * This method should stay internal and be called 
     * as a result of the "action" finishing, or for 
     * the public method getResponseString().
     */
    private static String formatResponse(Request r) {

        StringBuffer sb = new StringBuffer(4096);

        sb.append(xmlversion);
        sb.append(responses_open);
        sb.append(response_open);
        sb.append(data_open);

	sb.append(getProvider());
        /**
         * each hit can be handled in a method also
         */
        sb.append(hit_open);
        sb.append(link_open);
        sb.append("http://search.jxta.org");
        sb.append(link_close);
        sb.append(title_open);
        sb.append("JxtaSearch servlet adapter");
        sb.append(title_close);

        sb.append(detail_open);
        sb.append(r.query_space
                  +"/"+r.query_uuid
                  +"/"+r.query_lifetime
                  +"/"+r.max_fanout
                  +"/"+r.max_hits_per_provider
                  +"/"+r.flush_after_providers
                  +"/"+r.flush_after_ms);
        sb.append(detail_close);

        sb.append(hit_close);

        sb.append(data_close);
        sb.append(response_close);
        sb.append(responses_close);

        return sb.toString();
    }


  
    public static boolean test() {

	boolean passed = true;
	if(TestResponse.test()) {
	    System.out.println("Passed Response unit test");
	} else {
	    System.out.println("Failed Response unit test");
	    passed = false;
	}
	return passed;
    }


    public static void main (String [] args) {

	test();
    }


    private static final String xmlversion        = "<?xml version=\"1.0\"?>";
    private static final String responses_open    = "<responses>";
    private static final String responses_close   = "</responses>";
    private static final String response_open     = "<response>";
    private static final String response_close    = "</response>";
    private static final String data_open         = "<data>";
    private static final String data_close        = "</data>";
    private static final String hit_open          = "<hit>";
    private static final String hit_close         = "</hit>";
    private static final String title_open        = "<title>";
    private static final String title_close       = "</title>";
    private static final String link_open         = "<link>";
    private static final String link_close        = "</link>";
    private static final String detail_open       = "<detail>";
    private static final String detail_close      = "</detail>";


    /** Provider strings */

    private static final String provider_open     = "<provider>";
    private static final String provider_close    = "</provider>";
    private static final String name_open         = "<name>";
    private static final String name_close        = "</name>";
    private static final String description_open  = "<description>";
    private static final String description_close = "</description>";
    private static final String image_open        = "<image>";
    private static final String image_close       = "</image>";
    private static final String url_open          = "<url>";
    private static final String url_close         = "</url>";
    private static final String width_open        = "<width>";
    private static final String width_close       = "</width>";
    private static final String height_open       = "<height>";
    private static final String height_close      = "</height>";


}



class TestResponse {

    public static boolean test() {
   
	System.out.println("FIXME: implement Response unit testing, or justify its absence");
	return false;
    }

}
