/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: HttpRouterClient.java,v 1.2 2001/06/06 01:32:02 tcamarda Exp $
 */
package net.jxta.search.peer;

import java.io.*;
import java.net.*;

import net.jxta.search.util.HttpConnection;
import net.jxta.search.util.ValidatingXmlParser;
import net.jxta.search.util.XmlParser;

public class HttpRouterClient implements RouterClient {

    URL router;
    int debug;
    
    public HttpRouterClient (URL router, int debug) {
	this.router = router;
	this.debug = debug;
    }

	/*
    class AsyncSearch implements Runnable {

		Request request;
		QueryResponseCallback callback;
		
		AsyncSearch (Request request, QueryResponseCallback callback) {
			this.request = request;
			this.callback = callback;
		}
		
		public void run () {
			
			try {
				Document doc = search (request);
				
				if (doc != null)
					callback.onQueryResults (request.getQueryUuid (), doc);
			} catch (Exception e) {
				callback.onException (request.getQueryUuid (), e);
			}
		}
    }
	*/

	/*
    public String search (Request request, QueryResponseCallback callback)
		throws IOException
    {
		AsyncSearch as = new AsyncSearch (request, callback);
		
		new Thread (as).start ();
		
		return request.getQueryUuid ();
    }
	*/

    public void search (Request                       request, 
			RouterClient.ResponseCallback callback) 
		throws IOException
    {
	
	try {
	    HttpConnection.Response response = makeRouteCall (request);
	    if (this.debug > 0)
		System.out.println ("Received response from router.");
	    
	    if (response.getStatusCode () == HttpURLConnection.HTTP_OK) {
		System.out.println ("Router: HTTP_OK");
		ResponseHandler handler  = new ResponseHandler (callback); 
		byte[]          buf      = new byte[8192];
		
		ValidatingXmlParser.parse (response.getInputStream (),
					   buf, 
					   handler,
					   handler.root);
                if (this.debug > 0)
		    System.out.println ("Parsed response from router.");
	    } else {
		
		throw new IOException ("Router: " + 
				       response.getStatusMessage ());
	    }
	} catch (InterruptedException e) {
	    // should never get this because we aren't re-using http
	    // connections.
	    throw new Error ();
	} catch (XmlParser.Exception e) {
		callback.exception (request.getQueryUuid (), e);
	}
    }



    HttpConnection.Response makeRouteCall (Request request) 
	throws IOException, InterruptedException
    {
	HttpConnection         con;
	HttpConnection.Request req;
	OutputStream           out;
	
	System.out.println ("Setting http request timeout to " +
			    2 * request.getQueryLifetime ());

	con = new HttpConnection (router, 
				  2 * request.getQueryLifetime (),
				  null);

	req = con.beginRequest (router, "POST");
	
	req.setHeader ("Content-type", "text/xml");
	
	req.setHeader ("Content-length", String.valueOf (request.getLength()));
	
	if (debug > 0)
	    req.setHeader ("debug", String.valueOf (debug));
	
	/*
	  for (int i = 0;  i < cookies.length;  i++)
	  req.setHeader ("Cookie", 
	  cookies[i].getName () + "=" + 
	  cookies[i].getValue ());
	*/
	
	if (debug > 0) {
	    System.out.println ("Making route call: ");
	    request.toStream (System.out);
	}
	
	
	out = req.getOutputStream ();
	
	request.toStream (out);
	
	return req.getResponse ();
    }

}
