/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaQueryResponseProcessor.java,v 1.1.1.1 2001/06/01 16:49:51 jbeatty Exp $
 */
package net.jxta.search.peer;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;

import net.jxta.pipe.InputPipe;
import net.jxta.endpoint.Message;

import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;

// debug only
import java.io.ByteArrayOutputStream;

/** Waits for incoming messages and calls the callback that was mapped for
    that request **/
class JxtaQueryResponseProcessor implements Runnable {

    /** the pipe that we're going to poll for messages **/
    private InputPipe inputPipe;

    /** The request-response mapping of uuids to callbacks **/
    private ResponseCallbackMap map;

    private DocumentBuilder builder;

    /** flag controlling continued operation **/
    private boolean isRunning = true;

    /** Creates the processor, given a query response input pipe, the
	request-response callback map, and a DocumentBuilder **/
    public JxtaQueryResponseProcessor( InputPipe inputPipe,
				       ResponseCallbackMap map,
				       DocumentBuilder builder ) {
	this.inputPipe = inputPipe;
	this.map = map;
	this.builder = builder;
    }

    /** Stops the message polling. May take up to 1 sec. for the thread to
	die. **/
    public void stop() {
	isRunning = false;
    }

    /** Polls for messages. When a message arrives, the callback object is
	looked up in the map and executed. **/
    public void run() {	
	System.out.println( "JxtaQueryResponseProcessor: running.");
	while( isRunning ) {
	    try {
		Message msg = inputPipe.poll ( 1000 );
		    
		if (!isRunning)
		    break;
		if (msg == null)
		    continue;
		
		InputStream isResponse = msg.pop( "responses" );
		Document doc = builder.parse (isResponse);
		String uuid =
		    doc.getDocumentElement().getAttribute ("query-uuid");

		QueryResponseCallback callback = map.removeCallback(uuid);
		if( callback!= null ) {
		    callback.onQueryResults( uuid, doc );
		} else {
		    System.out.println( "No callback for uuid=" + uuid );
		}
	    } catch( Exception e ) {
		System.err.println( "Error processing response: " + e );
		e.printStackTrace();
	    }
	}
	System.out.println( "JxtaQueryResponseProcessor shut down" );
    }

    String streamToString (InputStream in, byte[] buf, int maxLen) 
	throws IOException
    {
	int pos = 0;
	int n;
	
	while ((n = in.read (buf, pos, maxLen - pos)) > 0)
	    pos += n;
	
	return new String (buf, 0, pos);
    }

    /** moves all bytes from is to os **/
    private void transfer( InputStream is, OutputStream os ) {
        int c;
	byte[] buf = new byte[128];
        try {
            while ( (c=is.read(buf,0,128))!= -1 ) {
                os.write(buf,0,c);
            }
        } catch ( IOException e ) {
            System.err.println( "Problem displaying search payload:" + e );
            e.printStackTrace();
        }
    }
}
