/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaResolverClient.java,v 1.1.1.1 2001/06/01 16:49:51 jbeatty Exp $
 */
package net.jxta.search.peer;

import net.jxta.platform.*;
import net.jxta.pipe.*;
import net.jxta.peergroup.*;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.*;
import net.jxta.protocol.*;
import net.jxta.exception.*;
import net.jxta.document.*;
import net.jxta.id.*;
import net.jxta.discovery.*;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.net.*;
import java.util.*;

import net.jxta.search.util.XmlParser;


public class JxtaResolverClient implements ResolverClient {

    PeerGroup  peerGroup;
    Pipe       pipeService;
    OutputPipe resolverPipe; /* requests are sent to resolver through here */
    InputPipe  responsePipe; /* responses from resolver come through here */

    static String     resolverPipeName = "InfraSearch Resolver Request Pipe";
    PipeAdvertisement responsePipeAdv;



    JxtaResolverClient (PeerGroup         peerGroup,
			PipeAdvertisement resolverAdv)
	throws PeerGroupException
    {
	this.peerGroup = peerGroup;
		
	init (resolverAdv);
    }


    /** Returns an enumeration of pipe advertisement for advertised jxtasearch
	resolvers **/
    public static Enumeration discoverResolverPipeAdvs( PeerGroup group )
    throws PeerGroupException {
	return JxtaDiscoveryUtil.discoverPipeAdvs( group,
						   resolverPipeName,
						   5000 );
    }

    void init (PipeAdvertisement resolverAdv) throws PeerGroupException {
	
	try {
	    pipeService     = (Pipe) peerGroup.lookupService
		("jxta.serverice.pipe");
	} catch (ServiceNotFoundException e) {
 	    throw new PeerGroupException ("Pipe service is necessary for " +
					  "the jxta resolver client.");
	}

	try {
	    resolverPipe = pipeService.createOutputPipe (resolverAdv, 
							 Pipe.NonBlocking, 
							 30000);
	} catch (IOException e) {
	    throw new PeerGroupException ("Could not create pipe to resolver.");
	}

	try {
	    /* set up pipe for return advertisements */
	    responsePipeAdv = (PipeAdvertisement)
		AdvertisementFactory.newAdvertisement
		("jxta:PipeAdvertisement");
	    
	    responsePipeAdv.setPipeID (new PipeID (peerGroup.getID ()));
	    responsePipeAdv.setName ("Return pipe for resolver responses.");
	    
	    responsePipe = pipeService.createInputPipe (responsePipeAdv);
	} catch (java.lang.reflect.InvocationTargetException e) {
	    throw new PeerGroupException
		("Could not create response pipe advertisement");
	} catch (IOException e) {
 	    throw new PeerGroupException ("Could not create response pipe.");
	}
	
    }


    public ProviderInfo[] resolve (Request request) 
	throws IOException, XmlParser.Exception
    {
	
	Message msg;

	msg = pipeService.createMessage ();

	msg.push ("request", request.getInputStream ());
	
	try {
	    msg.push ("returnPipe", 
		      responsePipeAdv.getDocument (new MimeMediaType
			  ("text", "xml")).
		      getStream ());
	} catch (Exception e) {
	    /* ugh, why does getDocument () throw Exception ?! */
	}
	
	resolverPipe.send (msg);
	
	msg = responsePipe.poll (1000 * 60);

        
	return HttpResolverClient.parseResolverResponse (msg.pop ("response"));
							 
        
    }

    public void deleteRoutes (URL[] urls, String[] ids, String[] reasons) {

    }

    PipeAdvertisement getResolverInputPipeAdvertisement () 
	throws java.lang.reflect.InvocationTargetException
    {
	PipeAdvertisement adv;
	
	adv = (PipeAdvertisement)
	    AdvertisementFactory.newAdvertisement
	    (PipeAdvertisement.getAdvertisementType ());

	/* need to get the pipe id of the input pipe the resolver is using! */
	adv.setPipeID (new PipeID (peerGroup.getID ()));
	adv.setName ("InfraSearch Resolver Pipe");

	return adv;
    }

}




