/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaResolverClientApp.java,v 1.1.1.1 2001/06/01 16:49:52 jbeatty Exp $
 */
package net.jxta.search.peer;

import net.jxta.platform.*;
import net.jxta.pipe.*;
import net.jxta.peergroup.*;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.*;
import net.jxta.protocol.*;
import net.jxta.exception.*;
import net.jxta.document.*;
import net.jxta.id.*;
import net.jxta.discovery.*;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Enumeration;


public class JxtaResolverClientApp implements Application {

    PeerGroup  peerGroup;
    Pipe       pipeService;
    OutputPipe resolverPipe; /* requests are sent to resolver through here */
    InputPipe  responsePipe; /* responses from resolver come through here */

    String            resolverPipeName = "InfraSearch Resolver Request Pipe";
    PipeAdvertisement responsePipeAdv;

    public void init (PeerGroup pg, Advertisement adv) 
	throws PeerGroupException
    {
	PipeAdvertisement resolverAdv;
	Discovery         discoveryService;
	Enumeration       discovered;

	peerGroup       = pg;
	
	try {
	    resolverAdv     = getResolverInputPipeAdvertisement ();
	} catch (java.lang.reflect.InvocationTargetException e) {
	    throw new PeerGroupException ("Could not create resolver pipe " +
					  "advertisement.");
	}

	try {
	    pipeService     = (Pipe) peerGroup.lookupService
		("jxta.serverice.pipe");
	} catch (ServiceNotFoundException e) {
 	    throw new PeerGroupException ("Pipe service is necessary for " +
					  "the jxta resolver client.");
	}

	try {
	    discoveryService = (Discovery) peerGroup.lookupService
		("jxta.service.discovery");
	} catch (ServiceNotFoundException e) {
 	    throw new PeerGroupException ("Discovery service is necessary " +
					  "for the jxta resolver client.");
	}

	discoveryService.getRemoteAdvertisements (null,
						  Discovery.ADV,
						  "name",
						  resolverPipeName,
						  100);
	
	/* xxx god, jxta's stupid */
	try {
	    Thread.sleep (5000);
	} catch (InterruptedException e) {
	    return;
	}

	try {
	    discovered = discoveryService.getLocalAdvertisements (Discovery.ADV,
								  "name",
								  resolverPipeName);
	} catch (IOException e) {
	    throw new PeerGroupException ("Could not get local advertisements.");
	}
	    
	while (discovered != null && discovered.hasMoreElements ()) {
	    resolverAdv = (PipeAdvertisement) discovered.nextElement ();

	    if (resolverAdv.getName () != null &&
		resolverAdv.getName ().equals (resolverPipeName))
		
		break;
	}
	
	try {
	    resolverPipe = pipeService.createOutputPipe (resolverAdv, 
							 Pipe.NonBlocking, 
							 30000);
	} catch (IOException e) {
	    throw new PeerGroupException ("Could not create pipe to resolver.");
	}

	try {
	    /* set up pipe for return advertisements */
	    responsePipeAdv = (PipeAdvertisement)
		AdvertisementFactory.newAdvertisement
		("jxta:PipeAdvertisement");
	    
	    responsePipeAdv.setPipeID (new PipeID (peerGroup.getID ()));
	    responsePipeAdv.setName ("Return pipe for resolver responses.");
	    
	    responsePipe = pipeService.createInputPipe (responsePipeAdv);
	} catch (java.lang.reflect.InvocationTargetException e) {
	    throw new PeerGroupException
		("Could not create response pipe advertisement");
	} catch (IOException e) {
 	    throw new PeerGroupException ("Could not create response pipe.");
	}
	
    }


    /**
     * @param argv0 query-space
     * @param argv1 max number of providers to match
     * @param argv2 query in the format of &lt;query&gt; ... &lt;/query&gt;
     */
    public int startApp (String[] argv) {
	
	Message msg;

	msg = pipeService.createMessage ();

	try {
	    msg.push ("request", 
		      new ByteArrayInputStream
			  (("<?xml version='1.0'?>"+
			    "<request xmlns='" + Request.DEFAULT_NAMESPACE +
			    "'\n" +
			    "  query-space='"+argv[0]+"'\n"+
			    "  max-fanout="+argv[1]+"'>\n" +
			    argv[2] + 
			    "</request>"
			    ).getBytes ()));
	    msg.push ("returnPipe", 
		      responsePipeAdv.getDocument (new MimeMediaType
			  ("text", "xml")).
		      getStream ());

	
	    resolverPipe.send (msg);

	    return 0;
	} catch (IOException e) {
	    /* goddamn, this api is ridiculous */
	    return -1;
	} catch (Exception e) {
	    /* anything else was thrown by "getDocument throws Exception"! */
	    return -2;
	}
    }

    public void stopApp () {

    }


    PipeAdvertisement getResolverInputPipeAdvertisement () 
	throws java.lang.reflect.InvocationTargetException
    {
	PipeAdvertisement adv;
	
	adv = (PipeAdvertisement)
	    AdvertisementFactory.newAdvertisement
	    (PipeAdvertisement.getAdvertisementType ());

	/* need to get the pipe id of the input pipe the resolver is using! */
	adv.setPipeID (new PipeID (peerGroup.getID ()));
	adv.setName ("InfraSearch Resolver Pipe");

	return adv;
    }

}




