/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ProviderInfo.java,v 1.1.1.1 2001/06/01 16:49:53 jbeatty Exp $
 */
package net.jxta.search.peer;

import java.net.*;
import java.util.*;

public class ProviderInfo {
    URL     queryURL;          // should be named queryURL or such
    String  id;                // Provider ID
    String  title;             // Provider Name/Title
    URL     link;              // URL to homepage
    String  description;       // Provider Motto/Tagline/description
    URL     imageURL;          // Provider Icon
    int     imageWidth   = 0; // Provider Icon Width
    int     imageHeight  = 0; // Provider Icon Height
	long    timestamp;

    public URL getQueryURL () { return this.queryURL; }

    public String getId () { return this.id; }

    public String getTitle () { return this.title; }

    public URL getLink () { return this.link; }

    public String getDescription () { return this.description; }

    public URL getImageURL () { return this.imageURL; }

    public int getImageWidth () { return this.imageWidth; }

    public int getImageHeight () { return this.imageHeight; }

    public void adjustImageSize (int maxWidth, int maxHeight) {
       if (imageHeight >= maxHeight || imageWidth >= maxWidth) { // too big, adjust size

          float aspect = 0;             // aspect ratio width:height
          aspect = (float) this.imageWidth / (float) this.imageHeight;

          if (aspect > 1) {              // wider than tall
              this.imageWidth  = maxWidth;
              this.imageHeight = Math.round (this.imageWidth / aspect);

          } else if (aspect < 1) {       // taller than wide
              this.imageHeight = maxHeight;
              this.imageWidth  = Math.round (this.imageHeight * aspect);

          } else {                       // equal
              int smaller = Math.min (maxHeight, maxWidth);

              this.imageWidth  = smaller;
              this.imageHeight = smaller;
          }
      }
    }
	
    public long getTimestamp () { return timestamp; }

    public void setQueryURL (String queryURL) throws MalformedURLException {
      this.queryURL = new URL (queryURL);
    }   

    public void setId (String id) {
      this.id = id;
    }

    public void setTitle (String title) {
      this.title = title;
    }

    public void setLink (String link) throws MalformedURLException {
      this.link = new URL (link);
    }

    public void setDescription (String description) {
      this.description = description;
    }

    public void setImageURL (String imageURL) throws MalformedURLException {
      this.imageURL = new URL (imageURL);
    }

    public void setImageWidth (String imageWidth) throws NumberFormatException {
      this.imageWidth = Integer.parseInt (imageWidth);
    }

    public void setImageHeight (String imageHeight) throws NumberFormatException {
      this.imageHeight = Integer.parseInt (imageHeight);
    }

    public String toString () {
      StringBuffer result = new StringBuffer (200);

      if (this.queryURL != null) {
        result.append ("queryURL=" + this.queryURL.toString ());
      }

      if (this.id != null) {
        result.append (",id=" + this.id);
      }

      if (this.title != null) {
        result.append (",title=" + this.title);
      }

      if (this.link != null) {
        result.append (",link=" + this.link.toString ());
      }

      if (this.description != null) { 
        result.append (",description=" + this.description);
      }

      if (this.imageURL != null) {
        result.append (",image=" + this.imageURL.toString ());
        result.append (",width=" + this.imageWidth);
        result.append (",height=" + this.imageHeight);
      }


      return (result.toString ());
    }

    public boolean validate () {
      boolean valid = true;

      if (this.queryURL == null)
          valid = false;
      if (this.id == null)
          valid = false;
      if (this.title == null)
          valid = false;
      if (this.link == null)
          valid = false;
      if (this.description == null)
          valid = false;
      // providerIcon is optional
      return true;
    }

}
