/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Request.java,v 1.2 2001/06/12 17:58:12 yaroslav Exp $
 */
package net.jxta.search.peer;

import java.io.*;

import net.jxta.search.util.Tag;
import net.jxta.search.util.XmlParser;
import net.jxta.search.util.ValidatingXmlParser;


class RequestParserCallback implements ValidatingXmlParser.ParserCallback {

    static final int INSTRUCTION = 1;
    static final int REQUEST     = 2;

    static Tag root = new Tag ("?",
			       INSTRUCTION,
			       new Tag[] { 
				   new Tag ("request",
					    REQUEST,
					    new Tag [0]) 
				       // treat anything within a <request>
				       // </request> as character data
				       }
			       );
    
    static byte[] QUERY_SPACE            = "QUERY-SPACE".getBytes ();
    static byte[] QUERY_UUID             = "QUERY-UUID".getBytes ();
    static byte[] QUERY_LIFETIME         = "QUERY-LIFETIME".getBytes ();
    static byte[] MAX_FANOUT             = "MAX-FANOUT".getBytes ();
    static byte[] MAX_HITS_PER_PROVIDER  = "MAX-HITS-PER-PROVIDER".getBytes ();
    static byte[] FLUSH_AFTER_PROVIDERS  = "FLUSH-AFTER-PROVIDERS".getBytes ();
    static byte[] FLUSH_AFTER_MS         = "FLUSH-AFTER-MS".getBytes ();

    static int    REQUEST_DATA_LIMIT     = 2048;
 
    Request               request;
    ByteArrayOutputStream out;

    

    /*
    class Attribute {
	byte[] name;

	Attribute (String name) {
	    this.name = name.toUpperCase ().getBytes ();
	}
    }

    Attribute[] attrs = 
        new Attribute[] { new Attribute ("query_space"),
			  new Attribute ("query_uuid"),
			  new Attribute ("max_fanout"),
			  new Attribute ("max_hits_per_provider"),
			  new Attribute ("flush_after_providers"),
			  new Attribute ("flush_after_ms") };
    */

    RequestParserCallback (Request request) {
	this.request = request;
	this.out     = new ByteArrayOutputStream (REQUEST_DATA_LIMIT);
    }
    
    public void startTag (Tag tag, byte[] chars, int start, int len) {
	
	switch (tag.getNumber ()) {


	case REQUEST:
	    
	    
	    request.querySpace = 
		XmlParser.getAttributeValue (QUERY_SPACE, chars, start, len);
	    
	    request.queryUuid =
		XmlParser.getAttributeValue (QUERY_UUID, chars, start, len);

	    request.queryLifetime = 
		XmlParser.getIntAttribute (QUERY_LIFETIME, chars, start, len,
					   60 * 1000);

	    request.maxFanout = 
		XmlParser.getIntAttribute (MAX_FANOUT, chars, start, len,
					   Integer.MAX_VALUE);
	    
	    request.maxHitsPerProvider =
		XmlParser.getIntAttribute (MAX_HITS_PER_PROVIDER, 
					   chars, start, len,
					   100);

	    request.flushAfterProviders =
		XmlParser.getIntAttribute (FLUSH_AFTER_PROVIDERS, 
					   chars, start, len,
					   5);
	    
	    request.flushAfterMs =
		XmlParser.getIntAttribute (FLUSH_AFTER_MS, chars, start, len,
					   1000);

	    
	}
    }

	public void chars (Tag tag, String s) 
		throws XmlParser.Exception 
	{

	}

    public void cdata (Tag tag, byte[] chars, int start, int len) 
	throws XmlParser.Exception
    {
	switch (tag.getNumber ()) {

	case REQUEST:
	    // try {
		out.write (chars, start, len);
		/*
		  } catch (IOException e) {
		throw new XmlParser.Exception ("Request character data " +
					       "is limited to " + 
					       REQUEST_DATA_LIMIT + " bytes.");
	    }
		*/

	    break;
	}
    }
    


    public void endTag (Tag tag, byte[] chars, int start, int len) {
	switch (tag.getNumber ()) {

	case REQUEST:
	    request.data = out.toByteArray ();
	    break;
	}
    }

}


public class Request {
	
    String querySpace;
    String queryUuid;
    int    queryLifetime;
    int    maxFanout = Integer.MAX_VALUE;
    int    maxHitsPerProvider;
    int    flushAfterProviders;
    int    flushAfterMs;

    /**
     * Default XML namespace for JXTA Search request and response messages.
     */
    public static String DEFAULT_NAMESPACE = "http://search.jxta.org/";
    public String getQuery () { return new String (data); }
    public String getQuerySpace () { return querySpace; }
    public String getQueryUuid () { return queryUuid; }
    public int    getQueryLifetime () { return queryLifetime; }
    public int    getMaxFanout () { return maxFanout; }
    public int    getMaxHitsPerProvider () { return maxHitsPerProvider; }
    public int    getFlushAfterProviders () { return flushAfterProviders; }
    public int    getFlushAfterMs () { return flushAfterMs; }

    /** Value of the element.  May be null, as Request is sometimes used
	without having an data (e.g. resolver clients) **/
    byte[] data;
    

    Request ( String querySpace,
	      String queryUuid,
	      int    queryLifetime,
	      int    maxFanout,
	      int    maxHitsPerProvider,
	      int    flushAfterProviders,
	      int    flushAfterMs)
    {
	this.querySpace    = querySpace;
	this.queryUuid     = queryUuid;
	this.queryLifetime = queryLifetime;
	this.maxFanout     = maxFanout;
	this.maxHitsPerProvider  = maxHitsPerProvider;
	this.flushAfterProviders = flushAfterProviders;
	this.flushAfterMs        = flushAfterMs;
    }
    

    public Request ( String querySpace,
		     String queryUuid,
		     int    queryLifetime,
		     int    maxFanout,
		     int    maxHitsPerProvider,
		     int    flushAfterProviders,
		     int    flushAfterMs,
		     byte[] data)
    {
	this( querySpace,
	      queryUuid,
	      queryLifetime,
	      maxFanout,
	      maxHitsPerProvider,
	      flushAfterProviders,
	      flushAfterMs);

	this.data = data;
    }

    public Request (InputStream in, byte[] buffer) 
	throws IOException, XmlParser.Exception
    {
	RequestParserCallback cb = new RequestParserCallback (this);
	
	ValidatingXmlParser.parse (in, buffer, cb, cb.root);
    } 
    
    private String getStartTag (String uuid) {
	return 
	    "<?xml version='1.0'?>\n" +
	    "<request xmnls='" + DEFAULT_NAMESPACE + "'\n" +
	    "         query-space='" + querySpace + "'\n" +
 	    "         query-uuid='" + uuid + "'\n" +
	    "         query-lifetime='" + queryLifetime + "'\n" +
	    "         max-fanout='" + maxFanout + "'\n" +
	    "         max-hits-per-provider='" + maxHitsPerProvider + "'\n" + 
	    "         flush-after-providers='" + flushAfterProviders + "'\n" +
	    "         flush-after-ms='" + flushAfterMs + "'>\n";
	
    }
    
    private String getCloseTag() {
	return "</request>\n";
    }

    
    void writeStartTag (OutputStream out) 
	throws IOException
    {
	writeStartTag (queryUuid, out);
    }
    
    void writeStartTag (String uuid, OutputStream out) throws IOException {

	out.write (getStartTag (uuid).getBytes ());
    }


    void writeCloseTag ( OutputStream out ) throws IOException {

	out.write (getCloseTag ().getBytes ());
    }


    public void toStream (OutputStream out) throws IOException {
	toStream (queryUuid, out);
    }

    public void toStream (String newUuid, OutputStream out) 
	throws IOException
    {
	if (data == null)
	    throw new IllegalStateException ("Request body must be set.");
	
	writeStartTag (newUuid, out);
	out.write (data);
	writeCloseTag (out);
    }

    public int getLength () {
	return getLength (queryUuid);
    }

    public int getLength (String uuid) {
	return 
	    getStartTag (uuid).length () + 
	    data.length + 
	    getCloseTag ().length ();
    }

    
    /** 
     * Returns an input stream of a <request> element with the specified
     * value
     */
    public InputStream getInputStream () {
	return getInputStream (queryUuid);
    }

    public InputStream getInputStream(String newUuid) {
	StringBuffer sb = new StringBuffer();
	
	sb.append (getStartTag (newUuid));

	if( data != null )
	    sb.append (new String (data).trim ()).append ('\n');

	sb.append (getCloseTag ());

	return new ByteArrayInputStream ( sb.toString().getBytes() );
    }
}
