/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ResolverResponseHandler.java,v 1.3 2001/06/12 15:45:27 yaroslav Exp $
 */
package net.jxta.search.peer;


import net.jxta.search.util.Tag;
import net.jxta.search.util.ValidatingXmlParser;
import net.jxta.search.util.XmlParser;


import java.io.*;
import java.net.*;
import java.util.*;


class ResolverResponseHandler implements ValidatingXmlParser.ParserCallback {

    static final int INSTRUCTION = 1, RESULTS = 2, MATCH = 3, SCORE = 4,
	ID = 5, URL = 6, TITLE = 7, LINK = 8, DESCR = 9, IMAGE = 10, 
	I_URL = 11, I_WIDTH = 12, I_HEIGHT = 13;

    Tag root = 
	new Tag ("?xml",
		 INSTRUCTION,
		 new Tag[] { 
		     new Tag ("results",
			      RESULTS,
			      new Tag[] { 
				  new Tag ("match",
					   MATCH,
					   new Tag[] {
					       new Tag ("score", SCORE),
					       new Tag ("id",    ID),
					       new Tag ("url",   URL),
					       new Tag ("title", TITLE),
					       new Tag ("link",  LINK),
					       new Tag ("description", DESCR),
					       new Tag ("image", 
							IMAGE,
							new Tag[] {
							    new Tag ("url",
								     I_URL),
							    new Tag ("width",
								     I_WIDTH),
							    new Tag ("height",
								     I_HEIGHT)
								}) } ) } ) } );
					       
    Vector       providers = new Vector ();
    ProviderInfo provider;


    ProviderInfo[] getProviders () {
	
	ProviderInfo[] arr = new ProviderInfo [providers.size ()];
	
	providers.copyInto (arr);

	return arr;
    }

        
    public void startTag (Tag tag, byte[] chars, int start, int len) 
	throws XmlParser.Exception
    {
	switch (tag.getNumber ()) {

	case MATCH:
	    provider = new ProviderInfo ();
	    break;
	}
    }

    
    public void endTag (Tag tag, byte[] chars, int start, int len) 
	throws XmlParser.Exception
    {
	switch (tag.getNumber ()) {
	
	case MATCH:

	    if (provider != null && provider.validate ()) 
		
		providers.addElement (provider);
	    

	    break;
	}
    }

    
    
    public void cdata (Tag tag, byte[] ch, int start, int len) 
	throws XmlParser.Exception
    {

    }


    public void chars (Tag tag, String s)
	throws XmlParser.Exception
    {
	
	
	try {

	    switch (tag.getNumber ()) {
		
		// case SCORE:    provider.score = Integer.parseInt (s); break;
	    case URL:      provider.queryURL = new URL (s);           break;
	    case ID:       provider.id = s;                          break;
	    case TITLE:    provider.title = s;                       break;
	    case LINK:     provider.link  = new URL (s);             break;
	    case DESCR:    provider.description = s;                 break;
	    case I_URL:    provider.imageURL = new URL (s);          break;
	    case I_WIDTH:  provider.imageWidth = Integer.parseInt (s);  break;
	    case I_HEIGHT: provider.imageHeight = Integer.parseInt (s); break;
		
	    }

	} catch (MalformedURLException e) {
	    e.printStackTrace (); // xxx should log
	    // provider will be discarded after provider.validate () call
	}
    }

}
