/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ResponseHandler.java,v 1.3 2001/06/12 00:19:57 yaroslav Exp $
 */
package net.jxta.search.peer;

import net.jxta.search.util.Tag;
import net.jxta.search.util.ValidatingXmlParser;
import net.jxta.search.util.XmlParser;
import net.jxta.search.util.MemoryStream;

import java.io.*;
import java.net.*;
import java.util.*;


/**
 * ParserCallback to parse rsponses from router
 */

class ResponseHandler implements ValidatingXmlParser.ParserCallback {
    static final boolean DEBUG = false;

    static final int NONE      = 0;
    static final int RESPONSES = 1;
    static final int RESPONSE  = 2;
    static final int PROVIDER  = 3;
    static final int URL       = 4;
    static final int ID        = 5;
    static final int TIMESTAMP = 7;
    static final int COOKIE    = 8;
    static final int NAME      = 9;
    static final int ICON      = 10;
    static final int HOME      = 11;
    static final int DATA      = 12;
    static final byte[] DATA_TAG = "DATA".getBytes ();
    static final byte[] QUERY_UUID_ATTR = "QUERY-UUID".getBytes ();
    static final byte[] SRC_ATTR = "SRC".getBytes ();
    static final byte[] HEIGHT_ATTR = "HEIGHT".getBytes ();
    static final byte[] WIDTH_ATTR = "WIDTH".getBytes ();
    
    Tag root = 
	new Tag ("?xml",
		 -2,
		 new Tag[] {
		     new Tag ("responses",
			      RESPONSES,
			      new Tag [] { 
				  new Tag ("response",
					   RESPONSE,
					   new Tag[] { 
					       new Tag ("data",
							DATA,
							new Tag[0]),
					       new Tag ("provider",
							PROVIDER,
							new Tag[] {
							    new Tag ("url",URL),
							    new Tag ("id", ID),
							    new Tag ("timestamp",
								     TIMESTAMP),
							    new Tag ("cookie",
								     COOKIE),
							    new Tag ("name",
								     NAME),
							    new Tag ("icon",
								     ICON),
							    new Tag ("home",
								     HOME) 
								})})})});;
    
    
	

	String       uuid;
    ProviderInfo provider;
    Vector       providers;
	MemoryStream out;

	RouterClient.ResponseCallback cb;

    ResponseHandler (RouterClient.ResponseCallback cb) {
        this.cb = cb;
	}


    public void startTag (Tag tag, byte[] chars, int start, int len) 
		throws XmlParser.Exception
	{
		
		switch (tag.getNumber ()) {

		case RESPONSES:
                    if (DEBUG) 
			System.out.println ("Parsing <responses> tag ...");
		    uuid  = XmlParser.getAttributeValue (QUERY_UUID_ATTR, 
							 chars, start, len);
                    if (DEBUG)
			System.out.println ("uuid = " + uuid);
		    break;
		case RESPONSE:
                    if (DEBUG)
			System.out.println ("Parsing <response> tag ...");
		    providers = new Vector (2, 2);
		    		    
		    break;

		case PROVIDER:
		    provider = new ProviderInfo ();
		    out      = new MemoryStream ();

		    try {
			out.write ("<?xml version='1.0'?>\n".getBytes ());
						   
		    } catch (IOException e) {

		    }

		    break;

		case ICON:  
		    try {
			provider.imageURL = 
			    new URL (XmlParser.getAttributeValue (SRC_ATTR,
								  chars, 
								  start, 
								  len));
			provider.imageHeight = 
			    Integer.parseInt (XmlParser.getAttributeValue (HEIGHT_ATTR,
									   chars,
									   start, 
									   len));

			provider.imageWidth = 
			    Integer.parseInt (XmlParser.getAttributeValue (WIDTH_ATTR,
									   chars,
									   start,
									   len));
			
		    } catch (MalformedURLException e) {
			// xxx log
		    } catch (NumberFormatException e) {
			// xxx log
		    }
		    break;

		case DATA:
			
			// if (! XmlParser.startsWith (DATA_TAG, chars, start+1, len-1)) 
				try {
					out.write (chars, start, len);
				} catch (IOException e) {
					// memory stream ;)
				}

			break;
		}
    }

    public void endTag (Tag tag, byte[] chars, int start, int len)
		throws XmlParser.Exception
    {
		switch (tag.getNumber ()) {
		case PROVIDER:
		    System.out.println ("\t from " + provider);
			providers.addElement (provider);
			provider = null;
			break;

		case DATA:
			// if (! XmlParser.startsWith (DATA_TAG, chars, start+2, len-2)) 
				try {
					out.write (chars, start, len);
				} catch (IOException e) {
					// memory stream ;)
				}
			break;
		case RESPONSE:
			ProviderInfo[] arr = new ProviderInfo[providers.size ()];
			providers.copyInto (arr);
						
			try {
				if (out != null) {

					System.out.println ("Response from " + arr[0] + 
										" has data.");
					
					
					InputStream in = out.getInputStream ();
                                        if (DEBUG) {
					    byte[] buf = new byte[128];
					    int r;
					
					    System.out.println ("\t Response document: ");
					    while ((r = in.read (buf)) > 0)
						    System.out.print (new String (buf, 0, r));
                                        }
					
					cb.response (uuid, arr, out.getInputStream ());
					
				} else {
					System.out.println ("Response from " + arr[0] + 
										" doesn't have any data.");
				}
				
			} catch (IOException e) {
				throw new Error ("IOException can't happen on memory stream.");
			}
				

			break;
		}
    }


    public void chars (Tag tag, String chars) throws XmlParser.Exception {
	try {
	    switch (tag.getNumber ()) {
	    case URL:   
		provider.queryURL = new URL (chars);
		break;
	    case NAME:  
		provider.title = chars;
		break;
	    case ID:    
		provider.id  = chars;
		break;
	    case HOME:
		provider.link = new URL (chars);
		break;
	    }
	} catch (MalformedURLException e) {
	    
	}
    }
    

    public void cdata (Tag tag, byte[] ch, int start, int len)
	throws XmlParser.Exception
    {

	switch (tag.getNumber ()) {
	case DATA:
	    try {
		out.write (ch, start, len);
	    } catch (IOException e) {
		// memory stream ;)
	    }
	    break;
	}
    }
    
    
}
