/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ManagerServlet.java,v 1.1.1.1 2001/06/01 16:50:00 jbeatty Exp $
 */
package net.jxta.search.resolver;

import java.io.*;
import java.net.URL;
import javax.servlet.*;
import javax.servlet.http.*;
import org.xml.sax.SAXException;

public class ManagerServlet extends HttpServlet {

    ResolverService resolver;
    File registrationDir;
    
    public ManagerServlet () {
	resolver = ServiceLoader.getResolverService ();
    }


    public void init() throws ServletException {

	ServletContext context = getServletContext ();
	
	System.out.println ("ManagerServlet.init: begin");

	try {
	    URL regDirURL = context.getResource ("regs");
	    
	    String regDirName = regDirURL.getFile ();
	    
	    if (regDirName == null) {
		URL resource = getServletContext().getResource("regs");
		if (resource != null && "file".equals(resource.getProtocol()))
		    regDirName = resource.getFile();
		else
		    throw new ServletException("ManagerServlet: Can't find " +
					       "registrations");
	    }
	
	    registrationDir = new File(regDirName);
	} catch( Exception e ) {
	    throw new ServletException( e );
	}
    }

    public void doPost (HttpServletRequest req,
			HttpServletResponse res)
	throws ServletException, IOException
    {
	String       action     = req.getHeader ("Resolver-Action");
	String       externalId = req.getHeader ("Resolver-ExternalId");

	System.out.println ("ManagerServlet: action = " + action + 
			    ", externaId = " + externalId);

	if (action.equals ("add")) {
	    doAdd (externalId, req, res);
	} else if (action.equals ("remove")) {
	    doRemove (externalId, req, res);
	} 

    }

    private void doAdd (String externalId, 
			HttpServletRequest req,
			HttpServletResponse res)
	throws ServletException, IOException
    {

	InputStream  in         = req.getInputStream ();
	File         file       = new File (registrationDir,
					    externalId+".xml");
	OutputStream out        = new FileOutputStream (file);
	byte[]       buf        = new byte[8192];
	int          r;

	

	try {
	    while ((r = in.read (buf)) > 0) {
		
		out.write (buf, 0, r);
	    }
	    
	    out.close ();
	    out = null;
	    
	    resolver.addProvider (externalId, file);
	} catch (SAXException e) {
	    res.sendError (400, "Cannot parse: " + e.getMessage ());
	} catch (IOException e) {
	    try {
		out.close ();
		out = null;
	    } catch (IOException ex) {
		
	    }
	    
	    file.delete ();
	    
	    res.sendError (400, "Cannot create registration file.");
	} finally {
	    if (out != null) {
		try { 
		    out.close ();
		} catch (IOException ex) {
		    
		}
		
	    }
	}
    }

	
    private void doRemove (String externalId, 
			   HttpServletRequest req,
			   HttpServletResponse res)
	throws ServletException, IOException
    {
	
	resolver.removeProvider (externalId);
    	
    }

}
