/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Predicate.java,v 1.1.1.1 2001/06/01 16:50:01 jbeatty Exp $
 */

package net.jxta.search.resolver;
import java.util.ArrayList;

/** Used by a provider to identify queries that it should respond to.
 * Predicates are in conjunctive normal form: they consist of a conjunction of
 * disjunctions of patterns.  Patterns are strings representing keywords and
 * the XML tags containing them. */
class Predicate {

  ArrayList conjunction;


  Predicate(ArrayList conjunction) {
    this.conjunction = conjunction;
  }

  /** Returns the conjunction of disjunctions. */
  public ArrayList getConjunction() { return conjunction; }

  /** For debugging.  Presents the disjunctions in an 'and' tag. */



  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("    <and>\n");
    for (int i = 0; i < conjunction.size(); i++) {
      buffer.append(conjunction.get(i));
    }
    buffer.append("    </and>\n");
    return buffer.toString();
  }


  /* FIXME: This is a documentation issue: When constructing 
   * a predicate directly from a String, should each term in
   * the String be added to the ArrayList (conjunction)
   * separately, or should the whole string with all the
   * terms be added all at once.
   */
  public static void test() {

     String words = "foo ";
     ArrayList conjunction = new ArrayList();
     conjunction.add(words);
     conjunction.add("bar ");
     conjunction.add("baz ");
     conjunction.add("\n");
     conjunction.add("oof rab zab\n");
   
     Predicate p = new Predicate(conjunction);

     System.out.println(p);

   } /* close test() */

 
   public static void main(String [] args) {

      test();
   }


}  /* class Predicate */
