/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ProviderSimilarity.java,v 1.1.1.1 2001/06/01 16:50:05 jbeatty Exp $
 */

package net.jxta.search.resolver;
import java.util.*;

class ProviderSimilarity 
{
    class ScoreProviderPair implements Comparable
    {
	int providerId=0;
	int score=0;
	public int compareTo(Object object) {
	    ScoreProviderPair other = (ScoreProviderPair) object;
	  if (this.score == other.score)
	      return this.providerId - other.providerId;
	  else return (other.score - this.score); // descending
	}
	public boolean equals(Object object) {
	    ScoreProviderPair other = (ScoreProviderPair)object;
      return ((this.score == other.score)
	      && (this.providerId == other.providerId)); 
	}
	
	ScoreProviderPair(int providerId, int score)
	{
	    this.providerId = providerId;
	    this.score = score;
	}

	public int getProviderId() 
	{ return providerId; }
	
	public int getScore()
	{ return score; }
    }
    int maxValue;//keep track of the max sim value and use it for normalization 
    int nUpdates = 0; // keeps track of how many updateSimilarity calls. 
    int minUpdates; // min num of updates for valid method returns. 
    int multiplier = 5;// used to determine min num of updates
    int nProviders; // numberOfProviders
    
    //  int[][] similarity;
    Vector similarity; //a vecor of int[] instead of int[][] so that we can add 
                       //or remove rows easily
    

    /* getSimilarProviders return providers with similar profiles based on 
       their similarity matrix */
    /* array returned does not include self similarity */
    /* this method is similar to sortProviders */
    public int[] getSimilarProviders(int providerId, int maxSimilars)
    {
	if(nUpdates < minUpdates) return null;
	int[] providerIds = sortProviders(providerId, maxSimilars);
	return providerIds;
    }
    
    // sort providers in a descending score order
    // returns the nSim providers with the highest scores
    // excludes sim(providerId, providerId)
    private int[] sortProviders(int providerId, int nSim)
    {
	if(nUpdates < minUpdates) return null;
	TreeSet setOfScoreProviderPairs = new TreeSet();

	for(int i=0; i<providerId;i++)
	    {
	     //ScoreProviderPair x = new ScoreProviderPair(i, similarity[providerId][i]);
		ScoreProviderPair x = new ScoreProviderPair(i, ((int[]) similarity.get(providerId))[i]);
		setOfScoreProviderPairs.add(x);
	    }
	for(int i=providerId+1;i < nProviders;i++)
	    {
	     //ScoreProviderPair x = new ScoreProviderPair(i, similarity[i][providerId]);
		ScoreProviderPair x = new ScoreProviderPair(i, ((int [])similarity.get(i))[providerId]);
		setOfScoreProviderPairs.add(x);
	    }
	Iterator it = setOfScoreProviderPairs.iterator();
	int[] result = new int[nSim];
	int i = 0;
	while(it.hasNext()){
	    result[i++] = ((ScoreProviderPair) it.next()).getProviderId();
	    if(i == nSim) break;
	}
	return result;
    }
    
    /* updateSimilarity updates the similarity based on similar responses to queries 
    * updates are symmetric */
    public void updateSimilarity(int[] similarProviders)
    {
	nUpdates++;
	int n = similarProviders.length;
	// update all combinations of similarProviders
	// also updates sim(a,a)
	for(int j = 0; j < n ; j++)
	    {
		for(int i=j; i < n; i++)
		    {
			int x = similarProviders[i];
			int y = similarProviders[j];
			if (x >= y) maxValue = Math.max(maxValue, ++((int[])similarity.get(x))[y]);
			else if(y > x) maxValue = Math.max(maxValue, ++((int[])similarity.get(y))[x]);
		    }
	    }
    }
    
    /* getSimilarity returns the similarity between two providers 
     * i.e. proportional to the probability the 2 providers respond to the same query */
        
    public float getSimilarity(int providerA, int providerB)
    {
	if(nUpdates < minUpdates) return 0;
	int[] providers = new int[2];
	providers[0] = providerA;
	providers[1] = providerB;
	if(areValidProviders(providers))
	    {
		if(providerA >= providerB)
		    return ((float)((int[])similarity.get(providerA))[providerB]/maxValue);
		else return ((float)((int[])similarity.get(providerB))[providerA]/maxValue);
	    }
	else return 0;  // one or both providers are not valid
    }
    
    // returns the "maxNum" providerIds with the most probability being similar to others
    // ranked in a descending order
    public int[] getMostPopular(int maxNum)
    {
	if(nUpdates < minUpdates) return null;

	TreeSet setOfScoreProviderPairs = new TreeSet();

	for(int i=0; i<nProviders;i++)
	    {
		ScoreProviderPair x = new ScoreProviderPair(i, ((int[])similarity.get(i))[i]);
		setOfScoreProviderPairs.add(x);
	    }
	Iterator it = setOfScoreProviderPairs.iterator();
	int[] result = new int[maxNum];
	int i = 0;
	while(it.hasNext()){
	    result[i++] = ((ScoreProviderPair) it.next()).getProviderId();
	    if(i == maxNum) break;
	}
	return result;
    }


    /* check if providers are valid */
    private boolean areValidProviders(int[] providers)
    {
	for(int i=0; i< providers.length; i++)
	    {
		if((providers[i]<0)||(providers[i]>=nProviders)) return false;
	    }
	return true;
    }

    public void addProvider()
    {
	nProviders++;
	minUpdates += multiplier;
	similarity.add(new int[nProviders]);
    }
    ProviderSimilarity(Index index)
    {
	try {
	    initSimilarity(index);
	}
	catch(Exception e)
	    {
		System.err.println("ProviderSimilarity initialization failed");
		e.printStackTrace();
	    }
    }
    

    public void initSimilarity(Index index) throws Exception
    {
	try {
	    nProviders = index.getMaxProviderId();
	    maxValue = 0;
	    minUpdates=nProviders*multiplier;// min number of calls to updateSimilarity
	}
	catch(Exception e) 
	    {
		throw new Exception(e.toString());
	    }
	if(nProviders > 0)
	    {
		similarity = new Vector(nProviders);
		for(int i=0; i < nProviders; i++)
		    {
			similarity.add(new int[i+1]);
		    }
	    }
	else similarity = new Vector();
	// TODO  (not necessary)
	// get initial similarity from index
    }


    /** No parameter test method is useful for providing 
     * "internal documentation" of command line parameters.
     */
    public static void testProviderSimilarity() {

	String [] args = {"foo", "bar", "baz"};
        testProviderSimilarity(args);
    }


    /** Arguments can be passed in from main (command line),
     * or built in a no-parameter test call and passed in.
     */
    public static void testProviderSimilarity(String[] args)
    {
	//TODO
	System.out.println("TODO: Implement unit testing for ProviderSimilarity");
    }

    public static void main(String[] args)
    {
	testProviderSimilarity();
    }

}
