/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Query.java,v 1.1.1.1 2001/06/01 16:50:05 jbeatty Exp $
 */

package net.jxta.search.resolver;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import java.util.*;
import java.io.*;
import net.jxta.search.peer.Request;


/** The set of patterns extracted from an XML query. */
class Query {

  TreeSet patterns;
  String  space;
  int     maxProviders;

  /** Constructs a query from a sequence of keyword tokens. */
  public Query(Enumeration tokens, String queryspace) {
    this.patterns = new TreeSet();
    this.space = queryspace;
    while (tokens.hasMoreElements())		  // tokenize text
      patterns.add((String)tokens.nextElement()); // add pattern per word
	
  }

  /** Constructs a query from an XML-formatted String. */
  public Query(XMLReader reader, String queryString)
      throws ParserConfigurationException, SAXException, IOException {
      this(reader, new ByteArrayInputStream(queryString.getBytes()));
  }

  /** Constructs a query from an XML-formatted InputStream. */
  public Query(XMLReader reader, InputStream inputStream)
      throws ParserConfigurationException, SAXException, IOException {

    QueryHandler handler = new QueryHandler();

    reader.setContentHandler (handler);

    reader.parse(new InputSource (inputStream));

    this.patterns = handler.getPatterns();
    this.space    = handler.getSpace();
    this.maxProviders = handler.getMaxProviders ();
  }



    public static void testQuery() throws SAXException {

	XMLReader reader = XMLReaderFactory.createXMLReader ();

	/* Take the xml tags out and it fails...
	 * TODO: write a test for that case...
	 */
	String kwstring = "<text>foo bar baz</text>";
	/* FIXME: Check the calling code to make sure this 
	 * is really what we want to do...
	 */
	QueryTokenizer qt = new QueryTokenizer();
        Enumeration enum = qt.newTokenizer(kwstring);
	String querySpace = "http://infrasearch.com/text";
        Query query1 = new Query(enum, querySpace); 
	System.out.println("Tokenized Query:\n" + query1.patterns.toString());
	System.out.println("query-space: " + query1.space);

        try {
	    String queryString = "<?xml version='1.0'?> <request xmlns='" + Request.DEFAULT_NAMESPACE + "' query-space='http://infrasearch.com/text' uuid=1CBEF> <response-schema>http://purl.org/rss/1.0</response-schema> <query> <text>foo bar</text> </query> </request> " ;

	    Query query2 = new Query(reader, queryString);
  	    System.out.println("Keyword string Query:\n" + query2.patterns.toString());
	    System.out.println("query-space: " + query2.space);
	} catch (Exception e) {
	    System.out.println("Query(kwstring) failed.\n" + e);
	}

	/* FIXME: the CNN.xml.com is just a handy xml file 
	 * to throw at the constructor.  Should probably have
	 * some sort of "query" file for testing.
         */
	File queryfile = new File("./queryhandletest.xml");

	try {
	    FileInputStream fis = new FileInputStream(queryfile);
	    Query query3 = new Query(reader, fis);
   	    System.out.println("Regs file Query:\n" + query3.patterns.toString());
	    System.out.println("query-space: " + query3.space);
	    
	} catch (ParserConfigurationException pce) {
	    System.out.println(pce);
	} catch (SAXException se) {
	    System.out.println(se);
	} catch (IOException ioe) {
	    System.out.println(ioe);
	}

    }  /* close testQuery() */


    public static void main(String [] args) throws SAXException {

	testQuery();

    }  /* close main() */


}  /* class Query */
