/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: QueryTokenizer.java,v 1.1.1.1 2001/06/01 16:50:05 jbeatty Exp $
 */

package net.jxta.search.resolver;
import java.util.*;

/** Used to tokenize query textual data into pattern words. */
class QueryTokenizer {

    /** Tokenize a query by returning an inner class object.
     *
     * @param String text: a valid query string.
     * @return Enumeration object inner class object for 
     *         tokenizing the query string.
     * @see Query#testQuery
     * @see QueryHandler#addText
     * @see ResolverServlet#doGet
     */

    /* FIXME: parameters validation is not done here.  If it 
     * not done anywhere, a case should be made that it isn't
     * necessary.  If this case is not convincing, then some
     * sort of validation needs to done and documented.
     */
  public static Enumeration newTokenizer(String text) {

    final String normalizedText = text.trim().toLowerCase();

    return new Enumeration() {
	private Enumeration tokenizer =
	  TextTokenizer.newTokenizer(normalizedText);
	private boolean done;
	private String token;

	public boolean hasMoreElements() {
	  if (done)
	    return false;
	  if (tokenizer.hasMoreElements())
	    return true;
	  if (token != null && token.equals(normalizedText))
	    return false;
	  return true;
	}

	public Object nextElement() {
	  if (tokenizer.hasMoreElements())
	    return token = (String)tokenizer.nextElement();
	  done = true;
	  if (normalizedText.length() != 0) {
	    return normalizedText;
	  } else
	    throw new NoSuchElementException();
	}

      };

  }  /* close newTokenizer() */



    /** FIXME: More better documentation needed for testing function,
     * including valid/invalid parameter specification.
     */
    private static void testQueryTokenizer() {

	String instring = "foo bar baz";
	Enumeration enum = QueryTokenizer.newTokenizer(instring);
	while (enum.hasMoreElements()) {
	    System.out.println(enum.nextElement());
	}

    }  /* close testQueryTokenizer() */

    public static void main(String [] args) {

	testQueryTokenizer();

    }  /* close main() */


}  /* class QueryTokenizer */
