/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ResolverService.java,v 1.1.1.1 2001/06/01 16:50:06 jbeatty Exp $
 */

package net.jxta.search.resolver;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;

/** Service to access resolver */
public class ResolverService {

    private Hashtable querySpaceToIndex = new Hashtable();


    public void addProvider (String id, File file) 
	throws SAXException, IOException	       
    {
	Provider provider = new Provider (file);
	TreeSet querySpaces = provider.getQuerySpaces();
	Iterator iterator = querySpaces.iterator();	
	while(iterator.hasNext())
	    {
		String qs = (String) iterator.next();
		Index    index    = (Index) querySpaceToIndex.get(qs);
		if (index == null) 
		    querySpaceToIndex.put (qs, index = new Index (qs));
		index.add (id, provider);
	    }
    }


    public void addProvider (String id, File file, File categorydir) 
	throws SAXException, IOException	       
    {
	Provider provider = new Provider (file);
	TreeSet querySpaces = provider.getQuerySpaces();
	Iterator iterator = querySpaces.iterator();	
        provider.setCategoryDir(categorydir);
	while(iterator.hasNext())
	    {
		String qs = (String) iterator.next();
		Index    index    = (Index) querySpaceToIndex.get(qs);
		if (index == null) 
		    querySpaceToIndex.put (qs, index = new Index (qs));
		index.add (id, provider);
	    }
    }


    /** Delete a provider registration */
    public void removeProvider (String id) {
	Enumeration e = querySpaceToIndex.elements ();

	while (e.hasMoreElements ()) {
	    Index index = (Index) e.nextElement ();
	    index.remove (id);
	}
    }

    /** Used to mark a provider not available. *
     *  This method does not alter a provider registration */
    public void setOffline(String id)
    {
	Enumeration e = querySpaceToIndex.elements ();
	while (e.hasMoreElements()) {
	    Index index = (Index) e.nextElement ();
	    index.setProviderStatus(id, false);
	}
    }
    
    /** Used to mark a provider available to answer queries */
    public void setOnline(String id)
    {
	Enumeration e = querySpaceToIndex.elements ();
	while (e.hasMoreElements()) {
	    Index index = (Index) e.nextElement ();
	    index.setProviderStatus(id, true);
	}
	
    }

    /** Update a provider registration with a new one */
    public void changeRegistration (String id, File file)
	throws SAXException, IOException
    {
	removeProvider (id);
	addProvider (id, file);
    }


    /** Increase score based on user positive feedback */
    public void goodFeedback(Query query, String providerId)
    {
	Index index = (Index) querySpaceToIndex.get(query.space);
	if(index == null) return;
	UserFeedback.goodFeedback(query, index, providerId);
    }

    /** Decrease score based on user negative feedback */
    public void badFeedback(Query query, String providerId)
    {
	Index index = (Index) querySpaceToIndex.get(query.space);
	if(index == null) return;
	UserFeedback.badFeedback(query, index, providerId);
    }

    /** Get providers who match the query patterns */
    public Searcher.Match[] doSearch (Query  query) {
	
	Index    index = (Index) querySpaceToIndex.get(query.space);
	
	if (index == null) {
	    return null;
	} else {
	    Searcher searcher = new Searcher();
	    
	    return searcher.search(query, index);
	}
    }


    public Index getIndex (String qs) {
	return (Index) querySpaceToIndex.get (qs);
    }

    private static void testResolverService() {

	System.out.println("Failed ResolverService test, no tests implemented.");

    }  /* close testResovrService() */


    public static void main (String [] args) {

	testResolverService();

    }  /* close main() */

}
