/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ResolverServlet.java,v 1.1.1.1 2001/06/01 16:50:07 jbeatty Exp $
 */

package net.jxta.search.resolver;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

/* xml imports */
import javax.xml.parsers.*;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.w3c.dom.*;

import net.jxta.search.util.ThreadService;
import net.jxta.search.util.PriorityQueue;
import net.jxta.search.jxtastarter.JxtaStarter;

/* jxta imports */
import net.jxta.peergroup.PeerGroup;


/** Used to access the resolver. */
public class ResolverServlet extends HttpServlet {

    private static ThreadService      threads;
    private static ResolverService    resolver;

    private PeerGroup       peerGroup;
    private ResolverJxtaApp resolverJxtaApp;
    
    static {
	System.out.println ("(((((((((((((((RESOLVER)))))))))))))))");
    }
    
    
    public ResolverServlet () {

	
	peerGroup = JxtaStarter.getMyPeerGroup ();
	threads   = new ThreadService (100, 200, 100, false, false);
	resolver  = ServiceLoader.getResolverService  ();

	

    }

    /** Indexes all provider registrations found in a directory.  The directory
     * is named by the servlet initialization parameter named "regDir".  The
     * default directory pathname is "regs". */
    
    public void init() throws ServletException {
	ServletContext context = getServletContext ();
	
	System.out.println ("ResolverServlet.init: begin");

	try {
	    String regDirName =
		getServletConfig().getInitParameter("RegistrationsDir");
	    if (regDirName==null) {
		regDirName="regs";
		System.out.println( "RegistrationsDir not specified in "+
				    "web.xml. Using 'regs' as default" );
	    }
	    File regDir = new File(regDirName);
	    System.out.println( "ResolverServlet: Registration directory: "+
				regDir.getAbsolutePath() );
	    if( !regDir.exists() ) {
		try {
		    regDir.mkdir();
		    System.out.println( "Registration directory created: " +
					regDir.getAbsolutePath() );
		} catch (SecurityException e) {
		    throw new ServletException( "Registration directory does "+
						"not exist. No privileges to "+
						"create it: " + 
						regDir.getAbsolutePath() );
		}
	    }


	    URL catsDirURL = context.getResource( "categorydata" );
	    String catsDirName = catsDirURL.getFile ();
	    if( catsDirName == null )
		throw new ServletException( "Can't find category data" );
	    File catsDir = new File( catsDirName );


	    File[] files = regDir.listFiles();
	    for (int i = 0; i < files.length; i++) {
		if (files[i].getName ().endsWith (".xml")) {
		    File file = files[i];
		    System.out.println("Indexing " + file);
		    try {
			resolver.addProvider (file.getName (), file, catsDir);
		    }
		    catch(Exception e)
			{
			    System.err.println("Error in " + file.getName() + " registration");
			}
		}
	    }

	    String documentBuilderFactoryClass = getServletConfig ().
		getInitParameter ("DocumentBuilderFactory");
	    String xmlReaderClass = getServletConfig ().
		getInitParameter ("XMLReader");
	    
	    System.setProperty ("javax.xml.parsers.DocumentBuilderFactory",
				documentBuilderFactoryClass);
	    System.setProperty ("org.xml.sax.driver",
				xmlReaderClass);

	    DocumentBuilderFactory factory = 
		DocumentBuilderFactory.newInstance ();

	    
	    resolverJxtaApp = 
		new ResolverJxtaApp (resolver,
				     threads,
				     factory.newDocumentBuilder ());
	    
	    
	    resolverJxtaApp.init (peerGroup, null);
	    resolverJxtaApp.startApp (null);
	    
	    new Thread( new JxtaProviderInterface( peerGroup,
						   regDir ) ).start();

	    System.out.println ("ResolverServlet: initted");

	    
	} catch (Exception e) {
	    throw new ServletException(e);
	}
    }


    /** Displays matches for the URL parameter "query". */
    public void doGet (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
    {
	System.out.println ("ResolverServlet.doGet: begin");

	String queryString = req.getParameter("query");
	if (queryString == null) {
	    System.out.println ("no query specified.");
	    throw new ServletException("no query specified");
	}
	
	try {
	    XMLReader reader = XMLReaderFactory.createXMLReader ();
	    Query     query  = new Query(reader, queryString);

	    doResolve (req, res, query);
	} catch (ParserConfigurationException e) {
	    System.out.println ("Error parsing query: " + e);
	    throw new ServletException("Error parsing query: " + e);
	} catch (SAXException e) {
	    throw new ServletException("Error creating xml parser: " + e);
	}
    }

    /** Displays matches for a posted XML query. */
    public void doPost (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
    {
	System.out.println ("ResolverServlet.doPost: begin");

	String contentType = req.getContentType();
	InputStream in;
	
	if (("text/xml".equals(contentType) ||
	     "application/xml".equals(contentType))) {
	    in = new BufferedInputStream(req.getInputStream());
	} else if (contentType != null &&
		   contentType.startsWith("multipart/form-data")) {
	    in = getXmlPart(contentType, req);
	} else {
	    System.out.println ("content type not XML: " + contentType);
	    throw new ServletException("content type not XML: " + contentType);
	}
	
	try {
	    XMLReader reader = XMLReaderFactory.createXMLReader ();
	    Query query = new Query (reader, in);
	    doResolve (req, res, query);
	    System.out.println ("ResolverServlet.doPost: end");
	} catch (SAXException e) {
	    System.out.println ("Error parsing query: " + e);
	    throw new ServletException("Error parsing query: " + e);
	} catch (ParserConfigurationException e) {
	    System.out.println ("Error parsing query: " + e);
	    throw new ServletException("Error parsing query: " + e);
	} catch (Throwable t) {
	    t.printStackTrace ();
	}
    }
    

    private InputStream getXmlPart(String contentType, HttpServletRequest req)
	throws ServletException, IOException
    {
	final String BOUNDARY = "boundary=";
	int boundaryStart = contentType.indexOf(BOUNDARY);
	if (boundaryStart == -1)
	    throw new ServletException("no boundary: " + contentType);
	String boundary =
	    "--" + contentType.substring(boundaryStart+BOUNDARY.length());
	BufferedReader reader =
	    new BufferedReader(new InputStreamReader(req.getInputStream()));
	String line = reader.readLine();
	if (!line.equals(boundary))
	    throw new ServletException("no boundary: " + line);
	for (line = reader.readLine(); line != null && !"".equals(line);
	     line = reader.readLine()) {
	    line = line.toLowerCase();
	    if (line.startsWith("content-type: ")) {
		if (!(line.startsWith("content-type: text/xml") ||
		      line.startsWith("content-type: application/xml")))
		    throw new ServletException("not XML: " + line);
	    }
	}
	StringBuffer part = new StringBuffer();
	boundary = boundary + "--";
	for (line = reader.readLine(); line != null && !boundary.equals(line);
	     line = reader.readLine()) {
	    part.append(line);
	    part.append('\n');
	}
	return new ByteArrayInputStream(part.toString().getBytes());
    }
    

    private void doResolve (HttpServletRequest  req, 
			    HttpServletResponse res,
			    Query               query)
	throws ServletException, IOException
    {
	Searcher.Match[] matches = resolver.doSearch (query);
	PrintWriter out = res.getWriter();
	
	res.setContentType("text/xml");
	
	out.println("<?xml version='1.0'?>");
	out.println("<results>");
	for (int i = 0; matches != null && i < matches.length; i++) {
	    Searcher.Match match = matches[i];
	    out.println("<match>");
	    out.println("<score>" + match.getScore() + "</score>");
	    out.println("<id>" + match.getExternalId() + "</id>");
	    out.println("<url>" + match.getQueryServer()+ "</url>");
            out.println("<title>" + match.getProviderTitle() + "</title>");
            out.println("<link>" + match.getProviderLink().toString() + "</link>"); 
            out.println("<description>" + match.getProviderDescription() + "</description>");
            out.println("<image>");
            out.println("<url>" + match.getProviderImageURL().toString() + "</url>");
            out.println("<width>" + match.getProviderImageWidth() + "</width>");
            out.println("<height>" + match.getProviderImageHeight() + "</height>");
            out.println("</image></match>");
	}
	out.println("</results>");
    }


    
}

