/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: StringLongReader.java,v 1.1.1.1 2001/06/01 16:50:11 jbeatty Exp $
 */

package net.jxta.search.resolver;
import net.jxta.search.resolver.io.*;
import java.io.*;
import java.util.*;

class StringLongReader {


  private Directory directory;
  private String fileName;
  private boolean deltaDecodeLongs;

  private long[] longs;
  private String[] strings;

  public StringLongReader(Directory directory, String fileName,
			  boolean deltaDecodeLongs)
      throws IOException {
    this.directory = directory;
    this.fileName = fileName;
    this.deltaDecodeLongs = deltaDecodeLongs;
  }

  public long get(String key) throws IOException {
    int index = Arrays.binarySearch(getStrings(), key);
    if (index < 0)
      return -1;
    else
      return getLongs()[index];
  }

  public String getKey(int i) throws IOException {
    return getStrings()[i];
  }

  public int length() throws IOException {
    return getLongs().length;
  }

  private long[] getLongs() throws IOException {
    if (longs == null)
      longs = directory.readVLongFile(fileName+"x", deltaDecodeLongs);
    return longs;
  }

  public String[] getStrings() throws IOException {
    if (strings == null) {
      int length = getLongs().length;
      String[] result = new String[length];
      Input in = directory.openFile(fileName+"s");
      try {
	for (int i = 0; i < length; i++)
	  result[i] = readString(in);
	this.strings = result;
      } finally {
	in.close();
      }
    }
    return strings;
  }

  private char[] stringBuffer = {};

  private synchronized String readString(Input i) throws IOException {
    int start = i.readVInt();
    int length = i.readVInt();
    int totalLength = start + length;
    if (stringBuffer.length < totalLength) {
      char[] newBuffer = new char[totalLength];
      System.arraycopy(stringBuffer, 0, newBuffer, 0, stringBuffer.length);
      stringBuffer = newBuffer;
    }
    i.readChars(stringBuffer, start, length);
    return new String(stringBuffer, 0, totalLength);
  }

}
