/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: StringLongWriter.java,v 1.1.1.1 2001/06/01 16:50:11 jbeatty Exp $
 */

package net.jxta.search.resolver;

import net.jxta.search.resolver.io.*;
import net.jxta.search.util.*;
import java.io.IOException;

class StringLongWriter {
  private Directory directory;
  private String fileName;
  private boolean deltaEncodeLongs;
  
  private Output stringOut;

  private LongVector longs = new LongVector();

  public StringLongWriter(Directory directory, String fileName,
			  boolean deltaEncodeLongs)
      throws IOException {
    this.directory = directory;
    this.fileName = fileName;
    this.deltaEncodeLongs = deltaEncodeLongs;

    stringOut = directory.createFile(fileName+"s");
  }

  public void close() throws IOException {
    stringOut.close();
    directory.writeVLongFile(longs.array(), longs.length(),
			     fileName+"x", deltaEncodeLongs);
  }

  public void add(String s, long l) throws IOException {
    writeString(s);
    longs.add(l);
  }

  private String lastString = "";

  private final void writeString(String string)
       throws IOException {
    int start = stringDifference(lastString, string);
    int length = string.length() - start;
    
    stringOut.writeVInt(start);			  // write shared prefix length
    stringOut.writeVInt(length);		  // write delta length
    stringOut.writeChars(string, start, length);  // write delta chars

    lastString = string;
  }

  private static final int stringDifference(String s1, String s2) {
    int len1 = s1.length();
    int len2 = s2.length();
    int len = len1 < len2 ? len1 : len2;
    for (int i = 0; i < len; i++)
      if (s1.charAt(i) != s2.charAt(i))
	return i;
    return len;
  }
}
