/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: HttpRouteConnectionService.java,v 1.1.1.1 2001/06/01 16:50:15 jbeatty Exp $
 */
package net.jxta.search.router;

import java.io.*;
import java.net.*;
import java.util.*;

import org.w3c.dom.*;

import net.jxta.search.peer.Request;
import net.jxta.search.peer.ProviderInfo;
import net.jxta.search.util.LogService;
import net.jxta.search.util.HttpConnectionService;
import net.jxta.search.util.HttpConnection;
import net.jxta.search.util.MemoryStream;
import net.jxta.search.util.XmlParser;

import javax.servlet.http.Cookie;


class HttpRouteConnectionService implements Router.ConnectionService {

    HttpConnectionService httpConnectionService;
    LogService            logService;
    XmlParser             parser;

    HttpRouteConnectionService (LogService logService) {
	
	HashMap dontCacheHosts = new HashMap ();

	try {
	    dontCacheHosts.put (new URL ("http", "localhost", 8080, "/"), 
				Boolean.TRUE);
	    dontCacheHosts.put (new URL ("http", "127.0.0.1", 8080, "/"),
				Boolean.TRUE);
	} catch (MalformedURLException e) {
	    
	}

	this.httpConnectionService = 
	    new HttpConnectionService (dontCacheHosts);
	this.logService = logService;
	this.parser     = new XmlParser ();
    }

    public Router.Connection openConnection (Router.Merger merger,
                                             ProviderInfo  providerInfo,
					     Cookie[]      cookies,
					     Request       request)
    {
	return new HttpRouteConnection (merger, 
					providerInfo, 
					cookies, 
					request);
    }




/**
   This class implements the RouteConnection interface with
   HTTP as the connection transport and XML as the encoding.
*/

class HttpRouteConnection implements Router.Connection {
    
    Router.Merger         merger;
    ProviderInfo          providerInfo;
    Cookie[]              cookies;
    Request               request;
    MemoryStream          out;
    long                  time;
    volatile int          state;

    
    HttpRouteConnection (Router.Merger merger, 
                         ProviderInfo  providerInfo,
			 Cookie[]      cookies,
			 Request       request)
    {
	this.merger = merger;
        this.providerInfo = providerInfo;
	this.cookies = cookies;
	this.request = request;
	this.state  = INIT;
    }

    public URL getURL () { return providerInfo.getQueryURL (); }
	
    public int getState () { return state; }
	
    public String getId () { return providerInfo.getId (); }
	
    public void getResponse (OutputStream out) throws IOException { 
		this.out.writeToStream (out); 
    }
	
    public int getResponseLength () { 
		return out.getLength (); 
    }
	
    public long getResponseTime () { 
		return time; 
    }

    public void stop () {
		state = DONE;
    }


    public void run () {
	HttpConnection.Request  req = null;
	HttpConnection.Response res = null;
	long                    start = System.currentTimeMillis ();
	String                  error = null;
	ProviderResponse        providerRes = null;

	// merger.start (this);
	    
	    
	try {
	    state = CONNECTING;
	    
	    logService.log (logService.DEBUG, 
			    merger.uuid, 
			    "Connecting to " + 
				providerInfo.getQueryURL ());
	    
	    req = httpConnectionService.beginRequest
		(providerInfo.getQueryURL (), "POST", 60 * 1000);
		
	    req.setHeader ("Content-Type", "text/xml");
	    req.setHeader ("Content-Length", 
			   String.valueOf (request.getLength ()));
	    
	    if (state == DONE) {
		error = "Timed out.";
	    } else {
		logService.log (logService.DEBUG, 
				merger.uuid, 
				"Connected to " + 
				providerInfo.getQueryURL ());
		
		/* make sure the cookies propagate to the provider nodes */
		/* not all queries have cookies (e.g. jxta queries) */
		if( cookies != null ) {
		    for (int i = 0;  i < cookies.length;  i++) {
			if (cookies[i].getName ().
			    equals (providerInfo.getId ())) {
			    String c = 
				net.jxta.search.util.TextTools.base64decode
				(cookies[i].getValue ().getBytes ());
				// System.out.println ("-->Cookie: " + c);
			    req.setHeader ("Cookie", c);
				
			}
		    }
		}
		    
		request.toStream (req.getOutputStream ());
				    
		res = req.getResponse ();
		    
		if (res.getStatusCode () == 200) {
		    InputStream in = res.getInputStream ();
			
		    state = READING;
			
		    time = System.currentTimeMillis () - start;
			
		    Enumeration cookies = res.getHeaderValues ("Set-Cookie");
			
		    providerRes = new ProviderResponse (providerInfo);
			
		    out = new MemoryStream ();
		    
		    providerRes.parse (cookies, in, out, 
				       parser, new byte[8192]);
			
			
		} else {
		    error = "Response status " + 
			res.getStatusCode () + " " + 
			res.getStatusMessage ();
		}
		    
		state = DONE;

	    }
	    
	
	} catch (Throwable e) {
	    System.out.println ("Exception while getting response from " +
				providerInfo.getQueryURL ());
	    e.printStackTrace ();
		
	    error = e.getMessage ();
		
		
	} finally {
		
	    if (res != null) {
		res.done ();
	    }
	    
	    if (error == null)
		merger.merge (providerRes);
	    else
		merger.fail (providerInfo, error);
	}
    }

}

}
	

