/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ProviderResponse.java,v 1.3 2001/06/06 01:36:15 tcamarda Exp $
 */
package net.jxta.search.router;

import java.io.*;
import java.util.*;

import net.jxta.search.peer.ProviderInfo;

import net.jxta.search.util.MemoryStream;
import net.jxta.search.util.XmlParser;



/**
 * This class is responsbible for parsing/verifying the provider's response
 * and modifying it in such a way that it can be merged into a stream of
 * responses going to the requestor.
 */

public class ProviderResponse {

    static final boolean DEBUG = false;

    ProviderInfo info;
    MemoryStream out;
    long         time;

    static Calendar cal = Calendar.getInstance (TimeZone.getTimeZone ("GMT"));
    
    ProviderResponse (ProviderInfo info) {
	this.info = info;
    }

    ProviderInfo getProviderInfo () { 
	return info;
    }

    long getTime () { 
	return time; 
    }


    int getLength () {
	return out.getLength ();
    }

    void writeToStream (OutputStream out) throws IOException {
        if (DEBUG) {
	    System.out.println ("ProviderResponse rewritten begin:");
            this.out.writeToStream (System.out);
            System.out.println ("ProviderResponse rewritten end.");
        }
	this.out.writeToStream (out);
    }
    
    /**
     * @return number of hits parsed.
     */
    void parse (Enumeration  cookies,
		InputStream  in, 
		MemoryStream out,
		XmlParser    parser,
		byte[]       buf)
	throws IOException, XmlParser.Exception
    {
	ResponseParserCallback cb = new ResponseParserCallback (out);
	
	StringBuffer s = new StringBuffer 
	    ("<response>\n" +
	     "  <provider>\n" +
	     "    <url>" + info.getQueryURL () + "</url>\n" +
	     "    <id>" + info.getId () + "</id>\n" +
	     "    <name>" + info.getTitle () + "</name>\n" +
	     "    <icon src='" + info.getImageURL () + "'\n" +
	     "          width='" + info.getImageWidth () + "'\n" +
	     "          height='" +  info.getImageHeight () + "' />" + 
	     "    <home>" + info.getLink () + "</home>\n" +
	     "    <timestamp>" + cal.getTime ().getTime () + "</timestamp>\n");
	     
	
	
	while ( cookies != null  &&  cookies.hasMoreElements () ) 

	    s.append ("<cookie>").append (cookies.nextElement ()).
		append ("</cookie>");
	
	s.append ("</provider>\n");

	out.write (s.toString ().getBytes ());
	if (DEBUG)
	    System.out.println ("parsing response from " + info);
	
	parser.parse (in, buf, cb);

	if (DEBUG)
	    System.out.println ("parsed response from " + info);
	
	out.write (" </response>\n".getBytes ());
	
	this.out = out;

	// System.out.println ("Collated provider responses start:");
        /* InputStream intest = out.getInputStream ();
	byte[] buftest = new byte[128];
	int r;
	while ((r = intest.read (buftest)) > 0)
		System.out.println (new String (buftest, 0, r));
	*/
	// out.writeToStream (System.out);
	// System.out.println ("Collated provider responses end.");
    }

    
    
    
    static class ResponseParserCallback implements XmlParser.ParserCallback {
	OutputStream out;
	int          state;
		
	static final byte[] RESPONSE_TAG       = "RESPONSE>".getBytes ();
	static final int    NO_STATE           = 0;
	static final int    RESPONSE_TAG_STATE = 1;
	static final int    DONE_STATE         = 2;
	
	ResponseParserCallback (OutputStream out) {
	    this.out     = out;
	    this.state   = NO_STATE;
	}

	
	public void startTag (byte[] chars, int start, int len) {
	    
		
		
	    switch (state) {
		
	    case NO_STATE:
		if (XmlParser.startsWith (RESPONSE_TAG, chars, start+1, len-1)) {
		    state = RESPONSE_TAG_STATE;
		    if (DEBUG)
		        System.out.println ("Provider response startTag.");
		} else {
			
				// System.out.println ("Provider sent tag: " + 
				// new String (chars, start, len));
		    
		}
		break;

	    case RESPONSE_TAG_STATE:
		if (DEBUG)
		    System.out.println ("Provider sent tag in response: " + 
					new String (chars, start, len));
		
		try {
		    out.write (chars, start, len);
		} catch (IOException e) {
				// writing to memory stream!
		}
		break;
	    }
	}


	public void endTag (byte[] chars, int start, int len) 
	    throws XmlParser.Exception
	{
	    switch (state) {

	    case RESPONSE_TAG_STATE:
		
			if (XmlParser.startsWith (RESPONSE_TAG, chars, start+2, len-2)) {
				// System.out.println ("Saw ending response tag.");
				state = DONE_STATE;
			} else {
				if (DEBUG)
				    System.out.println ("Provider sent end tag in response: " + 
				new String (chars, start, len));
				
				try {
					out.write (chars, start, len);
				} catch (IOException e) {
					// writing to memory stream!
				}
			}
			break;
	    }
	}

	
	public void chars (byte[] chars, int start, int len) 
	    throws XmlParser.Exception
	{
	    switch (state) {

	    case RESPONSE_TAG_STATE: 
			
			try {
				out.write (chars, start, len);
			} catch (IOException e) {
				// writing to memory stream!
			}
			break;
			
	    }
	}
    }


}
