/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RouterServlet.java,v 1.2 2001/06/02 01:52:15 tcamarda Exp $
 */
package net.jxta.search.router;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import org.w3c.dom.*;

import net.jxta.search.jxtastarter.JxtaStarter;
import net.jxta.search.util.LogService;
import net.jxta.search.util.ThreadService;
import net.jxta.search.util.PriorityQueue;
import net.jxta.search.util.Http;

import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import net.jxta.peergroup.PeerGroupFactory;
import net.jxta.peergroup.PeerGroup;
import net.jxta.exception.PeerGroupException;

import net.jxta.search.peer.Request;
import net.jxta.search.peer.ResolverClient;
import net.jxta.search.peer.HttpResolverClient;
import net.jxta.search.peer.ProviderInfo;
import net.jxta.search.util.XmlParser;


/** Used to test the resolver. */
public class RouterServlet extends HttpServlet {

    private static LogService    log;
    public static  Router        router;
    public static  ThreadService threads;
    
    private ResolverClient  resolverClient;
    private RouterJxtaApp   routerJxtaApp;
    private DocumentBuilder builder;

    
    static {
	threads = new ThreadService (100, 200, 100, false, false);
	try {
	    log     = new LogService ("router", LogService.ALL);
	} catch (Throwable t) {
	    t.printStackTrace ();
	}
    }
    
        
    public void init () throws ServletException {

	System.out.println ("RouterServlet: initting");
	
	ServletConfig config = getServletConfig ();


	String documentBuilderFactoryClass = getServletConfig ().
		getInitParameter ("DocumentBuilderFactory");
		
	System.setProperty ("javax.xml.parsers.DocumentBuilderFactory",
			    documentBuilderFactoryClass);
		
	DocumentBuilderFactory factory = 
	    DocumentBuilderFactory.newInstance ();
	
	
	try {
	    builder = factory.newDocumentBuilder ();
	    
	    resolverClient = 
		new HttpResolverClient (new URL 
		    (config.getInitParameter ("resolverURL")),
					log);
	} catch ( javax.xml.parsers.ParserConfigurationException e) {
	    throw new ServletException (e);
	} catch (MalformedURLException e) {
	    throw new ServletException (e);
	}
	
	
	
	
        System.out.println ("RouterServlet: booting jxta");
        PeerGroup peerGroup = JxtaStarter.getMyPeerGroup ();

	try {
            router  = new Router (new Router.Manager () {
    		
    		public void deleteRoutes (URL[]    routes,
    					  String[] ids,
    					  String[] reasons) {
    		    
    		    resolverClient.deleteRoutes (routes, ids, reasons);
    		    
    		}
		},
				  threads, 
                                  peerGroup,
				  log);
	    
    	    routerJxtaApp = new RouterJxtaApp (router, 
    					       resolverClient, 
					       log,
					       threads);
    	
            routerJxtaApp.init ( peerGroup, null /* XXX need a service adv */ );
	    routerJxtaApp.startApp (null);
		    
	} catch ( PeerGroupException e ) {
	    e.printStackTrace ();
	    throw new ServletException ("Unable to boot jxta: ", e);
	} catch (Throwable t) {
	    t.printStackTrace ();
	}

	System.out.println ("RouterServlet: initted");
    }


    private PeerGroup bootJxta () {
	try {
	    PeerGroupFactory.setPlatformClass
		(Class.forName ("net.jxta.impl.peergroup.Platform"));
	    
	    PeerGroupFactory.setStdPeerGroupClass
		(Class.forName ("net.jxta.impl.peergroup.StdPeerGroup"));
	
	} catch(Exception e) {
	    e.printStackTrace();
	}
        
	return PeerGroupFactory.newPlatform();
    }


    public void doGet (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
    {
	System.out.println ("RouterServlet.doGet: " + 
			    req.getParameter ("action"));

	if (req.getParameter ("action") == null) {
	    try {
		doRoute (req, res);
	    } catch (SAXException e) {
		e.printStackTrace ();
		throw new ServletException ("Invalid input: "+e.getMessage());
	    }
	
	} else if (req.getParameter ("action").equals ("doStats")) {
	    doStats (req, res);
	} else if (req.getParameter ("action").equals ("doRolloverLog")) {
	    log.rollover (true);
	    PrintWriter w = res.getWriter ();
	    w.println ("done.");
	}
    }

    public void doPost (HttpServletRequest req, HttpServletResponse res) 
	throws ServletException, IOException
    {
	System.out.println ("RouterServlet: doPost");
	System.out.println ("Content-Length: " + req.getContentLength ());

	try {
	    doRoute (req, res);
	} catch (SAXException e) {
	    e.printStackTrace ();
	    throw new ServletException ("Invalid input: " + e.getMessage ());
	} catch (Throwable t) {
	    t.printStackTrace ();
	}
    }

    
    void doStats (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
    {
	String  urlStr = req.getParameter ("url");
	URL     url    = null;
	
	// System.out.println ("RouterServlet.doStats: " + urlStr);
	try {
	    url = new URL (urlStr);
	} catch (Throwable t) {
	    t.printStackTrace ();
	    // System.out.println ("Sending 404");
	    res.sendError (404);
	    
	    return;
	}
	
	// System.out.println ("Getting attempt.");
	Router.RouteStats a = router.getRouteStats (url);
	
	PrintWriter    w = res.getWriter ();
	
	res.setContentType ("text/xml");
	w.println ("<?xml version='1.0'?>");
	
	w.println ("<attempt>");
	w.println ("  <url>" + url + "</url>");
	
	if (a == null) {
	    w.println ("  <queries>0</queries>");
	    w.println ("  <failed>0</failed>");
	    w.println ("  <last>-1</last>");
	    w.println ("  <average-time>0</average-time>");
	} else {
	    w.println ("  <queries>" + a.getQueryCount () + "</queries>");
	    w.println ("  <failed>" + a.getFailedAttemptCount () + "</failed>");
	    w.println ("  <last>" + a.getLastAttemptTime () + "</last>");
	    w.println ("  <average-time>" + a.getAverageResponseTime () + "</average-time>");
	}
	    
	w.println ("</attempt>");
    }


    

    
    void doRoute (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException, SAXException
    {
	Request        request       = null;
	ProviderInfo[] providerInfos = null;
        byte[]         buf           = new byte[1024];

	try {
	    
	    request = new Request (req.getInputStream (),  buf);
	    
	    providerInfos = resolverClient.resolve (request);
	    
	} catch (XmlParser.Exception e) {

	    log.log (log.ERROR, null, 
		     "Error parsing request: " + e.getMessage ());

	    res.sendError (401, "Error parsing request: " + e.getMessage ());

	    return;

	} catch (IOException e) {
	    
	    if (request != null)
		log.log (log.ERROR, request.getQueryUuid (), e.getMessage ());
	    
	    res.sendError (401, e.getMessage ());
	    
	    return;
	}

	
	res.setBufferSize (8 * 1024);
	
	res.setStatus (HttpServletResponse.SC_OK);
	res.setContentType ("text/xml");
	
	Cookie[]            cookies = req.getCookies ();
	ServletOutputStream pout    = res.getOutputStream ();
	long                start;
	long                ms;
	

	try {
	    log.log (log.DEBUG, request.getQueryUuid (), " start");
	} catch (Throwable t) {
	    t.printStackTrace ();
	}

	pout.println ("<?xml version='1.0'?>");
	pout.println ("<responses xmlns='" + Request.DEFAULT_NAMESPACE + "'\n"
		      +
		      "           query-uuid='" + 
		      request.getQueryUuid () + "'>");
	
	if (providerInfos != null && providerInfos.length > 0) {
		RouterCallback cb = new RouterCallback (req, res, pout);
		router.route (request,
			      providerInfos,
			      cookies,
			      cb);
			      // new RouterCallback (req, res, pout));
			  
	   	System.out.println ("Written " + cb.written + " bytes to router client."); 
	} else {
	    System.out.println ("Request resolved to no providers.");
	}
	
	pout.println ("</responses>");

	log.log (log.DEBUG, request.getQueryUuid (), "done");

	// System.out.println ("Router done");
    }



    


    static class RouterCallback implements Router.Callback {
	
	HttpServletRequest  req;
	HttpServletResponse res;
	ServletOutputStream pout;

        int written = 0;
	
	RouterCallback (HttpServletRequest   req,
			HttpServletResponse  res,
			ServletOutputStream  pout)
	{
	    this.req = req;
	    this.res = res;
	    this.pout = pout;
	}
	
	public void setProviderNumbers (int totalProviders, 
					int queriedProviders) 
	    throws IOException
	{
	    res.setHeader ("TotalProviders", 
			   String.valueOf (totalProviders));
	    res.setHeader ("QueriedProviders", 
			   String.valueOf (queriedProviders));
	}
	
	public void process (ProviderResponse response)
	    throws IOException
	{
	    written += response.getLength ();
	    response.writeToStream (pout);
	}
	
	public void flush () 
	    throws IOException
	{
	    System.out.println ("Router: Flushing buffer ..." +
				res.isCommitted ());
	    System.out.println ("Written " + written + " bytes so far to router client."); 
	    res.flushBuffer ();
	}
    }
    
}
    
