/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Http.java,v 1.1.1.1 2001/06/01 16:51:17 jbeatty Exp $
 */
// $Id: Http.java,v 1.1.1.1 2001/06/01 16:51:17 jbeatty Exp $

package net.jxta.search.util;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

/** Some utitility methods for HTTP. */
public final class Http {
  private Http() {}				  // can't construct

  /** Returns an input stream containing XML posted in a request. */
  public static InputStream getPostedXml(HttpServletRequest req) 
    throws ServletException, IOException {

    String contentType = req.getContentType();

    // if request is simple post of XML, just return the body
    if (("text/xml".equals(contentType) ||
	 "application/xml".equals(contentType)))
      return new BufferedInputStream(req.getInputStream());

    // if request is multipart, assume it's an uploaded file
    if (contentType != null &&
	contentType.startsWith("multipart/form-data")) {

      final String BOUNDARY = "boundary=";
      int boundaryStart = contentType.indexOf(BOUNDARY);
      if (boundaryStart == -1)
	throw new ServletException("no boundary: " + contentType);
      String boundary =
	"--" + contentType.substring(boundaryStart+BOUNDARY.length());
      BufferedReader reader =
	new BufferedReader(new InputStreamReader(req.getInputStream()));
      String line = reader.readLine();
      if (!line.equals(boundary))
	throw new ServletException("no boundary: " + line);
      for (line = reader.readLine(); line != null && !"".equals(line);
	   line = reader.readLine()) {
	line = line.toLowerCase();
	if (line.startsWith("content-type: ")) {
	  if (!(line.startsWith("content-type: text/xml") ||
		line.startsWith("content-type: application/xml")))
	    throw new ServletException("not XML: " + line);
	}
      }
      StringBuffer part = new StringBuffer();
      boundary = boundary + "--";
      for (line = reader.readLine(); line != null && !boundary.equals(line);
	   line = reader.readLine()) {
	part.append(line);
	part.append('\n');
      }
      return new ByteArrayInputStream(part.toString().getBytes());
    }

    throw new ServletException("content type not XML: " + contentType);
  }
}
