/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ISVector.java,v 1.1.1.1 2001/06/01 16:51:21 jbeatty Exp $
 */

package net.jxta.search.util;

import java.io.PrintStream;
import java.io.IOException;

public class ISVector {





    public static void setZero(int [] array) {
       
	for (int i = 0; i<array.length; i++)
	    array[i] = 0;

    }  /* close setZero() */


    public static void setZero(float [] array) {
       
	for (int i = 0; i<array.length; i++)
	    array[i] = 0;

    }  /* close setZero() */



   public static void print(double [] array, PrintStream ps, String s) {
       ps.println(s); 
       print(array,ps);
   }
 

   public static void print(double [] array, PrintStream ps) {

      for (int i = 0; i < array.length; i++) {
           ps.println (array[i]);
      }

      ps.println (" ");
   }

   public static void print(float [] array, PrintStream ps, String s) {
       ps.println(s); 
       print(array,ps);
   }

   public static void print(float [] array, PrintStream ps) {

      for (int i = 0; i < array.length; i++) {
           ps.println (array[i]);
      }

      ps.println (" ");
   }

   public static void print(int [] array, PrintStream ps, String s) {
       ps.println(s); 
       print(array,ps);
   }

   public static void print(int [] array, PrintStream ps) {

      for (int i = 0; i < array.length; i++) {
           ps.println (array[i]);
      }

      ps.println (" ");
   }

   public static void print(String [] array, PrintStream ps, String s) {
       ps.println(s); 
       print(array,ps);
   }

   public static void print(String [] array, PrintStream ps) {

      for (int i = 0; i < array.length; i++) {
           ps.println (array[i]);
      }

      ps.println (" ");
   }


    /** 
     * Compare to arrays of floats with respect to a specified
     * tolerance.  Equivalence with respect to the tolerance
     * returns true, else false.
     *
     * @param float [] a1 an array of floats
     * @param float [] a2 an array of floats
     * @param float eps a user specified tolerance value.
     *
     * @return boolean If the difference between each entry of the 
     *         array is less than the tolerance value, returns
     *         true, else returns false.
     */   
    public static boolean equals(float [] a1, float [] a2, float tolerance) throws 
                  Exception {
   
	/* Might relax this in the future to compare the shorter 
         * array length to the first n elements of the longer 
         * array. 
         */
	if (a1.length != a2.length) {
	    throw new Exception("Arrays must be same size");
        }
 
        for (int i=0; i<a1.length; i++) {
	    if ( (a1[i]-a2[i]) > tolerance) {
		return false;
            }
        }
  
        return true;

    }


    /** 
     * Compare array of ints
     *
     * @param int [] a1 an array of ints
     * @param int [] a2 an array of ints
     *
     * @return boolean true if every entry is equal, 
     *         false otherwise;
     */   
    public static boolean equals(int [] a1, int [] a2) throws Exception {
   
	/* Might relax this in the future to compare the shorter 
         * array length to the first n elements of the longer 
         * array. 
         */
	if (a1.length != a2.length) {
	    throw new Exception("Arrays must be same size");
        }
 
        for (int i=0; i<a1.length; i++) {
	    if ( (a1[i] != a2[i])) {
		return false;
            }
        }
  
        return true;

    }



    /** 
     * Compare array of ints
     *
     * @param String [] a1 an array of Strings
     * @param String [] a2 an array of Strings
     *
     * @return boolean true if every entry is equal, 
     *         false otherwise;
     */   
    public static boolean equals(String [] a1, String [] a2) throws Exception {
   
	/* Might relax this in the future to compare the shorter 
         * array length to the first n elements of the longer 
         * array. 
         */
	if (a1.length != a2.length) {
	    throw new Exception("Arrays must be same size");
        }
 
        for (int i=0; i<a1.length; i++) {
	    if (!a1[i].equals(a2[i])) {
		return false;
            }
        }
  
        return true;

    }



}
