/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: KeywordService.java,v 1.1.1.1 2001/06/01 16:51:22 jbeatty Exp $
 */

package net.jxta.search.util;

import java.util.*;
import java.io.*;

public class KeywordService
{
    Hashtable categoryToKeywords;
    File file;
    int categoryNameStart; //an index marking the start of the cat. name in the filename string



    public KeywordService(File d) {

	file = d;
	categoryToKeywords = new Hashtable();

	/* Traverse the directory tree to read all the files */

	Vector allCategoryFiles = new Vector(); // terminal nodes
	Stack directoryStack; //Stack of non terminal nodes

	if(file.isDirectory()) {
		String fName = file.getAbsolutePath();
		categoryNameStart = fName.length();
		directoryStack = new Stack();
		directoryStack.push(file);
	    } else { 
	    readFile(file);
	    return; 
         }



	while(!directoryStack.isEmpty()) {

	    File[] files = ((File) directoryStack.pop()).listFiles();

	    for(int i=0; i<files.length;i++)
		{
		    if(files[i].isDirectory())
			directoryStack.push(files[i]);
		    else allCategoryFiles.add(files[i]) ;
		}
	}
	
	/* Read all the category files */
	for (int i=0; i < allCategoryFiles.size(); i++)
	    {
		File f = (File) allCategoryFiles.get(i) ;
		readFile(f);
	    }
    }



    /* readFiles reads the category files from disk and adds them to the hashtable.
       CategoryNames are the fileNames relative to the root without the extension.
     */
    private void readFile(File f)
    {
	String categoryName;
	/* if there is only one category file */
	if (f.equals(file) ) 
	    categoryName = f.getName();
	else 
	    categoryName = (f.getAbsolutePath()).substring(categoryNameStart+1);
	/* remove file extension */
	int indx = categoryName.lastIndexOf(".");
	if(indx != -1)
	    categoryName = categoryName.substring(0,indx);
	try {
	    BufferedReader br = new BufferedReader(new FileReader(f)) ;
	    StringBuffer sb = new StringBuffer();
	    String line = null;
	/* read the file */
	    while((line = br.readLine())!= null)
		sb.append(line+" ");
	    StringTokenizer st = new StringTokenizer(sb.toString());
	    Vector keywords = new Vector();
	    while (st.hasMoreTokens())
		keywords.add(st.nextToken());
	    categoryToKeywords.put(categoryName, keywords);
	}
	catch(Exception e) {
	    System.out.println("Error reading file");
	    e.printStackTrace();
	}
    }


    public String readKeywords(File categoryfile) {


        FileInputStream fis;
        byte [] kwbuf = new byte[(int)categoryfile.length()];

        try {
            fis = new FileInputStream(categoryfile);
            fis.read(kwbuf);
        } catch (IOException ioe) {
            System.out.println("Problem reading keyword file: "
                               + categoryfile);
        }

        return  new String(kwbuf);

    }


    
    public Hashtable getCategoryKeywords()
    {
	return categoryToKeywords;
    }



    /** returns a Vector of keywords asssociated with a given categoryName */

    public Vector getKeywords(String categoryName)
    {
	return ((Vector) categoryToKeywords.get(categoryName));
    }


    
    /* Methods to test the KeywordService class */
    public static void main(String[] args)
    {
	testKeywordService(new File("../resolver/keywordtest"));
    }


    public static void testKeywordService(File f) {

	KeywordService kws = new KeywordService(f);
	Hashtable catKeywords = kws.getCategoryKeywords();
	System.out.println("Number of Categories: " + catKeywords.size());
	Enumeration en = catKeywords.keys();
	while (en.hasMoreElements()) {
	    String catName = (String) en.nextElement();
	    System.out.println("Category Name: " + catName);
	    Vector listOfKeywords = (Vector) catKeywords.get(catName);
	    System.out.println(listOfKeywords);
	}
	
    }
}
