/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: OPMPHFTest.java,v 1.1.1.1 2001/06/01 16:51:26 jbeatty Exp $
 */
package net.jxta.search.util;


class OPMPHFTest {
    
    double[] getRanking (String[] documents, String query) {
	OPMPHF hashFunc;
	
	char[][]             documentArrays = new char[documents.length][];
	char[]               queryArray     = query.toCharArray ();
	CharArrayTokenizer   queryTokenizer = 
	new CharArrayTokenizer (queryArray, " \t\n.,;");
	CharArrayTokenizer[] tokenizers = 
	new CharArrayTokenizer[documents.length];
	int[] docWordTotals = new int[documents.length];
	int[][] docWords = new int[documents.length][];
	double[] docLengths = new double[documents.length];
	int[] queryWords;
	int totalWords = 0;
	double[] rankings = new double[documents.length];

	for (int i = 0;  i < documents.length;  i++) {
	    documentArrays[i] = documents[i].toCharArray ();
	    
	    tokenizers[i] = new CharArrayTokenizer (documentArrays[i], 
						    " \t\n.,;");
	    docWordTotals[i] = tokenizers[i].countTokens ();
	    
	    totalWords += docWordTotals[i];
	}

	hashFunc = new OPMPHF (totalWords);

	do {
	    hashFunc.beginGenerate ();
	    
	    for (int i = 0;  i < documents.length;  i++) {
				
		while (tokenizers[i].nextToken ()) {
		    hashFunc.addWord (documentArrays[i],
				      tokenizers[i].getTokenStart (), 
				      tokenizers[i].getTokenLength ());
		}

		tokenizers[i].rewind ();
	    }

	    while (queryTokenizer.nextToken ()) {
		hashFunc.addWord (queryArray, 
				  queryTokenizer.getTokenStart (),
				  queryTokenizer.getTokenLength ());
	    }
	    queryTokenizer.rewind ();

	} while (! hashFunc.endGenerate ());


	queryWords = getVector (queryTokenizer, hashFunc);
	
	for (int i = 0;  i < documents.length;  i++) {
	    /* compute the document vectors */

	    docWords[i] = getVector (tokenizers[i], hashFunc);
	    
	    /* compute the lengths of the document vectors */
	    docLengths[i] = getVectorLength (docWords[i]);
	
	    rankings[i] = 
		getDotProduct (queryWords, docWords[i]) / docLengths[i];
	}


	

	return rankings;
	
    }


    int[] getVector (CharArrayTokenizer doc, OPMPHF hashFunc) {
	int[] v = new int[hashFunc.getLexiconTotal ()];
	
	while (doc.nextToken ()) {
	    int word = hashFunc.hash (doc.arr, 
				      doc.tokenStart, 
				      doc.tokenLength);
	    v[word] = v[word] + 1;
	}

	return v;
    }
    

    double getVectorLength (int[] v) {
	
	int sum = 0;
	
	for (int word = 0;  word < v.length;  word++) {
	    
	    if (v[word] > 0) {
		sum += v[word] * v[word];
	    }

	}
	
	return Math.sqrt (sum);
    }


    int getDotProduct (int[] v, int[] u) {
	int p = 0;
	
	for (int i = 0;  i < v.length;  i++) {
	    p += v[i] * u[i];
	}

	return p;
    }
}


class BitSparseVector {
    byte[] bits;
    
    BitSparseVector (int size) {
	bits = new byte[size];
    }

    void set (int i) {

	int bit = i % 8;
	int byt = (int) Math.floor ((i - bit) / 8);

	bits[byt] = (byte) (bits[byt] | (((byte)0x1) << bit));
    }
}



class IntSparseVector {
    
    int[] rowPtrs;
    int[] data;
    int curI;
    
    IntSparseVector (int size) {
	rowPtrs = new int[size];
	data    = new int[size];
    }
    
    void set (int i, int value) {
	rowPtrs[curI] = i;
	data[curI] = value;

	curI++;
    }
    
}

class CharArrayTokenizer {
    char[] arr;
    int pos;
    int tokenStart;
    int tokenLength;
    String separators;
    
    CharArrayTokenizer (char[] arr, String separators) {
	this.arr = arr;
	this.separators = separators;
    }

    int countTokens () {
	int tokens = 0;
	
	while (nextToken ()) {
	    tokens ++;
	}

	rewind ();

	return tokens;
    }
    
    boolean nextToken () {

	if (pos == arr.length)
	    return false;

	tokenStart = pos;

	// skip leading spaces
	while (pos < arr.length && isSpace (arr[pos]))
	    pos++;

	if (pos == arr.length)
	    return false;

	while (pos < arr.length && ! isSpace (arr[pos]))
	    pos++;

	tokenLength = pos - tokenStart;

	return true;
    }

    int getTokenStart () {
	return tokenStart;
    }

    int getTokenLength () {
	return tokenLength;
    }

    void rewind () {
	pos = 0;
	tokenStart = 0;
	tokenLength = 0;
    }

    private final boolean isSpace (char c) {
	return separators.indexOf (c) != -1;
    }
}
