/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: PrimeService.java,v 1.1.1.1 2001/06/01 16:51:26 jbeatty Exp $
 */
package net.jxta.search.util;

import net.jxta.search.util.*;


/**
 * Prime number generator taken out of libbow-20000915/primes.c
 * See <a href=http://www.cs.cmu.edu/~mccallum/bow>http://www.cs.cmu.edu/~mccallum/bow</a>
 */

public class PrimeService {

    int[] primes;
    int[] next_multiple;
    int primes_size;
    int primes_len;
    int next_sieve;
    int max_prime;
    static int SQRT_INT_MAX = (int) Math.floor (Math.sqrt
						(Integer.MAX_VALUE));
    

    private static void testPrimeService() {

	PrimeService service = new PrimeService ();
	
	int prime = 0;
	for (int i = 0;  i < 32;  i++) {
	    prime = service.nextPrime (prime+1);
	    System.out.println ("Prime #" + i + " is " + prime);
	}
    }  /* close testPrimeService() */


    public static void main (String[] argv) {
	testPrimeService();
    }


    public PrimeService () {
	primes_size = 128;
	primes = new int[primes_size];
	next_multiple = new int[primes_size];

	primes[0] = 2;
	primes[1] = 3;
	primes[2] = 5;
	
	primes_len = 3;
	
	next_multiple[0] = 6;
	next_multiple[1] = 9;
	next_multiple[2] = 15;

	
	next_sieve = primes[primes_len-1] + 1;
    }


    /**
     * Return the next prime greater than or equal to n.
     */

    public int nextPrime (int n) {
	
	if (n <= primes[0]) {
	    return primes[0];
	}
	
	while (n > (max_prime = primes[primes_len-1])) {

	    int start = next_sieve;
	    int end   = start + max_prime + 1;
	    char[] sieve = new char[end - start];

	    for (int i = 1;  i < primes_len;  i++) {
		int twice_prime = 2 * primes[i];
		int multiple;

		for (multiple = next_multiple[i];  
		     multiple < end;  
		     multiple+= twice_prime) {
		  
		    sieve[multiple-start] = 1;
		}
		next_multiple[i] = multiple;
		

	    }


	    for (int i = start + 1; i < end; i += 2)
		if (sieve[i-start] == 0) {
		    if (primes_len >= primes_size) {
		        primes_size *= 2;
			primes = realloc (primes, primes_size); 
			next_multiple = realloc (next_multiple, primes_size);
			
		    }
		    primes[primes_len] = i;
		    if (i >= SQRT_INT_MAX)
			next_multiple[primes_len] = Integer.MAX_VALUE;
		    else
			next_multiple[primes_len] = i * i;
		    primes_len++;
		}
	    
	    next_sieve = end;
	}



	/* Now we have at least one prime >= n.  Find the smallest such.  */
	return findSmallest (primes, 0, primes_len, n);

    }
    
    /**
     * Find smallest number m s.t. m >= n.
     */
    int findSmallest (int[] arr, int start, int len, int n) {
	int bottom = 0;
	int top = len;
	int mid;

	while (bottom < top) {
	    mid = (bottom + top) / 2;

	    if (arr[mid] < n)
		bottom = mid +1;
	    else
		top = mid;
	}

	return primes[top];
    }

    int[] realloc (int[] arr, int newSize) {
	int[] newArr = new int[newSize];

	System.arraycopy (arr, 0, newArr, 0, arr.length);

	return newArr;
    }
}
