/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RainbowClient.java,v 1.1.1.1 2001/06/01 16:51:28 jbeatty Exp $
 */
package net.jxta.search.util;

import java.net.*;
import java.io.*;

/**
   Simple client for the Rainbow category matching server.
   @author John Beatty (jbeatty@gonesilent.com)
**/
public class RainbowClient {

    /**
       Queries the Rainbow server give a hostname, port, and query.
       Returns a string of the responses in unmodified format.
       The last newline at the end of the response is truncated off,
       such that the response looks like this:

       "Arts 0.5550379481
       Regional 0.1598615846
       World 0.07620988059"

       TODO Currently the method throws IOException. This may be
       changed to suite your environment.
     **/
    public String query( String aHostname, String aPort, String aQuery )
    throws IOException {
	int port = new Integer( aPort ).intValue();

	Socket sock=null;
	String result=null;
	try {
	    sock = new Socket( aHostname, port );

	    // write the string in US-ASCII form, followed by \r\n.\r\n
	    OutputStream os = sock.getOutputStream();
	    byte[] buff = aQuery.getBytes( "US-ASCII" );
	    os.write( buff );
	    byte[] buff2 = { '\r', '\n', '.', '\r', '\n' };
	    os.write( buff2 );
	    
	    // read the response coming down, waiting for a sequence of <LF>.
	    InputStream is = sock.getInputStream();
	    StringBuffer response = new StringBuffer();
	    int currentChar;
	    int previousChar=-1;
	    while( true ) {
		currentChar = is.read();
		if( currentChar == -1 ) break;
		if( previousChar == '\n' && currentChar == '.' ) break;
		System.out.print((char) currentChar);
		response.append((char) currentChar);
		previousChar = currentChar;
	    }
	    
	    // trim the result
	    result = response.toString().trim();
	} catch( IOException e ) {
	    throw e;
	} finally {
	    if( sock != null ) sock.close();
	}

	// return the trimmed response.  The trim is necessary to
	// pull off the last newline.
	return result;
    }

    /** RainbowClient unit test.  You can either past in hostname, port, and query on the command line,
	or you can pass in nothing.  In the case that you pass in nothing, the unit test defaults
	to something I picked when writing this driver. **/
    public static void main(String[] args) {
	try {
	    String hostname,port,query;

	    if( args.length == 3 ) {
		hostname=args[0];
		port=args[1];
		query=args[2];
	    } else {
		System.out.println("usage: " + RainbowClient.class.getName() + " [hostname] [port] [query]");
		hostname="10.10.10.71";
		port="2300";
		query="computing";
	    }

	    System.out.println("host:" + hostname + " port:"+ port + " query:"+ query );

	    RainbowClient rainbow = new RainbowClient();
	    String retval = rainbow.query( hostname, port, query );
	    System.out.println( retval );
	    return;
	} catch( IOException e ) {
	    System.err.println( "Error occured: " + e );
	    e.printStackTrace();
	}
    }
}
