/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RulingMemoryStream.java,v 1.1.1.1 2001/06/01 16:51:28 jbeatty Exp $
 */
package net.jxta.search.util;

import java.io.*;


class RulingMemoryStream extends InputStream {

    byte[]  buf;
    int     start;
    int     len;
    boolean closed;

    RulingMemoryStream () {
	closed = false;
    }

    synchronized void post (byte[] buf, int start, int len) {

	while (! closed && start < len) {
	    
	    // wait until the previously-posted buffer has been read
	    try {
		wait ();
	    } catch (InterruptedException e) {
		return;
	    }
	}
	
	if (! closed) {
	    this.buf   = buf;
	    this.start = start;
	    this.len   = len;
	
	    // notify everyone who wants to read
	    notifyAll ();
	}
	
    }

    synchronized void done () {
	closed = true;
	notifyAll ();
    }

    synchronized void waitForData () throws IOException {
	
	while (!closed && start == len)
	    try {
		wait ();
	    } catch (InterruptedException e) {
		throw new IOException ("Interrupted read: " + e.getMessage ());
	    }
    }

    public synchronized int read (byte[] buf, int start, int len) 
	throws IOException
    {
	int read;
	
	waitForData ();

	if (closed) {
	    read = -1;
	} else {
	    read = (int) Math.min (this.len, len);

	    System.arraycopy (this.buf, this.start, 
			      buf,      start, 
			      read);

	    this.start += read;
	    
	    // notify the writers that it's safe to post new data
	    if (this.start == this.len)
		notifyAll ();
	    
	}

	return read;
    }


    public synchronized int read (byte[] buf) throws IOException {
	return read (buf, 0, buf.length);
    }

    
    public synchronized int read () throws IOException {

	int r;
	
	waitForData ();
	
	if (closed) {
	    r = -1;
	} else {
	    r = buf[start];
	
	    start++;
	    
	    // notify the writer that it's safe to post new data
	    if (start == len)
		notifyAll ();
	}
	
	return r; 
    }

    public synchronized void close () throws IOException {
	
	closed = true;

	// notify the writers that the reader no longer cares
	notify ();
    }
}
