/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: TextTools.java,v 1.1.1.1 2001/06/01 16:51:31 jbeatty Exp $
 */
package net.jxta.search.util;

/**
 * TextTools encodes text data as base64, just as HexTools encodes
 * hex data.
 */

public class TextTools {

    static byte[] encode = new byte[0x3f];/* mapping of 64 -> 128 */
    static int[]  decode = new int[128];  /* mapping of 128 -> 64 */

    
    static {

	/* inititalize encoding mapping */
	for (int d = 0;  d < 0x3f;  d++) {
	    if (d < 26)
		encode[d] = (byte) ('A' + d);
	    else if (d < 52)
		encode[d] = (byte) ('a' + d - 26);
	    else if (d < 62)
		encode[d] = (byte) ('0' + d - 52);
	    else if (d == 62)
		encode[d] = (byte) '+';
	    else
		encode[d] = (byte) '/';
	}
	
	/* inititalize decoding mapping */
	for (int i = 'A';  i <= 'Z';  i++)
	    decode[i] = i - 'A';
	
	for (int i = 'a';  i <= 'z';  i++)
	    decode[i] = i - 'a' + 26;
	
	for (int i = '0';  i <= '9';  i++)
	    decode[i] = i - '0' + 52;

	decode['+'] = 62;
	decode['/'] = 63;



    }


    /**
     * Encode text in the given buffer.
     */
    public static String base64encode (byte[] buf) {
	int    len;
	byte[] out;
	int    triplet; /* set of three bytes under examination */
	int    i; /* pointer into buf */
	int    j; /* pointer into out */

	
	if (buf.length % 3 == 0)
	    len = buf.length / 3 * 4;
	else
	    len = (buf.length + 3 -(buf.length % 3) ) / 3 * 4;
	out = new byte[len];
	
		
	/* for each three bytes we read, write out four. */
	for (i = 0, j  = 0;  i < buf.length - buf.length % 3;  i+=3, j+=4) 

	    base64encodeTriplet (buf[i], buf[i+1], buf[i+2], out, j);
	    
	
	
	/* are there two characters left or one? */
	if (buf.length % 3 == 2) 

	    base64encodeTriplet (buf[i], buf[i+1], (byte)'=', out, j);
	    
	else if (buf.length % 3 == 1) 

	    base64encodeTriplet (buf[i], (byte) '=', (byte) '=', out, j);



	return new String (out);
    }

    /**
     * Encode the given three characters as four chars in <em>to</em>
     */

    private static final void base64encodeTriplet (byte   c1,
						   byte   c2,
						   byte   c3,
						   byte[] to, 
						   int    i) {
	int triplet = (c1 << 16) | (c2 << 8) | (c3 << 0);

	to[i]   = encode[(triplet >> 18) & 0x003f];
	to[i+1] = encode[(triplet >> 12) & 0x003f];
	to[i+2] = encode[(triplet >> 6)  & 0x003f];
	to[i+3] = encode[(triplet >> 0)  & 0x003f];
    }

    /**
     * Read four characters from <em>from</em>, write three to <em>to</em>
     */
    
    static final void base64decodeToTriplet (byte[] from, int i,
					     byte[] to,   int j)
    {
	
	int triplet = ( (decode[from[i]]   << 18) |
			(decode[from[i+1]] << 12) |
			(decode[from[i+2]] << 6)  |
			(decode[from[i+3]] << 0) );
	
	to[j]   = (byte) ((triplet >> 16) & 0xff);
	to[j+1] = (byte) ((triplet >> 8) & 0xff);
	to[j+2] = (byte) ((triplet >> 0) & 0xff);

    }
    

    /**
     * Decode base64-encoded text in the given buffer.
     */

    public static String base64decode (byte[] buf) {
	int    tripletCount;
	int    len;
	byte[] out;
	int    i;
	int    j;
	int    triplet;
	
	if (buf.length % 4 != 0)
	    throw new IllegalArgumentException ("Buffer length must be a " +
						"multiple of 4!");
	len = buf.length / 4 * 3;
	out = new byte[len];

	for (i = 0, j = 0;  j < len;  i+=4, j+= 3) {
	    
	    base64decodeToTriplet (buf, i, out, j);

	}

	if (out[len-2] == '=')
	    return new String (out, 0, len-2);
	else if (out[len-1] == '=')
	    return new String (out, 0, len-1);
	else 
	    return new String (out);
    }

    ////////////////////////// testing code

    static void testMappings () {
	byte b;
	
	for (int i = 0; i < 0x3f;  i++) {
	    b = encode[i];

	    if (i != decode[(int) b])
		System.out.println ("Mapping for " + i + " is incorrect");
	    
	}

	System.out.println ("Mappings passed.");
    }


    static void testTripletEncoding () {
	String[] tests = { "yar", "dav", "tom", "stv" };
	
	for (int i = 0;  i < tests.length;  i++) {
	    byte[] from  = tests[i].getBytes ();
	    byte[] to    = new byte[4];
	    byte[] from2 = new byte[3];
	    
	    base64encodeTriplet (from[0], from[1], from[2], to, 0);

	    base64decodeToTriplet (to, 0, from2, 0);

	    for (int j = 0;  j < 3;  j++)
		if (from[j] != from2[j]) {
		    System.out.println ("Triplet encoding fails: " +
					tests[i] + " encoded to " +
					new String (to) + 
					", which decoded to " + 
					new String (from2));
		    return;
		}

	    System.out.println ("Triplet " + tests[i] + 
				" encoded to " + new String (to) +
				" and decoded to " + new String (from2));
	}

	System.out.println ("Triplet encoding test passed.");
    }

    static void testStringEncoding (String original) {
	String encoded;
	String decoded;

	encoded  = base64encode (original.getBytes ());
	decoded  = base64decode (encoded.getBytes ());
	
	
	System.out.println ("Original: " + original +
			    ", length = " + original.length ());

	System.out.println ("Encoded: " + encoded + 
			    ", length = " + encoded.length ());
		
	System.out.println ("Decoded: " + decoded + 
			    ", length = " + decoded.length ());

	System.out.println ("BASE64 encoding test " +
			    (decoded.equals (original) ? "passed" : "failed") +
			    ".");
	    
    }

    public static void main (String[] argv) {

	testMappings ();

	testTripletEncoding ();

	if (argv.length == 0)
	    System.out.println ("Supply a command-line argument " +
				"to test string encoding.");
	else
	    for (int i = 0;  i < argv.length;  i++)
		testStringEncoding (argv[i]);
    }
}
