/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Dnrm2.java,v 1.1.1.1 2001/06/01 16:51:36 jbeatty Exp $
 */

/** Dnrm2.java slightly modified to work without the Dummy 
 *  class used by BLAS and LAPACK.  Turns out its not needed 
 *  anyway, so the offending lines are simply commented out.
 *  BLAS and LAPACK are distributed under a BSD-type license.
 *
 *  $Author: jbeatty $
 *
 *  $Log: Dnrm2.java,v $
 *  Revision 1.1.1.1  2001/06/01 16:51:36  jbeatty
 *  Initial commit of sourcebase
 *
 *  Revision 1.2  2001/06/01 11:58:35  yaroslav
 *   - new license stuff
 *
 *  Revision 1.1  2001/05/17 15:35:31  jbeatty
 *  Initial commit of util tree
 *
 *  Revision 1.1  2001/04/10 19:32:22  tcamarda
 *  - Second attempt at commit
 *
 *  Revision 1.1  2001/02/22 13:53:13  dave
 *  Dnrm2 implements a stable 2-norm, taken from f2j blas level 1 at netlib.
 *
 *
 */
//package org.netlib.blas;
package net.jxta.search.util.text;
import java.lang.*;
//import org.netlib.util.*;



public class Dnrm2
{

// *     .. Scalar Arguments ..
// *     .. Array Arguments ..
// *     ..
// *
// *  DNRM2 returns the euclidean norm of a vector via the function
// *  name, so that
// *
// *     DNRM2 := sqrt( x'*x )
// *
// *
// *
// *  -- This version written on 25-October-1982.
// *     Modified on 14-October-1993 to inline the call to DLASSQ.
// *     Sven Hammarling, Nag Ltd.
// *
// *
// *     .. Parameters ..
   static double one = 1.0e+0;
   static double zero = 0.0e+0;
// *     .. Local Scalars ..
   static int ix = 0;
   static double absxi = 0.0;
   static double norm = 0.0;
   static double scale = 0.0;
   static double ssq = 0.0;
// *     .. Intrinsic Functions ..
// *     ..
// *     .. Executable Statements ..
   static double dnrm2 = 0.0;

    /** Euclidean (2) norm of a vector using a stable algorithm.
     *
     *  @param n length of array x
     *  @param x array of values
     *  @param _x_offset Where in the array to start, since we can't
     *         pass in a pointer to an interior location.
     *  @param incx stride, how many values to accumulate in the inner loop.
     *         Used to increase caching performance, see dlamch for details.
     */
   public static double dnrm2 (int n, double[]x, int _x_offset, int incx)
   {

      if (n < 1 || incx < 1)
	{
	   norm = zero;
	}			// Close if()
      else if (n == 1)
	{
	   norm = Math.abs (x[(1) - 1 + _x_offset]);
	}			// Close else if()
      else
	{
	   scale = zero;
	   ssq = one;
// *        The following loop is equivalent to this call to the LAPACK
// *        auxiliary routine:
// *        CALL DLASSQ( N, X, INCX, SCALE, SSQ )
// *
	   {
	      int _ix_inc = incx;
//	      forloop10:
	      for (ix = 1;
		   (_ix_inc < 0) ? ix >= 1 + (n - 1) * incx : ix <=
		   1 + (n - 1) * incx; ix += _ix_inc)
		{
		   if (x[(ix) - 1 + _x_offset] != zero)
		     {
			absxi = Math.abs (x[(ix) - 1 + _x_offset]);
			if (scale < absxi)
			  {
			     ssq = one + ssq * Math.pow ((scale / absxi), 2);
			     scale = absxi;
			  }	// Close if()
			else
			  {
			     ssq = ssq + Math.pow ((absxi / scale), 2);
			  }	//  Close else.
		     }		// Close if()
		//   Dummy.label ("Dnrm2", 10);
		}		//  Close for() loop. 
	   }
	   norm = scale * Math.sqrt (ssq);
	}			//  Close else.
// *
      dnrm2 = norm;
//      Dummy.go_to ("Dnrm2", 999999);
// *
// *     End of DNRM2.
// *
//      Dummy.label ("Dnrm2", 999999);
      return dnrm2;
   }


    /** Since we aren't using the entire blas/lapack API, it is 
     *  convenient to run a simple test
     */
    public static void main(String [] args) {

	double [] testarray = {12.0, 17.0};
	int offset = 0;
	int stride = 1; // Stride 0 fails!
	double norm2 = dnrm2(testarray.length, testarray, offset, stride);
	System.out.println("2 norm: " + norm2);
    }


}				// End class.
