/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: NaiveBayes.java,v 1.1.1.1 2001/06/01 16:51:36 jbeatty Exp $
 */
/** Naive Bayes classifiers use the joint probabililities of words 
 *  and categories to estimate the probabilities of categories
 *  given a document.  "Naive" refers to independence between 
 *  words, an assumption rendering the computations tractable.
 *
 */

package net.jxta.search.util.text;

class NaiveBayes {


  SparseMatrix classes;
  int[]        wordCounts;


  
  NaiveBayes (SparseMatrix classes) {
    this.classes = classes;

    for (int clas = 0;  clas < classes.totalRows;  clas++) 
      wordCounts[clas] = sum (classes.data, 
			      classes.rowStarts[clas],
			      classes.rowLengths[clas]);
  }


  int sum (int[] arr, int start, int len) {
    int sum = 0;

    for (int i = start;  i < len;  i++)
      sum += arr[i];

    return sum;
  }



 /**  Equation 3.2, Verbeek 2000?
  */
  SparseVector computeConditionalProbabilities (SparseVector doc) {

      int          classCount     = classes.totalRows;

      SparseVector classification = new SparseVector (classCount);

      
      for (int clas = 0;  clas < classCount;  clas++) {
	
	int classStart = classes.rowStarts[clas];
	int classLen   = classes.rowLengths[clas];
	int docIndex   = 0;
	int word       = doc.rowIndicies[0];

	int c = classStart;
	int p_d_bar_c = 1;
	
	
	for (docIndex = 0;  docIndex < doc.len;  docIndex++) {
	  
	  word = doc.rowIndicies[docIndex];
	  
	  for (; c < classLen; c++)
	    if (classes.colIndicies[c] == word) {

	      p_d_bar_c *= classes.data[c] / wordCounts[docIndex];
	      break;
	    }

	  if (c == classLen)
	    break;
	}

	if (p_d_bar_c > 1)
	  classification.append (clas, p_d_bar_c);
	
      }

      return classification;
    } 

}
