/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: SparseVector.java,v 1.1.1.1 2001/06/01 16:51:37 jbeatty Exp $
 */

package net.jxta.search.util.text;
class SparseVector {
    int[] rowIndicies;
    int[] data;
    int   len;
    
    SparseVector (int size) {
	rowIndicies = new int[size];
	data        = new int[size];
    }

    void append (int i, int val) {
	
	if (len == data.length) {
	    realloc ();
	}

	rowIndicies[len] = i;
	data[len] = val;

	len++;
    }

    
    void sortByIndex () {
	sort (data, rowIndicies, len, 0, len-1);
    }

    static void sort (int[] a, int[] b, int length, int l, int r) {
	
	int v, t;
	int i, j;

	if (r > l) {
	    v = a[r];  i = l-1; j = r;
	    
	    for (;;) {

		while ((++i < length) && a[i] < v)
		    ;

		while ((--j > 0) && a[j] > v)
		    ;

		if (i >= j)
		    break;

		t = a[i]; a[i] = a[j]; a[j] = t;
		t = b[i]; b[i] = b[j]; b[j] = t;
	    }

	    t = a[i]; a[i] = a[r]; a[r] = t;
	    t = b[i]; b[i] = b[r]; b[r] = t;

	    sort (a, b, length, l, i-1);
	    sort (a, b, length, i+1, r);

	}
    }


    void realloc () {
	rowIndicies = realloc (rowIndicies, rowIndicies.length * 2);
	data        = realloc (data, data.length * 2);
    }

    int[] realloc (int[] a, int newLen) {
	int[] newA = new int[newLen];

	System.arraycopy (a, 0, newA, 0, a.length);

	return newA;
    }
}
