/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: EphemeralCluster.java,v 1.1.1.1 2001/06/01 16:50:47 jbeatty Exp $
 */

package net.jxta.search.relevance;

import java.util.ArrayList;
import java.util.TreeSet;
import java.util.Comparator;
import java.io.PrintStream;
import net.jxta.search.util.Html;
import net.jxta.search.util.ISMatrix;




/**
 * Ephemeral clustering takes a set of html documents 
 * and constructs clusters based on the contents of 
 * the documents.
 */
public class EphemeralCluster {

    private static boolean debug = false;

    public static void setDebug() {
        debug = true;
    }

    public static void clearDebug() {
	debug = false;
    }

   public static void findClusters(ArrayList docs) {

   }


   public static boolean test(PrintStream ps) {

      TestEphemeralCluster tec = new TestEphemeralCluster();
      return tec.test(ps);
   }

   
   public static void main(String [] args) {

      TestEphemeralCluster tec = new TestEphemeralCluster();
      tec.test(System.out);
   }


} 
 

 
/** 
 *  Since there are only 10 buckets total,
 * they can all be handled in the same class.
 */

class BucketArray {

   class Bucket extends ArrayList {
   }

  Bucket [] ba;

  public BucketArray () {

     ba = new Bucket[10];
     for (int i = 0; i<10; i++) {
        ba[i] = new Bucket();
     }
   }

   public void add(Cluster c) {
  
     /* Indices should range from 0..9 */
      int index = (int)Math.floor((10*c.getSimilarityValue())/10);

      ba[index].add(c);
   }



  /** 
   * Part 2 of the algorithm requires finding the bucket
   * (doubly linked list) corresponding to a certain 
   * similarity value, namely the bucket with the 
   * highest similarity value.  The method is so named 
   * after the description in the text of the algorithm.
   *
   * @return int The index of the "highest" bucket, that is,
   *         the list of clusters with the highest similarity 
   *         values.
   */
   public int getBiggestindex() {

      for (int i = 9; i>=0; i--) {

        if (this.hasCluster(i)) {
           return i;
        }
      }
    
      return 0;
 
   }  /* close getBiggestIndex() */





  /**
   * To find the bucket (e.g., linked list) containing
   * the clusters with the highest similarity value, we 
   * need to determine whether some particular bucket 
   * (e.g., linked list) has any elements.  An empty list
   * means there are no clusters with a similarity value
   * corresponding to this index.
   */
   private boolean hasCluster(int i) {
  
      Bucket b = ba[i];
      if (b.isEmpty()) {
         return false;
      } 
 
      return true;

   }




  /** Just to make sure that the bucket lists
   * and buckets are playing nicely together 
   * as they should, make sure that empty buckets
   * are really empty, and buckets with stuff in 
   * them can be seen.
   */
   public void test_hasCluster() {

   }

  /** 
   * getBiggestIndex() returns 0 if the bucket 
   * list is completely empty.  Test for empty and
   * none empty conditions.
   */
   public void test_getBiggestIndex() {

   }


}






/** Cluster takes a set of document links.  We only 
 * need to store the documents once, the clusters
 * just store pointers to the documents contained
 * contained in the cluster.
 */
class Cluster implements Comparator {


    /* Leading index points to ith Bucket, 
     * trailing index to jth cluster in ith 
     * Bucket.
     */
    Cluster [][] cur_pointer_mat;

    private static boolean debug = false;

    public static void setDebug() {
        debug = true;
    }
 
    public static void clearDebug() {
        debug = false;
    }


   /* This is probably turned around backwards.  The Clusters 
    * should probably be a part of the BucketList class, and not 
    * have the buckets here.
    */
    private static BucketArray ba;
    {ba = new BucketArray();}

    ArrayList children;

   /** 
    * A "marked" cluster means that is has been 
    * merged with the most similar cluster to itself,
    * and is now a child of a new unmarked parent cluster
    * consisting of itself and the sibling which is the 
    * cluster most similar.
    */
    public boolean marked = false;

    private String doc = null;
    private String doc1 = null;
    private String doc2 = null;

    private double similarityvalue = 0;
   


  /** Comparator interface methods. **/
    public int compare (Object o1, Object o2) {

        Cluster c1 = (Cluster) o1;
        Cluster c2 = (Cluster) o2;

        if (c1.similarityvalue < c2.similarityvalue) {
            return -1;
        } else if (c1.similarityvalue > c2.similarityvalue) {
            return 1;
        } else {
            return 0;
        }
    }

    public boolean equals(Cluster c) {
   
       if (this.similarityvalue == c.similarityvalue) {
           return true; 
       } 
  
       return false;

    }
 


   /**
    * FIXME: Abstract out common elements of the constructors.
    */
    public Cluster (ArrayList docs) {

       similarityvalue = computeSimilarity(docs);
       children = new ArrayList();
   
    }


    public Cluster(Cluster c1, Cluster c2) {

       children = new ArrayList();
       children.add(c1);
       children.add(c2);
  
    }


   /**
    * "Singleton" clusters consist of only a document, and 
    * have "no children".
    */
    public Cluster(String doc) {
    
       this.doc = doc;

       children = new ArrayList();
    }


    public void addChildren(Cluster c) {
       this.children.add(c);
       this.children.addAll(c.children);
    }

    
   /** All of the similarity values of all of the clusters 
    * are equal, so they all go into one big cluster.
    * The assumption in this method is that the 
    * the 2 clusters c1 and c2 that are passed in 
    * are compared to the similarity value of the 
    * this cluster.  Corresponds to Case 1 of part 
    * 2 of algorithm listed on p. 15.
    *
    * @param Cluster c1, c2: Clusters that have their
    *        similarity value compared to the similarity 
    *        value of the current cluster.
    */
    public void merge(Cluster c1, Cluster c2) {

    }

    public double getSimilarityValue() {
       
       return similarityvalue;

    }


    public void setSimilarity(double s) {
    
        this.similarityvalue = s;

    }


    public int getBucketIndex() {

        return (int)Math.floor( (this.similarityvalue*10.0) );

    }

   /* Assuming that we can store the actual documents just 
    * once, computing similarities may depend on whether the
    * cluster consists of a single document, or multiple documents,
    * i.e., other clusters.
    */ 
    private double computeSimilarity(Cluster cluster) {

       return 0;  // dummy

    }


   /* size 3 for testing mode. */
    //PointerMatrix pm = new PointerMatrix(3);
  
   /** 
    * The similarity value <i>s</i> of a cluster <i>c</i>
    * to be the minimum of the pairwise similarities of 
    * the elements of the cluster <i>c</i>.
    *
    * NOTE: This method is special purpose to deal with the 
    * initial clusters consisting of "singleton sets".  That
    * is, each cluster has one and only one document.
    */
    private double computeSimilarity(ArrayList docs) {

       double similarity = 1.0;
       double temp = 0;

       int size = docs.size();
 
      double [][] similarities = new double[size][size];

       String doc1 = null;
       String doc2 = null;

       Cluster cluster;
       

      /** This loop just tests the strictly lower triangular part.
       * When the clusters consist of single documents, the diagonals
       * equal 1.  This will have to be changed to test the diagonals.
       */
       for (int j = 0; j < size-1; j++) {
          for (int i = j+1; i<size; i++) {
            
	      /*
	      if (debug) { 
                  System.out.println("Computing similarity ("+i+","+j+")");
	      }
	      */

             doc1 = (String)docs.get(i);
             doc2 = (String)docs.get(j);     
        
             similarities[i][j] = computeDocumentSimilarity(doc1, doc2);

             if (temp < similarity) {
                similarity = temp; 
             }
  
           /* Initially, each two singletons becomes a cluster. */
            //cluster = new Cluster(doc1,doc2);
            //cluster.setSimilarity(similarity);
 

           /* now we need to stuff this cluster into the bucket list. */ 
            //ba.add(cluster);
 
           /* The bucket can be extracted from the cluster similarity. */
            //pm.pointers[i][j] = cluster;
                       
          }
       }

       if (debug) {
           ISMatrix.print(similarities,System.out); 
       }

       return similarity;
    
    }  /* close computeSimilarity(ArrayLIst) */



 
    public static double computeSimilarity(Cluster c1, Cluster c2) {

       if (c1.similarityvalue <= c2.similarityvalue) {
          return c1.similarityvalue;
       } else {
          return c2.similarityvalue;
       } 
    } 


    private float computeDocumentSimilarity(String doc1, String doc2) {


       /* All dummy values for now */
        //double sim = Math.random();
        //sim = Math.floor(10*sim)/10.0;
 
        ArrayList al = new ArrayList(1);
        al.add(doc2);
        float [] sim = TFIDFRelevance.findRel(doc1,al);
        System.out.println("Sim: " + sim[0]);
        return (float) (Math.floor(10.0*sim[0]));
    }


    /**
     * Each cluster has either subclusters or documents as
     * children.  To get all the document children from a 
     * cluster, recursively grab all the documents and add
     * them to an arraylist.
     *
     * @param Cluster that has either child subclusters, or 
     *        a document at the node.
     * 
     * @return ArrayList list of documents contained as children
     *        of the cluster.
     */
    public ArrayList getDocuments(Cluster c) {

	ArrayList docs = new ArrayList();
       docs.add(c.doc);

       int size = c.children.size(); 

       for (int i=0; i<size; i++)  {
	
         c =  (Cluster)this.children.get(i);
	 /* Recursive call */
         docs.addAll(c.getDocuments());
       }


       return docs;
       
    }

    public ArrayList getDocuments() {
	return getDocuments(this);
    }

   /**
    * Prints the documents for all of the children.
    */
    public String toString() {
   
       Cluster c;
       int size = this.children.size(); 
 
       StringBuffer sb = new StringBuffer();

       for (int i=0; i<size; i++)  {
         c =  (Cluster)this.children.get(i);
	 /* Recursive call */
         sb.append(c.toString());
       }

       sb.append(this.doc);
       return sb.toString();
    } 

}  /* class Cluster */



