/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: TestEphemeralCluster.java,v 1.1.1.1 2001/06/01 16:50:49 jbeatty Exp $
 */

package net.jxta.search.relevance;

import java.util.ArrayList;
import java.io.PrintStream;
import net.jxta.search.util.Html;
import net.jxta.search.util.ISMatrix;
import net.jxta.search.util.ISVector;

/** 
 * Separate test class for hard-wiring "proof" data.
 */
class TestEphemeralCluster {


   public TestEphemeralCluster() {
 
   }

 

   public boolean  test(PrintStream ps) {

      ps.println("---------  Begin Unit Testing Ephemeral Cluster: --------");
 
      
      String testdoc1 = Html.stripTags(doc1);
      //ps.println(testdoc1);
      String testdoc2 = Html.stripTags(doc2);
      //ps.println(testdoc2);
      String testdoc3 = Html.stripTags(doc3);
      //ps.println(testdoc3);


      ArrayList doclist = new ArrayList(100);

      doclist.add(testdoc1);
      doclist.add(testdoc2);
      doclist.add(testdoc3);
      doclist.add(doc4);
      doclist.add(doc5);
      doclist.add(doc6);

     /* Just for fun, compute the similarity between a 
      * document and itself:
      */
      float [] sims;
      //TFIDFRelevance.setDebug();
      //EphemeralCluster.setDebug();
      Cluster.setDebug();
     /* Hash term counting and naive term counting work.  The 
      * Boyer-Moore term counting appears to hang everything.
      */
      TFIDFRelevance.setTermCountNaive();

      sims = TFIDFRelevance.findRel("glitter gold Starling",doclist);
      ISVector.print(sims,ps);


      Cluster cluster = new Cluster(doclist);

 
     /** 
      * Initial clusters are "singletons", that is,
      * they consist of single documents each.
      */
      Cluster c1 = new Cluster(testdoc1);
      Cluster c2 = new Cluster(testdoc2);
      Cluster c3 = new Cluster(testdoc3);

      Cluster c4 = new Cluster(testdoc1);
      Cluster c5 = new Cluster(testdoc2);


      c4.addChildren(c3);
      c5.addChildren(c4);
 

      ps.println("Printing children of c4 (doc1)");
      ps.println(c4);
      ps.println("Printing children of c5 (doc1 and doc2)");
      ps.println(c5);
   


      ps.println("---------  Test printing child documents -----");
      ArrayList a  = c5.getDocuments();


      ps.println("Size of child doc list: " + a.size());

      for (int i = 0; i<a.size(); i++) {

	  ps.println("Doc " + i + " from c5: " + (String)a.get(i));

      }
      ps.println("---------  end test printing child documents -------");


      ps.println("---------  End Unit Testing Ephemeral Cluster: --------\n");
     /* Haven't really tested anything yet. */
      return false;  

   }


   /** 
    * Some hardwired test documents to make testing easier.
    * There will need to be a corpus of standard documents available
    * for actual clustering testing.  The following are for demonstration
    * and for testing that algorithm works without blowing up.
    */

   /** FWIW: can you find the mistake in the text of doc1? :) */
    String doc1 = "<html>\n<head>\n<title>Test document 1</title>\n</head>\n"
                + "<body>\n <h1>Doc 1</h1>\n"
                + "And as we wind on down the road\n"
                + "Our shadows taller than our souls\n"
                + "There walks a lady we all know\n"
                + "Who shines white light and wants to show\n"
                + "How everything still turns to gold\n"  
                + "</body>\n</html>";

    String doc2 = "<html>\n<head>\n<title>Test document 2</title>\n</head>\n"
                + "<body>\n <h1>Doc 2</h1>\n"
                + "Thousands of gold-seekers steamed up the Inside Passage\n"
                + "to Skagway, then hauled their 1,200 pounds of supplies\n"
                + "over the arduous Chilkoot or White Pass trails, then built\n"
                + "boats, floated the Yukon River to Dawson, where they\n"
                + "thought they'd find all that glitters - gold for the picking.\n" 
                + "</body>\n</html>";



    String doc3 = "<html>\n<head>\n<title>Test document 3</title>\n</head>\n"
                + "<body>\n <h1>Doc 3</h1>\n"
                + "<p>Fukushima has many mountainous roads that invite us into nature.\n"
                + "Together with the flowers and trees spread out before us, beautifully\n"
                + "colored in gold and red, let us forget the passage of time.\n</p>"
                + "<p>If you feel a sense of regret as the scenery whizzes past\n"
                + "your car window, please get out and walk.  You can often find a path around you. \n"
                + "If you walk slowly, enjoying the songs of the wind and \n"
                + "the birds, time will pass slowly, as well.\n </p>"
                + "</p>You may find an open, natural scene on a new road along the sea.\n"
                + "Faint sunlight glitters gently on the surface of the Pacific,\n"
                + "echoing the beauty of a mirror-like lake.\n"
                + "This is a charm of the seaside, which cannot be enjoyed in summer.\n</p>"
                + "</body>\n</html>";


    String doc4 = "The telecommunications "
	        + "thrust is classic Intel: "
                + "Attack an established "
                + "competitor in a new market "
                + "with large numbers of "
                + "inexpensive computers "
                + "offering \"twice the "
                + "performance for half the "
                + "price,\" said Abhi "
                + "Talawalker, vice president "
                + "and general manager of "
                + "Intel's Enterprise Platforms "
                + "and Services Division.";
 
    String doc5 = "Face it, the Internet we know today "
                + "is slow and clunky. It's getting "
                + "better, but it's still not the "
                + "lightning-fast medium everyone wants, "
                + "and the functionality is limited by "
                + "the network pipelines. But all that "
                + "could change with Internet2 -- yes, "
                + "that's right, the Internet has a "
                + "sequel.";


    String doc6 = "Danish ornithologists say that birds, especially Starlings, "
                + "have begun incorporating " 
                + "the sound of a ringing cellular phone into their own songs. So far, reports of wireless "
                + "warbling have been restricted to Copenhagen, where birds seem to favor Nokia's "
                + "classic ring tone. "
                + "Birds imitating sounds produced by technology is nothing new. They choose simple "
                + "tunes to reproduce. The standard ring tone on a phone usually comprises any "
                + "combination of nine tones. And the tunes themselves don't typically contain "
                + "harmonies, which are made by playing multiple musical tones at the same time. "
                + "Usually, birds copy what they hear the most. Birds in rural areas have added the "
                + "sound of horses whinnying, lawn mowers and even chainsaws to their repertoires. In "
                + "cities, birds have added car alarms, the warning beep of a truck backing up and police "
                + "sirens to their calls, experts say.";

    /* TODO: Add a few more dummy documents. */

}


