/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaRegRetrievalService.java,v 1.1.1.1 2001/06/01 16:50:50 jbeatty Exp $
 */
package net.jxta.search.webprovider;

import net.jxta.search.jxtastarter.JxtaStarter;

import java.net.URL;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;
import java.lang.reflect.InvocationTargetException;

import net.jxta.id.IDFactory;
import net.jxta.peergroup.PeerGroup;
import net.jxta.exception.PeerGroupException;
import net.jxta.exception.ServiceNotFoundException;
import net.jxta.exception.JxtaException;
import net.jxta.pipe.Pipe;
import net.jxta.pipe.PipeID;
import net.jxta.pipe.InputPipe;
import net.jxta.pipe.OutputPipe;
import net.jxta.document.AdvertisementFactory;
import net.jxta.protocol.PipeAdvertisement;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredDocumentFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.Element;
import net.jxta.endpoint.Message;

import net.jxta.search.util.MemoryStream;

/** Retrieves registrations from JXTA peers **/
class JxtaRegRetrievalService implements RegRetrievalService {
    
    static final int OUTPIPE_RESOLVE_TIMEOUT = 10000;
    static final int RESPONSE_TIMEOUT = 15000;

    int cl;
    InputStream in;
    PeerGroup peerGroup;
    
    public int getContentLength() {
        return cl;
    }

    public InputStream getInputStream() {
        return in;
    }

    public JxtaRegRetrievalService() {
        peerGroup = JxtaStarter.getMyPeerGroup ();
    }

    public void retrieveRegistration( URL url ) throws IOException {
        try {
            PipeAdvertisement inPipeAdv;
            PipeAdvertisement outPipeAdv;
            InputPipe inPipe;
            OutputPipe outPipe;
            Message msg;

            Pipe pipeService = peerGroup.getPipe();
            
            outPipeAdv = (PipeAdvertisement)
                AdvertisementFactory.newAdvertisement
                (PipeAdvertisement.getAdvertisementType ());

            outPipeAdv.setPipeID ((PipeID) IDFactory.fromURL (url));
            outPipeAdv.setName (url.getPath ());

            inPipeAdv = (PipeAdvertisement)
                AdvertisementFactory.newAdvertisement
                (PipeAdvertisement.getAdvertisementType ());

            inPipeAdv.setPipeID (new PipeID (peerGroup.getID ()));
            inPipeAdv.setName ("Return pipe for input pipe " + url.getHost ());

            outPipe = pipeService.createOutputPipe (outPipeAdv, 
                                                    Pipe.NonBlocking, 
                                                    OUTPIPE_RESOLVE_TIMEOUT );

            inPipe  = pipeService.createInputPipe (inPipeAdv);

            msg = pipeService.createMessage ();

            msg.push ("request", 
		      new ByteArrayInputStream
			  ("<registration-request></registration-request>"
			   .getBytes() ) );

	    msg.push ("responsePipe", inPipeAdv.getDocument (new MimeMediaType
		("text/xml")).getStream ());

	    // send message
            outPipe.send (msg);

            Message response = inPipe.poll( RESPONSE_TIMEOUT );

            // if there is a non-null registration in the response
            // transfer it to a memory buffer and count the bytes
            // all this hoopla is necessary to do an efficient length
            // counter
            InputStream tmp = response.pop( "registration-response" );         
            if( tmp == null ) {
                cl=0;
                in=null;
                return;
            } else {
                MemoryStream mem = new MemoryStream();            
                transfer( tmp, mem );
                cl=mem.getLength();
                in=mem.getInputStream();
            }
        } catch( InvocationTargetException e ) {
            throw new IOException( "Could not create jxta advertisements." );
        } catch( Exception e ) {
	    e.printStackTrace();
            throw new IOException( "Could not get xml document from adv." );
        }
    }
    
    /** moves all bytes from is to os. this is a cheesy impl. **/
    private void transfer( InputStream is, OutputStream out ) {
        final int BUFFLEN = 1024;
        byte[] buf = new byte[BUFFLEN];
        int c;
        try {
            while ( (c=is.read( buf, 0, BUFFLEN ))!= -1 ) {
                out.write( buf, 0, c );
            }
        } catch ( IOException e ) {
            System.err.println( "Problem displaying search payload:" + e );
            e.printStackTrace();
        }
    }

    /** Tests registration retrieval. Should succeed given any pipe url
	for an up-and-running jxta information provider.  Should fail
	when info provider is not running or the pipe url is invalid. **/
    private static void test( URL url ) throws Exception {
	JxtaRegRetrievalService svc = new JxtaRegRetrievalService();
	svc.retrieveRegistration( url );
	InputStream is = svc.getInputStream();

	if( is!=null ) {
	    int c;
	    while ( (c=is.read())!= -1 )
		System.out.write( c );

	    System.out.println( "Success" );
	} else {
	    System.out.println( "Failure" );
	}
    }

    private static void main( String[] args ) throws Exception {
	if( args.length != 1 ) {
	    System.out.println( "usage: JxtaRegRetrievalService <url>" );
	}
	test( new URL( args[0] ) );
    }
}
