/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: MailService.java,v 1.1.1.1 2001/06/01 16:50:50 jbeatty Exp $
 */
package net.jxta.search.webprovider;

import java.util.Properties;

import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.AddressException;
import javax.mail.internet.MimeMessage;


/** Thin wrapper on JavaMail. This simplifies sending email for the limited
    uses in infrasearch **/
public class MailService {
    private Session session;
    private Address defaultSenderAddr;

    /** Creates a MailService instance.
        @param smtpServer The smtp server to use.
        @param defaultSender The sending email address to use by default.
        
        @throw javax.mail.internet.AddressException Thrown in case the
        defaultSender is not a valid email address.
    **/
    public MailService( String smtpServer, String defaultSender )
    throws AddressException {
        Properties props = new Properties();
        props.put("mail.smtp.host", smtpServer );
        session = Session.getDefaultInstance(props, null);
        defaultSenderAddr = new InternetAddress( defaultSender );
    }

    /** <b>Synchronous</b> call to send mail **/
    public void sendMail( String recipient, String subject, String msg ) 
    throws MessagingException {
        // prepare message
        MimeMessage message = new MimeMessage(session);
        message.setSubject( subject );
        message.setText( msg );
        message.setFrom( defaultSenderAddr );
        InternetAddress recipAddr = new InternetAddress( recipient );
        message.addRecipient( Message.RecipientType.TO, recipAddr );

        // send
        Transport.send( message );
    }

    /** Simple test that sends a test email.  Pass in the following args:
         <li>smtp host (smtp.bar.com)
         <li>mail sender (foo@bar.com)
         <li>to (baz@bar.com)
    **/ 
    private static void main(String[] args) throws Exception {
        
        MailService service = new MailService(
            args[0],
            args[1] );

        service.testMailService( args[2] );
    }

    private void testMailService( String to ) throws Exception {
        sendMail( to, "FooSubject", "FooMail" );
    }
}
