/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ProviderServlet.java,v 1.1.1.1 2001/06/01 16:50:54 jbeatty Exp $
 */
package net.jxta.search.webprovider;

import net.jxta.search.util.TextTools;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.mail.internet.AddressException;
import javax.mail.MessagingException;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import net.jxta.search.util.UuidService;

import net.jxta.exception.JxtaException;

public class ProviderServlet extends HttpServlet {

    PersistenceService  persistence;
    UuidService         uuidService;
    MailService         mailService;
    HashMap             regRetrievalServices;
    
    String[] varNames = {"companyName", "contactName", 
			 "emailAddress", "password", 
			 "confirmPassword" };
    String[] varText  = { "Your Company Name", "Your Name",
			  "Your Email Address", "Password", 
			  "Confirm Password" };
    String[] changedVarNames = {"companyName", "contactName", 
				"emailAddress", "oldPassword", 
				"newPassword", "confirmNewPassword" };
    String[] changedVarText  = { "Your Company Name", "Your Name",
				 "Your Email Address", "Your Old Password", 
				 "Your New Password", 
				 "Your Confirmed New Password" };
    String[] loginVarNames = { "emailAddress", "password" };
    String[] loginVarText  = { "Email Address", "Password" };

    DocumentBuilder builder;
    URL[]           routers;
    URL[]           resolvers;

    public void init () throws ServletException {
	ServletConfig config   = getServletConfig ();
	ServletContext context = getServletContext ();
	Enumeration   e        = config.getInitParameterNames ();
	Vector        v1       = new Vector ();
	Vector        v2       = new Vector ();
	regRetrievalServices   = new HashMap ();

	String documentBuilderFactoryClass = getServletConfig ().
		getInitParameter ("DocumentBuilderFactory");
		
	System.setProperty ("javax.xml.parsers.DocumentBuilderFactory",
			    documentBuilderFactoryClass);
		

	try {

	    regRetrievalServices.put ("http", 
				      new HttpRegRetrievalService ());

	    regRetrievalServices.put( "jxta",
				      new JxtaRegRetrievalService ());
	
	    DocumentBuilderFactory factory = 
		DocumentBuilderFactory.newInstance ();
	    builder = factory.newDocumentBuilder ();
	    
	    uuidService = new UuidService ();
	    
	    String smtpServer = config.getInitParameter( "SmtpServer" );
	    String mailSender = config.getInitParameter( "MailSender" );
	    try {
		mailService = new MailService(smtpServer, mailSender );
	    } catch( AddressException ex ) {
		ex.printStackTrace();
		throw new ServletException( "web.xml has illegal value for "+
					    "MailSender init-param:" + ex );
	    }
	    
	    if (! e.hasMoreElements ())
		System.out.println ("ProviderServlet: no init parameters");
	    
	    while (e.hasMoreElements ()) {
		String name = (String) e.nextElement ();
		String value = (String) config.getInitParameter (name);

		if (name.startsWith ("routerURL")) {
		    
		    try {
			v1.addElement (new URL (value));
		    } catch (MalformedURLException ex) {
			System.out.println ("Bad url for init parameter " + name + 
					    ": " + config.getInitParameter (name));
		    }
		} else if (name.startsWith ("resolverURL")) {
		    
		    try {
			v2.addElement (new URL (value));
		    } catch (MalformedURLException ex) {
			System.out.println ("Bad url for init parameter " + name + 
					    ": " + config.getInitParameter (name));
		    }
		}
	    }
	    
	    routers   = new URL[v1.size ()];
	    resolvers = new URL[v2.size ()];
	    v1.copyInto (routers);
	    v2.copyInto (resolvers);
	    
	    System.out.println("ProviderServlet: loading persistence service");
	    persistence = ServiceLoader.getPersistenceService(getServletContext());
	    
	    System.out.println ("ProviderServlet: initted");
	} catch( Exception ex ) {
	    throw new ServletException( ex );
	}
    }



    public void service (HttpServletRequest req,
			 HttpServletResponse res)
	throws ServletException, IOException
    {
	ServletContext context = getServletContext ();
	
	String         action  = req.getParameter ("action");


	if (action == null ||  action.equals ("viewSignup"))
	    viewSignup (context, req, res);
	else if (action.equals ("doSignup"))
	    doSignup (context, req, res);
	else if (action.equals ("doChange"))
	    doChange (context, req, res);
	else if (action.equals ("doLogin")) 
	    doLogin (context, req, res);
	else if (action.equals ("doConfirmAccount"))
	    doConfirmAccount (context, req, res);
        else if (action.equals ("doFinalConfirm"))
	    doFinalConfirm (context, req, res);
	else if (isInSession (req)) {
	    if (action.equals ("viewChange"))
		viewChange (context, req, res);
	    else if (action.equals ("viewHistoricalStats"))
		viewHistoricalStats (context, req, res);
	    else if (action.equals ("createRegistration"))
		createRegistration (context, req, res);

	    else if (action.equals ("doRegister"))
		doRegister (context, req, res);
	    else if (action.equals ("doRegisterCategory"))
		doRegisterCategory (context, req, res);
	    else if (action.equals ("doRating")) 
		doRating (context, req, res);
	    else if (action.equals ("doVerify"))
		doVerify (context, req, res);
	    else if (action.equals ("viewRealtimeStats"))
		viewRealtimeStats (context, req, res);
	    else if (action.equals ("doForgotPassword")) 
		doForgotPassword (context, req, res);
	    else if (action.equals ("doLogout"))
		doLogout (context, req, res);
	} else {
	    RequestDispatcher disp = 
		context.getRequestDispatcher ("/index.jsp");
	    req.setAttribute ("errors", "Your session has expired.");
	    disp.forward (req, res);
	}
    }


    private boolean isInSession (HttpServletRequest req) {
	HttpSession session = req.getSession (false);

	return (session != null && session.getAttribute ("provider") != null);
    }

    public void doLogout (ServletContext context,
			  HttpServletRequest req,
			  HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp = context.getRequestDispatcher ("/index.jsp");
	HttpSession session = req.getSession (false);

	if (session != null)
	    session.invalidate ();
	
	req.setAttribute ("errors", "Thanks for using our Provider Program.");
	disp.forward (req, res);
    }

    public void doForgotPassword (ServletContext context,
				  HttpServletRequest req,
				  HttpServletResponse res)
	throws ServletException, IOException
    {
	String email = req.getParameter ("emailAddress");

	if (email == null) {
	    req.setAttribute ("errors", "Please enter the email address " +
			      "witch which you registered.");
	} else {
	    ProviderInfo pi = persistence.get (email);
	    
	    if (pi == null) {
		req.setAttribute ("errors", "No such email address registered.");
	    } else {
		Random r = new Random ();
		byte[] buf = new byte[4];
		String newPassword;
		
		r.nextBytes (buf);
		
		newPassword = newPassword = UuidService.hexify (buf);
		
		persistence.changePassword (pi, newPassword);
		
		StringBuffer msgbuf = new StringBuffer();
		msgbuf.append( "Hi, " + pi.getContactName () );
		msgbuf.append( "The password for your provider account has "+
			       "been reset to " + newPassword );
		msgbuf.append( "Please change it when you log in." );
		msgbuf.append( "\n\nSincerely yours," );
		msgbuf.append( "Mr. Winky" );

		try {
		    mailService.sendMail( pi.getEmailAddress (),
					  "Your OpenSearch Provider Account",
					  msgbuf.toString() );

		    req.setAttribute ("success",
				      "A new password has been sent.");
		} catch( MessagingException e ) {
		    req.setAttribute( "errors", "Unable to send email. Sorry.");
		}
	    }
	}

	RequestDispatcher disp = 
	    context.getRequestDispatcher ("/provider_forgot_password.jsp");

	disp.forward (req, res);
    }
    

    public void viewChange (ServletContext context, 
			    HttpServletRequest req,
			    HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;
	HttpSession session = req.getSession (false);
	
	disp = context.getRequestDispatcher ("/provider_change.jsp");
	
	disp.forward (req, res);
    }
    
    public void viewSignup (ServletContext context,
			    HttpServletRequest req,
			    HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;

	disp = context.getRequestDispatcher ("/provider_signup.jsp");
	disp.forward (req, res);
    }

    public void doRating (ServletContext context,
			  HttpServletRequest req,
			  HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;
	
	req.setAttribute ("rated", Boolean.TRUE);

	disp = context.getRequestDispatcher ("/rating.jsp");
	disp.forward (req, res);
    }


    void viewHistoricalStats (ServletContext context,
			      HttpServletRequest req,
			      HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp = context.getRequestDispatcher ("/provider_history.jsp");

	disp.forward (req, res);
    }
   void createRegistration (ServletContext context,
			      HttpServletRequest req,
			      HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp = context.getRequestDispatcher ("/provider_reg.jsp");

	disp.forward (req, res);
    }
    void viewRealtimeStats (ServletContext context,
			    HttpServletRequest req,
			    HttpServletResponse res)
	throws ServletException, IOException
    {
	HttpSession session = req.getSession (false);

	ProviderInfo info = (ProviderInfo) session.getAttribute ("provider");
	RequestDispatcher disp = 
	    context.getRequestDispatcher ("/provider_stats.jsp");
	
	Stat[] stats = collectRouterStats (routers, 
					   info.getRegistrationUrl ());
	    
	req.setAttribute ("stats", stats);
	
	disp.forward (req, res);
    } 

    public void doLogin (ServletContext context,
			 HttpServletRequest req,
			 HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;
	String[]          loginVarValues = new String[loginVarNames.length];
	String            errors         = "";
	ProviderInfo      pi             = null;


	for (int i = 0;  i < loginVarNames.length;  i++) {
	    if (! defined (req.getParameter (loginVarNames[i])))
		errors += "" + loginVarText[i] + " is missing.<br>";
	    else
		loginVarValues[i] = req.getParameter (loginVarNames[i]);
	}
    
	if (defined (loginVarValues[0]) && 
	    loginVarValues[0].indexOf ('@') == -1)
	    errors += "Email address does not contain @ character.<br>";
	
	if (errors.length () > 0) {
	    req.setAttribute ("errors", errors);
	    disp = context.getRequestDispatcher ("/index.jsp");
	    disp.forward (req, res);
	} else {
	    pi = persistence.get (loginVarValues[0], loginVarValues[1]);

	    if (pi == null) {
		errors = "Invalid email address or password<br>";
		req.setAttribute ("errors", errors);
		disp = context.getRequestDispatcher ("/index.jsp");
		disp.forward (req, res);
	    } else if( pi.getStatus() == null ) {
                errors = "Your account is deprecated: it does not have a status.";
                req.setAttribute ("errors", errors);
                disp = context.getRequestDispatcher ("/index.jsp");
                disp.forward (req, res);
	    } else if( pi.getStatus().equals(
                    ProviderInfo.STATUS_PENDING_CONFIRM ) ) {

                errors = "Your account has not yet been activated. Please " +
                         "click on the activation link in the activation " +
                         "email that was sent.<br>";
                req.setAttribute ("errors", errors);
                disp = context.getRequestDispatcher ("/index.jsp");
                disp.forward (req, res);
            } else if( pi.getStatus().equals( ProviderInfo.STATUS_ACTIVE ) ) {
		disp = context.getRequestDispatcher ("/provider.jsp");
		HttpSession session = req.getSession (true);
		
		session.setAttribute ("provider", pi);
		req.setAttribute ("provider", pi);
		disp.forward (req, res);
	    } else {
                errors = "Your account is in an unknown state.<br>";
                req.setAttribute ("errors", errors);
                disp = context.getRequestDispatcher ("/index.jsp");
                disp.forward (req, res);
            }
	}

    }
			 

    public void doVerify (ServletContext context,
			  HttpServletRequest req,
			  HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp = 
	    context.getRequestDispatcher ("/provider_verify.jsp");
	String reg = req.getParameter ("registration");
	String errors = null;
	boolean passed;
	ByteArrayInputStream in = new ByteArrayInputStream (reg.getBytes ());

	try {
	    DocumentBuilderFactory factory = 
		DocumentBuilderFactory.newInstance();
	    
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document        doc     = builder.parse (in);
	    // Provider        p       = new Provider (doc);
	    passed = true;
	} catch (Exception e) {
	    e.printStackTrace ();
	    errors = e.getMessage ();
	    passed = false;
	}
	
	
	req.setAttribute ("passed", new Boolean (passed));
	if (errors != null)
	    req.setAttribute ("errors", errors);
	
	disp.forward (req, res);
    }


    public void doRegisterCategory(ServletContext context,
			    HttpServletRequest req,
			    HttpServletResponse res)
	throws ServletException, IOException
    {

	RequestDispatcher disp;

         disp = context.getRequestDispatcher ("./testcat.html");
	    disp.forward (req, res);


    }  /* close doRegisterCategory() */

    private String parseQueryServer( InputStream in )
    throws SAXException, IOException {

        String retval=null;
        Document doc = builder.parse (in);
        
        Element root  = doc.getDocumentElement ();
        NodeList toplevelnodes = root.getElementsByTagName("register");
        Node register = toplevelnodes.item(0);
        NodeList nodes = register.getChildNodes();
        Node node;
        for (int i = 0;  i < nodes.getLength ();  i++) {
            node = nodes.item (i);
            String name = node.getNodeName ();
            
            if (name.equals ("query-server")) {
                retval  = ((Text) node.getFirstChild ()).getData();
            }
        }

        return retval;
    }

    public void doRegister (ServletContext context,
			    HttpServletRequest req,
			    HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;
	HttpSession       session = req.getSession (false);
	ProviderInfo      pi      = null;
	String            errors  = null;

	String emailAddress = req.getParameter ("emailAddress");
	String password     = req.getParameter ("password");

	if (emailAddress != null && password != null) {
	    if ((pi = persistence.get (emailAddress, password)) == null)
		errors = "Invalid email or password.<br>";
	}
	
	if (pi == null)
	    pi = (ProviderInfo) session.getAttribute ("provider");
	
	if (errors != null) {
	    req.setAttribute ("errors", errors);
	    disp = context.getRequestDispatcher ("/index.jsp");
	    disp.forward (req, res);
	} else {

	    String registrationUrl = req.getParameter ("registrationUrl");
	    
	    
	    pi.setRegistrationUrl (registrationUrl);
	    
	    try {
		
		URL    url  = new URL (registrationUrl);
                callResolverManager(new URL(resolvers[0], "/resolver/manager"),
				    "add",
				    pi.getUuid (),
				    url);
	    
            } catch (MalformedURLException e) {
                e.printStackTrace();
		errors = "You have specified an invalid url.";
	    } catch (IOException e) {
		errors = e.getMessage ();
		e.printStackTrace ();
	    } 
	    
	    
	    if (errors == null) {
		try {
		    /*
		    DocumentBuilderFactory factory = 
			DocumentBuilderFactory.newInstance();
		    
		    DocumentBuilder builder = factory.newDocumentBuilder();
		    System.out.println("Indexing " + file);
		    Document doc = builder.parse (file);
		    */
		    // resolver.addProvider (doc);
		    
		    errors = "Parsed and registered contents of url.";
		} catch (Exception e) {
		    errors = "Could not parse xml registration: " +
			e.getMessage ();
		}
	    }

	    if (errors != null) {
		req.setAttribute ("errors", errors);
	    }

	    disp = context.getRequestDispatcher ("/provider.jsp");
	    disp.forward (req, res);
	}
    }

    public void doChange (ServletContext      context,
			  HttpServletRequest  req,
			  HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;
	HttpSession       session = req.getSession (false);
	
	ProviderInfo      pi = (ProviderInfo) session.getAttribute ("provider");
	String            oldEmailAddress = req.getParameter ("oldEmailAddress");

	String[]          changedVarValues =new String[changedVarNames.length];
	String            errors = "";


	for (int i = 0;  i < changedVarNames.length - 2;  i++) {
	    if (! defined (req.getParameter (changedVarNames[i])))
		errors += "" + changedVarText[i] + " is missing.<br>";
	    else
		changedVarValues[i] = req.getParameter (changedVarNames[i]);
	}
	changedVarValues[4] = req.getParameter (changedVarNames[4]);
	changedVarValues[5] = req.getParameter (changedVarNames[5]);
	

	if (defined (changedVarValues[2]) && 
	    changedVarValues[2].indexOf ('@') == -1)
	    errors += "Email address does not contain @ character.<br>";
	
	if (defined (changedVarValues[3]) &&
	    persistence.get (oldEmailAddress, changedVarValues[3]) == null)
	    errors += "You specified an incorrect old password.";
	
	if (defined (changedVarValues[4]) && ! defined (changedVarValues[5]))
	    errors += "You specified a new password but have not confirmed it.";
    
	if (defined (changedVarValues[4]) && defined (changedVarValues[5]) &&
	    ! changedVarValues[4].equals (changedVarValues[5]))
	    errors += "Your confirmed password does not match.<br>";
    
	boolean emailChanged = false;
	if (! changedVarValues[2].equals (oldEmailAddress))
	    emailChanged = true;

	if (errors.length () == 0) {
	    pi.set (changedVarValues[0], 
		    changedVarValues[1], 
		    oldEmailAddress,
		    req.getParameter ("canEmail") != null);
	    if (defined (changedVarValues[4]))
		persistence.changePassword (pi, changedVarValues[4]);	    
	}

	if (errors.length () > 0) {
	    req.setAttribute ("errors", errors);
	    disp = context.getRequestDispatcher ("/provider_change.jsp");
	    disp.forward (req, res);
	} else {
	    if ( emailChanged ) {
		errors = "An email confirming your email address change has "+
		    "been sent to " + pi.getEmailAddress() + ". You need to "+
		    "confirm this change by clicking on the activation link "+
		    "in the email.";
	    }
	    req.setAttribute ("provider", pi);
	    session.setAttribute ("provider", pi);
	    disp = context.getRequestDispatcher ("/provider.jsp");
	    disp.forward (req, res);
	}
    }


    public void doSignup (ServletContext      context,
			  HttpServletRequest  req,
			  HttpServletResponse res)
	throws ServletException, IOException
    {
	RequestDispatcher disp;
	String[]          varValues = new String[varNames.length];
	String            errors = "";
	ProviderInfo      pi = null;
	
	for (int i = 0;  i < varNames.length;  i++) {
	    if (! defined (req.getParameter (varNames[i])))
		errors += "" + varText[i] + " is missing.<br>";
	    else
		varValues[i] = req.getParameter (varNames[i]);
	}
    
	if (defined (varValues[2]) && varValues[2].indexOf ('@') == -1)
	    errors += "Email address does not contain @ character.<br>";
	
	if (req.getParameter ("acceptTC") == null ||	
	    ! req.getParameter ("acceptTC").equals ("Yes")) 
	    
	    errors += "You need to accept the Terms and Conditions.<br>";
	
	if (defined (varValues[3]) && defined (varValues[4]) &&
	    ! varValues[3].equals (varValues[4]))
	    errors += "Your confirmed password does not match.<br>";
	
	String uuid = null;
	if (errors.length () == 0) {
            uuid = uuidService.hexify (uuidService.getUuid ());
	    pi = persistence.put (varValues[2], varValues[3], uuid);
	    
	    if (pi == null)
		errors += "Another account is already registered with " +
		    "the email address <b>" + varValues[2] + "</b>.";
	    else
		pi.set (varValues[0], 
			varValues[1], 
			varValues[2],
			req.getParameter ("canEmail") != null);
	}
	
	if (errors.length () > 0) {
	    req.setAttribute ("errors", errors);
	    disp = context.getRequestDispatcher ("/provider_signup.jsp");
	    disp.forward (req, res);
	} else {
            String confirmCode = uuid;
            pi.setStatus( ProviderInfo.STATUS_PENDING_CONFIRM );

            try {
                sendConfirmationEmail( req, pi.getEmailAddress(), false );
            } catch( MessagingException e ) {
                errors = "Unable to send account activation email. Please try " +
                    "creating your account again.";
                
                // TODOjdb: HOW TO REVERSE ACCOUNT CREATION???
                req.setAttribute ("errors", errors);
	        disp = context.getRequestDispatcher ("/provider_signup.jsp");
	        disp.forward (req, res);
            }
            
            errors = "An account activation email has been sent to " +
                pi.getEmailAddress() + ". You can activate your account " +
                "by clicking on the activation link in the email.";
            req.setAttribute ("errors", errors);
            disp = context.getRequestDispatcher ("/provider_confirm_msg.jsp");
            disp.forward (req, res);
	}
    }

    private void sendConfirmationEmail( HttpServletRequest req,
					String emailAddress,
					boolean isExistingAccount )
    throws MessagingException {
	String cannedMsg;
	if( !isExistingAccount ) {
	    cannedMsg = "Thank you for signing up with " +
		"JXTASearch!  To activate your account, \nplease click the "+
		"link below:\n\n\n";
	} else {
	    cannedMsg = "You have changed your email address on the /JXTASearch " +
		"service.\nPlease confirm your change of email address by " +
		"clicking on the\nlink below.\n\n\n";
	}

	String url = HttpUtils.getRequestURL( req ).toString() +
	    "?action=doConfirmAccount&confirmcode=" +
	    TextTools.base64encode( emailAddress.getBytes() );

        mailService.sendMail( emailAddress,
                              "JXTASearch Account Confirmation",
                              cannedMsg + url );

    }

    public void doConfirmAccount( ServletContext context,
			          HttpServletRequest  req,
			          HttpServletResponse res )
    throws ServletException, IOException {


        RequestDispatcher disp =
            context.getRequestDispatcher ("/provider_confirm.jsp");

        disp.forward (req, res);
    }

    public void doFinalConfirm( ServletContext context,
			          HttpServletRequest  req,
			          HttpServletResponse res )
    throws ServletException, IOException {
        
	
        RequestDispatcher disp;
	String[]          loginVarValues = new String[loginVarNames.length];
	String            errors         = "";
	ProviderInfo      pi             = null;


	for (int i = 0;  i < loginVarNames.length;  i++) {
	    if (! defined (req.getParameter (loginVarNames[i])))
		errors += "" + loginVarText[i] + " is missing.<br>";
	    else
		loginVarValues[i] = req.getParameter (loginVarNames[i]);
	}
    
	if (defined (loginVarValues[0]) && 
	    loginVarValues[0].indexOf ('@') == -1)
	    errors += "Email address does not contain @ character.<br>";
	
	if (errors.length () > 0) {
	    req.setAttribute ("errors", errors);
	    disp = context.getRequestDispatcher ("/provider_confirm.jsp");
	    disp.forward (req, res);
	} else {
	    pi = persistence.get (loginVarValues[0], loginVarValues[1]);

	    // handle the email address change
	    String base64Email = req.getParameter( "confirmcode" );
	    if( base64Email != null ) {
		String newEmailAddress = TextTools.base64decode( base64Email.getBytes() );
		persistence.changeEmail( pi, newEmailAddress );
	    }

	    if (pi == null) {
		errors = "Invalid email address or password<br>";
		req.setAttribute ("errors", errors);
		disp = context.getRequestDispatcher ("/provider_confirm.jsp");
		disp.forward (req, res);
            } else if( pi.getStatus() == null ) {
                errors = "Your account is deprecated: it does not have a status.";
                req.setAttribute ("errors", errors);
                disp = context.getRequestDispatcher ("/provider_confirm.jsp");
                disp.forward (req, res);
            } else if( pi.getStatus().equals(
                    ProviderInfo.STATUS_PENDING_CONFIRM ) ) {

                pi.setStatus( ProviderInfo.STATUS_ACTIVE );
		disp = context.getRequestDispatcher ("/provider.jsp");
		HttpSession session = req.getSession (true);		
		session.setAttribute ("provider", pi);
		req.setAttribute ("provider", pi);
		disp.forward (req, res);
            } else if( pi.getStatus().equals( ProviderInfo.STATUS_ACTIVE ) ) {
                disp = context.getRequestDispatcher ("/provider.jsp");
                HttpSession session = req.getSession (true);
                session.setAttribute ("provider", pi);
                req.setAttribute ("provider", pi);
                disp.forward (req, res);
            } else {
                errors = "Your account is in an unknown state.<br>";
                req.setAttribute ("errors", errors);
                disp = context.getRequestDispatcher ("/provider_confirm.jsp");
                disp.forward (req, res);
            }
	}        
    }


    boolean defined (String val) {
	return val != null && val.length () > 0;
    }
    


    Stat[] collectRouterStats (URL[] routers, String providerURL) {
	Stat[] stats = new Stat[routers.length];
	
	for (int i = 0;  i < routers.length;  i++) {
	    
	    try {
		stats[i] = callRouterStat (routers[i], providerURL);
	    } catch (IOException e) {
		/* mark that router as being down */
		stats[i] = new Stat (routers[i]);
		e.printStackTrace ();
	    } catch (SAXException e) {
		stats[i] = new Stat (routers[i]);
		e.printStackTrace ();
	    }
	}
	
	return stats;
    }
    
    Stat callRouterStat (URL router, String providerURL) 
	throws IOException, SAXException
    {
	URL url = new URL (router, "?action=doStats&url=" +
			   URLEncoder.encode (providerURL));
	HttpURLConnection con = (HttpURLConnection) url.openConnection ();

	con.setRequestMethod ("GET");
	con.setDoInput (true);
	con.setDoOutput (false);
		
	if (con.getResponseCode () == HttpURLConnection.HTTP_OK) {
	    Document doc      = builder.parse (con.getInputStream ());
	    
	    Element  root     = doc.getDocumentElement ();
	    Element  attempt  = (Element) root.getElementsByTagName
		("attempt").item (0);
	    NodeList children = attempt.getChildNodes ();

	    int  queries = 0;
	    int  failed = 0;
	    long last   = 0;
	    long averageTime = 0;

	    for (int i = 0;  i < children.getLength ();  i++) {
		Node c = children.item (i);
		String name = c.getNodeName ();
		String value;
		
		if (name.equals ("queries")) {
		    value = ((Text) c.getFirstChild ()).getData ();
		    
		    queries = Integer.parseInt (value);

		} else if (name.equals ("failed")) {
		    value = ((Text) c.getFirstChild ()).getData ();

		    failed = Integer.parseInt (value);
		} else if (name.equals ("last")) {
		    value = ((Text) c.getFirstChild ()).getData ();
		    
		    last   = Long.parseLong (value);
		} else if (name.equals ("average-time")) {
		    value = ((Text) c.getFirstChild ()).getData ();
		    
		    averageTime = Long.parseLong (value);
		}
		
	    }
	    
	    return new Stat (router, queries, failed, last, averageTime);
	    
	} else {
	    return new Stat (router, 0, 0, 0, 0);
	}
	    

    }

    void callResolverManager (URL    manager, 
			      String action, 
			      String id, 
			      URL    registration)
	throws IOException
    {
	HttpURLConnection callCon;
	InputStream       in;
	OutputStream      out;
	byte[]            buf;
	int               r;
	int               cl;

	try {

            RegRetrievalService service = (RegRetrievalService)
                regRetrievalServices.get(registration.getProtocol() );
            
            service.retrieveRegistration( registration );
            cl=service.getContentLength();
            in=service.getInputStream();

	} catch (IOException e) {
	    throw new IOException ("Could not retrieve registration."); 
	}

		
	try {
	    callCon     = (HttpURLConnection) manager.openConnection ();
	    callCon.setRequestMethod ("POST");
	    callCon.setRequestProperty ("Resolver-Action", action);
	    callCon.setRequestProperty ("Resolver-ExternalId", "" + id);
	    String contentLenStr = String.valueOf(cl);
	    if (cl > 0) 
		callCon.setRequestProperty ("Content-length", 
					    contentLenStr );
	    callCon.setDoOutput (true);
	    callCon.connect ();
	    out = callCon.getOutputStream ();
            if( out == null ) System.out.println( "IT'S NULL" );
            else System.out.println( out.toString() );
	} catch (IOException e) {
            e.printStackTrace();
	    throw new IOException ("Could not connect to resolver manager.");
	}

	buf = new byte[8192];

	try {
	    while ((r = in.read (buf)) > 0) {
		try {
		    out.write (buf, 0, r);
		} catch (IOException e) {
                    System.out.println( "exception writing: " + e );
                    e.printStackTrace();
		    throw new IOException ("Perhaps the resolver manager is " +
					   "down: " + e.getMessage ());
		}
	    }
	    
	    out.flush ();
	    
	    if (callCon.getResponseCode () != callCon.HTTP_OK) {
		throw new IOException ( "Problem with resolver manager add command" );
	    }
	    
	} finally {
	    in.close ();
	    out.close ();
	}
	System.out.println ("callResolverManager: Closed everything.");
	
    }

    public class Stat {

	URL     router;
	int     queryCount;
	int     failedCount;
	long    last;
	long    averageResponseTime;
	boolean isDown;
	boolean hasData;
	
	Stat (URL router, int queryCount, int failedCount, long last,
	      long averageResponseTime)
	{
	    this.router = router;

	    if (last == -1) {
		this.hasData = false;
		this.isDown  = false;
	    } else {
		this.queryCount = queryCount;
		this.failedCount = failedCount;
		this.last = last;
		this.averageResponseTime = averageResponseTime;
		this.isDown = false;
		this.hasData = true;

	    }
	}

	Stat (URL router) {
	    this.router = router;
	    this.isDown = true;
	}

	public URL     getRouter () { return router; }
	public int     getQueryCount () { return queryCount; }
	public int     getFailedCount () { return failedCount; }
	public long    getLastTime () { return last; }
	public boolean isDown () { return isDown; }
	public boolean hasData () { return  hasData; }
	public long    getAverageResponseTime () { 
	    return averageResponseTime; 
	}
    }
}
