/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: PreferencesServlet.java,v 1.1.1.1 2001/06/01 16:51:00 jbeatty Exp $
 */

package net.jxta.search.websearch;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;


public class PreferencesServlet extends HttpServlet {
    
    PreferencesService preferencesService;
    
    public void init () throws ServletException {
        ServletContext context = getServletContext ();

        preferencesService = PreferencesService.getPreferencesService (context);
        
    }

    Preferences getPreferences (HttpServletRequest req) {
        HttpSession session = req.getSession (true);
        Preferences prefs   =
            (Preferences) session.getAttribute ("preferences");

        
        if (prefs == null) {
            
            Cookie[] cookies = req.getCookies ();
            for (int i = 0; cookies != null && i < cookies.length;  i++)
                if (cookies[i].getName ().equals ("PREFS")) {
                    System.out.println ("Using cookie preferences");
                    prefs = preferencesService.getPreferences (cookies[i].getValue ());
                }
            
            if (prefs == null) {
                System.out.println ("Using default peferences");
                prefs = preferencesService.getDefault ();
            }
            session.setAttribute ("preferences", prefs);
        } else {
            System.out.println ("Using session preferences");
        }
        
        return prefs;
    }


    public void doGet (HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        ServletContext    context   = getServletContext ();
        RequestDispatcher prefsPage = 
            context.getRequestDispatcher ("/preferences.jsp");

        /* make sure there's a preferences object in the session */
        getPreferences (req);

        prefsPage.forward (req, res);
    }

    public void doErrorPage (HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        ServletContext    context   = getServletContext ();
        RequestDispatcher errorPage = context.getRequestDispatcher ("/error.jsp");
        req.setAttribute ("error", "there was bad or missing information in your request");
        errorPage.forward (req, res);
    }

    public void doPost (HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        ServletContext    context   = getServletContext ();
        RequestDispatcher indexPage = context.getRequestDispatcher ("/index.jsp");

        String  languages       = null;
        int     sorting;
        int     resultsPerPage;
        int     resultsPerProvider;
        int     enableImages;
        int     queryLifetime;
        int     flushTime;

        if (req.getParameter ("images") != null)     //
            enableImages = Preferences.IMAGES_ENABLED;
        else
            enableImages = Preferences.IMAGES_DISABLED;

        try {
            sorting            = Integer.parseInt (req.getParameter ("sort"));
            resultsPerPage     = Integer.parseInt (req.getParameter ("perpage"));
            resultsPerProvider = Integer.parseInt (req.getParameter ("perprovider"));
            queryLifetime      = Integer.parseInt (req.getParameter ("querylife"));
            flushTime          = Integer.parseInt (req.getParameter ("flushtime"));

        } catch (NumberFormatException ne) {
          this.doErrorPage (req, res);
          return;
        }

        
        if (req.getParameter ("alllanguages") != null) {
            languages = "all";
        } else {
            String[] vals = req.getParameterValues ("languages");
            if (vals.length > 0) {
                languages = vals[0];
                for (int i = 1;  i < vals.length;  i++)
                    languages += "," + vals[i];
            }
        }

        
        Cookie[] cookies = req.getCookies ();
        Cookie   prefsCookie = null;
        Preferences prefs;
        if (cookies != null) 
            for (int i = 0;  i < cookies.length;  i++) {
                if (cookies[i].getName ().equals ("PREFS")) {
                    prefsCookie = cookies[i];
                    break;
                }
            }

        if (prefsCookie == null ||
            (prefs  = 
             preferencesService.getPreferences (prefsCookie.getValue ())) == null) {
            
            /* create the preferences object, set the values, and store. */
            prefs = preferencesService.newPreferences ();
            preferencesService.putPreferences (prefs);
            System.out.println ("Creating new prefs: " + prefs.getId ());
            prefsCookie = new Cookie ("PREFS", prefs.getId ());
            // prefsCookie.setDomain ("24.1.6.138:8080");
            prefsCookie.setPath ("/");
            prefsCookie.setMaxAge (600 * 24 * 3600);
            res.addCookie (prefsCookie);
        } 


        prefs.set (languages, 
                   resultsPerPage, 
                   resultsPerProvider, 
                   sorting,
                   enableImages, 
                   queryLifetime * 1000, // user gives seconds, api takes millis
                   flushTime * 1000);    // user gives seconds, api takes millis

    
        HttpSession session = req.getSession (true);

        session.setAttribute ("preferences", prefs);
        
//       indexPage.forward (req, res);
        res.sendRedirect ("index.jsp");
          
    }



}

