/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RedirectServlet.java,v 1.1.1.1 2001/06/01 16:51:00 jbeatty Exp $
 */

package net.jxta.search.websearch;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import net.jxta.search.util.LogService;

public class RedirectServlet extends HttpServlet {
  private static String  DEF_REDIRECT_LOG  = "redirect";
  private static String  DEF_REDIRECT_ROOT = "/r";

  LogService redirectLog;
  String redirectRoot;


  public void init () throws ServletException {

    System.out.println ("HI Redirect Inited");

    ServletContext context = getServletContext ();
    ServletConfig  config  = getServletConfig ();

    this.redirectLog  = new LogService (config.getInitParameter ("redirectLog"), LogService.ALL);
    this.redirectRoot = config.getInitParameter ("redirectRoot");
  }

  /**
   *  Calls doGet ()
   */
  public void doPost (HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    doGet (req, res);
  }

  public void doErrorPage (HttpServletRequest req,
                           HttpServletResponse res)
    throws ServletException, IOException
  {
    ServletContext    context   = getServletContext ();
    RequestDispatcher errorPage = context.getRequestDispatcher ("/error.jsp");
    req.setAttribute ("error", "there was bad or missing information in your request");
    errorPage.forward (req, res);
  }

  /**
   * Parses request and redirects user to appropriate URL then log event.
   */
  public void doGet (HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {

      String servletPath  = req.getServletPath ();
      String uri          = req.getRequestURI ();
      String queryString  = req.getQueryString ();
      String utid;
      String link;

      int utidStart = 0;
      int utidEnd   = 0;

      try {
        utidStart = uri.indexOf (servletPath, 0) + servletPath.length () + 1;
        utidEnd   = uri.indexOf ("/", utidStart);
        utid      = uri.substring (utidStart, utidEnd);
        link      = uri.substring (utidEnd + 1);

        if (queryString != null)
          link = link + "?" + queryString;

        URL target =  new URL (link);
        res.sendRedirect (target.toString ());
        if (utid != null)
          redirectLog.log (redirectLog.INFO, utid, target.toString ());

      } catch (MalformedURLException me) {
        doErrorPage (req, res);

      } catch (IndexOutOfBoundsException ie) {
        doErrorPage (req, res);
      }


  }



}
