/*
 * Copyright (c) 2001 Sun Microsystems, Inc. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Sun Microsystems, Inc. for Project JXTA."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA"
 * must not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 * nor may "JXTA" appear in their name, without prior written
 * permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA. For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Search.java,v 1.1.1.1 2001/06/01 16:51:02 jbeatty Exp $
 */

package net.jxta.search.websearch;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;

import net.jxta.search.peer.RouterClient;
import net.jxta.search.peer.Request;
import net.jxta.search.util.Html;
import net.jxta.search.util.PriorityQueue;
import net.jxta.search.util.SchedulerService;
import net.jxta.search.util.UuidService;
import net.jxta.search.util.Soundex;
import net.jxta.search.util.HexTools;
import net.jxta.search.util.LogService;



/**
 * Manages access to the Router application
 */
class Search implements Runnable {
    long        lastAccess;
    boolean     done;
    int         debug = 0;

    RouterClient         routerClient;
    Request              request;
    ResponseCallbackImpl callback;

    int totalProviders   = 0;
    int totalTime        = 0;
    int queriedProviders = 0;
    int providerCount    = 0;
    int hitCount         = 0;

    Search (RouterClient    routerClient, 
            Request         request, 
	    DocumentBuilder builder,
            LogService      logService) 
            throws IOException
    {
        this.done   = false;
        this.debug  = debug;
        this.routerClient = routerClient;
        this.request      = request;
        this.callback     = new ResponseCallbackImpl (request.getQuery (), 
                                                      request.getMaxHitsPerProvider (),
                                                      this, 
						      builder,
                                                      logService);
    }

    public void run () {
        lastAccess = System.currentTimeMillis ();        
            
        try {

            routerClient.search (request, callback);
        
	} catch (IOException e) {
            
	    e.printStackTrace ();
            	
	} catch (Throwable t) {
            
	    t.printStackTrace ();
        
	} finally {
	
	    terminate ();
	}
    }

    synchronized public void updateLastAccess () {
        lastAccess = System.currentTimeMillis ();
    }

    synchronized void newProvider () {
        providerCount++;
        lastAccess = System.currentTimeMillis ();
        notify ();
    }

    synchronized void newHit () {
        hitCount++;
        lastAccess = System.currentTimeMillis ();
        notify ();
    }

    synchronized void waitFor (int start, int len, long timeout) {
        lastAccess = System.currentTimeMillis ();

        try {
            
            while (! done && (hitCount + 1 < start + len)) { 
                //compensate for starts 1 relativeness
                wait ();
            }
            
        } catch (InterruptedException e) {
            
        }
    }
    
    synchronized void terminate () {
        done = true;
        notify ();
    }

    synchronized boolean isDone () {
	return done;
    }
}
