/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Cipher.java,v 1.2 2001/04/23 23:18:35 yeager Exp $
 */

package jxta.security.cipher;

import jxta.security.util.Util;
import jxta.security.util.Description;
import jxta.security.exceptions.CryptoException;

// An implementation of Cipher can be for any symmetric crypto
// algorithm
public interface Cipher extends Description {

  public static final String Ciper = "CIPHER";

  // Supported Cipers
  public static final byte ALG_RC4 = 4;

  // Actions: Encrypt Decrypt
  public static final byte MODE_ENCRYPT = 1;
  public static final byte MODE_DECRYPT = 2;
  public static final byte MODE_UNKNOWN = 99;
  // Methods - straight from java card 2.1 api's



  /**
   *   Initiate Cipher object with key. Here, either no parameters,
   *   or constant parameter values
   */
  public void init(Key theKey, byte theMode)
    throws CryptoException;

  /**
   * Initiate the instance of Cipher with the right key, parameters, etc ...
   */
  public void init(Key theKey,
		   byte theMode,
		   byte[] bArray,
		   int bOff,
		   int bLen)
    throws CryptoException;

  /**
   *   // return the algorithm used
   */
  public byte getAlgorithm();


  // Generates encrypted/decrypted output from input data.
  //    If used, then intermediate storage required
  //    Only used if all input data is not available in one byte array
  public int update(byte[] inBuff,
		       int inOffset,
		       int inLength,
		       byte[] outBuff,
		       int outOffset)
    throws CryptoException;

  /**
   *  Generate encrypted/decrypted output for all | last input data
   */
  public abstract int doFinal(byte[] inBuff,
			      int inOffset,
			      int inLength,
			      byte[] outBuff,
			      int outOffset)
    throws CryptoException;

}

