/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: JxtaCrypto.java,v 1.7 2001/04/30 22:18:29 yeager Exp $
 */

package jxta.security.crypto;

import jxta.security.cipher.Cipher;
import jxta.security.publickey.PublicKeyAlgorithm;
import jxta.security.hash.Hash;
import jxta.security.keyexchange.KeyExchange;
import jxta.security.mac.MAC;
import jxta.security.exceptions.CryptoException;

import jxta.security.signature.Signature;

// Mutiple implementations instantialte different security systems.

public interface JxtaCrypto {

  static final String name = "JxtaCrypto";

  // Accessors
  public Cipher getJxtaCipher();	// Cipher 
  public Cipher getJxtaCipher(byte type) // selected member
    throws CryptoException;

  public Hash getJxtaHash();	// first member
  public Hash getJxtaHash(byte type) // selected member
    throws CryptoException;

  public KeyExchange getJxtaKeyExchange(); // Key exchange

  public MAC getJxtaMAC();	// MAC
  // public MAC getJxtaMAC(byte type) // selected member
  //   throws CryptoException;

  public PublicKeyAlgorithm getJxtaPublicKeyAlgorithm(); // Public Key

  public Signature getJxtaSignature(); // Signature

  // profile member definitions
  static final byte MEMBER_RC4  = 1;
  static final byte MEMBER_RSA  = 2;
  static final byte MEMBER_SHA1 = 4;
  static final byte MEMBER_MD5  = 8;

  // number of digests, cipers and macs
  static final int nDigests = 2;
  static final int nCiphers = 1;
  static final int nMacs = 1;

  // profiles
  static final byte PROFILE_RSA_RC4_SHA1 = (MEMBER_RC4+MEMBER_RSA+MEMBER_SHA1);
  static final byte PROFILE_RSA_RC4_MD5  = (MEMBER_RC4+MEMBER_RSA+MEMBER_MD5);
  static final byte PROFILE_RSA_SHA1 = (MEMBER_RSA+MEMBER_SHA1);
  static final byte PROFILE_RSA_MD5  = (MEMBER_RSA+MEMBER_MD5);

  static final byte PROFILE_RSA_RC4_SHA1_MD5 = (PROFILE_RSA_RC4_MD5 + MEMBER_SHA1);
  static final byte PROFILE_RC4_SHA1 = (MEMBER_RC4+MEMBER_SHA1);
  static final byte PROFILE_RC4_MD5 = (MEMBER_RC4+MEMBER_MD5);
}

