/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: CryptoException.java,v 1.4 2001/04/17 21:56:41 yeager Exp $
 */

package jxta.security.exceptions;


// For cryto exceptions (those found in javacard.security.CryptoException)
public class CryptoException extends Exception {
  /**
   * Indicates illegal use of a method
   */
  public static final short ILLEGAL_USE       = 1;
  /** 
   * Indicates a method was passed illegal parameter values
   */
  public static final short ILLEGAL_VALUE     = 2;
  /** 
   * Indicates an object has not been properly initilaized for the
   * requested operation.
   */
  public static final short INVALID_INIT      = 3;
  /** 
   * Indicates that a requested algorithm or key type is not supported.
   */
    public static final short NO_SUCH_ALGORITHM = 4;
  /** 
   * Indicates that is key object has not been properly initialized 
   */
  public static final short UNINITIALIZED_KEY = 5;

  /**
   * Flags a method as not yet implemented
   */
  public static final short NOT_IMPLEMENTED = 6;

  /**
   * Must set RSA public key before setting RSA Private key
   */
  public static final short INVALID_RSA_INIT = 7;
  /**
   * Must set RSA public key before setting RSA Private key
   */
  public static final short INVALID_RSA_PADDING = 8;

  public static final short INVALID_RSA_DATALENGTH = 9;

  private short why;
    
  /**
   * Constructs a CryptoException with the specified reason code.
   */ 
  public CryptoException(short reason) {
    why = reason;
  }
  /**
   * Throws a CryptoException with the specified reason code.
   */ 
  public static void throwIt(short reason) throws CryptoException {
    throw new CryptoException(reason);
  }
  /**
   * Returns a human readable string describing the CryptoException.
   */ 
  public String toString() {
    String tmp = "jxta.security.CryptoException ("; 
    switch (why) {
    case ILLEGAL_USE:
      tmp += "Illegal use";
      break;
    case ILLEGAL_VALUE:
      tmp += "Illegal value";
      break;
    case INVALID_INIT:
      tmp += "Invalid initialization";
      break;
    case NO_SUCH_ALGORITHM:
      tmp += "No such algorithm";
      break;
    case UNINITIALIZED_KEY:
      tmp += "Uninitialized key";
      break;
    case NOT_IMPLEMENTED:
      tmp += "Not yet implemented";
      break;
    case INVALID_RSA_INIT:
      tmp += "Must set RSA Public/Private key first";
      break;
    case INVALID_RSA_PADDING:
      tmp += "Invalid RSA padding";
      break;
    case INVALID_RSA_DATALENGTH:
      tmp += "RSA data length exceeds (modulus - 11) bytes";
    }
    return (tmp + ")");
  }
}
