/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: MAC.java,v 1.2 2001/04/27 21:58:33 yeager Exp $
 */

package jxta.security.mac;

import jxta.security.util.Description;
import jxta.security.cipher.Key;
import jxta.security.exceptions.CryptoException;

// Message Authentication Code
public interface MAC extends Description {

  static final String name = "MAC";
  static final byte ALG_NULL = 0;
  static final byte ALG_RC4_SHA1 = 1;
  static final byte ALG_RC4_MD5  = 2;
  static final byte MODE_ENCRYPT = 1;
  static final byte MODE_VERIFY = 2;
  
   /**
   * return algorithm name as a String
   */
  public String getAlgorithmName();

  /**
   * return algorithm type value
   */
  public byte getAlgorithm();
  /**
   * get the length of the MAC
   */
  public int getLength();
  /**
   * initialize the MAC object behaviour
   */
  public void init(byte theMode, Key theKey, byte[] privateKey)
    throws CryptoException;
  /**
   * updates data to be encrypted.
   */
  public void update(byte[] inbuf, 
		     int offset, 
		     int length)
    throws CryptoException;

  /**
   * Encrypts the digest after updating the last buffer of data
   * (may be the only buffer)
   */
  public int encrypt(byte[] inbuff, 
		     int offset, 
		     int inLength,
		     byte[] macBuff, 
		     int macOffset)
    throws CryptoException;
  /**
   * Verifies a MAC for last/only inBuff against
   * the signature in sigBuff. One can use update as with
   * encrypt().
   */
  public boolean verify(byte[] inBuff,
			int inOffset,
			int inLength,
			byte[] macBuff,
			int macOffset,
			int macLength)
      throws CryptoException; 
}
