/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: RSAPrivateKey.java,v 1.3 2001/04/23 23:18:35 yeager Exp $
 */

package jxta.security.publickey;

import jxta.security.util.Description;
import jxta.security.cipher.Key;
import jxta.security.exceptions.CryptoException;

// RSAPublic Key extends key
// Note that public-key refers to the algorithm.
// We have RSAPublicKey and RSAPrivateKey for convenience.
public interface RSAPrivateKey extends Description,Key {
  static final String name = "RSA Private Key";

  // "this" equals key
  public boolean equals(RSAPrivateKey key);
  /**
   *   Given two primes, p and q, and [e,(p-1)(q-1)] = 1, n = pq
   *      Private Key: d = 1/e mod n
   *                   c = m**d mod n, d is exponent, n modulus
   *      We also have methods to set parameters for the Chinese
   *      remainder theorem algorithm
   *         d mod p-1, d mod q-1, 1/q mod p
   */
  public void setPrivateExponent(byte[] exponent, int offset, int length)
    throws CryptoException;
  public void setModulus(byte[] modulus, int offset, int length)
    throws CryptoException;

  // Lengths (modulus is in public key interface)
  public int getPrimePLength();
  public int getPrimeQLength();
  public int getPrimeExponentPLength();
  public int getPrimeExponentQLength();
  public int getCrtCoefficientLength();

  // Additional params for CRT private key algorithm
  public void setPrimeP(byte[] primeP, int offset, int length)
    throws CryptoException;
  public void setPrimeQ(byte[] primeQ, int offset, int length)
    throws CryptoException;
  public void setPrimeExponentP(byte[] primeExponentP, int offset, int length)
    throws CryptoException;
  public void setPrimeExponentQ(byte[] primeExponentQ, int offset, int length)
    throws CryptoException;
  public void setCrtCoefficient(byte[] crtCoefficient, int offset, int length)
    throws CryptoException;

  // accessors
  public int getPrivateExponent(byte[] exponent, int offset);
  public int getModulus(byte[] modulus, int offset);
  public int getPrimeP(byte[] primeP, int offset);
  public int getPrimeQ(byte[] primeQ, int offset);
  public int getPrimeExponentP(byte[] primeExponentP, int offset);
  public int getPrimeExponentQ(byte[] primeExponentQ, int offset);
  public int getCrtCoefficient(byte[] crtCoefficient, int offset);
}


