/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: Util.java,v 1.2 2001/04/02 23:26:12 yeager Exp $
 */

package jxta.security.util;

// Utility functions for security purposes
public class Util {

  public Util() {}

  static final char[] ttable = {
    '0', '1', '2', '3', '4', '5', '6', '7', 
    '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
  };
  /**
   *  Byte array to hex ascii string
   * 
   */
  static public String hexEncode(byte[] buf) 
    throws ArrayIndexOutOfBoundsException 
  {
    int blen = buf.length;
    if (buf == null || blen == 0) return ("");

    // allocate output buffer
    char[] obuf = new char[blen << 1];// 2 hex ascii chars per byte
    int i = 0, j = 0;
    byte b = 0;
    try {
      for (i = 0, j = 0; i < blen; ++i) {
	b = buf[i];
	obuf[j++] = ttable[(b >>> 4) & 0x0F];
	obuf[j++] = ttable[(b & 0x0F)];
      }
    } catch (ArrayIndexOutOfBoundsException aob) {
      System.out.println("Index out of bounds: i =" + i + " j =" + j +
			 " blen =" + blen + " b =" + b +
			 " left =" + ((b >>> 4) & 0X0F) +
			 " right =" +  (b & 0x0F));
      return "";
    }
    return (new String(obuf));
  }
  /**
   *  Byte array to hex ascii string with buffer
   *  offset
   *  
   */
  static public String hexEncode(byte[] buf, short offset) 
    throws ArrayIndexOutOfBoundsException
  {
    int blen = buf.length;
    if (blen == 0 || offset >= blen ) return ("");
    int i = 0, j = 0;
    // allocate output buffer
    char[] obuf = new char[blen << 1];// 2 hex ascii chars per byte
    byte b = 0;
    try {
      for (i = offset, j = 0; i < blen; ++i) {
	b = buf[i];
	obuf[j++] = ttable[(b >>> 4) & 0x0F];
	obuf[j++] = ttable[(b & 0x0F)];
      }
    } catch (ArrayIndexOutOfBoundsException aob) {
      System.out.println("Index out of bounds: i =" + i + " j =" + j +
			 " blen =" + blen + " b =" + b +
			 " left =" + ((b >>> 4) & 0X0F) +
			 " right =" +  (b & 0x0F));
      return "";
    }    
    return (new String(obuf));
  }
  /**
   *  nbytes of byte array to hex ascii string with buffer
   *  offset
   *  
   */
  static public String hexEncode(byte[] buf, short offset, short nbytes) 
    throws ArrayIndexOutOfBoundsException
  {
    int blen = buf.length;
    if (blen == 0 || offset + nbytes > blen) return ("");
    int i = 0, j = 0;
    // allocate output buffer
    char[] obuf = new char[nbytes << 1];// 2 hex ascii chars per byte
    byte b = 0;
    try {
      for (i = offset, j = 0; i < nbytes; ++i) {
	b = buf[i];
	obuf[j++] = ttable[(b >>> 4) & 0x0F];
	obuf[j++] = ttable[(b & 0x0F)];
      }
    } catch (ArrayIndexOutOfBoundsException aob) {
      System.out.println("Index out of bounds: i =" + i + " j =" + j +
			 " blen =" + blen + " b =" + b +
			 " left =" + ((b >>> 4) & 0X0F) +
			 " right =" +  (b & 0x0F));
      return "";
    }    
    return (new String(obuf));
  }
}
