/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: KeyBuilder.java,v 1.5 2001/04/13 20:31:42 yeager Exp $
 */

package jxta.security.impl.cipher;

import jxta.security.cipher.Key;
import jxta.security.exceptions.CryptoException;

import jxta.security.impl.publickey.RSAKey;

// Builds a Key of a given type, length
public class KeyBuilder {

  // constructor
  public KeyBuilder() {}

  // Our known key types
  public static final byte TYPE_RSA_PUBLIC   = 1;
  public static final byte TYPE_RSA_PRIVATE  = 2;
  public static final byte TYPE_RSA = (TYPE_RSA_PUBLIC+TYPE_RSA_PRIVATE);
  public static final byte TYPE_DES          = 4;
  public static final byte TYPE_RC4          = 8;

  // Our known key sizes
  public static final short LENGTH_RC4 = 128;// rfc2246 - Max Exportable
  public static final short LENGTH_RSA_MIN = 384;
  public static final short LENGTH_RSA_512 = 512; // rfc2246 - as above
  
  /** Add new keytype above and add appropriate
   *  if statement below
   */
  public static Key buildKey(byte keyType,
			     short keyLength,
			     boolean notUsed)
    throws CryptoException
  {
    if (keyType == TYPE_RC4) {	// symmetric key
      return new SecretKey(keyType, keyLength);
    } else if ((keyType & TYPE_RSA) != 0) {
      // do both public and private
      return  new RSAKey(TYPE_RSA, keyLength);
    } else
      throw new CryptoException(CryptoException.NO_SUCH_ALGORITHM);
  }
}

