/*
 * $Id: NullCipher.java,v 1.3 2001/04/23 23:18:35 yeager Exp $
 *
 * Implements RC4 encryption/decryption.
 * 
 * $Log  TestRC4
 * Revision 1.3  2001/03/16 byeager
 * adapted to JXTA security
 *
 * $Log: NullCipher.java,v $
 * Revision 1.3  2001/04/23 23:18:35  yeager
 * Finished PublicKey Interface. Changed APIs to use ints rather than shorts.
 * We do not adhere strictly to the javacard.security API.
 * Issue number:
 * Obtained from:
 * Submitted by:
 * Reviewed by:
 *
 * Revision 1.2  2001/04/17 21:56:41  yeager
 * Clean up implementation + set/get RSA key optimization
 * Issue number:
 * Obtained from:
 * Submitted by:
 * Reviewed by:
 *
 * Revision 1.1  2001/03/27 01:16:10  yeager
 * Initial checkin of the jxta security project
 * Issue number:
 * Obtained from:
 * Submitted by:
 * Reviewed by:
 *
 * Revision 1.2  2000/03/31 02:35:13  vgupta
 * Removed those checks from doFinal that are duplicated in update.
 *
 * Revision 1.1  2000/03/29 02:55:17  vgupta
 * Initial revision
 *
 * Note: The RC4 algorithm is taken directly from "Applied Crytography: by
 *       Bruce Schneier.
 *
 */

package jxta.security.impl.cipher;

import jxta.security.cipher.Cipher;
import jxta.security.cipher.Key;
import jxta.security.exceptions.CryptoException;

/**
 * This class does nothing
 */
final public class NullCipher implements jxta.security.cipher.Cipher {
  private static String myname = "NullCipher";

  // Algorithm name
  public String getAlgorithmName() { return myname; }    
   
  // for debugging
  private boolean debug = false;
  /**
   * turn on debugging
   */
  public void setDebug() { debug = true; }
  /**
   * turn off debugging
   */
  public void clearDebug() { debug = false; }

  public NullCipher() {;}

  public byte getAlgorithm() {
    return 0;
  }
  /**
   * Key initialization (not used for RC4 since additional
   * parameters are not required)
   *
   */
  public void init(Key theKey, 
		   byte theMode, 
		   byte[] b, 
		   int off, 
		   int len) 
    throws CryptoException {
      throw new CryptoException(CryptoException.ILLEGAL_USE);
  }

  /**
   * Initializes the cipher S based on the key
   * Crypto code from Bruce Schneier
   */
  public void init(Key theKey, byte theMode) throws CryptoException {
    // Validate the key type and mode
    throw new CryptoException(CryptoException.ILLEGAL_VALUE);
  }
  
  public int update(byte[] inBuf, int inOff, int inLen, 
		      byte[] outBuf, int outOff) 
    throws CryptoException {
      throw new CryptoException(CryptoException.ILLEGAL_USE);
  }
    
  public int doFinal(byte[] inBuf, int inOff, int inLen, 
		       byte[] outBuf, int outOff) throws CryptoException 
  {
    return (0);
  }
}
