/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: NullHash.java,v 1.3 2001/04/23 23:18:36 yeager Exp $
 */

package jxta.security.impl.hash;

import jxta.security.hash.Hash;

// This does nothing and is stored as the hash when no
// hash object is in a profile.
public class NullHash implements jxta.security.hash.Hash {

  // constructor
  public NullHash() {}

  private static String myname = "nullHash";

  // for debugging
  private boolean debug = false;
  /**
   * turn on debugging
   */
  public void setDebug() { debug = true; }
  /**
   * turn off debugging
   */
  public void clearDebug() { debug = false; }

  /**
   *  // Algorithm name
   */
  public String getAlgorithmName() { return myname; }
  public byte getAlgorithm() {
    return ALG_NULL;
  }
  /**
   * No digest used
   */
  public byte getDigestLength() {
    return (byte) 0;
  }
  /**
   * Nothing to reset
   */
  public void reset() {
    return;
  }
  /**
   * Clone ourself?
   */
  public Object clone() {
    return new NullHash();
  }
  /**
   * Nothing to update
   */
  public void update(byte[] buf, int off, int len) 
  {
    return;
  }
  /**
   * Nothing to finally do
   */
  public int doFinal(byte[] inBuf, int inOff,
		     int inLen, byte[] outBuf,
		     int outOff) 
  {
    return 0;
  }
}
