/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: NullRSA.java,v 1.1 2001/05/02 20:47:16 yeager Exp $
 */

package jxta.security.impl.publickey;

import jxta.security.exceptions.CryptoException;

// A does nothing
public class NullRSA implements jxta.security.publickey.PublicKeyAlgorithm {
  /**
   *   // constructor
   */
  public NullRSA() {}

  private static String myname = "NullRSA";

  /**
   *   // Algorithm name
   */
  public String getAlgorithmName() { return myname; }

  // for debugging
  private boolean debug = false;
  /**
   * turn on debugging
   */
  public void setDebug() { debug = true; }
  /**
   * turn off debugging
   */
  public void clearDebug() { debug = false; }
  /*
   * Other do nothing methods
   */
  public void reset() {;}

  // Maximum Data Block on can encrypt
  public int getMaxInputDataBlockLength() { return 0; }
  public int getEncryptionBlockLength() { return 0; }
  // Set public key
  public void setPublicKey() throws CryptoException {
      throw new CryptoException(CryptoException.NOT_IMPLEMENTED);
  }
  public void setPublicKey(Object publickeyData)
    throws CryptoException {
      throw new CryptoException(CryptoException.NOT_IMPLEMENTED);
  }
  public void setPublicKey(byte[] nModulus)
    throws CryptoException {
      throw new CryptoException(CryptoException.NOT_IMPLEMENTED);
  }
  
  // Set the private key
  public void setPrivateKey() throws CryptoException {
      throw new CryptoException(CryptoException.NOT_IMPLEMENTED);
  }
  public void setPrivateKey(Object privatekeyData)
    throws CryptoException {
      throw new CryptoException(CryptoException.NOT_IMPLEMENTED);
  }

  // Get the public and private key objects
  public Object getPublickey() throws CryptoException { return (Object)null; }
  public Object getPrivatekey() throws CryptoException { return (Object)null; }

  /** The Algorithm:
   *     Encrypt | decrypt, encrypt == true | false 
   */
  public byte[] Algorithm(byte[] data, // data
			  int offset,  // offset in byte array
			  int length,  // n bytes 
			  byte type,   // TYPE_RSA_PUBLIC | TYPE_RSA_PRIVATE
			  boolean encrypt) // == true for encryption
    throws CryptoException {
      throw new CryptoException(CryptoException.NOT_IMPLEMENTED);
  }
}
