/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: TestRC4.java,v 1.1 2001/05/10 20:39:04 yeager Exp $
 */

package jxta.security.impl.test;

import jxta.security.cipher.Cipher;
import jxta.security.cipher.Key;
import jxta.security.exceptions.CryptoException;
import jxta.security.crypto.JxtaCrypto;
import jxta.security.util.Util;

import jxta.security.impl.cipher.RC4Cipher;
import jxta.security.impl.cipher.KeyBuilder;
import jxta.security.impl.cipher.SecretKey;
import jxta.security.impl.random.JRandom;
import jxta.security.impl.crypto.JxtaCryptoSuite;

// A general test program for security algorithms we use

public class TestRC4 {
  static final String bonjour = "Bonjour mes amis. Oui c'est une histoire" +
    " que je vous raconte. \nAlors, un peu de patience ... .";

  public static void main(String argv[]) {
    // Create a random key
    JRandom ran = null;
    try {
      ran = new JRandom();
    } catch (CryptoException cex) {
      System.out.println("Init JRandom: " + cex.toString());
      System.exit(1);
    }
    byte[] buzzword = new byte[KeyBuilder.LENGTH_RC4 >>> 3];
    ran.nextBytes(buzzword);

    // Get a suite to do RC4 only    
    JxtaCrypto suite = null;
    try {
      suite = new JxtaCryptoSuite(JxtaCrypto.MEMBER_RC4, 
				  null, (byte)0, (byte)0);
    } catch (CryptoException cex) {
      ;
    }
    Cipher rc4 = suite.getJxtaCipher();
    SecretKey k1 = null;
    System.out.println("\nCrypto test of " + rc4.getAlgorithmName());
    try {
      // build a secret key
      k1 = (SecretKey)KeyBuilder.buildKey(KeyBuilder.TYPE_RC4, 
					  KeyBuilder.LENGTH_RC4, 
					  false);
      k1.setKey(buzzword, 0); // set our test symmetric key
      rc4.init(k1, Cipher.MODE_ENCRYPT); // Initialize RC4 S-boxes.
    } catch (CryptoException cex) {
      System.out.println("Initialization failure 1: " + cex.getMessage());
      System.exit(1);
    }
    // Ok, we encrypt bonjour
    byte[] obuf = new byte[bonjour.length()];
    try {
      rc4.doFinal(bonjour.getBytes(), 0, bonjour.length(), obuf, 0);
    } catch (CryptoException cex) {
      System.out.println("Initialization failure 2: " + cex.getMessage());
      System.exit(1);
    }    

    // OK. Now we pretend we have received the cipher text and decrypt
    // It's in obuf. The key, k1, is still set. We need to init.
    byte[] clearText = new byte[obuf.length];// 
    try {
      rc4.init(k1, Cipher.MODE_DECRYPT);	// Initialize RC4 S-boxes.
      rc4.doFinal(obuf, 0, obuf.length, clearText, 0);
    } catch (CryptoException cex) {
      System.out.println("Initialization failure 2: " + cex.getMessage());
      System.exit(1);
    }    
    String text = new String(clearText);
    System.out.println("\nEncrypted:\n\"" + bonjour + "\"" +
		       "\n[" + obuf.length + " bytes]Sent: \n" + Util.hexEncode(obuf) + 
		       "\nDecrypted to: \n\"" + text + "\"");
    if (text.length() == bonjour.length() && text.equals(bonjour)) {
      System.out.println("\nSuccess!");
    } else {
      System.out.println("\nFailure )-:");
    }
    System.exit(0);
  }
}

