/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * 
 */

/* 
 * Some parts of the following code were taken from Cryptix sources.
 */

/*
Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000 
The Cryptix Foundation Limited. All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

1. Redistributions of source code must retain the copyright notice, 
   this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

THIS SOFTWARE IS PROVIDED BY THE CRYPTIX FOUNDATION LIMITED ``AS IS'' 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

 */

/* $Id: TestRSA.java,v 1.1 2001/05/10 20:39:04 yeager Exp $
 *
 * Copyright (C) 1995-1999 The Cryptix Foundation Limited.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject
 * the terms and conditions of the Cryptix General Licence. You should have
 * received a copy of the Cryptix General Licence along with this library;
 * if not, you can download a copy from http://www.cryptix.org/ .
 */

package jxta.security.impl.test;

import jxta.security.exceptions.CryptoException;
import jxta.security.util.Util;
import jxta.security.util.GetOpt;
import java.lang.IllegalArgumentException;

import jxta.security.impl.publickey.RSAKey;
import jxta.security.impl.publickey.RSA;
import jxta.security.impl.publickey.RSAPublickeyData;
import jxta.security.impl.publickey.RSAPrivatekeyData;
import jxta.security.impl.cipher.KeyBuilder;


// A general test program for security algorithms we use

public class TestRSA {
  static byte[] btext = {
    (byte)'a', (byte)'b', (byte)'c', (byte)'d', (byte)'e', (byte)'f', (byte)'g', (byte)'h', 
    (byte)'i', (byte)'j', (byte)'k', (byte)'l', (byte)'m', (byte)'n', (byte)'o', (byte)'p',
    (byte)'q', (byte)'r', (byte)'s', (byte)'t', (byte)'u', (byte)'v', (byte)'w', (byte)'x', 
    (byte)'y', (byte)'z', (byte)'0', (byte)'1', (byte)'2', (byte)'3', (byte)'4', (byte)'5'
  };

  private static RSAKey rKey = null;
  private static RSA rsa = null;
  private static boolean debug = false, timing = false, keyRestore = false;

  private static long encryptM0, encryptTotal;
  private static long decryptM0, decryptTotal;
  private static long  mKeyTotal = 0;
  private static int maxTimingCnt = 10;
  private static double dMaxTimingCnt = 10.0, dKeyMaxTimingCnt = 10;
  private static int factor = 0;

  public static void main(String argv[]) {
    getOptions(argv);		// get the options
    System.out.println("Testing RSA algorithm, debug = " + debug +
		       " timing = " + timing);

    // get RSA Key object
    try {
      rKey = (RSAKey)KeyBuilder.buildKey(KeyBuilder.TYPE_RSA,
					 KeyBuilder.LENGTH_RSA_512,
					 false);
    }  catch (CryptoException ce) {
      System.out.println("KeyBuilder instance failure");
      System.exit(1);
    }
    
    // Get our RSA algorithm object
    try {
      rsa = new RSA(rKey);
    } catch (CryptoException ce) {
      System.out.println("RSA algorithm instance failure");
      System.exit(1);
    }

    // set our debugging/timing if required
    if (debug) rsa.setDebug();

    System.out.println("Building public and private keys ...");

    int i = 0, maxCount = 1;
    long mKey0 = 0;
    if (timing) {
      maxCount = maxTimingCnt;
      mKey0 = System.currentTimeMillis();
    }
    
    // Create the private and public keys
    for (i = 0; i < maxCount; i++) {
      // reset key parameters (only required for timing loop)
      rsa.reset();		
      try {
	if (debug)
	  System.out.println("Setting Public Key");

	rsa.setPublicKey();

	if (debug)
	  System.out.println("Setting Private Key");

	rsa.setPrivateKey();
      }  catch (CryptoException ce) {
	System.out.println("set Key Error: " + ce.toString());
	System.exit(1);
      }
    }

    // total milliseconds for key generation
    if (timing) mKeyTotal = System.currentTimeMillis() - mKey0;

    System.out.println("Encrypting/decrypting text ...");

    // get our keys for later testing
    RSAPublickeyData rPublicD = null;
    RSAPrivatekeyData rPrivateD = null;
    if (keyRestore) {
      try {
	rPublicD = (RSAPublickeyData)rsa.getPublickey();
	rPrivateD = (RSAPrivatekeyData)rsa.getPrivatekey();
      } catch (CryptoException ce) {
	System.out.println("get Key Error: " + ce.toString());
	System.exit(1);      
      }
    }
    
    // our tests are here
    resetTimers();

    try {
      // Encrypt/decrypt the fixed text
      doit(btext, true);
			   
    } catch (CryptoException cex) {
      System.exit(1);
    }

    // OK. If we timed things, then show the results:
    timingOutput(true);		// Key building was timed

    // reset cipher timers
    resetTimers();

    // test maximum data encrypt/decrypt RSA padding
    int dataMax = rsa.getMaxInputDataBlockLength();

    System.out.println("\nTest Max data - Min RSA Padding");
    byte[] data = new byte[dataMax];

    for (i = 0; i < dataMax; i++) data[i] = (byte)0xCC;	// 10101010
    try {
      // Encrypt/decrypt the fixed text
      doit(data, false);
    } catch (CryptoException cex) {
      System.exit(1);
    }   
    timingOutput(false);	// Key building NOT timed

    if (keyRestore) {
      // Reverse the Test with saved keys
      // 1. reset key parameters
      rsa.reset(); 
      System.out.println("\n Testing restoring saved keys");
      byte[] Y = null;
      try {
	// 2. Restore the private key
	rsa.setPrivateKey((Object)rPrivateD);

	// 3. Encrypt data with private key
	Y = doPrivate(data, true);

      } catch (CryptoException ce) {
	System.out.println("Private Key Error: " + ce.toString());
	System.exit(1);      
      }
      // Public key stuff
      try {
	// 4. Restore public key
	rsa.setPublicKey((Object)rPublicD);

	// 5. Decrypt with public key
	byte[] Z = doPublic(Y, false);

	// success or failure?
	reportResults(Z, data, false);

      } catch  (CryptoException ce) {
	System.out.println("RSA Crypto Error: " + ce.toString());
	System.exit(1);      
      }
    }

    System.exit(0);
  }

  // get options
  private static void getOptions(String argv[]) {
    // get our options
    //   -d           do debugging output
    //   -t           generate timing statistics
    //   -k           test key get/restore
    // The above are mutually exclusive.
    //
    GetOpt options = new GetOpt(argv, "dt:k");
    try {
      int c;
      String multiplier;
      Integer vx;
      while ((c = options.getNextOption()) != -1) {
	switch (c) {
	case 'd': debug = true; timing = false; break;
	case 'k': keyRestore = true; break;
	case 't': 
	  timing = true; debug = false; 
	  multiplier = options.getOptionArg();
	  vx = new Integer(multiplier);
	  factor = vx.intValue();
	  if (factor < 0 || factor > 3) factor = 1;
	  break;
	default: break;
	}
      }
    } catch (IllegalArgumentException iae) {
      debug = timing = false;
      System.out.println("Usage: TestRSA [-d -k -t N]" +
			 "\n       -d   debugging output" +
			 "\n       -k   Test restoring saved keys" +
			 "\n       -t N Time algorithms." +
			 "\n            N = 1 (loop dilator) by default," + 
			 "\n            1 <= n <= 3.");
    }
  }

  public static void timingOutput(boolean keyTimed)
  {
    if (timing) {
      System.out.println("\n***************************************");
      String keyResult = "";
      if (keyTimed) {
	double keyBuildMs = (double)(mKeyTotal)/dKeyMaxTimingCnt;
	keyResult = "\n      Key calculations = " + keyBuildMs;
      } else
	keyResult = "";
      double encryptMs  = (double)(encryptTotal)/dMaxTimingCnt;
      double decryptMs  = (double)(decryptTotal)/dMaxTimingCnt;
      System.out.println("Timing Results in milliseconds:" +
			 keyResult +
			 "\n public Key encryption = " + encryptMs +
			 "\nprivate Key decryption = " + decryptMs +
			 "\n  Loop Cnt = " + maxTimingCnt*factor + " Total ms = " + decryptTotal);
      System.out.println("***************************************");
    }
  }

  public static void resetTimers() {
    encryptM0 = encryptTotal = 0L;
    decryptM0 = decryptTotal = 0L;
  }

  /**
   * Do public key algorithm
   */
  public static byte[] doPublic(byte[] data, boolean encrypt)
    throws CryptoException
  {
    return rsa.Algorithm(data, 
			 0,
			 data.length,
			 KeyBuilder.TYPE_RSA_PUBLIC,
			 encrypt);
  }

  /**
   * Do private key algorithm
   */
  public static byte[] doPrivate(byte[] data, boolean encrypt)
    throws CryptoException
  {
    return rsa.Algorithm(data, 
			 0,
			 data.length,
			 KeyBuilder.TYPE_RSA_PRIVATE,
			 encrypt);
  }

  public static void doit(byte[] data, boolean isText)
    throws CryptoException
  {
    byte[] Y = null, Z = null;

    // let's encrypt our data with the public key
    if (debug)
      System.out.println("Public Key Encryption of " + data.length + " bytes");
    int maxCount;
    if (timing) {
      encryptM0 = System.currentTimeMillis();
      maxCount = maxTimingCnt*factor;
      dMaxTimingCnt = (double)maxCount;
    } else maxCount = 1;
    // encrypting ...
    for (int i = 0; i < maxCount; i++) {
      Y = doPublic(data, true);
    }

    if (timing) {
      encryptTotal = System.currentTimeMillis() - encryptM0;
      if (encryptTotal == 0) encryptTotal = 1;
    }
    // decrypt with the private
    if (debug)
      System.out.println("Private Key Decryption of Y, length = " + Y.length);
 
    if (timing) {
      decryptM0 = System.currentTimeMillis();
      maxCount = maxTimingCnt*factor;
      dMaxTimingCnt = (double)maxCount;    } else maxCount = 1;
    // decrypting
    for (int i = 0; i < maxCount; i++) {
      Z = doPrivate(Y, false);
    }

    if (timing) {
      decryptTotal = System.currentTimeMillis() - decryptM0;
      if (decryptTotal == 0) decryptTotal = 1;
    }
      
    // report the results
    reportResults(Z, data, isText);
  }

  public static void reportResults(byte[] Z, byte[] data, boolean isText)
  {
    // what we encrypted/decrypted
    String str = null;
    if (isText) 
      str = new String(Z);
    else str = Util.hexEncode(Z);

    if (Z.length == data.length && areEqual(Z, data)) {
      System.out.println("Text block encrypt/decrypt successful\n" + str);
    } else {
      System.out.println("Text block encrypt/decrypt NOT successful\n" + 
			 " Z.length = " + Z.length + 
			 "\nZ = " + Util.hexEncode(Z) +
			 " str.length = " + str.length() +
			 "\nstr = " + str);
    }
  }

  static public boolean areEqual(byte[] a, byte[] b) {
    if (a.length != b.length) return false;
    for (int i = 0; i < b.length; i++)
      if (a[i] != b[i]) return false;
    return true;
  }
}

