/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ShellEnv.java,v 1.6 2001/06/21 21:28:41 ddoolin Exp $
 */

package net.jxta.impl.shell;

import java.io.*;
import java.util.*;

/**
 */

public final class ShellEnv {
    private Hashtable table;
    private int random;

    public ShellEnv () {
	table = new Hashtable();
	random = 1;
    }

    public ShellEnv (ShellEnv srcEnv) {
	table = (Hashtable) srcEnv.table.clone();
	random = srcEnv.random;
    }

    public synchronized String createName() {
	return "env" + Integer.toString(random++);
    }

    public synchronized ShellObject add (ShellObject obj) {
        return add ( (String) null, obj );
    }

    public synchronized ShellObject add (String name, ShellObject obj) {
	if (obj == null) return null;
	if (name == null) {
	    name = createName();
	}
	try {
	    Object old = table.put (name, obj);
	    return (ShellObject) old;
	} catch (Exception e) {
	    // This case should not happen. Do nothing
	    e.printStackTrace();
	    return null;
	}
    }

    public synchronized ShellObject get (String name) {
	if (name == null) return null;
	return (ShellObject) table.get (name);
    }

    public synchronized boolean contains (String name) {
      return table.containsKey (name);
    }

    public synchronized boolean contains (ShellObject obj) {
      return table.contains (obj);
    }

    public synchronized boolean rename (String oldName, String newName) {
	if (contains(newName)) {
	    return false;
	}
	if (contains(oldName)) {
	    Object obj = table.remove (oldName);
          try {
	    Object old = table.put (newName, obj);
	    return true;
          } catch (Exception e) {
	    // This case should not happen. Do nothing
	    e.printStackTrace();
	    return false;
	  }
	} else {
	    return false;
	}
    }

    public synchronized boolean remove (String name) {
	return table.remove (name) != null;
    }

    public synchronized Enumeration elements() {
	return table.keys();
    }

    public String toString() {
	return table.toString();
    }

}

