/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: ShellOutputPipe.java,v 1.9 2001/06/14 06:49:05 bondolo Exp $
 */

package net.jxta.impl.shell;

import net.jxta.peergroup.*;
import net.jxta.exception.*;
import net.jxta.pipe.*;
import net.jxta.endpoint.*;

import java.io.*;
import java.util.*;

/**
 * This class implements the default  JXTA Shell OutputPipe.
 * Message received on the pipe are displayed onto the console.
 */

public class ShellOutputPipe implements OutputPipe, Runnable {
    
    private PeerGroup group;
    private Pipe pipes;
    private ShellConsole cons;

    private Thread thread = null;
    private ShellMessageQueue queue;
    
    public ShellOutputPipe(PeerGroup group, ShellConsole cons) {
        this.group = group;
        this.cons = cons;
        queue = new ShellMessageQueue();
        pipes = group.getPipe();
        
        thread = new Thread(this, "ShellOutputPipe-" + cons.getConsoleName());
        thread.start();
    }
    
    public void run() {
        
        Message msg = null;
        InputStream inputStream = null;
        String name = null;
        Enumeration names = null;
        int res = 0;
        byte[] buf = null;
        
        while (true) {
            
            msg = queue.waitForMessage();
            
            if (msg == null) {
                break;
            }
            
            // Get all the chunks of data in the message
            names = msg.getNames();
            for (; names.hasMoreElements();) {
                
                try {
                    name = (String) names.nextElement();
                    inputStream = msg.pop(name);
                    if (inputStream != null) {
                        try {
                            buf = new byte [inputStream.available()];
                            res = inputStream.read(buf);
                            if( res > 0 )
                                cons.write( new String(buf) );
                        } catch (Exception e) {
                            e.printStackTrace();
                            continue;
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    continue;
                }
            }
        }
    }
    
    /**
     * Allocate a new Message for this pipe
     *
     * @return a Message
     */
    public Message createPipeMessage() {
        return pipes.createMessage();
    }
    
    /**
     * send a message
     *
     * @param msg is the Message to be sent.
     */
    public void send(Message msg) {
        queue.push(msg);
    }
    
    public void close() {
        queue.close();
        if (thread != null) {
            thread.interrupt();
        }
    }
}
