/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: cat.java,v 1.16 2001/05/22 23:42:57 lomax Exp $
 */

package net.jxta.impl.shell.bin.cat;

import net.jxta.impl.shell.*;

import java.io.*;
import java.util.*;

import net.jxta.pipe.*;
import net.jxta.document.*;
import net.jxta.protocol.*;
import net.jxta.endpoint.*;


/**
 * @version     $Revision: 1.16 $
 * @since       JXTA 1.0
 */
public class cat extends ShellApp {
    ShellEnv env;
    
    public cat() {
    }
    
    public void stopApp () {
    }
    
    public int startApp (String[] args) {
        String    name = null;
        boolean   pretty = false;
        
        if ((args == null) || (args.length < 1)) {
            println ("error: cat [-p] <var>");
            return ShellApp.appParamError;
        }
        int argsIndex = 0;
        
        if( args[argsIndex].toLowerCase().equals( "-p" ) ) {
            pretty = true;
            argsIndex++;
        }
        
        name = args[argsIndex];
        
        env = getEnv();
        ShellObject obj = env.get (name);
        if (obj == null) {
            println ("cat: cannot access " + name);
            return ShellApp.appMiscError;
        }
        
        Object catobj = obj.getObject();
        
        if( Advertisement.class.isInstance( catobj ) ) {
            try {
                catAdvertisement ((Advertisement) obj.getObject(), pretty );
                return ShellApp.appNoError;
            } catch (Exception e) {
                println( "cat: error printing object. error : " +
			 e.toString());
                e.printStackTrace();
                return ShellApp.appMiscError;
            }
        }
        else if( StructuredDocument.class.isInstance( catobj ) ) {
            
            try {
                catStructuredDocument ((StructuredDocument) obj.getObject());
                return ShellApp.appNoError;
            } catch (Exception e) {
                println( "cat: error printing object. error : " +
			 e.toString() );
                e.printStackTrace();
                return ShellApp.appMiscError;
                
            }
        }
        else if( Message.class.isInstance( catobj ) ) {
            
            try {
                catJxtaMessage ((Message) obj.getObject());
                return ShellApp.appNoError;
            } catch (Exception e) {
                println( "cat: error printing object. error : " +
			 e.toString() );
                e.printStackTrace();
                return ShellApp.appMiscError;
                
            }
        }
	
        else {
            println("cat: cannot read this kind of object");
            return ShellApp.appMiscError;
        }
    }
    
    
    private void catStructuredDocument (StructuredDocument doc) 
      throws Exception {
        OutputStream out = new ByteArrayOutputStream();
        
        doc.sendToStream( out );
        
        print ( out.toString() );
    }
    

    private void catAdvertisement (Advertisement adv,
				   boolean pretty ) throws Exception {
        
        StringWriter    out = new StringWriter();
        MimeMediaType displayAs;
        
        if( pretty )
            displayAs = new MimeMediaType( "text/plain" );
        else
            displayAs = new MimeMediaType( "text/xml" );
        
        StructuredTextDocument doc = (StructuredTextDocument)
	  adv.getDocument( displayAs );
        
        doc.sendToWriter( out );
        
        print ( out.toString() );
    }
    
    private void catJxtaMessage (Message msg) {
      Enumeration names = msg.getNames();
     
      while (names.hasMoreElements()) {
	String name = null;
	try {
	  name = (String) names.nextElement();
	} catch (Exception e) {
	  break;
	}
	catJxtaMessage (msg, name);
      }
      return;
    }
     
    private void catJxtaMessage (Message msg, String name) {
     
      InputStream input = null;
      try {
	input = msg.pop (name);
      } catch (Exception e) {
	println("cat: message has an empty tag body: " + name);
      }
      if (input == null) {
	println("cat: message has an empty tag body: " + name);
	return;
      }
     
      try {

	StructuredDocument doc = 
	  StructuredDocumentFactory.newStructuredDocument (
	  new MimeMediaType ("text/xml"), input);

	println("Tag: " + name);
	println("Body:");
	catStructuredDocument(doc);
     
	// Put the tag body back into the message
	// this is kind of dangerous as we may lost the message
	// but...

	msg.push (name,doc.getStream());
      } catch (Exception e) {
	print ("cat: the message has been corrupted. Reason:");
	println (e.toString());
      }
    }
  
  public void help() {
        println("NAME");
        println("     cat  - concatane and display a Shell object");
        println(" ");
        println("SYNOPSIS");
        println(" ");
        println("     cat [-p] <objectName>");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        println("'cat' is the Shell command that displays on stdout the content ");
        println("of objects stored in environment variables. 'cat' knows");
        println("how to display a limited (but growing) set of JXTA objects");
        println("Advertisement, Message and StructuredDocument)");
        println("If you are not sure, try to cat the object anyway: the");
        println("command will let you know if it can or not display that");
        println("object.");
        println(" ");
        println("OPTIONS");
        println(" ");
	println("    -p Pretty display");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("    JXTA> importfile -f /home/tra/myfile myfile");
        println("    JXTA> cat -p myfile");
        println(" ");
        println("This command imports the file '/home/tra/myfile' into the");
        println("'myfile' environment variable and displays it on stdout");
        println(" ");
        println("SEE ALSO");
    }
}






