/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: env.java,v 1.5 2001/05/22 23:42:57 lomax Exp $
 */

package net.jxta.impl.shell.bin.env;

import net.jxta.impl.shell.*;

import java.util.*;

/**
 * Echo command
 */

public class env extends ShellApp {
    private ShellEnv myEnv;

    public env () {
    }

    private void display (String objName) {
      if (myEnv == null) return;

      ShellObject obj = myEnv.get (objName);
      if (obj == null) return;
      println (objName + " = " + obj.getName() + " (" + obj.getObjectClass().toString() + ")");
    }

    public int startApp (String[] args) {
      myEnv = getEnv();

      Enumeration enum = myEnv.elements();
      while (enum.hasMoreElements()) {
        try {
          String objName = (String) enum.nextElement();
          display (objName);
        } catch (Exception e) {
          return ShellApp.appMiscError;
        }
      }
      return ShellApp.appNoError;
    }

    public void stopApp () {
    }

    public void help() {
      println("NAME");
      println("     env  - display environment variable");
      println(" ");
      println("SYNOPSIS");
      println(" ");
      println("     env");
      println(" ");
      println("DESCRIPTION");
      println(" ");
      println("'env' displays all the defined environment");
      println("variables in the Shell seesion.");
      println(" ");
      println("The following environment variables are defined by default:");
      println(" consin    = Default Console InputPipe");
      println(" consout   = Default Console OutputPipe");
      println(" stdout    = Default OutputPipe");
      println(" stdin     = Default InputPipe");
      println(" Shell     = Root Shell");
      println(" stdgroup  = Default peer group");
      println(" rootgroup = Default NetPeerGroup");
      println(" ");
      println("Shell environement variables are defined as a result of executing");
      println("Shell commands. The '=' operator can be used to assign");
      println("value to a particular variable. For example `myenv = mkmsg` will");
      println("assign a new message object to the 'myenv' environment variable");
      println(" ");
      println("EXAMPLE");
      println(" ");
      println("    JXTA> env");
      println(" ");
      println("This command will display all environment variables");
      println(" ");
      println("SEE ALSO");
      println("     setenv Shell");
    }
}
