/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: exit.java,v 1.9 2001/06/21 21:28:41 ddoolin Exp $
 */


package net.jxta.impl.shell.bin.exit;

import net.jxta.impl.shell.*;
import net.jxta.impl.shell.bin.Shell.*;
import java.util.*;

/**
 * exit command
 */

/* TODO: exit assumes that the current shell that is 
 * running is a Shell sh, and that subshells are also
 * Shells.  If a sub-shell that is not a Shell is run
 * in Shell, exit does not "see" that it is actually 
 * a shell, and kills the entire process.  The example
 * is using an sqlshell.  SEE UPDATE BELOW:
 * 
 * UPDATE:  The previous comment may now be out of date.
 * By adding a variable to the environment to grab the correct 
 * shell process that is running, we are then able to determine
 * whether it has a parent and what that parent may be.
 * After some cleanup, and a few non-breaking commits, most of 
 * these comments and the output logging can be deleted.
 */
public class exit extends ShellApp {

    public exit () {
    }

    public int startApp (String[] args) {

	ShellEnv env = getEnv();
	/* We need to know what shell, exactly, is running. 
	 * Maybe it should be called HCURRENTSHELL... (heh)
	 */
	ShellObject obj = env.get ("SHELL");
	//ShellObject obj = env.get (ShellApp.getThisShell());
	//ShellObject obj = env.get ("sqlshell");

        

	/* This should probably be cast to a ShellApp.
	 * Then, every sub-shell derived from ShellApp
	 * could be handled correctly.  If not, then the 
	 * case where a subshell which is not of Shell 
	 * class may kill the entire process.
	 */
	ShellApp sh = null;
	//Shell sh = null;

	try {
	    sh = (ShellApp) obj.getObject();
	    //sh = (Shell) obj.getObject();


	} catch (Exception e) {
	    println ("exit: cannot exit (app error)");
	    return ShellApp.appMiscError;
	}


	if (sh == null) {
	    println ("exit: cannot exit (null shell)");
	    return ShellApp.appMiscError;
	}
	if (sh.isRootShell()) {
	    // This is the root Shell. Shutdown
            System.out.println("Shutting down root shell");
            System.out.println(env.toString());
	    System.exit(0);
	    return ShellApp.appNoError;
	} else {
	    System.out.println("Shutting down non-root shell");
	    sh.stopApp();
	    return ShellApp.appNoError;
	}
    }

    public void stopApp () {
    }

    public void help() {
      println("NAME");
      println("     exit - exit the Shell");
      println(" ");
      println("SYNOPSIS");
      println(" ");
      println("     exit");
      println(" ");
      println("DESCRIPTION");
      println(" ");
      println("'exit' command is used to exit the Shell");
      println(" ");
      println("SEE ALSO");
      println("    Shell");
    }
}
