/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: join.java,v 1.18 2001/05/29 16:51:36 msrdanovic Exp $
 */

package net.jxta.impl.shell.bin.join;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.*;
import java.net.*;

import net.jxta.impl.shell.*;

import net.jxta.credential.AuthenticationCredential;
import net.jxta.membership.Authenticator;
import net.jxta.membership.Membership;

import net.jxta.protocol.*;
import net.jxta.peergroup.*;
import net.jxta.document.*;
import net.jxta.discovery.*;
import net.jxta.exception.*;

// Should be removed when the Advertisement factory is there
import net.jxta.impl.protocol.*;

/**
 * mkgrp command list the local peer information
 */

public class join extends ShellApp {

    private ShellEnv env;
    private String name = null;
    private String advDoc = null;
    private PeerGroupAdvertisement adv = null;
    private PeerGroupAdvertisement origAdv = null;
    private PeerGroup pg = PeerGroupFactory.newPeerGroup();
    private ShellObject obj = null;
    private PeerGroup parent = null;
    private boolean beRendezvous = false;

    public join() {
    }

    public int startApp(String[] args) {

        env = getEnv();

        int i = 0;
        name = null;
        String credDoc = null;
        StructuredDocument creds = null;

        if (args != null) {
            for (i=0; i < args.length; ++i) {

                if (args[i].equals("-d")) {
                    advDoc = args[++i];
                    continue;
                }

                if (args[i].equals("-c")) {
                    credDoc = args[++i];
                    continue;
                }


                if (args[i].equals("-r")) {
		    beRendezvous = true;
                    continue;
                }

                name = args[i];
            }

        }

        if (advDoc != null) {
	    //we are joining a new group, need to make it first
            // extract the parent peerGroup. In all cases
            // we only creates child peergroup  of the NetPeergroup
            // why, it was thought for now by marketing that hierarchy
            // of peergroup may be too complex. Nothing in the core
            // precluse us from doing it. We just limmiting it
            // within the Shell (need to revisit this at some point)

            obj = env.get("rootgroup");
            if (obj == null) // no root use stdgroup - root only created by 1st child
                obj = env.get("stdgroup");
            parent = (PeerGroup) obj.getObject();

	    /*
	     * create the new Peergroup from the passed doc or create a
	     * clone of the parent. This kind of make sense for now...
	     */

            if (advDoc != null) {
                obj = env.get(advDoc);
                if (obj == null) {
                    println("join: cannot access " + advDoc);
                    return ShellApp.appMiscError;
                }

                try {
                    adv = (PeerGroupAdvertisement) obj.getObject();
                    // save a copy of the original we might publish as a peer if it is
                    // not ours
                    origAdv =adv;
                    name = adv.getName();
		    adv.setIsRendezvous (beRendezvous);

                    try {
                      adv.setKeywords(InetAddress.getLocalHost().getHostName());
                    } catch (Exception e) {}
                    // test if this group already exist
                    //

                    ShellObject obj = env.get("PG@"+name);
                    if (obj != null) {
                        println("A group of this name already exist");
                        return ShellApp.appParamError;
                    }
		    publishAdv (parent, pg, origAdv);
		    
		    pg.init(parent, adv); // just initialize do no start

		    if (beRendezvous) {
			// If the advertisement has not yet been locally
			// published into ADV, publish it.
			adv.setPid (parent.getPeerID().toString());
			addRendezvous (adv, parent.getPeerID().toString());
			publishAdv(parent, pg, adv);
		    }


                } catch (Exception e) {
                    println("join: " + name +
                    " is not a proper PeerGroup advertisement");
                    e.printStackTrace();
                }

                // Create the new frameShell environment variable for that peergroup

                if (env != null) { // There is a need for a root shell
                    // store the new group env in the parent
                    env.add("PG#"+name, new ShellObject("PeerGroup", pg));
                } else {
                    println("lost the parent Shell frame, this is bad !");
                    return ShellApp.appParamError;
                }
            }
        }

	// Do some usefull work

        if (name == null) {
            // list all the PeerGroups Active

            PeerGroup pg = (PeerGroup) env.get("stdgroup").getObject();
            String cname = pg.getAdvertisement().getName();

            for (Enumeration e = env.elements(); e.hasMoreElements();) {
                String group = (String) e.nextElement();
                if (group.indexOf("PG#") != -1)
                    println(" Unjoined Group : " + group.substring(3)); //shift PG#
                if (group.indexOf("PG@") != -1) {
                    print(" Joined Group   : " + group.substring(3)); //shift PG@
                    if (cname.equals(group.substring(3)))
                        println("       (current)"); // mark this as the current group
                    else
                        println(" ");
                }
            }
            return ShellApp.appNoError;
        } else { // OK we have a name let's try it

            ShellObject obj = env.get("PG#"+name);
            if (obj == null) {
                println("The peergroup '" + name +
                "' does not exist, use '-d group#' if you discovered the group,");
                println("or you may already have joined this group.");
                return ShellApp.appParamError;
            }
            PeerGroup npg = (PeerGroup) obj.getObject();
            if (npg == null) {
                println("Sorry this peergroup does not exist. You need to create it");
                return ShellApp.appParamError;
            }


            if( null != credDoc ) {
                obj = env.get( credDoc );
                if ( !StructuredDocument.class.isAssignableFrom(obj.getObjectClass()) ) {
                    println( "Provided Credential was not a Structured Document");
                    println( "Group was joined, but login was not incorrect.");
                    return ShellApp.appMiscError;
                }

                creds = (StructuredDocument) obj.getObject();
            }

            try {
                AuthenticationCredential authCred = new AuthenticationCredential(
                npg,
                null,   // use default authenticator
                creds );
		println("Enter the identity you want to use when joining this peergroup (nobody)");
                Membership membership = (Membership) npg.getMembership();

                Authenticator auth = membership.apply( authCred );

                completeAuth( auth );

                if( !auth.isReadyForJoin() ) {
                    println( "Failure in authentication.");
                    println( "Group was not joined. Does not know how to complete authenticator");
                return ShellApp.appMiscError;
                }

                membership.join( auth );
            }
            catch( Exception all ) {
                println( "Failure in authentication.");
                println( "Group was not joined. Login was incorrect.");
                // all.printStackTrace();
                return ShellApp.appMiscError;
            }


            // Remove and mark the group as active
            // this is done via changing #->@ why this don't ask me?

            env.remove("PG#"+name);
            env.add("PG@"+name, new ShellObject("PeerGroup", npg));

            /**
             * PeerGroup Env Switch, cehck if we have the root peergroup
             */

            obj = env.get("rootgroup");
            if (obj == null) { // add the root group to come back
                obj = env.get("stdgroup");
                PeerGroup pg = (PeerGroup) obj.getObject();
                env.add("rootgroup",  new ShellObject("PeerGroup", pg));
            }

            // ok we can switch to the new group
            env.add("stdgroup",  new ShellObject("PeerGroup", npg));

            return ShellApp.appNoError;
        }

    }

    public void stopApp() {
    }

    private void completeAuth( Authenticator auth ) throws Exception {

        Method [] methods = auth.getClass().getMethods();
        Vector authMethods = new Vector();

        // go through the methods of the auth class and insert them in name sorted
        // order into a vector.
        for( int eachMethod = 0; eachMethod < methods.length; eachMethod++ ) {
            if( methods[eachMethod].getName().startsWith( "setAuth" ) ) {
                if( Modifier.isPublic( methods[eachMethod].getModifiers() ) ) {
                    // sorted insertion.
                    for( int doInsert = 0; doInsert <= authMethods.size(); doInsert++ ) {
                        int insertHere = -1;
                        if( doInsert == authMethods.size() )
                            insertHere = doInsert;
                        else {
                            if( methods[eachMethod].getName().compareTo( ((Method)authMethods.elementAt( doInsert )).getName() ) <= 0 )
                                insertHere = doInsert;
                        }

                        if( -1 != insertHere ) {
                            authMethods.insertElementAt( methods[eachMethod], insertHere );
                            break;
                        }
                    }
                }
            }
        }

        // get input for each of the methods.
        for( int eachAuthMethod = 0; eachAuthMethod < authMethods.size(); eachAuthMethod++ ) {
            Method doingMethod = (Method) authMethods.elementAt( eachAuthMethod );

            String authStepName = doingMethod.getName().substring( 7 );
            print( authStepName + " : " );
            String input = waitForInput();

            Object [] params = { input };

            doingMethod.invoke( auth, params );
        }
    }

    // Add the given PeerId in the list of rendezvous
    private void addRendezvous (PeerGroupAdvertisement adv,
				String peerId) {

        Hashtable sAdvs = null;

        sAdvs = adv.getServiceAdvertisements ();
        if (sAdvs == null) {
            // No service advertisement, very unlikely, but in that
	    // case, we cannot add the rendezvous
	    return;
        }
        // Get the resolver ServiceAdvertisement
        ServiceAdvertisement r = null;
        try {
            r = (ServiceAdvertisement) sAdvs.get ("jxta.service.resolver");
            if (r == null) {
		// No resolver service advertisement. Not much we can do.
		return;
            }
        } catch (Exception e) {
            return;
        }

        Vector p = r.getParams();

        if (p == null) {
	    // Not rendezvous has been set up yet. Create a new Vector
	    p = new Vector();
        }

        String s = null;
        boolean found = false;
        Vector res = (Vector) p.clone();

	// If the param list does not yet contain the given peerid,
	// let's add it.
        for (int i = 0; i < p.size(); ++i) {
            try {
                found = false;
                s = (String) p.elementAt (i);
		if (s.equals (peerId)) {
		    found = true;
		    break;
                }
            } catch (Exception e) {
                continue;
            }
	}
	if (!found) {
	    // Add it
	    res.addElement (peerId);
	}

        // Set adv with the new update service advertisement
        r.setParams (res);
        // Replace this service advertisement
        sAdvs.put ("jxta.service.resolver", r);
        // Replace this list of service advertisement
        adv.setServiceAdvertisements (sAdvs);
        return;
    }

    private void publish(PeerGroup parent,
			 PeerGroup child,
			 PeerGroupAdvertisement pgAdv) {

        //get the Discovery for this group
        //Publish the New Peer in its group discovery

        Discovery discovery;
        try {
            discovery = parent.getDiscovery();
            discovery.publish(pgAdv, Discovery.GROUP);
            // let's publish the peer adv
            if (origAdv != null){
             // we could check if it is indeed ours
             // but it does not hurt to publish it any way
             discovery = child.getDiscovery();
             discovery.publish(origAdv, Discovery.PEER);
            }
        } catch (Exception e) {
            println("could no find the service");
        }
    }


    private void publishAdv (PeerGroup parent,
			     PeerGroup child,
			     PeerGroupAdvertisement pgAdv) {

        //get the Discovery for this group
        //Publish the New Peer in its group discovery

        Discovery discovery;
        try {
            discovery = parent.getDiscovery();
	    discovery.publish(pgAdv, Discovery.GROUP);

	    if (beRendezvous) {
		// Remote publish - optimization
		discovery.remotePublish (pgAdv, Discovery.GROUP);
	    }
        } catch (Exception e) {
            println("publishAdv failed with " + e);
        }
    }


    public int syntaxError() {
        println("Usage: join [-r] [-d <adv>] [-c <credential>] [<name>]");
        return ShellApp.appParamError;
    }

    public void help() {
        println("NAME");
        println("     join - join a peer group");
        println(" ");
        println("SYNOPSIS");
        println(" ");
        println("     join [-d <adv>] [-c <credential>] [<name>]");
        println("     ");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        println("The 'join' command is used to join a peergroup that was created");
        println("via a 'mkpgrp' command or using an advertisement that was previously discovered.");
        println("In order to join a group, the group needs to have been discovered or");
        println("created first.");
        println("If no argument is given, join lists all the existing groups and their");
        println("status (join, unjoined) and the current group on the local peer.");
        println("After a group was joined successfully, the 'PG@<group name>'");
        println("environment variable is created. This variable holds the group info.");
        println("When joining a new peer group, the new peer group is advertised in");
        println("the 'NetPeerGroup'. No hierarchy of peer groups is supported.");
        println("Upon joining the peer group depending on the membership authentication");
	println("required, the user will be asked for the identity he/she wants to have");
	println("in the peer group. An identity is used to assign credential");
	println("to users when accessing peer group ressouces. Each peer group");
	println("can define their own set of identities available in the peer group.");
        println("The identities available for a peer group can be found in the peer");
	println("group advertisement via the 'cat' command in the membership service");
        println("section:");
	println(" ");
        println("   jxta:ServiceAdvertisement :");  
	println("	Name : jxta.service.membership");
	println("	Version : JXTA1.0");
	println("	Keywords : Membership");
	println("	PipeService : ");
	println("	Uri : http:/www.jxta.org/download/jxta.jar");
	println("	Provider : sun.com");
	println("	Security : TBD");
	println("	Code : net.jxta.impl.membership.PaswdMembership");
	println("       Param: nobody:xyrtsts");
	println("       Param: tra:djskjjkj");
	println("  ");
	println("The following shows the two identity 'nobody' and 'tra' allowed in this");
	println("peer group. These identities require a passwd.");
	println(" ");
	println("For example");
	println(" ");
	println("    JXTA> join mygroup");
	println("     identity:tra");
	println("     passwd:XXXXX");
	println("    JXTA>");
        println(" ");
        println("OPTIONS");
        println("");
	println("     [-r] sets the peer as a rendezvous for this group (default is not a");
	println("          rendezvous.");
        println("     [-d adv]  specify a shell variable holding a  peergroup advertisement");
        println("     [-c credential]  specify a credential to join the peer group");
        println("      <name> specify the name for an already created peer group");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("      JXTA>mygroupadv = mkadv -g mygroup");
        println("      JXTA>mkpgrp -d mygroupadv mygroup");
        println("      JXTA>join mygroup");
        println(" ");
        println(" This creates a new peer group which is cloning the services");
        println(" of the parent peer group ('NetPeerGroup'). You can find the services of the");
        println(" current peergroup via the command 'whoami -g'. The new group");
        println(" is given the name 'mygroup'. Before you can do anything with");
        println(" the group you need to join the group via the 'join' command.");
        println(" ");
        println("SEE ALSO");
        println("      mkadv mkpgrp leave chpgrp who");
    }
}
