/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: leave.java,v 1.8 2001/05/22 23:42:58 lomax Exp $
 */

package net.jxta.impl.shell.bin.leave;


import java.util.*;

import net.jxta.membership.Membership;

import net.jxta.protocol.*;
import net.jxta.peergroup.*;
import net.jxta.document.*;

import net.jxta.impl.shell.*;


// Should be removed when the Advertisement factory is there
import net.jxta.impl.protocol.*;

/**
 * mkgrp command list the local peer information
 */

public class leave extends ShellApp {

  private ShellEnv env;
  private String name = null;
  private String advDoc = null;
  private PeerGroupAdvertisement adv = null;
  private PeerGroup pg = PeerGroupFactory.newPeerGroup();
  private ShellObject obj = null;
  private PeerGroup parent = null;

  public leave() {
  }

  public int startApp (String[] args) {

    env = getEnv();
    ShellObject obj;

    if (args != null) 
      return syntaxError();

    obj = env.get("stdgroup");
    PeerGroup pg = (PeerGroup) obj.getObject();
    String name = pg.getAdvertisement().getName();

    obj = env.get("PG@"+ name);
    if (obj == null) {
      println("Sorry we cannot find any current peer group");
      return ShellApp.appParamError;
    }
    PeerGroup npg = (PeerGroup) obj.getObject();
    
    try {
    // Resign our identity.
    
    Membership membership = (Membership) npg.lookupService( "jxta.service.membership" );
    membership.resign();
    }
    catch( Exception all ) {
      println("sorry we could not resign from the group.");
      }
    // Remove and mark the group as unjoined
    // this is done via changing @-># why this don't ask me?

    env.remove("PG@"+name);
    env.add("PG#"+name, new ShellObject("PeerGroup", npg));

    /**
     * PeerGroup Env Switch, cehck if we have the root peergroup
     */

    obj = env.get("rootgroup");
    if (obj == null) { // add the root group to comme back 
      println("sorry we cannot find the rootgroup variable, please define it");
    }
   
    pg = (PeerGroup) obj.getObject();
  
    // ok we can switch to the root group
    env.add("stdgroup",  new ShellObject("PeerGroup", pg));

    return ShellApp.appNoError;
  }

  public void stopApp() {
  }

  public int syntaxError() {
    println ("Usage: leave");
    return ShellApp.appParamError;
  }
  
  public void help() {
      println("NAME");
      println("     leave - leave a peer group");
      println(" ");
      println("SYNOPSIS");
      println(" ");
      println("     leave");
      println(" ");
      println("DESCRIPTION");
      println(" ");
      println("The 'leave' command is used to leave a group that was previously joined");
      println("via a 'join' command. The 'join' command is used to join a peergroup.");
      println("After leaving the group, the Shell 'stdgroup' variable");
      println("is reset to the value of the default 'rootgroup' variable (NetPeerGroup).");
      println("Before a user can use the group again, the user will have to rejoin");
      println("the group via the 'join' command.");
      println(" ");
      println("EXAMPLE");
      println(" ");
      println("      JXTA>mygroupadv = mkadv -g mygroup");
      println("      JXTA>mkpgrp -d mygroupadv mygroup");
      println("      JXTA>join mygroup");
      println("      JXTA>leave");
      println(" ");
      println(" This creates a new peergroup which is cloning the services");
      println(" of the parent peer group (NetPeerGroup). You can find the services of the");
      println(" current peergroup via the command 'whoami -g'. The new group");
      println(" is given the name 'mygroup'. Before you can do anything with");
      println(" the group you need to join the group via the 'join' command.");
      println(" the 'leave' command is used to leave the group.");
      println(" ");
      println("SEE ALSO");
      println("    mkadv mkpgrp join chpgrp ");
  }
}
