/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: man.java,v 1.8 2001/06/17 15:29:14 wiarda Exp $
 */

package net.jxta.impl.shell.bin.man;

import net.jxta.impl.shell.*;


import java.io.*;
import java.util.Enumeration;


public class man extends ShellApp {
    ShellEnv env;
    
    public man() {
    }
    
    public void stopApp() {
    }
    
    public int startApp(String[] args) {
        if (args.length == 0) {
            selfHelp();
            return ShellApp.appNoError;
        }
        if (args.length > 1) {
            println("error: man [command]");
            return ShellApp.appParamError;
        }
        
        String cmd = args[0];
        
        try {
            Class appClass = Class.forName("net.jxta.impl.shell.bin." +
            cmd + "." + cmd);
            ShellApp app = (ShellApp) appClass.newInstance();
            app.setOutputPipe(outputPipe);
            app.setGroup(group);
            app.help();
            return ShellApp.appNoError;
        } catch (Exception e) {
            println("man: cannot find " + cmd);
            return ShellApp.appMiscError;
        }
        
    }
    
    private void selfHelp() {
        println("The 'man' command is the primary manual system for the JXTA Shell.");
        println("The usage of man is:");
        println(" ");
        println("   JXTA> man <commandName>");
        println(" ");
        println("  For instance typing");
        println("   JXTA> man Shell");
        println("         displays man page about the Shell");
        println(" ");
        println("The following is the list of commands available:");
        println(" ");
        
        println(" cat        Display the content of an environment variable");
        println(" chpgrp     Change the default peer group");
        println(" clear      Clear the shell's screen");
        println(" env        Display all environment variables ");
        println(" exit       Exit the Shell ");
        println(" exportfile Export a Shell variable into an external file");
        println(" get        Extract a message tag body from a pipe messsage ");
        println(" grep       Search for matching patterns");
        println(" groups     Discover and list peer groups ");
        println(" history     Use history to recall previous issued commands");
        println(" man        Use man to display man pages about commands");
        println(" importfile Import an external file into a Shell variable");
        println(" join       Join a new peer group");
        println(" leave      Leave a peer group");
        println(" man        On-line man page about commands ");
        println(" mkadv      Make a new pipe or peer group advertisement  ");
        println(" mkmsg      Make a new pipe message");
        println(" mkpgrp     make a new peer group ");
        println(" mkpipe     Make a new pipe ");
        println(" more       Browse or page through a Shell object ");
        println(" peerconfig Peer configuration");
        println(" peers      Discover and list peers ");
        println(" put        Put a new tag body into a pipe message");
        println(" recv       Receive a pipe message");
        println(" rdvstatus  Show connected rendezvous peers");
        println(" search     Search a shared advertisement in a peer group");
        println(" send       Send a pipe message");
        println(" setenv     Set the value of an environment variable");
        println(" share      Share an advertisement in a peer group");
        println(" Shell      Fork a new Shell");
        println(" talk       Instant messaging talk command");
        println(" version    Display version and build info");
        println(" wc         Count lines, words, and characters");
        println(" who        Display member identity in a peer group");
        println(" whoami     Display peer and peer group information");
        
        
        
        println(" ");
    }
    
    public void help() {
        println("NAME");
        println("     man - get man page info about a command");
        println(" ");
        println("SYNOPSIS");
        println("     man  <command>");
        println(" ");
        println("DESCRIPTION");
        println("'man' is an on-line help command that displays information about a");
        println("specific Shell command.");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("       JXTA> man mkpgrp");
        println(" ");
        println("This will display information about the mkpgrp command");
        println(" ");
        println("SEE ALSO");
    }
    
}




