/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: mkadv.java,v 1.12 2001/05/22 23:42:59 lomax Exp $
 */

package net.jxta.impl.shell.bin.mkadv;

import net.jxta.impl.shell.*;

import java.lang.reflect.InvocationTargetException;
import java.io.*;
import java.util.*;

import net.jxta.pipe.*;
import net.jxta.peergroup.*;
import net.jxta.id.*;
import net.jxta.document.*;
import net.jxta.protocol.*;

import net.jxta.impl.pipe.*;
import net.jxta.impl.protocol.*;

/**
 * mkadv Shell command
 *
 *
 */

public class mkadv extends ShellApp {
    private ShellEnv env;
    private String name = null;
    private String advDoc = null;
    private String var = null;

    public mkadv() {
    }

    public void stopApp () {
    }

    private int syntaxError () {
      println ("Usage: mkadv -g|p [[-t type] [-d doc] [name]]");
      return ShellApp.appParamError;
    }

    public int startApp (String[] args) {

      if ((args == null) || (args.length < 1)) {
	return syntaxError();
      }

      env = getEnv();

      int i = 0;
      name = null;

      for (i=1; i < args.length; ++i) {
	if (args[i].equals("-t")) {
	  println("PipeService type is not supported yet");
	  return syntaxError();
	} else if (args[i].equals("-d")) {
	  advDoc = args[++i];
	}
	else
	  name = args[i];
      }

	// Parse parameters
        // PipeService or group ?
	if (args[0].equals ("-p")) {
	  // This is a PipeService Advertisement
	  return createPipeAdv (args);
	} else { 
	  if (args[0].equals ("-g")) { // we create a group
	    return createPGroupAdv(args);
	  } else {
	    return syntaxError();
	  }
	}
    }
    

    private int createPGroupAdv (String[] args) {

	PeerGroupAdvertisement adv = null;

	if (advDoc != null) {
	    ShellObject obj = env.get (advDoc);
	    if (obj == null) {
		println ("mkadv: cannot access " + advDoc);
		return ShellApp.appMiscError;
	    }

	    try {
                StructuredTextDocument doc = null;
		doc = (StructuredTextDocument) obj.getObject();
		adv = (PeerGroupAdvertisement) 
              AdvertisementFactory.newAdvertisement(doc); // should use the factory
	    } catch (Exception e) {
		println ("mkadv: " + advDoc +
			 " is not a proper PeerGroup advertisement");
		return ShellApp.appMiscError;
	    }
	} else {

	  if (name == null) {
	    println("missing argument need to give a name to the group");
	    return syntaxError();
	  }

	  // Building a PeerGroup Advertisement that is a clone
	  // of the parent
	  
	  PeerGroup parent = (PeerGroup) env.get("stdgroup").getObject();
      
      try {
      adv = (PeerGroupAdvertisement) 
	AdvertisementFactory.newAdvertisement( 
		   PeerGroupAdvertisement.getAdvertisementType() );
      }
      catch( InvocationTargetException failed ) {
	    println("Cannot create a new PeerGroup Advertisement. Cause : " +
		    failed.getTargetException().toString() );
	    return ShellApp.appMiscError;
      }
      
      adv.setPid ("");
      adv.setGid ("");
      adv.setName (name);
      adv.setKeywords(null);
      adv.setServiceAdvertisements(
	  parent.getAdvertisement().getServiceAdvertisements());
      adv.setEndpointAdvertisements(new Hashtable());
      adv.setApp(parent.getAdvertisement().getApp());
      adv.setGroupImpl(parent.getAdvertisement().getGroupImpl());
	}
	    
	if (adv == null) {
	    println("Cannot create a new PeerGroup Advertisement");
	    return ShellApp.appMiscError;
	}

        ShellObject old = env.add(getReturnVariable(),
		  new ShellObject ("PeerGroup Advertisement", adv));

        return ShellApp.appNoError;
    }


    private int createPipeAdv (String[] args) {

	PipeAdvertisement adv = null;

	if (advDoc != null) {
	    ShellObject obj = env.get (advDoc);
	    if (obj == null) {
		println ("mkadv: cannot access " + advDoc);
		return ShellApp.appMiscError;
	    }

	    try {
                StructuredTextDocument doc = null;
		doc = (StructuredTextDocument) obj.getObject();
		adv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
					  doc);
	    } catch (Exception e) {
		println ("mkadv: " + 
			 advDoc + " is not a proper pipe advertisement");
		e.printStackTrace();
		return ShellApp.appMiscError;
	    }
	} else {
	  try {
 	    ShellObject obj = env.get("stdgroup");
        // extract the advertisement
        PeerGroup group = (PeerGroup) obj.getObject();

	    adv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
                     PipeAdvertisement.getAdvertisementType());
	    adv.setPipeID( new PipeID(group.getID()) );
	    adv.setName(name);
	  } catch (Exception e) {
	    e.printStackTrace();
	    return ShellApp.appMiscError;
	  }
	}
	    

        // Right now, the pipe type is not implemented. Just ignore
	// XXX: to be changed when PipeService's type is implemented
	// lomax@jxta.org

	if (adv == null) {
	    println("Cannot create a new PipeService Advertisement");
	    return ShellApp.appMiscError;
	}

        ShellObject old = env.add(getReturnVariable(),
				  new ShellObject ("PipeService Advertisement", adv));

        return ShellApp.appNoError;
    }

  public void help() {
    println("NAME");
      println("     mkadv - make an advertisement");
      println(" ");
      println("SYNOPSIS");
      println("     mkadv -g|p [[-t <pipetype>] [-d <doc>] [<name>]]");
      println(" ");
      println("DESCRIPTION");
      println(" ");
      println("'mkadv' creates a peer group or pipe advertisement from a document");
      println("stored in a shell environment variable. A specific type can be");
      println("given for the pipe (TBD). A symbolic name can be associated with");
      println("the advertisement. This name can be used to search for the");
      println("advertisement. A name must be supplied when creating a new");
      println("group advertisement. This is the name of the peer group.");
      println(" ");
      println("OPTIONS");
      println(" ");
      println("  -p is used to create a pipe advertisement.");
      println("  -t is only valid for pipe and define the type of pipes (TBD)");
      println("  -g option is used to create a new peer group advertisement");
      println("     if no document is given a clone of the current peergroup");
      println("     is created. The new peer group will have the same services");
      println("     and endpoints that the current group. if a doc argument is");
      println("     supplied, the document must be a peer group advertisement and");
      println("     is used to create the new peer group.");
      println("  -d specifies a Shell environment variable that contains a structured that hold an advertisement");
      println("     document that holds an advertisement");
      println(" ");
      println("EXAMPLE");
      println(" ");
      println("      JXTA>importfile -f saveadv groupadv"); 
      println("      JXTA>mygroupadv = mkadv -g -d groupadv");
      println("      JXTA>mkpgrp -d mygroupadv mygroup");
      println(" ");
      println(" This creates a new peergroup from an advertisement that was created");
      println(" from a StructuredDocument that was stored in the Shell environment");
      println(" variable 'groupadv'.You can check the services of the");
      println(" new peergroup after it has been created via the command 'whoami -g'.");
      println(" The new group is given the name 'mygroup'. Before you can do anything");
      println(" with the group, you need to join the group via the 'join' command.");
      println(" ");
      println("SEE ALSO");
      println("     mkpipe mkpgrp join whoami");
  }
}
