/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: mkpgrp.java,v 1.14 2001/05/22 23:42:59 lomax Exp $
 */

package net.jxta.impl.shell.bin.mkpgrp;


import java.util.*;
import java.net.*;

import net.jxta.impl.shell.*;

import net.jxta.protocol.*;
import net.jxta.peergroup.*;
import net.jxta.document.*;
import net.jxta.discovery.*;
import net.jxta.exception.*;

import java.lang.reflect.InvocationTargetException;

/**
 * mkgrp command list the local peer information
 */

public class mkpgrp extends ShellApp {
    
    private ShellEnv env;
    private String name = null;
    private String advDoc = null;
    private boolean pubFlag = true;
    private PeerGroupAdvertisement adv = null;
    private PeerGroup pg = PeerGroupFactory.newPeerGroup();
    private ShellObject obj = null;
    private PeerGroup parent = null;
    
    public mkpgrp() {
    }
    
    public int startApp(String[] args) {
        
        // parse the argument
        if ((args == null) || (args.length < 1)) {
            println("You need to give the name of group at least");
            return syntaxError();
        }
        
        env = getEnv();
        
        int i = 0;
        name = null;
        
        for (i=0; i < args.length; ++i) {
            
            if (args[i].equals("-d")) {
                advDoc = args[++i];
                continue;
            }
            if (args[i].equals("-p")) {
                pubFlag = false;
                continue;
            }
            if (args[i].equals("-m")) {
                println("Membership service  not implemented yet");
                return ShellApp.appMiscError;
            }
            
            // pickup the name
            
            name = args[i];
        }
        
        if (name == null && advDoc == null )
            // we need to have a name for this peergroup
            return syntaxError();
        
        
        // extract the parent peerGroup. In all cases
        // we only creates child peergroup  of the NetPeergroup
        // why, it was thought for now by marketing that hierarchy
        // of peergroup may be too complex. Nothing in the core
        // precluse us from doing it. We just limmiting it
        // within the Shell (need to revisit this at some point)
        
        obj = env.get("rootgroup");
        if (obj == null) // not root so get the stdgroup by default
            obj = env.get("stdgroup");
        
        parent = (PeerGroup) obj.getObject();
        
    /*
     * create the new Peergroup from the passed doc or create a
     * clone of the parent. This kind of make sense for now...
     */
        
        if (advDoc != null) {
            obj = env.get(advDoc);
            if (obj == null) {
                println("mkpgrp: cannot access " + advDoc);
                return ShellApp.appMiscError;
            }
            
            try {
                if( StructuredTextDocument.class.isAssignableFrom( obj.getObjectClass() ) )
                    adv = (PeerGroupAdvertisement)
                    AdvertisementFactory.newAdvertisement(
                    (TextElement) obj.getObject() );
                else
                    adv = (PeerGroupAdvertisement) obj.getObject();
                
                name = adv.getName();
                
                if (exist(name)) {
                    println("a group of this name already exists");
                    return ShellApp.appMiscError;
                }
                
                pg.init(parent, adv); // just initialiaze do no start
                publish(parent, pg, adv);
            } catch (Exception e) {
                println("mkpgrp: " + name +
                " is not a proper PeerGroup advertisement");
                e.printStackTrace();
            }
            
        } else {
            
            if (exist(name)) {
                println("a group of this name already exists");
                return ShellApp.appMiscError;
            }
            
            // create a clone of the parent we did not get an advertisment
            // Building a PeerGroup Advertisement that is a clone
            // of the parent
            
            PeerGroupAdvertisement adv = null;
            try {
                adv = (PeerGroupAdvertisement) AdvertisementFactory.newAdvertisement( PeerGroupAdvertisement.getAdvertisementType() );
            }
            catch( InvocationTargetException failed ) {
                println("Cannot create a new PeerGroup Advertisement. Cause : " + failed.getTargetException().toString() );
                return ShellApp.appMiscError;
            }
            
            adv.setPid("");
            adv.setGid("");
            adv.setName(name);
            adv.setIsRendezvous(true);
            try {
                adv.setKeywords(InetAddress.getLocalHost().getHostName());
            } catch (Exception e) {}
            
	    // Remove the list of the rendezvous from the parent group, and
	    // replace it with our own peerid.
	    Hashtable sAdvs = parent.getAdvertisement().getServiceAdvertisements();
	    // Get the resolver ServiceAdvertisement
	    ServiceAdvertisement r = null;
	    try {
		r = (ServiceAdvertisement) sAdvs.get ("jxta.service.resolver");
		Vector p = new Vector();
		p.addElement (parent.getPeerID().toString());
		r.setParams (p);
		sAdvs.put ("jxta.service.resolver", r);
	    } catch (Exception e) {
	    }
	    adv.setServiceAdvertisements(sAdvs);
            adv.setEndpointAdvertisements(new Hashtable());
            adv.setApp(parent.getAdvertisement().getApp());
            adv.setGroupImpl(parent.getAdvertisement().getGroupImpl());
            
            // initialize but to no start the application
            // this is done by the join command
            
            try {
                pg.init(parent, adv);
            } catch (PeerGroupException e) {
                return ShellApp.appMiscError;
            }
            publish(parent, pg, adv);
        }
        
        // Create the new frameShell environment variable for that peergroup
        
        if (env != null) { // There is a need for a root shell
            
            // store the new group env in the root env
            env.add("PG#"+name, new ShellObject("PeerGroup", pg));
            return ShellApp.appNoError;
        } else {
            println("lost the parent Shell frame, this is bad !");
            return ShellApp.appParamError;
        }
    }
    
    // publish the new peergroup
    
    private void publish(PeerGroup parent, PeerGroup child,
    PeerGroupAdvertisement adv) {
        
        //get the Discovery for this group
        //Publish the New Peer Group in the rootpeergroup NetPeerGroup
        
        Discovery discovery;
        try {
            discovery = parent.getDiscovery();
            discovery.publish(adv, Discovery.GROUP);
            if ( pubFlag )discovery.remotePublish(adv, Discovery.GROUP);
        } catch (Exception e) {
            println("could no find the service");
        }
    }
    
    
    private boolean exist(String name) {
        ShellObject obj = env.get("PG#"+name);
        if (obj == null) {
            obj = env.get("PG@"+name);
            if (obj == null)
                return false;
            else
                return true;
        } else
            return true;
    }
    
    public void stopApp() {
    }
    
    public int syntaxError() {
        println("Usage: mkpgrp [-d doc] [-p] [-m service] name");
        return ShellApp.appParamError;
    }
    
    public void help() {
        println("NAME");
        println("     mkpgrp - create a new peer group");
        println(" ");
        println("SYNOPSIS");
        println(" ");
        println("     mkpgrp [-d <doc>] [-p ] [-m <service>]  <name>");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        println("mkgrp creates a new peer group using the supplied peer group advertisement");
        println("If no advertisement is provided. The command will create a clone");
        println("of the NetPeerGroup peer group with the name specified.");
        println("The command 'mkadv -g' is used to create a peer group advertisment");
        println("The environment variable 'PG#<group name>' is created to store the");
        println("the new group info. The new peer group is advertised in the");
        println("NetPeerGroup. All peer groups are created in the NetPeerGroup which");
        println("acts like a world peer group.Every peer group can be found in this");
        println("group. By default mkpgrp remotely publishes the newly created group,");
        println("to skip remote publish specify -p.");
        println(" ");
        println("OPTIONS");
        println("");
        println("     [-d doc]    specify an environment variable that contain a");
        println("                 peergroup advertisment");
        println("     [-m service] specify a membership service");
        println("     [-p] do not remotely publish");
        println("     name        specify the name for the peergroup");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("          JXTA>mygroupadv = mkadv -g");
        println("          JXTA>mypgrp = mkpgrp -d mygroupadv mygroup");
        println(" ");
        println(" This creates a new peergroup which is cloning the services");
        println(" of the parent peer group. You can find the services of the");
        println(" current peergroup via the command 'whoami -g'. The new group");
        println(" is given the name 'mygroup'. Before you can do anything with");
        println(" the group you need to join the group via the 'join' command.");
        println(" ");
        println("SEE ALSO");
        println("    whoami join leave peers chpgrp mkadv");
    }
}
