/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: more.java,v 1.9 2001/06/14 07:17:55 bondolo Exp $
 */

package net.jxta.impl.shell.bin.more;

import net.jxta.impl.shell.*;

import java.io.*;
import java.util.*;

import net.jxta.pipe.*;
import net.jxta.document.*;
import net.jxta.protocol.*;


/**
 *  More Nothing really new here
 * @version     $Revision: 1.9 $
 * @since       JXTA 1.0
 */

public class more extends ShellApp {
    
    private ShellEnv env;
    private int count = 0;
    
    public more() {
    }
    
    public void stopApp() {
    }
    
    public int startApp(String[] args) {
        
        try {
            String name = null;
            env = getEnv();
            
            if ( (null == args) || (args.length > 1) ) {
                println("Synopsis :");
                println("more <var>");
                return ShellApp.appParamError;
            }
            
            if ( args.length == 1 ) {
                name = args[0];
                
                ShellObject obj = env.get(name);
                if (obj == null) {
                    println("more: cannot access " + name);
                    return ShellApp.appMiscError;
                }
                
                try {
                    moreAdvertisement((Advertisement) obj.getObject());
                    return ShellApp.appNoError;
                } catch (Exception e) {
                    // This is not an Advertisement
                }
                
                try {
                    moreStructuredDocument((StructuredDocument) obj.getObject());
                    return ShellApp.appNoError;
                } catch (Exception e) {
                    
                }
                
                println("more: cannot read this kind of object");
                return ShellApp.appMiscError;
            } else {
                moreStdin();
            }
            
            return ShellApp.appNoError;
            
        } catch (Exception ex) {
            ex.printStackTrace();
            return ShellApp.appNoError;
        }
    }
    
    
    private void moreStructuredDocument(StructuredDocument doc) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            doc.sendToStream( out );
        }
        catch( IOException ignored ) {
        }
        
        pagein( out.toString() );
    }
    
    private void moreAdvertisement(Advertisement adv) {
        StringWriter out = new StringWriter();
        try {
            StructuredTextDocument doc = (StructuredTextDocument)
            adv.getDocument( new MimeMediaType( "text/plain" ) );
            doc.sendToWriter( out );
        }
        catch( Exception ignored ) {
        }
        
        pagein(out.toString());
    }
    
    private void moreStdin() {
        
        try {
            do {
                String entry = waitForInput() ;
                if (entry == null)
                    break;
                pagein(entry);
            }
            while( true );
        } catch (Exception ex) {
        }
    }
    
    private void pagein(String data) {
        
        StringTokenizer tokens = new StringTokenizer(data,"\n");
        String line = null;
        
        try {
            while (tokens.hasMoreElements()) {
                while (count < MAX_CONSOLE) {
                    println(tokens.nextToken());
                    count++;
                }
                // reset the counter since we exited
                count = 0;
                println("-----More-----");
                String entry = consWaitForInput();
                if (entry.equals("q"))  return;
            }
        } catch (Exception ex) { // This is ok leave before the for loop
        }
    }
    
    public void help() {
        println("NAME");
        println("     more  - page through a Shell object");
        println(" ");
        println("SYNOPSIS");
        println(" ");
        println("     more [<doc>]");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        println("'more' is a Shell command that pages through the content ");
        println("of an object stored in environment variable. The name of the");
        println("is provided as an argument. If no argument is supplied, the");
        println("command takes its inputs from the 'stdin' shell pipe.");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("    JXTA> cat myfile | more");
        println(" ");
        println("This command pipes the content of the variable 'myfile' into the");
        println("'more' command and displays it to the Shell console.");
        println(" ");
        println("SEE ALSO");
    }
    
    private final static int MAX_CONSOLE = 20;
}
