/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: peers.java,v 1.32 2001/05/22 23:43:00 lomax Exp $
 */

package net.jxta.impl.shell.bin.peers;

import java.io.IOException;
import java.net.*;

import net.jxta.impl.shell.*;
import net.jxta.protocol.*;
import net.jxta.endpoint.*;
import net.jxta.discovery.*;
import net.jxta.peergroup.*;
import net.jxta.exception.*;
import net.jxta.pipe.*;


import java.util.*;

/**
 * peers command lists peers known locally, and discovers peers
 *
 * @version
 * @since       JXTA 1.0
 */
public class peers extends ShellApp  {
    private ShellEnv env;
    private static int i=0;
    private Discovery discovery=null;
    private int threshold=10;
    private String pid=null;
    private String attr=null;
    private String val=null;
    private boolean nflag=false;
    private boolean rflag=false;
    private boolean lflag=false;
    private boolean aflag=false;
    private boolean vflag=false;
    private boolean pflag=false;

    public peers() {
    }

    public void stopApp () {
    }

    public int startApp (String[] args) {

       env = getEnv();
       discovery = group.getDiscovery();

       if (args == null) {
        return(getLocal(null,null));
       }

       GetOpt getopt = new GetOpt(args, "rflp:n:a:v:");

       int c;
       try {
            while ((c = getopt.getNextOption()) != -1) {
            switch(c) {
                case 'p': pid=getopt.getOptionArg();
                          pflag=true;
                          break;

                case 'r': rflag=true;
                          break;
                case 'l': lflag=true;
                          break;

                case 'f': try{
                          discovery.flushAdvertisements(null,
                                               Discovery.PEER);
                          discovery.publish (group.getAdvertisement(),
                                               Discovery.PEER);
                          return ShellApp.appNoError;
                          } catch (IOException e){}

                case 'n': nflag=true;
                          threshold=Integer.parseInt(getopt.getOptionArg());
                          break;

                case 'a': aflag=true;
                          attr=getopt.getOptionArg();
                          break;

                case 'v': vflag=true;
                          val=getopt.getOptionArg();
                          if ( (val.length()==1) && (val.indexOf("*") !=-1) ){
                              //let's stop here
                           println("* is not allowed, you must specify at least one char");
                           return ShellApp.appNoError;
                          }
                          break;
            default:
                     println("Error: peers this should have never happened");

            }
            }

            // both values are required
            if ( ((attr==null) || (val==null)) && (aflag || vflag) ) {
                println("You must specify both attribute, and value");
                println("defaulting to generic retrieval");
                attr=null;
                val=null;
            }

            if (rflag || pflag ) {
                return(discover(pid,attr,val));
            } else {
                return(getLocal(attr,val));
            }
        }
        catch(Exception ex) {
            println("Illegal option");
            shortHelp();
        }
     return ShellApp.appNoError;
    }

    private int discover (String address,  String attr, String val ) {

         discovery.getRemoteAdvertisements(
                  address,discovery.PEER, attr, val,threshold);
        println ("peer discovery message sent");

     return ShellApp.appNoError;
    }


    private int getLocal(String attr, String val) {

         Enumeration res;

         try {
           res = discovery.getLocalAdvertisements(discovery.PEER, attr, val);
         } catch (Exception e) {
             println("nothing stored");
             return ShellApp.appNoError;
         }

         int j=0;
         if (res == null) return ShellApp.appNoError;
          while (res.hasMoreElements()) {
            PeerGroupAdvertisement peer=(PeerGroupAdvertisement)res.nextElement();

            if (peer.getEndpointAdvertisements() == null) {
             try {
              discovery.flushAdvertisements(peer.getPid(),discovery.PEER);
             } catch (IOException e ) {
             }
             continue;
            }

            if (peer.getEndpointAdvertisements().size() == 0) {
             // let's zap it
             try {
              discovery.flushAdvertisements(peer.getPid(),discovery.PEER);
             } catch (IOException e ) {
             }
             continue;
            }
            EndpointAdvertisement endp = (EndpointAdvertisement)
            (peer.getEndpointAdvertisements().elements()).nextElement();
            env.add ("peer"+j,new ShellObject ("stored local peer advertisement", peer));

            String hex="";
            try {
               URL url = new URL(peer.getPid().toString());
               hex = Integer.toHexString((url.getHost()).hashCode());
            } catch (Exception e ) {}

	    String name = peer.getPeerName();
	    if (name.equals("")) {
	      name = peer.getKeywords();
	      if (name.equals("")) {
		name = "NetPeerGroup";
	      }
	    }

            if (lflag) {
             println("peer"+j+": ID = " + hex +" name = " +
		     name + " addr = " + endp.getEndpointAddress());
            } else {
             println("peer"+j+ ": name = " + name);
            }

            j++;
            if (nflag  && j == threshold) break;
           }
     return ShellApp.appNoError;
    }


    public void shortHelp () {
      println("NAME");
      println("     peers - discover peers ");
      println(" ");
      println("SYNOPSIS");
      println("     peers [-p peerid name attribute] ");
      println("           [-n n] limit the number of responses to n from a single peer");
      println("           [-r] discovers peers using propagate");
      println("           [-l] displays peer id as a hex string");
      println("           [-a] specify Attribute name to limit discovery to");
      println("           [-v] specify Attribute value to limit discovery to. wild card is allowed");
      println("           [-f] flush peer advertisements");
    }

    public void help() {
      shortHelp();
      println(" ");
      println("DESCRIPTION");
      println(" ");
      println("use \"peers\" to discover other peers within a peer group or at a");
      println("specified peer location. Running \"peers\" command with no options lists");
      println("only the peers already known by the peer (cached). The '-r' option is used");
      println("to send a propagate request to find new peers.");
      println("peers stores results in the local cache, and inserts ");
      println("advertisement(s) into the environment, using the default");
      println("naming: peerX where X is a growing integer number.");
      println(" ");
      println("OPTIONS");
      println("-p peerid");
      println("     discovers peers at a given peer location");
      println("-r");
      println("     discovers peers using remote propagation");
      println("-l");
      println("     displays peer id as a hex string");
      println("-a");
      println("     specify Attribute name to limit discovery to");
      println("-v");
      println("     specify Attribute value to limit discovery to");
      println("-n");
      println("     limit the number of responses to n from a single peer");
      println("-f");
      println("     flush peer advertisements");
      println(" ");
      println("EXAMPLE");
      println(" ");
      println("    JXTA>peers -r");
      println("or");
      println("    JXTA>peers -r -aName -vluxor* ");
      println(" ");
      println("SEE ALSO");
      println("    whoami chpgrp join groups");


    }
}
