/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: search.java,v 1.19 2001/05/22 23:43:00 lomax Exp $
 */

package net.jxta.impl.shell.bin.search;

import java.io.IOException;

import net.jxta.impl.shell.*;
import net.jxta.protocol.*;
import net.jxta.endpoint.*;
import net.jxta.discovery.*;
import net.jxta.peergroup.*;
import net.jxta.exception.*;
import net.jxta.document.Advertisement;
import net.jxta.pipe.*;


import java.util.*;

/**
 * search command lists jxta advertisement known locally, and
 * discovers remote ones.
 *
 * @version
 * @since       JXTA 1.0
 */
public class search extends ShellApp  {
    private ShellEnv env;
    private static int i=0;
    private Discovery discovery=null;
    private int threshold=10;
    private String pid=null;
    private String attr=null;
    private String val=null;
    private boolean nflag=false;
    private boolean rflag=false;
    private boolean aflag=false;
    private boolean vflag=false;
    private boolean pflag=false;
    private boolean sflag=false;


    public search() {
    }

    public void stopApp () {
    }

    public int startApp (String[] args) {

       env = getEnv();
       discovery = group.getDiscovery();

       if (args == null) {
        return(getLocal(null,null));
       }

       GetOpt getopt = new GetOpt(args, "rfp:n:a:v:");

       int c;
       try {
            while ((c = getopt.getNextOption()) != -1) {
            switch(c) {
                case 'p': pid=getopt.getOptionArg();
                          pflag=true;
                          break;

                case 'r': rflag=true;
                          break;

                case 'f': try{
                          discovery.flushAdvertisements(null,
                                               Discovery.ADV);
                          return ShellApp.appNoError;
                          } catch (IOException e){}

                case 'n': nflag=true;
                          threshold=Integer.parseInt(getopt.getOptionArg());
                          break;

                case 'a': aflag=true;
                          sflag=true;
                          attr=getopt.getOptionArg();
                          break;

                case 'v': vflag=true;
                          sflag=true;
                          val=getopt.getOptionArg();
                          if ( (val.length()==1) && (val.indexOf("*") !=-1) ){
                              //let's stop here
                           println("* is not allowed, you must specify at least one char");
                           return ShellApp.appNoError;
                          }
                          break;
            default:
                     println("Error: peers this should have never happened");

            }
            }

            // both values are required
            if ( ((attr==null) || (val==null)) && (aflag || vflag) ) {
                println("You must specify both attribute, and value");
                println("defaulting to generic retrieval");
                attr=null;
                val=null;
                sflag=false;
            }

            if (rflag || pflag ) {
                return(discover(pid,attr,val));
            } else {
                return(getLocal(attr,val));
            }
        }
        catch(Exception ex) {
            println("Illegal option");
            shortHelp();
        }
     return ShellApp.appNoError;
    }

    private int discover (String address,  String attr, String val ) {

         discovery.getRemoteAdvertisements(
                  address,discovery.ADV, attr, val,threshold);
        println ("JXTA Advertisement search message sent");

     return ShellApp.appNoError;
    }


    private int getLocal(String attr, String val) {

         Enumeration res;

         try {
           res = discovery.getLocalAdvertisements(discovery.ADV, attr, val);
         } catch (Exception e) {
             println("nothing stored");
             return ShellApp.appNoError;
         }

         int j=0;

          if (res==null) return ShellApp.appNoError;
          while (res.hasMoreElements()) {
            Advertisement adv = (Advertisement) res.nextElement();
            String msg = "";
            if (sflag){
              msg= "(Search criteria: Attribute=\""+ attr+"\" Value=\""+val+"\")";
            }
             env.add ("adv"+j,new ShellObject ("local jxta advertisement"+msg, adv));
             println("JXTA Advertisement adv"+j+" "+ msg);
            j++;
            if (nflag  && j == threshold) break;
           }
     return ShellApp.appNoError;
    }


    public void shortHelp () {
      println("NAME");
      println("     search - discover jxta advertisements ");
      println(" ");
      println("SYNOPSIS");
      println("    search [-p peerid name attribute] ");
      println("           [-n n] limit the number of responses to n from a single peer");
      println("           [-r] discovers advertisements using propagate");
      println("           [-a] specify Attribute name to limit search to");
      println("           [-v] specify Attribute value to limit search to. wild card is allowed");
      println("           [-f] flush jxta advertisements");
      println(" ");
    }

    public void help() {
      shortHelp();
      println("DESCRIPTION");
      println(" ");
      println("use \"search\" to search for advertisements within a peer group or at a");
      println("specified peer location. Running \"search\" command with no options lists");
      println("only the advertisements already known by the peer (cached). The '-r' option is used");
      println("to send a propagate request to find new advertisements.");
      println("search stores results in the local cache, and inserts ");
      println("advertisement(s) into the environment, using the default");
      println("naming: advX where X is a growing integer number.");
      println(" ");
      println("OPTIONS");
      println("-p peerid");
      println("     searchs for advertisements at a given peer location");
      println(" ");
      println("-r");
      println("     search advertisements using remote propagation");
      println("-a");
      println("     specify Attribute name to limit search to");
      println("-v");
      println("     specify Attribute value to limit discovery to");
      println("-n");
      println("     limit the number of responses to n from a single peer");
      println("-f");
      println("     flush jxta advertisements");
      println(" ");
      println("EXAMPLE");
      println(" ");
      println("    JXTA>search -r");
      println(" ");
      println("    search for new advertisements in the current peer group");
      println(" ");
      println("SEE ALSO");
      println("    whoami chpgrp join groups");


    }
}
