/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: sql.java,v 1.3 2001/06/21 21:28:41 ddoolin Exp $
 */

package net.jxta.impl.shell.bin.sql;

import java.io.*;
import net.jxta.impl.shell.*;

/**
 * The sql class provides client access to a database
 * base server.  
 *
 * NOTE: This initial implementation is not very clever, 
 * nor very extensible.  It does, however, raise several 
 * issues that may occur in other commands.  For example,
 * it might be more interesting to run an "sql subshell"
 * instead of a command command handler in the currently
 * running shell.  Or the sql command could induce modal
 * behavior on the shell, where shell commands behavior
 * depends on the current shell mode.  Another issue that
 * is raised here is how command line options for commands 
 * are handled.  Currently options are spread between the
 * Shell code and the GetOpts class.  Short of rewriting 
 * Getopts, this class has to do further command line 
 * processing. 
 *
 * @version $Revision: 1.3 $
 * @since       JXTA 1.0
 */

public class sql extends ShellApp {

    String url;
    MysqlDBHandler dbh = null;

    public sql() {
    }

    public void stopApp () {

	dbh.closeAllConnections();
	System.out.println("Closed db connection");
    }

    public int startApp (String [] args) {


	/*  protected var in ShellApp, why does this not work? */
	System.out.println("Current command line: " + currentcmdline);

	/* Since the previous does not work, cat everything in 
	 * args back together, and rewrite the getopts.  Because 
	 * the initial tokenizing was the java StringTokenizer
	 * default, adding a space to the front of every element
	 * in the args array functionally recreates the the command
	 * line.  kludgy
	 */
	StringBuffer sb = new StringBuffer(1024);
	for (int i=0; i<args.length; i++) {
	    sb.append(" " + args[i]);
	}



	/* Nasty: we gotta find where -c is in the option string 
	 * because everything after that will be SQL and sent in toto
	 * to the jdbc for processing.  
	 */
	String sqlcommand = findSQLCommand(sb.toString());
        if (sqlcommand != null) {
	    System.out.println("SQL command: " + sqlcommand);
	}

	/* After slicing out the sql substring, we gotta
	 * process the rest of the commands. 
	 * The argument to the 'c' option we just ignore.
	 *
	 * For now, assume that all we have is an sql command.
	 * Implementing other commands, like being able to set
	 * user names, etc will require splitting the db handler
	 * class into 2 classes: 1 class for handler parameters
	 * and urls etc., and another class to actually make the 
	 * connections.  Actually, paremeter handling could go into 
	 * an abstract class that the mysqlhandler could subclass.
         */
	GetOpt opts = new GetOpt(args,"c:");
	int i = 0;
	while (opts.getNextOption() != -1) {
	    System.out.println("Option argument: " + opts.getOptionArg());
	}


	/* Now we have a small problem.  If there is no database 
	 * connection open, we need to open one.  If there is a 
	 * connection (or more) we should use that instead, but 
	 * somehow need to either 1. make dbh persistent, or 2.
	 * modify the shell to store the connection(list) and 
	 * modify the dbh constructor to accept a connection(list).
	 * For now, kludge it.
	 */
	MysqlDBHandler dbh = getDBConnection();

	/* Ok, so what will follow next is assuredly one humongous design
	 * flaw.  The running commentary will point out each defect in 
	 * turn, discuss alternatives, and (hopefully) indicate some 
	 * way of rendering the current idiotic code into something 
	 * clean and elegant.    And as it turns out, nothing further
	 * gets done until quote or string handling in the commandline 
	 * parser works.  grrr...
	 */
	dbh.useDatabase("jxtasearch");
	dbh.makeSQLCommand(sqlcommand);

         return ShellApp.appNoError;
    }


    /* FIXME: Rewrite all of the connection code to be
     * database independent.  
     */
    private MysqlDBHandler getDBConnection() {
	MysqlDBHandler dbh = new MysqlDBHandler();
	dbh.setDBUser("jxtasearch");
	dbh.setDBHost("localhost");
	dbh.setDBPort("3306");
	dbh.setDBPass("gonesilent");
	dbh.connect();
	return dbh;
    }





  public void help() {
      println("NAME");
      println("     sql - issue an SQL command (not implemented)");
      println(" ");
      println("SYNOPSIS");
      println("     sql <sql_command>");
      println(" ");
      println("DESCRIPTION");
      println(" ");
      println("Issue an SQL command. (not yet implemented)");
      println("");
      println("EXAMPLE");
      println(" ");
      println("    JXTA>sql -d mydb;");
      println(" ");
      println("This example directs the current database connection"); 
      println("to use the database named mydb"); 
      println(" ");
  }


    /* The -c option indicates that everything after is
     * an SQL command that should be sent directly to 
     * the jdbc driver for processing.  
     */
    private String findSQLCommand(String options) {
     
	int index = options.indexOf("-c");
	
	/* Check to see if there is actually a "-c" 
	 * in the command line.
	 */
	if (index == -1) {
	    return null;
	}

	String sqlstring = options.substring(index+2);

	System.out.println("-c index: " + index);
	return sqlstring;
    }

}


