/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: sqlshell.java,v 1.2 2001/06/21 21:23:10 ddoolin Exp $
 */

package net.jxta.impl.shell.bin.sqlshell;

import java.io.*;
import java.util.*;

import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.id.*;
import net.jxta.peergroup.*;
import net.jxta.pipe.*;
import net.jxta.protocol.*;

import net.jxta.impl.shell.*;

import net.jxta.impl.shell.bin.history.HistoryQueue;


/**
 * This class will probably get moved into its own project.
 * For now, it is staying in the shell app to help fix a 
 * couple of design problems with shell that need to 
 * be fixed anyway.
 */
public class sqlshell extends ShellApp implements  Runnable {
    
    static public final String ThisShell = "sqlshell";
    static public final String PromptString = "JXTA-SQL> ";
    static public final String History = "History";
    static public final String ParentShell = "parentShell";
    
    static private int shellInstance = 1;
    
    private boolean gotMyOwnFrame = false;

    private ShellConsole cons = null;
    
    private Thread thread = null;
    
    /**
     *  If true then this shell is a sub-shell of a another shell.
     **/
    private boolean gotParent = false;
    
    /**
     *  The shell environment of our parent shell.
     **/
    private ShellEnv parentEnv = null;
    
    private String shellRootEnvName = null;
    
    private BufferedReader scriptReader = null;
    
    private String  pipecmd = null;

    private boolean printPrompt = true; 
    
    public boolean isRootShell() {
        return !gotParent;
    }

    public void init(PeerGroup thisGroup, Advertisement a) {
        super.init( thisGroup, a );
    }


    
    public int startApp(String[] args) {

        super.startApp( args );

	/* handleOptions should probably throw exceptions */
	if (handleOptions(args) != 0) { 
            return ShellApp.appMiscError;
	}        

	handleEnvironment();

	/* This has 2 different returns, depending on whats 
	 * going on in the shell.  Confusing.
	 */
	return handleFrame();

    }


    private int handleOptions(String [] args) {

        try {
            GetOpt getopt = new GetOpt(args, "sf:");
            
            int c;
            while ((c = getopt.getNextOption()) != -1) {
                switch(c) {
                    case 'f':
	/* FIXME: This should throw a FileNotFoundException
	 * or something similarly descriptive.
	 */        
                        if( !initScriptFile( getopt.getOptionArg() ) ) {
                            println("SQL Shell: cannot access " + args[1]);
                            return ShellApp.appMiscError;
                        }
                        break;
                        
                    case 's':
                        gotMyOwnFrame = true;
                        break;
                        
                    default:
                        println(" SQL Shell : Error -- this should have never happened");
                }
            }
        }        catch(Exception ex) {
            println("Illegal option");
            println("usage : Shell [-f filename] [-s] ");
        }

	return 0;
    }



    private void handleEnvironment(){

        if (null != getEnv()) {
            // This is a child Shell.

	    System.out.println("Child (sql) shell");
            
            gotParent = true;
            
            // Recover the parent env, and duplicate it
            parentEnv = getEnv();
            
            setEnv( new ShellEnv(parentEnv) );
            
            // Store this Shell into the parent's environment
            shellRootEnvName = parentEnv.createName();
            parentEnv.add(shellRootEnvName, 
                new ShellObject("Child SQL Shell", this));
            
            // and store our parent into our environment
            ShellObject old = parentEnv.get(ThisShell);
            if (old != null) {
                getEnv().add(ParentShell, old);
            }
        } else {
            // There is no Parent Shell

	    System.out.println("Top level (sql) shell");
            
            gotMyOwnFrame = true;
            setEnv( new ShellEnv() );
            
            ShellObject old = getEnv().add("stdgroup",
            new ShellObject("Default Group" , group));
        }
        
        // Store this Shell into the environment.  
	/* This code has been superceded by the following to 
	 * allow other types of shells to be accessed from 
	 * commands.  If it turns out to be completely 
	 * superfluous, it should be deleted.
	 */
        getEnv().add(ThisShell, new ShellObject("SQL Shell", this));

	/* Hardwiring the SHELL environment variable here allows
	 * us to retrieve it from exit (or wherever), when we need
	 * to get the current instance of the shell.
	 * TODO: Move this into ShellApp somehow.  Every shell
	 * will need to do something like this, so it should be 
	 * in the base class.
	 */
        getEnv().add("SHELL", new ShellObject("SQL Shell", this));
    }


    private int handleFrame() {

        if (gotMyOwnFrame) {
            String myName = "JXTA SQL Shell - " + shellInstance++;
            
            cons = new ShellConsole( myName, 30, 80);
            
            // Create the default InputPipe
            ShellInputPipe defaultInputPipe = new ShellInputPipe(group, cons);
            
            getEnv().add("stdin",
            new ShellObject("Default InputPipe", defaultInputPipe));
            
            getEnv().add("consin",
            new ShellObject("Default Console InputPipe",
            defaultInputPipe));
            
            setInputPipe(defaultInputPipe);
            setInputConsPipe(defaultInputPipe);
            
            // Create the default OutputPipe
            ShellOutputPipe defaultOutputPipe = new ShellOutputPipe(group, cons);
            
            getEnv().add("stdout",
            new ShellObject("Default OutputPipe", defaultOutputPipe));
            
            getEnv().add("consout",
            new ShellObject("Default Console OutputPipe", defaultOutputPipe));
            
            setOutputPipe(defaultOutputPipe);
            setOutputConsPipe(defaultOutputPipe);
            
            // start the shell on its own thread.
            thread = new Thread(this, myName );
            thread.start();
            return ShellApp.appSpawned;
        }
        else {
            // a child shell just runs until exit.
            if (pipecmd != null)
                processCmd(pipecmd);
            else
                runShell();
            
            return ShellApp.appNoError;
        }

    }

    
    public void stopApp() {

        // Remove itself from the parent ShellEnv (GC)
        if (parentEnv != null) {
            parentEnv.remove(shellRootEnvName);
        }
        
        // Destroy ourself
        if (gotMyOwnFrame) {
            cons.destroy();
        }
        
        // Interrupt our parsing thread.
        if (thread != null)
            thread.interrupt();
        
        super.stopApp();
    }
    
    public void run() {
        if( null != pipecmd )
            startApp( new String[0] );
        else
            runShell();
    }
    
    private void initPipe( ShellEnv penv, String cmd ) {
        String myName = "JXTA SQL Shell - " + shellInstance++;
        
        setEnv(penv);
        pipecmd = cmd;
        thread = new Thread(this, myName );
        thread.start();
    }

    
    /**
     * This method implements the default input stream (keyboard).
     */
    private void runShell() {
        
        if (scriptReader == null) {
            consprintln("=====================================");
            consprintln("=== Welcome to the JXTA SQL Shell ===");
            consprintln("=====================================");
            info();
        }
        
        // check if there is a .jshrc file
        startupFile();
        
        while (true) {
            String cmd = null;
            
            if (stopped) break;
            
            try {
                if (scriptReader == null ) {
                    if( printPrompt)  consprint(PromptString);
                    cmd = waitForInput();
                } else {
                    cmd = scriptReader.readLine();
                }
                if (cmd == null) {
                    return;
                }
                
            } catch (IOException e) {
                System.err.println("SQL Shell is reconnecting to console");
                // This shell has lost its standard InputPipe. Try
                // to reconnect to the special keyboard InputPipe.
                setInputPipe( consin );
            }
            
            processMultipleCmd(cmd);
        }
    }
    
    private boolean initScriptFile(String fn) {
        try {
            scriptReader = new BufferedReader(new FileReader(fn));
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    
    private void startupFile() {
        try {
            BufferedReader scriptReader = new BufferedReader(new FileReader
		(".jsqlshrc"));
            if (scriptReader == null) return; //nothing to do
            
            String cmd = scriptReader.readLine();
            
            while(cmd != null) {
                processMultipleCmd(cmd);
                cmd = scriptReader.readLine();
            }
            scriptReader.close();
        } catch (Exception e) {
        }
    }

    
    /**
     *  Process the <cmd;cmd>* commands
     *
     *  FIXME 20010611 bondolo@jxta.org does not handle quoting in any form.
     *
     *  @param cmd the command string.
     */
    private void processMultipleCmd(String cmd) {

        StringTokenizer tokens = new StringTokenizer(cmd, ";");
        HistoryQueue queue = getHistoryQueue();
        
	// make sure this is set - some commands may override it
        printPrompt = true;
        if( queue != null) queue.addCommand(cmd);        
        while (tokens.hasMoreElements())
            processPipeCmd(tokens.nextToken());
    }

    /** 
    *  Returns the HistoryQueue that holds the cmds in a history list
    *  If the queue does not exist, it is created.
    * @return HistoryQueue object used to retrieve commands
    */
    private HistoryQueue getHistoryQueue(){
	ShellObject obj = getEnv().get(History);
        Object object;
        HistoryQueue queue = null; 

        if(obj != null){
            object = obj.getObject();
            if( HistoryQueue.class.isInstance( object )){
                queue = (HistoryQueue)object;
	    }
	}
        else if( cons != null){
            String exclude[]={ cons.getCursorDownName(),
			       cons.getCursorUpName() };
           
	    // if the cons returns null for either of those two
	    // methods, it is not running in window mode - history is
            // not enabled in that mode
            if( exclude[0] != null && exclude[1] != null) {
                queue = new HistoryQueue(exclude);
                getEnv().add(History, new ShellObject("History", queue));
	    }
	}
        return queue;
    }
    
    /**
     *  Process the <cmd | cmd>* commands
     *
     *  FIXME 20010611 bondolo@jxta.org does not handle quoting in any form.
     *
     *  @param cmd the command string.
     */
    private void processPipeCmd(String cmd) {
        
        Vector cmds = new Vector();
        
        StringTokenizer tokens = new StringTokenizer(cmd, "|");
        
        while (tokens.hasMoreElements())
            cmds.addElement(tokens.nextToken());
        
        // at the beginning start with stdin and stdout
        
        PeerGroup pg =(PeerGroup) getEnv().get("stdgroup").getObject();
        InputPipe stdin =(InputPipe) getEnv().get("stdin").getObject();
        OutputPipe stdout = (OutputPipe) getEnv().get("stdout").getObject();
        
        // these are for building the pipeline
        
        InputPipe pipein = null;
        OutputPipe pipeout = null;
        InputPipe lastin = stdin;
        Thread willDependOn = null;
        
        // The first and last command in the pipe needs to be treated separatly
        
        for (int i=0; i < cmds.size()-1; i++) {
            /*
             * create Shell cmd pipe to link the two
             */
            
            PipeAdvertisement padv = null;
            
            try {
                padv = (PipeAdvertisement)
                AdvertisementFactory.newAdvertisement(
                PipeAdvertisement.getAdvertisementType() );
                padv.setPipeID( new PipeID( pg.getID() ) );
                
                pipein = pipes.createInputPipe(padv);
                pipeout = pipes.createOutputPipe(padv,Pipe.NonBlocking, -1);
            } catch (Exception ex) {
            }
            
            /*
             * create the environment by cloning the parent. I think
             * this is right (will see)
             */
            
            ShellEnv pipeenv = new ShellEnv(getEnv());
            
            /*
             * Set the pipeline for the first command and let it go
             * only stdout since stdin is the right one
             */
            
            pipeenv.add("stdout", new ShellObject("Default OutputPipe", pipeout));
            
            pipeenv.add("stdin", new ShellObject("Default InputPipe", lastin));
            
            pipeenv.add("stdgroup", new ShellObject("Default Peer Group", pg));
            
            /*
             * create a new Shell process to run this pipe command
             */
            
            sqlshell pipeShell = 
                (sqlshell) loadApp( null, "sqlshell", pipeenv );
            
            pipeShell.dependsOn = willDependOn;
            pipeShell.initPipe( pipeenv, (String) cmds.elementAt(i));
            willDependOn = pipeShell.thread;
            
            /*
             * update last in pipe for the next command
             */
            
            lastin = pipein;
        }
        
        /*
         * Set the pipeline for the first command and let it go
         * only stdout since stdin is the right one
         */
        
        getEnv().add("stdout", new ShellObject("Default OutputPipe", stdout));
        
        ShellObject oldin = getEnv().get( "stdin" );
        
        getEnv().add("stdin", new ShellObject("Default InputPipe", lastin));
        
        dependsOn = willDependOn;
        
        processCmd((String) cmds.lastElement());
        
        dependsOn = null;
        
        // restore the original stdin
        getEnv().add( "stdin", oldin );
    }
    
    /**
     * Each command on the command line needs to be 
     * processed individually.  This method does not return
     * anything but may set  values of variables 
     * declared in the abstract superclass ShellApp
     * (e.g., currentcmdline).
     */
    /* FIXME: Quoting needs to be handled here. 
     */
    private void processCmd(String cmd) {

	/* The entire command line may need to be available 
	 * in one of the commands in bin.  Since those commands
	 * subclass ShellApp, the protected variable 
	 * currentcmdline can be set here and accessed by any
	 * of those classes later.  This technique doesn't seem
	 * to work, but may be superfluous anyway once quoting 
	 * is properly implemented.
	 */
	currentcmdline = new String(cmd);
	System.out.println("Command from processCmd: " + cmd);


	/* FIXME: This next block of code needs to go away. 
	 * Soon.
         */
	// Is there an "=" in the string ?
	/* Why is this being done here?  If it to set an 
	 * environment variable, should that not be done 
	 * using "set" or "export" commands...?
	 * What am I missing here....?
	 */
	int index = cmd.indexOf("=");
        String var = null;
	if (index > 0) {
	    // There is a variable assignement. Get the name
	    // of the variable to use.
	    var = cmd.substring(0, index).trim();
            
	    // Remove the left operand and operator from the string
	    cmd = cmd.substring(index + 1);
	}
	// echo the command if the echo enviroment variable is defined
	if( getEnv().contains( "echo" ) )
	consprintln( cmd );



        /* Snip the name of the command from the front of the 
	 * command string.  The command lives in the bin directory.
	 * If there is a metacharacter in this string, the loadApp()
	 * method will deal with it.
	 */
	String app = getAppName(cmd);
	if (app == null) {
	    return;
	}

	/* The assumption we make in this call is that there is 
	 * something for StringTokenizer to tokenize.  
	 */
	String [] args = oldCommandParser(cmd);
        
	/* This var thing is a just a single string. If it
	 * is in fact an environment variable, perhaps it should
	 * be set and used that way.
	 */
        ShellApp appl = loadApp( var, app, getEnv() );
        
        if( null != appl )
            run( appl, args );
    }


    
    private String [] newCommandParser(String cmd) {

	/* Find the first space, grab a substring from there
	 * to the end of the string.  This gets rid of the 
	 * command name.
	 */

	/* Scan the string for occurrences of '-'.  The 
	 * number of these will be the number of command
	 * line options, either short or long.
	 */

	/* Make a string [] array of args from '-' options. */

	/* For each of these arrays, tokenize on "" */

	return null;
    }


    /* If there are no tokens, the null string will 
     * make the calling function return.
     */
    private String getAppName(String cmd) {
	String appname;
	StringTokenizer st = new StringTokenizer(cmd);
	try {
	    appname = st.nextToken();
	} catch (Exception e) {
	    appname = null;
	}
	return appname;
    }

    private String [] oldCommandParser(String cmd) {

	/* Delimits on space, tab, form feed, newline, carriage
	 * return: " \t\f\n\r" . Not good enough.  StringTokenizer
	 * is brain dead for what we need to do for quoting. But 
	 * use this until a quoting parser works. 
	 */
	StringTokenizer tokens = new StringTokenizer(cmd);
	int size = tokens.countTokens();

	if (size < 1) {
 	   return null;
	}

	String[] args = new String [size - 1];        
	/* Advance past the command name, because that 
	 * is already snipped out.
	 */
	tokens.nextToken();

	/* The command name is not in the args array. */
	int index = 0;
	try {
	    while (tokens.hasMoreElements()) {
		args[index] = tokens.nextToken();
		index++;  // more maintainable
	    }
	} catch (Exception e) {
	    return null;
	}

	return args;
    }



 
    /* FIXME: If var is an environment variable, should it not be 
     * set as such, then called from the particular app that needs 
     * it?  var is only used in one place at the end of this method.
     */
    /*
     * FIXME: Why is this method so large?  It looks like it is 
     * doing at least 2 different things, maybe more.  Can it not
     * be refactored?
     */    
    private ShellApp loadApp( String var, String appName, ShellEnv env ) {
        ShellApp app = null;
        
        // "clear" is an internal command; just handle it here, nothing
        // to load.

	if(cons != null){
	    if (appName.equals("clear")) {
		cons.clear();
		return null;
	    }
	    else if (appName.equals(cons.getCursorUpName())) {
		HistoryQueue queue = getHistoryQueue();
		if( queue != null){
		    cons.setCommandLine(null,queue.getNextCommand());
		}
		printPrompt = false;
		return null;
	    }
	    else if (appName.equals(cons.getCursorDownName())) {
		HistoryQueue queue = getHistoryQueue();
		if( queue != null){
		    cons.setCommandLine(null,queue.getPreviousCommand());
		}
		printPrompt = false;
		return null;
	    }
	    else if(appName.startsWith("!")){
		try{
		    int number =
                        Integer.valueOf(appName.substring(1)).intValue();
		    HistoryQueue queue = getHistoryQueue();
		    if( queue != null){
			queue.removeLastCommand();
			cons.setCommandLine(PromptString,
					    queue.getCommand(number));
			printPrompt = false;
			return null;
		    }                   
		}
		catch(Exception iox){
		    // was not a history command, let the remainder of
                    // the method handle the command
		    /* If there are no commands starting with 
		     * '!', then why not catch the exception,
		     * tell the user that there is nothing like this
		     * in the history, and return a null?
		     */
		}
	    }
	}

        printPrompt = true;

        try {

            // get the leaf name of the command.
            int periodAt = appName.lastIndexOf( '.' );
            String appLeafName = (0 != periodAt) ?
		appName.substring( periodAt + 1) : appName;
            
            // build the class name string
            String appClassName = "net.jxta.impl.shell.bin." +
		appName +  "." + appLeafName;
            
            // find the class
            Class appClass = Class.forName( appClassName );
            app = (ShellApp) appClass.newInstance();
            
            // Set up the default environment and pipes
            
            app.setEnv( env );
            
            ShellObject obj = env.get( "stdgroup" );
            app.setGroup( (PeerGroup) obj.getObject() );
            
            app.dependsOn = dependsOn;
            
            app.setInputConsPipe( getInputConsPipe() );
            
            app.setOutputConsPipe( getOutputConsPipe() );
            
            obj = env.get( "stdin" );
            app.setInputPipe( (InputPipe) obj.getObject() );
            
            obj = env.get( "stdout" );
            app.setOutputPipe( (OutputPipe) obj.getObject() );
            
            // Set the variable name for the return value
            
            app.setReturnVariable(var);
        } catch (Throwable e) {
            println("command not found '" + appName + "'");
            //println (e.toString());
            //e.printStackTrace();
            //println(" ");
            return null;
        }
        
        app.init( getGroup(), null );
        
        return app;
    }
    
    private void run( ShellApp app, String[] args ) {
        try {
            // start the app
            
            int result = app.startApp(args);
            if( ( ShellApp.appNoError != result )
            && (ShellApp.appSpawned != result) ) {
                //println( "# '" + appName + "' returned error code : " + result );
            }
        } catch (Throwable e) {
            println("Exception in command." );
            StringWriter    theStackTrace = new StringWriter();
            e.printStackTrace( new PrintWriter( theStackTrace ) );
            print( theStackTrace.toString() );
            println(" ");
        }
    }
    
    public void help() {
        println("NAME");
        println("     sqlhell  - JXTA SQL Shell command interpreter");
        println(" ");
        println("SYNOPSIS");
        println(" ");
        println("     sqlshell ");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        info();
        println(" ");
        println(" ");
        println(" ");
        println("SEE ALSO");
    }
    
    
    public void info() {
        println(" ");
        println("The JXTA SQL Shell provides an interactive environment to the JXTA    ");
        println("To exit the Shell, use the 'exit' command, or Control-D.");
    }
    
}
