/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: talk.java,v 1.24 2001/05/22 23:43:01 lomax Exp $
 */


package net.jxta.impl.shell.bin.talk;

import net.jxta.impl.shell.*;

import java.io.*;
import java.util.*;

import net.jxta.exception.*;
import net.jxta.discovery.*;
import net.jxta.endpoint.*;
import net.jxta.pipe.*;
import net.jxta.impl.pipe.*;
import net.jxta.peergroup.*;
import net.jxta.document.*;
import net.jxta.id.*;
import net.jxta.protocol.*;

/**
 * talk Shell command: send and receive message from other users.
 *
 */

public class talk extends ShellApp implements Runnable {

    private Discovery discovery=null;

    private ShellEnv env = null;
    private PipeAdvertisement userAdv = null;
    private static final int WaitingTime = 2000;
    private static final int  MAXRETRIES = 5;
    public static final String TalkNameTag = "JxtaTalkUserName";
    public static final String TalkIDTag = "JxtaTalkPipeID";
    private static final String EnvName = "talkd";
    private String userName = null;
    private static final String SenderName = "JxtaTalkSenderName";
    private static final String SenderMessage = "JxtaTalkSenderMessage";
    private Thread thread = null;

    public talk() {
    }

    public void stopApp() {
    }

    private int syntaxError() {

        println("Usage: talk -register userName ");
        println("       talk -login userName ");
        println("       talk -logout userName ");
        println("       talk [-u user] userName ");
        println("       talk -search ");

        return ShellApp.appParamError;
    }

    public int startApp(String[] args) {

        if ((args == null) || (args.length == 0)) {
            return syntaxError();
        }

        env = getEnv();
        discovery= group.getDiscovery();

        if (args[0].equals("-register")) {
            return registerNewUser(args);
        }

        if (args[0].equals("-login")) {
            return login(args);
        }

        if (args[0].equals("-logout")) {
            return logout(args);
        }
        if (args[0].equals("-search")) {
            return findUsers();
        }

        return sendMessage(args);
    }


    private boolean deamonRunning(String name) {

        ShellObject obj = env.get(EnvName + "." + name);
        if (obj == null) {
            return false;
        } else {
            return true;
        }
    }

    private int login(String[] args) {

        if (args.length != 2) {
            return syntaxError();
        }

        String name = args[1];
        if (deamonRunning(name)) {
            println("talk: user " + name + " is already listening");
            return ShellApp.appMiscError;
        }

        PipeAdvertisement adv = findUserAdv(name);
        discovery.remotePublish(adv, Discovery.ADV);
        if (adv == null) {
            println("talk: " + name + " is not a registered user");
            return ShellApp.appMiscError;
        }

        runDeamon(name, adv);

        return ShellApp.appNoError;
    }



    private int findUsers() {

    Enumeration enum;
    int i =0;



            discovery.getRemoteAdvertisements(null, Discovery.ADV,
                "name", TalkNameTag + "*",
                20);

           while (true) {
            try {
                if (i>MAXRETRIES){
                    println("");
                    break;
                }
                Thread.sleep(WaitingTime);
                print(".");
                i++;
            } catch (Exception e) {
            }
           }

        try {

            enum = discovery.getLocalAdvertisements(Discovery.ADV,
            "name", TalkNameTag + "*");
            if ((enum != null) && (enum.hasMoreElements())) {
                PipeAdvertisement adv = null;
                println("found the following registrations:");
                while (enum.hasMoreElements()) {

                    try {
                        adv = (PipeAdvertisement) enum.nextElement();
                        println(adv.getName());
                    } catch(Exception e) {
                        continue;
                    }
                }

            }
        } catch (Exception e) {
        }
        return ShellApp.appNoError;
    }

    private int logout(String[] args) {

        if (args.length != 2) {
            return syntaxError();
        }

        String name = args[1];
        if (!deamonRunning(name)) {
            println("talk: user " + name + " is not listening");
            return ShellApp.appMiscError;
        }
        stopDeamon(name);
        return ShellApp.appNoError;
    }


    private int sendMessage(String[] args) {

        String name = null;
        String srcName = null;

        if (args[0].equals("-u")) {
            if (args.length != 3) {
                return syntaxError();
            }
            srcName = args[1];
            name = args[2];

            // check if the name is registered
            if (!deamonRunning(srcName)) {
                println("talk: user " + srcName + " is not logged");
                return ShellApp.appMiscError;
            }
        } else {
            name = args[0];
            // There is no name for the person sending message
            // use the peer id instead.
            srcName = group.getPeerID().toString();
        }

        PipeAdvertisement adv = findUserAdv(name);
        if (adv == null) {
            println("talk: " + name + " is not a registered user");
            return ShellApp.appMiscError;
        }

        OutputPipe pipeOut = null;
        try {
            println("found user's advertisement attempting to connect");
            pipeOut = pipes.createOutputPipe(adv, Pipe.NonBlocking, 50000);
            if (pipeOut == null) {
                println("talk:  user " + name + " is not listening. Try again later");
                return ShellApp.appMiscError;
            }
        } catch (Exception e) {
            println("talk: user " + name + " is not listening. Try again later");
            return ShellApp.appMiscError;
        }


        // Get the user text

        println("talk is connected to user " + name);
        println("Type your message. To exit, type \".\" at begining of line");

        Message msg = null;
        String userInput = null;
        InputStream ip = null;

        while (true) {

            try {
                userInput = waitForInput();
                if (userInput == null) {
                    break;
                }
                if (userInput.equals(".")) {
                    break;
                }
                // Build a message
                msg = pipes.createMessage();

                ip = new ByteArrayInputStream(userInput.getBytes());
                msg.push(SenderMessage, ip);

                if (srcName != null) {
                    ip = new ByteArrayInputStream(srcName.getBytes());
                    msg.push(SenderName, ip);
                }

                pipeOut.send(msg);

            } catch (Exception e) {
                println("talk failed to send message to user :" + name);
                return ShellApp.appMiscError;
            }
        }
        pipeOut.close();
        return ShellApp.appNoError;
    }


    private int registerNewUser(String[] args) {

        if (args.length != 2) {
            return syntaxError();
        }
        String name = args[1];

        // Check if there is already a registered user of the
        // same name.
        PipeAdvertisement adv = findUserAdv(name);
        if (adv != null) {
            println("talk: sorry, user " + name +
            " is already registered");
            return ShellApp.appMiscError;
        }

        try {
            // Create a pipe advertisement for this pipe.
            adv = (PipeAdvertisement)
            AdvertisementFactory.newAdvertisement(
            PipeAdvertisement.getAdvertisementType() );
        }
        catch( Exception all ) {
            println( "Advertisement document could not be created" );
            return ShellApp.appMiscError;
        }

        ShellObject obj = env.get("stdgroup");
        // extract the advertisement
        PeerGroup group = (PeerGroup) obj.getObject();

        adv.setPipeID( new PipeID( group.getID() ) );
        adv.setName(TalkNameTag + ":" + name);

        try {
            // Save the document into the public folder
            discovery.publish(adv, Discovery.ADV);
            discovery.remotePublish(adv, Discovery.ADV);
        } catch (Exception e2) {
            println( "Advertisement could not be saved" );
            e2.printStackTrace();
            return ShellApp.appMiscError;
        }
        println("User : " + name + " is now registered");
        return ShellApp.appNoError;
    }


    private boolean checkUserAdv(PipeAdvertisement adv, String name) {

        if (adv == null) return false;
        if (adv.getName() == null) return false;
        if (adv.getName().equals(TalkNameTag + ":" + name)) return true;
        return false;
    }


    private PipeAdvertisement findUserAdv(String name) {

        Enumeration enum = null;

        // First look in the local storage
        try {
            enum = discovery.getLocalAdvertisements(Discovery.ADV,
            "name", TalkNameTag + ":" + name);
            if ((enum != null) && (enum.hasMoreElements())) {
                PipeAdvertisement adv = null;

                while (enum.hasMoreElements()) {

                    try {
                        adv = (PipeAdvertisement) enum.nextElement();
                    } catch(Exception e) {
                        continue;
                    }

                    if (checkUserAdv(adv, name)) {
                        return adv;
                    }
                }

            }
        } catch (Exception e) {
        }



        // Now, search remote
        discovery.getRemoteAdvertisements(null, Discovery.ADV,
        "name", TalkNameTag + ":" + name,
        2);

        // Wait a bit in order to get an answer.

       int i=0;
       while (true) {
        try {
            if (i>MAXRETRIES){
                println("");
                break;
            }
            Thread.sleep(WaitingTime);
            print(".");
            i++;
        } catch (Exception e) {
        }

        // Look in the local storage again
        try {
            enum = discovery.getLocalAdvertisements(Discovery.ADV,
            "name",
            TalkNameTag + ":" + name);

            if ((enum != null) && (enum.hasMoreElements())) {
                PipeAdvertisement adv = null;

                while (enum.hasMoreElements()) {

                    try {
                        adv = (PipeAdvertisement) enum.nextElement();
                    } catch(Exception e) {
                        continue;
                    }

                    if (checkUserAdv(adv, name)) {
                        return adv;
                    }
                }
            }
        } catch (Exception e) {
        }
       }

        return null;
    }

    private void runDeamon(String name, PipeAdvertisement adv) {

        userAdv = adv;
        userName = name;
        thread = new Thread(this, "Talk:Talk Deamon" );
        thread.start();
        // Store this object
        ShellObject obj = env.add(EnvName + "." + name,
        new ShellObject("Talk Deamon for " + name,
        this));
    }

    private void stopDeamon(String name) {

        ShellObject obj = env.get(EnvName + "." + name);
        if (obj == null) {
            return;
        }
        talk d = null;
        try {
            d = (talk) obj.getObject();
            d.thread.interrupt();
            env.remove(EnvName + "." + name);
        } catch (Exception e) {
            println("talk: cannot stop deamon for " + name);
            return;
        }
    }


    public void run() {


        InputPipe pipeIn = null;
        try {
            pipeIn = pipes.createInputPipe(userAdv);
        } catch (Exception e) {
            return ;
        }
        if (pipeIn == null) {
            println("talk: cannot open InputPipe");
            return;
        }

        // Listen on the pipe
        Message msg = null;

        while (true) {

            try {
                msg = pipeIn.waitForMessage();
                if (msg == null) {
                    if (Thread.interrupted()) {
                        // We have been asked to stop
                        println("talk: stop listening for user " + userName);
                        pipeIn.close();
                        return;
                    }
                }
            } catch (Exception e) {
                // This exception probaly happened because the
                // deamon has been interrupted. In any case, that
                // shows that talk cannot receive messages anyway.
                // clean up.
                pipeIn.close();
                println("talk: stop listening for user :" + userName);
                return;
            }
            // Get the first tag
            InputStream ip = null;
            Enumeration enum = msg.getNames();
            if ((enum == null) || (!enum.hasMoreElements())) {
                // Empty message
                continue;
            }
            int res = 0;
            String senderName = "Anonymous";
            byte[] buffer = null;

            // Get sender information
            try {
                ip = msg.pop(SenderName);
                if (ip != null) {
                    buffer = new byte [ip.available()];
                    res = ip.read(buffer);
                    senderName = new String(buffer);
                }
            } catch (Exception e) {
                // No user information
            }

            String senderMessage = null;
            // Get message
            try {
                ip = msg.pop(SenderMessage);
                if (ip != null) {
                    buffer = new byte [ip.available()];
                    res = ip.read(buffer);
                    senderMessage = new String(buffer);
                } else {
                    println("talk: receive from user " + senderName +
                    " an empty message for " + userName);
                    continue;
                }
            } catch (Exception e) {
                println("talk: receive from user " + senderName +
                " an empty message for " + userName);

            }
            println("talk: from " + senderName + " to " + userName);
            println("Message: " + senderMessage);
        }
    }

    public void help() {
        println("NAME");
        println("     talk - talk to another peer ");
        println(" ");
        println("SYNOPSIS");
        println(" ");
        println("    talk -register <userName> ");
        println("    talk -login <userName>");
        println("    talk -logout <userName>");
        println("    talk [-u <user>] <userName>");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        println("The 'talk command implements a simple instant messaging command");
        println("where two users on two remote peers can exchange messages.");
        println("Messages are displayed on the Shell stdout. In order");
        println("to use 'talk'. The user needs to register himself. This is done");
        println("via the following steps:");
        println("Step 1: Register via 'talk -register <username>' command. This command");
        println("        creates a Talk advertisement for that user. This has to");
        println("        be done only once, the first time the user registers with");
        println("        talk. The system remembers it accross reboot.");
        println(" ");
        println("Step 2: Login  via 'talk -login <username>' command. This command");
        println("        logs the user and starts a listener daemon. This has to");
        println("        to be done everytime the peer is restarted.");
        println(" ");
        println("Step 3: User can talk to another user via the command");
        println("        'talk -u <myusername> <destusername>'. This command will prompt the user");
        println("        to enter the message he/she wants to send");
        println(" ");
        println("         JXTA>talk -u moi mike");
        println("         talk is connected to user mike");
        println("         Type your message. To exit, type '.' at begining of line");
        println("  ");
        println("To stop receiving any more talk messages. The user can stop the talk");
        println("listener daemon by entering the command 'talk -logout <username>'");
        println(" ");
        println("OPTIONS");
        println(" ");
        println("    -register register a new user name  ");
        println("    -login    log user and set default user");
        println("    -logout   logout");
        println("    -u        specify current user name");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("      JXTA>talk -register me");
        println("      JXTA>talk -login me");
        println("      JXTA>talk -u me you");
        println(" ");
        println("This example shows how a new user 'me'  can register and log into talk,");
        println(" an talk to the user 'you'. User 'you' needs to be registered and logged on.");
        println(" ");
        println("SEE ALSO");
    }

}


