/*
 * Copyright (c) 2001 Sun Microsystems, Inc.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *       Sun Microsystems, Inc. for Project JXTA."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Sun", "Sun Microsystems, Inc.", "JXTA" and "Project JXTA" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact Project JXTA at http://www.jxta.org.
 *
 * 5. Products derived from this software may not be called "JXTA",
 *    nor may "JXTA" appear in their name, without prior written
 *    permission of Sun.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SUN MICROSYSTEMS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of Project JXTA.  For more
 * information on Project JXTA, please see
 * <http://www.jxta.org/>.
 *
 * This license is based on the BSD license adopted by the Apache Foundation.
 *
 * $Id: whoami.java,v 1.15 2001/05/22 23:43:01 lomax Exp $
 */


package net.jxta.impl.shell.bin.whoami;

import java.util.*;
import java.io.StringWriter;

import net.jxta.impl.shell.*;
import net.jxta.protocol.*;
import net.jxta.peergroup.*;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredTextDocument;

/**
 * whoami command list the local peer information
 */

public class whoami extends ShellApp {
    
    private ShellEnv myEnv;
    
    public whoami() {
    }
    
    public int startApp (String[] args) {
        
        int i = 0;
        boolean full = false;
        boolean viewpg = false;
        ShellObject obj =  null;
        PeerGroup group =  null;
        PeerGroupAdvertisement adv = null;
        
        myEnv = getEnv();
        
        if (args != null) {
            for (i=0; i < args.length; ++i) {
                if (args[i].equals("-l")) {
                    full = true;
                    continue;
                }
                if (args[i].equals("-g")) {
                    viewpg = true;
                    continue;
                }
                return syntaxError();
            }
        }
        
        if (viewpg) { // Select the default peerGroup
            obj = myEnv.get ("stdgroup");
        } else {
            obj = myEnv.get ("rootgroup");
            if (obj == null) // we really want the netpeergroup
                obj = myEnv.get ("stdgroup");
        }
        
        // extract the advertisement
        group = (PeerGroup) obj.getObject();
        adv = group.getAdvertisement();
        
        if( full ) {
            try {
                StringWriter    out = new StringWriter();
                MimeMediaType displayAs = new MimeMediaType( "text/plain" );;
                
                StructuredTextDocument doc = (StructuredTextDocument) adv.getDocument( displayAs );
                
                doc.sendToWriter( out );
                println(out.toString());
            }
            catch( Exception all ) {
                return syntaxError();
            }
        }
        else if (!viewpg ) {
            String name = adv.getPeerName();
            if (name != null)
                println("<Peer>"+name+"</Peer>");
            String keyword = adv.getKeywords();
            if (keyword != null)
                println("<Keywords>"+keyword+"</Keywords>");
            println("<PeerId>"+ adv.getPid()+"</PeerId>");


	    if (((EndpointAdvertisement) adv.getEndpointAdvertisements().get("TCP")
		 != null)) {
		println("<TransportAddress>"+
			((EndpointAdvertisement) adv.
			 getEndpointAdvertisements().get("TCP")).getEndpointAddress() +
			"</TransportAddress>");
	    }

	    if (((EndpointAdvertisement) adv.getEndpointAdvertisements().get("HTTP")
		 != null)) {
		println("<TransportAddress>"+
			((EndpointAdvertisement) adv.
			 getEndpointAdvertisements().get("HTTP")).getEndpointAddress() +
			"</TransportAddress>");
	    }

        } else {
            println("<PeerGroup>"+
            adv.getName()+"</PeerGroup>");
            String keyword = adv.getKeywords();
            if (keyword != null)
                println("<Keywords>"+keyword+"</Keywords>");
            println("<PeerGroupId>"+
            adv.getGid()+"</PeerGroupId>");
            for (Enumeration e=adv.getServiceAdvertisements().
            elements(); e.hasMoreElements();) {
                ServiceAdvertisement p
                    = (ServiceAdvertisement) e.nextElement();
                println("<Service>" + p.getName() + "</Service>");
            }
        }
        
        return ShellApp.appNoError;
    }
    
    
    
    public int syntaxError() {
        println ("error: whoami [-g] [-l]");
        return ShellApp.appParamError;
    }
    
    
    public void stopApp () {
    }
    
    public void help() {
        println("NAME");
        println("     whoami - display information about a peer or peergroup");
        println(" ");
        println("SYNOPSIS");
        println("     whoami [-l] [-g]");
        println(" ");
        println("DESCRIPTION");
        println(" ");
        println("whoami displays information about a peer or a");
        println("peergroup. With no option, whoami returns");
        println("information about the local peer. The '-g' option returns");
        println("information about the current peer group joined.");
        println("");
        println("By default the Shell is brought up in the 'NetPeerGroup' group. Peers");
        println("do not have name. A user can assign a symbolic name to a peer");
        println("via the 'setinfo' command. There is no guarantee that the name");
        println("is unique. Only the peerId and the peerGroup ID are guaranteed");
        println("to be unique.");
        println("");
        println("OPTIONS");
        println("");
        println("     [-g]  return info about the current peergroup");
        println("     [-l]  return a long version of the advertisement");
        println(" ");
        println("EXAMPLE");
        println(" ");
        println("    JXTA>whoami -l");
        println(" ");
        println("This example displays a long version of the peer group information");
        println(" ");
        println("SEE ALSO");
        println("    setinfo peers");
    }
    
}
